#!/usr/bin/env python
# Author: Kristjan Haule, Jan 2008


import os,sys,re,glob
from bisect import bisect
from scipy import *

from pylab import *
from matplotlib.font_manager import fontManager, FontProperties
import copy


font = {'family' : 'serif', 'weight' : 'bold', 'size'   : 20,}
rc('font', **font)  # pass in the font dict as kwargs
    


        
if __name__ == '__main__':

    Ry2eV = 13.6058
    inpdir = '.'
    sig = 'sig.inp'
    gamma = [0.01,0.01]
    Nkp=9                 # Number of k-points between two successive points in the k-path
    upfold=False
    total=True
    vmm=[0,1.]            # Color adjustment: minimum and maximum value plotted (in units of maximum value)
    om_mm = [(-5,5),100]
    
    execfile('PARAMS.akw') # File containing all parameters!


    a_outdir = inpdir

    scs = glob.glob(a_outdir+'/*.scs')[0]
    fscs = open(scs, 'r')
    data = fscs.readlines()
    muLDA = float(data[-1].split()[0])*Ry2eV
    print 'muLDA=', muLDA
    
    ldb = open(a_outdir+'/bandsFile.dat')
    Npnt = int(ldb.next().split()[5])
    Nbnd = int(ldb.next().split()[5])
    Ndir = int(ldb.next().split()[5])
    Ndiv = int(ldb.next().split()[5])
    ldb.next()
    psymb=[]
    for i in range(Ndir):
        line = ldb.next().split()
        if int(line[3]) != i+1: print 'Something wrong reading bandsFile.dat 0'
        psymb.append([line[5],line[7]])
    plngt=[]
    for i in range(Ndir):
        line = ldb.next().split()
        if int(line[3]) != i+1: print 'Something wrong reading bandsFile.dat 1'
        plngt.append(line[5])
    ldb.next()

    Ek=[]
    Xk=[]
    unique=zeros(Npnt,dtype=int)
    for ib in range(Nbnd):
        ldb.next()
        ldb.next()
        xk=[]
        ek=[]
        xp=-100
        for ik in range(Npnt):
            line = ldb.next().split()
            if int(line[0])!=ik+1 : print 'Something wrong reading bandsFile.dat 2', line[0], ik+1
            xt = float(line[1])
            if (abs(xt-xp)>1e-10):
                xk.append(float(line[1]))
                ek.append(float(line[2]))
                unique[ik]=1
            xp=xt
        Ek.append(ek)
        Xk.append(xk)
    Ek=array(Ek)
    Xk=array(Xk)

    
    
    #------------------------------------------------------
    #  Plotting
    #------------------------------------------------------
    Ninterp=5.            # Number of additional points to make spagetti look smooth
    color_bar = True
    
    execfile('PARAMS.akw') # File containing all parameters!

    xmm=[0,len(kpath)]
    # Plotting of LDA spagetti
    Ek = Ek*Ry2eV - muLDA
    # for x-axis
    X = array(range(len(Ek[0])))*(xmm[1])/float(len(Ek[0]))
    # adding many points for smooth spagetti plot
    lX=[]
    for i in range(len(X)-1):
        a = X[i]
        b = X[i+1]
        dh = (b-a)/Ninterp
        lX += arange(a,b,dh).tolist()
    lX.append(X[-1])

    for ib in range(len(Ek)):
        # interpolate on denser msh
        tckr = interpolate.splrep(X, Ek[ib], s=0)
        lEk = interpolate.splev(lX, tckr)
        plot(lX, lEk,  '-k', linewidth = 2)

    ax = axis()
    plot((ax[0], ax[1]), (0, 0), '--b', linewidth = 3, zorder=1)
    Klabel = [kpath[i][0][0] for i in range(len(kpath))]
    Klabel.append(kpath[-1][0][1])
    
    xticks(range(len(Klabel)), tuple(Klabel), fontsize='x-large')
    yticks(fontsize='x-large')
    ylabel('$\omega$', fontsize='x-large')

    for ik,kl in enumerate(Klabel):
        plot((ik, ik), (ax[2], ax[3]), '--g', linewidth = 1, zorder=1)

    axis([ax[0],ax[1], om_mm[0][0], om_mm[0][1]])
    show()
