import tutils as utl
from scipy import *

def read_nrad(fh_func, nsort):
    nrad = zeros(nsort,dtype=int)
    ilst = zeros(nsort,dtype=int)
    # reading nrad[nsort] and ilst[nsort] arrays
    for isort in range(nsort):
        (nrad[isort], ilst[isort], nspin) = utl.read_type(fh_func, int, 3)

    return nrad

def read_LmtSym(fh_func, isrt, ilsv):
    # Reading comment *** SYM DATA ***
    cm = fh_func.next() # comment
    # Reading dimension of arrays Sym,LmSym
    (limvmem, natommem) = utl.read_type(fh_func, int, 2)

    # Reading boolean array Sym
    Sym = array(utl.read_type(fh_func, utl.fort_bool, limvmem*natommem))
    # Reading integer array LmSym
    tLmSym = array(utl.read_type(fh_func, int, limvmem*natommem))
    # Reshaping
    Sym   = reshape(Sym,   (limvmem,natommem),order='F')
    tLmSym = reshape(tLmSym, (limvmem,natommem),order='F')
    # Set to zero non-existing Sym's
    # LmSym will contain only indexes which are actually used in calculation
    LmSym = zeros((limvmem,natommem),dtype=int,order='F')
    lmsym_max=0
    for iatom in range(natommem):
        isort=isrt[iatom]-1
        for l in range(ilsv[isort]+1):
            for m in range(-l,l+1):
                lm= l*(l+1) + m
                if (Sym[lm,iatom]):
                    isym = tLmSym[lm,iatom]
                    LmSym[lm,iatom]  = isym
                    if isym>lmsym_max:
                        lmsym_max = isym
    
    return LmSym

def read_RadialMesh(fh_func, nrad, nsort):
    # Reading comment **** Radial mesh ****
    cm = fh_func.next() # comment
    # Reading radial mesh for all sorts
    nrad_max = max(nrad)+1
    tot_rad = sum(nrad)+nsort
    # Reading all radial meshes into single array
    Rt = utl.read_type(fh_func, float, tot_rad)
    # Allocating proper fortran array
    Rm = zeros((nrad_max,nsort), dtype=float, order='F')
    # And inserting data into proper place
    ofset=0
    for isort in range(nsort):
        N = nrad[isort]+1
        Rm[0:N,isort] = Rt[ofset:ofset+N]
        ofset += nrad[isort]+1

    return Rm

def read_GauntCoeff(fh_func):
    # Reading comment "*** Gaunt coefficients ****"
    cm = fh_func.next() 
#    print 'This should be comment on Gaunt', cm
    # size of gaunt coefficients
    (lmax0, lmax1) = utl.read_type(fh_func, int, 2)
    tot_size = lmax0*lmax1*lmax1
    # reading Gaunt coefficients
    Cf = utl.read_type(fh_func, float, tot_size)
    # Allocating proper fortran array
    return reshape(Cf, (lmax0, lmax1, lmax1), order='F')

def read_RadialFunctions(fh_func, isort, nrad, ilst, nspin):
    # Reading comment "**** Radial functions ******"
    cm = fh_func.next() 
#    print 'This should be comment on RADIAL ', cm

    # length of data for radial functions
    n1 = nrad[isort]+1
    n2 = ilst[isort]+1
    n3 = nspin
    # reading LMTO wave functions
    Fih = utl.read_complex(fh_func, n1*n2*n3)
    Fij = utl.read_complex(fh_func, n1*n2*n3)
    # reshaping wave functions
    Fih = reshape(Fih, (n1, n2, n3), order='F')
    Fij = reshape(Fij, (n1, n2, n3), order='F')
    
    return (Fih, Fij)

def read_Gam(fh_func, ilst_sort):
    cm = fh_func.next() # reading comment "**** GAM data ******"
#    print 'This should be comment on GAM', cm
    
    # Reading GAMM data
    return array(utl.read_type(fh_func, float, ilst_sort+2))


def read_RhoSemicore(fh_func, nrad, LmSym, isrt, ilsv, nspin, norbs):
    cm = fh_func.next() # reading comment "**** SEMICORE DENSITY ******"
#    print 'This should be comment on SEMICORE ', cm

    natom = len(isrt)
    nrad_max = max(nrad)+1

    rho_semcr=[]
    for iatom in range(natom):
        isort=isrt[iatom]-1
        
        rhos = zeros((nrad[isort]+1, max(LmSym[:,iatom]), nspin, norbs), dtype=complex, order='F')
        for iorbs in range(norbs):
            for iupdn in range(nspin):
                for l in range(ilsv[isort]+1):
                    for m in range(-l,l+1):
                        lm= l*(l+1) + m
                        isym = LmSym[lm,iatom]
                        if (isym):
                            lm1 = utl.read_type(fh_func, int, 1)[0]
                            if lm+1 != lm1 : s.stderr.write('CMP_RHO: Error while reading semicore density - lm!=lm from file\n')                            
                            trho = utl.read_complex(fh_func, nrad[isort]+1)
                            rhos[:,isym-1,iupdn,iorbs] = trho
        rho_semcr.append(rhos)
        
    return rho_semcr

def read_RadialMeshDetails(fh_func, nrad, nsort):
    # Reading comment "****** RADIAL MESH DETAILS *****"
    cm = fh_func.next() 
#    print 'This should be comment on  RADIAL MESH ', cm
    cm = fh_func.next() # reading comment "H(isort)"
#    print 'This should be comment on H(isort) ', cm
    dh = utl.read_type(fh_func, float, nsort)
    cm = fh_func.next() # reading comment "weight(r,isort)"
#    print 'This should be comment on weight ', cm
    tot_rad = sum(nrad)+nsort
    tweight = utl.read_type(fh_func, float, tot_rad)    
    # Allocating proper fortran array
    # And inserting data into proper place
    rweight=[]
    ofset=0
    for isort in range(nsort):
        N = nrad[isort]+1
        rweight.append(array(tweight[ofset:ofset+N]))
        ofset += N
        
    return (rweight, dh)

def read_ChargeAndSphere(fh_func, nsort):
    cm = fh_func.next() # reading comment "****** VALENCE CHARGE *********"
#    print 'This should be comment on VALENCE CHARGE ', cm
    zvalt = utl.read_type(fh_func, float, 1)[0]
    cm = fh_func.next() # reading comment "******* atomic sphere radius *****"
#    print 'This should be comment on atomic shpere  ', cm
    SASA = utl.read_type(fh_func, float, nsort)
    cm = fh_func.next() # reading comment "******* MT-sphere radius *********"
#    print 'This should be comment on MT-sphere  ', cm
    SMT = utl.read_type(fh_func, float, nsort)

    return (zvalt, SASA, SMT)

def read_CoreDensity(fh_func, nrad, nspin):
    cm = fh_func.next() # reading comment "******** CORE DENSITY *************"
#    print 'This should be comment on CORE DENSITY  ', cm
    nsort = len(nrad)
    tot_rad = sum(nrad)+nsort
    trho_cor = utl.read_type(fh_func, float, tot_rad*nspin)

    # Allocating proper fortran array
    rho_cor=[]
    for isort in range(nsort):
        rho_cor.append(zeros((nrad[isort]+1,nspin), dtype=float, order='F'))
        
    # And inserting data into proper place
    ofset=0
    for ispin in range(nspin):
        for isort in range(nsort):
            N = nrad[isort]+1
            rho_cor[isort][:,ispin] = trho_cor[ofset:ofset+N]
            ofset += N
            
    return rho_cor

