#!/usr/bin/env python

import sys, re
from scipy import *

def match_next_ini(fini, text_to_match):
    line = fini.next()   # <SECTION=HEAD>
    m = re.search(text_to_match, line)
    if m is None:
        print 'The input file is not recognized!', text_to_match
        sys.exit(1)
    return m.groups()
        
if __name__ == '__main__':

    Azero = -1

    au2ang = 0.52918
    
    if len(sys.argv)<2:
        print 'Give the name of the project.'
        sys.exit(0)
     
    pr_name = sys.argv[1]
    
    fini = open(pr_name + '.ini', 'r')

    lines = fini.readlines()
    slines=[]
    for line in lines:
        slines.append(line.lower())

    if re.search(r'file=inifile', slines[0]) is None:
        print 'The input file is not recognized!', 'FILE=INIFILE'
        sys.exit(0)
    
    il=0
    while re.search('title', slines[il]) is None: il+=1
    m = re.search('[T|t][I|i][T|t][L|l][E|e]\s*=\s*(\w+)', lines[il])
    if m is not None: title = m.group(1)

    print 'title=', title

    il=0
    while re.search('natom', slines[il]) is None: il+=1
    m = re.search('natom\s*=\s*(\d+)', slines[il])
    if m is not None: natom = int(m.group(1))

    print 'natom=', natom

    il=0
    while re.search('nsort', slines[il]) is None: il+=1
    m = re.search('nsort\s*=\s*(\d+)', slines[il])
    if m is not None: nsort = int(m.group(1))

    print 'nsort=', nsort


    il=0
    while re.search('is', slines[il]) is None: il+=1
    m = re.search('is.*\s*=\s*((\d+\s+)+)', slines[il])
    if m is not None: IS = map(int, m.group(1).split())
    print 'is=', IS

    il=0
    while re.search('par0', slines[il]) is None: il+=1
    m = re.search('par0\s*=\s*(\d+(\.\d*)?)', slines[il])
    if m is not None: Par0 = float(m.group(1))
    print 'par0=', Par0

    
    Atoms=[]
    for isort in range(nsort):
        
        while re.search('sort', slines[il]) is None: il+=1
        
        while re.search('name', slines[il]) is None: il+=1
        
        m = re.search('[N|n][A|a][M|m][E|e]\s*=\s*(\w+)', lines[il])
        if m is not None: name = m.group(1)
        #print name
        
        while re.search('znuc', slines[il]) is None: il+=1
        
        m = re.search('znuc\s*=\s*(\d+(\.\d*)?)', slines[il])
        if m is not None:  znuc = int(float(m.group(1))+0.1)
        #print znuc

        Atoms.append([name, znuc])
        
    print 'Atoms=', Atoms
    

    fstr = open(pr_name + '.str', 'r')


    lines = fstr.readlines()
    slines=[]
    for line in lines:
        slines.append(line.lower())

    if re.search(r'file=strfile', slines[0]) is None:
        print 'The input file is not recognized!', 'FILE=STRFILE'
        sys.exit(0)
    

    
    il=0
    while re.search('btoa', slines[il]) is None: il+=1
    m = re.search('btoa\s*=\s*(\d+(\.\d*)?)', slines[il])
    if m is not None: BtoA = float(m.group(1))
    print BtoA

    il=0
    while re.search('ctoa', slines[il]) is None: il+=1
    m = re.search('ctoa\s*=\s*(\d+(\.\d*)?)', slines[il])
    if m is not None: CtoA = float(m.group(1))
    print CtoA

    il=0
    while re.search('icrd', slines[il]) is None: il+=1
    m = re.search('icrd\s*=\s*(\d?)', slines[il])
    if m is not None: iCrd = int(m.group(1))
    print 'iCrd=', iCrd

    il=0
    while re.search('<section=tran>', slines[il]) is None: il+=1
    primvec=[]
    for i in range(3):
        il+=1
        v = slines[il].replace(',', ' ').split()[:3]
        vn=[]
        for n in v:
            if re.search('/', n):
                m = re.search('(\d+)/(\d+)', n)
                if m is not None:
                    nn = map(float, m.groups())
                    nr = nn[0]/nn[1]
            else:
                nr = float(n)
            vn.append(nr)
        primvec.append(vn)

    while re.search('<section=bass>', slines[il]) is None: il+=1
    bass=[]
    for iatom in range(natom):
        il+=1
        v = slines[il].replace(',', ' ').split()[:3]
        vn=[]
        for n in v:
            if re.search('/', n):
                m = re.search('(\d+)/(\d+)', n)
                if m is not None:
                    nn = map(float, m.groups())
                    nr = nn[0]/nn[1]
            else:
                nr = float(n)
            vn.append(nr)
        bass.append(vn)
        
    print bass

    Par0 = Par0*au2ang

    print 'Par0=', Par0

    if iCrd==1:
        Bass=[]
        for v in bass:
            Bass.append([v[0]*Par0, v[1]*Par0*BtoA, v[2]*Par0*CtoA])
        print 'Bass=', Bass
    else:
        Bass=[]
        for v in bass:
            rb=zeros(3,dtype=float)
            for i in range(3):
                rb += array([primvec[i][0]*Par0, primvec[i][1]*Par0*BtoA, primvec[i][2]*Par0*CtoA])*v[i]
            Bass.append(rb)
            
    

    fxc = open(pr_name + '.xsf', 'w')
    print >> fxc, '# this is a specification of the LaOFeAs crystal structure\n'
    print >> fxc, 'CRYSTAL\n'
    print >> fxc, '# these are primitive lattice vectors (in Angstroms)'
    print >> fxc, 'PRIMVEC'
    for i in range(3):
        print >> fxc, "%f "*3 % (primvec[i][0]*Par0, primvec[i][1]*Par0*BtoA, primvec[i][2]*Par0*CtoA)
    print >> fxc
    print >> fxc, '# these are conventional lattice vectors (in Angstroms)'
    print >> fxc, 'CONVEC'
    for i in range(3):
        print >> fxc, "%f "*3 % (primvec[i][0]*Par0, primvec[i][1]*Par0*BtoA, primvec[i][2]*Par0*CtoA)

    print >> fxc
    print >> fxc, '# these are atomic coordinates in a primitive unit cell (in Angstroms)'
    print >> fxc, 'PRIMCOORD'

    natom_true=0
    for i in range(natom):
        if Atoms[IS[i]-1][1]>Azero : natom_true += 1
         
    print >> fxc, natom_true, 1
    for i in range(natom):
        atom = Atoms[IS[i]-1]
        if atom[1]>Azero: # to avoid empty spheres
            print >> fxc, "%3d" % atom[1], "%10.6f "*3 % tuple(Bass[i])
        
    

    fxc.close()
