#!/usr/bin/env python

import sys, re, os, glob, shutil

from scipy import *

ROOT = os.environ.get('LMTO_DMFT_ROOT')

class IMP_CTQMC(object):
    """Wrapper for ctqmc
    """
    def __init__(self, Znuc, dire, Impq, params, UpdateAtom=False):
        # impurity directory
        self.dir = dire+'/'
        self.UpdateAtom = UpdateAtom
        
        if len(glob.glob(dire))==0 : os.mkdir( dire )

        # log-file is opened
        self.fh_info = open(self.dir + 'info.out', 'w')
        self.fh_infoj = open(self.dir + 'jinfo.out', 'w')

        self.sparams={'exe':'ctqmc', 'Sig':'Sig.out', 'Delta':'Delta.inp', 'cix':'actqmc.cix', 'nom':100, 'aom':10, 'sderiv': 0.02, 'Ntau':1000, 'SampleGtau':1000, 'Naver':100000000, 'Ncout':1000000, 'Nmax':1024, 'GlobalFlip':10000, 'Gf':'Gf.out'}
        
        status = glob.glob("status.*")
        for fil in status:
            shutil.copy2(fil, self.dir+'/'+fil)
        

        # file from database which contains atomic information for this particular atom
        self.atom_arg = ROOT + '/impurity/database/' + 'atom.'+str(Znuc)+'.py'
        self.PARAMS = 'PARAMS'

        execfile(self.atom_arg)
        lcs = locals() # it seems python does not know about variables obtained by execfile
        # stores some of just defined variables
        self.l = lcs['l']
        self.J = lcs['J']
        self.cx = lcs['cx']

        self.spr={}
        if params.has_key('nc'): self.spr['n'] = params['nc'][0]
        else: self.spr['n'] = lcs['nc']
        if params.has_key('Ncentral'): self.spr['Ncentral'] = params['Ncentral'][0]
        else: self.spr['Ncentral'] = self.spr['n'][1:len(self.spr['n'])-1]
        
        self.deg={}
        for b in range(len(Impq)):
            if self.deg.has_key(Impq[b]): self.deg[Impq[b]]+=1
            else: self.deg[Impq[b]]=1
        
        if type(Impq)!= list :  Impq = Impq.tolist()
        self.Impq = Impq
        print >> self.fh_info, 'Impq=', Impq
        
        # Below exact diagonalization of the atom is run to produce 'cix.out' file
        # information is read from the file in database
        if not self.UpdateAtom:
            # executable for ED of the atom
            self.atom_exe = ROOT + '/impurity/bin/atomh'
            # argument list is created to later run the executable for exact diagonalization of the atom
            atom_arg = '-ns %d -ne %d -l %d -J %f -cx %f -para %d -Ekeep %f -Nmax %d -Ekeepc %f -Nmaxc %d -Ncentral %s' % (self.spr['n'][0], self.spr['n'][-1], lcs['l'], lcs['J'], lcs['cx'], lcs['para'], lcs['Ekeep'], lcs['Nmax'], lcs['Ekeepc'], lcs['Nmaxc'], self.spr['Ncentral'])
            atom_arg += ' -Impq "'+str(Impq)+'"'
            # runs ED for the atom
            stdin, stdout, stderr = os.popen3('cd '+ self.dir + '; ' + self.atom_exe +' ' +atom_arg+' > nohup.out 2>&1')
            print >> self.fh_info, stdout.read()
            print >> self.fh_info, stderr.read()
        else:
            self.atom_exe = ROOT + '/impurity/bin/atom_d.py'

        self.fh_info.flush()
        self.fh_infoj.flush()


        

    def _exe(self, params, mpi_prefix, lom, om, Delta, extn, Eimps, maxAc=20.):
        """ Executes the CTQMC impurity solver
        """
        if self.UpdateAtom:
            
            print 'Eimps=', Eimps
            print 'Ed=', params['Ed']
            print 'Impq=', self.Impq

            E0 = params['Ed'][0][0]+2*self.cx
            Eimps = (array(Eimps)-E0).tolist()
            
            mu_QMC = -E0
            
            print 'Eimps_new=', Eimps, ' muQMC=', mu_QMC
            
            atom_arg = self.atom_arg + ' \"Eimp='+str(Eimps)+'\" \"n='+str(self.spr['n'])+'\" \"Ncentral='+str(self.spr['Ncentral'])+'\"'
            # runs ED for the atom
            atom_arg += ' "Impq='+str(self.Impq)+'"'
            
            print atom_arg
            
            stdin, stdout, stderr = os.popen3('cd '+ self.dir + '; ' + self.atom_exe +' ' +atom_arg+' > nohup.out 2>&1')
            print >> self.fh_info, stdout.read()
            print >> self.fh_info, stderr.read()

            for filename in glob.glob(self.dir+'/status.*') :
                os.remove( filename )

                    
        else:
            # impurity level for 5/2 is E_{5/2}=E0-2*cx and for 7/2 is E_{7/2}=E0+3/2*cx
            # Here we correct E_{5/2} for 2*cx, which is already counted in cix file
            mu_QMC = -(params['Ed'][0][0]+2*self.cx)
            
        params['mu'] = [mu_QMC, '# QMC chemical potential']
        
        # executable
        shutil.copy2(ROOT + '/impurity/bin/' + params['exe'][0], self.dir+params['exe'][0])
        self.q_exe = './'+params['exe'][0]
        
        # Preparing params.dat file
        fp = open(self.dir+self.PARAMS, 'w')
        kparams = params.keys()
        for p in self.sparams.keys():
            if p not in kparams:  # if this parameter is overwritten by the input parameters, skip it
                fp.write(p + '  ' + str(self.sparams[p]) + '\n')
            
        for p in params.keys():
            print >> fp, "%s  %s " % (p, params[p][0]), '\t ', params[p][1]
        fp.close()

        # Preparing Delta.inp file
        #print 'Printing to ', self.dir+self.sparams['Delta']
        fp = open(self.dir+self.sparams['Delta'], 'w')
        # interpolate on big mesh
        Delta=array(Delta)
        for im in range(shape(Delta)[0]):
            for ib in range(shape(Delta)[1]):
                if abs(Delta[im,ib])>maxAc*pi : Delta[im,ib] = -1e-10j
                if Delta[im,ib].imag>0 : Delta[im,ib] = Delta[im,ib].real-1e-10j
                
        lDelta=[]
        for ib in range(shape(Delta)[1]):
            tckr = interpolate.splrep(om, Delta[:,ib].real, s=0)
            tcki = interpolate.splrep(om, Delta[:,ib].imag, s=0)
            lDelta.append( interpolate.splev(lom, tckr) + interpolate.splev(lom, tcki)*1j )
            
        for im,ome in enumerate(lom):
            print >> fp, ome,
            for b in range(len(lDelta)):
                print >> fp, "%f  %f " % (lDelta[b][im].real, lDelta[b][im].imag),
            print >> fp
        fp.close()
        
        shutil.copy2(self.dir+self.sparams['Delta'], self.dir+self.sparams['Delta']+'.'+extn)
        
        # Below we execute ctqmc
        cmd = 'cd '+self.dir+'; '+mpi_prefix+' '+self.q_exe+' '+self.PARAMS+' > nohup_imp.out 2>&1 '
        print cmd
        print 'Running ---- ctqmc -----'
        stdin, stdout, stderr = os.popen3(cmd)
        print >> self.fh_infoj, stdout.read(), stderr.read()

        
    def HighFrequency(self, params, DCs, da, extn):
        """ Reads impurity output and prepares for further execution
          Output:
               Sig[b][iom]  -- DMFT dynamic self-energy which vanishes at infinity
               sinf[b]      -- DMFT self-energy at infinity
               Edc[b]       -- double counting using DMFT occupancies
               
               """
        # Reading Sig.out which was produced by ctqmc
        fS = open(self.dir + self.sparams['Sig'])
        # Sig.out contains nf, moment, ntot,...
        first_line = fS.next().strip()
        adat = first_line.split()
        for par in adat: # setting nf from ctqmc output
            m = re.search('nf=', par)
            if m is not None : exec(par)
        ntot = nf
        
        om=[]
        Sigo=[]
        for line in fS:
            dat = line.split()
            om.append(float(dat[0]))
            del dat[0]
            sb=[]
            nbath = len(dat)/2
            for b in range(nbath):
                sb.append(float(dat[2*b]) + float(dat[2*b+1])*1j)
            Sigo.append(sb)            
        Sigo=array(Sigo)
        om=array(om)

        # self-energy at infinity
        sinf = copy(Sigo[-1][:]).real
        # subtracting value at infinity
        for im in range(len(Sigo)): Sigo[im] -= sinf

        
        #da = 0
        #if DCs=='default' : da = DCs['a']
        # Here we compute self-energy(infinity) and 
        # double-counting using impurity occupancy
        Edc = params['U'][0]*(nf-0.5-da) - self.J*(nf/2.-0.5)

        # If user fixes double counting by certain predescribed nf0
        # (fixn=True), we need to normalized actual nf[:] such that
        # sum(nf[:]) is equal to predefined nf0
        # Here we do not change Sigma_{infinity} but only Edc.
        # Sigma_{infinity} is determined by actual nf, while Edc is determined by modified nf.
        if DCs=='fixn':
            nf0 = params['nf0'][0]
            Edcn = params['U'][0]*(nf0-0.5) - self.J*(nf0/2.-0.5)
            ta = (Edc-Edcn)/params['U'][0]
            print >>self.fh_info, ('### a=%f' % ta)
            Edc = Edcn

        # Copying data at each iteration to follow the evolution
        shutil.copy2(self.dir+self.sparams['Sig'], self.dir+self.sparams['Sig']+'.'+extn)
        shutil.copy2(self.dir+self.sparams['Gf'], self.dir+self.sparams['Gf']+'.'+extn)
        shutil.copy2(self.dir+'nohup_imp.out', self.dir+'nohup_imp.out'+'.'+extn)
        shutil.copy2(self.dir+'Probability.dat', self.dir+'Probability.dat'+'.'+extn)
        


        # Reading Gf.out to see how good is doping
        E0 = array(params['Ed'][0])+array(sinf)
        
        fGo = io.read_array(self.dir+self.sparams['Gf'], columns=(0,-1), lines=(1,-1))
        nfb=zeros(nbath)
        for ig in fGo:
            ome = ig[0]
            for b in range(nbath):
                nfb[b] += ig[2*b+1] - 1/(ome*1j-E0[b])

        T=om[0]/pi
        dn = [ferm(E0[b]/T) for b in range(nbath)]
        n_imp = nfb*2*T + array(dn)

        nt_imp=0
        for b in range(nbath):
            nt_imp += n_imp[b]*self.deg[b] 
        
        print 'nimp,gnimp=', ntot, nt_imp
        print >> self.fh_info, 'nimp,gnimp=', ntot, nt_imp
        
        self.fh_info.flush()
        self.fh_infoj.flush()
        
        return (Sigo.transpose(), sinf, [Edc]*nbath, ntot)


def ferm(x):
    """Fermi function for T=1"""
    if (x>100): return 0
    if (x<-100): return 1
    return 1/(exp(x)+1)
