#!/usr/bin/env python
# Author: Kristjan Haule, Jan 2008

import os,sys,re

if 'ROOT' not in globals().keys():
    ROOT = os.environ.get('LMTO_DMFT_ROOT')
    sys.path.append( ROOT )
    sys.path.append( ROOT + '/fort')

from ld_base import *
import cmtrans

def cmp_Transport(lmt, T, SigO, sZO, alphaV, fh_info, gamma=[0.0,0.0]):
    """ Actually computes transport properties"""

    alphaV1 = zeros((3,3), order='F')
    for i in range(3): alphaV1[i,i] = alphaV[i]

    opt = cmtrans.cmp_trans3(lmt.mu, T, SigO, sZO, lmt.vel, lmt.hamf, lmt.olap, alphaV1, lmt.latcon, lmt.unit_cell_volume, lmt.wk)
    
    print >> fh_info, opt
    fh_info.flush()
    return opt



def read_velocity(lmt):
    """ Reads velocities outputed by LmtArt.

    Output:
            matrix: lmt.vel[3,ndim,ndim,nkp]

    """
    #------------------- info ---------------------------------------
    lmt.fh_info.write("-"*80+"\n")
    lmt.fh_info.write("READ_VEL:   Velocities\n\n")
    #------------------- info ---------------------------------------
    file_vel  = lmt.inpdir + "/vel.dat"
    #-------------------- Reading Velocities ---------------
    lmt.fh_info.write("Reading velocities from "+file_vel+"\n")
    # fortran read
    lmt.vel = fort_read.read_v(file_vel, 3, lmt.ndim, lmt.ndim, lmt.nkp, 1, 1)


    # from Rydbergs to eV
    lmt.fh_info.write("Assuming velocities are in Rydberg->Changing to electron volts\n")
    Ry2eV = 13.6058
    lmt.vel *= Ry2eV*2j/lmt.latcon # due to SS definition of velocities
    

#-------------- for upfolding -------------------------
def strip_comments(data):
    newdata=[]
    for i in range(len(data)):
        stripped  = re.sub(r'(.*)\#.*', r"\1",  data[i].strip())
        stripped = stripped.split()
        if (len(stripped)>0):
            newdata.append(stripped)
    return newdata

def pars_index(ndim, lines, i0):
    """ reading the rules to transform
    columns of the sigma file to matrix"""
    sind=zeros((ndim,ndim), dtype=int)
    coef = zeros((ndim,ndim), dtype=complex)
    for p in range(ndim):
        ph = lines[i0]
        del lines[i0]

        for q in range(ndim):
            cols = re.findall(r'\$(\d+)', ph[q])
            if (cols):
                sind[p,q] = int(cols[0])

                tsc = re.sub(r'\$'+cols[0], '1', ph[q])
                coef[p,q] = eval(tsc)

        mess=[]
    return (sind, coef)


def pars_coeff(sc):
    """ Given a list string such as '$1$1 + 1./4.*$6$1 + (1.+1j)*$3$3*2'
        returns list of column numbers [(1,1),(6,1),(3,3)] and coefficients
        which correspond to each column  [1, 0.25, 2+2j]
    """
    cols = re.findall(r'\$(\d+)\$(\d+)', sc)
    scols = re.findall(r'(\$\d+\$\d+)', sc)
    cols = [(int(x[0]),int(x[1])) for x in cols]

    coef=[]
    for ic in range(len(cols)):
        tsc = sc
        for iq in range(len(cols)):
            rg = r'\$'+str(cols[iq][0])+'\$'+str(cols[iq][1])
            if (cols[ic]==cols[iq]):
                tsc = re.sub(rg, '1', tsc)
            else:
                tsc = re.sub(rg, '0', tsc)

        coef.append(eval(tsc))

    return (scols, cols, coef)

def read_sind(fh_sind, ndim):
    """ Reads index array to make a matrix of self-energy from columns of the file
    """
    f = open(fh_sind, 'r')

    data = f.readlines()
    lines = strip_comments(data)

    # Transformation from momentum to real space base
    Uk = identity(ndim)
    if (['transformation'] in lines):
        i0 = lines.index(['transformation'])
        del lines[i0]

        # parsing transformation
        Uk0=[]
        for p in range(ndim):
            Uk0.append(map(complex,lines[i0]))
            del lines[i0]
        Uk = array(Uk0)


    corind = ones((ndim,ndim),dtype=int)
    if (['correlated_blocks'] in lines):
        i0 = lines.index(['correlated_blocks'])
        del lines[i0]

        ones_ = ones(ndim, dtype=int)
        # parsing the shape of correlated blocks
        corind=[]
        for p in range(ndim):
            ind = map(int, lines[i0]) - ones_
            corind.append(ind)
            del lines[i0]
        corind = array(corind)

    NMcoef = ones((ndim,ndim))
    NMsind = ones((ndim,ndim),dtype=int)
    if (['NM_structure'] in lines):
        i0 = lines.index(['NM_structure'])
        del lines[i0]
        # reading the rules to transform
        # columns of the sigma file to matrix
        (NMsind, NMcoef) = pars_index(ndim, lines, i0)

    return (Uk, corind, NMsind, NMcoef)


def CreateSelfEnergyMatrixUpfold(Sig, Uk, sind, coef, gamma=0.01):
    # Here we create a matrix of self-energy Sig. The most important array in this class
    sig = matrix(zeros((len(sind),len(sind)),dtype=complex))
    #print shape(sig), shape(Sig), shape(Uk), shape(sind), shape(coef)
    for p in range(len(sig)):
        for q in range(len(sig)):
            ind = sind[p,q]-1
            #print p, q, ind
            if (ind>=0):
                if (ind<shape(Sig)[0]):
                    sig[p,q] = Sig[ind]*coef[p,q]
                else:
                    sig[p,q] = -gamma*1j

    return matrix(Uk) * matrix(sig) * matrix(conjugate(Uk.transpose()))

def CreateZMatrixUpfold(Zc, Uk, sind, coef):
    # Here we create a matrix of self-energy Sig. The most important array in this class
    sZr = identity(len(sind), dtype=complex)
    for p in range(len(Zc)):
        for q in range(len(Zc)):
            ind = sind[p,q]-1
            if (ind>=0):
                if (ind<len(Zc)):
                    sZr[p,q] = sqrt(Zc[ind])*coef[p,q]

    return matrix(Uk) * matrix(sZr) * matrix(conjugate(Uk.transpose()))


#-------------- for upfolding -------------------------

def CreateZMatrix(Zvec, Sigind):
    Z_out = identity(len(Sigind),dtype=complex)
    for i in range(shape(Sigind)[0]):
        for j in range(shape(Sigind)[1]):
            if (Sigind[i,j] > 0):
                Z_out[i,j] = sqrt(Zvec[Sigind[i,j]-1])

    return Z_out


if __name__ == '__main__':

    inpdir = '.'
    sig = 'sig.inp'
    gamma = [1e-4,1e-4]
    alphaV = [1/2.,1/2.,0.] # Sets the direction for conductivity
    
    execfile('PARAMS.trans') # File containing all parameters!

    if (len(sys.argv)>1): sig= sys.argv[1]

    fh_info = open(inpdir+'/trans.info.dat', 'w')
    print >> fh_info, '-'*80
    print >> fh_info, '*'*20, 'Transport calculation'
    print >> fh_info, '-'*80, '\n'

    
    # Reads LMTO data
    lmt = Lmtart(inpdir, fh_info)
    lmt.read_cix()    
    trn.transform2(lmt.olap, lmt.hamf, lmt.Utk)

    # Correlated bands
    dcorr = utl.flatten(lmt.vSigind)
    icorr = [lmt.Sigind[ic][ic] for ic in dcorr]
    if sum(sort(icorr)-array(range(1,len(icorr)+1)))==0: # icor contains right seuqence of integers
        corr = [dcorr[i-1] for i in icorr]               # kcix.dat contains  sequence of orbitals
        print >> fh_info, 'reordering corr'
    else:
        corr = dcorr
    print >> fh_info, 'corr = ', corr



    # Reads self-energy from file
    fs = open(inpdir+"/"+sig, 'r')
    fs.next(); data = fs.next().split()
    T = float(data[0])
    fs.next(); data = fs.next().split()
    Sigc = [float(data[2*i])+float(data[2*i+1])*1j for i in range(len(data)/2)]
    fs.next(); data = fs.next().split()
    Zc = map(float,data)
    print >> fh_info, 'Temperature=', T
    
    if upfold:
        s_ndim = len(corr)
        (Uk, corind, NMsind, NMcoef) = read_sind(f_sind, s_ndim)
        
        sig = CreateSelfEnergyMatrixUpfold(Sigc, Uk, NMsind, NMcoef, gamma[0])
        szo = CreateZMatrixUpfold(Zc, Uk, NMsind, NMcoef)

        SigO = zeros((lmt.ndim,lmt.ndim), dtype=complex)
        sZO = identity(lmt.ndim, dtype=complex)
        for ip in range(len(SigO)): SigO[ip,ip] = -gamma[1]*1j
        
        for iq0,q0 in enumerate(corr):
            for iq1,q1 in enumerate(corr):
                SigO[q0,q1] = sig[iq0,iq1]
                sZO[q0,q1] = szo[iq0,iq1]
        
        lmt.mu += dmu
        
    else:
        SigO = CreateSelfEnergyMatrix(Sigc, lmt.Sigind, gamma)
        sZO = CreateZMatrix(Zc, lmt.Sigind)
        lmt.mu = mu
            
    print >> fh_info, 'Self-energy created'

    
    
    # Reads velocities
    read_velocity(lmt)
    trn.transform3(lmt.vel, lmt.Utk) # transforms velocities to DMFT base
        
        
    # Parallel execution of optics calculation
    opt =  cmp_Transport(lmt, T, SigO, sZO, alphaV, fh_info, gamma)
    print T, opt
    
