#!/usr/bin/env python
# Author: Kristjan Haule, December 2007

import os,sys,errno
ROOT = os.environ.get('LMTO_DMFT_ROOT')
if ROOT is not None:
    sys.path.append( ROOT )
    sys.path.append( ROOT + '/fort')
else:
    print >> sys.stderr, "Environment variable LMTO_DMFT_ROOT must be set!"
    print "Environment variable LMTO_DMFT_ROOT must be set!"
    sys.exit(1)

# specific imports to this module
import shutil
from numpy.linalg import *

from read_lmtart import *
from smf import *
import tutils as utl

import copy

def print_m(Hr):
    for i in range(len(Hr)):
        for p in range(len(Hr[i])):
            for q in range(len(Hr[i][p])):
                ff = Hr[i][p,q]
                if (abs(ff.imag)<1e-4): print "%8.5f " % ff.real,
                else: print "%4.2f+%4.2f*j" % (ff.real, ff.imag),
            print
        print
        
def print_m0(Hr):
    for p in range(len(Hr)):
        for q in range(len(Hr[p])):
            ff = Hr[p,q]
            if (abs(ff.imag)<1e-4): print "%8.5f " % ff.real,
            else: print ("%4.2f+%4.2f*j" % (ff.real, ff.imag)),
        print
    print


def DOS_LDA(lmt, om):
    
    (ndim,nkp) = (lmt.ndim, lmt.nkp)
    mu = lmt.mu
    # Computes eigensystem 
    Et = array(zeros((ndim,nkp), dtype=complex), order='F')
    Al = array(zeros((ndim,ndim,nkp), dtype=complex), order='F')
    Ar = array(zeros((ndim,ndim,nkp), dtype=complex), order='F')
    for ikp in range(nkp):
        (Et[:,ikp], Al[:,:,ikp], Ar[:,:,ikp]) = Eigensystemf(lmt.hamf[:,:,ikp], lmt.olap[:,:,ikp])

    DOS=[]
    for im,ome in enumerate(om):
        # Computes tetrahedron weights
        wgh = tetra.tetra_weights(ome+mu+1e-10j, Et, lmt.itt, 0)

        # Computes eigensystem and local green's function using
        # above computed tetrahedron weights
        gloc = matrix(zeros((ndim,ndim),dtype=complex))
        for ikp in range(nkp):
            gloc += matrix(la.zproduct_ada(Ar[:,:,ikp], wgh[:,ikp], Al[:,:,ikp])) # matrix product: Ar*wgh*Al
            
        gloc = array(gloc, order='F')
        # Symmetrization replaces summation over full 1-st BZ
        #  sym.symmetrize_cubic(gloc, lmt.T2C)  !!!!????brisi

        D = [-gloc[p,p].imag/pi for p in range(ndim)]

        dDOS = [sum(D)] + D[:]
        
        DOS.append(dDOS)
        print im, ome, sum(D)
        
    return DOS

def DOS_LDA_small(Hk, mu, lmt, om):
    
    (ndim,nkp) = (len(Hk[0]), lmt.nkp)
    # Computes eigensystem
    Et = array(zeros((ndim,nkp), dtype=complex), order='F')
    Ar = array(zeros((ndim,ndim,nkp), dtype=complex), order='F')
    for ikp in range(nkp):
        (tEt, tAr) = eigh(Hk[ikp])
        Et[:,ikp] = tEt
        Ar[:,:,ikp] = tAr

    DOS=[]
    for im,ome in enumerate(om):
        # Computes tetrahedron weights
        wgh = tetra.tetra_weights(ome+mu+1e-10j, Et, lmt.itt, 0)

        # Computes eigensystem and local green's function using
        # above computed tetrahedron weights
        gloc = matrix(zeros((ndim,ndim),dtype=complex))
        for ikp in range(nkp):
            gloc += matrix(la.zproduct_ada(Ar[:,:,ikp], wgh[:,ikp], conjugate(Ar[:,:,ikp].transpose()))) # matrix product: Ar*wgh*Al
            
        gloc = array(gloc, order='F')
        # Symmetrization replaces summation over full 1-st BZ
        #  sym.symmetrize_cubic(gloc, lmt.T2C)  !!!!????brisi

        D = [-gloc[p,p].imag/pi for p in range(ndim)]

        dDOS = [sum(D)] + D[:]
        
        DOS.append(dDOS)
        print im, ome, sum(D)
        
    return DOS
        

def GivePath(arr, Np):
    """ given array of k-points in the path, generates a
    dense list of momentum points in the choosen bath
    (Np points between each pair of points in the array)
    """
    kmsh = arange(0, 1, 1./Np)
    tot_path = []
    arr = array(arr)
    for i in range(len(arr)-1):
        path = map(lambda x: (arr[i] + (arr[i+1]-arr[i])*x), kmsh)
        tot_path.extend(path)
    return tot_path

def ComputeDistance(kps):
    """ given a list of k-points returns the length of the path along
    the choosen path for each k-point.
    """
    dst=[0]
    totd=0
    for ik in range(1,len(kps)):
        dk = kps[ik]-kps[ik-1]
        dl = sqrt(dot(dk, dk))
        totd += dl
        dst.append(totd)
    return dst


def symmetrize(Hr, Rs):    
    for iR in range(len(Rs)):
        HR = array(Hr[iR]); n = len(HR)
        Hd = 0.5*(HR[:n/2,:n/2]+HR[n/2:n,n/2:n])
        V = 0.5*(HR[:n/2,n/2:n]+HR[n/2:n,:n/2].transpose())
        Hr[iR] = hstack([vstack([Hd,V.transpose()]),vstack([V,Hd])])
    
def prntnum(z, space=20, small=1e-16):
    if abs(z.imag)<small :
        form = '%'+str(space)+'f'
        return form % z.real
    elif abs(z.real)<small :
        form = '%'+str(space-1)+'fj'
        return form % z.imag
    else:
        form = '%'+str(space)+'s'
        return form % z

def prnta(fh, A, space=20, small=1e-6):
    print >> fh, '['
    for p in range(len(A)):
        print >> fh, '[',
        for q in range(len(A[p])-1): print >> fh, prntnum(A[p,q],space,small), ',',
        print >> fh, prntnum(A[p,-1],space,small), '],'
    print >> fh, ']'

if __name__ == '__main__':

    fh_info = sys.stdout
    inpdir = '.'
    pcix = {(1,2):2,(3,4,5,6):1}
    La=-8.; Lb=6.;
    om = arange(La,Lb,(Lb-La)/300.)
    kpath = [[0,0,0],[0.5,0.5,0.5],[0,0.5,0.5],[0,0.5,0],[0,0,0]]
    Nkp = 30
    minb = 16 - 4*3 # 16
    maxb = 16-1 + 2*3 + 2*2 # 21
    cmp_DOS = False
    cmp_DOS_small = True
    symmetrizeQ = False
    
    lmt = Lmtart(inpdir, fh_info)

    lmt.check_cix(pcix)
    lmt.read_cix()
    
    trn.transform2(lmt.olap, lmt.hamf, lmt.Utk)  # transforms to DMFT base

    
    if (cmp_DOS):
        # Computes DOS and writes to file 'lda.dos'
        (DOS) = DOS_LDA(lmt, om)
        fh_dos=open('lda.dos', 'w')
        for im in range(len(om)):
            print >> fh_dos, om[im], "%10.5f "*len(DOS[im]) % tuple(DOS[im])
        fh_dos.close()


    # Computes Hk in all points of the Brillouin zone
    kmesh=[]
    Hk=[]
    Ok=[]
    kind=[]
    for k0 in range(lmt.ndiv[0]):
        for k1 in range(lmt.ndiv[1]):
            for k2 in range(lmt.ndiv[2]):
                kmesh.append(array([k0/float(lmt.ndiv[0]), k1/float(lmt.ndiv[1]), k2/float(lmt.ndiv[2])]))
                ikp = lmt.kirrptr[k0, k1, k2]-1
                kind.append(ikp)
                hamk = array(copy.deepcopy(lmt.hamf[:,:,ikp]),order='F')
                olpk = array(copy.deepcopy(lmt.olap[:,:,ikp]),order='F')
                Hk.append(hamk)
                Ok.append(olpk)
                
                
    kmesh=array(kmesh, order='F',dtype=float)
    Hk=array(Hk, order='F', dtype=complex)
    Ok=array(Ok, order='F', dtype=complex)

    dcorr = utl.flatten(lmt.vSigind)
    icorr = [lmt.Sigind[ic][ic] for ic in dcorr]    
    if sum(sort(icorr)-array(range(1,len(icorr)+1)))==0: # icor contains right seuqence of integers
        corr = zeros(len(icorr),dtype=int)
        for i in range(len(icorr)): corr[icorr[i]-1] = dcorr[i]               # kcix.dat contains  sequence of orbitals
        print 'reordering corr'
    else:
        corr = dcorr
        
    print 'corr = ', corr

    Hks=[]
    for ik,ks in enumerate(kmesh):
        (En, Al, Ar) = Eigensystemf(Hk[ik], Ok[ik])

        B = linalg.inv(Ar)
        C = linalg.inv(Al)
        # temp([low-energy-bands,correlated-orbitals])
        tempB=[]
        tempC=[]
        for p in corr:
            tempB.append(B[minb:maxb+1,p])
            tempC.append(C[p,minb:maxb+1])
        tempB = array(tempB).transpose()
        tempC = array(tempC)
        print 'shape(tempB)=', shape(tempB)
        print 'shape(tempC)=', shape(tempC)

        Z,D,V=svd(tempB) # singular value decompostion to orthogonalize...
        I=zeros((len(Z),len(V)), dtype=complex)
        for i in range(min(len(Z),len(V))):I[i,i]=1.0 # construct identity...
        Tr = matrix(Z)* matrix(I) * matrix(V)  # form the orthogonalized wannier...

        Z,D,V=svd(tempC) # singular value decompostion to orthogonalize...
        I=zeros((len(Z),len(V)), dtype=complex)
        for i in range(min(len(Z),len(V))):I[i,i]=1.0 # construct identity...
        Tl = matrix(Z)* matrix(I) * matrix(V)  # form the orthogonalized wannier...
        
        # eigenvalues in matrix form
        EV = zeros((maxb-minb+1, maxb-minb+1), dtype=complex)
        for i in range(len(EV)): EV[i,i] = En[i+minb]-lmt.mu
        hmk = matrix(Tl) * matrix(EV) * matrix(Tr)


        hmkn = 0.5*(hmk + conjugate(hmk.transpose()))
        #hmkn = hmk
        
        Hks.append(array(hmkn))
        
        #Al = conjugate(Ar.transpose())
        #E0 = array(matrix(Al) * matrix(Hk[ik]) * matrix(Ar))
        #print ik
        #print_m0(E0)

    Hks=array(Hks, order='F')

    r0 = int(lmt.ndiv[0]/2); d0 = (lmt.ndiv[0]+1)%2
    r1 = int(lmt.ndiv[1]/2); d1 = (lmt.ndiv[1]+1)%2
    r2 = int(lmt.ndiv[2]/2); d2 = (lmt.ndiv[2]+1)%2
    Rs=[]
    for i0 in range(-r0+d0,r0+1):
        for i1 in range(-r1+d1,r1+1):
            for i2 in range(-r2+d2,r2+1):
                Rs.append([i0,i1,i2])
    Rs = array(Rs, order='F', dtype=float)
    vec = identity(3)*2*pi

    if len(Rs)!= lmt.ndiv[0]*lmt.ndiv[1]*lmt.ndiv[2]:
        print 'len(Rs) is wrong!!!', len(Rs), lmt.ndiv[0]*lmt.ndiv[1]*lmt.ndiv[2]


    Hr = rfourier(Rs, vec, Hks, kmesh)


    if (symmetrizeQ): symmetrize(Hr, Rs)
            

    fh_ham=open('hamiltonian.dat', 'w')
    print >> fh_ham, 'lattice_unit=[',
    for r in range(3):
        print >> fh_ham, tuple(lmt.rvec[r][:]), ',',
    print >> fh_ham, ']'
    print >> fh_ham
    print >> fh_ham, 'orbital_positions=['
    for p in range(len(Hr[0])-1):
        print >> fh_ham, '(0.0, 0.0, 0.0),'
    print >> fh_ham, '(0.0, 0.0, 0.0)]'
    print >> fh_ham
    print >> fh_ham, 'mu=', 0.0
    print >> fh_ham
    print >> fh_ham, 'Hopping={}'
    for iR,R in enumerate(Rs):
        print >> fh_ham, 'Hopping[', tuple(map(int,R)), ']=',
        HR = Hr[iR]

        #print '['
        #for p in range(len(HR)-1):
        #    print >> fh_ham, '        [','%24s,'*len(HR[p]) % tuple(HR[p]), '],'
        #p = len(HR)-1
        #print >> fh_ham, '        [','%24s,'*len(HR[p]) % tuple(HR[p]), ']]\n'
        prnta(fh_ham, HR)
            

        

    kpts = GivePath(kpath, Nkp)
    dst = ComputeDistance(kpts)
    
    fh_bnd = open('bnds.dat', 'w')
    Hkn = kfourier(kpts, Hr, Rs, vec)
    for ik in range(len(kpts)):
        En = eigh(Hkn[ik])[0]
        print >> fh_bnd, dst[ik], "%8.4f "*len(En) % tuple(En)
        #print >> fh_bnd, ik, "%8.4f "*len(En) % tuple(En)


    Hr = rfourier(Rs, vec, Hk, kmesh)
    Or = rfourier(Rs, vec, Ok, kmesh)
    fh_bnd = open('bnds0.dat', 'w')
    Hkn = kfourier(kpts, Hr, Rs, vec)
    Okn = kfourier(kpts, Or, Rs, vec)
    for ik in range(len(kpts)):
        (En, Al, Ar) = Eigensystemf(Hkn[ik], Okn[ik])
        #En = eigh(Hkn[ik])[0]-lmt.mu
        print >> fh_bnd, dst[ik], "%8.4f "*len(En) % tuple(En-lmt.mu)
        #print >> fh_bnd, ik, "%8.4f "*len(En) % tuple(En)


    if (cmp_DOS_small):
        # Computes DOS and writes to file 'lda.dos'
        (DOS) = DOS_LDA_small(Hks, 0.0, lmt, om)
        fh_dos=open('slda.dos', 'w')
        for im in range(len(om)):
            print >> fh_dos, om[im], "%10.5f "*len(DOS[im]) % tuple(DOS[im])
        fh_dos.close()
