SUBROUTINE findNext(x, ix, R, n)
  IMPLICIT NONE
  INTEGER, intent(inout) :: ix
  REAL*8, intent(in)  :: R(n)
  REAL*8, intent(in)  :: x
  INTEGER, intent(in) :: n
  !f2py integer intent(hide), depend(R)      :: n = shape(R,0)
  INTEGER :: i
  
  if (x.le.R(1)) then
     ix=1
     return 
  endif
  if (x.gt.R(n-1)) then
     ix=n-1
     return
  endif
  if (ix.lt.1) then
     ix=1
  endif
  if (ix.gt.n) then
     ix=n
  endif
  if (x.ge.R(ix)) then
     do i=ix,n
        if (R(i).ge.x) then
           ix = i-1
           return
        endif
     enddo
     ix = n-1
     return
  else
     do i=ix,1,-1
        if (x.gt.R(i)) then
           ix = i
           return
        endif
     enddo
     ix=1
     return
  endif
end SUBROUTINE findNext

subroutine interp(res, f, x, ix, R, n)
  IMPLICIT NONE
  COMPLEX*16, intent(out) :: res
  COMPLEX*16, intent(in)  :: f(n)
  REAL*8, intent(in)  :: x
  INTEGER, intent(in) :: ix
  REAL*8, intent(in)  :: R(n)
  INTEGER, intent(in) :: n
  !f2py integer intent(hide), depend(R)      :: n = shape(R,0)
  if (x.lt.R(ix) .or. x.gt.R(ix+1)) then
     print *, 'Tezave:', x, R(ix), R(ix+1), ix, n
  endif
  res = f(ix) + (f(ix+1)-f(ix))*(x-R(ix))/(R(ix+1)-R(ix))
  return
end subroutine interp

subroutine YlmAYlm(Ar, Alm, Rm, Rxy, Ylm, Smt, nrad_s, nlm, Nxy)
  IMPLICIT NONE
  COMPLEX*16, intent(out) :: Ar(Nxy,Nxy)
  COMPLEX*16, intent(in)  :: Alm(nrad_s,nlm,nlm)  ! nlm=l*(l+2)+1 where l is lmaxb
  REAL*8, intent(in)      :: Rm(nrad_s)
  REAL*8, intent(in)      :: Rxy(Nxy,Nxy)
  COMPLEX*16, intent(in)  :: Ylm(Nxy,Nxy,nlm)
  REAL*8, intent(in)      :: Smt
  !REAL*8, intent(in)      :: coordinate(Nxy,Nxy, 5)
  INTEGER, intent(in)     :: nrad_s, nlm, Nxy
  !f2py integer intent(hide), depend(Rm)      :: nrad_s = shape(Rm,0)
  !f2py integer intent(hide), depend(Alm)     :: nlm = shape(Alm,1)
  !f2py integer intent(hide), depend(Ylm)     :: Nxy = shape(Ylm,0)
!!! Local varaibles
  INTEGER :: ilm1, ilm2, ix, iy
  INTEGER :: ir
  COMPLEX*16 :: A
  REAL*8 :: r
  Ar=0
  do ilm1=1,nlm
     ir=1
     do ilm2=1,nlm
        do ix=1,Nxy
           do iy=1,Nxy
              !r = coordinate(ix,iy,1)
              r = Rxy(ix,iy)
              if (r<=Smt) then
                 CALL findNext(r, ir, Rm, nrad_s)
                 CALL interp(A, Alm(:,ilm1,ilm2), r, ir, Rm, nrad_s)
                 !print *, r.ge.Rm(ir), r.le.Rm(ir+1), A
                 !print Ylm[ix,iy,ilm1]
                 !print Ylm[ix,iy,ilm2]
                 !print fAlm(Rxy[ix,iy])[0]
                 Ar(ix,iy) = Ar(ix,iy) + Ylm(ix,iy,ilm1)*A*conjg(Ylm(ix,iy,ilm2))
              endif
           enddo
        enddo
     enddo
  enddo
end subroutine YlmAYlm


subroutine testxx()
  IMPLICIT NONE
  INTEGER :: n
  REAL*8 :: R(11)
  COMPLEX*16 :: f(11)
  REAL*8 :: x
  INTEGER :: ix
  COMPLEX*16 :: res

  n=11
  R(1) = 0
  R(2) = 0.1
  R(3) = 0.2
  R(4) = 0.3
  R(5) = 0.4
  R(6) = 0.5
  R(7) = 0.6
  R(8) = 0.7
  R(9) = 0.8
  R(10)= 0.9
  R(11)= 1.0
  
  f(1) = 0
  f(2) = 0.1
  f(3) = 0.2
  f(4) = 0.3
  f(5) = 0.4
  f(6) = 0.5
  f(7) = 0.6
  f(8) = 0.7
  f(9) = 0.8
  f(10) = 0.9
  f(11) = 1.0

  ix=0
  x=-0.1
  CALL findNext(x, ix, R, n)
  CALL interp(res, f, x, ix, R, n)
  print *, x, ix, R(ix), R(ix+1), res
  x=0.35
  CALL findNext(x, ix, R, n)
  CALL interp(res, f, x, ix, R, n)
  print *, x, ix, R(ix), R(ix+1), res
  x=0.45
  CALL findNext(x, ix, R, n)
  CALL interp(res, f, x, ix, R, n)
  print *, x, ix, R(ix), R(ix+1), res
  x=0.48
  CALL findNext(x, ix, R, n)
  CALL interp(res, f, x, ix, R, n)
  print *, x, ix, R(ix), R(ix+1), res
  x=0.51
  CALL findNext(x, ix, R, n)
  CALL interp(res, f, x, ix, R, n)
  print *, x, ix, R(ix), R(ix+1), res
  x=1.1
  CALL findNext(x, ix, R, n)
  CALL interp(res, f, x, ix, R, n)
  print *, x, ix, R(ix), R(ix+1), res
  
end subroutine testxx

