!**************************************************!
!*******  Copyright by Kristjan Haule, 2002 ******!
!**************************************************!
SUBROUTINE cmp_olocef(olap, Utk, wk, T2CJ, symmetrize_cubic, inf_buf, ndim, nkp)
  IMPLICIT NONE
  !---------- Passed variables ----------
  COMPLEX*16,     intent(in)    :: olap(ndim,ndim,nkp)
  COMPLEX*16,     intent(in)    :: Utk(ndim,ndim,nkp)
  REAL*8,         intent(in)    :: wk(nkp)
  COMPLEX*16,     intent(in)    :: T2CJ(ndim,ndim)
  INTEGER,        intent(in)    :: ndim, nkp
  CHARACTER*10000, intent(inout) :: inf_buf
  !f2py integer intent(hide), depend(olap)  :: ndim=shape(olap,0)
  !f2py integer intent(hide), depend(olap)  :: nkp=shape(olap,2)
  EXTERNAL                :: symmetrize_cubic
  !f2py COMPLEX*16        :: dummy_argm1(ndim,ndim), dummy_argm2(ndim,ndim)
!!!  !f2py CALL symmetrize_cubic(dummy_argm,ndim)
!f2py CALL symmetrize_cubic(dummy_argm1,dummy_argm2,ndim)
  !---------- Local variables ----------
  CHARACTER*256 :: fh_info
  INTEGER       :: i, j
  INTEGER       :: ikp
  COMPLEX*16    :: ctmp
  COMPLEX*16, ALLOCATABLE :: olminv(:,:)
  ! For inverse and blas multiplication only
  COMPLEX*16, ALLOCATABLE :: oloceff(:,:)
  COMPLEX*16, ALLOCATABLE :: work(:)
  INTEGER :: info
  !---------- Allocate local memory ----------
  ALLOCATE( olminv(ndim,ndim), work(ndim), oloceff(ndim,ndim))
  oloceff = 0
  !---------- Compute the rotated average ----------
  ! The matrix oloceff is the inverse of local overlap. 
  ! To symmetrize oloceff, one needs to be in spheric
  ! harmonics, later we go to cubic harmonics
  DO ikp=1,nkp
     olminv = olap(:,:,ikp)
     CALL CINV (ndim,olminv,ndim,work,info)
     CALL ZTransform(olminv, Utk(:,:,ikp),'C',ndim)
     oloceff = oloceff + olminv*wk(ikp)
  ENDDO
  ! Symmetrizes the result
!  CALL symmetrize_spheric(oloceff,ndim)
  CALL symmetrize_cubic(oloceff,T2CJ,ndim)
!  CALL ZTransform(oloceff,T2CJ,'N',ndim) ! To cubic harmonics

  inf_buf = ""
  WRITE(fh_info,*) "CMP_OLOCEF: The effective local overlap matrix"
  inf_buf = TRIM(inf_buf)//'\n'//TRIM(fh_info)
  WRITE(fh_info,*) '          |      |       diagonal    off-diagonal |'
  inf_buf = TRIM(inf_buf)//'\n'//TRIM(fh_info)
  DO i=1,ndim
     ctmp=0
     DO j=1,ndim
        IF (i.NE.j) ctmp = ctmp + abs(oloceff(i,j))
     ENDDO
     ctmp = ctmp/ndim
     WRITE(fh_info,fmt='(11x,"|",2x,I3,2x,"|",3x,2f12.6,"|",3x,2f12.6,5x,"|")') i, dreal(oloceff(i,i)), dreal(ctmp)
     inf_buf = TRIM(inf_buf)//'\n'//TRIM(fh_info)
  ENDDO
  inf_buf = TRIM(inf_buf)//'\n'

  DEALLOCATE(olminv, work, oloceff)
  !---------- Look at the clock ----------
  RETURN
END SUBROUTINE cmp_olocef
