
SUBROUTINE cmp_rho_coefficients(ghh, ghj, gjh, gjj, inpdir, weight, Al, Ar, bndind, lbndind, lndim, ndim, nkp)
  !******************************************************************************!
  !*  To calculate DOS, rho(r) or Gf, we need                                   *!
  !*   1) ghh = A^r.w.A^l                                                       *!
  !*   1) ghj = A^r.w.A^l.Sk^+                                                  *!
  !*   2) gjh = Sk.A^r.w.A^l                                                    *!
  !*   3) gjj = Sk.A^r.w.A^l.Sk^+                                               *!
  !******************************************************************************!
  USE msk, ONLY: open_file, close_file, readS
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out) :: ghh(ndim,ndim), ghj(ndim,lndim), gjh(lndim,ndim), gjj(lndim,lndim)
  CHARACTER*1000, intent(in) :: inpdir
  COMPLEX*16, intent(in)  :: weight(ndim,nkp)
  COMPLEX*16, intent(in)  :: Al(ndim,ndim,nkp)
  COMPLEX*16, intent(in)  :: Ar(ndim,ndim,nkp)
  INTEGER, intent(in)     :: bndind(ndim,4)
  INTEGER, intent(in)     :: lbndind(lndim,4)
  INTEGER, intent(in)     :: lndim, ndim, nkp
  !f2py integer intent(hide), depend(weight)  :: ndim=shape(weight,0)
  !f2py integer intent(hide), depend(weight)  :: nkp=shape(weight,1)
  !f2py integer intent(hide), depend(lbndind) :: lndim=shape(lbndind,0)
  !---------- Local variables ----------
  INTEGER    :: ikp              ! Loop index for k-point loop
  COMPLEX*16 :: Sk(lndim,ndim)   ! temporary memory for structure constant
  COMPLEX*16 :: cnhh(ndim,ndim), cnhj(ndim,lndim) ! temporary
  COMPLEX*16 :: temp(ndim,ndim)
  COMPLEX*16 :: ena
  INTEGER    :: iatom, natom, p

  ena = dcmplx(1.0,0.0)

  natom=0 ! will become max(bndind(#,1))
  DO p=1,ndim
     iatom = bndind(p,1)
     IF (iatom.GT.natom) natom = iatom
  ENDDO

  ghh=0
  ghj=0
  gjh=0
  gjj=0

  CALL open_file(inpdir, natom)
  DO ikp=1,nkp
     CALL readS(Sk, ikp, lbndind, bndind, lndim, ndim)
     CALL ZProduct_ADAt(cnhh, Ar(:,:,ikp), weight(:,ikp), Al(:,:,ikp), ndim, ndim, ndim, temp) !cnhh=A^R.w.A^L
     CALL ZProduct_MM(cnhj, cnhh,Sk,'N','C', ndim, ndim, lndim, ndim, ndim, lndim) ! cnhj = cnhh.Sk^+
     CALL ZProduct_sum_MM(gjh, Sk, cnhh, 'N', 'N', lndim, ndim, ndim, ndim, lndim, ndim) ! gjh = gjh + Sk.cnhh
     CALL ZProduct_sum_MM(gjj, Sk, cnhj, 'N', 'N', lndim, ndim, ndim, lndim, lndim, lndim) ! gjj = gjj + Sk.cnhj
     ghh = ghh + cnhh
     ghj = ghj + cnhj
  ENDDO
  CALL close_file

END SUBROUTINE cmp_rho_coefficients

SUBROUTINE give_density(nxy, gxy, gyx, nspin, ndim1, ndim2)
  !*********************************************************!
  !* From G computes density by n=(G-G^+)/(2*pi*i)         *!
  !*********************************************************!
  IMPLICIT NONE
  COMPLEX*16, intent(out) :: nxy(ndim1,ndim2)
  COMPLEX*16, intent(in)  :: gxy(ndim1,ndim2)
  COMPLEX*16, intent(in)  :: gyx(ndim2,ndim1)
  INTEGER, intent(in)     :: nspin, ndim1, ndim2
  !f2py integer intent(hide), depend(gxy)  :: ndim1=shape(gxy,0)
  !f2py integer intent(hide), depend(gxy)  :: ndim2=shape(gxy,1)
! temporary variables
  REAL*8, parameter :: pi     = 3.14159266!358979323844
  COMPLEX*16 :: cxy(ndim1,ndim2)
  
  cxy = transpose(conjg(gyx))
  ! additional conjugation for using advaced rather than retarded functions
  nxy = conjg(gxy-cxy)/(2*pi*dcmplx(0,1.)*nspin/2.)
END SUBROUTINE give_density

SUBROUTINE cmp_occupancy(mocc, nhh,nhj,njh,njj, moo, bndind, is, llmtind, ndim, lndim, lmax, nsort, nspin, illsbmax2, natom)
  !***************************************************************************!
  !*  From the matrix elements of Density in the LMTO base computed above    *!
  !*  (nhh,nhj,njh,njj), computes a full density matrix                      *!
  !***************************************************************************!
  IMPLICIT NONE
!!! Passed variables
  COMPLEX*16, intent(out) :: mocc(ndim,ndim)
  COMPLEX*16, intent(in)  :: nhh(ndim,ndim), nhj(ndim,lndim), njh(lndim,ndim), njj(lndim,lndim)
  COMPLEX*16, intent(in)  :: moo(4,0:lmax,nsort,nspin)
  INTEGER, intent(in)     :: bndind(ndim,4)
  INTEGER, intent(in)     :: is(natom)
  INTEGER, intent(in)     :: llmtind(illsbmax2,natom)
  INTEGER, intent(in)     :: ndim, lndim, lmax, nsort, nspin, illsbmax2, natom
  !f2py integer intent(hide), depend(nhh)  :: ndim=shape(nhh,0)
  !f2py integer intent(hide), depend(njj)  :: lndim=shape(njj,0)
  !f2py integer intent(hide), depend(is)   :: natom=shape(is,0)
  !f2py integer intent(hide), depend(moo)  :: lmax=shape(moo,1)-1
  !f2py integer intent(hide), depend(moo)  :: nsort=shape(moo,2)
  !f2py integer intent(hide), depend(moo)  :: nspin=shape(moo,3)
  !f2py integer intent(hide), depend(llmtind) :: illsbmax2=shape(llmtind,0)
!!! Local variables
  INTEGER    :: iatom,  l,  m,  ispin,  lm,  idimh, idimj
  INTEGER    :: iatom1, l1, m1, ispin1, lm1, jdimh, jdimj
  INTEGER    :: isort
!  COMPLEX*16 :: nhh(ndim,ndim), nhj(ndim,lndim), njh(lndim,ndim), njj(lndim,lndim)


!  CALL give_density(nhh, ghh, ghh, nspin,  ndim,  ndim)
!  CALL give_density(nhj, ghj, gjh, nspin,  ndim, lndim)
!  CALL give_density(njh, gjh, ghj, nspin, lndim,  ndim)
!  CALL give_density(njj, gjj, gjj, nspin, lndim, lndim)

  mocc=0
  DO idimh=1,ndim
     iatom = bndind(idimh,1)
     l     = bndind(idimh,2)
     m     = bndind(idimh,3)
     ispin = bndind(idimh,4)
     lm = l*(l+1)+m+1
     idimj = llmtind(lm,iatom) + (ispin-1)*lndim/2
     isort = is(iatom)
     DO jdimh=1,ndim
        iatom1 = bndind(jdimh,1)
        l1     = bndind(jdimh,2)
        ispin1 = bndind(jdimh,4)
        IF (.NOT.(iatom.EQ.iatom1 .AND. l.EQ.l1 .AND. ispin.EQ.ispin1)) CYCLE
        m1     = bndind(jdimh,3)
        lm1 = l*(l+1)+m1+1
        jdimj = llmtind(lm1,iatom) + (ispin-1)*lndim/2
        mocc(idimh,jdimh) = conjg(nhh(idimh,jdimh))*moo(1,l,isort,ispin)&
             -conjg(njh(idimj,jdimh))*moo(2,l,isort,ispin)&
             -conjg(nhj(idimh,jdimj))*moo(3,l,isort,ispin)&
             +conjg(njj(idimj,jdimj))*moo(4,l,isort,ispin)
     ENDDO
  ENDDO

END SUBROUTINE cmp_occupancy

SUBROUTINE cmp_rho_valence(rhoval, isym_max, norbs, nhh,nhj,njh,njj, LmSym_a, ltpan_a, lmtind_a, llmtind_a, ilsv_s, Cf, gam, Fih, Fij, &
     ilst_s, ndim, lndim, nrad_s, nspin, limvmem, lmax, ilsbmax2, illsbmax2, lmax0, lmax1)
  IMPLICIT NONE
!!! Passed variables
  COMPLEX*16, intent(out)  :: rhoval(0:nrad_s,isym_max,nspin,norbs)
  COMPLEX*16, intent(in)   :: nhh(ndim,ndim), nhj(ndim,lndim), njh(lndim,ndim), njj(lndim,lndim)
  INTEGER, intent(in)      :: LmSym_a(limvmem)
  INTEGER, intent(in)      :: ltpan_a(0:lmax)
  INTEGER, intent(in)      :: lmtind_a(ilsbmax2)
  INTEGER, intent(in)      :: llmtind_a(illsbmax2)
  INTEGER, intent(in)      :: ilsv_s               ! maximum l for this sort
  REAL*8, intent(in)       :: Cf(0:lmax0-1,lmax1,lmax1), gam(0:ilst_s+1)
  COMPLEX*16, intent(in)   :: Fih(0:nrad_s,0:ilst_s,nspin)
  COMPLEX*16, intent(in)   :: Fij(0:nrad_s,0:ilst_s,nspin)
  INTEGER, intent(in)      :: isym_max, nspin, norbs
  INTEGER, intent(in)      :: ndim, lndim
  INTEGER, intent(in)      :: ilst_s               ! maximum l for this sort
  INTEGER, intent(in)      :: limvmem, lmax
  INTEGER, intent(in)      :: nrad_s
  INTEGER, intent(in)      :: ilsbmax2, illsbmax2
  INTEGER, intent(in)      :: lmax0, lmax1
  !f2py integer intent(hide), depend(Fih)      :: nrad_s = shape(Fih,0)-1
  !f2py integer intent(hide), depend(Fih)      :: nspin = shape(Fih,2)
  !f2py integer intent(hide), depend(nhh)      :: ndim = shape(nhh,0)
  !f2py integer intent(hide), depend(njj)      :: lndim = shape(njj,0)
  !f2py integer intent(hide), depend(LmSym_a)  :: limvmem = shape(LmSym_a,0)
  !f2py integer intent(hide), depend(ltpan_a)  :: lmax = shape(ltpan_a,0)-1
  !f2py integer intent(hide), depend(lmtind_a) :: ilsbmax2 = shape(lmtind_a,0)
  !f2py integer intent(hide), depend(llmtind_a):: illsbmax2 = shape(llmtind_a,0)
  !f2py integer intent(hide), depend(Cf)       :: lmax0 = shape(Cf,0)
  !f2py integer intent(hide), depend(Cf)       :: lmax1 = shape(Cf,1)
  !f2py integer intent(hide), depend(gam)      :: ilst_s = shape(gam,0)-2
!!! Local variables
  INTEGER    :: l, l1, l2, isym
  INTEGER    :: iupdn, iorbs, ispin, ispin1, m, m1, m2, lm, lm1, lm2
  INTEGER    :: idim1h, idim1j, idimh, idimj, irad
  LOGICAL    :: BLOG, BLOG1
  COMPLEX*16 :: CGNT

!  print *, 'ilst=', ilst_s 
!  print *, 'nrad_s=', nrad_s
!  print *, 'isym_max=', isym_max

  rhoval = 0

  DO iorbs=1,norbs !!! over spins
     DO iupdn=1,nspin !!! over spins
        IF(iorbs.EQ.1)THEN
           ispin1=iupdn
           ispin =iupdn
        ELSEIF(IORBS.EQ.2)THEN
           ispin1=iupdn
           ispin =nspin+1-iupdn
        ENDIF
        DO l1=0,ilst_s !!! over l1
           BLOG1=.FALSE.
           IF(ltpan_a(l1)) BLOG1=.TRUE.
           DO m1=-l1,l1 !!! over m1
              lm1=l1*(l1+1)+m1+1
              IF (BLOG1) idim1h = lmtind_a(lm1)+(ispin1-1)*ndim/2
              idim1j = llmtind_a(lm1)+(ispin1-1)*lndim/2
              DO l=0,ilst_s !!! over l
                 BLOG=.FALSE.
                 IF(ltpan_a(l)) BLOG=.TRUE.
                 DO m=-l,l !!! over m
                    lm=l*(l+1)+m+1
                    IF (BLOG) idimh = lmtind_a(lm)+(ispin-1)*ndim/2
                    idimj = llmtind_a(lm)+(ispin-1)*lndim/2
                    DO l2=ABS(l1-l),l1+l,2 !!! over l2 (expansion of ro)
                       IF(l2.LE.ilsv_s)THEN
                          m2=m-m1
                          lm2=l2*(l2+1)+m2+1
                          IF(ABS(m2).LE.l2)THEN !!! over m2 (expansion of ro)
                             isym=LmSym_a(lm2)
                             IF(isym.GT.0) THEN
                                CGNT=(0.D0,1.D0)**(L-L1-L2)*Cf(L2/2,LM,LM1)
                                DO irad=0,nrad_s	!!! over radial mesh
                                   IF(BLOG1.AND.BLOG)RHOVAL(irad,isym,iupdn,iorbs)=RHOVAL(irad,isym,iupdn,iorbs)+(nhh(idim1h,idimh)*CONJG(FIH(irad,l1,ispin1))*FIH(irad,l,ispin))*CGNT
                                   IF(BLOG1)         RHOVAL(irad,isym,iupdn,iorbs)=RHOVAL(irad,isym,iupdn,iorbs)-(nhj(idim1h,idimj)*CONJG(FIH(irad,l1,ispin1))*FIJ(irad,l,ispin)*gam(L))*CGNT
                                   IF(BLOG)          RHOVAL(irad,isym,iupdn,iorbs)=RHOVAL(irad,isym,iupdn,iorbs)-(njh(idim1j,idimh)*CONJG(FIJ(irad,l1,ispin1))*FIH(irad,l,ispin)*gam(L1))*CGNT
                                   RHOVAL(irad,isym,iupdn,iorbs)=RHOVAL(irad,isym,iupdn,iorbs)+(njj(idim1j,idimj)*CONJG(FIJ(irad,l1,ispin1))*FIJ(irad,l,ispin)*gam(L1)*gam(L))*CGNT
                                ENDDO	!!! over radial mesh
                             ENDIF
                          ENDIF !!! over m2
                       ENDIF !!!      l2
                    ENDDO !!! over l2 
                 ENDDO !!! over m
              ENDDO !!! over l
           ENDDO !!! over m1
        ENDDO !!! over l1
     ENDDO
  ENDDO
END SUBROUTINE cmp_rho_valence


SUBROUTINE cmp_tgf(ghh, ghj, gjh, gjj, hh, jj, dioloc, norb, gf, ndim, lndim)
  !******************************************************************************!
  !*  To calculate Gloc in DMFT base from coefficients ghh,ghj,gjh,gjj          *!
  !*   1) ghh = A^r.w.A^l                                                       *!
  !*   1) ghj = A^r.w.A^l.Sk^+                                                  *!
  !*   2) gjh = Sk.A^r.w.A^l                                                    *!
  !*   3) gjj = Sk.A^r.w.A^l.Sk^+                                               *!
  !******************************************************************************!
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out) :: gf(ndim,ndim)
  COMPLEX*16, intent(in)  :: ghh(ndim,ndim), ghj(ndim,lndim), gjh(lndim,ndim), gjj(lndim,lndim)
  COMPLEX*16, intent(in)  :: hh(ndim), jj(ndim)
  REAL*8, intent(in)      :: dioloc(ndim)
  INTEGER, intent(in)     :: norb
  INTEGER, intent(in)     :: ndim, lndim
  !f2py integer intent(hide), depend(ghh)  :: ndim=shape(ghh,0)
  !f2py integer intent(hide), depend(gjj)  :: lndim=shape(gjj,0)
  INTEGER    :: ph, pj, qh, qj
  COMPLEX*16 :: gf0
  do ph=1,ndim
     pj = ph + (lndim/2-ndim/2)*int(ph/(norb+1))
     do qh=1,ndim
        qj = qh + (lndim/2-ndim/2)*int(qh/(norb+1))
        gf0 = hh(ph)*ghh(ph,qh)*conjg(hh(qh))-hh(ph)*ghj(ph,qj)*conjg(jj(qh))-jj(ph)*gjh(pj,qh)*conjg(hh(qh))+jj(ph)*gjj(pj,qj)*conjg(jj(qh))
        gf(ph,qh) = dioloc(ph)*gf0*dioloc(qh)
     enddo
  enddo
END SUBROUTINE cmp_tgf

SUBROUTINE cmp_oho(oho, olap, hamf, wk, ndim, nkp)
  !******************************************************************************!
  ! *  Computes  sum_k O^{-1}*Hk*O^{-1} in DMFT base
  !******************************************************************************!
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out) :: oho(ndim,ndim)
  COMPLEX*16, intent(in)  :: olap(ndim,ndim,nkp)
  COMPLEX*16, intent(in)  :: hamf(ndim,ndim,nkp)
  REAL*8,     intent(in)  :: wk(nkp)
  INTEGER, intent(in)     :: ndim, nkp
  !f2py integer intent(hide), depend(hamf)  :: ndim=shape(hamf,0)
  !f2py integer intent(hide), depend(hamf)  :: nkp=shape(hamf,2)
  COMPLEX*16 :: ham(ndim,ndim), oinv(ndim,ndim), oh(ndim,ndim), work(ndim)
  INTEGER :: ikp, info
  oho = 0.0
  DO ikp=1,nkp
     ham = hamf(:,:,ikp)*wk(ikp)
     oinv = olap(:,:,ikp)
     CALL CINV (ndim, oinv, ndim, work, info)
     CALL ZProduct_MM(oh, oinv, ham, 'N','N', ndim, ndim, ndim, ndim, ndim, ndim)       ! oh = O^{-1}*Hk
     CALL ZProduct_sum_MM(oho, oh, oinv, 'N', 'N', ndim, ndim, ndim, ndim, ndim, ndim)  ! oho += oh*oinv 
  ENDDO
END SUBROUTINE cmp_oho
SUBROUTINE cmp_ohol(oho, olap, hamf, Utk, wk, ndim, nkp)
  !******************************************************************************!
  ! *  Computes  sum_k O^{-1}*Hk*O^{-1} in LMTO base
  !******************************************************************************!
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out) :: oho(ndim,ndim)
  COMPLEX*16, intent(in)  :: olap(ndim,ndim,nkp)
  COMPLEX*16, intent(in)  :: hamf(ndim,ndim,nkp)
  COMPLEX*16, intent(in)  :: Utk(ndim,ndim,nkp)
  REAL*8,     intent(in)  :: wk(nkp)
  INTEGER, intent(in)     :: ndim, nkp
  !f2py integer intent(hide), depend(hamf)  :: ndim=shape(hamf,0)
  !f2py integer intent(hide), depend(hamf)  :: nkp=shape(hamf,2)
  COMPLEX*16 :: ham(ndim,ndim), oinv(ndim,ndim), oh(ndim,ndim), woho(ndim,ndim), work(ndim)
  INTEGER :: ikp, info
  oho = 0.0
  DO ikp=1,nkp
     ham = hamf(:,:,ikp)*wk(ikp)
     oinv = olap(:,:,ikp)
     CALL CINV (ndim, oinv, ndim, work, info)
     CALL ZProduct_MM(oh, oinv, ham, 'N','N', ndim, ndim, ndim, ndim, ndim, ndim)       ! oh = O^{-1}*Hk
     CALL ZProduct_MM(woho, oh, oinv, 'N', 'N', ndim, ndim, ndim, ndim, ndim, ndim)     ! oho = oh*oinv 
     CALL ZTransform(woho, Utk(:,:,ikp),'C', ndim)
     oho(:,:) = oho(:,:) + woho(:,:)
  ENDDO
END SUBROUTINE cmp_ohol

SUBROUTINE cmp_oso(oso, olap, sinf, wk, ndim, nkp)
  !******************************************************************************!
  ! *  Computes  sum_k O^{-1}*Sigma*O^{-1}
  !******************************************************************************!
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out) :: oso(ndim,ndim)
  COMPLEX*16, intent(in)  :: olap(ndim,ndim,nkp)
  COMPLEX*16, intent(in)  :: sinf(ndim,ndim)
  REAL*8,     intent(in)  :: wk(nkp)
  INTEGER, intent(in)     :: ndim, nkp
  !f2py integer intent(hide), depend(olap)  :: ndim=shape(olap,0)
  !f2py integer intent(hide), depend(olap)  :: nkp=shape(olap,2)
  COMPLEX*16 :: ham(ndim,ndim), oinv(ndim,ndim), os(ndim,ndim), work(ndim)
  INTEGER :: ikp, info
  oso = 0.0
  DO ikp=1,nkp
     ham = sinf*wk(ikp)
     oinv = olap(:,:,ikp)
     CALL CINV (ndim, oinv, ndim, work, info)
     CALL ZProduct_MM(os, oinv, ham, 'N','N', ndim, ndim, ndim, ndim, ndim, ndim)     ! os = O^{-1}*Sigma
     CALL ZProduct_sum_MM(oso, os, oinv, 'N', 'N', ndim, ndim, ndim, ndim, ndim, ndim) ! oso += os*oinv 
  ENDDO
  oso(:,:) = oso(:,:)
END SUBROUTINE cmp_oso
SUBROUTINE cmp_osol(oso, olap, sinf, Utk, wk, ndim, nkp)
  !******************************************************************************!
  ! *  Computes  sum_k O^{-1}*Sigma*O^{-1}
  !******************************************************************************!
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out) :: oso(ndim,ndim)
  COMPLEX*16, intent(in)  :: olap(ndim,ndim,nkp)
  COMPLEX*16, intent(in)  :: sinf(ndim,ndim)
  COMPLEX*16, intent(in)  :: Utk(ndim,ndim,nkp)
  REAL*8,     intent(in)  :: wk(nkp)
  INTEGER, intent(in)     :: ndim, nkp
  !f2py integer intent(hide), depend(olap)  :: ndim=shape(olap,0)
  !f2py integer intent(hide), depend(olap)  :: nkp=shape(olap,2)
  COMPLEX*16 :: ham(ndim,ndim), oinv(ndim,ndim), os(ndim,ndim), work(ndim)
  INTEGER :: ikp, info
  oso = 0.0
  DO ikp=1,nkp
     ham = sinf*wk(ikp) ! Maybe a bug! I guess sinf is still in DMFT!
     oinv = olap(:,:,ikp)
     CALL CINV (ndim, oinv, ndim, work, info)
     CALL ZProduct_MM(os, oinv, ham, 'N', 'N', ndim, ndim, ndim, ndim, ndim, ndim) ! os = O^{-1}*Sigma
     CALL ZProduct_MM(ham, os, oinv, 'N', 'N', ndim, ndim, ndim, ndim, ndim, ndim) ! ham = os*oinv 
     CALL ZTransform(ham, Utk(:,:,ikp),'C', ndim)
     oso(:,:) = oso(:,:) + ham(:,:)
  ENDDO
  oso(:,:) = oso(:,:)
END SUBROUTINE cmp_osol


!!!! **************************************************************************!
!!!! Here is the new part of the code (29.10.2008) which is equivalent to      !
!!!! the above routine "cmp_rho_valence", but written in more readable way     !
!!!! with fewer do-loops, which were copied from LmtArt.                       !
!!!! The new reorganization of the code is actually used in the stm-density    !
!!!! which is computed below.                                                  !
!!!! **************************************************************************!
SUBROUTINE rho_inside(rhoval, FIX, FIY, nxy, l1,m1,l,m, LmSym_a, Cf, ilsv_s, nrad_s, isym_max, limvmem, lmax0, lmax1, ilst_s)
! rho_inside is auxiliary function called from "cmp_rho_valence2" 
  IMPLICIT NONE
  COMPLEX*16, intent(inout) :: rhoval(0:nrad_s,1:isym_max)
  COMPLEX*16, intent(in)    :: FIX(0:nrad_s), FIY(0:nrad_s)
  COMPLEX*16, intent(in)    :: nxy
  INTEGER, intent(in)       :: l1,m1,l,m
  INTEGER, intent(in)       :: LmSym_a(limvmem)
  REAL*8, intent(in)        :: Cf(0:lmax0-1,lmax1,lmax1)
  INTEGER, intent(in)       :: ilsv_s, nrad_s, isym_max, limvmem, lmax0, lmax1, ilst_s
  ! local varaibles
  INTEGER :: l2, m2, lm2, isym, lm1, lm, irad
  COMPLEX*16 :: CGNT, wnn

  lm1 = l1*(l1+1)+m1+1
  lm  = l *(l +1)+m +1
  do l2=abs(l1-l),l1+l,2 !!! over l2 (expansion of ro)
     m2=m-m1             !!! over m2 (expansion of ro)
     if (l2.le.ilsv_s .and. abs(m2).le.l2) then
        lm2=l2*(l2+1)+m2+1
        isym=LmSym_a(lm2)
        if (isym.gt.0) then
           CGNT=(0.D0,1.D0)**(l-l1-l2)*Cf(l2/2,lm,lm1)
           wnn = nxy*CGNT
           do irad=0,nrad_s	!!! over radial mesh
              rhoval(irad,isym) = rhoval(irad,isym) + wnn*CONJG(FIX(irad))*FIY(irad)
           enddo	!!! over radial mesh
        endif
     endif !!! over m2
  enddo    !!! over l2
end SUBROUTINE rho_inside

SUBROUTINE cmp_rho_valence2(rhoval, atom, isym_max, norbs, nhh,nhj,njh,njj, LmSym_a, bndind, lbndind,  &
     ilsv_s, Cf, gam, Fih, Fij, worbs, wupdn, ilst_s, ndim, lndim, nrad_s, nspin, limvmem, lmax0, lmax1)
!!! The equivalent function to "cmp_rho_valence", but rewritten to avoid multiple do lops and arange
!!! as sum over all full matrix rather then sum over small parts of the matrix.
  IMPLICIT NONE
!!! Passed variables
  COMPLEX*16, intent(out)  :: rhoval(0:nrad_s,isym_max,nspin,norbs)
  INTEGER, intent(in)      :: atom
  COMPLEX*16, intent(in)   :: nhh(ndim,ndim), nhj(ndim,lndim), njh(lndim,ndim), njj(lndim,lndim)
  INTEGER, intent(in)      :: LmSym_a(limvmem)
  INTEGER, intent(in)      :: bndind(ndim,4), lbndind(lndim,4)
  INTEGER, intent(in)      :: ilsv_s               ! maximum l for this sort
  REAL*8, intent(in)       :: Cf(0:lmax0-1,lmax1,lmax1), gam(0:ilst_s+1)
  COMPLEX*16, intent(in)   :: Fih(0:nrad_s,0:ilst_s,nspin)
  COMPLEX*16, intent(in)   :: Fij(0:nrad_s,0:ilst_s,nspin)
  INTEGER, intent(in)      :: worbs(2,2), wupdn(2,2)
  INTEGER, intent(in)      :: isym_max, nspin, norbs
  INTEGER, intent(in)      :: ndim, lndim
  INTEGER, intent(in)      :: ilst_s               ! maximum l for this sort
  INTEGER, intent(in)      :: limvmem
  INTEGER, intent(in)      :: nrad_s
  INTEGER, intent(in)      :: lmax0, lmax1
  !f2py integer intent(hide), depend(Fih)      :: nrad_s = shape(Fih,0)-1
  !f2py integer intent(hide), depend(Fih)      :: nspin = shape(Fih,2)
  !f2py integer intent(hide), depend(nhh)      :: ndim = shape(nhh,0)
  !f2py integer intent(hide), depend(njj)      :: lndim = shape(njj,0)
  !f2py integer intent(hide), depend(LmSym_a)  :: limvmem = shape(LmSym_a,0)
  !f2py integer intent(hide), depend(Cf)       :: lmax0 = shape(Cf,0)
  !f2py integer intent(hide), depend(Cf)       :: lmax1 = shape(Cf,1)
  !f2py integer intent(hide), depend(gam)      :: ilst_s = shape(gam,0)-2
!!! Local variables
  INTEGER    :: l, l1
  INTEGER    :: iupdn, iorbs, ispin, ispin1, m, m1, lm
  INTEGER    :: idim1h, idim1j, idimh, idimj, iatom, iatom1
  COMPLEX*16 :: nxy
  
  rhoval = 0
  !! HH term
  do idim1h=1,ndim
     iatom1 = bndind(idim1h,1)
     l1     = bndind(idim1h,2)
     m1     = bndind(idim1h,3)
     ispin1 = bndind(idim1h,4)
     do idimh=1,ndim
        iatom  = bndind(idimh,1)
        l      = bndind(idimh,2)
        m      = bndind(idimh,3)
        ispin  = bndind(idimh,4)
        if (iatom1.eq.atom .and. iatom.eq.atom) then
           iupdn = wupdn(ispin1,ispin)
           iorbs = worbs(ispin1,ispin)
           nxy = nhh(idim1h,idimh)
           CALL rho_inside(rhoval(:,:,iupdn,iorbs), Fih(:,l1,ispin1), Fih(:,l,ispin), nxy, l1,m1,l,m, LmSym_a, Cf, ilsv_s, nrad_s, isym_max, limvmem, lmax0, lmax1, ilst_s)
        endif
     enddo
  enddo

  !! JJ term
  do idim1j=1,lndim
     iatom1 = lbndind(idim1j,1)
     l1     = lbndind(idim1j,2)
     m1     = lbndind(idim1j,3)
     ispin1 = lbndind(idim1j,4)
     do idimj=1,lndim
        iatom = lbndind(idimj,1)
        l     = lbndind(idimj,2)
        m     = lbndind(idimj,3)
        ispin = lbndind(idimj,4)
        if (iatom1.eq.atom .and. iatom.eq.atom) then
           iupdn = wupdn(ispin1,ispin)
           iorbs = worbs(ispin1,ispin)
           nxy = gam(l1)*njj(idim1j,idimj)*gam(l)
           CALL rho_inside(rhoval(:,:,iupdn,iorbs), Fij(:,l1,ispin1), Fij(:,l,ispin), nxy, l1,m1,l,m, LmSym_a, Cf, ilsv_s, nrad_s, isym_max, limvmem, lmax0, lmax1, ilst_s)
           !write(6, '(A,2x,I,2x,I,2x,I,2x,I,2x,I,2x,I,2x,I,2x,I)'), 'hh=', l1, m1, ispin1, l, m, ispin, iupdn, iorbs
        endif
     enddo
  enddo

  !! HJ term
  do idim1h=1,ndim
     iatom1 = bndind(idim1h,1)
     l1     = bndind(idim1h,2)
     m1     = bndind(idim1h,3)
     ispin1 = bndind(idim1h,4)
     do idimj=1,lndim
        iatom = lbndind(idimj,1)
        l     = lbndind(idimj,2)
        m     = lbndind(idimj,3)
        ispin = lbndind(idimj,4)
        if (iatom1.eq.atom .and. iatom.eq.atom) then
           iupdn = wupdn(ispin1,ispin)
           iorbs = worbs(ispin1,ispin)
           nxy = -nhj(idim1h,idimj)*gam(l)
           CALL rho_inside(rhoval(:,:,iupdn,iorbs), Fih(:,l1,ispin1), Fij(:,l,ispin), nxy, l1,m1,l,m, LmSym_a, Cf, ilsv_s, nrad_s, isym_max, limvmem, lmax0, lmax1, ilst_s)
        endif
     enddo
  enddo

  !! JH term
  do idim1j=1,lndim
     iatom1 = lbndind(idim1j,1)
     l1     = lbndind(idim1j,2)
     m1     = lbndind(idim1j,3)
     ispin1 = lbndind(idim1j,4)
     do idimh=1,ndim
        iatom  = bndind(idimh,1)
        l      = bndind(idimh,2)
        m      = bndind(idimh,3)
        ispin  = bndind(idimh,4)
        if (iatom1.eq.atom .and. iatom.eq.atom) then
           iupdn = wupdn(ispin1,ispin)
           iorbs = worbs(ispin1,ispin)
           nxy = -gam(l1)*njh(idim1j,idimh)
           CALL rho_inside(rhoval(:,:,iupdn,iorbs), Fij(:,l1,ispin1), Fih(:,l,ispin), nxy, l1,m1,l,m, LmSym_a, Cf, ilsv_s, nrad_s, isym_max, limvmem, lmax0, lmax1, ilst_s)
        endif
     enddo
  enddo
END SUBROUTINE cmp_rho_valence2


!!!!************************************************************************************************
!!!!
!!!! Computes A(r,omega) as measured in STM experiemnst (inside the muffin tin sphere)
!!!!
!!!! A(vec{r},omega) is expanded in terms of spherical harmonics Y_{lm}=Y_L and it takes the form
!!!! A(vec{r},omega) =  Y_L(angle) A(omega,iatom)(r,L,L') Y_{L'}(angle) 
!!!! where r is radial distance from the  center of the muffin thin.
!!!! This routine computes spectral density at fixed omega and fixed iatom. It is a function of 
!!!! radial distance r and matrix in L,L' indices.
!!!!
!!!!************************************************************************************************
SUBROUTINE cmp_stm(Alm, atom, nhh,nhj,njh,njj, bndind, lbndind, gam, Fih, Fij, nlm, ilst_s, ndim, lndim, nrad_s, nspin)
  IMPLICIT NONE
!!! Passed variables
  COMPLEX*16, intent(out)  :: Alm(0:nrad_s,nlm,nlm)  ! nlm=l*(l+2)+1 where l is lmaxb
  INTEGER, intent(in)      :: atom
  COMPLEX*16, intent(in)   :: nhh(ndim,ndim), nhj(ndim,lndim), njh(lndim,ndim), njj(lndim,lndim)
  INTEGER, intent(in)      :: bndind(ndim,4), lbndind(lndim,4)
  REAL*8, intent(in)       :: gam(0:ilst_s+1)
  COMPLEX*16, intent(in)   :: Fih(0:nrad_s,0:ilst_s,nspin)
  COMPLEX*16, intent(in)   :: Fij(0:nrad_s,0:ilst_s,nspin)
  INTEGER, intent(in)      :: nlm
  INTEGER, intent(in)      :: nspin
  INTEGER, intent(in)      :: ndim, lndim
  INTEGER, intent(in)      :: ilst_s               ! maximum l for this sort
  INTEGER, intent(in)      :: nrad_s
  !f2py integer intent(hide), depend(Fih)      :: nrad_s = shape(Fih,0)-1
  !f2py integer intent(hide), depend(Fih)      :: nspin = shape(Fih,2)
  !f2py integer intent(hide), depend(nhh)      :: ndim = shape(nhh,0)
  !f2py integer intent(hide), depend(njj)      :: lndim = shape(njj,0)
  !f2py integer intent(hide), depend(gam)      :: ilst_s = shape(gam,0)-2
!!! Local variables
  INTEGER    :: l, l1
  INTEGER    :: iupdn, iorbs, ispin, ispin1, m, m1, lm, lm1
  INTEGER    :: idim1h, idim1j, idimh, idimj, iatom, iatom1, irad
  COMPLEX*16 :: nxy

  Alm = 0
  !! HH term
  do idim1h=1,ndim
     iatom1 = bndind(idim1h,1)
     l1     = bndind(idim1h,2)
     m1     = bndind(idim1h,3)
     ispin1 = bndind(idim1h,4)
     lm1 = l1*(l1+1)+m1+1
     do idimh=1,ndim
        iatom  = bndind(idimh,1)
        l      = bndind(idimh,2)
        m      = bndind(idimh,3)
        ispin  = bndind(idimh,4)
        lm = l*(l+1)+m+1
        if (iatom1.eq.atom .and. iatom.eq.atom .and. ispin1.eq.ispin) then
           nxy = nhh(idim1h,idimh) * (0.D0,1.D0)**(l-l1)
           do irad=0,nrad_s	!!! over radial mesh
              Alm(irad,lm1,lm) = Alm(irad,lm1,lm) + nxy*CONJG(Fih(irad,l1,ispin1))*Fih(irad,l,ispin)
           enddo	!!! over radial mesh
        endif
     enddo
  enddo

  !! JJ term
  do idim1j=1,lndim
     iatom1 = lbndind(idim1j,1)
     l1     = lbndind(idim1j,2)
     m1     = lbndind(idim1j,3)
     ispin1 = lbndind(idim1j,4)
     lm1 = l1*(l1+1)+m1+1
     do idimj=1,lndim
        iatom = lbndind(idimj,1)
        l     = lbndind(idimj,2)
        m     = lbndind(idimj,3)
        ispin = lbndind(idimj,4)
        lm = l*(l+1)+m+1
        if (iatom1.eq.atom .and. iatom.eq.atom .and. ispin1.eq.ispin .and. lm1.le.nlm .and. lm.le.nlm) then
           nxy = gam(l1)*njj(idim1j,idimj)*gam(l) * (0.D0,1.D0)**(l-l1)
           do irad=0,nrad_s	!!! over radial mesh
              Alm(irad,lm1,lm) = Alm(irad,lm1,lm) + nxy*CONJG(Fij(irad,l1,ispin1))*Fij(irad,l,ispin)
           enddo	!!! over radial mesh
        endif
     enddo
  enddo

  !! HJ term
  do idim1h=1,ndim
     iatom1 = bndind(idim1h,1)
     l1     = bndind(idim1h,2)
     m1     = bndind(idim1h,3)
     ispin1 = bndind(idim1h,4)
     lm1 = l1*(l1+1)+m1+1
     do idimj=1,lndim
        iatom = lbndind(idimj,1)
        l     = lbndind(idimj,2)
        m     = lbndind(idimj,3)
        ispin = lbndind(idimj,4)
        lm = l*(l+1)+m+1
        if (iatom1.eq.atom .and. iatom.eq.atom .and. lm.le.nlm) then
           nxy = -nhj(idim1h,idimj)*gam(l) * (0.D0,1.D0)**(l-l1)
           do irad=0,nrad_s	!!! over radial mesh
              Alm(irad,lm1,lm) = Alm(irad,lm1,lm) + nxy*CONJG(Fih(irad,l1,ispin1))*Fij(irad,l,ispin)
           enddo	!!! over radial mesh
        endif
     enddo
  enddo

  !! JH term
  do idim1j=1,lndim
     iatom1 = lbndind(idim1j,1)
     l1     = lbndind(idim1j,2)
     m1     = lbndind(idim1j,3)
     ispin1 = lbndind(idim1j,4)
     lm1 = l1*(l1+1)+m1+1
     do idimh=1,ndim
        iatom  = bndind(idimh,1)
        l      = bndind(idimh,2)
        m      = bndind(idimh,3)
        ispin  = bndind(idimh,4)
        lm = l*(l+1)+m+1
        if (iatom1.eq.atom .and. iatom.eq.atom .and. lm1.le.nlm) then
           nxy = -gam(l1)*njh(idim1j,idimh) * (0.D0,1.D0)**(l-l1)
           do irad=0,nrad_s	!!! over radial mesh
              Alm(irad,lm1,lm) = Alm(irad,lm1,lm) + nxy*CONJG(Fij(irad,l1,ispin1))*Fih(irad,l,ispin)
           enddo	!!! over radial mesh
        endif
     enddo
  enddo

END SUBROUTINE cmp_stm
