SUBROUTINE cmp_trans3(conduc, mu, T, sig, sZ, vel, hamf, olap, alphaV, latcon, unit_cell_volume, wk, ndim, nkp)
  use intgs, ONLY: init, ICP1, ICP2, ICQ2
  IMPLICIT NONE
  !---------- Passed variables ----------     
  REAL*8, intent(out)    :: conduc
  REAL*8, intent(in)     :: mu, T
  COMPLEX*16, intent(in) :: sig(ndim,ndim), sZ(ndim,ndim)
  COMPLEX*16, intent(in) :: vel(3,ndim,ndim,nkp)
  COMPLEX*16, intent(in) :: hamf(ndim,ndim,nkp)
  COMPLEX*16, intent(in) :: olap(ndim,ndim,nkp)
  REAL*8, intent(in)     :: alphaV(3,3)
  REAL*8, intent(in)     :: wk(nkp)
  INTEGER, intent(in)    :: ndim, nkp
  REAL*8, intent(in)     :: latcon, unit_cell_volume
  !f2py integer intent(hide), depend(hamf)   :: ndim=shape(hamf,0)
  !f2py integer intent(hide), depend(hamf)   :: nkp=shape(hamf,2)
  !---------- Local variables ----------
  REAL*8, PARAMETER :: pi = 3.14159265358979d0
  REAL*8, PARAMETER :: elresunit = 0.2173985  ! micro Ohm meter !!! Natural unit for electrical resistivity: [ aBohr*hbar/ecoul^2 ]
  REAL*8, PARAMETER :: big = 1e6
  REAL*8, PARAMETER :: small = 1e-12
  REAL*8, PARAMETER :: tiny = 5e-4
  COMPLEX*16, ALLOCATABLE :: Ar(:,:), Al(:,:), ek(:,:), Cv(:,:,:), Dv(:,:,:) ! Eigensystem data 
  COMPLEX*16, ALLOCATABLE :: htm(:,:), hqp(:,:) ! Eigensystem data 
  COMPLEX*16, ALLOCATABLE :: L1(:,:,:), L2(:,:,:), L3(:,:,:), L4(:,:,:), Ltmp(:,:) ! temporary data
  COMPLEX*16, ALLOCATABLE :: Sqp(:,:), Rqp(:,:)
  INTEGER    :: p,q,i                                ! Orbital loop indices
  INTEGER    :: ikp                                  ! Loop index for k-point loop
  COMPLEX*16 :: P1(ndim), P2(ndim), Q2(ndim)         ! Temporary variables for fast computation of logs
  COMPLEX*16 :: ctmp, denom_es, denom_os, wes, wos   ! Complex temporary variables
  INTEGER    :: idxarr(ndim)

  CALL INIT()
  ALLOCATE(Ar(ndim,ndim), Al(ndim,ndim), htm(ndim,ndim), hqp(ndim,ndim), ek(ndim,nkp), Cv(ndim,ndim,nkp), Dv(ndim,ndim,nkp))
  ALLOCATE(L1(ndim,ndim,3), L2(ndim,ndim,3), L3(ndim,ndim,3), L4(ndim,ndim,3), Ltmp(ndim,ndim))
  ALLOCATE(Sqp(ndim,ndim), Rqp(ndim,ndim))
!!!------------- Calculation of nominators consisting of velocities and eigenvectors
  DO ikp=1,nkp
     ! eigensystem
     ! quasiparticle hamiltonian
     
     hqp = matmul(sZ, (hamf(:,:,ikp) + sig - olap(:,:,ikp)*mu))
     hqp = matmul(hqp, sZ)

     CALL eigsys(hqp, olap(:,:,ikp), ek(:,ikp), Al, Ar, ndim)

     ! orders eigenvalues according to their real part. Also orders eigenvectors correspondingly
     CALL check_causality(ek(:,ikp), small, ndim)
     CALL eig_order_real_part(ek(:,ikp), idxarr, ndim)
     CALL permute_eigensystem(idxarr, ek(:,ikp), Al, Ar, ndim)
     
     Ar = matmul(sZ, Ar)
     Al = matmul(Al, sZ)

     ! total V
     CALL cmp_C_and_D(Cv(:,:,ikp), Dv(:,:,ikp), Al, Ar, Al, Ar, vel(:,:,:,ikp), alphaV, L1, L2, L3, L4, Ltmp, ndim)
  ENDDO

  ctmp = 0.0
  DO ikp=1,nkp

     DO i=1,ndim
        !print *, 'z=', ek(i,ikp)/T
        P1(i) = ICP1(ek(i,ikp)/T)/T
        !print *, 'p1=', P1(i)
        P2(i) = ICP2(ek(i,ikp)/T)/T**2
        !print *, 'p2=', P2(i), ICP2(ek(i,ikp)/T)/T**2
        Q2(i) = ICQ2(ek(i,ikp)/T)/T**2
        !print *, 'q2=', Q2(i)
        !print *, ikp, P1(i), P2(i), Q2(i)
     ENDDO

     DO p=1,ndim
        DO q=1,ndim
           if (abs(ek(p,ikp)-ek(q,ikp)) > tiny) then
              Sqp(q,p) = (P1(p)-P1(q))/(ek(p,ikp)-ek(q,ikp))
              Rqp(q,p) = (conjg(P1(p))-P1(q))/(conjg(ek(p,ikp))-ek(q,ikp))
           else
              Sqp(q,p) = Q2(p)
              Rqp(q,p) = P2(p)
           endif
           ctmp = ctmp + (Cv(p,q,ikp)*Sqp(q,p) - Dv(p,q,ikp)*Rqp(q,p))*wk(ikp)
        ENDDO
     ENDDO
  ENDDO
  conduc = -dreal(ctmp)/(2*pi*unit_cell_volume*latcon*elresunit)
  
  DEALLOCATE(Sqp, Rqp)
  DEALLOCATE(L1, L2, L3, L4, Ltmp)
  DEALLOCATE(Ar, Al, htm, hqp, ek, Cv, Dv)
END SUBROUTINE cmp_trans3


SUBROUTINE cmp_C_and_D(Cv, Dv, Al_p, Ar_p, Al_m, Ar_m, Ve, alphaV, L1, L2, L3, L4, Ltmp, ndim)
  !--------------------------------------
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out)   :: Cv(ndim,ndim), Dv(ndim,ndim)
  COMPLEX*16, intent(in)    :: Al_p(ndim,ndim), Ar_p(ndim,ndim), Al_m(ndim,ndim), Ar_m(ndim,ndim)
  COMPLEX*16, intent(in)    :: Ve(3,ndim,ndim)
  REAL*8, intent(in)        :: alphaV(3,3)
  COMPLEX*16, intent(inout) :: L1(ndim,ndim,3), L2(ndim,ndim,3), L3(ndim,ndim,3), L4(ndim,ndim,3), Ltmp(ndim,ndim)
  INTEGER, intent(in)       :: ndim
  !---------- Local variables -----------
  COMPLEX*16 :: c1tmp, c2tmp
  INTEGER    :: p,q, i,j,l

  ! Calculation of matrices C and D that are multiplied to the double denominator
  ! C is for equal sign and D for opposite sign
  DO i=1,3
     Ltmp = matmul(Al_m, Ve(i,:,:))
     L1(:,:,i) = matmul(Ltmp, Ar_p)
     L3(:,:,i) = matmul(Ltmp, conjg(transpose(Al_p)))
     Ltmp = matmul(Ve(i,:,:), Ar_m)
     L2(:,:,i) = matmul(Al_p, Ltmp)
     L4(:,:,i) = matmul(conjg(transpose(Ar_p)), Ltmp)
  ENDDO
  Cv = 0
  Dv = 0
  DO p=1,ndim
     DO q=1,ndim
        c1tmp=0
        c2tmp=0
        DO j=1,3
           DO l=1,3
              c1tmp = c1tmp + L2(p,q,j)*L1(q,p,l)*alphaV(l,j)
              c2tmp = c2tmp + L4(p,q,j)*L3(q,p,l)*alphaV(l,j)
           ENDDO
        ENDDO
        Cv(p,q) = Cv(p,q) + c1tmp
        Dv(p,q) = Dv(p,q) + c2tmp
     ENDDO
  ENDDO
END SUBROUTINE cmp_C_and_D
