SUBROUTINE cmp_opt_simple(conduc, omega, mu, eps, a, b, sig_p, sig_m, vel, hamf, olap, alphaV, latcon, unit_cell_volume, wk, ndim, nkp)
  IMPLICIT NONE
  !---------- Passed variables ----------     
  REAL*8, intent(out)    :: conduc
  REAL*8, intent(in)     :: omega, mu, eps, a, b
  COMPLEX*16, intent(in) :: sig_p(ndim,ndim), sig_m(ndim,ndim)
  COMPLEX*16, intent(in) :: vel(3,ndim,ndim,nkp)
  COMPLEX*16, intent(in) :: hamf(ndim,ndim,nkp)
  COMPLEX*16, intent(in) :: olap(ndim,ndim,nkp)
  REAL*8, intent(in)     :: alphaV(3,3)
  REAL*8, intent(in)     :: wk(nkp)
  INTEGER, intent(in)    :: ndim, nkp
  REAL*8, intent(in)     :: latcon, unit_cell_volume
  !f2py integer intent(hide), depend(hamf)   :: ndim=shape(hamf,0)
  !f2py integer intent(hide), depend(hamf)   :: nkp=shape(hamf,2)
  !---------- Local variables ----------
  REAL*8, PARAMETER :: pi = 3.14159265358979d0
  REAL*8, PARAMETER :: elresunit = 0.2173985  ! micro Ohm meter !!! Natural unit for electrical resistivity: [ aBohr*hbar/ecoul^2 ]
  COMPLEX*16, ALLOCATABLE :: Ar_p(:,:), Al_p(:,:), Ar_m(:,:), Al_m(:,:) , ek_p(:,:), ek_m(:,:), Cv(:,:,:), Dv(:,:,:) ! Eigensystem data 
  COMPLEX*16, ALLOCATABLE :: L1(:,:,:), L2(:,:,:), L3(:,:,:), L4(:,:,:), Ltmp(:,:) ! temporary data
  INTEGER    :: p,q                    ! Orbital loop indices
  INTEGER    :: ikp                    ! Loop index for k-point loop
  COMPLEX*16 :: ctmp                   ! Complex temporary variables
  
  ALLOCATE(Ar_p(ndim,ndim), Al_p(ndim,ndim), Ar_m(ndim,ndim), Al_m(ndim,ndim), ek_p(ndim,nkp), ek_m(ndim,nkp), Cv(ndim,ndim,nkp), Dv(ndim,ndim,nkp))
  ALLOCATE(L1(ndim,ndim,3), L2(ndim,ndim,3), L3(ndim,ndim,3), L4(ndim,ndim,3), Ltmp(ndim,ndim))
!!!------------- Calculation of nominators consisting of velocities and eigenvectors
  DO ikp=1,nkp
     ! for epsilon
     CALL eigsys(hamf(:,:,ikp) + sig_p, olap(:,:,ikp), ek_p(:,ikp), Al_p, Ar_p, ndim)
     ! for epsilon-omega
     CALL eigsys(hamf(:,:,ikp) + sig_m, olap(:,:,ikp), ek_m(:,ikp), Al_m, Ar_m, ndim)
     ! total V
     CALL cmp_C_and_D(Cv(:,:,ikp), Dv(:,:,ikp), Al_p, Ar_p, Al_m, Ar_m, vel(:,:,:,ikp), alphaV, L1, L2, L3, L4, Ltmp, ndim)
  ENDDO
  ctmp   = 0.0
  DO ikp=1,nkp
     DO p=1,ndim
        DO q=1,ndim
           ctmp = ctmp + Cv(p,q,ikp)*wk(ikp)/((eps+mu-ek_p(p,ikp))*(eps-omega+mu-ek_m(q,ikp)))
           ctmp = ctmp - Dv(p,q,ikp)*wk(ikp)/(conjg(eps+mu-ek_p(p,ikp))*(eps-omega+mu-ek_m(q,ikp)))
        ENDDO
     ENDDO
  ENDDO
  conduc = -dreal(ctmp)*(b-a)/(2*pi*unit_cell_volume*latcon*elresunit)
  DEALLOCATE(L1, L2, L3, L4, Ltmp)
  DEALLOCATE(Ar_p, Al_p, Ar_m, Al_m, ek_p, ek_m, Cv, Dv)
END SUBROUTINE cmp_opt_simple



SUBROUTINE cmp_opt3(conduc, omega, mu, eps, a, b, sig_p, sig_m, vel, hamf, olap, alphaV, latcon, unit_cell_volume, wk, ndim, nkp)
  IMPLICIT NONE
  !---------- Passed variables ----------     
  REAL*8, intent(out)    :: conduc
  REAL*8, intent(in)     :: omega, mu, eps, a, b
  COMPLEX*16, intent(in) :: sig_p(ndim,ndim), sig_m(ndim,ndim)
  COMPLEX*16, intent(in) :: vel(3,ndim,ndim,nkp)
  COMPLEX*16, intent(in) :: hamf(ndim,ndim,nkp)
  COMPLEX*16, intent(in) :: olap(ndim,ndim,nkp)
  REAL*8, intent(in)     :: alphaV(3,3)
  REAL*8, intent(in)     :: wk(nkp)
  INTEGER, intent(in)    :: ndim, nkp
  REAL*8, intent(in)     :: latcon, unit_cell_volume
  !f2py integer intent(hide), depend(hamf)   :: ndim=shape(hamf,0)
  !f2py integer intent(hide), depend(hamf)   :: nkp=shape(hamf,2)
  !---------- Local variables ----------
  REAL*8, PARAMETER :: pi = 3.14159265358979d0
  REAL*8, PARAMETER :: elresunit = 0.2173985  ! micro Ohm meter !!! Natural unit for electrical resistivity: [ aBohr*hbar/ecoul^2 ]
  REAL*8, PARAMETER :: big = 1e6
  REAL*8, PARAMETER :: small = 1e-12
  COMPLEX*16, ALLOCATABLE :: Ar_p(:,:), Al_p(:,:), Ar_m(:,:), Al_m(:,:) , ek_p(:,:), ek_m(:,:), Cv(:,:,:), Dv(:,:,:) ! Eigensystem data 
  COMPLEX*16, ALLOCATABLE :: L1(:,:,:), L2(:,:,:), L3(:,:,:), L4(:,:,:), Ltmp(:,:) ! temporary data
  INTEGER    :: p,q,i                                ! Orbital loop indices
  INTEGER    :: ikp                                  ! Loop index for k-point loop
  COMPLEX*16 :: lg_p(ndim), lg_q(ndim)               ! Temporary variables for fast computation of logs
  COMPLEX*16 :: ctmp, denom_es, denom_os, wes, wos   ! Complex temporary variables
  INTEGER    :: idxarr(ndim)

  ALLOCATE(Ar_p(ndim,ndim), Al_p(ndim,ndim), Ar_m(ndim,ndim), Al_m(ndim,ndim), ek_p(ndim,nkp), ek_m(ndim,nkp), Cv(ndim,ndim,nkp), Dv(ndim,ndim,nkp))
  ALLOCATE(L1(ndim,ndim,3), L2(ndim,ndim,3), L3(ndim,ndim,3), L4(ndim,ndim,3), Ltmp(ndim,ndim))

!!!------------- Calculation of nominators consisting of velocities and eigenvectors
  DO ikp=1,nkp
     ! for epsilon
     CALL eigsys(hamf(:,:,ikp) + sig_p, olap(:,:,ikp), ek_p(:,ikp), Al_p, Ar_p, ndim)

     ! orders eigenvalues according to their real part. Also orders eigenvectors correspondingly
     CALL check_causality(ek_p(:,ikp), small, ndim)
     CALL eig_order_real_part(ek_p(:,ikp), idxarr, ndim)
     CALL permute_eigensystem(idxarr, ek_p(:,ikp), Al_p, Ar_p, ndim)


     ! for epsilon-omega
     CALL eigsys(hamf(:,:,ikp) + sig_m, olap(:,:,ikp), ek_m(:,ikp), Al_m, Ar_m, ndim)

     ! orders eigenvalues according to their real part. Also orders eigenvectors correspondingly
     CALL check_causality(ek_m(:,ikp), small, ndim)
     CALL eig_order_real_part(ek_m(:,ikp), idxarr, ndim)
     CALL permute_eigensystem(idxarr, ek_m(:,ikp), Al_m, Ar_m, ndim)

     ! total V
     CALL cmp_C_and_D(Cv(:,:,ikp), Dv(:,:,ikp), Al_p, Ar_p, Al_m, Ar_m, vel(:,:,:,ikp), alphaV, L1, L2, L3, L4, Ltmp, ndim)
  ENDDO

  ctmp = 0.0
  DO ikp=1,nkp

     DO i=1,ndim
        lg_q(i) = log(b+mu-omega-ek_m(i,ikp)) - log(a+mu-omega-ek_m(i,ikp))
        lg_p(i) = log(b+mu-      ek_p(i,ikp)) - log(a+mu-      ek_p(i,ikp))
     ENDDO

     DO p=1,ndim
        DO q=1,ndim

           denom_es = omega -       ek_p(p,ikp)  + ek_m(q,ikp)
           denom_os = omega - conjg(ek_p(p,ikp)) + ek_m(q,ikp)
           
           wes = (lg_q(q)-      lg_p(p)) /denom_es
           wos = (lg_q(q)-conjg(lg_p(p)))/denom_os
           
           if (abs(wes).gt.big) then 
              print *, 'Warning: the weightes is very big at ', omega, eps, ikp, p, q, wes
              wes=0
           endif
           if (abs(wos).gt.big) then
              print *, 'Warning: the weightos is very big at ', omega, eps, ikp, p, q, wos
              wos=0
           endif
           
           ctmp = ctmp + (Cv(p,q,ikp)*wes - Dv(p,q,ikp)*wos)*wk(ikp)
        ENDDO
     ENDDO
  ENDDO
  conduc = -dreal(ctmp)/(2*pi*unit_cell_volume*latcon*elresunit)

  DEALLOCATE(L1, L2, L3, L4, Ltmp)
  DEALLOCATE(Ar_p, Al_p, Ar_m, Al_m, ek_p, ek_m, Cv, Dv)
END SUBROUTINE cmp_opt3


SUBROUTINE cmp_C_and_D(Cv, Dv, Al_p, Ar_p, Al_m, Ar_m, Ve, alphaV, L1, L2, L3, L4, Ltmp, ndim)
  !--------------------------------------
  IMPLICIT NONE
  !---------- Passed variables ----------     
  COMPLEX*16, intent(out)   :: Cv(ndim,ndim), Dv(ndim,ndim)
  COMPLEX*16, intent(in)    :: Al_p(ndim,ndim), Ar_p(ndim,ndim), Al_m(ndim,ndim), Ar_m(ndim,ndim)
  COMPLEX*16, intent(in)    :: Ve(3,ndim,ndim)
  REAL*8, intent(in)        :: alphaV(3,3)
  COMPLEX*16, intent(inout) :: L1(ndim,ndim,3), L2(ndim,ndim,3), L3(ndim,ndim,3), L4(ndim,ndim,3), Ltmp(ndim,ndim)
  INTEGER, intent(in)       :: ndim
  !---------- Local variables -----------
  COMPLEX*16 :: c1tmp, c2tmp
  INTEGER    :: p,q, i,j,l

  ! Calculation of matrices C and D that are multiplied to the double denominator
  ! C is for equal sign and D for opposite sign
  DO i=1,3
     Ltmp = matmul(Al_m, Ve(i,:,:))
     L1(:,:,i) = matmul(Ltmp, Ar_p)
     L3(:,:,i) = matmul(Ltmp, conjg(transpose(Al_p)))
     Ltmp = matmul(Ve(i,:,:), Ar_m)
     L2(:,:,i) = matmul(Al_p, Ltmp)
     L4(:,:,i) = matmul(conjg(transpose(Ar_p)), Ltmp)
  ENDDO
  Cv = 0
  Dv = 0
  DO p=1,ndim
     DO q=1,ndim
        c1tmp=0
        c2tmp=0
        DO j=1,3
           DO l=1,3
              c1tmp = c1tmp + L2(p,q,j)*L1(q,p,l)*alphaV(l,j)
              c2tmp = c2tmp + L4(p,q,j)*L3(q,p,l)*alphaV(l,j)
           ENDDO
        ENDDO
        Cv(p,q) = Cv(p,q) + c1tmp
        Dv(p,q) = Dv(p,q) + c2tmp
     ENDDO
  ENDDO
END SUBROUTINE cmp_C_and_D
