!**************************************************!
!*******  Copyright by Kristjan Haule, 2002 ******!
!**************************************************!
subroutine rot_mat(matout, matin, ikp, ir, rdc, bk, wigmat, ltpan, ilsb, igatom, lmtind, is, ikov, &
                   norb, natom, nkp, ngroup, ng, imax, lmax, nsort, ilsbmax2)
!!!--------------------------------------------------------------------------!!!
!!!--------------------------------------------------------------------------!!!
!!!  This routine can be used to rotate a matrix, such as the Hamiltonian or !!!
!!!  the overlap matrix.  The original matrix which is assumed to correspond !!!
!!!  to k point ikp, is stored in matin and the rotated matrix is stored in  !!!
!!!  matout.  The matrices are stored according to the convention used in    !!!
!!!  Sergei Savrasov's program and we use the Wigner matrices from his code. !!!
!!!--------------------------------------------------------------------------!!!
!!!--------------------------------------------------------------------------!!!
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER,    INTENT(in) :: ikp                 ! Index to irreducible k-point
  INTEGER,    INTENT(in) :: ir                  ! Index to group operation
  COMPLEX*16, INTENT(out):: matout(norb,norb)   ! input matrix
  COMPLEX*16, INTENT(in) :: matin(norb,norb)    ! output rotated matrix
  REAL*8,     INTENT(in) :: rdc(3,natom,ngroup) ! coordinate of the atom after rotation
  REAL*8,     INTENT(in) :: bk(3,nkp)           ! k-point coordinate iside 1BZ
  COMPLEX*16, INTENT(in) :: wigmat(imax,ng)     ! Wigner matrices
  INTEGER,    INTENT(in) :: ltpan(0:lmax,natom) ! l-presence array
  INTEGER,    INTENT(in) :: ilsb(nsort)         ! maximum l in base
  INTEGER,    INTENT(in) :: igatom(natom,ngroup)! index of the atom after rotation
  INTEGER,    INTENT(in) :: lmtind(ilsbmax2,natom)
  INTEGER,    INTENT(in) :: is(natom)
  INTEGER,    INTENT(in) :: ikov(ngroup)
! dimensions obtained dynamically with f2py
  INTEGER :: norb, natom, nkp, ngroup, ng, imax, lmax, nsort, ilsbmax2
!f2py integer intent(hide), depend(matin)  :: norb=shape(matin,0)
!f2py integer intent(hide), depend(rdc)    :: natom=shape(rdc,1)
!f2py integer intent(hide), depend(rdc)    :: ngroup=shape(rdc,2)
!f2py integer intent(hide), depend(bk)     :: nkp=shape(bk,1)
!f2py integer intent(hide), depend(wigmat) :: imax=shape(wigmat,0)
!f2py integer intent(hide), depend(wigmat) :: ng=shape(wigmat,1)
!f2py integer intent(hide), depend(ltpan)  :: lmax=shape(ltpan,0)-1
!f2py integer intent(hide), depend(ilsb)   :: nsort=shape(ilsb,0)
!f2py integer intent(hide), depend(lmtind) :: ilsbmax2=shape(lmtind,0)
  !---------- Local variables ----------
  INTEGER :: i                 ! Loop index
  INTEGER :: lm                ! Combined angular momentum index
  INTEGER :: ig                ! Group operation index for Wigner matrices 
  INTEGER :: iatom1,iatom2     ! Pointers to atoms
  INTEGER :: ratom1,ratom2     ! Pointers to rotated atoms
  INTEGER :: isort1,isort2     ! Atomic sorts
  INTEGER :: l1,l2             ! Total angular momentum indices
  INTEGER :: m1,m2             ! Angular momentum component indices
  INTEGER :: m1s,m2s           ! Angular momentum component indices for sum
  INTEGER :: orb1,orb2         ! Matrix orbit indices
  INTEGER :: sorb1,sorb2       ! Matrix orbit indices for sum 
  INTEGER :: cidx1,cidx2       ! Complex indices for Wigner matrices 
  REAL*8  :: r1(1:3)           ! Coordinates of shift vector for atom1
  REAL*8  :: r2(1:3)           ! Coordinates of shift vector for atom2
  REAL*8  :: kirr(1:3)         ! Coordinates of irreducible k-point
  REAL*8  :: rtmp              ! Real dummy variable
  REAL*8  :: pi                ! Good old pi
  COMPLEX*16 :: arg            ! Argument for exponential factor
  COMPLEX*16 :: expfac         ! Exponential factor 
  COMPLEX*16 :: ctmp1          ! Complex dummy variable
  COMPLEX*16 :: ctmp2          ! Complex dummy variable
  pi = 4*ATAN(1.d0)
  !---------- Change from index ir to ig ----------
  ig = ikov(ir)
  !---------- Create matrix element of rotated matrix ----------
  matout = 0.d0
  !---------- Loop over atoms ----------
  DO iatom1 = 1,natom
     DO iatom2 = 1,natom
        !---------- Compute exponential factor ----------
        r1 = rdc(:,iatom1,ir)
        r2 = rdc(:,iatom2,ir)
        kirr = bk(:,ikp)
        rtmp = 0.d0
        DO i = 1,3
           rtmp = rtmp+kirr(i)*(r1(i)-r2(i))
        ENDDO
        arg = dcmplx(0.d0,2*pi*rtmp)
        expfac = EXP(arg)
        !---------- Find rotated atoms ----------
        ratom1 = igatom(iatom1,ir)
        ratom2 = igatom(iatom2,ir)
        !---------- Find what sort the atoms are ----------
        isort1 = is(iatom1)
        isort2 = is(iatom2)
        !---------- Loop over total angular momentum ----------
        DO l1 = 0,ilsb(isort1)
           !---------- Cycle if this l not present in matrix ----------
           IF(.NOT.ltpan(l1,iatom1)) CYCLE
           DO l2 = 0,ilsb(isort2)
              !---------- Cycle if this l not present in matrix ----------
              IF(.NOT.ltpan(l2,iatom2)) CYCLE
              !---------- Loop over angular momentum component ----------
              DO m1 = -l1,l1
                 DO m2 = -l2,l2
                    !---------- Find corresponding orbits of matrix ----------
                    lm = l1*(l1+1)+m1+1
                    orb1 = lmtind(lm,iatom1)
                    lm = l2*(l2+1)+m2+1
                    orb2 = lmtind(lm,iatom2)
                    !---------- Sum over angular momentum components ----------
                    ctmp1 = 0.d0
                    DO m1s = -l1,l1
                       !---------- Find orbits of matrix in sum ----------
                       lm = l1*(l1+1)+m1s+1
                       sorb1 = lmtind(lm,ratom1)
                       !---------- Find complex index for Wigner matrices ----------
                       cidx1 = l1*(2*l1-1)*(2*l1+1)/3+(2*l1+1)*(l1+m1s)+l1+m1+1
                       ctmp2 = 0.d0
                       DO m2s = -l2,l2
                          !---------- Find orbits of matrix in sum ----------
                          lm = l2*(l2+1)+m2s+1
                          sorb2 = lmtind(lm,ratom2)
                          !---------- Find complex index for Wigner matrices ----------
                          cidx2 = l2*(2*l2-1)*(2*l2+1)/3+(2*l2+1)*(l2+m2s)+l2+m2+1
                          !---------- Do the  sum ----------
                          ctmp2 = ctmp2+matin(sorb1,sorb2)*CONJG(wigmat(cidx2,ig))
                       ENDDO
                       ctmp1 = ctmp1+ctmp2*wigmat(cidx1,ig)
                    ENDDO
                    !---------- Sum is finished, store results ----------
                    matout(orb1,orb2) = ctmp1*expfac
                 ENDDO
              ENDDO
              !---------- End of angular momentum component loop ----------
           ENDDO
        ENDDO
        !---------- That was the end of the total angular momentum loops ----------
     ENDDO
  ENDDO
  !---------- And this was the end the atom loops ----------
  RETURN
END SUBROUTINE rot_mat

subroutine phase_right(matout,ikp,ir,rdc,bk,ltpan,ilsb,lmtind,is,ikov,norb,natom,nkp,ngroup,lmax,nsort,ilsbmax2)
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER,    INTENT(in) :: ikp                 ! Index to irreducible k-point
  INTEGER,    INTENT(in) :: ir                  ! Index to group operation
  COMPLEX*16, INTENT(out):: matout(norb)        ! input matrix
  REAL*8,     INTENT(in) :: rdc(3,natom,ngroup) ! coordinate of the atom after rotation !
  REAL*8,     INTENT(in) :: bk(3,nkp)           ! k-point coordinate iside 1BZ          !
  INTEGER,    INTENT(in) :: ltpan(0:lmax,natom) ! l-presence array                      !
  INTEGER,    INTENT(in) :: ilsb(nsort)         ! maximum l in base                     !
  INTEGER,    INTENT(in) :: lmtind(ilsbmax2,natom)                                      !
  INTEGER,    INTENT(in) :: is(natom)           !                                       !
  INTEGER,    INTENT(in) :: ikov(ngroup)        !                                       !
! dimensions obtained dynamically with f2py
  INTEGER :: norb, natom, nkp, ngroup, lmax, nsort, ilsbmax2
!f2py integer intent(hide), depend(rdc)    :: natom=shape(rdc,1)
!f2py integer intent(hide), depend(rdc)    :: ngroup=shape(rdc,2)
!f2py integer intent(hide), depend(bk)     :: nkp=shape(bk,1)
!f2py integer intent(hide), depend(ltpan)  :: lmax=shape(ltpan,0)-1
!f2py integer intent(hide), depend(ilsb)   :: nsort=shape(ilsb,0)
!f2py integer intent(hide), depend(lmtind) :: ilsbmax2=shape(lmtind,0)
  !---------- Local variables ----------
  INTEGER :: i                 ! Loop index
  INTEGER :: lm                ! Combined angular momentum index
  INTEGER :: ig                ! Group operation index for Wigner matrices 
  INTEGER :: iatom1            ! Pointers to atoms
  INTEGER :: isort1            ! Atomic sorts
  INTEGER :: l1                ! Total angular momentum indices
  INTEGER :: m1                ! Angular momentum component indices
  INTEGER :: orb1              ! Matrix orbit indices
  REAL*8  :: r1(1:3)           ! Coordinates of shift vector for atom1
  REAL*8  :: kirr(1:3)         ! Coordinates of irreducible k-point
  REAL*8  :: rtmp              ! Real dummy variable
  REAL*8  :: pi                ! Good old pi
  COMPLEX*16 :: arg            ! Argument for exponential factor
  COMPLEX*16 :: expfac         ! Exponential factor 

  pi = 4*ATAN(1.d0)
  !---------- Change from index ir to ig ----------
  ig = ikov(ir)
  !---------- Create matrix element of rotated matrix ----------
  matout = 1.d0
  !---------- Loop over atoms ----------
  DO iatom1 = 1,natom
     !---------- Compute exponential factor ----------
     r1 = rdc(:,iatom1,ir)
     kirr = bk(:,ikp)
     rtmp = 0.d0
     DO i = 1,3
        rtmp = rtmp+kirr(i)*(-r1(i))
     ENDDO
     arg = dcmplx(0.d0,2*pi*rtmp)
     expfac = EXP(arg)
     !---------- Find what sort the atoms are ----------
     isort1 = is(iatom1)
     !---------- Loop over total angular momentum ----------
     DO l1 = 0,ilsb(isort1)
        !---------- Cycle if this l not present in matrix ----------
        IF(.NOT.ltpan(l1,iatom1)) CYCLE
        DO m1 = -l1,l1
           !---------- Find corresponding orbits of matrix ----------
           lm = l1*(l1+1)+m1+1
           orb1 = lmtind(lm,iatom1)
           !---------- Sum over angular momentum components ----------
           matout(orb1) = expfac
        ENDDO
     ENDDO
     !---------- End of angular momentum component loop ----------
  ENDDO

RETURN
END SUBROUTINE phase_right


subroutine rot_mat_right_nophase(matout, matin, ir, wigmat, ltpan, ilsb, igatom, lmtind, is, ikov, &
     norb, natom, ngroup, ng, imax, lmax, nsort, ilsbmax2)
!!!--------------------------------------------------------------------------!!!
!!!--------------------------------------------------------------------------!!!
!!!  This routine can be used to rotate a matrix, such as the Hamiltonian or !!!
!!!  the overlap matrix.  The original matrix which is assumed to correspond !!!
!!!  to k point ikp, is stored in matin and the rotated matrix is stored in  !!!
!!!  matout.  The matrices are stored according to the convention used in    !!!
!!!  Sergei Savrasov's program and we use the Wigner matrices from his code. !!!
!!!--------------------------------------------------------------------------!!!
!!!--------------------------------------------------------------------------!!!
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER,    INTENT(in) :: ir                  ! Index to group operation
  COMPLEX*16, INTENT(out):: matout(norb,norb)   ! input matrix
  COMPLEX*16, INTENT(in) :: matin(norb,norb)    ! output rotated matrix
  COMPLEX*16, INTENT(in) :: wigmat(imax,ng)     ! Wigner matrices
  INTEGER,    INTENT(in) :: ltpan(0:lmax,natom) ! l-presence array
  INTEGER,    INTENT(in) :: ilsb(nsort)         ! maximum l in base
  INTEGER,    INTENT(in) :: igatom(natom,ngroup)! index of the atom after rotation
  INTEGER,    INTENT(in) :: lmtind(ilsbmax2,natom)
  INTEGER,    INTENT(in) :: is(natom)
  INTEGER,    INTENT(in) :: ikov(ngroup)
! dimensions obtained dynamically with f2py
  INTEGER :: norb, natom, ngroup, ng, imax, lmax, nsort, ilsbmax2
!f2py integer intent(hide), depend(matin)  :: norb=shape(matin,0)
!f2py integer intent(hide), depend(is)     :: natom=shape(is,0)
!f2py integer intent(hide), depend(ikov)   :: ngroup=shape(ikov,0)
!f2py integer intent(hide), depend(wigmat) :: imax=shape(wigmat,0)
!f2py integer intent(hide), depend(wigmat) :: ng=shape(wigmat,1)
!f2py integer intent(hide), depend(ltpan)  :: lmax=shape(ltpan,0)-1
!f2py integer intent(hide), depend(ilsb)   :: nsort=shape(ilsb,0)
!f2py integer intent(hide), depend(lmtind) :: ilsbmax2=shape(lmtind,0)
  !---------- Local variables ----------
  INTEGER :: i                 ! Loop index
  INTEGER :: lm                ! Combined angular momentum index
  INTEGER :: ig                ! Group operation index for Wigner matrices 
  INTEGER :: iatom1,iatom2     ! Pointers to atoms
  INTEGER :: ratom2            ! Pointers to rotated atoms
  INTEGER :: isort1,isort2     ! Atomic sorts
  INTEGER :: l1,l2             ! Total angular momentum indices
  INTEGER :: m1,m2             ! Angular momentum component indices
  INTEGER :: m1s,m2s           ! Angular momentum component indices for sum
  INTEGER :: orb1,orb2         ! Matrix orbit indices
  INTEGER :: sorb1,sorb2       ! Matrix orbit indices for sum 
  INTEGER :: cidx1,cidx2       ! Complex indices for Wigner matrices 
  REAL*8  :: r1(1:3)           ! Coordinates of shift vector for atom1
  REAL*8  :: r2(1:3)           ! Coordinates of shift vector for atom2
  REAL*8  :: kirr(1:3)         ! Coordinates of irreducible k-point
  REAL*8  :: rtmp              ! Real dummy variable
  REAL*8  :: pi                ! Good old pi
  COMPLEX*16 :: arg            ! Argument for exponential factor
  COMPLEX*16 :: expfac         ! Exponential factor 
  COMPLEX*16 :: ctmp1          ! Complex dummy variable
  COMPLEX*16 :: ctmp2          ! Complex dummy variable

  pi = 4*ATAN(1.d0)
  !---------- Change from index ir to ig ----------
  ig = ikov(ir)
  !---------- Create matrix element of rotated matrix ----------
  matout = 0.d0
  !---------- Loop over atoms ----------
  DO iatom1 = 1,natom
     DO iatom2 = 1,natom
        !---------- Find rotated atoms ----------
        ratom2 = igatom(iatom2,ir)
        !---------- Find what sort the atoms are ----------
        isort1 = is(iatom1)
        isort2 = is(iatom2)
        !---------- Loop over total angular momentum ----------
        DO l1 = 0,ilsb(isort1)
           !---------- Cycle if this l not present in matrix ----------
           IF(.NOT.ltpan(l1,iatom1)) CYCLE
           DO l2 = 0,ilsb(isort2)
              !---------- Cycle if this l not present in matrix ----------
              IF(.NOT.ltpan(l2,iatom2)) CYCLE
              !---------- Loop over angular momentum component ----------
              DO m1 = -l1,l1
                 DO m2 = -l2,l2
                    !---------- Find corresponding orbits of matrix ----------
                    lm = l1*(l1+1)+m1+1
                    orb1 = lmtind(lm,iatom1)
                    lm = l2*(l2+1)+m2+1
                    orb2 = lmtind(lm,iatom2)
                    !---------- Sum over angular momentum components ----------
                    ctmp2 = 0.d0
                    DO m2s = -l2,l2
                       !---------- Find orbits of matrix in sum ----------
                       lm = l2*(l2+1)+m2s+1
                       sorb2 = lmtind(lm,ratom2)
                       !---------- Find complex index for Wigner matrices ----------
                       cidx2 = l2*(2*l2-1)*(2*l2+1)/3+(2*l2+1)*(l2+m2s)+l2+m2+1
                       ctmp2 = ctmp2+matin(orb1,sorb2)*CONJG(wigmat(cidx2,ig))
                    ENDDO
                    !---------- Sum is finished, store results ----------
                    matout(orb1,orb2) = ctmp2
                 ENDDO
              ENDDO
              !---------- End of angular momentum component loop ----------
           ENDDO
        ENDDO
        !---------- That was the end of the total angular momentum loops ----------
     ENDDO
  ENDDO
  !---------- And this was the end the atom loops ----------
  RETURN
END SUBROUTINE rot_mat_right_nophase

! How to call for small or long version:
! long   -> (false/true)
! norb   -> (norb/lnorb)
! ilsx   -> (ilsb/ilst)
! lmtind -> (lmtind/llmtind)
subroutine rot_mat_right_nophase_(matout, matin, long, ir, wigmat, ltpan, ilsx, igatom, lmtind, is, ikov, &
     norb, natom, ngroup, ng, imax, lmax, nsort, ilsbmax2)
!!!--------------------------------------------------------------------------!!!
!!!--------------------------------------------------------------------------!!!
!!!  This routine can be used to rotate a matrix, such as the Hamiltonian or !!!
!!!  the overlap matrix.  The original matrix which is assumed to correspond !!!
!!!  to k point ikp, is stored in matin and the rotated matrix is stored in  !!!
!!!  matout.  The matrices are stored according to the convention used in    !!!
!!!  Sergei Savrasov's program and we use the Wigner matrices from his code. !!!
!!!--------------------------------------------------------------------------!!!
!!!--------------------------------------------------------------------------!!!
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER,    INTENT(in) :: ir                  ! Index to group operation
  COMPLEX*16, INTENT(out):: matout(norb,norb)   ! input matrix
  COMPLEX*16, INTENT(in) :: matin(norb,norb)    ! output rotated matrix
  LOGICAL,    INTENT(in) :: long                ! does it go over l in base (norb) or l in expansion(norb)
  COMPLEX*16, INTENT(in) :: wigmat(imax,ng)     ! Wigner matrices
  INTEGER,    INTENT(in) :: ltpan(0:lmax,natom) ! l-presence array
  INTEGER,    INTENT(in) :: ilsx(nsort)         ! maximum l in base
  INTEGER,    INTENT(in) :: igatom(natom,ngroup)! index of the atom after rotation
  INTEGER,    INTENT(in) :: lmtind(ilsbmax2,natom)
  INTEGER,    INTENT(in) :: is(natom)
  INTEGER,    INTENT(in) :: ikov(ngroup)
! dimensions obtained dynamically with f2py
  INTEGER :: norb, natom, ngroup, ng, imax, lmax, nsort, ilsbmax2
!f2py integer intent(hide), depend(matin)  :: norb=shape(matin,0)
!f2py integer intent(hide), depend(is)     :: natom=shape(is,0)
!f2py integer intent(hide), depend(ikov)   :: ngroup=shape(ikov,0)
!f2py integer intent(hide), depend(wigmat) :: imax=shape(wigmat,0)
!f2py integer intent(hide), depend(wigmat) :: ng=shape(wigmat,1)
!f2py integer intent(hide), depend(ltpan)  :: lmax=shape(ltpan,0)-1
!f2py integer intent(hide), depend(ilsx)   :: nsort=shape(ilsx,0)
!f2py integer intent(hide), depend(lmtind) :: ilsbmax2=shape(lmtind,0)
!!!  COMPLEX*16 :: matin(1:lnorb,1:lnorb)
!!!  COMPLEX*16 :: matout(1:lnorb,1:lnorb)
  !---------- Local variables ----------
  INTEGER :: i                 ! Loop index
  INTEGER :: lm                ! Combined angular momentum index
  INTEGER :: ig                ! Group operation index for Wigner matrices 
  INTEGER :: iatom1,iatom2     ! Pointers to atoms
  INTEGER :: ratom2            ! Pointers to rotated atoms
  INTEGER :: isort1,isort2     ! Atomic sorts
  INTEGER :: l1,l2             ! Total angular momentum indices
  INTEGER :: m1,m2             ! Angular momentum component indices
  INTEGER :: m1s,m2s           ! Angular momentum component indices for sum
  INTEGER :: orb1,orb2         ! Matrix orbit indices
  INTEGER :: sorb1,sorb2       ! Matrix orbit indices for sum 
  INTEGER :: cidx1,cidx2       ! Complex indices for Wigner matrices 
  REAL*8  :: r1(1:3)           ! Coordinates of shift vector for atom1
  REAL*8  :: r2(1:3)           ! Coordinates of shift vector for atom2
  REAL*8  :: kirr(1:3)         ! Coordinates of irreducible k-point
  REAL*8  :: rtmp              ! Real dummy variable
  REAL*8  :: pi                ! Good old pi
  COMPLEX*16 :: arg            ! Argument for exponential factor
  COMPLEX*16 :: expfac         ! Exponential factor 
  COMPLEX*16 :: ctmp1          ! Complex dummy variable
  COMPLEX*16 :: ctmp2          ! Complex dummy variable

  pi = 4*ATAN(1.d0)
  !---------- Change from index ir to ig ----------
  ig = ikov(ir)
  !---------- Create matrix element of rotated matrix ----------
  matout = 0.d0
  !---------- Loop over atoms ----------
  DO iatom1 = 1,natom
     DO iatom2 = 1,natom
        !---------- Find rotated atoms ----------
        ratom2 = igatom(iatom2,ir)
        !---------- Find what sort the atoms are ----------
        isort1 = is(iatom1)
        isort2 = is(iatom2)
        !---------- Loop over total angular momentum ----------
        DO l1 = 0,ilsx(isort1)
           !---------- Cycle if this l not present in matrix ----------
           IF(.NOT. long .AND. .NOT.ltpan(l1,iatom1)) CYCLE
           DO l2 = 0,ilsx(isort2)
              !---------- Cycle if this l not present in matrix ----------
              IF(.NOT. long .AND. .NOT.ltpan(l2,iatom2)) CYCLE
              !---------- Loop over angular momentum component ----------
              DO m1 = -l1,l1
                 DO m2 = -l2,l2
                    !---------- Find corresponding orbits of matrix ----------
                    lm = l1*(l1+1)+m1+1
                    orb1 = lmtind(lm,iatom1)
                    lm = l2*(l2+1)+m2+1
                    orb2 = lmtind(lm,iatom2)
                    !---------- Sum over angular momentum components ----------
                    ctmp2 = 0.d0
                    DO m2s = -l2,l2
                       !---------- Find orbits of matrix in sum ----------
                       lm = l2*(l2+1)+m2s+1
                       sorb2 = lmtind(lm,ratom2)
                       !---------- Find complex index for Wigner matrices ----------
                       cidx2 = l2*(2*l2-1)*(2*l2+1)/3+(2*l2+1)*(l2+m2s)+l2+m2+1
                       ctmp2 = ctmp2+matin(orb1,sorb2)*CONJG(wigmat(cidx2,ig))
                    ENDDO
                    !---------- Sum is finished, store results ----------
                    matout(orb1,orb2) = ctmp2
                 ENDDO
              ENDDO
              !---------- End of angular momentum component loop ----------
           ENDDO
        ENDDO
        !---------- That was the end of the total angular momentum loops ----------
     ENDDO
  ENDDO
  !---------- And this was the end the atom loops ----------
  RETURN
END SUBROUTINE rot_mat_right_nophase_

