!**************************************************!
!*******  Copyright by Kristjan Haule, 2002 ******!
!**************************************************!
MODULE rtm
!!! Other modules used here
  COMPLEX*16, ALLOCATABLE :: Ug(:,:,:)       ! Transformation matrix for rot-mat withought phase factor: Ug(:,:,group)
  COMPLEX*16, ALLOCATABLE :: Ugll(:,:,:)     ! Transformation matrix for rot-mat withought phase factor: Ugll(:,:,group)
  COMPLEX*16, ALLOCATABLE :: Ph(:,:,:,:)     ! Phase factor vector for rot-mat: Ph(:,k1,k2,k3)
  COMPLEX*16, ALLOCATABLE :: SJ(:,:,:)
  INTEGER :: norb, lnorb, ndim, ngroup, nspin ! local copy of dimansions
  INTEGER :: lndim
CONTAINS

  subroutine clean_up()
    if (allocated(Ug)) deallocate (Ug)
    if (allocated(Ugll)) deallocate (Ugll)
    if (allocated(Ph)) deallocate (Ph) 
    if (allocated(SJ)) deallocate (SJ)
  end subroutine clean_up

  subroutine init(rdc, bk, wigmat, ltpan, ilsb, ilst, igatom, lmtind, llmtind, is, ikov, SJ_, &
       norb_, lnorb_, ndim_, lndim_, nspin_, kirrptr, gropptr, &
       ngroup_, natom, nkp, ng, imax, lmax, nsort, ilsbmax2, illsbmax2, ndiv1, ndiv2, ndiv3)  
!!!--------------------------------------------------------------------------------------------------!!!
!!! Allocates memory for global transformation matrices Ug and Ph and initializes them.              !!!
!!! The matrix Ug depends only on group operation and Ph is a vector (equivalent to diagonal matrix) !!! 
!!! of phase factors and depends only on k point.                                                    !!!
!!! The rotation matrix is finaly : Ug . Ph' , where . is a matrix product and Ph'                   !!!
!!! is assumed to be digonal matrix build from the vector Ph.                                        !!!
!!!--------------------------------------------------------------------------------------------------!!!
    IMPLICIT NONE
    REAL*8,     INTENT(in) :: rdc(3,natom,ngroup_) ! coordinate of the atom after rotation
    REAL*8,     INTENT(in) :: bk(3,nkp)           ! k-point coordinate iside 1BZ
    COMPLEX*16, INTENT(in) :: wigmat(imax,ng)     ! Wigner matrices
    INTEGER,    INTENT(in) :: ltpan(0:lmax,natom) ! l-presence array
    INTEGER,    INTENT(in) :: ilsb(nsort)         ! maximum l in base
    INTEGER,    INTENT(in) :: ilst(nsort)         ! maximum l in base
    INTEGER,    INTENT(in) :: igatom(natom,ngroup_)! index of the atom after rotation
    INTEGER,    INTENT(in) :: lmtind(ilsbmax2,natom)
    INTEGER,    INTENT(in) :: llmtind(illsbmax2,natom)
    INTEGER,    INTENT(in) :: is(natom)
    INTEGER,    INTENT(in) :: ikov(ngroup_)
    INTEGER,    intent(in) :: kirrptr(ndiv1,ndiv2,ndiv3)
    INTEGER,    intent(in) :: gropptr(ndiv1,ndiv2,ndiv3)
    COMPLEX*16, intent(in) :: SJ_(2,2,ngroup_)
! dimensions obtained dynamically with f2py
    INTEGER,    intent(in) :: norb_, lnorb_, ndim_, lndim_, nspin_, ngroup_, natom, nkp, ng, imax, lmax, nsort, ilsbmax2, illsbmax2
    INTEGER,    intent(in) :: ndiv1, ndiv2, ndiv3
!f2py integer intent(hide), depend(rdc)      :: natom=shape(rdc,1)
!f2py integer intent(hide), depend(rdc)      :: ngroup_=shape(rdc,2)
!f2py integer intent(hide), depend(bk)       :: nkp=shape(bk,1)
!f2py integer intent(hide), depend(wigmat)   :: imax=shape(wigmat,0)
!f2py integer intent(hide), depend(wigmat)   :: ng=shape(wigmat,1)
!f2py integer intent(hide), depend(ltpan)    :: lmax=shape(ltpan,0)-1
!f2py integer intent(hide), depend(ilsb)     :: nsort=shape(ilsb,0)
!f2py integer intent(hide), depend(lmtind)   :: ilsbmax2=shape(lmtind,0)
!f2py integer intent(hide), depend(llmtind)  :: illsbmax2=shape(llmtind,0)
!f2py integer intent(hide), depend(kirrptr)  :: ndiv1=shape(kirrptr,0)
!f2py integer intent(hide), depend(kirrptr)  :: ndiv2=shape(kirrptr,1)
!f2py integer intent(hide), depend(kirrptr)  :: ndiv3=shape(kirrptr,2)
  !---------- Local variables ----------
    INTEGER :: i, ikp, ir, i1, i2, i3
    COMPLEX*16, ALLOCATABLE :: unity(:,:)

!!! Initialize module members
    ngroup = ngroup_
    norb = norb_
    lnorb = lnorb_
    ndim = ndim_
    lndim = lndim_
    nspin = nspin_
    
!!! Allocate module memory - will be shared among members of the module
    allocate( SJ(2,2,ngroup))
    allocate( Ug(norb,norb,ngroup))
    allocate( Ugll(lnorb,lnorb,ngroup))
    allocate( Ph(norb,ndiv1,ndiv2,ndiv3))
!!! Initialize the rest of module members
    SJ = SJ_

    allocate( unity(norb,norb))

    unity=0
    do i=1,norb
       unity(i,i)=1.0
    enddo

    do i=1,ngroup
       call rot_mat_right_nophase_(Ug(:,:,i), unity, .FALSE., i, &
            wigmat, ltpan, ilsb, igatom, lmtind, is, ikov, norb, natom, ngroup, ng, imax, lmax, nsort, ilsbmax2)
       ! call rot_mat_right_nophase(unity,Ug(:,:,i),i)
    enddo
    
    deallocate(unity)
    allocate(unity(1:lnorb,1:lnorb))
    unity=0
    do i=1,lnorb
       unity(i,i)=1.0
    enddo

    do i=1,ngroup
       call rot_mat_right_nophase_(Ugll(:,:,i), unity, .TRUE., i, &
            wigmat, ltpan, ilst, igatom, llmtind, is, ikov, lnorb, natom, ngroup, ng, imax, lmax, nsort, illsbmax2)
       ! call rot_mat_right_nophase_long_long(unity,Ugll(:,:,i),i)
    enddo

    do i3=1,ndiv3
       do i2=1,ndiv2
          do i1=1,ndiv1
             ikp = kirrptr(i1,i2,i3)
             ir  = gropptr(i1,i2,i3)
             call phase_right(Ph(:,i1,i2,i3),ikp,ir, &
                  rdc,bk,ltpan,ilsb,lmtind,is,ikov,norb,natom,nkp,ngroup,lmax,nsort,ilsbmax2)
             ! call phase_right(Ph(:,i1,i2,i3),ikp,ir)
          enddo
       enddo
    enddo
    deallocate(unity)
  endsubroutine init

  subroutine cmp_Ukn_phase(Uk, ir, i1,i2,i3)
    !------------------------------------------------------------------------------------------------
    ! Returns the rotaion matrix Uk that can be used to rotate Hamiltonian as follows: Uk^+ . H_k . Uk
    ! The rotation matrix is obtained by : Ug . Ph' , where . is a matrix product and Ph'
    ! is assumed to be digonal matrix build from the vector Ph.
    !------------------------------------------------------------------------------------------------
    IMPLICIT NONE
!!! Passed variables
    COMPLEX*16, intent(out) :: Uk(ndim,ndim)    ! output transformation matrix 
    INTEGER, intent(in)     :: i1, i2, i3           ! k-point
    INTEGER, intent(in)     :: ir                   ! group operation
!               !f2py integer intent(hide), depend(SJ)  :: ngroup=shape(SJ,2)
    !---------- Local variables ----------
    INTEGER :: i, j
    COMPLEX*16 :: Ukw(1:norb,1:norb)

    IF (.NOT. allocated(Ug) .OR. .NOT. allocated(Ph)) THEN
       WRITE(0,*) 'In module "rtm" memory not allocated. Call rtm.init() first '
       CALL EXIT(1)
    ENDIF

    Uk=0
    do j=1,norb
       do i=1,norb
          Ukw(i,j) = Ug(i,j,ir)*Ph(j,i1,i2,i3)
       enddo
    enddo
    if (ndim.eq.norb) then
       Uk = conjg(Ukw)
    else
       Uk(1:norb,1:norb) = conjg(Ukw)*SJ(1,1,ir)
       Uk(norb+1:ndim,norb+1:ndim) = conjg(Ukw)*SJ(2,2,ir)
       Uk(1:norb,norb+1:ndim) = conjg(Ukw)*SJ(2,1,ir)
       Uk(norb+1:ndim,1:norb) = conjg(Ukw)*SJ(1,2,ir)
    endif
    return
  endsubroutine cmp_Ukn_phase

  subroutine cmp_Ukn(Uk, ir)
    !------------------------------------------------------------------------------------------------
    ! Returns the rotaion matrix Uk that can be used to rotate Hamiltonian as follows: Uk^+ . H_k . Uk
    ! The rotation matrix is obtained by : Ug . Ph' , where . is a matrix product and Ph'
    ! is assumed to be digonal matrix build from the vector Ph.
    !------------------------------------------------------------------------------------------------
    IMPLICIT NONE
!!! Passed variables
    COMPLEX*16, intent(out) :: Uk(1:ndim,1:ndim)    ! output transformation matrix 
!    COMPLEX*16, intent(in)  :: SJ(2,2,ngroup)
    INTEGER, intent(in)     :: ir                   ! group operation
!      !f2py integer intent(hide), depend(SJ)  :: ngroup=shape(SJ,2)
    !---------- Local variables ----------
    INTEGER :: i, j

    IF (.NOT. allocated(Ug) .OR. .NOT. allocated(SJ)) THEN
       WRITE(0,*) 'In module "rtm" memory not allocated. Call rtm.init() first '
       CALL EXIT(1)
    ENDIF

    Uk=0
    if (ndim.eq.norb) then
       Uk = conjg(Ug(:,:,ir))
    else
       Uk(1:norb,1:norb) = conjg(Ug(:,:,ir))*SJ(1,1,ir)
       Uk(norb+1:ndim,norb+1:ndim) = conjg(Ug(:,:,ir))*SJ(2,2,ir)
       Uk(1:norb,norb+1:ndim) = conjg(Ug(:,:,ir))*SJ(2,1,ir)
       Uk(norb+1:ndim,1:norb) = conjg(Ug(:,:,ir))*SJ(1,2,ir)
    endif
    return
  endsubroutine cmp_Ukn

  subroutine cmp_Tall(Tall, T2CJ, gropptr, ndiv1, ndiv2, ndiv3, ndim)
    !---------------------------------------------------------------------
    ! Calculates the large Transformation matrix for each k-point
    ! that rotates the matrix to particular k-point and at the same time 
    ! transform to the cubic harmonics : Tall = Ug . Ph' . T2c
    !---------------------------------------------------------------------
    IMPLICIT NONE
    COMPLEX*16, intent(out) :: Tall(ndim,ndim,ndiv1,ndiv2,ndiv3)
    COMPLEX*16, intent(in)  :: T2CJ(ndim,ndim)
!    COMPLEX*16, intent(in)  :: SJ(2,2,ngroup)
    integer, intent(in)     :: gropptr(ndiv1,ndiv2,ndiv3)
    INTEGER, intent(in)     :: ndiv1, ndiv2, ndiv3, ndim
!f2py integer intent(hide), depend(T2CJ):: ndim=shape(T2CJ,0)
!f2py integer intent(hide), depend(gropptr):: ndiv1=shape(gropptr,0)
!f2py integer intent(hide), depend(gropptr):: ndiv2=shape(gropptr,1)
!f2py integer intent(hide), depend(gropptr):: ndiv3=shape(gropptr,2)
    ! ---------------------------------------------------------------------
    COMPLEX*16 :: UtoJ(ndim,ndim)
    COMPLEX*16 :: R(1:ndim,1:ndim)
    INTEGER    :: i3, i2, i1, ir, i, j, iat
    REAL*8     :: j0, mj, l, ml, ms, s0
    do i3=1,ndiv3
       do i2=1,ndiv2
          do i1=1,ndiv1
             ir  = gropptr(i1,i2,i3)
             CALL cmp_Ukn_phase(R,ir,i1,i2,i3)
             ! Add transformation to cubic harmonics
             Tall(:,:,i1,i2,i3) = matmul(R,T2CJ)
          enddo
       enddo
    enddo
  endsubroutine cmp_Tall

END MODULE rtm
