subroutine ksm(funr, Rs, vec, funk, kmesh, nr, nk)
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER, intent(in) :: nr, nk
  REAL*8, intent(in) :: kmesh(nk,3)
  REAL*8, intent(in) :: Rs(nr,3)
  REAL*8, intent(in) :: vec(3,3)
  DOUBLE COMPLEX, intent(in)  :: funk(nk)
  DOUBLE COMPLEX, intent(out) :: funr(nr)
  !f2py integer intent(hide), depend(kmesh) :: nk=shape(kmesh,0)
  !f2py integer intent(hide), depend(Rs)    :: nr=shape(Rs,0)
  !---------- Local variables ----------
  INTEGER :: i, k
  DOUBLE COMPLEX :: sm
  REAL*8  :: th, dvec(3)
  !---------- Subroutine body ----------
  do i=1,nr
     dvec(:) = Rs(i,1)*vec(1,:) + Rs(i,2)*vec(2,:) + Rs(i,3)*vec(3,:)
     sm=0
     do k=1,nk
        th = DOT_PRODUCT(kmesh(k,:),dvec)
        sm = sm + dcmplx(cos(th),-sin(th))*funk(k)
     enddo
     funr(i) = sm/nk
  enddo
end subroutine ksm

subroutine rsm(funk, kmesh, funr, Rs, vec, nr, nk)
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER, intent(in) :: nr, nk
  REAL*8, intent(in) :: kmesh(nk,3)
  REAL*8, intent(in) :: Rs(nr,3)
  REAL*8, intent(in) :: vec(3,3)
  DOUBLE COMPLEX, intent(out):: funk(nk)
  DOUBLE COMPLEX, intent(in) :: funr(nr)
  !f2py integer intent(hide), depend(kmesh) :: nk=shape(kmesh,0)
  !f2py integer intent(hide), depend(Rs)    :: nr=shape(Rs,0)
  !---------- Local variables ----------
  INTEGER :: i, k
  DOUBLE COMPLEX :: sm
  REAL*8  :: th, dvec(3)
  !---------- Subroutine body ----------
  do k=1,nk
     sm=0
     do i=1,nr
        dvec(:) = Rs(i,1)*vec(1,:) + Rs(i,2)*vec(2,:) + Rs(i,3)*vec(3,:)
        th = DOT_PRODUCT(kmesh(k,:),dvec)
        sm = sm + dcmplx(cos(th),sin(th))*funr(i)
     enddo
     funk(k) = sm
  enddo
end subroutine rsm


subroutine rFourier(funr, Rs, vec, funk, kmesh, nr, nk, N)
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER, intent(in) :: nr, nk, N
  REAL*8, intent(in) :: kmesh(nk,3)
  REAL*8, intent(in) :: Rs(nr,3)
  REAL*8, intent(in) :: vec(3,3)
  DOUBLE COMPLEX, intent(in)  :: funk(nk,N,N)
  DOUBLE COMPLEX, intent(out) :: funr(nr,N,N)
  !f2py integer intent(hide), depend(kmesh) :: nk=shape(kmesh,0)
  !f2py integer intent(hide), depend(Rs)    :: nr=shape(Rs,0)
  !f2py integer intent(hide), depend(funk)  :: N=shape(funk,1)
  !---------- Local variables ----------
  INTEGER :: i, k
  DOUBLE COMPLEX :: ph, sm(N,N)
  REAL*8  :: th, dvec(3)
  !---------- Subroutine body ----------
  do i=1,nr
     dvec(:) = Rs(i,1)*vec(1,:) + Rs(i,2)*vec(2,:) + Rs(i,3)*vec(3,:)
     sm=0
     do k=1,nk
        th = DOT_PRODUCT(kmesh(k,:),dvec)
        ph = dcmplx(cos(th),-sin(th))
        sm = sm + ph*funk(k,:,:)
     enddo
     funr(i,:,:) = sm(:,:)/nk
  enddo
end subroutine rFourier

subroutine kFourier(funk, kmesh, funr, Rs, vec, nr, nk, N)
  IMPLICIT NONE
  !---------- Passed variables ----------
  INTEGER, intent(in) :: nr, nk, N
  REAL*8, intent(in) :: kmesh(nk,3)
  REAL*8, intent(in) :: Rs(nr,3)
  REAL*8, intent(in) :: vec(3,3)
  DOUBLE COMPLEX, intent(out):: funk(nk,N,N)
  DOUBLE COMPLEX, intent(in) :: funr(nr,N,N)
  !f2py integer intent(hide), depend(kmesh) :: nk=shape(kmesh,0)
  !f2py integer intent(hide), depend(Rs)    :: nr=shape(Rs,0)
  !f2py integer intent(hide), depend(funr)  :: N=shape(funr,1)
  !---------- Local variables ----------
  INTEGER :: i, k
  DOUBLE COMPLEX :: ph, sm(N,N)
  REAL*8  :: th, dvec(3)
  !---------- Subroutine body ----------
  do k=1,nk
     sm=0
     do i=1,nr
        dvec(:) = Rs(i,1)*vec(1,:) + Rs(i,2)*vec(2,:) + Rs(i,3)*vec(3,:)
        th = DOT_PRODUCT(kmesh(k,:),dvec)
        ph = dcmplx(cos(th),sin(th))
        sm = sm + ph*funr(i,:,:)
     enddo
     funk(k,:,:) = sm(:,:)
  enddo
end subroutine kFourier


