!**************************************************!
!*******  Copyright by Kristjan Haule, 2002 ******!
!**************************************************!

SUBROUTINE tetrahedron( z, e, weights) ! former int_tet1
!!!--------------------------------------------------!!!
!!!  These subroutines compute the weight factors for!!!
!!!  the analytical tetrahedron method as described  !!!
!!!  in the paper by Lambin and Vigneron, Physical   !!!
!!!  Review B, Vol. 29,#6, page 3430.  The formulas  !!!
!!!  used here are written slightly differently from !!!
!!!  the ones given in the article and hopefully in  !!!
!!!  a more stable form.  In particular we try to    !!!
!!!  express the answer as much as possible in a new !!!
!!!  function, the socalled Lambin-Vigneron function,!!!
!!!  lv, which is easy to compute and has quite nice !!!
!!!  asymptotic limits.                              !!!
!!!--------------------------------------------------!!!
  ! Returned type
  DOUBLE COMPLEX, intent(out) :: weights(1:4)
  !---------- Passed variables ----------
  DOUBLE COMPLEX, INTENT(in) :: z         ! ( In ) Frequency
  DOUBLE COMPLEX, INTENT(in) :: e(4)      ! ( In ) Energy eigenvalues
  !---------- Local variables ----------
  INTEGER    :: flg         ! Corner degeneracy index
  INTEGER    :: isrt(1:4)   ! Corner reordering index
  DOUBLE COMPLEX :: ecp(1:4)    ! Local copy of energy values
  DOUBLE COMPLEX :: res(1:4)    ! Linear weight factors 
  !---------- Subroutine body ----------
  ! Make local copy of corner energies
  ecp(:) = e(:)
  ! Reorder the corner energies
  CALL creorder( ecp,isrt,flg )
  ! Select the appropriate routine to compute the integral
  SELECT CASE ( flg )
  CASE(1)
     CALL arbek_optimized( z,ecp,res )     !  Arbitrary eigenvalues
  CASE(2)
     CALL ek2idn( z,ecp,res )    !  Two eigenvalues are identical
  CASE(3)
     CALL ek22idn( z,ecp,res )   !  Two pairs of energy are identical
  CASE(4)
     CALL ek3idn( z,ecp,res )    !  Three eigenvalues are identical
  CASE(5)
     CALL ek4idn( z,ecp,res )    !  Four (all) eigenvalues are identical
  END SELECT
  ! Put integration weights in correct order
  weights(isrt(1)) = res(1)
  weights(isrt(2)) = res(2)
  weights(isrt(3)) = res(3)
  weights(isrt(4)) = res(4)
END SUBROUTINE tetrahedron
