# My python imports
import re
import sys as s
from scipy import *
import dia_gho

# My fortran imports
import linalg as la
import tetra

def ReadSelfEnergyColumns(filename, contains_DC=False):
    fh = open(filename, 'r')
    
    if (contains_DC):
        s_oo = None
        Edc = None
        for i in range(2):
            line = fh.next()
            m = re.search(r's_oo\s*=(.*)', line)
            if (m is not None):
                s_oo = array(eval(m.group(1)))
            m = re.search(r'Edc\s*=(.*)', line)
            if (m is not None):
                Edc = array(eval(m.group(1)))
        
    omega=[]
    Sigma=[]
    for line in fh:
        while (re.match('^\s*#', line) is not None): line = fh.next() # comment is skipped
        field = map(float,line.split())
        omega.append(field[0])
        sig=[]
        for i in range((len(field)-1)/2):
            sig.append(field[2*i+1] + field[2*i+2]*1j)
            
        Sigma.append(sig)
    fh.close()
    
    if (contains_DC):
        if (Edc==None):
            print 'Could not find Edc in input file', filename, ' which is supposed to contain that information'
            Edc = zeros(shape(Sigma[0]),dtype=float)
        if (s_oo==None):
            print 'Could not find s_oo in input file', filename, ' which is supposed to contain that information'
            s_oo = zeros(shape(Sigma[0]),dtype=float)
        return (omega,Sigma,s_oo,Edc)
    else:
        return (omega,Sigma)

def CreateSelfEnergyMatrix(Sigma_vec, Sigind, gamma=[0.0,0.0]):
    Sig_out = array(zeros(shape(Sigind)),order='F',dtype=complex)
    for i in range(shape(Sigind)[0]):
        for j in range(shape(Sigind)[1]):
            if (Sigind[i,j] == 0):
                Sig_out[i,j] = 0j
                if (i==j) :
                    Sig_out[i,j] = -gamma[0]*1j  # broadening of non-correlated bands
            else:
                Sig_out[i,j] = Sigma_vec[Sigind[i,j]-1]
                if (i==j) and (abs(Sig_out[i,j].imag)<gamma[1]) :
                    Sig_out[i,j] = Sig_out[i,j].real - gamma[1]*1j # broadening of non-correlated bands

    return Sig_out


def Eigensystem(ham, olp):
    """
    Computes eigensystem.
    Input:
        ham[ndim,ndim]   -- hamiltonian+self_energy
        olp[ndim,ndim]   -- overlap
    Output:
        topol of (Et, Al, Ar) where
          Et[ndim]                     -- band energies
          Al[ndim,ndim], Ar[ndim,ndim] -- left and right eigenvectors
    """
    # make a private copy to change
    hmn = array(ham,order='F')

    # diagonalization of overlap
    (wo, Up) = la.seigval(olp) # created Up such that olp = Up*D*Up^H
    
    # Di = 1/sqrt(wo)
    Di = array(map(lambda x: 1/sqrt(abs(x)), wo),dtype=complex,order='F')
    
    # Rotation to an arthonormal base where overlap is zero
    Rp = la.zproduct_md(Up, Di) # Rp = Up*Di = Up/sqrt(D)

    # Transforms H by Rp  Hnew = Rp^H*H*Rp
    la.ztransform(hmn, Rp, 'N')

    # Solves usual non-hermitian eigenvalue problem using zgeev
    (Et, vlt, vrt) = la.zeigsys(hmn) # call to zgeev

    # Checks causality of eigenvalues. They should be negative definite
    la.check_causality(Et,1e-12)

    # Creates eigenvectors of the original Hamiltonian of non-orthogonal base
    Ar = la.zproduct_nn(Rp, vrt) # rotates right eigenvector back : vr_new = Rp * vr
    Al = la.zproduct_nc(vlt, Rp) # rotates left eigenvector back  : vl_new = vl * Rp^H
    
    # orders eigenvalues according to their real part. Also orders eigenvectors correspondingly
    idxarr = la.eig_order_real_part(Et)
    la.permute_eigensystem(idxarr, Et, Al, Ar)
    
    
    return (Et, Al, Ar)


def Eigensystemf(ham, olp):
    """
    Computes eigensystem.
    Input:
        ham[ndim,ndim]   -- hamiltonian+self_energy
        olp[ndim,ndim]   -- overlap
    Output:
        topol of (Et, Al, Ar) where
          Et[ndim]                     -- band energies
          Al[ndim,ndim], Ar[ndim,ndim] -- left and right eigenvectors
    """
    # Call fortran subroutine for the diagonalization
    (Et, Al, Ar) = dia_gho.eigsys(ham, olp)

    # Checks causality of eigenvalues. They should be negative definite
    la.check_causality(Et,1e-12)

    # orders eigenvalues according to their real part. Also orders eigenvectors correspondingly
    idxarr = la.eig_order_real_part(Et)
    la.permute_eigensystem(idxarr, Et, Al, Ar)
    
    return (Et, Al, Ar)

def Eigenvaluesf(ham, olp):
    """
    Computes eigenvalues only.
    Input:
        ham[ndim,ndim]   -- hamiltonian+self_energy
        olp[ndim,ndim]   -- overlap
    Output:
          Et[ndim]                     -- band energies
    """
    # Call fortran subroutine for the diagonalization
    Et = dia_gho.eigvals(ham, olp)

    # Checks causality of eigenvalues. They should be negative definite
    la.check_causality(Et,1e-20) # correct back !!!!??????

    # orders eigenvalues according to their real part. Also orders eigenvectors correspondingly
    idxarr = la.eig_order_real_part(Et)
    la.permute_eigenvals(idxarr, Et)
    
    return Et


def Eigenvalues(ham, olp):
    """
    Computes bands - energies only
    Input:
        ham[ndim,ndim]   -- hamiltonian+self_energy
        olp[ndim,ndim]   -- overlap
    Output:
        Et[ndim]         -- band energies
    """
    # make a private copy to change
    hmn = array(ham,order='F')

    # diagonalization of overlap
    (wo, Up) = la.seigval(olp) # created Up such that olp = Up*D*Up^H
    
    # Di = 1/sqrt(wo)
    Di = array(map(lambda x: 1/sqrt(abs(x)), wo),dtype=complex,order='F')
    
    # Rotation to an arthonormal base where overlap is zero
    Rp = la.zproduct_md(Up, Di) # Rp = Up*Di = Up/sqrt(D)

    # Transforms H by Rp  Hnew = Rp^H*H*Rp
    la.ztransform(hmn, Rp, 'N')

    # Solves usual non-hermitian eigenvalue problem using zgeev
    Et = la.zeigval(hmn) # call to zgeev

    # Checks causality of eigenvalues. They should be negative definite
    la.check_causality(Et,1e-12)

    # orders eigenvalues according to their real part. Also orders eigenvectors correspondingly
    idxarr = la.eig_order_real_part(Et)
    la.permute_eigenvals(idxarr, Et)
    
    return (Et)

def lcmp_Gloc(z, olp, ham, sig, itt, max_metropolis_steps=0):
    """
    Computes local Green's function using only DMFT base.
    This routine does not store eigenvectors for all k-points and
    requires less memory. This is convenient for large-scale projects.
    Tetrahedron integration is used to sum over the Brillouin zone.
    (Symmetrization is not very precise in DMFT base.)
    Input:
       z                       -- omega+mu
       olp[ndim,ndim,nkp]      -- overlap matrix
       ham[ndim,ndim,nkp]      -- hamiltonian+self_energy
       itt[5,ntet]             -- tetrahedron index pointer
             where
                 itt[0,itet]   -- multiplicity of a tetrahedron
                 itt[1:4,itet] -- corner indexes of a tetrahedron
       max_metropolis_steps=0  -- metropolis steps to sort eigenvalues
    """
    (ndim,ndim1,nkp) = shape(ham)
    # Computes eigenvalues only
    # Convenient for large scale problems where eigenvectors can not be stored for all k-points
    Et = array(zeros((ndim,nkp), dtype=complex), order='F')
    for ikp in range(nkp):
        Et[:,ikp] = Eigenvalues(ham[:,:,ikp]+sig, olp[:,:,ikp])

    # Computes tetrahedron weights
    wgh = tetra.tetra_weights(z, Et, itt, max_metropolis_steps)

    # Computes eigensystem and local green's function using
    # above computed tetrahedron weights
    gloc = matrix(zeros((ndim,ndim),dtype=complex))
    for ikp in range(nkp):
        (Et, Al, Ar) = Eigensystem(ham[:,:,ikp]+sig, olp[:,:,ikp])
        gloc += matrix(la.zproduct_ada(Ar, wgh[:,ikp], Al)) # matrix product: Ar*wgh*Al

    return gloc

def cmp_Gloc(z, olp, ham, sig, itt, max_metropolis_steps=0, cut_a=-1000, cut_b=1000):
    """
    Computes local Green's function using only DMFT base.
    Tetrahedron integration is used to sum over the Brillouin zone.
    (Symmetrization is not very precise in DMFT base.)
    Input:
       z                       -- omega+mu
       olp[ndim,ndim,nkp]      -- overlap matrix
       ham[ndim,ndim,nkp]      -- hamiltonian
       sig[ndim,ndim]          -- local self-energy
       itt[5,ntet]             -- tetrahedron index pointer
             where
                 itt[0,itet]   -- multiplicity of a tetrahedron
                 itt[1:4,itet] -- corner indexes of a tetrahedron
       max_metropolis_steps=0  -- metropolis steps to sort eigenvalues
    """
    (ndim,ndim1,nkp) = shape(ham)
    # Computes eigensystem only
    Et = array(zeros((ndim,nkp), dtype=complex), order='F')
    Al = array(zeros((ndim,ndim,nkp), dtype=complex), order='F')
    Ar = array(zeros((ndim,ndim,nkp), dtype=complex), order='F')
    for ikp in range(nkp):
        (Et[:,ikp], Al[:,:,ikp], Ar[:,:,ikp]) = Eigensystem(ham[:,:,ikp]+sig, olp[:,:,ikp])



    maxE=[]
    minE=[]
    for p in range(ndim):
        maxE.append(max(Et[p,:].real))
        minE.append(min(Et[p,:].real))

    for p in range(ndim):
        p_min = p+1
        if maxE[p]>cut_a: break
    for p in range(ndim-1,-1,-1):
        p_max = p+1
        if minE[p]<cut_b: break
        
    
    # Computes tetrahedron weights
    wgh = tetra.tetra_weights(z, Et, itt, max_metropolis_steps, p_min, p_max)

    # Computes eigensystem and local green's function using
    # above computed tetrahedron weights
    gloc = matrix(zeros((ndim,ndim),dtype=complex))
    for ikp in range(nkp):
        gloc += matrix(la.zproduct_ada(Ar[:,:,ikp], wgh[:,ikp], Al[:,:,ikp])) # matrix product: Ar*wgh*Al

    return gloc


