	SUBROUTINE BND_LMTDFT
****************************************************************
*   Control module for package of programs solved generalized  *
*         eigenvalue problem of the LMTO method                *
****************************************************************
C SET EXTERNAL CONNECTIONS
	USE COM_CONTROL ,ONLY: LIFT,LMTO,IASA,IDMF,NPFR               !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS,NHSORT      !!! from readini
      USE COM_NBANDS  ,ONLY: NBNDF,NBNDE,NBNDT,NBNDC                !!! from readini
	USE COM_FERMI   ,ONLY: EF,ZDFT                                !!! from ttrints
      USE COM_KAPPAR  ,ONLY: NPAN,NKAP                              !!! from readini
	USE COM_KMESH   ,ONLY: NPNT,PNT                               !!! from tetra
      USE COM_WMESH   ,ONLY: NPNW,PNW                               !!! from tetra
	USE COM_LMTOUT  ,ONLY: ICCON,ICOPT,ICCHI,ICODF                !!! from readini
	USE COM_CHANNELS,ONLY: IUCON,IUNMT,IUPIN,IUPDS,IUPFC,IUSCF    !!! from channels
	USE COM_LMTINP  ,ONLY: ICENR                                  !!! from storage
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                            !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION VEL(3),SPTOS(2),SPDOS(2),SPVEL(3,2)
     &   ,VELM(3,3),SPVELM(3,3,2)
	DATA PI/3.141592653589793D0/
C COMPUTE EIGENVALUES AND EIGENVECTORS
	DO IPAN=1,NPAN               !!! over panels
C Memory allocations
	CALL BND_ALLOCATE('A',IPAN)
C Initialzation
	CALL BND_NULTOS(IPAN)        !!! set tos-matrix to zero
	CALL BND_NULINT(IPAN)        !!! set int.density to zero
	CALL FRC_NULPUL(IPAN)        !!! set pulay corrections to zero
C Potential parameters
	CALL BND_SETENY(IPAN)        !!! place Eny somewhere
	CALL BND_CORENY(IPAN)        !!! find binding energies for semicore
	CALL BND_POTPAR(IPAN)        !!! different potential partameters
	CALL BND_KKRPAR(IPAN)        !!! KKR-partameters (only to print)
	CALL BND_BNDPAR(IPAN)        !!! print potential parameters
	CALL BND_LMTPAR(IPAN)        !!! some combinations of parameters
	CALL BND_OVRPAR(IPAN)        !!! potential parameters for overlapping
	CALL BND_NMTPAR(IPAN)        !!! NMT potential parameters
      CALL BND_INTPAR(IPAN)        !!! potential parameters in the int.region
	CALL HUB_HUBPAR(IPAN)        !!! Hubbard corrections
 	CALL BND_ORBPAR(IPAN)        !!! SO-coupling parameters
C PREPARE BANDS FOR WEIGHTS
      IF(NPNW(IPAN).GT.NPNT(IPAN).AND.IPAN.EQ.1.AND.ICENR.EQ.0)THEN
      DO IPNW=1,NPNW(IPAN)     !!! over k-points
C Structure constants
      CALL BND_STRSUM(IPNW,IPAN,1) !!! calculate lattice sums
      CALL BND_STRGNT(IPNW,IPAN,1,0)!! restore bare structure matrices
      CALL HOP_STRALP(IPNW,IPAN,1) !!! screen structure constants
      CALL HOP_SCRSTR(IPNW,IPAN,1) !!! restore screened structure constants
C Hamiltonian and Overlap
      CALL BND_INTMAT(IPNW,IPAN,1) !!! pseudopotential matrix elements
      CALL BND_LMTMAT(IPNW,IPAN,1) !!! hamiltonian and overlap
      CALL BND_LMTBND(IPNW,IPAN,1) !!! solve eigenvalue problem
      ENDDO                    !!! over k-points
	CLOSE(UNIT=IUCON,STATUS='DELETE')
	ELSEIF(NPNW(IPAN).GT.NPNT(IPAN).AND.IPAN.EQ.1.AND.ICENR.EQ.2)THEN
	CALL BND_GETENR
      ENDIF
C COMPUTE BANDS AND WAVE FUNCTIONS
	DO IPNT=1,NPNT(IPAN)     !!! over k-points
C Structure constants
	CALL BND_STRSUM(IPNT,IPAN,0) !!! calculate lattice sums
	CALL BND_STRGNT(IPNT,IPAN,0,0)!! restore bare structure matrices
      CALL HOP_STRALP(IPNT,IPAN,0) !!! screen structure constants
      CALL HOP_SCRSTR(IPNT,IPAN,0) !!! restore screened structure constants
C Hamiltonian and Overlap
      CALL BND_INTMAT(IPNT,IPAN,0) !!! pseudopotential matrix elements
	CALL BND_LMTMAT(IPNT,IPAN,0) !!! hamiltonian and overlap
      CALL HOP_GETHTB(IPNT,IPAN,0) !!! get tight-binding parameters
	CALL BND_LMTBND(IPNT,IPAN,0) !!! solve eigenvalue problem
C Density/Forces
      CALL FRC_LMTVAR(IPNT,IPAN,0) !!! calculate kj-contribution to Pulay force
 	CALL BND_TOSPAR(IPNT,IPAN,0) !!! BZ-integrals from the occupied states
      CALL FRC_INTVAR(IPNT,IPAN,0) !!! pseudopotential contribution to  pulay
      CALL BND_TOSINT(IPNT,IPAN,0) !!! pseudodensity from the occupied states
      CALL FRC_PULINT(IPNT,IPAN,0) !!! Pulay forces from the occupied states
	CALL BND_LMTFAT(IPNT,IPAN,0) !!! tl-characters of the bands
      ENDDO                    !!! over k-points
      CALL BND_TOSEND(IPAN,0)      !!! make final pseudodensity
      CALL FRC_PULEND(IPAN)        !!! make final Pulay force
	CLOSE(IUNMT,STATUS='DELETE')
	CLOSE(IUPIN,STATUS='DELETE')
	CLOSE(IUPDS,STATUS='DELETE')
	CLOSE(IUPFC,STATUS='DELETE')
	CALL BND_ALLOCATE('D',IPAN)
      ENDDO                        !!! over panels
	IF(ICCON.EQ.0)CLOSE(UNIT=IUCON,STATUS='DELETE')
	IF(ICCON.NE.0)CLOSE(UNIT=IUCON,STATUS='KEEP')
	IF(IDMF.GT.2)RETURN
C Store Data
	CALL BND_SETFAT              !!! store fat bands
	CALL BND_SETDOS              !!! store density of states
	CALL BND_SETFRS              !!! store fermi surface
	CALL BND_SETENR              !!! store bands for weights
	IF(LIFT.LE.2)RETURN
C FIND FERMI LEVEL
	WRITE(IUN,10010)
10010 FORMAT(1X,'Start finding chemical potential -->')
	CALL BND_INSUL(EF,ZDFT)      !!! recognize insulating case
	CALL BND_FNDFRM(EF,ZDFT,EF,100,1.D-09,0.8D0)
	CALL TTR_TTRVEL(EF,DOS,VEL,SPTOS,SPDOS,SPVEL,VELM,SPVELM,0)
	CALL BND_NBAND(EF)                      !!! define # of filled bands
	CALL BND_LRCHI(EF,MBNDF,MBNDE,MBNDC)    !!! define # of bands for LR
C TETRAHEDRON METHOD FOR CONDUCTION BANDS
	IF(NBNDF(1).GE.NBNDE(1))THEN
	CALL BND_ALLOCATE('A',1)
	CALL BND_NULINT(1)
	CALL FRC_NULPUL(1)
	DO IPNT=1,NPNT(1)                        !!! over k-points
	CALL BND_TOSPAR(IPNT,1,1)
	CALL BND_TOSINT(IPNT,1,1)
	CALL FRC_PULINT(IPNT,1,1)
      ENDDO                                    !!! over k-points
	CALL BND_TOSEND(1,0)
	CALL FRC_PULEND(1)
	CLOSE(UNIT=IUPDS,STATUS='DELETE')
	CLOSE(UNIT=IUPFC,STATUS='DELETE')
	CALL BND_ALLOCATE('D',1)
	ENDIF
C SYMMETRIZE NUMBER OF STATES MATRIX
      CALL RHO_RHOFILES(IUSCF,'!','A')         !!! initialize new charge density
	CALL BND_SYMTOS                          !!! symmetrization
	END






	SUBROUTINE BND_INSUL(EF,ZVALTOT)
***********************************************************
*  Recognizes insulating case.                            *
***********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA                        !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS          !!! from readini
      USE COM_KAPPAR  ,ONLY: NPAN,NKAP                        !!! from readini
	USE COM_NBANDS  ,ONLY: NBNDF,NBNDE,NBNDT,NBNDC          !!! from readini
      USE COM_KMESH   ,ONLY: NPNT,PNT                         !!! from tetra
	USE COM_INPTTR  ,ONLY: NDIV,NDIC,NDIW                   !!! from readini
      USE COM_WMESH   ,ONLY: NPNW,PNW                         !!! from tetra
      USE COM_ENERGY  ,ONLY: EARR,EARW                        !!! from bands
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DATA PI/3.141592653589793D0/
	IF(ABS(MOD(ZVALTOT,2.D0)).GE.1.D-10)RETURN
	NB=ZVALTOT/2.D0
	EVMAX=EARR(1,1,1)
	EEMIN=EARR(NBNDT(1)*NSPIN,1,1)
	DO IPNT=1,NPNT(1)
	DO IV=1,NB
	DO IBNDS=1,NSPIN
	IBAND=IBNDS+NSPIN*(IV-1)
	EVMAX=MAX(EVMAX,EARR(IBAND,IPNT,1))
	ENDDO
	ENDDO
	DO IE=NB+1,NBNDT(1)
	DO IBNDS=1,NSPIN
	IBAND=IBNDS+NSPIN*(IE-1)
	EEMIN=MIN(EEMIN,EARR(IBAND,IPNT,1))
	ENDDO
	ENDDO
	ENDDO
	GAP=EEMIN-EVMAX
	IF(GAP.LE.0.D0)RETURN
	EF=0.5D0*(EEMIN+EVMAX)
	WRITE(IUN,1010)
	WRITE(IUN,1020)GAP,GAP*13.6058D0
1010  FORMAT(1X,'Semiconducting band structure discovered --------->')
1020  FORMAT(1X,'dielectric gap value is ',G14.7,' Ry ; ',G14.7,' eV')
	END






	SUBROUTINE BND_NBAND(EF)
*************************************************
*   Define a number of filled and empty bands   *
*************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA,NPFR                   !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS          !!! from readini
      USE COM_KAPPAR  ,ONLY: NPAN,NKAP                        !!! from readini
      USE COM_NBANDS  ,ONLY: NBNDF,NBNDE,NBNDC,NBNDT          !!! from readini
     &                      ,MBNDF,MBNDE,MBNDC
      USE COM_KMESH   ,ONLY: NPNT,PNT                         !!! from tetra
      USE COM_ENERGY  ,ONLY: EARR                             !!! from bands
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
      USE COM_LABELS  ,ONLY: TXTSP                            !!! from setup
C SET LOCAL VARIBLES
	IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER NBNDEMP(2),NBNDFIL(2)
C DEFINE # OF FILLED BANDS IF THERE IS NO SPIN-ORBIT COUPLING
	IF(NORBS.EQ.1)THEN
	DO IBNDS=1,NSPIN
	NBNDEMP(IBNDS)=0
	NBNDFIL(IBNDS)=0
	DO IBNDT=1,NBNDT(1)
	IBAND=IBNDS+NSPIN*(IBNDT-1)
	N1=0
	N2=0
	DO IPNT=1,NPNT(1)
	IF(EARR(IBAND,IPNT,1).LT.EF)THEN
	N1=IBNDT
	ELSEIF(EARR(IBAND,IPNT,1).GE.EF)THEN
	N2=IBNDT
	ENDIF
	ENDDO 
	IF(N1.NE.0.AND.N2.EQ.0)THEN
	NBNDEMP(IBNDS)=N1+1
	ELSEIF(N1.EQ.0.AND.N2.NE.0)THEN
	NBNDFIL(IBNDS)=N2-1
	EXIT
	ENDIF
      ENDDO   
      ENDDO    
	MBNDFIL=MAX(NBNDFIL(1),NBNDFIL(NSPIN))
	MBNDEMP=MIN(NBNDEMP(1),NBNDEMP(NSPIN))
	IF(MBNDEMP.EQ.0)MBNDEMP=1
C DEFINE # OF FILLED BANDS IF THERE IS SPIN-ORBIT COUPLING
	ELSEIF(NORBS.EQ.2)THEN
	MBNDEMP=0
	MBNDFIL=0
	DO IBAND=1,NBNDT(1)*NSPIN
	N1=0
	N2=0
	DO IPNT=1,NPNT(1)
	IF(EARR(IBAND,IPNT,1).LT.EF)THEN
	N1=IBAND
	ELSEIF(EARR(IBAND,IPNT,1).GT.EF)THEN
	N2=IBAND
	ENDIF
	ENDDO
	IF(N1.NE.0.AND.N2.EQ.0)THEN
	MBNDEMP=N1+1
	ELSEIF(N1.EQ.0.AND.N2.NE.0)THEN
	MBNDFIL=N2-1
	EXIT
	ENDIF
      ENDDO   
	IF(MOD(MBNDFIL,2).NE.0)MBNDFIL=MBNDFIL-1
	IF(MOD(MBNDEMP,2).NE.0)MBNDEMP=MBNDEMP+1
	IF(MBNDFIL.LT.0)MBNDFIL=0
	MBNDFIL=MBNDFIL/2
	MBNDEMP=MBNDEMP/2
	ENDIF
	IF(NBNDF(1).LT.MBNDFIL)WRITE(IUT,*)'# of filled bands mismatches'
	IF(NBNDE(1).GT.MBNDEMP)WRITE(IUT,*)'# of empty  bands mismatches'
	WRITE(IUN,1010)MBNDEMP-1,NBNDE(1)-1
	WRITE(IUN,1020)MBNDFIL-MBNDEMP+1,NBNDF(1)-NBNDE(1)+1
C SET NEW # OF FILLED AND EMPTY BANDS NOW
	MBNDF=NBNDF(1)
	MBNDE=NBNDE(1)
	IF(NBNDF(1).LT.MBNDFIL)MBNDF=MBNDFIL
	IF(NBNDE(1).GT.MBNDEMP)MBNDE=MBNDEMP
	MBNDC=MBNDF-MBNDE+1
C PRINT BANDS AT THE G-POINT
	IPNTG=0
	DO IPNT=1,NPNT(1)
	ABSPNT=SQRT(PNT(1,IPNT,1)**2+PNT(2,IPNT,1)**2+PNT(3,IPNT,1)**2)
	IF(ABSPNT.LE.1.D-04)IPNTG=IPNT
	ENDDO
	IF(IPNTG.NE.0.AND.NORBS.EQ.1)THEN
	DO IBNDS=1,NSPIN
	IF(NSPIN.EQ.1)WRITE(IUN,1030)
	IF(NSPIN.EQ.2)WRITE(IUN,1040)TXTSP(IBNDS)
	WRITE(IUN,1050)(EARR(IBNDS+NSPIN*(IBNDT-1),IPNTG,1),
     &IBNDT=1,NBNDT(1))
      ENDDO   
	ELSEIF(IPNTG.NE.0.AND.NORBS.EQ.2)THEN
	WRITE(IUN,1030)
	WRITE(IUN,1050)((EARR(IBNDS+NSPIN*(IBNDT-1),IPNTG,1),
     &IBNDS=1,NSPIN),IBNDT=1,NBNDT(1))
	ENDIF
1010  FORMAT(1X,'# of fully filled bands =',I4,'; input # =',I4)
1020  FORMAT(1X,'# of bands crossing Ef  =',I4,'; input # =',I4)
1030  FORMAT(1X,'Energy bands at the Gamma-point are')
1040  FORMAT(1X,'Energy bands at the Gamma-point for spin-',
     &A2,' states are')
1050  FORMAT(1X,5(G12.5,1X))
	END



      SUBROUTINE BND_LRCHI(EF,MBNDF,MBNDE,MBNDC)
***********************************************************
*  Prints the bands within +/- window from Ef.            *
***********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA,NPFR                         !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS                !!! from readini
      USE COM_KAPPAR  ,ONLY: NPAN,NKAP                              !!! from readini
      USE COM_NBANDS  ,ONLY: NBNDF,NBNDE,NBNDT,NBNDC,NBNDH,WINDOW   !!! from readini
      USE COM_KMESH   ,ONLY: NPNT,PNT                               !!! from tetra
      USE COM_ENERGY  ,ONLY: EARR,EARW                              !!! from bands
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                            !!! from setup
      USE COM_LABELS  ,ONLY: TXTSP                                  !!! from setup
C SET LOCAL VARIBLES
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER NBNDFIL(2),NBNDEMP(2)
      DATA PI/3.141592653589793D0/
C DEFINE # OF FILLED BANDS IF THERE IS NO SPIN-ORBIT COUPLING
      IF(NORBS.EQ.1)THEN
      DO IBNDS=1,NSPIN
      NBNDFIL(IBNDS)=0
      DO IBNDT=1,NBNDT(1)
      IBAND=IBNDS+NSPIN*(IBNDT-1)
      IWINF=0
      DO IPNT=1,NPNT(1)
      IF(EARR(IBAND,IPNT,1).LT.EF-WINDOW)IWINF=IWINF+1
      ENDDO
      IF(IWINF.EQ.NPNT(1))NBNDFIL(IBNDS)=IBNDT
      ENDDO
      NBNDEMP(IBNDS)=NBNDT(1)+1
      DO IBNDT=NBNDT(1),1,-1
      IBAND=IBNDS+NSPIN*(IBNDT-1)
      IWINE=0
      DO IPNT=1,NPNT(1)
      IF(EARR(IBAND,IPNT,1).GT.EF+WINDOW)IWINE=IWINE+1
      ENDDO
      IF(IWINE.EQ.NPNT(1))NBNDEMP(IBNDS)=IBNDT
      ENDDO
      ENDDO
      MBNDFIL=MIN(NBNDFIL(1),NBNDFIL(NSPIN))
      MBNDEMP=MAX(NBNDEMP(1),NBNDEMP(NSPIN))
C DEFINE # OF FILLED BANDS IF THERE IS SPIN-ORBIT COUPLING
      ELSEIF(NORBS.EQ.2)THEN
      MBNDFIL=0
      DO IBAND=1,NBNDT(1)*NSPIN
      IWINF=0
      DO IPNT=1,NPNT(1)
      IF(EARR(IBAND,IPNT,1).LT.EF-WINDOW)IWINF=IWINF+1
      ENDDO
      IF(IWINF.EQ.NPNT(1))MBNDFIL=IBAND
      ENDDO
      MBNDEMP=NBNDT(1)*NSPIN+1
      DO IBAND=NBNDT(1)*NSPIN,1,-1
      IWINE=0
      DO IPNT=1,NPNT(1)
      IF(EARR(IBAND,IPNT,1).GT.EF+WINDOW)IWINE=IWINE+1
      ENDDO
      IF(IWINE.EQ.NPNT(1))MBNDEMP=IBAND
      ENDDO
      IF(MOD(MBNDFIL,2).NE.0)MBNDFIL=MBNDFIL-1
      IF(MOD(MBNDEMP,2).NE.0)MBNDEMP=MBNDEMP+1
      IF(MBNDFIL.LT.0)MBNDFIL=0
      MBNDFIL=MBNDFIL/2
      MBNDEMP=MBNDEMP/2
      ENDIF
      WRITE(IUN,1010)MBNDFIL
      WRITE(IUN,1020)MBNDEMP-MBNDFIL-1
      MBNDF=MBNDEMP-1
      MBNDE=MBNDFIL+1
      MBNDC=MBNDEMP-MBNDFIL-1
1010  FORMAT(1X,'LR information: # of fully filled bands (nff) =',I4)
1020  FORMAT(1X,'LR information: # of bands crossing  EF (nef) =',I4)
      END
