!---calculate Hamiltonian matrix-------------------------------------------------------------------------------------------------------------
subroutine cls_Hamiltonian1(Ns, Nd, N, subdim, Es, Ed, UMAT, VMAT, B1, B2, B3, SOC, phi, invphi, precont, H)

implicit none

integer Ns
integer Nd
integer N
integer subdim
complex*16 Es(0:Ns-1,0:Ns-1)
complex*16 Ed(Ns:N-1,Ns:N-1)
real*8 UMAT(0:Nd/2-1,0:Nd/2-1,0:Nd/2-1,0:Nd/2-1)
complex*16 VMAT(Ns:N-1,0:Ns-1)
real*8 B1
real*8 B2
real*8 B3
real*8 SOC
integer phi(subdim)
integer invphi(2**N)
integer precont
complex*16 H(subdim,subdim)

integer dummy1, dummy2, dummy3, dummy4
integer Ld
integer sign, sign1, sign2, sign3, sign4, isign
integer i, j
integer m1, m2, k1, k2, m
integer ms1, ms2, ks1, ks2, ms
integer spin1, spin2, spin
integer ipr
complex*16, allocatable :: H0s(:,:)
complex*16, allocatable :: H0d(:,:)
complex*16, allocatable :: Hdd(:,:)
complex*16, allocatable :: Hsd(:,:)
complex*16, allocatable :: Hso(:,:)
complex*16, allocatable :: Hmagd(:,:)
complex*16, allocatable :: Hmags(:,:)

integer,external :: cls_parity
integer,external :: cls_opsign
integer,external :: cls_spind
integer,external :: cls_spins
real*8,external :: cls_SLM

!---------------------------------------------------------------------------------
N = Ns + Nd
ipr = Nd/2 + Ns - 1
Ld = (Nd - 2)/4

!----Unperturbed H0s --------------------------------------------------
allocate(H0s(subdim, subdim))
H0s = (0.D0,0.D0)
do j = 1, subdim
  do ms2 = 0, Ns-1
    if (.NOT.btest(phi(j),ms2)) cycle
	dummy1 = phi(j)
	dummy1 = ibclr(dummy1,ms2)
	sign2 = cls_opsign(ms2,dummy1)
	do ms1 = 0, Ns-1
	   if (btest(dummy1,ms1)) cycle
	   dummy2 = dummy1
	   dummy2 = ibset(dummy2,ms1)
	   sign1 = cls_opsign(ms1,dummy2)
	   sign = sign1 * sign2
	   if (invphi(dummy2) == 0) cycle
	   i = invphi(dummy2) - precont
	   H0s(i,j) = H0s(i,j) + sign * Es(ms1,ms2)
    end do
  end do
end do

!----Unperturbed H0d --------------------------------------------------
allocate(H0d(subdim, subdim))
H0d = (0.D0,0.D0)
do j = 1, subdim
  do ms2 = Ns, N-1
    if (.NOT.btest(phi(j),ms2)) cycle
	dummy1 = phi(j)
    dummy1 = ibclr(dummy1,ms2)
	sign2 = cls_opsign(ms2,dummy1)
	do ms1 = Ns, N-1
	   if (btest(dummy1,ms1)) cycle
	   dummy2 = dummy1
	   dummy2 = ibset(dummy2,ms1)
	   sign1 = cls_opsign(ms1,dummy2)
	   sign = sign1 * sign2
	   if (invphi(dummy2) == 0) cycle
	   i = invphi(dummy2) - precont
	   H0d(i,j) = H0d(i,j) + sign * Ed(ms1,ms2)
    end do
  end do
end do

!----Correlation ------------------------------------------------------------------
allocate(Hdd(subdim, subdim))
Hdd = (0.D0,0.D0)
do j = 1, subdim
  do ks1 = Ns, N-1
    if (.NOT.btest(phi(j),ks1)) cycle
	dummy1 = phi(j)
	dummy1 = ibclr(dummy1,ks1)
	sign1 = cls_opsign(ks1,dummy1)
	do ks2 = Ns, N-1
	  if (.NOT.btest(dummy1,ks2)) cycle
      dummy2 = dummy1
	  dummy2 = ibclr(dummy2,ks2)
	  sign2 = cls_opsign(ks2,dummy2)
	  do ms2 = Ns, N-1
	  	if (cls_spind(ms2,ipr) /= cls_spind(ks2,ipr)) cycle
		if (btest(dummy2,ms2)) cycle
		dummy3 = dummy2
		dummy3 = ibset(dummy3,ms2)
		sign3 = cls_opsign(ms2,dummy3)
	    do ms1 = Ns, N-1
		  if (cls_spind(ms1,ipr) /= cls_spind(ks1,ipr)) cycle
		  if (btest(dummy3,ms1)) cycle
		  dummy4 = dummy3
		  dummy4 = ibset(dummy4,ms1)
		  sign4 = cls_opsign(ms1,dummy4)
		  sign = sign1 * sign2 * sign3 * sign4
		  m1 = mod((ms1 - Ns), (Nd/2))
		  k1 = mod((ks1 - Ns), (Nd/2))
		  m2 = mod((ms2 - Ns), (Nd/2))
		  k2 = mod((ks2 - Ns), (Nd/2))
		  if (invphi(dummy4) == 0) cycle
		  i = invphi(dummy4) - precont
          Hdd(i,j) = Hdd(i,j) + sign * 0.5 * UMAT(m1,k1,m2,k2)
		end do
	  end do
	end do
  end do
end do

!----Hybridization -------------------------------------------------------
allocate(Hsd(subdim, subdim))
Hsd= (0.D0,0.D0)
if (Ns/=0) then
do j=1, subdim
  do ms1=Ns, N-1
    do ks1=0, Ns-1
	  if (btest(phi(j),ms1) .EQV. btest(phi(j),ks1)) cycle
	  dummy1 = phi(j)
	  if (btest(phi(j),ks1)) then
	    dummy1 = ibclr(dummy1,ks1)
	    sign1 = cls_opsign(ks1,dummy1)
	    dummy1 = ibset(dummy1,ms1)
	    sign2 = cls_opsign(ms1,dummy1)
	    if (invphi(dummy1) == 0) cycle
		i = invphi(dummy1) - precont
		Hsd(i,j) = Hsd(i,j) + sign1 * sign2 * VMAT(ms1,ks1)
	  else
	    dummy1 = ibclr(dummy1,ms1)
	    sign1 = cls_opsign(ms1,dummy1)
	    dummy1 = ibset(dummy1,ks1)
	    sign2 = cls_opsign(ks1,dummy1)
	    if (invphi(dummy1) == 0) cycle
		i = invphi(dummy1) - precont
		Hsd(i,j) = Hsd(i,j) + sign1 * sign2 * conjg(VMAT(ms1,ks1))
      end if
	end do
  end do
end do
end if

!----Impurity magnetic energy ------------------------------------------------------
allocate(Hmagd(subdim, subdim))
Hmagd= (0.D0,0.D0)
do i = 1, subdim
  do ms = Ns, N-1
    if (btest(phi(i),ms)) then
    spin = cls_spind(ms, ipr)
    m = mod((ms - Ns), (Nd/2)) - Ld
	Hmagd(i,i) = Hmagd(i,i) - B1*spin - B2*m
	end if
  end do
end do

!----Bath magnetic energy ------------------------------------------------------
allocate(Hmags(subdim, subdim))
Hmags= (0.D0,0.D0)
do i = 1, subdim
  do ms = 0, Ns
    if (btest(phi(i),ms)) then
    spin = cls_spins(ms, Ns)
	Hmags(i,i) = Hmags(i,i) - B3*spin
	end if
  end do
end do

!----Spin-orbital coupling ----------------------------------------------------------------
allocate(Hso(subdim, subdim))
Hso= (0.D0,0.D0)
do j = 1, subdim
  do ms2 = Ns, N-1
    if (.NOT.btest(phi(j),ms2)) cycle
	dummy1 = phi(j)
    dummy1 = ibclr(dummy1,ms2)
	sign2 = cls_opsign(ms2,dummy1)
	do ms1 = Ns, N-1
	  if (btest(dummy1,ms1)) cycle
	  dummy2 = dummy1
	  dummy2 = ibset(dummy2,ms1)
	  sign1 = cls_opsign(ms1,dummy2)
	  sign = sign1 * sign2
	  if (invphi(dummy2) == 0) cycle
	  i = invphi(dummy2) - precont
	  m1 = mod((ms1 - Ns), (Nd/2)) - Ld
	  m2 = mod((ms2 - Ns), (Nd/2)) - Ld
	  spin1 = cls_spind(ms1,ipr)
	  spin2 = cls_spind(ms2,ipr)
	  Hso(i,j) = Hso(i,j) + sign * cls_SLM(Ld, m1, m2, spin1, spin2)*SOC
	end do
  end do
end do

!----sum parts to get total Hamiltonian-----------------
H = (0.D0,0.D0)
H = H0s + H0d + Hdd + Hsd + Hso + Hmagd + Hmags

deallocate(H0s)
deallocate(H0d)
deallocate(Hdd)
deallocate(Hsd)
deallocate(Hso)
deallocate(Hmagd)
deallocate(Hmags)

end