subroutine cls_main(ver, Nd, Ns, Ed, Es, U, V, B1, B2, B3, SOC, E_grid, E_array, Mode, Ecut1, Ecut2, &
& Fill, Dfill, Dran, Chem, Temp, damp, Locc, Channel, filename, Green1, Green2, Self_E, Self_Einf, &
& mor, Mom, Den, DOS)

implicit none

!----Mode ( two-digit integer )------------------------------------------------------------------------------------------------------
! First digit controls:
! 1 -- Only calculates Eigenvalues & Eigenvectors.
! 2 -- Returns Eigenvalues & Eigenvectors, and Angular Momentums.
! 3 -- Returns Eigenvalues, Green's function, Self-energy, Density matrix, DOS.
! 4 -- Returns Eigenvalues, Green's function, Self-energy, Density matrix, DOS, and Angular Momentums.
! Second digit controls:
! 1 -- Fix mean filling, search for chemical potential.
! 2 -- Fix chemical potential, search for mean filling.

!-----All input & output matrices are in Spherical Harmonics representation----------------------------------
integer ver                                                          ! Version of solver (1 or 2)
integer Nd                                                           ! Degeneracy of correlated shell.
integer Ns                                                           ! Degeneracy of uncorrelated shell.
complex*16 Ed(Nd,Nd)                                     ! One-eletron energy matrix of correlated shell.
complex*16 Es(Ns,Ns)                                      ! One-eletron energy matrix of uncorrelated shell.
real*8 U(Nd/2,Nd/2,Nd/2,Nd/2)                          ! Correlation matrix.
complex*16 V(Nd,Ns)                                        ! Hybridization matrix.
real*8 B1                                                             ! Impurity spin magnetic field.
real*8 B2                                                             ! Orbital magnetic field.
real*8 B3                                                             ! Bath spin magnetic field.
real*8 SOC                                                         ! Spin-orbital coupling. 
integer E_grid                                                    ! Number of energy points in Green's function or self-energy.
complex*16 E_array(E_grid)                            ! Energy points array.
integer Mode                                                      ! Running mode control.
integer maxdim                                                  ! Maximum dimension allowed for Hamiltonian matrix.
real*8 Ecut1                                                        ! Energy truncation of Hamiltonian.
real*8 Ecut2                                                        ! Energy truncation of Hamiltonian.
real*8  Fill                                                            ! Mean filling.
integer  Dfill                                                        ! Impurity mean filling.
integer  Dran                                                      ! Impurity charge fluctuation range.
real*8 Chem                                                       ! Chemical potential.
real*8 Temp                                                       ! Temperature.
real*8 damp                                                        ! Damping parameter.
integer Locc(0:Nd+Ns)                                      ! Number of elecrons control array.
integer Channel(6)                                             ! I/O channel.
character(len=130) filename(5)                          ! Filename string.
complex*16 Green1(Ns+Nd, Ns+Nd, E_grid) ! Return array, the imaginary Green's function matrix.
complex*16 Green2(Ns+Nd, Ns+Nd, E_grid) ! Return array, the imaginary Green's function matrix.
complex*16 Self_E(Ns+Nd, Ns+Nd, E_grid)  ! Return array, the self-energy matrix.
complex*16 Self_Einf(Ns+Nd, Ns+Nd)           ! Return array, the self-energy at infinity.
integer Mor                                                        ! Order of moments.
complex*16 Mom(Ns+Nd, Ns+Nd, -mor:mor) ! Return moments of Green's function.
complex*16 Den(Ns+Nd, Ns+Nd, -mor:mor) ! Return array, the density matrix.
real*8 DOS(E_grid)                                           ! Return density of states

if (ver == 1) call cls_main1(Nd, Ns, Ed, Es, U, V, B1, B2, B3, SOC, E_grid, E_array, Mode, Fill, Ecut1, Chem, & 
& Temp, damp, Locc, Channel, filename, Green1, Green2, Self_E, Self_Einf, mor, Mom, Den, DOS)

if (ver ==2) call cls_main2(Nd, Ns, Ed, Es, U, V, B1, B2, B3, SOC, E_grid, E_array, Mode, Ecut1, Ecut2, &
& Fill, Dfill, Dran, Chem, Temp, damp, Locc, Channel, filename, Green1, Green2, Self_E, Self_Einf, &
& mor, Mom, Den, DOS)

end