subroutine CLS_main1(Nd, Ns, Ed, Es, U, V, B1, B2, B3, SOC, E_grid, E_array, Mode, Fill, Ese, Chem, & 
& Temp, damp, Locc, Channel, filename, Green1, Green2, Self_E, Self_Einf, mor, Mom, Den, DOS)

!include 'link_f90_static.h'

implicit none

!-----All input & output matrices are in Spherical Harmonics representation-----------------------------------
integer Nd                                  ! Degeneracy of correlated shell.
integer Ns                                  ! Degeneracy of uncorrelated shell.
complex*16 Ed(Nd,Nd)            ! One-eletron energy matrix of correlated shell.
complex*16 Es(Ns,Ns)             ! One-eletron energy matrix of uncorrelated shell.
real*8 U(Nd/2,Nd/2,Nd/2,Nd/2) ! Correlation matrix.
complex*16 V(Nd,Ns)               ! Hybridization matrix.
real*8 B1                                    ! Impurity spin magnetic field.
real*8 B2                                    ! Orbital magnetic field.
real*8 B3                                    ! Bath spin magnetic field.
real*8 SOC                                ! Spin-orbital coupling. 
integer E_grid                           ! Number of energy points in Green's function or self-energy.
complex*16 E_array(E_grid)   ! Energy points array.
integer Mode                            ! Running mode control.
real*8  Fill                                   ! Mean filling.
real*8 Ese                                  ! Energy truncation.
real*8 Chem                              ! Chemical potential.
real*8 Temp                              ! Temperature.
real*8 damp                              ! Damping parameter.
integer Locc(0:Nd+Ns)            ! Number of elecrons control array.
integer Channel(6)                   ! I/O channel.
character(len=130) filename(5)! Filename string.
complex*16 Green1(Ns+Nd, Ns+Nd, E_grid) ! Return array, the imaginary Green's function matrix.
complex*16 Green2(Ns+Nd, Ns+Nd, E_grid) ! Return array, the imaginary Green's function matrix.
complex*16 Self_E(Ns+Nd, Ns+Nd, E_grid)  ! Return array, the self-energy matrix.
complex*16 Self_Einf(Ns+Nd, Ns+Nd)           ! Return array, the self-energy at infinity.
integer Mor                                                        ! Order of moments.
complex*16 Mom(Ns+Nd, Ns+Nd, -5:5)          ! Return moments of Green's function.
complex*16 Den(Ns+Nd, Ns+Nd, -5:5)           ! Return array, the density matrix.
real*8 DOS(E_grid)                                           ! Return density of states

!--Mode ( two-digit integer )-------------------------------------------------------------------------------------------------------
! First digit controls:
! 1 -- Only calculates Eigenvalues & Eigenvectors.
! 2 -- Returns the above quantities plus angular moments.
! 3 -- Returns Eigenvalues, Green's function, self-energy, density matrix, DOS.
! 4 -- Returns Eigenvalues, Green's function, self-energy, density matrix, DOS, plus angular moments.
! Second digit controls:
! 1 -- Fix mean filling, search for chemical potential.
! 2 -- Fix chemical potential, search for mean filling.
!---------------------------------------------------------------------------------------------------------------------------------------------

integer N
integer in
integer dim, dimvar
integer precont
integer subdim
integer ni, nj, inp
complex*16, allocatable :: H(:,:)
complex*16, allocatable :: A(:,:)
real*8, allocatable :: E(:)
integer, allocatable :: phivar(:)
integer, allocatable :: phi(:)
integer, allocatable :: invphi(:)
integer, allocatable :: fcont(:)
integer, allocatable :: fcontin(:)
character(len=32) formats

real*8, allocatable :: ML(:)                          ! Projection of impurity orbital momentum.
real*8, allocatable :: MS(:)                         ! Projection of impurity spin momentum.
real*8, allocatable :: MJ(:)                          ! Projection of impurity total angular momentum.
real*8, allocatable :: S(:)                            ! Total impurity spin momentum.
real*8, allocatable :: L(:)                             ! Total impurity orbital momentum.
real*8, allocatable :: J(:)                             ! Total impurity angular momentum.
real*8, allocatable :: Occ(:,:)                      ! Occupation number of cluster.
real*8, allocatable :: MSC(:)                      ! Projection of cluster spin momentum.
real*8, allocatable :: SC(:)                         ! Total cluster spin momentum.

real*8, external :: cls_combination   ! Combination number function.
integer, external :: cls_dnum            ! d-electron number function.

!------------------------------------------------------------------------------------------------------------------------------------
write (channel(6),*) "Solver Version: QY1"
N = Ns + Nd

if (mod(Mode,10) == 1 .and. nint(Fill) > N) then 
  write(channel(6),*) 'Wrong!! Mean filling can not be larger than total deeneracy!'
  stop
end if

!----prepare Fock vectors -------------------------------------------------------------------------
write (channel(6),*) 'Generating Fock vectors...'
dimvar = 0
do in = 0, N
  if (locc(in) == 0) cycle
  dimvar = dimvar + nint(cls_combination(in, N))
end do
allocate (fcontin(0:N))
allocate (fcont(0:N))
fcontin = 0
fcont = 0
allocate (phivar(dimvar))
allocate (invphi(2**N))
phivar = 0
invphi = 0
call cls_Fock1(Ns, Nd, N, dimvar, phivar, invphi, locc, Ese, Es, Ed, U, B1, B2, B3, Soc, fcontin, fcont)
write (channel(6),*) 'Fock vectors finished'

dim = 0
do in = 0, N
  dim = dim + fcontin(in)
end do

allocate (phi(dim))
phi = 0
phi(1:dim) = phivar(1:dim)
deallocate (phivar)

!----------------------------------------------------------------------------------------------------------------------------
allocate (ML(dim))
allocate (MS(dim))
allocate (MJ(dim))
allocate (L(dim))
allocate (S(dim))
allocate (J(dim))
allocate (SC(dim))
allocate (MSC(dim))
allocate (OCC(N, dim))

!----------------------------------------------------------------------------------------------------------------------------
open (unit=channel(1), file=filename(1), form='UNFORMATTED')
open (unit=channel(2), file=filename(2), form='UNFORMATTED')

do in = 0, N

  if (locc(in)==0) cycle
  
  write (channel(6),*) 'Current number of electrons:', in

  subdim = fcontin(in)
  if (subdim == 0) cycle
  
  inp = fcont(in) -  fcontin(in) + 1
  
  if (in == 0) then 
    precont = 0
  else
    precont = fcont(in-1)
  end if
  
!----calculate block Hamiltonian Matrix ------------------------------------------------------------------------------------------------
  allocate (H(subdim,subdim))
  call cls_Hamiltonian1(Ns, Nd, N, subdim, Es, Ed, U, V, B1, B2, B3, SOC, phi(inp), invphi, precont, H)
  
!-----diagonalize the block Hamiltonian Matrix----------------------------------------------------------------------------------------
  allocate (A(subdim,subdim))
  allocate (E(subdim))
  call cls_exactdiag(subdim, H, A, E)
  deallocate(H)
  write (channel(6),*) 'Diagonalization finished, saving Eigenvalues and Eigenvectors...'

!---store eigenvalues & eigenvectors into files----------------------------------------------------------------
  do nj = 1, subdim
    write (channel(1)) E(nj)
    do ni = 1, subdim
      write (channel(2)) A(ni,nj)
    end do
  end do
  
!---calculate angular moments and occupation numbers-----------------------------------------------
  if (Mode/10 == 2 .or. Mode/10 == 4) then
    write (channel(6),*) 'Start calculating angular moments and occupation numbers...'
!    call cls_angmom1(Ns, Nd, N, subdim, phi(inp), invphi, precont, A, ML(inp), MS(inp), MJ(inp), L(inp), S(inp), J(inp), &
!                                 & Occ(1, inp), MSC(inp), SC(inp)) 
                                 
    write (channel(6), '(/, A35, I4)') 'Current number of electrons :  ', in
    write (channel(6), *) '  #   Energy    ML     MS    MJ    L     S     J    MSC   SC     Occupancy'
    formats = '(I4, F10.3, 8F6.2, 3X, 00F5.2)'
    write (formats(24:25), '(I2)') N
    do ni = 1, subdim
      nj = ni + inp - 1
      write (channel(6), formats)  ni, E(ni), ML(nj), MS(nj), MJ(nj), L(nj), S(nj), J(nj), MSC(nj), SC(nj), Occ(1:N, nj)
    end do
  end if
!---------------------------------------------------------------------------------------------------------------------------  
  
  deallocate(A)
  deallocate(E)

end do

close(channel(1))
close(channel(2))
close(55)

deallocate (ML)
deallocate (MS)
deallocate (MJ)
deallocate (L)
deallocate (S)
deallocate (J)
deallocate (SC)
deallocate (MSC)
deallocate (OCC)

if (Mode/10 == 1 .or. Mode/10 == 2) return

!-----find chemical potential------------------------------------------------------------------------------------------------
if (mod(Mode,10) == 1) then
  write (channel(6),*) 'Start searching for chemical potential...'
  call cls_chempotn1(fill, chem, temp, N, 0, locc, dim, fcontin, channel, filename)
  write (channel(6),*) 'Chemical potential found:  chem = ', chem
end if


!------find mean filling---------------------------------------------------------------------------------------------------------
if (mod(Mode,10) == 2) then
  write (channel(6),*) 'Start searching for mean filling...'
  call cls_meanfill1(N, dim, chem, temp, fill, fcontin, Locc, channel, filename)
  write (channel(6),*) 'Mean filling found:  Fill = ', fill
end if

!-----calculate and output Green's Function and self-energy---------------------------------------------------------
call cls_AGF1(N, Ns, Nd, Es, Ed, V, dim, phi, invphi, chem, damp, E_grid, E_array, Locc, Channel, &
&filename, Green1, Green2, Self_E, Self_Einf, mor, Mom, Den, DOS, fcontin, fcont, mode)

deallocate (phi)
deallocate (invphi)

end
