subroutine cls_main2(Nd, Ns, Ed, Es, U, V, B1, B2, B3, SOC, E_grid, E_array, Mode, Ecut1, Ecut2, Fill, Dfill, Dran, Chem, Temp, damp, &
                    & Locc, Channel, filename, Green1, Green2, Self_E, Self_Einf, mor, Mom, Den, DOS)

implicit none

!----Mode ( two-digit integer )------------------------------------------------------------------------------------------------------
! First digit controls:
! 1 -- Only calculates Eigenvalues & Eigenvectors.
! 3 -- Returns Eigenvalues, Green's function, Self-energy, Density matrix, DOS.

! Second digit controls:
! 1 -- Fix mean filling, search for chemical potential.
! 2 -- Fix chemical potential, search for mean filling.

!-----All input & output matrices are in Spherical Harmonics representation----------------------------------
integer Nd                              ! Degeneracy of correlated shell.
integer Ns                              ! Degeneracy of uncorrelated shell.
complex*16 Ed(Nd,Nd)                    ! One-eletron energy matrix of correlated shell.
complex*16 Es(Ns,Ns)                    ! One-eletron energy matrix of uncorrelated shell.
real*8 U(Nd/2,Nd/2,Nd/2,Nd/2)           ! Correlation matrix.
complex*16 V(Nd,Ns)                     ! Hybridization matrix.
real*8 B1                               ! Impurity spin magnetic field.
real*8 B2                               ! Orbital magnetic field.
real*8 B3                               ! Bath spin magnetic field.
real*8 SOC                              ! Spin-orbital coupling. 
integer E_grid                          ! Number of energy points in Green's function or self-energy.
complex*16 E_array(E_grid)              ! Energy points array.
integer Mode                            ! Running mode control.
integer maxdim                          ! Maximum dimension allowed for Hamiltonian matrix.
real*8 Ecut1                            ! Energy truncation of Hamiltonian.
real*8 Ecut2                            ! Energy truncation of Hamiltonian.
real*8  Fill                            ! Mean filling.
integer  Dfill                          ! Impurity mean filling.
integer  Dran                           ! Impurity charge fluctuation range.
real*8 Chem                             ! Chemical potential.
real*8 Temp                             ! Temperature.
real*8 damp                             ! Damping parameter.
integer Locc(0:Nd+Ns)                   ! Number of elecrons control array.
integer Channel(6)                      ! I/O channel.
character(len=130) filename(5)          ! Filename string.
complex*16 Green1(Ns+Nd, Ns+Nd, E_grid) ! Return array, the imaginary Green's function matrix.
complex*16 Green2(Ns+Nd, Ns+Nd, E_grid) ! Return array, the imaginary Green's function matrix.
complex*16 Self_E(Ns+Nd, Ns+Nd, E_grid) ! Return array, the self-energy matrix.
complex*16 Self_Einf(Ns+Nd, Ns+Nd)      ! Return array, the self-energy at infinity.
integer Mor                             ! Order of moments.
complex*16 Mom(Ns+Nd, Ns+Nd, -mor:mor)  ! Return moments of Green's function.
complex*16 Den(Ns+Nd, Ns+Nd, -mor:mor)  ! Return array, the density matrix.
real*8 DOS(E_grid)                      ! Return density of states

integer N
integer in
integer ind
integer ins
integer dim1, dim2
integer dimvar
integer dimvarmax
integer dimvarsub
integer precont
integer subdim
integer orgsubdim
integer totdim_imp
integer totdim_bath
integer ndmin, ndmax
integer nsmin, nsmax
integer inmin, inmax
integer i, ni, nj
integer, allocatable :: comb(:)
integer, allocatable :: phi_imp(:)
integer, allocatable :: phi_bath(:)
complex*16, allocatable :: H(:,:)
complex*16, allocatable :: A(:,:)
complex*16, allocatable :: D(:,:)
real*8, allocatable :: E(:)
real*8, allocatable :: Eimp(:)
real*8, allocatable :: Ebath(:)
real*8, allocatable :: Ecls1_tmp(:)
real*8, allocatable :: Ecls2_tmp(:)
real*8, allocatable :: Ecls1(:)
real*8, allocatable :: Ecls2(:)
integer, allocatable :: cin1_tmp(:,:)
integer, allocatable :: cin2_tmp(:,:)
integer, allocatable :: cin1(:,:)
integer, allocatable :: cin2(:,:)
integer, allocatable :: subdim_cls1(:)
integer, allocatable :: subdim_cls2(:)
integer, allocatable :: subdim_imp(:)
integer, allocatable :: subdim_bath(:)
integer, allocatable :: comb_imp(:)
integer, allocatable :: comb_bath(:)
integer, allocatable :: comb_cls(:)
character(len=32) formats
real*8 maxram

real*8, allocatable :: ML(:)          ! Projection of impurity orbital momentum.
real*8, allocatable :: MS(:)          ! Projection of impurity spin momentum.
real*8, allocatable :: MJ(:)          ! Projection of impurity total angular momentum.
real*8, allocatable :: S(:)           ! Total impurity spin momentum.
real*8, allocatable :: L(:)           ! Total impurity orbital momentum.
real*8, allocatable :: J(:)           ! Total impurity angular momentum.
real*8, allocatable :: Occ(:,:)       ! Occupation number of cluster.
real*8, allocatable :: MSC(:)         ! Projection of cluster spin momentum.
real*8, allocatable :: SC(:)          ! Total cluster spin momentum.

real*8, external :: cls_combination
integer, external :: cls_occnum
integer, external :: cls_dnum


!-------------------------------------------------------------------------------------------------------------------------
write (channel(6),*) "Solver Version: QY2"
write (channel(6),*) "Solver start running in mode", Mode
N = Ns + Nd

write (channel(6),*) "Total degeneracy:", N
write (channel(6),*) "Impurity degeneracy:", Nd
write (channel(6),*) "Bath degeneracy:", Ns

if (mod(Mode,10) == 1 .and. nint(Fill) > N) then 
  write(channel(6),*) 'Wrong!! Mean filling can not be larger than total degeneracy!'
  stop
end if

!----diagonalize the impurity Hamiltonian--------------------------------------------------------------------
ndmin = max(dfill - dran, 0)
ndmax = min(dfill + dran, Nd)

allocate (subdim_imp(0:Nd))
allocate (comb_imp(-1:Nd))
subdim_imp = 0
comb_imp = 0
totdim_imp = 0

do ind = ndmin, ndmax
  subdim_imp(ind) = nint(cls_combination(ind, Nd))
  comb_imp(ind) = comb_imp(ind-1) + subdim_imp(ind)
  totdim_imp = totdim_imp + subdim_imp(ind)
end do

allocate (Eimp(totdim_imp))

call cls_imp(Nd, totdim_imp, Ed, U, B1, B2, SOC, ndmin, ndmax, Eimp, channel, filename)

!----compute the bath eigenvalues---------------------------------------------------------------------------
inmax = 0
inmin = N
do in = 0, N
  if (Locc(in) == 0) cycle
  if (in > inmax) inmax = in
  if (in < inmin) inmin = in
end do
nsmax = min(inmax - ndmin, Ns)
nsmin = max(inmin - ndmax, 0)

allocate (subdim_bath(0:Ns))
allocate (comb_bath(-1:Ns))
subdim_bath = 0
comb_bath = 0
totdim_bath = 0

do ins = nsmin, nsmax
  subdim_bath(ins) = nint(cls_combination(ins, Ns))
  comb_bath(ins) = comb_bath(ins-1) + subdim_bath(ins)
  totdim_bath = totdim_bath + subdim_bath(ins)
end do

allocate (Ebath(totdim_bath))

call cls_bath(Ns, totdim_bath, Es, B3, nsmin, nsmax, Ebath, channel, filename)

!-------------------------------------------------------------------------------------------------------------------
allocate (subdim_cls1(0:N))
allocate (subdim_cls2(0:N))
allocate (comb_cls(-1:N))
subdim_cls1 = 0
subdim_cls2 = 0
comb_cls = 0

open (unit=channel(1), file=filename(1)//"index1_tmp.bin", form='UNFORMATTED')
open (unit=channel(2), file=filename(1)//"index2_tmp.bin", form='UNFORMATTED')
open (unit=channel(3), file=filename(1), form='UNFORMATTED')
open (unit=channel(4), file=filename(2), form='UNFORMATTED')

do in = 0, N !----start loop over number of electrons-----------------------------
  if (locc(in) == 0) cycle
  write (channel(6),*) 'Current number of electrons:', in
  
!----select vectors in basis----------------------------------------------------------------------------------------------------------------
  subdim = 0
  do ind = ndmin, ndmax
    if (ind > in .OR. ind < in - Ns) cycle
    ins = in - ind 
    subdim = subdim + subdim_imp(ind) * subdim_bath(ins)
  end do

  allocate (Ecls1_tmp(subdim))
  allocate (Ecls2_tmp(subdim))
  allocate (cin1_tmp(subdim, 6))
  allocate (cin2_tmp(subdim, 6))
  
  
  call cls_Fock2(in, Ns, Nd, subdim, totdim_imp, totdim_bath, Eimp, Ebath, Ecut1, Ecut2, ndmin, ndmax, subdim_imp, subdim_bath, &
               & comb_imp, comb_bath, dim1, dim2, Ecls1_tmp, Ecls2_tmp, cin1_tmp, cin2_tmp)

  subdim_cls1(in) = dim1
  subdim_cls2(in) = dim2
  comb_cls(in) = comb_cls(in-1) + subdim_cls1(in)
  
  write (channel(6),*) "Dimension of Fock space entering diagonalization", dim1
  write (channel(6),*) "Dimension of Fock space assumed diagonal", dim2
  
  maxram = 2 * (real(dim1)**2)*16 / (1024.0**2)
  write (channel(6),*) "Memory needed", maxram, "MB"
  
  allocate (Ecls1(dim1))
  allocate (cin1(dim1, 6))
  Ecls1 = Ecls1_tmp(1:dim1)
  do i = 1, dim1
    cin1(i, 1:6) = cin1_tmp(i, 1:6)
  end do
  
  if (dim2 /= 0) then
    allocate (Ecls2(dim2))
    allocate (cin2(dim2, 6))
    Ecls2 = Ecls2_tmp(1:dim2)
    do i = 1, dim2
      cin2(i, 1:6) = cin2_tmp(i, 1:6)
    end do
  end if
    
  deallocate (Ecls1_tmp)
  deallocate (Ecls2_tmp)
  deallocate (cin1_tmp)
  deallocate (cin2_tmp)
  
  do i = 1, dim1
    write (channel(1)) cin1(i, 1), cin1(i, 2), cin1(i, 3), cin1(i, 4), cin1(i, 5), cin1(i, 6)
  end do
  
  if (dim2 /= 0) then
    do i = 1, dim2
      write (channel(2)) Ecls2(i)
      write (channel(2)) cin2(i, 1), cin2(i, 2), cin2(i, 3), cin2(i, 4), cin2(i, 5), cin2(i, 6)
    end do
    deallocate (Ecls2) 
    deallocate (cin2)
  end if
  
!----calculate block Hamiltonian Matrix---------------------------------------------------------------------------------------
  allocate (H(dim1, dim1))
  call cls_Hamiltonian2(in, Ns, Nd, dim1, Ecls1, cin1, ndmin, ndmax, nsmin, nsmax, V, H, channel, filename)
  deallocate (Ecls1)
  deallocate (cin1)
  
!-----diagonalize the block Hamiltonian Matrix--------------------------------------------------------------------------------
  allocate (A(dim1, dim1))
  allocate (E(dim1))
  call cls_exactdiag(dim1, H, A, E)
  deallocate(H)

!---store eigenvalues & eigenvectors into files-------------------------------------------------------------------------------
  write (channel(6),*) 'Diagonalization finished, saving Eigenvalues and Eigenvectors...'
  do nj = 1, dim1
    write (channel(3)) E(nj)
    do ni = 1, dim1
      write (channel(4)) A(ni,nj)
    end do
  end do
    
  deallocate(A)
  deallocate(E)

end do !----end loop over number of electrons--------------------------------------------------------------------

close(channel(1))
close(channel(2))
close(channel(3))
close(channel(4))

open (unit = channel(1), file = filename(1)//"sdmat.bin", form = "UNFORMATTED")
close (channel(1), status = "DELETE")

if (Mode/10 == 1 .or. Mode/10 == 2) return

!-----find chemical potential---------------------------------------------------------------------------------------------
if (mod(Mode,10) == 1) then
  write (channel(6),*) 'Start searching for chemical potential...'
  call cls_chempotn2(fill, chem, temp, N, 0, locc, subdim_cls1, subdim_cls2, channel, filename)
  write (channel(6),*) 'Chemical potential found: ', chem
end if

!------find mean filling---------------------------------------------------------------------------------------------------
if (mod(Mode,10) == 2) then
  write (channel(6),*) 'Start searching for mean filling...'
  call cls_meanfill2(N, subdim_cls1, subdim_cls2, chem, temp, fill, Locc, channel, filename)
  write (channel(6),*) 'Mean filling found: ', fill
end if

!-----calculate Green's Function and self-energy---------------------------------------------------------------------------
call cls_AGF2(N, Ns, Nd, Es, Ed, V, ndmin, ndmax, nsmin, nsmax, subdim_cls1, subdim_cls2, Chem, damp, E_grid, E_array, &
            & Locc, Channel, filename, Green1, Green2, Self_E, Self_Einf, mor, Mom, Den, DOS, mode)
!--------------------------------------------------------------------------------------------------------------------------
end
