!--routine that creates a particle----------------------------------
subroutine cls_Create(vec, m)
implicit none
integer vec
integer m
vec = ibset(vec,m)
end 

!--routine that destroys a particle-------------------------------
subroutine cls_Destroy(vec, m)
implicit none
integer vec
integer m
vec = ibclr(vec,m)
end 

!--routine that generates comb array-----------------------
subroutine cls_getcomb(comb, N)
implicit none
integer N
integer comb(-1:N)
integer i
real*8,external :: cls_combination
comb = 0
do i = 0, N
  comb(i) = comb(i-1) + nint(cls_combination(i,N))
end do
end

!-------------------------------------------------------------------------
subroutine cls_getcomblocc(comb, N, Locc)
implicit none
integer N
integer comb(0:N)
integer Locc(0:N)
integer i
real*8 fac
real*8,external :: cls_combination
comb = 0
if (Locc(0) /= 0) comb(0) = 1
do i = 1, N
  if (Locc(i) == 0) then
    fac = 0.D0
  else
    fac = 1.D0
  end if
  comb(i) = comb(i-1) + fac*nint(cls_combination(i,N))
end do
end

!---------------------------------------------------------------------------------------------
subroutine cls_getfcnt(N, Ns, Nd, dn1, dn2, locc, fcontin, fcont)
implicit none
integer Ns, Nd, N
integer dn1, dn2
integer locc(0:N)
integer fcontin(0:N)
integer fcont(0:N)
integer ids, idn
integer in
real*8, external :: cls_combination

fcontin = 0
fcont = 0

do in = 0, N
  if (locc(in)==0) cycle
  do idn = dn1, dn2
    if (idn < in-Ns) cycle
    if (idn > in) cycle
    ids = in - idn
    fcontin(in) = fcontin(in) + nint(cls_combination(idn,Nd))*nint(cls_combination(ids,Ns))
  end do
  if (in == 0) then
    fcont(in) = 0
  else
    fcont(in) = fcont(in-1) + fcontin(in)
  end if
end do

end

!-----------------------------------------------------------------------------------------------------------------------------------------------
subroutine cls_convert(MM, MS, A, subdim)
implicit none
integer subdim
real*8 MM(subdim)
real*8 MS(subdim,subdim)
complex*16 A(subdim,subdim)
integer alpha
integer kapa1, kapa2

do alpha = 1, subdim
  do kapa1 = 1, subdim
    do kapa2 = 1, subdim
      MM(alpha) = MM(alpha) + real(conjg(A(kapa1,alpha)) * MS(kapa1,kapa2) * A(kapa2,alpha))
    end do
  end do
end do

end

!---------------------------------------------------------------------------------------------------------------------------
subroutine cls_getsm(SM)
implicit none
real*8 SM(-1:1, -1:1, -1:1)

SM = 0.D0
SM(-1, 1, -1) = 1/sqrt(2.0)
SM(1, -1, 1) = -1/sqrt(2.0)
SM(1, 1, 0) = 5.D-1
SM(-1, -1, 0) = -5.D-1

end

!----------------------------------------------------------------------------------------------------------------------------
real*8 function cls_lm(m1, m2, L, Ld)
implicit none
integer m1, m2
integer L
integer Ld
real*8 lmv
integer, external :: cls_delta

lmv = 0.D0

if (L == 0) then
  lmv = cls_delta(m1, m2) * real(m2)
else if (L == -1) then 
  lmv = cls_delta(m1, m2-1) * 1.D0/sqrt(2.D0) * sqrt(real((Ld + m2)*(Ld - m2 + 1)))
else if (L == 1) then
  lmv = cls_delta(m1, m2+1) * (-1.D0)/sqrt(2.D0) * sqrt(real((Ld - m2)*(Ld + m2 + 1)))
end if

cls_lm = lmv

end

!--------------------------------------------------------------------------------------------------------------------------------------
real*8 function cls_SMAT(m1, m2, m3, m4, sigma1, sigma2, sigma3, sigma4, SM)
implicit none
integer m1, m2, m3, m4
integer sigma1, sigma2, sigma3, sigma4
integer mu, mu_up
real*8 SM(-1:1, -1:1, -1:1)
real*8 sum
integer, external :: cls_delta

sum = 0.D0
do mu = -1, 1
  mu_up = - mu 
  sum = sum + (-1)**mu * SM(sigma1, sigma2, mu_up) * SM(sigma3, sigma4, mu)
end do

cls_SMAT = cls_delta(m1, m2) * cls_delta(m3, m4) * sum

end

!---------------------------------------------------------------------------------------------------------------------------------------
real*8 function cls_LMAT(m1, m2, m3, m4, sigma1, sigma2, sigma3, sigma4, Ld)
implicit none
integer m1, m2, m3, m4
integer sigma1, sigma2, sigma3, sigma4
integer Ld
integer mu, mu_up
real*8 sum
real*8, external :: cls_lm
integer, external :: cls_delta

sum = 0.D0
do mu = -1, 1
  mu_up = - mu 
  sum = sum + (-1)**mu * cls_lm(m1, m2, mu_up, Ld) * cls_lm(m3, m4, mu, Ld)
end do

cls_LMAT = cls_delta(sigma1, sigma2) * cls_delta(sigma3, sigma4) * sum

end

!-----------------------------------------------------------------------------------------------------------------------------------------
real*8 function cls_OMAT(m1, m2, m3, m4, sigma1, sigma2, sigma3, sigma4, Ld, SM)
implicit none
integer m1, m2, m3, m4
integer sigma1, sigma2, sigma3, sigma4
integer Ld
integer mu, mu_up
real*8 SM(-1:1, -1:1, -1:1)
real*8 sum1, sum2
real*8, external :: cls_lm
integer, external :: cls_delta

sum1 = 0.D0
sum2 = 0.D0
do mu = -1, 1
  mu_up = - mu 
  sum1 = sum1 + (-1)**mu * cls_lm(m1, m2, mu_up, Ld) * SM(sigma3, sigma4, mu)
  sum2 = sum2 + (-1)**mu * cls_lm(m3, m4, mu_up, Ld) * SM(sigma1, sigma2, mu)
end do

cls_OMAT = cls_delta(sigma1, sigma2) * cls_delta(m3, m4) * sum1 + cls_delta(sigma3, sigma4) * cls_delta(m1, m2) * sum2

end