	SUBROUTINE DMF_FTBQUP
************************************************************************
*   The calculation of Green's function using low energy
************************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                        !!! from setup
      USE COM_CONTROL ,ONLY: LIFT,LMTO,IDMF                 !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS        !!! from readini
      USE COM_KAPPAR  ,ONLY: NPAN,NKAP                      !!! from readini
      USE COM_KMESH   ,ONLY: NPNT,PNT                       !!! from tetra
      USE COM_WMESH   ,ONLY: NPNW,PNW                       !!! from tetra
      USE COM_MSBMESH ,ONLY: TEMP,NMSC,WMSC                 !!! from readhub
      USE COM_DMFPAR  ,ONLY: NDIT                           !!! from readhub
	USE COM_FERMI   ,ONLY: EF,ZFIL,ZBND,KFRM              !!! from ttrints
      USE COM_CHANNELS,ONLY: IUCEW,IUHAM,IUGRF,IUHYB        !!! from channels
      USE COM_LMTOUT  ,ONLY: ICGRF,ICHYB,ICFAT,ICDOS,ICFRS  !!! from storage
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT,IUI                !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
      DATA PI/3.141592653589793D0/
	IF(IDMF.NE.6)RETURN
C Memory allocations
	CALL DMF_ALLOCATE('A')
C THE DMFT LOOP
	DO 1000 ITER=1,NDIT
	WRITE(IUN,10010)iter
      CALL DMF_MIXDMF(0,ITER,IEXIT,0)       !!! initialize Broyden
C COMPUTE PARAMETERS
	CALL DMF_GRFPAR                       !!! prepare parameters
C COMPUTE POLES OF THE GREENS FUNCTION ON K-MESH
	CALL DMF_SGMMAS(NMSC+1)               !!! Set self-energy to Sigma(0)
	DO IPNT=1,NPNT(1)                     !!! over k-points
	CALL DMF_FTBMAT(IPNT,0,0)             !!! Build Hamiltonian
	CALL DMF_FTBMAS(IPNT,0,0)             !!! Renormalize Overlap matrix
	CALL DMF_GRFBND(IPNT,0,0,1)           !!! solve eigenvalue problem
	CALL DMF_FTBFAT(IPNT,0,0)             !!! store partial characters
      ENDDO                                 !!! over k-points
C Store Data
	CALL DMF_SETFAT                       !!! store fat bands
	CALL DMF_SETDOS                       !!! store density of states
	CALL DMF_SETFRS                       !!! store fermi surface
	IF(LIFT.EQ.2.AND.ICGRF.EQ.0.AND.ICHYB.EQ.0)RETURN
C REORDER COMPLEX EIGENVALUES BY LINKING IMAGINARY PARTS
      CALL DMF_ORDBND(0)
C COMPUTE INTEGRATION WEIGHTS
	CALL DMF_GRFWGT(0)                    !!! find projection weights
	CALL DMF_TTRINT(EF,0,(0.D0,0.D0),TOS,DOS,0,1,0)
C SET IMPURITY LEVELS AND LOCAL GREEN FUNCTION
	CALL DMF_GRFLEV(0)                    !!! find impurity levels
	CALL DMF_IMPLEV(0)                    !!! diagonalize impurty levels
	CALL DMF_GRFLOC(0)                    !!! compute local Green function (direct method)
C SET HYBRIDIZATION FUNCTION
	CALL DMF_IMPHYB                       !!! Compute hybridization function
 	CALL DMF_SETGRF                       !!! Store Green function
 	CALL DMF_SETHYB                       !!! Store Hybridization
	IF(ICGRF.GT.0.OR.ICHYB.GT.0)RETURN
C COMPUTE FERMI ENERGY AND WEIGHTS
	IF(KFRM.EQ.0)
     &CALL DMF_FNDFRM(EF,ZFIL,EF,NMSC+1,100,1.D-05,0.8D0)
	IF(KFRM.EQ.1)
     &CALL DMF_FNDFRM(EF,ZBND,EF,NMSC+1,100,1.D-05,0.8D0)
C COMPUTE INTEGRATION WEIGHTS
	CALL DMF_GRFWGT(0)                    !!! find projection weights
	CALL DMF_TTRINT(EF,0,(0.D0,0.D0),TOS,DOS,0,1,0)
	CALL DMF_HUBMAT(0)                    !!! compute density matrix
C Delete scratch files
	CLOSE(UNIT=IUCEW,STATUS='DELETE')
C MIX HYBRIDIZATIONS
      CALL DMF_MIXDMF(0,ITER,IEXIT,1)       !!! mix hybridizations
	CALL DMF_GRFERR(0,1,ITER)             !!! suppress errors
	IF(IEXIT.EQ.1)GOTO 2000
C FIND NEW SELF-ENERGY WITH IMPURITY SOLVER
      CALL DMF_IMPMOD                       !!! impuity solver
	CALL DMF_GRFERR(0,2,ITER)             !!! suppress errors
C COMPUTE TOTAL ENERGY OF THE MODEL
	CALL DMF_ENERGY                       !!! energies
      CALL DMF_MIXDMF(0,ITER,IEXIT,2)       !!! mix self-energies
	CALL HUB_WRITEHUB(2)                  !!! Store Self-energy
	IF(IEXIT.EQ.1)GOTO 2000
 1000	CONTINUE
C **************************************
C DMFT LOOP DONE, COMPUTE DENSITY MATRIX
 2000	CONTINUE
	CALL DMF_GRFPAR                       !!! prepare parameters
	CALL FTB_NULDEN
	CALL DMF_SGMMAS(NMSC+1)               !!! Set self-energy to Sigma(0)
	DO IPNT=1,NPNT(1)                     !!! over k-points
	CALL DMF_FTBMAT(IPNT,0,0)             !!! Build Hamiltonian
	CALL DMF_FTBMAS(IPNT,0,0)             !!! Renormalize Overlap matrix
	CALL DMF_GRFBND(IPNT,0,0,1)           !!! solve eigenvalue problem
	CALL DMF_ORDVEC(IPNT,0,0)             !!! reorder eigenvectors
	CALL DMF_DENMAT(IPNT,0,0)             !!! compute density matrix
      ENDDO                                 !!! over k-points
	CALL FTB_SYMDEN                       !!! Symmetrize density matrix
	CLOSE(UNIT=IUHAM,STATUS='DELETE')
	CLOSE(UNIT=IUCEW,STATUS='DELETE')
C Memory deallocations
	CALL DMF_ALLOCATE('D')
10010	FORMAT(' Dmft Iteration ',I3/)
	END
