	SUBROUTINE DPT_DELVCFTR(MY,INUC,IQPNT,ISCR)
*********************************************************
* Calculates coulomb part of pseudopotential which is   *
* true Coulomb potential in the interstitial region.    *
* Written by S.Yu.Savrasov (Lebedev Physical Institute) *
*********************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                     !!! from setup
      USE COM_CONTROL, ONLY : LIFT,LMTO,NPFR,MODE                        !!! from readini
	USE COM_PARAM,   ONLY : NSORT,IS,ILST,ILSB,ILSV,NATOM,NORBS,NSPIN  !!! from readini
	USE COM_LATPAR,  ONLY : PAR,ORTH,OMEGAC                            !!! from readini
	USE COM_QMESH,   ONLY : NQPNT,QPNT                                 !!! from readpnt
      USE COM_PLZCOM,  ONLY : NPLZ,LPLZ,IPLZ,NPTB,QPTB,INDPTB            !!! from readlrt
	USE COM_PLWMESH, ONLY : NPLW,GPLW                                  !!! from makeplw
	USE COM_SPHERE,  ONLY : SWZ,SMT                                    !!! from readini
	USE COM_DELRHO,  ONLY : DRHOPS,DRHOPS1                             !!! from delrho
	USE COM_DELPOT,  ONLY : DPOTPS,DPOTPS1                             !!! from delpot
	USE COM_MASHDEP, ONLY : NVAL                                       !!! from setup
      USE COM_UNIT,    ONLY : IUN,IUR,IUT,IUI                            !!! from setup
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 PSTOT
	DATA PI/3.141592653589793D0/
	DO IUPDN=1,NSPIN                     !!! over spins
	DO IPLW=0,NPLW                       !!! over plane waves
C Zeroise
	DO IORBS=1,NORBS
	DPOTPS(IPLW,IUPDN,IORBS)=(0.D0,0.D0)
	ENDDO
C Compute
	GX=(GPLW(1,IPLW)+QPNT(1,IQPNT))*2.D0*PI/PAR
	GY=(GPLW(2,IPLW)+QPNT(2,IQPNT))*2.D0*PI/PAR
	GZ=(GPLW(3,IPLW)+QPNT(3,IQPNT))*2.D0*PI/PAR
	G2=GX**2+GY**2+GZ**2
	PSTOT=(0.D0,0.D0)
	PSTOT=PSTOT+(DRHOPS(IPLW,1,1)+DRHOPS(IPLW,NSPIN,1))/
     &  2.D0*DBLE(NSPIN) 
	PSTOT=PSTOT+(DRHOPS1(IPLW,1,1)+DRHOPS1(IPLW,NSPIN,1))/
     &  2.D0*DBLE(NSPIN)
      PSTOT=PSTOT+(DRHOPS1(IPLW,1,2)+DRHOPS1(IPLW,NSPIN,2))/
     &  2.D0*DBLE(NSPIN)
	IF(G2.GE.1.D-10)DPOTPS(IPLW,IUPDN,1)=8.D0*PI*PSTOT/G2
	IF(G2.LT.1.D-10)DPOTPS(IPLW,IUPDN,1)=(0.D0,0.D0) ! for G=0 and q=0
      DO IPTB=1,NPTB
      IF(ISCR.EQ.1.AND.LPLZ.GT.0.AND.IPLW.EQ.INDPTB(IPTB))
     &DPOTPS(IPLW,IUPDN,1)=(0.D0,0.D0)
      ENDDO      
      ENDDO                                !!! over plane waves
      ENDDO                                !!! over spins
	END



	SUBROUTINE DPT_DELVPINT(MY,INUC,IQPNT)
*********************************************************
* Calculates convolution of pseudopotential with inter- *
* stitial plane-wave integral.                          *
* Written by S.Yu.Savrasov (Lebedev Physical Institute) *
*********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                     !!! from setup
      USE COM_CONTROL, ONLY : LIFT,LMTO,NPFR,MODE                        !!! from readini
	USE COM_PARAM,   ONLY : NSORT,IS,ILST,ILSB,ILSV,NATOM,NORBS,NSPIN  !!! from readini
	USE COM_LATPAR,  ONLY : PAR,ORTH,OMEGAC                            !!! from readini
	USE COM_SPHERE,  ONLY : SWZ,SMT                                    !!! from readini
      USE COM_PLWMESH, ONLY : NPLW,GPLW,KVEC,KGMIN,KGMAX,INDG            !!! from makeplw
      USE COM_PLW2MSH, ONLY : N2PLW,G2PLW,INDG2    	                   !!! from makeplw
	USE COM_POTDAT,  ONLY : POTPS,POTPS1                               !!! from readscf
	USE COM_DELPOT,  ONLY : DPOTPS,DPOTPS1                             !!! from delpot
	USE COM_QMESH,   ONLY : NQPNT,QPNT                                 !!! from readpnt
	USE COM_MASHDEP, ONLY : NVAL                                       !!! from setup
      USE COM_UNIT,    ONLY : IUN,IUR,IUT,IUI                            !!! from setup
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16, ALLOCATABLE ::  GINT(:)
	COMPLEX*16, ALLOCATABLE :: DGINT(:)
	COMPLEX*16 GVAL
	DATA PI/3.141592653589793D0/
	ALLOCATE( GINT(0:9*NPLWMEM))
	ALLOCATE(DGINT(0:9*NPLWMEM))
	M1D=2*KGMAX(1)-2*KGMIN(1)+1
	M2D=2*KGMAX(2)-2*KGMIN(2)+1
	M3D=2*KGMAX(3)-2*KGMIN(3)+1
	M12D=M1D*M2D
	M123D=M1D*M2D*M3D
C  SET CONVOLUTION FOR THE CHANGE IN THE PSEUDOPOTENTIAL
	CALL MAKEPLW_WINT(GINT)              !!! take the integral for PWs
	DO IORBS=1,NORBS                     !!! over iorbs
	DO IUPDN=1,NSPIN                     !!! over spins
	DO IPLW=0,NPLW                       !!! over plane waves
	DPOTPS1(IPLW,IUPDN,IORBS)=(0.D0,0.D0)
	M1I=KVEC(1,IPLW)+2*KGMIN(1)
	M2I=KVEC(2,IPLW)+2*KGMIN(2)
	M3I=KVEC(3,IPLW)+2*KGMIN(3)
	DO JPLW=0,NPLW                       !!! over plane waves
      M1=KVEC(1,JPLW)-M1I
      M2=KVEC(2,JPLW)-M2I
      M3=KVEC(3,JPLW)-M3I
      IDIM=1+M1+M1D*M2+M12D*M3
	I2PLW=INDG2(IDIM)
	DPOTPS1(IPLW,IUPDN,IORBS)=DPOTPS1(IPLW,IUPDN,IORBS)+
     &DPOTPS(JPLW,IUPDN,IORBS)*GINT(I2PLW)
      ENDDO                             !!! over plane waves
      ENDDO                             !!! over plane waves
      ENDDO                             !!! over spins
      ENDDO                             !!! over spins
C  SET CONVOLUTION FOR THE PSEUDOPOTENTIAL AND CHANGE IN THE INTEGRAL
C  THIS WILL AFFECT TO THE TOTAL CHANGE IN THE HAMILTONIAN MATRIX AND
C  DOES NOT AFFECT TO THE RIGID CHANGE WHICH ENTERS TO THE BAND FORCE
	IF(MODE.NE.2)THEN
	CALL INI_MAKEPLW_DINT2(DGINT,QPNT(1,IQPNT),MY,INUC)    !!! take the integral for PWs
	DO IORBS=1,NORBS                  !!! over iorbs
	DO IUPDN=1,NSPIN                  !!! over spins
	DO IPLW=0,NPLW                    !!! over plane waves
	M1I=KVEC(1,IPLW)+2*KGMIN(1)
	M2I=KVEC(2,IPLW)+2*KGMIN(2)
	M3I=KVEC(3,IPLW)+2*KGMIN(3)
	DO JPLW=0,NPLW                    !!! over plane waves
      M1=KVEC(1,JPLW)-M1I
      M2=KVEC(2,JPLW)-M2I
      M3=KVEC(3,JPLW)-M3I
      IDIM=1+M1+M1D*M2+M12D*M3
	I2PLW=INDG2(IDIM)
	DPOTPS1(IPLW,IUPDN,IORBS)=DPOTPS1(IPLW,IUPDN,IORBS)-
     &  POTPS(JPLW,IUPDN,IORBS)*DGINT(I2PLW)
      ENDDO                             !!! over plane waves
      ENDDO                             !!! over plane waves
      ENDDO                             !!! over spins
      ENDDO                             !!! over iorbs
	ENDIF
	DEALLOCATE(GINT,DGINT)
	END
