	SUBROUTINE DRO_DBROY4N(XOUT,XINP,WDIM,DELF,UVEC,NDIM,MTERM
     &               ,ADMIX,WTER,WINP,WMIX,IBRD,BRDFILE,MY,INUC)
***************************************************************
*  Prepares Xinp(m+1), input vector for the (m+1)th iteration *
*  from the knowledge of output, Xout(m), and input, Xinp(m), *
*  vectors at the M-th iteration. Broyden mixing is used as   *
*  described in D.D. Johnson, Phys.Rev.B38,12807(1988),       *
*  equation (15).                                             *
*  Short complex*16 version, which minimizes memory           *
*  requirements by file storage.                              *
*  Parameters:                                                *
*  Xout - output vector at the M-th iteration.                *
*         At the begining, when M=0 Xout must be set          *
*         as Xinp(1). Xout is NOT overwritten!                *
*  Xinp - Xinp(m+1) - input vector at the                     *
*         (M+1) iteration as guessed by Broyden procedure.    *
*  Wdim - weight function for Xout, the value actually        *
*         mixed is Xout*Wdim. For example Wdim(i)=r(i)^2      *
*         if X(i)=Rho(i) to suppress core region.             *
*         Just set Wdim(i)=1.0 for simplicity.                *
*  DelF,Uvec - work arrays                                    *
*  Ndim - dimension of Xout,Xinp                              *
*  Mter - Iteration number, i.e. M                            *
*  Admix - Initial mixing                                     *
*  Wter - weight of the iteration, will be found.             *
*  Winp - see comments below, in BROMIX subroutine            *
*  Wmix - see comments below, in BROMIX subroutine            *
*  Ibrd, BRDfile - channel # and the filename for             *
*         storage the data on previous iterations             *
*  Last modified 30.10.96                                     *
*  Adapted to the phonon program: 06.11.96                    *
***************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                         !!! from setup
	USE COM_MASHDEP, ONLY : NVAL                                       !!! from setup
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16, ALLOCATABLE :: WORK(:)
	COMPLEX*16, ALLOCATABLE :: BETA(:,:)
	REAL*8, ALLOCATABLE :: FNOM(:,:,:)
	INTEGER MTERM(-1:1,NATOMMEM)
	COMPLEX*16 XINP(NLENMEM),XOUT(NLENMEM),UVEC(NLENMEM)
     &   ,WMIX(NTERMEM,NTERMEM,-1:1,NATOMMEM)
     &   ,CMAT(NTERMEM,NTERMEM,-1:1,NATOMMEM)
     &   ,GAMA(NTERMEM,NTERMEM,-1:1,NATOMMEM)
     &   ,WINP(NTERMEM,-1:1,NATOMMEM),DELF(NLENMEM)
	DIMENSION WTER(0:NTERMEM,-1:1,NATOMMEM),WDIM(NLENMEM)
     &   ,ADMIX(-1:1,NATOMMEM)
	CHARACTER BRDFILE(-1:1,NATOMMEM)*130
	LOGICAL EXIST,OPEN,FIRST
	SAVE
	DATA FIRST/.TRUE./,W0/0.01D0/ !!! Inverse Jakobian weight 
C ITERATION
	MTER=MTERM(MY,INUC)
	IF(FIRST)THEN
	ALLOCATE(WORK(NTERMEM))
	ALLOCATE(FNOM(NTERMEM,-1:1,NATOMMEM))
	ALLOCATE(BETA(NTERMEM,NTERMEM))
	FIRST=.FALSE.
	ENDIF
C **************************************************
C FILE OPERATIONS
      NRECL=NVAL*NDIM
      INQUIRE(FILE=BRDFILE(MY,INUC),EXIST=EXIST,OPENED=OPEN)
      IF(EXIST.AND.(.NOT.OPEN))OPEN(UNIT=IBRD,FILE=
     &  BRDFILE(MY,INUC),STATUS='OLD',FORM='UNFORMATTED',
     &  ACCESS='DIRECT',RECL=NRECL)
      IF(EXIST.AND.MTER.EQ.0)
     &  CLOSE(UNIT=IBRD,STATUS='DELETE')
      IF(.NOT.EXIST.OR.(EXIST.AND.MTER.EQ.0))
     &  OPEN(UNIT=IBRD,FILE=BRDFILE(MY,INUC),STATUS='NEW',
     &  FORM='UNFORMATTED',ACCESS='DIRECT',RECL=NRECL)
C **************************************************
C DATA OPERATIONS
C INITIALIZE BROYDEN If M=0, Xout - input vector to the 1th iteration.
	IF(MTER.EQ.0)THEN
	DO IDIM=1,NDIM
	XINP(IDIM)=XOUT(IDIM)
	ENDDO
	WRITE(IBRD,REC=1)(XINP(IDIM),IDIM=1,NDIM)
	CLOSE(UNIT=IBRD)
	RETURN
C *** IF ITERATION=1, USE STANDARD MIXING
   	ELSEIF(MTER.EQ.1)THEN
	READ(IBRD,REC=1)(XINP(IDIM),IDIM=1,NDIM)
C Set weights
	WTER(0,MY,INUC)=0.D0
	DO IDIM=1,NDIM
	WTER(0,MY,INUC)=WTER(0,MY,INUC)+
     &  CONJG(XOUT(IDIM)-XINP(IDIM))*WDIM(IDIM)*
     &       (XOUT(IDIM)-XINP(IDIM))*WDIM(IDIM)
	ENDDO
	WTER(0,MY,INUC)=1.D0/SQRT(WTER(0,MY,INUC))
	WTER(1,MY,INUC)=1.D0
C Set arrays
	IREC=1+2*(MTER+0)
	WRITE(IBRD,REC=IREC)(-XINP(IDIM),IDIM=1,NDIM)!! part of U(m)
	DO IDIM=1,NDIM
	UVEC(IDIM)=XOUT(IDIM)-XINP(IDIM)            !!! F(1)
	XINP(IDIM)=XINP(IDIM)+ADMIX(MY,INUC)*
     &            (XOUT(IDIM)-XINP(IDIM))           !!! Xinp(m+1)
	ENDDO
	WRITE(IBRD,REC=1)(XINP(IDIM),IDIM=1,NDIM)   !!! keep Xinp(m+1)
	WRITE(IBRD,REC=2)(UVEC(IDIM),IDIM=1,NDIM)   !!! keep dF(1)
C *** IF ITERATION>1, USE BROYDEN MIXING
   	ELSEIF(MTER.GT.1)THEN
	READ(IBRD,REC=1)(XINP(IDIM),IDIM=1,NDIM)
C Set weights
	WTER(MTER,MY,INUC)=0.D0
	DO IDIM=1,NDIM
	WTER(MTER,MY,INUC)=WTER(MTER,MY,INUC)+
     &  CONJG(XOUT(IDIM)-XINP(IDIM))*WDIM(IDIM)*
     &       (XOUT(IDIM)-XINP(IDIM))*WDIM(IDIM)
	ENDDO
	WTER(MTER,MY,INUC)=1.D0/SQRT(WTER(MTER,MY,INUC))
	WTER(MTER,MY,INUC)=
     &  WTER(MTER,MY,INUC)/WTER(0,MY,INUC)
C Set norms
	READ(IBRD,REC=2)(UVEC(IDIM),IDIM=1,NDIM)   !!! read dF
	FNOM(MTER-1,MY,INUC)=0.D0
	DO IDIM=1,NDIM
	FNOM(MTER-1,MY,INUC)=
     &  FNOM(MTER-1,MY,INUC)+
     &  CONJG(XOUT(IDIM)-XINP(IDIM)-UVEC(IDIM))*WDIM(IDIM)*
     &       (XOUT(IDIM)-XINP(IDIM)-UVEC(IDIM))*WDIM(IDIM)
	ENDDO
	FNOM(MTER-1,MY,INUC)=SQRT(FNOM(MTER-1,MY,INUC))! |F(m)-F(m-1)|
	DO IDIM=1,NDIM
	DELF(IDIM)=(XOUT(IDIM)-XINP(IDIM)-UVEC(IDIM))/
     &  FNOM(MTER-1,MY,INUC)                       !!! dF(m-1)
	ENDDO
	IREC=2+2*(MTER-1)
	WRITE(IBRD,REC=IREC)(DELF(IDIM),IDIM=1,NDIM)
C Set Uvec(m-1)
	IREC=1+2*(MTER-1)
	READ(IBRD,REC=IREC)(UVEC(IDIM),IDIM=1,NDIM)
	DO IDIM=1,NDIM
	UVEC(IDIM)=UVEC(IDIM)/FNOM(MTER-1,MY,INUC)+
     &  ADMIX(MY,INUC)*DELF(IDIM)+XINP(IDIM)/
     &  FNOM(MTER-1,MY,INUC)                       !!! U(m-1)
	ENDDO
	IREC=1+2*(MTER-1)
	WRITE(IBRD,REC=IREC)( UVEC(IDIM),IDIM=1,NDIM)!! U(m-1)
	IREC=1+2*(MTER+0)
	WRITE(IBRD,REC=IREC)(-XINP(IDIM),IDIM=1,NDIM)!! part of U(m)
C Set matrix C(i,m-1)
	DO ITER=1,MTER-1	
	CMAT(ITER,MTER,MY,INUC)=(0.D0,0.D0)
	IREC=2+2*(ITER+0)
	READ(IBRD,REC=IREC)(DELF(IDIM),IDIM=1,NDIM)
	DO IDIM=1,NDIM
	CMAT(ITER,MTER,MY,INUC)=CMAT(ITER,MTER,MY,INUC)+
     &  WTER(ITER,MY,INUC)*WDIM(IDIM)*
     &  CONJG(DELF(IDIM))*(XOUT(IDIM)-XINP(IDIM))*WDIM(IDIM)
	ENDDO
	ENDDO
C Set matrix Beta^{-1}(i,j)
	DO ITER=1,MTER-1
	IREC=2+2*(ITER+0)
	READ(IBRD,REC=IREC)(DELF(IDIM),IDIM=1,NDIM) !!! dF(i)
	DO JTER=1,MTER-1
	IREC=2+2*(JTER+0)
	READ(IBRD,REC=IREC)(UVEC(IDIM),IDIM=1,NDIM) !!! dF(j) 
	BETA(ITER,JTER)=(0.D0,0.D0)
	DO IDIM=1,NDIM
	BETA(ITER,JTER)=BETA(ITER,JTER)+WDIM(IDIM)*WDIM(IDIM)*
     &  WTER(JTER,MY,INUC)*CONJG(UVEC(IDIM))*WTER(ITER,MY,INUC)*
     &  DELF(IDIM)
	ENDDO
	IF(ITER.EQ.JTER)BETA(ITER,JTER)=BETA(ITER,JTER)+W0**2
	ENDDO
	ENDDO
      CALL LIB_CINV(MTER-1,BETA,NTERMEM,WORK,IFAIL)
      IF(IFAIL.NE.0)
     &CALL FULLSTOP('<BROYDEN>:: ERROR IN INVERSION')
C Set Gamma(m,i)
	DO ITER=1,MTER-1
	GAMA(MTER,ITER,MY,INUC)=0.D0
	DO KTER=1,MTER-1
	GAMA(MTER,ITER,MY,INUC)=GAMA(MTER,ITER,MY,INUC)+
     &  CMAT(KTER,MTER,MY,INUC)*BETA(KTER,ITER)
	ENDDO
	ENDDO
C Set finally X(m+1)
	DO IDIM=1,NDIM
	UVEC(IDIM)=XOUT(IDIM)-XINP(IDIM)
	XINP(IDIM)=XINP(IDIM)+ADMIX(MY,INUC)*
     &            (XOUT(IDIM)-XINP(IDIM))
	ENDDO
	WRITE(IBRD,REC=2)(UVEC(IDIM),IDIM=1,NDIM)
	DO ITER=1,MTER-1
	IREC=1+2*(ITER+0)
	READ(IBRD,REC=IREC)(UVEC(IDIM),IDIM=1,NDIM)
	DO IDIM=1,NDIM
	XINP(IDIM)=XINP(IDIM)-WTER(ITER,MY,INUC)*
     &  GAMA(MTER,ITER,MY,INUC)*UVEC(IDIM)
	ENDDO
	ENDDO
C Vector Xinp is an input to the next (m+1)th iteration. Save it here.
	WRITE(IBRD,REC=1)(XINP(IDIM),IDIM=1,NDIM)
	ENDIF
C  Get mixing array WMIX
	CALL DRO_BROMIX(MTER,ADMIX(MY,INUC),WINP(1,MY,INUC)
     &        ,WMIX(1,1,MY,INUC),GAMA(1,1,MY,INUC)
     &        ,FNOM(1,MY,INUC),WTER(0,MY,INUC))
	CLOSE(UNIT=IBRD)
	END



	SUBROUTINE DRO_BROMIX(MTER,ADMIX,WINP,WMIX,GAMA,FNOM,WTER)
******************************************************************
*  Represents Xinp(m+1) as a combination of Xinp(1) and          *
*  all Xout(i) as follows:      m                                *
*                              ---                               *
*    Xinp(m+1)=Winp(m)*Xinp(1)+>  Wmix(m,i)*Xout(i)              *
*                              ---                               *
*  Output: WINP,WMIX           i=1                               * 
*  This form shows an obvious generalization over the simple     *
*  mixing scheme.                                                *
*  Last modified: 25.10.96                                       *
******************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                         !!! from setup
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16, ALLOCATABLE :: XINP(:)
	COMPLEX*16, ALLOCATABLE :: XOUT(:)
	COMPLEX*16, ALLOCATABLE :: DELF(:)
	COMPLEX*16 WINP(NTERMEM),WMIX(NTERMEM,NTERMEM),X
     &   ,GAMA(NTERMEM,NTERMEM),UVEC(NTERMEM)
	DIMENSION WTER(0:NTERMEM),FNOM(NTERMEM)
	LOGICAL FIRST
	SAVE
	DATA FIRST/.TRUE./
	IF(FIRST)THEN
	ALLOCATE(XINP(NTERMEM))
	ALLOCATE(XOUT(NTERMEM))
	ALLOCATE(DELF(NTERMEM))
	FIRST=.FALSE.
	ENDIF
C SET MIXING PARAMETERS FOR XOUT(I) ------>
	XINP(1)=(0.D0,0.D0)
	DO KTER=1,MTER
	WMIX(MTER,KTER)=(0.D0,0.D0)
	DO LTER=1,MTER
	IF(LTER.EQ.KTER)XOUT(LTER)=(1.D0,0.D0)
	IF(LTER.NE.KTER)XOUT(LTER)=(0.D0,0.D0)
	ENDDO
	DO LTER=1,MTER
C *** IF ITERATION=1, USE STANDARD MIXING
   	IF(LTER.EQ.1)THEN
	X=XINP(LTER)+ADMIX*(XOUT(LTER)-XINP(LTER))
C *** IF ITERATION>1, USE BROYDEN MIXING
   	ELSEIF(LTER.GT.1)THEN
	DELF(LTER-1)=(XOUT(LTER)-XINP(LTER)-
     &  XOUT(LTER-1)+XINP(LTER-1))/FNOM(LTER-1)
C Set Uvec(l-1)
	UVEC(LTER-1)=ADMIX*DELF(LTER-1)+
     &  (XINP(LTER)-XINP(LTER-1))/FNOM(LTER-1) 
C Set finally X(l+1)
	X=XINP(LTER)+ADMIX*(XOUT(LTER)-XINP(LTER))
	DO ITER=1,LTER-1
	X=X-WTER(ITER)*GAMA(LTER,ITER)*UVEC(ITER)
	ENDDO
	ENDIF
C Vector X is an input to the next (m+1)th iteration.
	XINP(LTER+1)=X
	ENDDO
	WMIX(MTER,KTER)=X
	ENDDO
C SET MIXING PARAMETERS FOR XINP(1) ------>
	XINP(1)=(1.D0,0.D0)
	WINP(MTER)=(0.D0,0.D0)
	DO ITER=1,MTER
	XOUT(ITER)=(0.D0,0.D0)
	ENDDO
	DO LTER=1,MTER
C *** IF ITERATION=1, USE STANDARD MIXING
    	IF(LTER.EQ.1)THEN
	X=XINP(LTER)+ADMIX*(XOUT(LTER)-XINP(LTER))
C *** IF ITERATION>1, USE BROYDEN MIXING
   	ELSEIF(LTER.GT.1)THEN
	DELF(LTER-1)=(XOUT(LTER)-XINP(LTER)-
     &  XOUT(LTER-1)+XINP(LTER-1))/FNOM(LTER-1)
C Set Uvec(m-1)
	UVEC(LTER-1)=ADMIX*DELF(LTER-1)+
     &  (XINP(LTER)-XINP(LTER-1))/FNOM(LTER-1) 
C Set finally X(m+1)
	X=XINP(LTER)+ADMIX*(XOUT(LTER)-XINP(LTER))
	DO ITER=1,LTER-1
	X=X-WTER(ITER)*GAMA(LTER,ITER)*UVEC(ITER)
	ENDDO
	ENDIF
C Vector X is an input to the next (m+1)th iteration. Save it here.
	XINP(LTER+1)=X
	ENDDO
	WINP(MTER)=X
	END
