	SUBROUTINE HOP_ALPCON
******************************************************************* 
*  Calculates screened constants alpha using logarithimic deriv.  *
*******************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL,ONLY : LMTO                             !!! from readini
      USE COM_PARAM,  ONLY : NATOM,NSORT,NSPIN,IS,ILST,ILSB   !!! from readini
	USE COM_LATPAR, ONLY : PAR,ORTH,OMEGAC                  !!! from readini
	USE COM_KAPPAR, ONLY : NPAN,NKAP,EKAP,EK                !!! from readini
	USE COM_SPHERE, ONLY : SWZ,SMT,SASA                     !!! from readini
	USE COM_LATTICE,ONLY : RBAS,GBAS,TAU                    !!! from readstr
	USE COM_SCRINP, ONLY : RLOC,ALPHA,ALPDOT,DTBLOG         !!! from alpcon
	USE COM_UNIT,   ONLY : IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16, ALLOCATABLE :: HAN(:,:)
	COMPLEX*16, ALLOCATABLE :: BES(:,:)
	COMPLEX*16, ALLOCATABLE :: HANDOT(:,:)
	COMPLEX*16, ALLOCATABLE :: BESDOT(:,:)
	COMPLEX*16, ALLOCATABLE :: DDOT(:)
	DIMENSION DDAT(0:3,0:3),DDER(0:3,0:3)
	COMPLEX*16 EWZ,BN,BD,HN,HD
	LOGICAL FIRST
	DATA PI/3.141592653589793D0/,FIRST/.TRUE./
	DATA DDAT/0.75D0,0.00D0,0.00D0,0.00D0   !!! screening Dlog for s-el.
     &           ,1.35D0,1.55D0,0.00D0,0.00D0   !!! screening Dlog for sp-el.
     &           ,2.30D0,2.40D0,2.60D0,0.00D0   !!! screening Dlog for spd-el.
     &           ,3.35D0,3.35D0,3.45D0,3.65D0/  !!! screening Dlog for spdf-el.
	DATA DDER/0.10D0,0.00D0,0.00D0,0.00D0   !!! screening Ddot for s-el.
     &           ,0.10D0,0.10D0,0.00D0,0.00D0   !!! screening Ddot for sp-el.
     &           ,0.10D0,0.10D0,0.10D0,0.00D0   !!! screening Ddot for spd-el.
     &           ,0.10D0,0.10D0,0.10D0,0.10D0/  !!! screening Ddot for spdf-el.
	DATA ELIM/-1.3D0/                       !!! energy limit for screening
        CALL ABORTNOW()
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(ALPHA(0:LMAXTMEM+1,0:NSORTMEM,NKAPMEM,NPANMEM))
	ALLOCATE(ALPDOT(0:LMAXTMEM+1,0:NSORTMEM,NKAPMEM,NPANMEM))
	ALLOCATE(DTBLOG(0:LMAXTMEM+1,0:NSORTMEM,NKAPMEM,NPANMEM))
C MEMORY ALLOCATIONS FOR SCRATCH ARRAYS
	ALLOCATE(HAN(3,-1:LMAXBMEM))
	ALLOCATE(BES(3,0:LMAXBMEM+1))
	ALLOCATE(HANDOT(3,0:LMAXBMEM))
	ALLOCATE(BESDOT(3,0:LMAXBMEM))
	ALLOCATE(DDOT(0:LMAXBMEM))
C ****  DO GLOBALLY OVER PANELS, ENERGIES AND ATOMS ****
	DO 1300 IPAN=1,NPAN                       !!! over panels
	DO 1200 IKAP=1,NKAP(IPAN)                 !!! over energies
	DO 1100 ISORT=1,NSORT                     !!! over atoms
	ST=SASA(ISORT)/PAR
	SM=SMT (ISORT)/PAR
	EWZ=EK(IKAP,IPAN)*ST**2
	IF(LMTO.NE.1.AND.DREAL(EKAP(IKAP,IPAN)).GT.ELIM)THEN
C  USE UNIVERSAL FIT FOUND BY POSTNIKOV
	ISC=ILSB(ISORT)
	IF(ISC.GT.3)CALL ALPCON_MESG01
	DO L=0,ILSB(ISORT)
	DTBLOG(L,ISORT,IKAP,IPAN)=DDAT(L,ISC)-EWZ*DDER(L,ISC)
	DDOT(L)=-DDER(L,ISC)*ST**2
	ENDDO
C  CALCULATE BESSEL AND HANKEL FUNCTIONS AT SCREENING RADIUS (ATOMIC SPHERE)
	CALL LIB_BESSEL(ST,EK(IKAP,IPAN),SM,BES(1,0),ILSB(ISORT)+1)
	CALL LIB_HANKEL(ST,EK(IKAP,IPAN),SM,HAN(1,0),ILSB(ISORT))
	IF(DREAL(EK(IKAP,IPAN)).LT.0.D0)THEN
	HAN(1,-1)=-(0.D0,1.D0)/SQRT(EK(IKAP,IPAN))/SM*HAN(1,0)
	HAN(2,-1)=-(0.D0,1.D0)/SQRT(EK(IKAP,IPAN))/SM*HAN(2,0)
	ELSEIF(ABS(EK(IKAP,IPAN)).LE.1.D-10)THEN
	HAN(1,-1)=(1.D0,0.D0)
	HAN(2,-1)=(0.D0,0.D0)
	ELSEIF(DREAL(EK(IKAP,IPAN)).GT.0.D0)THEN
	HAN(1,-1)=BES(1,0)
	HAN(2,-1)=BES(2,0)
	ENDIF
C  SET ENERGY DERIVATIVES OF THE BESSEL AND HANKEL FUNCTIONS
	DO L=0,ILSB(ISORT)
	BESDOT(1,L)=-0.5D0*ST*SM/(2.D0*L+3)*BES(1,L+1)
	BESDOT(2,L)=-0.5D0*SM/(2.D0*L+3)*(BES(1,L+1)+ST*BES(2,L+1))
	HANDOT(1,L)=0.5D0*ST*SM/(2.D0*L-1.D0)*HAN(1,L-1)
	HANDOT(2,L)=0.5D0*SM/(2.D0*L-1.D0)*(HAN(1,L-1)+ST*HAN(2,L-1))
	ENDDO
C  FIND SCREENING CONSTANTS ALPHA AND THEIR ENERGY DERIVATIVES
	DO L=0,ILSB(ISORT)
	GAM=1.D0/(SMT(ISORT)/PAR*DBLE(2*L+1))
	BN=DTBLOG(L,ISORT,IKAP,IPAN)*BES(1,L)-SASA(ISORT)/PAR*BES(2,L)
	HN=DTBLOG(L,ISORT,IKAP,IPAN)*HAN(1,L)-SASA(ISORT)/PAR*HAN(2,L)
	ALPHA(L,ISORT,IKAP,IPAN)=BN/HN*GAM
	BD=DDOT(L)*BES(1,L)+DTBLOG(L,ISORT,IKAP,IPAN)*BESDOT(1,L)-
     &  SASA(ISORT)/PAR*BESDOT(2,L)
	HD=DDOT(L)*HAN(1,L)+DTBLOG(L,ISORT,IKAP,IPAN)*HANDOT(1,L)-
     &  SASA(ISORT)/PAR*HANDOT(2,L)
   	ALPDOT(L,ISORT,IKAP,IPAN)=(BD*HN-BN*HD)/(HN*HN)*GAM
C	ALPDOT(L,ISORT,IKAP,IPAN)=(0.D0,0.D0)
	ENDDO
	DO L=ILSB(ISORT)+1,ILST(ISORT)+1
	ALPHA(L,ISORT,IKAP,IPAN)=(0.D0,0.D0)
	ALPDOT(L,ISORT,IKAP,IPAN)=(0.D0,0.D0)
	DTBLOG(L,ISORT,IKAP,IPAN)=(0.D0,0.D0)
	ENDDO
C  NO SCREENING FOR NEGATIVE ENERGIES OR FOR BARE LMTO
	ELSE
	DO L=0,ILST(ISORT)+1
	ALPHA(L,ISORT,IKAP,IPAN)=(0.D0,0.D0)
	ALPDOT(L,ISORT,IKAP,IPAN)=(0.D0,0.D0)
	DTBLOG(L,ISORT,IKAP,IPAN)=(0.D0,0.D0)
	ENDDO
	ENDIF
1100    CONTINUE                                 !!! over atoms
1200    CONTINUE                                 !!! over energies
1300    CONTINUE                                 !!! over panels
C MEMORY DEALLOCATIONS
	DEALLOCATE(HAN)
	DEALLOCATE(BES)
	DEALLOCATE(HANDOT)
	DEALLOCATE(BESDOT)
	DEALLOCATE(DDOT)
	CONTAINS
C ERROR HANDLINGS
	SUBROUTINE ALPCON_MESG01
	WRITE(IUT,*)'Terminated in <ALPCON>:'
	WRITE(IUT,*)'Screening parameters ALPHA are not currently'
	WRITE(IUT,*)'avaiable for l=4 (g-electrons) or more.'
	WRITE(IUT,*)'Decrease LmaxB to 3 or less in the INIFILE.'
	CALL FULLSTOP(' ')
	END SUBROUTINE ALPCON_MESG01
	END SUBROUTINE HOP_ALPCON
