	SUBROUTINE HUB_HUBPOT
**********************************************************
*  Computes Hubbard correction to the LDA potential in   *
*  the LDA+U scheme.                                     *
*  The important parameter is KDBLCNT(L,IATOM)           *
*  in MODULE COM_HUBDAT. See formulae below for a        *
*  particular scheme.                                    *
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL,ONLY: IDMF                                      !!! from readini
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS,ILSB           !!! from readini
      USE COM_PANPAR ,ONLY: LTPAN,LSPAN                               !!! from readini
      USE COM_HUBDAT ,ONLY: NHUB,MCRL,LCRL,
     &                      KSOLVER,KDBLCNT,CDBLCNT,KSIGMOO           !!! from readhub
      USE COM_HUBCOM ,ONLY: DHUB,VHUB,VCNS,VDBC                       !!! from readhub
	USE COM_IMPDAT ,ONLY: AMAGIMP,BSPLIMP,ISPLIMP                   !!! from readhub
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                               !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: UMAT(:,:)
	REAL*8, ALLOCATABLE :: GMAT(:,:)
	REAL*8, ALLOCATABLE :: UBAR(:,:)
	REAL*8, ALLOCATABLE :: GBAR(:,:)
	COMPLEX*16, ALLOCATABLE :: DBAR(:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: DAVR(:,:)
	COMPLEX*16 AMAG(-1:1)
	DATA PI/3.141592653589793D0/
      CALL ABORTNOW()
	IF(IDMF.NE.1.AND.IDMF.NE.2)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(UMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(GMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(UBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(GBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(DBAR(0:LMAXBMEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DAVR(0:LMAXBMEM,NATOMMEM))
C DO GLOBALLY OVER ATOMS
	DO 1000 IATOM=1,NATOM                     !!! over atoms
	ISORT=IS(IATOM)
	CALL LIB_HUBMAT_L2(IATOM,UMAT,GMAT,UBAR,GBAR) ! set Hubbard and Exchange matrices
C  SET AVERAGE OCCUPANCIES <Nup> and <Ndn>
	DO IORBS=1,NORBS                          !!! over iorbs
	DO IUPDN=1,NSPIN                          !!! over spins
	DO L=0,ILSB(ISORT)                        !!! over l
	DBAR(L,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	DO M=-L,L                                 !!! over m
	LM=L*(L+1)+M+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M+1           !  (l,m m) packed
	DBAR(L,IATOM,IUPDN,IORBS)=DBAR(L,IATOM,IUPDN,IORBS)+
     &DHUB(IU,IATOM,IUPDN,IORBS)
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over iorbs
C  SET MEAN NUMBER OF ELECTRONS
	DO L=0,ILSB(ISORT)                        !!! over l
	DAVR(L,IATOM)=(0.D0,0.D0)
	DO IUPDN=1,NSPIN                          !!! over spins
	DAVR(L,IATOM)=DAVR(L,IATOM)+DBAR(L,IATOM,IUPDN,1)
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over l
C  SET HUBBARD CORRECTION TO THE POTENTIAL
	DO IUPDN=1,NSPIN                          !!! over spins
	IF(IUPDN.EQ.1)IUPDN1=NSPIN
	IF(IUPDN.EQ.NSPIN)IUPDN1=1
	DO L=0,ILSB(ISORT)                        !!! over l
C SET LDAU POTENTIAL AND DOUBLE COUNTING
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	VHUB(IU,IATOM,IUPDN,    1)=(0.D0,0.D0)
	VHUB(IU,IATOM,IUPDN,NORBS)=(0.D0,0.D0)
	VDBC(IU,IATOM,IUPDN,    1)=(0.D0,0.D0)
	VDBC(IU,IATOM,IUPDN,NORBS)=(0.D0,0.D0)
	DO M1=-L,L                                !!! over m1
	DO K1=-L,L                                !!! over k1
	LM1=L*(L+1)+M1+1
	LK1=L*(L+1)+K1+1
	IU1=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K1)+L+M1+1  !  (l,m1 k1) packed
	IF(KSIGMOO(L,IATOM).EQ.11.OR.KSIGMOO(L,IATOM).EQ.12)
C  Hartree-Fock contribution (Rotationally Invariant, Liechtenstein et.al)
C  dVup=U*Ndn+(U-J)Nup-d*[<U>(<N>-1/2)+<J>(<Nup>-1/2)]
C  dVdn=U*Nup+(U-J)Ndn-d*[<U>(<N>-1/2)+<J>(<Ndn>-1/2)]
     &VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)+
     & UMAT(IU,IU1)*DHUB(IU1,IATOM,IUPDN1,1)/2.D0*DBLE(NSPIN)+
     &(UMAT(IU,IU1)-GMAT(IU,IU1))
     &             *DHUB(IU1,IATOM,IUPDN ,1)/2.D0*DBLE(NSPIN)
C ADD RELATIVISTIC CORRECTIONS
C <<<Note that VHUB here is already matrix element <B^{+/-}*S_{+/-}>/sqrt(2)
C    not just a field <B^{+/-}, additional 0.5 is due to various sqrt(2) introduced.>>>
C Note on LmtART 6.60 or later -> 0.5 factor moved to HUBPAR, now VHUB can be properly rotated
C by Wigner matrices
	IF((KSIGMOO(L,IATOM).EQ.11.OR.KSIGMOO(L,IATOM).EQ.12).AND.NORBS.EQ.2)
     &VHUB(IU,IATOM,IUPDN,NORBS)=VHUB(IU,IATOM,IUPDN,NORBS)-
     &GMAT(IU,IU1)*DHUB(IU1,IATOM,IUPDN1,NORBS)
	ENDDO                                     !!! over k1
	ENDDO                                     !!! over m1
C  Spherically averaged Hartree-Fock contribution (also rotationally invariant)
C  dVup=U*Ndn+(U-J)Nup-d*[<U>(<N>-1/2)+<J>(<Nup>-1/2)]
C  dVdn=U*Nup+(U-J)Ndn-d*[<U>(<N>-1/2)+<J>(<Ndn>-1/2)]
	IF(K.EQ.M.AND.(KSIGMOO(L,IATOM).EQ.21.OR.KSIGMOO(L,IATOM).EQ.22))
     &VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)+
     &UBAR(L,L)*DBAR(L,IATOM,IUPDN1,1)/2.D0*DBLE(NSPIN)+
     &(UBAR(L,L)-GBAR(L,L))
     &         *DBAR(L,IATOM,IUPDN ,1)/2.D0*DBLE(NSPIN)-
     &(UBAR(L,L)-GBAR(L,L))
     &         *DBAR(L,IATOM,IUPDN,1)/2.D0*DBLE(NSPIN)
C SET DOUBLE COUNTING TERM
C Case of Spin Dependent Fully Localized limit
      IF(K.EQ.M.AND.
     &(KDBLCNT(L,IATOM).EQ.11.OR.KDBLCNT(L,IATOM).EQ.13))  !!! spin-up/down
     &VDBC(IU,IATOM,IUPDN,1)=VDBC(IU,IATOM,IUPDN,1)+
     &UBAR(L,L)*DBAR(L,IATOM,IUPDN1,1)/2.D0*DBLE(NSPIN)+
     &(UBAR(L,L)-GBAR(L,L))
     &         *DBAR(L,IATOM,IUPDN ,1)/2.D0*DBLE(NSPIN)-
     &(UBAR(L,L)-GBAR(L,L))*CDBLCNT(L,IATOM)
C Note on LmtART 6.60 or later -> 0.5 factor moved to HUBPAR
	IF(K.EQ.M.AND.NORBS.EQ.2.AND.
     &(KDBLCNT(L,IATOM).EQ.11.OR.KDBLCNT(L,IATOM).EQ.13))  !!! off-diagonal
     &VDBC(IU,IATOM,IUPDN,NORBS)=VDBC(IU,IATOM,IUPDN,NORBS)-
     &GBAR(L,L)*DBAR(L,IATOM,IUPDN1,NORBS)
C Case of Spin Averaged Fully Localized limit
      IF(K.EQ.M.AND.                                       !!! spin-up/down
     &(KDBLCNT(L,IATOM).EQ.12.OR.KDBLCNT(L,IATOM).EQ.14))  !!! spin-up/down
     &VDBC(IU,IATOM,IUPDN,1)=VDBC(IU,IATOM,IUPDN,1)+
     &UBAR(L,L)*DAVR(L,IATOM)/2.D0+
     &(UBAR(L,L)-GBAR(L,L))
     &         *DAVR(L,IATOM)/2.D0-
     &(UBAR(L,L)-GBAR(L,L))*CDBLCNT(L,IATOM)
	IF(K.EQ.M.AND.NORBS.EQ.2.AND.                        !!! off-diagonal=0
     &(KDBLCNT(L,IATOM).EQ.12.OR.KDBLCNT(L,IATOM).EQ.14))  !!! spin-up/down
     &VDBC(IU,IATOM,IUPDN,NORBS)=VDBC(IU,IATOM,IUPDN,NORBS)
C Case of Spin Dependent Around Mean Field
      IF(K.EQ.M.AND.
     &(KDBLCNT(L,IATOM).EQ.21.OR.KDBLCNT(L,IATOM).EQ.23))  !!! spin-up/down
     &VDBC(IU,IATOM,IUPDN,1)=VDBC(IU,IATOM,IUPDN,1)+
     &UBAR(L,L)*DBAR(L,IATOM,IUPDN1,1)/2.D0*DBLE(NSPIN)+
     &(UBAR(L,L)-GBAR(L,L))
     &         *DBAR(L,IATOM,IUPDN ,1)/2.D0*DBLE(NSPIN)-
     &(UBAR(L,L)-GBAR(L,L))
     &         *DBAR(L,IATOM,IUPDN ,1)/2.D0*DBLE(NSPIN)/DBLE(2*L+1)
C Note on LmtART 6.60 or later -> 0.5 factor moved to HUBPAR
	IF(K.EQ.M.AND.NORBS.EQ.2.AND.
     &(KDBLCNT(L,IATOM).EQ.21.OR.KDBLCNT(L,IATOM).EQ.23))  !!! off-diagonal
     &VDBC(IU,IATOM,IUPDN,NORBS)=VDBC(IU,IATOM,IUPDN,NORBS)-
     &GBAR(L,L)*DBAR(L,IATOM,IUPDN1,NORBS)
C Case of Spin Averaged Around Mean Field
      IF(K.EQ.M.AND.                                       !!! spin-up/down
     &(KDBLCNT(L,IATOM).EQ.22.OR.KDBLCNT(L,IATOM).EQ.24))  !!! spin-up/down
     &VDBC(IU,IATOM,IUPDN,1)=VDBC(IU,IATOM,IUPDN,1)+
     &UBAR(L,L)*DAVR(L,IATOM)/2.D0+
     &(UBAR(L,L)-GBAR(L,L))
     &         *DAVR(L,IATOM)/2.D0-
     &(UBAR(L,L)-GBAR(L,L))*
     &         *DAVR(L,IATOM)/2.D0/DBLE(2*L+1)
	IF(K.EQ.M.AND.NORBS.EQ.2.AND.                        !!! off-diagonal=0
     &(KDBLCNT(L,IATOM).EQ.22.OR.KDBLCNT(L,IATOM).EQ.24))  !!! spin-up/down
     &VDBC(IU,IATOM,IUPDN,NORBS)=VDBC(IU,IATOM,IUPDN,NORBS)
C Other cases not supported
      IF(KDBLCNT(L,IATOM).GE.30)
     &CALL FULLSTOP('Wrong double counting for this MODE')
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
1000	CONTINUE                                  !!! over atoms
C ADD SPLITTING
	DO IATOM=1,NATOM                          !!! over iatom
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)                        !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	AMAG(+0)=  AMAGIMP(3,L,IATOM)
	AMAG(-1)=+(AMAGIMP(1,L,IATOM)+(0.D0,1.D0)*
     &           AMAGIMP(2,L,IATOM))/SQRT(2.D0)
	AMAG(+1)=-(AMAGIMP(1,L,IATOM)-(0.D0,1.D0)*
     &           AMAGIMP(2,L,IATOM))/SQRT(2.D0)
	AMAG(+0)=AMAG(+0)*BSPLIMP(L,IATOM)
	AMAG(-1)=AMAG(-1)*BSPLIMP(L,IATOM)
	AMAG(+1)=AMAG(+1)*BSPLIMP(L,IATOM)
	DO M=-L,L                                 !!! over m
	LM=L*(L+1)+M+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M+1  !  (l,m m) packed
      VHUB(IU,IATOM,1,1)=VHUB(IU,IATOM,1,1)+AMAG(+0)
      VHUB(IU,IATOM,NSPIN,1)=VHUB(IU,IATOM,NSPIN,1)-AMAG(+0)
	IF(NORBS.EQ.2)THEN
      VHUB(IU,IATOM,1,NORBS)=VHUB(IU,IATOM,1,NORBS)-AMAG(+1)*SQRT(2.D0)
      VHUB(IU,IATOM,NSPIN,NORBS)=VHUB(IU,IATOM,NSPIN,NORBS)
     &                                             +AMAG(-1)*SQRT(2.D0)
	ENDIF
	ENDDO                                     !!! over m
C SET UP SPLITTING TO ZERO
      IF(ISPLIMP(L,IATOM).NE.1)BSPLIMP(L,IATOM)=0.D0
	ENDDO                                     !!! over l
	ENDDO                                     !!! over atoms
C MEMORY DEALLOCATIONS
	DEALLOCATE(UMAT,GMAT,UBAR,GBAR,DBAR,DAVR)
	END




	SUBROUTINE FRC_HUBRENER(EHUB)
**********************************************************
*  Computes Hubbard correction to the LDA energy in      *
*  the LDA+U scheme.                                     *
*  The important parameter is KDBLCNT(L,IATOM) which is  *
*  in MODULE  COM_HUBDAT. See formulae below for a       *
*  particular scheme.                                    *
*  Last revised 29.05.97                                 *
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL,ONLY: IDMF                                      !!! from readini
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS,ILSB           !!! from readini
      USE COM_PANPAR ,ONLY: LTPAN,LSPAN                               !!! from readini
      USE COM_HUBDAT ,ONLY: NHUB,MCRL,LCRL,
     &                      KSOLVER,KSIGMOO,KDBLCNT,CDBLCNT           !!! from readhub
      USE COM_HUBCOM ,ONLY: DHUB,VHUB,VCNS,VDBC                       !!! from readhub
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                               !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: UMAT(:,:)
	REAL*8, ALLOCATABLE :: GMAT(:,:)
	REAL*8, ALLOCATABLE :: UBAR(:,:)
	REAL*8, ALLOCATABLE :: GBAR(:,:)
	COMPLEX*16, ALLOCATABLE :: DBAR(:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: DAVR(:,:)
	DATA PI/3.141592653589793D0/
	EHUB=0.D0
	IF(IDMF.NE.1.AND.IDMF.NE.2)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(UMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(GMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(UBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(GBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(DBAR(0:LMAXBMEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DAVR(0:LMAXBMEM,NATOMMEM))
C DO GLOBALLY OVER ATOMS
	ETHUB=0.D0
	EBHUB=0.D0
	ECEXT=0.D0
	DO 1000 IATOM=1,NATOM                     !!! over atoms
	ISORT=IS(IATOM)
      CALL LIB_HUBMAT_L2(IATOM,UMAT,GMAT,UBAR,GBAR) ! set Hubbard and Exchange matrices
C SET AVERAGE OCCUPANCIES <Nup> and <Ndn>
	DO IORBS=1,NORBS                          !!! over iorbs
	DO IUPDN=1,NSPIN                          !!! over spins
	DO L=0,ILSB(ISORT)                        !!! over l
	DBAR(L,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	DO M=-L,L                                 !!! over m
	LM=L*(L+1)+M+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M+1           !  (l,m m) packed
	DBAR(L,IATOM,IUPDN,IORBS)=DBAR(L,IATOM,IUPDN,IORBS)+
     &DHUB(IU,IATOM,IUPDN,IORBS)
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over iorbs
C  SET MEAN NUMBER OF ELECTRONS
	DO L=0,ILSB(ISORT)                        !!! over l
	DAVR(L,IATOM)=(0.D0,0.D0)
	DO IUPDN=1,NSPIN                          !!! over spins
	DAVR(L,IATOM)=DAVR(L,IATOM)+DBAR(L,IATOM,IUPDN,1)
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over l
C SET HUBBARD CORRECTION TO THE TOTAL ENERGY
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	DO M1=-L,L                              !!! over m1
	DO K1=-L,L                              !!! over k1
	LM1=L*(L+1)+M1+1
	LK1=L*(L+1)+K1+1
	IU1=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K1)+L+M1+1  !  (l,m1 k1) packed
	IF(KSIGMOO(L,IATOM).EQ.11.OR.KSIGMOO(L,IATOM).EQ.12)
C Hartree Fock averaging (Rotationally Invariant, after Liechtenstein et.al )
     &ETHUB=ETHUB+0.5D0*UMAT(IU,IU1)*(
     &DHUB(IU,IATOM,1,1)*DHUB(IU1,IATOM,NSPIN,1)+
     &DHUB(IU,IATOM,1,1)*DHUB(IU1,IATOM,NSPIN,1))
     &           *0.5D0*DBLE(NSPIN)*0.5D0*DBLE(NSPIN)
     &           +0.5D0*(UMAT(IU,IU1)-GMAT(IU,IU1))*(
     &DHUB(IU,IATOM,1    ,1)*DHUB(IU1,IATOM,1    ,1)+
     &DHUB(IU,IATOM,NSPIN,1)*DHUB(IU1,IATOM,NSPIN,1))
     &           *0.5D0*DBLE(NSPIN)*0.5D0*DBLE(NSPIN)
C Relativistic corrections
	IF((KSIGMOO(L,IATOM).EQ.11.OR.KSIGMOO(L,IATOM).EQ.12).AND.NORBS.EQ.2)
     &ETHUB=ETHUB-0.5D0*GMAT(IU,IU1)*(
     &DHUB(IU,IATOM,1,NORBS)*DHUB(IU1,IATOM,NSPIN,NORBS)+
     &DHUB(IU,IATOM,NSPIN,NORBS)*DHUB(IU1,IATOM,1,NORBS))
	ENDDO                                     !!! over k1
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
C SUBTRACT DOUBLE COUNTING TERM
C Spin polarized fully localized limit
      IF(KDBLCNT(L,IATOM).EQ.11.OR.KDBLCNT(L,IATOM).EQ.13)        !!! spin-up/down
     &ETHUB=ETHUB-0.5D0*UBAR(L,L)*(
     &DBAR(L,IATOM,1,1)*DBAR(L,IATOM,NSPIN,1)+
     &DBAR(L,IATOM,1,1)*DBAR(L,IATOM,NSPIN,1))
     &           *0.5D0*DBLE(NSPIN)*0.5D0*DBLE(NSPIN)
     &           -0.5D0*(UBAR(L,L)-GBAR(L,L))*(
     &DBAR(L,IATOM,1    ,1)*DBAR(L,IATOM,1    ,1)+
     &DBAR(L,IATOM,NSPIN,1)*DBAR(L,IATOM,NSPIN,1))
     &           *0.5D0*DBLE(NSPIN)*0.5D0*DBLE(NSPIN)
     &+CDBLCNT(L,IATOM)*(UBAR(L,L)-GBAR(L,L))*(
     &DBAR(L,IATOM,1    ,1)+DBAR(L,IATOM,NSPIN,1))
     &           *0.5D0*DBLE(NSPIN)
	IF((KDBLCNT(L,IATOM).EQ.11.OR.KDBLCNT(L,IATOM).EQ.13)
     &.AND.NORBS.EQ.2)                                            !!! off-diagonal
     &ETHUB=ETHUB+0.5D0*GBAR(L,L)*(
     &DBAR(L,IATOM,1,NORBS)*DBAR(L,IATOM,NSPIN,NORBS)+
     &DBAR(L,IATOM,NSPIN,NORBS)*DBAR(L,IATOM,1,NORBS))
C Spin restricted fully localized limit
      IF(KDBLCNT(L,IATOM).EQ.12.OR.KDBLCNT(L,IATOM).EQ.14)        !!! spin-up/down
     &ETHUB=ETHUB-0.5D0*UBAR(L,L)*(
     &DAVR(L,IATOM)*DAVR(L,IATOM)+
     &DAVR(L,IATOM)*DAVR(L,IATOM))
     &           *0.5D0*0.5D0
     &           -0.5D0*(UBAR(L,L)-GBAR(L,L))*(
     &DAVR(L,IATOM)*DAVR(L,IATOM)+
     &DAVR(L,IATOM)*DAVR(L,IATOM))
     &           *0.5D0*0.5D0
     &+CDBLCNT(L,IATOM)*(UBAR(L,L)-GBAR(L,L))*(
     &DAVR(L,IATOM)+DAVR(L,IATOM))
     &           *0.5D0
	IF((KDBLCNT(L,IATOM).EQ.12.OR.KDBLCNT(L,IATOM).EQ.14)
     &.AND.NORBS.EQ.2) !!! off-diagonal
     &ETHUB=ETHUB
C No energy correction in other cases
      IF(KDBLCNT(L,IATOM).GE.20)
     &ETHUB=0.D0
      IF(KDBLCNT(L,IATOM).GE.20.AND.NORBS.EQ.2)
     &ETHUB=0.D0
	ENDDO                                     !!! over l
C SET HUBBARD CORRECTION TO THE BAND ENERGY
	DO IORBS=1,NORBS                          !!! over spins
	DO IUPDN=1,NSPIN                          !!! over spins
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	EBHUB=EBHUB+DHUB(IU,IATOM,IUPDN,IORBS)*
     &(VHUB(IU,IATOM,IUPDN,IORBS)-VDBC(IU,IATOM,IUPDN,IORBS))
	ECEXT=ECEXT+DHUB(IU,IATOM,IUPDN,IORBS)*
     &VCNS(IU,IATOM,IUPDN,IORBS)
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over iorbs
1000	CONTINUE                                  !!! over atoms
	IF(IDMF.EQ.1)THEN
	EHUB=-ECEXT
	WRITE(IUN,1020)ECEXT
	ELSEIF(IDMF.EQ.2)THEN
	EHUB=ETHUB-EBHUB
	WRITE(IUN,1010)ETHUB,EBHUB
	ENDIF
C MEMORY DEALLOCATIONS
	DEALLOCATE(UMAT,GMAT,UBAR,GBAR,DBAR,DAVR)
1010  FORMAT(1X,'E{hub} : Et,bnd-parts are',2(1X,G14.7))
1020  FORMAT(1X,'E{ext} : due to constrain',1X,G14.7)
	END


