	SUBROUTINE IMP_AGFIRAT_COFS(NRAT,EPOL,WGAT,ELOC,TPAR,
     &           SIGM0,SIGMI,ZMASS,MRAT,WSIG,PSIG,IERR)
***************************************************************
* Finds rational interpolation for self-energy, returns       *
* coefficients Wi and Pi which represents Sigma(w) as         *
* Sigma(w)=Sigma(oo)+Sum{i=1,M)Wi/(w-Pi)                      *
***************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                               !!! from setup
      USE COM_MSBMESH ,ONLY: TEMP                                  !!! from readhub
      USE COM_UNIT    ,ONLY: IUN,IUT,IUR,IUI                       !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
C Passed arrays
	REAL*8 EPOL(NRAT),WGAT(NRAT)
      COMPLEX*16 WSIG(NRAT),PSIG(NRAT)
C Allocatable arrays
	REAL*8, ALLOCATABLE :: GMOM(:)
	REAL*8, ALLOCATABLE :: SMOM(:)
	COMPLEX*16, ALLOCATABLE :: ACOF(:)
	COMPLEX*16, ALLOCATABLE :: BCOF(:)
	COMPLEX*16, ALLOCATABLE :: OMGH(:)
	COMPLEX*16, ALLOCATABLE :: SIGH(:)
	COMPLEX*16, ALLOCATABLE :: OMGG(:)
	COMPLEX*16, ALLOCATABLE :: SIGG(:)
	COMPLEX*16, ALLOCATABLE :: APOL(:,:)
	COMPLEX*16, ALLOCATABLE :: CPOL(:)
	COMPLEX*16, ALLOCATABLE :: RPOL(:)
	COMPLEX*16, ALLOCATABLE :: BCOL(:)
	COMPLEX*16, ALLOCATABLE :: AMAT(:,:)
	COMPLEX*16, ALLOCATABLE :: BREN(:)
	COMPLEX*16, ALLOCATABLE :: WCOL(:)
	COMPLEX*16, ALLOCATABLE :: WMAT(:,:)
C Internal variables
	COMPLEX*16 DIM,CNUM,CDEN,CSIG,ZOMG,OMG0M,OMG0P,SIG0M,SIG0P
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/,EPSW/1.D-04/
     &    ,ACCU/1.D-06/,ISBMF/0/
C CURRENT DENSITY FROM PROBABILITIES
	ZPSI=0.D0
	DO IRAT=1,NRAT
	IF(EPOL(IRAT).LE.0.D0)
     &ZPSI=ZPSI+WGAT(IRAT)
	ENDDO
C MOMENTS OF THE ATOMIC GREEN FUNCTION WITH W-PROBABILITIES
	ALLOCATE(GMOM(0:10))
	ALLOCATE(SMOM(0:10))
	GMOM(0)=0.D0
	DO IMOM=1,10
	GMOM(IMOM)=0.D0
	DO IRAT=1,NRAT
	GMOM(IMOM)=GMOM(IMOM)+WGAT(IRAT)*EPOL(IRAT)**(IMOM-1)
	ENDDO
	ENDDO
C MOMENTS OF THE ATOMIC SELF-ENERGY
	SMOM(0)=-ELOC+GMOM(2) ! This is Sigma(inf)
	SMOM(1)=GMOM(3)-GMOM(2)**2
	SMOM(2)=(GMOM(4)-2.D0*GMOM(2)*GMOM(3)+GMOM(2)**3)
	DO IMOM=3,10
	SMOM(IMOM)=0.D0
	ENDDO
C COUNT NON-ZERO CONTRIBUTIONS
	NHUB=0
	DO IRAT=1,NRAT
	IF(WGAT(IRAT).GT.EPSW)NHUB=NHUB+1
	ENDDO
C ACCOUNT FOR FILLED OR EMPTY CASE
	IF(NHUB.LE.1)THEN
	MRAT=0
	DEALLOCATE(GMOM,SMOM)
	RETURN
	ENDIF
C Memory Allocations
	ALLOCATE(OMGH(NHUB))
	ALLOCATE(SIGH(NHUB))
C POSITIONS OF HUBBARD BANDS
	IHUB=0
	DO IRAT=1,NRAT
	IF(WGAT(IRAT).LE.EPSW)CYCLE
	IHUB=IHUB+1
	OMGH(IHUB)=DCMPLX(EPOL(IRAT),-0.0D0)
	ENDDO
C SELF-ENERGY AT THOSE POSITIONS
	DO IHUB=1,NHUB
	SIGH(IHUB)=OMGH(IHUB)-ELOC
	ENDDO
C Select the pole which is closest to zero
	OMGH0=ABS(DREAL(OMGH(1)))
	DO IHUB=1,NHUB-1
	OMGH0=MIN(OMGH0,ABS(DREAL(OMGH(IHUB))))
	ENDDO
	IHUBH0=0
	DO IHUB=1,NHUB-1
	IF(ABS(OMGH0-ABS(DREAL(OMGH(IHUB)))).LE.1.D-10)IHUBH0=IHUB
	ENDDO
	IF(IHUBH0.EQ.0)CALL FULLSTOP('IHUBH0=0')
C ZEROES OF THE ATOMIC GREEN FUNCTION
	ALLOCATE(OMGG(NHUB-1))
	ALLOCATE(SIGG(NHUB-1))
	ALLOCATE(APOL(0:NHUB-1,NHUB))
	ALLOCATE(CPOL(0:NHUB-1))
	ALLOCATE(RPOL(NHUB-1))
	DO IHUB=1,NHUB
C Singularities at Hubbard bands
	IROT=0
	DO JHUB=1,NHUB
	IF(JHUB.EQ.IHUB)CYCLE
	IROT=IROT+1
	RPOL(IROT)=OMGH(JHUB)
	ENDDO
	MPOL=NHUB-1
C Coefficients
	CALL LIB_PCOEFS(MPOL,RPOL,APOL(0,IHUB))
	ENDDO
C Convolve with weights
	DO IPOL=0,NHUB-1
	CPOL(IPOL)=(0.D0,0.D0)
	IHUB=0
	DO IRAT=1,NRAT
	IF(WGAT(IRAT).LE.EPSW)CYCLE
	IHUB=IHUB+1
	CPOL(IPOL)=CPOL(IPOL)+WGAT(IRAT)*APOL(IPOL,IHUB)
	ENDDO
	ENDDO
C Find zeroes now
	CALL LIB_PZEROS(NHUB-1,CPOL,OMGG,1.D-14,IFAIL)
	IF(IFAIL.NE.0)WRITE(IUT,*)'Zeroes fail'
C Self-energy is infinite at those points, reverse is 0
	DO IHUB=1,NHUB-1	
	SIGG(IHUB)=(0.D0,0.D0)
	ENDDO
C Misplace the pole closest to zero a little to avoid numerical errors
	DO IHUB=1,NHUB-1	
	IF(ABS(OMGG(IHUB)).LT.1.D-14)OMGG(IHUB)=DCMPLX(1.D-14,0.D0)
	ENDDO
C Check atomic Green function
	DO IHUB=1,NHUB-1
	GATM=0.D0
	DO IRAT=1,NRAT
	IF(WGAT(IRAT).LE.EPSW)CYCLE
	DENM=OMGG(IHUB)-EPOL(IRAT)
	IF(ABS(DENM).LE.1.D-10)CYCLE
	GATM=GATM+WGAT(IRAT)/DENM
	ENDDO
	IF(ABS(GATM).GT.ACCU)WRITE(IUT,*)'Wrong zeroes',GATM,OMGG(IHUB)
	ENDDO
C Select the pole which is closest to zero
	OMGG0=ABS(DREAL(OMGG(1)))
	DO IHUB=1,NHUB-1
	OMGG0=MIN(OMGG0,ABS(DREAL(OMGG(IHUB))))
	ENDDO
	IHUBG0=0
	DO IHUB=1,NHUB-1
	IF(ABS(OMGG0-ABS(DREAL(OMGG(IHUB)))).LE.1.D-10)IHUBG0=IHUB
	ENDDO
	IF(IHUBG0.EQ.0)CALL FULLSTOP('IHUBG0=0')
C Check mass renormalization
	IF(ZMASS.GT.1.D-04)ZMAS1=1.D0/ZMASS-1.D0
	IF(ZMASS.LE.1.D-04)ZMAS1=1.D+04-1.D0
C Set self energies at two points closest to zero
	OMG0M=-TPAR*ZMASS
	OMG0P=+TPAR*ZMASS
	IF(DREAL(OMGH(IHUBH0)).LT.0.D0)OMG0P=(0.D0,0.D0)
	IF(DREAL(OMGH(IHUBH0)).GE.0.D0)OMG0M=(0.D0,0.D0)
	SIG0M=-ZMAS1*OMG0M+SIGM0
	SIG0P=-ZMAS1*OMG0P+SIGM0
C Imaginary axis case
	OMG0M=(0.D0,0.D0)
	OMG0P=TPAR*DIM
	SIG0M=-ZMAS1*OMG0M+SIGM0
	SIG0P=-ZMAS1*OMG0P+SIGM0
C Do not decide which information to throw away
C	IF(OMGG0.LT.OMGH0)IHUBH0=0 ! Throw Green function zero
C	IF(OMGG0.GE.OMGH0)IHUBG0=0 ! Throw Atomic Pole
C Always throw away Green function pole
	IHUBH0=0
C Check Mott insulating regime
	IF(ZMASS.GT.1.D-04)IMOTT=0 ! Metalic regime
	IF(ZMASS.LE.1.D-04)IMOTT=1 ! Insulating regime
	IF(ZMASS.LE.1.D-04)IHUBH0=0 ! Account for all zeroes of Green function
	IF(ZMASS.LE.1.D-04)IHUBG0=0 ! Account for all Hubbard bands
C FIND INTERPOLATION COEFFICIENTS
	IF(IMOTT.EQ.0)MRAT=NHUB
	IF(IMOTT.EQ.1)MRAT=NHUB-1
	IF(IMOTT.EQ.0)NCON=2*NHUB+2
	IF(IMOTT.EQ.1)NCON=2*NHUB
	ALLOCATE(ACOF(0:MRAT))
	ALLOCATE(BCOF(0:MRAT))
	DO IRAT=0,MRAT
	ACOF(IRAT)=(0.D0,0.D0)
	BCOF(IRAT)=(0.D0,0.D0)
	ENDDO
	ALLOCATE(AMAT(NCON,2*MRAT+2))
	ALLOCATE(BCOL(NCON))
	DO ICON=1,NCON
	BCOL(ICON)=0.D0
	DO IRAT=1,2*MRAT+2
	AMAT(ICON,IRAT)=0.D0
	ENDDO
	ENDDO
C Count conditions
	ICON=0
C 1. Poles of atomic Green function except the one which is closest to w=0
	DO IHUB=1,NHUB
	IF(IHUB.EQ.IHUBH0)CYCLE
	ICON=ICON+1
	DO IRAT=0,MRAT
	AMAT(ICON,IRAT+1)=OMGH(IHUB)**IRAT
	ENDDO
	DO IRAT=0,MRAT
	AMAT(ICON,MRAT+1+IRAT+1)=-SIGH(IHUB)*OMGH(IHUB)**IRAT
	ENDDO
	BCOL(ICON)=(0.D0,0.D0)
	ENDDO
C 2. Zeroes of atomic Green functions except the one which is closest to w=0
	DO IHUB=1,NHUB-1
	IF(IHUB.EQ.IHUBG0)CYCLE
	ICON=ICON+1
	DO IRAT=0,MRAT
	AMAT(ICON,IRAT+1)=SIGG(IHUB)*OMGG(IHUB)**IRAT
	ENDDO
	DO IRAT=0,MRAT
	AMAT(ICON,MRAT+1+IRAT+1)=-OMGG(IHUB)**IRAT
	ENDDO
	BCOL(ICON)=(0.D0,0.D0)
	ENDDO
C 3. b0 is always 1
	ICON=ICON+1
	AMAT(ICON,MRAT+1+1)=(1.D0,0.D0)   ! b0=1
	BCOL(ICON)=(1.D0,0.D0)
C 4. Sigma(infinity) limit for metallic case only
	IF(IMOTT.EQ.0)THEN
	ICON=ICON+1
	AMAT(ICON,MRAT+1)=(1.D0,0.D0)
	AMAT(ICON,MRAT+1+MRAT+1)=-DCMPLX(SIGMI,0.D0)
	BCOL(ICON)=(0.D0,0.D0)
	ENDIF
C 5. SBMF for metallic case with Sigma(0) and Slope(0)
	IF(IMOTT.EQ.0.AND.ISBMF.EQ.0)THEN
	ICON=ICON+1
	AMAT(ICON,1)=(1.D0,0.D0)          ! a0=Sigma(0)
	BCOL(ICON)=DCMPLX(SIGM0,0.D0)
	ICON=ICON+1
	AMAT(ICON,2)=DCMPLX(1.D0,0.D0)
	AMAT(ICON,MRAT+1+2)=-DCMPLX(SIGM0,0.D0)
	BCOL(ICON)=DCMPLX(-ZMAS1,0.D0)
C 5. SBMF for metallic case with Sigma(+/-)
	ELSEIF(IMOTT.EQ.0.AND.ISBMF.EQ.1)THEN
	ICON=ICON+1
	DO IRAT=0,MRAT
	AMAT(ICON,IRAT+1)=OMG0M**IRAT
	ENDDO
	DO IRAT=0,MRAT
	AMAT(ICON,MRAT+1+IRAT+1)=-SIG0M*OMG0M**IRAT
	ENDDO
	BCOL(ICON)=(0.D0,0.D0)
	ICON=ICON+1
	DO IRAT=0,MRAT
	AMAT(ICON,IRAT+1)=OMG0P**IRAT
	ENDDO
	DO IRAT=0,MRAT
	AMAT(ICON,MRAT+1+IRAT+1)=-SIG0P*OMG0P**IRAT
	ENDDO
	BCOL(ICON)=(0.D0,0.D0)
	ENDIF
	IF(ICON.NE.NCON)CALL FULLSTOP('Wrong iCon')
C Solve for coefficients now
	CALL LIB_CSIMQ(AMAT,BCOL,NCON,2*MRAT+2,IFAIL)
	IF(IFAIL.GT.0)WRITE(IUT,*)'Simq fails:',IFAIL
	DO IRAT=0,MRAT
	ACOF(IRAT)=BCOL(IRAT+1)
	ENDDO
	DO IRAT=0,MRAT
	BCOF(IRAT)=BCOL(MRAT+1+IRAT+1)
	ENDDO
C FIND POLE REPRESENTATION
	ALLOCATE(BREN(0:MRAT))
	DO IRAT=0,MRAT
	BREN(IRAT)=BCOF(IRAT)/BCOF(MRAT)
	ENDDO
	CALL LIB_PZEROS(MRAT,BREN,PSIG,1.D-14,IFAIL)
	IF(IFAIL.NE.0)WRITE(IUT,*)'Zeroes fail'
C Coefficients
	DO IPOL=1,MRAT
	IROT=0
	DO JPOL=1,MRAT
	IF(JPOL.EQ.IPOL)CYCLE
	IROT=IROT+1
	RPOL(IROT)=PSIG(JPOL)
	ENDDO
	MPOL=MRAT-1
	IF(MPOL.EQ.0)APOL(0,1)=(1.D0,0.D0)
	IF(MPOL.GT.0)
     &CALL LIB_PCOEFS(MPOL,RPOL,APOL(0,IPOL))
	ENDDO
C Solve for weights now
	ALLOCATE(WMAT(0:MRAT-1,MRAT))
	ALLOCATE(WCOL(0:MRAT-1))
	DO IRAT=0,MRAT-1
	WCOL(IRAT)=(ACOF(IRAT)-BCOF(IRAT)*ACOF(MRAT)/BCOF(MRAT))
     &                                            /BCOF(MRAT)
	DO IPOL=1,MRAT
	WMAT(IRAT,IPOL)=APOL(IRAT,IPOL)
	ENDDO
	ENDDO
	CALL LIB_CSIMQ(WMAT,WCOL,MRAT,MRAT,IFAIL)
	IF(IFAIL.GT.0)WRITE(IUT,*)'Simq fails:',IFAIL
	IPOL=0
	DO IRAT=0,MRAT-1
	IPOL=IPOL+1
	WSIG(IPOL)=WCOL(IRAT)
	ENDDO
	IF(ISBMF.EQ.0)THEN
C Check Sigma(0) using rational approximation
	ZOMG=(0.D0,0.D0)
	CNUM=0.D0
	DO IRAT=0,MRAT
	CNUM=CNUM+ACOF(IRAT)*ZOMG**IRAT
	ENDDO
	CDEN=0.D0
	DO IRAT=0,MRAT
	CDEN=CDEN+BCOF(IRAT)*ZOMG**IRAT
	ENDDO
	CSIG=CNUM/CDEN
	IF(ABS(CSIG-SIGM0).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Sigma(0) fails'
C Check Sigma(0) using pole representation
	CSIG=DCMPLX(SIGMI,0.D0)
	DO IPOL=1,MRAT
	IF(ABS(ZOMG-PSIG(IPOL)).LE.ACCU)CYCLE
	CSIG=CSIG+WSIG(IPOL)/(ZOMG-PSIG(IPOL))
	ENDDO
	IF(ABS(CSIG-SIGM0).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Sigma(0) fails'
C Check slope at zero using pole representation
	CSIG=DCMPLX(0.D0,0.D0)
	DO IPOL=1,MRAT
	IF(ABS(PSIG(IPOL)).LE.ACCU)CYCLE
	CSIG=CSIG-WSIG(IPOL)/PSIG(IPOL)**2
	ENDDO
	IF(ABS(CSIG+ZMAS1).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Slope(0) fails'
	ELSEIF(ISBMF.EQ.1)THEN
C Check Sigma(-) using rational approximation
	ZOMG=OMG0M
	CNUM=0.D0
	DO IRAT=0,MRAT
	CNUM=CNUM+ACOF(IRAT)*ZOMG**IRAT
	ENDDO
	CDEN=0.D0
	DO IRAT=0,MRAT
	CDEN=CDEN+BCOF(IRAT)*ZOMG**IRAT
	ENDDO
	CSIG=CNUM/CDEN
	IF(ABS(CSIG-SIG0M).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Sigma(-) fails'
C Check Sigma(-) using pole representation
	CSIG=DCMPLX(SIGMI,0.D0)
	DO IPOL=1,MRAT
	IF(ABS(ZOMG-PSIG(IPOL)).LE.ACCU)CYCLE
	CSIG=CSIG+WSIG(IPOL)/(ZOMG-PSIG(IPOL))
	ENDDO
	IF(ABS(CSIG-SIG0M).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Sigma(-) fails'
C Check Sigma(+) using rational approximation
	ZOMG=OMG0P
	CNUM=0.D0
	DO IRAT=0,MRAT
	CNUM=CNUM+ACOF(IRAT)*ZOMG**IRAT
	ENDDO
	CDEN=0.D0
	DO IRAT=0,MRAT
	CDEN=CDEN+BCOF(IRAT)*ZOMG**IRAT
	ENDDO
	CSIG=CNUM/CDEN
	IF(ABS(CSIG-SIG0P).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Sigma(+) fails'
C Check Sigma(-) using pole representation
	CSIG=DCMPLX(SIGMI,0.D0)
	DO IPOL=1,MRAT
	IF(ABS(ZOMG-PSIG(IPOL)).LE.ACCU)CYCLE
	CSIG=CSIG+WSIG(IPOL)/(ZOMG-PSIG(IPOL))
	ENDDO
	IF(ABS(CSIG-SIG0P).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Sigma(+) fails'
	ENDIF
C Check Sigma at Hubbard bands using rational approximation
	DO IHUB=1,NHUB
	IF(IHUB.EQ.IHUBH0)CYCLE
	ZOMG=OMGH(IHUB)
	CNUM=0.D0
	DO IRAT=0,MRAT
	CNUM=CNUM+ACOF(IRAT)*ZOMG**IRAT
	ENDDO
	CDEN=0.D0
	DO IRAT=0,MRAT
	CDEN=CDEN+BCOF(IRAT)*ZOMG**IRAT
	ENDDO
	CSIG=CNUM/CDEN
	IF(ABS(CSIG-SIGH(IHUB)).GT.ACCU)
     &WRITE(IUT,*)'Sigma(Whub) fails'
	ENDDO
C Check Sigma at Hubbard bands using pole representation
	DO IHUB=1,NHUB
	IF(IHUB.EQ.IHUBH0)CYCLE
	ZOMG=OMGH(IHUB)
	CSIG=ACOF(MRAT)/BCOF(MRAT) ! this sigma(oo) for metallic case
	DO IPOL=1,MRAT
	CSIG=CSIG+WSIG(IPOL)/(ZOMG-PSIG(IPOL))
	ENDDO
	IF(ABS(CSIG-SIGH(IHUB)).GT.ACCU)
     &WRITE(IUT,*)'Sigma(Whub) fails'
	ENDDO
C Check Sigma(oo). In insulating case, the input Sigma(oo) can be different.
	CSIG=ACOF(MRAT)/BCOF(MRAT)
	IF(ABS(CSIG-SIGMI).GT.ACCU.AND.IMOTT.EQ.0)
     &WRITE(IUT,*)'Sigma(oo) fails'
C Check singularties
	DO IHUB=1,NHUB-1
	IF(IHUB.EQ.IHUBG0)CYCLE
	ZOMG=OMGG(IHUB)
	CNUM=0.D0
	DO IRAT=0,MRAT
	CNUM=CNUM+ACOF(IRAT)*ZOMG**IRAT
	ENDDO
	CDEN=0.D0
	DO IRAT=0,MRAT
	CDEN=CDEN+BCOF(IRAT)*ZOMG**IRAT
	ENDDO
	CSIG=CDEN/CNUM
	IF(ABS(CSIG-SIGG(IHUB)).GT.ACCU)
     &WRITE(IUT,*)'Sigma(Wgrn) fails'
	ENDDO
C Sort poles so that the closest to zero is the first pole
      DO IPOL=1,MRAT
      DO JPOL=IPOL+1,MRAT
	IF(ABS(DREAL(PSIG(IPOL))).GT.ABS(DREAL(PSIG(JPOL))))THEN
	CSIG=PSIG(IPOL)
	PSIG(IPOL)=PSIG(JPOL)
	PSIG(JPOL)=CSIG
	CSIG=WSIG(IPOL)
	WSIG(IPOL)=WSIG(JPOL)
	WSIG(JPOL)=CSIG
	ENDIF
	ENDDO
	ENDDO
C Print poles and coefficients
	WRITE(IUN,10100)
	WRITE(IUN,10110)
      DO IPOL=1,MRAT
	WRITE(IUN,10120)IPOL,PSIG(IPOL),WSIG(IPOL)
	IF(DREAL(WSIG(IPOL)).LT.0.D0)THEN
	WHATTODO=1.D0
	WSIG(IPOL)=(0.D0,0.D0)
	ENDIF
	ENDDO
10100 FORMAT(1X,'Self-Energy Pole Expansion Results:')
10110 FORMAT(1X,'Pole            Position                      Weight')
10120 FORMAT(1X,I3,3X,'(',G12.5,',',G12.5,')',
     &             2X,'(',G12.5,',',G12.5,')')
C Memory deallocations
	DEALLOCATE(APOL,CPOL,RPOL,AMAT,BCOL,WCOL,WMAT)
	DEALLOCATE(GMOM,SMOM,OMGH,SIGH,OMGG,SIGG,BREN,ACOF,BCOF)
	END
