	SUBROUTINE IMP_CRFHUB1_STAT(NDEG,NREP,NSPIN,LREP,NCNF,NCNF0,
     &           EATS,TPAR,UPAR,GPAR,ELOC,DEL,GFF,SIG,XHUB,YHUB,
     &           IFIX,SIGI,VRAT,WRAT,POLE,OVRL,ZIMP,ZREP,
     &           KFIX,TEMP,NMSC,WMSC,FMSC,DAMP,IERR)
****************************************************************
* Solves Anderson impurity model, returns Greens function      *
* and Self-energy                                              *
* Essential input:                                             *
*   nDeg - total degeneracy including spin (always even)       *
*   nRep - number of sublevels                                 *
*   lRep(iRep) - remaining degeneracy for each spin sublevel   *
*   eLoc(iRep) - positions of each spin sublevel               *
*   nCnf - total number of configurations within each spin     *
*   channel, product of lRep+1                                 *
*   Upar - Hubbard U                                           *
*   Gpar - Exchange J                                          *
* Essential output:                                            *
*   Gff(iW) - impurity Green function                          *
*   Sigma(iW) - self-energy                                    *
****************************************************************
      USE COM_UNIT    ,ONLY: IUN,IUT,IUR,IUI            !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
C PASSED ARRAYS
	INTEGER LREP(NREP),IFIX(NCNF,NCNF0)
	DIMENSION ELOC(NREP,NSPIN),FMSC(0:NMSC),EATS(NCNF,NCNF0)
     &    ,ZIMP(NREP,NSPIN),ZREP(NREP,NSPIN)
     &    ,DIMP(NREP,NSPIN),DREP(NREP,NSPIN)
     &    ,XHUB(NCNF,NCNF0),YHUB(NCNF,NCNF0)
     &    ,VRAT(NREP,NSPIN),WRAT(NREP,NSPIN),OVRL(NREP,NSPIN)
     &    ,POLE(NREP,NSPIN),SIGI(NREP,NSPIN)
	COMPLEX*16 DEL(0:NMSC+1,NREP,NSPIN),WMSC(0:NMSC)
     &    ,GFF(0:NMSC+1,NREP,NSPIN),SIG(0:NMSC+1,NREP,NSPIN)
	CHARACTER HELP*40,TCTRL*10,TCTRM*10,LIB_TDP*10
C ALLOCATABLE ARRAYS
	INTEGER, ALLOCATABLE :: MREP(:)
	INTEGER, ALLOCATABLE :: LCFG(:)
	INTEGER, ALLOCATABLE :: IPCF(:)
	INTEGER, ALLOCATABLE :: IPRF(:)
	INTEGER, ALLOCATABLE :: IPNF(:)
	INTEGER, ALLOCATABLE :: IPNF0(:)
	INTEGER, ALLOCATABLE :: IPNFP1(:)
	REAL*8, ALLOCATABLE :: COMB(:,:)
	REAL*8, ALLOCATABLE :: COMR(:)
	REAL*8, ALLOCATABLE :: ZTOT(:)
	REAL*8, ALLOCATABLE :: ZHUB(:)
	REAL*8, ALLOCATABLE :: ZSUM(:)
	REAL*8, ALLOCATABLE :: ZLEV(:,:)
	REAL*8, ALLOCATABLE :: XSTA(:,:)
	COMPLEX*16, ALLOCATABLE :: GAT(:,:,:)
	COMPLEX*16, ALLOCATABLE :: GDGN(:,:,:,:)
C Local variables
	COMPLEX*16 DIM,ZMSC
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/,EPS/1.D-05/
C CONFIGURATIONS, Minimum: NDEG/2+1, Maximum 2**(NDEG/2)
	ALLOCATE(MREP(0:NREP))
	MREP(0)=1
	DO IREP=1,NREP
	MREP(IREP)=MREP(IREP-1)*(LREP(IREP)+1)
	ENDDO
C SORT BY NUMBERS OF ELECTRONS
	ALLOCATE(IPNF(NREP))
	ALLOCATE(IPNF0(NREP))
	ALLOCATE(IPCF(NCNF))
	ALLOCATE(IPRF(NCNF))
	ALLOCATE(LCFG(0:NDEG/NSPIN))
C Looping over total allowed numbers of electrons in all sublevels
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	LCFG(IDEG)=0
C Configurations are numbered in one way from 1 to NCNF.
C Start looping over them
	DO ICNF=1,NCNF
C Find out numbers of electrons at each sublevel for given configuration ICNF
C Return array is IPNF(IREP) for spin channel
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	NELC=0 ! count # of electrons for given configuration
	DO IREP=1,NREP
	NELC=NELC+IPNF(IREP)
	ENDDO
C if this configuration contains right number of electrons, count it
	IF(NELC.NE.IDEG)CYCLE 
	LCFG(IDEG)=LCFG(IDEG)+1
	ICFR=ICFR+1
C this pointer relates second-to-first numbering schemes for configurations
	IPCF(ICFR)=ICNF 
	ENDDO
	ENDDO
C CHECK ALGORITHM
	ICFR=0 ! second way of numbering configurations
C looping over allowed total numbers of electrons of the same spin
	DO IDEG=0,NDEG/NSPIN
C looping over configurations within given number of electrons of the same spin
	DO ICFG=1,LCFG(IDEG) 
	ICFR=ICFR+1
	ICNF=IPCF(ICFR) ! find out the configuration in first numbering scheme
C Find out array showing how many electrons are in each sublevel
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF) 
	NELC=0 ! count # of electrons
	DO IREP=1,NREP
	IF(IPNF(IREP).LT.0.OR.IPNF(IREP).GT.LREP(IREP))
     &CALL FULLSTOP('Wrong configuration')
	NELC=NELC+IPNF(IREP)
	ENDDO
	ENDDO
	IF(NELC.NE.IDEG)CALL FULLSTOP('# of electrons mismatch')
	ENDDO
C Set reciprocal pointer which relates first-to-second numbering scheme for configirations
	DO ICNF=1,NCNF 
	IPRF(ICNF)=-1
	ENDDO
	DO ICFR=1,NCNF
	ICNF=IPCF(ICFR)
	IPRF(ICNF)=ICFR
	ENDDO
	DO ICNF=1,NCNF
	IF(IPRF(ICNF).EQ.-1)CALL FULLSTOP('Wrong reciprocal pointer')
	ENDDO
C PRINT BLOCK
	WRITE(IUN,10010)NDEG,NREP,NSPIN
	WRITE(IUN,10020)(LREP(IREP),IREP=1,NREP)
	MCNF=0
	WRITE(IUN,10030)
	DO IDEG=0,NDEG/NSPIN
	WRITE(IUN,10040)IDEG,LCFG(IDEG)
	MCNF=MCNF+LCFG(IDEG)
	ENDDO
	IF(MCNF.NE.NCNF)CALL FULLSTOP('Configurations mismatch')
	WRITE(IUN,10050)NCNF
10010	FORMAT(1X,'Total degeneracy, # of representations,spins = ',
     &I2,1X,I2,1X,I2)
10020	FORMAT(1X,'Subdegeneracies left => ',14(1X,I2))
10030	FORMAT(1X,' n     Cnf ')
10040	FORMAT(1X,I2,2X,I5)
10050	FORMAT(1X,'Total # of configurations = ',I5)
C *******************************************
C Compute product of all combinatorics using second scheme of 
C numbering configurations
	ALLOCATE(COMR(NCNF))
	ALLOCATE(COMB(0:NDEG,0:NDEG))
	CALL LIB_GETCOMB(COMB,NDEG)
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	ICNF=IPCF(ICFR)
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	COMR(ICFR)=1.D0 
	DO IREP=1,NREP
	COMR(ICFR)=COMR(ICFR)*COMB(IPNF(IREP),LREP(IREP))
	ENDDO
	ENDDO
	ENDDO
C CHECK COMBINATORICS
	SUS=0.D0
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	SUM=0.D0
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	SUM=SUM+COMR(ICFR)
	SUS=SUS+COMR(ICFR)
	ENDDO
	IF(ABS(SUM-COMB(IDEG,NDEG/NSPIN)).GT.1.D-07)
     &CALL FULLSTOP('Combinatorics mismatch')
	ENDDO
	IF(ABS(SUS-2.D0**(NDEG/NSPIN)).GT.1.D-07)
     &CALL FULLSTOP('Combinatorics mismatch')
C *************************************
C Many-body atomic levels EATS numbered using second scheme of numbering configurations
	IF(NSPIN.EQ.2.AND.NCNF.NE.NCNF0)CALL FULLSTOP('nCnf.ne.nCnf0')
	IF(NSPIN.EQ.1.AND.NCNF0.NE.1)CALL FULLSTOP('nCnf0 should be 1')
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	ICNF=IPCF(ICFR)
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	ICFR0=0
	DO IDEG0=0,NDEG/NSPIN
	DO ICFG0=1,LCFG(IDEG0)
	ICFR0=ICFR0+1
	IF(NSPIN.EQ.1.AND.ICFR0.GT.1)CYCLE
	ICNF0=IPCF(ICFR0)
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF0,NCNF,IPNF0)
	EATS(ICFR,ICFR0)=0.D0
	NELC =0
	NELC0=0
	DO IREP=1,NREP
	IF(NSPIN.EQ.1.AND.IPNF0(IREP).NE.0)CALL FULLSTOP('Ipnf0.ne.0')
	EATS(ICFR,ICFR0)=EATS(ICFR,ICFR0)+
     &ELOC(IREP,1)*DBLE(IPNF(IREP))+ELOC(IREP,NSPIN)*DBLE(IPNF0(IREP))
	NELC =NELC +IPNF(IREP)
	NELC0=NELC0+IPNF0(IREP)
	ENDDO
	IF(NELC .NE.IDEG )CALL FULLSTOP('# of electrons mismatch')
	IF(NELC0.NE.IDEG0)CALL FULLSTOP('# of electrons mismatch')
	ITOT=IDEG+IDEG0 ! total number of electrons
	IF(NSPIN.EQ.1)DIUP=DBLE(ITOT)/2.D0
	IF(NSPIN.EQ.1)DIDN=DBLE(ITOT)/2.D0
	IF(NSPIN.EQ.2)DIUP=DBLE(IDEG)
	IF(NSPIN.EQ.2)DIDN=DBLE(IDEG0)
	EATS(ICFR,ICFR0)=EATS(ICFR,ICFR0)+
     &UPAR*DBLE(ITOT*(ITOT-1))/2.D0-
     &GPAR*    (DIUP*(DIUP-1))/2.D0-
     &GPAR*    (DIDN*(DIDN-1))/2.D0
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	WRITE(IUN,10110)
	WRITE(IUN,10120)TEMP
	WRITE(IUN,10130)UPAR
	WRITE(IUN,10135)GPAR
	WRITE(IUN,10140)NDEG
	WRITE(IUN,10150)NMSC
10110	FORMAT(1X,'Solving impurity model at low frequencies:')
10120	FORMAT(1X,'Temp = ',G12.5)
10130	FORMAT(1X,'Upar = ',G12.5)
10135	FORMAT(1X,'Jpar = ',G12.5)
10140	FORMAT(1X,'Ndeg = ',I5)
10150	FORMAT(1X,'Nmsc = ',I5)
C ***************************************************************************************
C MEMORY ALLOCATIONS
	ALLOCATE(IPNFP1(NREP))
	ALLOCATE(GDGN(0:NMSC,NCNF,NREP,NSPIN))
	ALLOCATE(GAT(0:NMSC,NREP,NSPIN))
	ALLOCATE(ZLEV(NREP,NSPIN))
	ALLOCATE(XSTA(NCNF,NCNF0))
	ALLOCATE(ZTOT(NSPIN))
	ALLOCATE(ZHUB(NSPIN))
	ALLOCATE(ZSUM(NSPIN))
	IERR=0
C Find correlational functions
      CALL IMP_CRFHUB1_XNOT(NCNF,NCNF0,EATS,TEMP,COMR,XSTA)
      DO ICNF=1,NCNF
      DO JCNF=1,NCNF0
	IF(KFIX.EQ.0.AND.IFIX(ICNF,JCNF).EQ.0)
     &XHUB(ICNF,JCNF)=XSTA(ICNF,JCNF)
	ENDDO
	ENDDO
C Renormalize 
      XSUM=0.D0
      CSUM=0.D0
      DO ICNF=1,NCNF
      DO JCNF=1,NCNF0
      XSUM=XSUM+COMR(ICNF)*COMR(JCNF)*XHUB(ICNF,JCNF)
      CSUM=CSUM+COMR(ICNF)*COMR(JCNF)
      ENDDO
	ENDDO
	IF(XSUM.LE.0.D0)XSUM=CSUM
      DO ICNF=1,NCNF
      DO JCNF=1,NCNF0
      XHUB(ICNF,JCNF)=XHUB(ICNF,JCNF)/XSUM
      YHUB(ICNF,JCNF)=0.D0
      ENDDO
      ENDDO
C Estimate level charges from statistical weights
	DO ISPIN=1,NSPIN
	DO IREP=1,NREP
	ZREP(IREP,ISPIN)=0.D0
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	ICNF=IPCF(ICFR)
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	DO JCFR=1,NCNF0
	IF(ISPIN.EQ.1)
     &ZREP(IREP,ISPIN)=ZREP(IREP,ISPIN)+DBLE(IPNF(IREP))*
     &COMR(ICFR)*COMR(JCFR)*XHUB(ICFR,JCFR)
	IF(ISPIN.EQ.2)
     &ZREP(IREP,ISPIN)=ZREP(IREP,ISPIN)+DBLE(IPNF(IREP))*
     &COMR(JCFR)*COMR(ICFR)*XHUB(JCFR,ICFR)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C  Estimate total charge from statistical weights
	DO ISPIN=1,NSPIN
	ZHUB(ISPIN)=0.D0
	DO IREP=1,NREP
	ZLEV(IREP,ISPIN)=ZREP(IREP,ISPIN)/DBLE(LREP(IREP))
	ZHUB(ISPIN)=ZHUB(ISPIN)+ZREP(IREP,ISPIN)
	ENDDO
	ENDDO
C  SET SPIN DENSITIES FOR SIGMA(INF) BEHAVIOR
C  The spin-up and down densities are given by current atomic filling
	ZTOT(1)    =ZHUB(1)    /2.D0*DBLE(NSPIN)
	ZTOT(NSPIN)=ZHUB(NSPIN)/2.D0*DBLE(NSPIN)
C  Set rational interpolation arrays
	DO ISPIN=1,NSPIN
	DO IREP=1,NREP
	IF(ISPIN.EQ.1)ISPIN1=NSPIN
	IF(ISPIN.EQ.NSPIN)ISPIN1=1
	SIGI(IREP,ISPIN)=UPAR*ZTOT(ISPIN1)+
     &(UPAR-GPAR)*(ZTOT(ISPIN)-ZLEV(IREP,ISPIN))
	VRAT(IREP,ISPIN)=(UPAR-GPAR)*
     &SQRT(ABS(ZLEV(IREP,ISPIN)*(1.D0-ZLEV(IREP,ISPIN))))
	WRAT(IREP,ISPIN)=(UPAR-GPAR)*
     &SQRT(ABS(ZLEV(IREP,ISPIN)*(1.D0-ZLEV(IREP,ISPIN))))
	POLE(IREP,ISPIN)=ELOC(IREP,ISPIN)+UPAR*ZTOT(ISPIN1)+
     &(UPAR-GPAR)*(ZTOT(ISPIN)+ZLEV(IREP,ISPIN)-1.D0)
	OVRL(IREP,ISPIN)=1.D0
	ENDDO
	ENDDO
C Set atomic G22
	DO ISPIN=1,NSPIN            ! looping over spins
	DO IREP=1,NREP              ! looping over sublevels
	ICFR=0
	DO IDEG=0,NDEG/NSPIN        ! looping over all configurations
	DO ICFG=1,LCFG(IDEG)        ! using second numbering scheme
	ICFR=ICFR+1
	DO IMSC=0,NMSC
      GDGN(IMSC,ICFR,IREP,ISPIN)=(0.D0,0.D0)
	ENDDO
	ICNF=IPCF(ICFR)
C Find real occupancies for each sublevel within given configuration
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
C Increase occupancy of sublevel IREP by 1
	DO IREP1=1,NREP
	IPNFP1(IREP1)=IPNF(IREP1)
	ENDDO
	IPNFP1(IREP)=IPNFP1(IREP)+1
C Find out corresponding configuration number in the first numbering scheme
	CALL LIB_SETCONF(NREP,LREP,MREP,ICNFP1,NCNF,IPNFP1)
	ICFRP1=0 ! try to understand that we did not exceed the total allowed 
	IF(IPNF(IREP).NE.LREP(IREP))ICFRP1=IPRF(ICNFP1) ! number of electrons
C Set contribution to atomic Green function from this excitation
C the prefactor (N-n)/N comes because combinatorics we are defined as C(n,N) while
C they should be C(n,N-1) for excitation at a given sublevel.
	IF(ICFRP1.NE.0)THEN
	DO JCFR=1,NCNF0
	DO IMSC=0,NMSC
	ZMSC=WMSC(IMSC)
	IF(ISPIN.EQ.1)
     &GDGN(IMSC,ICFR,IREP,ISPIN)=GDGN(IMSC,ICFR,IREP,ISPIN)+
     &DBLE(LREP(IREP)-IPNF(IREP))/
     &DBLE(LREP(IREP))*COMR(ICFR)*COMR(JCFR)*
     &(XHUB(ICFR,JCFR)+XHUB(ICFRP1,JCFR))/
     &(ZMSC-(EATS(ICFRP1,JCFR)-EATS(ICFR,JCFR)))
	IF(ISPIN.EQ.2)
     &GDGN(IMSC,ICFR,IREP,ISPIN)=GDGN(IMSC,ICFR,IREP,ISPIN)+
     &DBLE(LREP(IREP)-IPNF(IREP))/
     &DBLE(LREP(IREP))*COMR(JCFR)*COMR(ICFR)*
     &(XHUB(JCFR,ICFR)+XHUB(JCFR,ICFRP1))/
     &(ZMSC-(EATS(JCFR,ICFRP1)-EATS(JCFR,ICFR)))
	ENDDO
	ENDDO
	ENDIF
	ENDDO
	ENDDO
	ENDDO                  !!! over sublevels
	ENDDO                  !!! over spins
C  SPECTRAL FUNCTIONS
	DO ISPIN=1,NSPIN       !!! over spins
	DO IREP=1,NREP         !!! over representations
C  Atomic Green function
	DO IMSC=0,NMSC
	GAT(IMSC,IREP,ISPIN)=(0.D0,0.D0)
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	GAT(IMSC,IREP,ISPIN)=GAT(IMSC,IREP,ISPIN)+
     &GDGN(IMSC,ICFR,IREP,ISPIN)
	ENDDO
	ENDDO
	ENDDO
C  Self energy
	DO IMSC=0,NMSC
	ZMSC=WMSC(IMSC)
      SIG(IMSC,IREP,ISPIN)=ZMSC-ELOC(IREP,ISPIN)-
     &1.D0/GAT(IMSC,IREP,ISPIN)
	ENDDO
C  Physical Green function
	DO IMSC=0,NMSC
	ZMSC=WMSC(IMSC)
      GFF(IMSC,IREP,ISPIN)=
     &1.D0/(ZMSC-ELOC(IREP,ISPIN)-
     &DEL(IMSC,IREP,ISPIN)-SIG(IMSC,IREP,ISPIN))
	ENDDO
	ENDDO                  !!! over representations
	ENDDO                  !!! over spins
C Estimate charge from Green's function
	DO ISPIN=1,NSPIN
	ZSUM(ISPIN)=0.D0
	DO IREP=1,NREP
	ZIMP(IREP,ISPIN)=0.D0
	DO IMSC=0,NMSC
	IF(ABS(WMSC(IMSC)).LE.1.D-14)CYCLE
      ZIMP(IREP,ISPIN)=ZIMP(IREP,ISPIN)+
     &2.D0*TEMP*DREAL(GFF(IMSC,IREP,ISPIN)-
     &1.D0/(WMSC(IMSC)))*FMSC(IMSC)
	ENDDO
	ZIMP(IREP,ISPIN)=ZIMP(IREP,ISPIN)+0.5D0
	ZIMP(IREP,ISPIN)=ZIMP(IREP,ISPIN)*DBLE(LREP(IREP))
	ZSUM(ISPIN)=ZSUM(ISPIN)+ZIMP(IREP,ISPIN)
	ENDDO
	ZSUM(ISPIN)=ZSUM(ISPIN)/2.D0*DBLE(NSPIN)
	ENDDO
C Print
	IF(NSPIN.EQ.1)THEN
	ICFR=0
	WRITE(IUN,10210)
	DO IDEG=0,NDEG
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	ICNF=IPCF(ICFR) ! find out the configuration in first numbering scheme
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF) 
	TCTRL=LIB_TDP(COMR(ICFR)*COMR(1)*XHUB(ICFR,1),1.D-10)
	TCTRM=LIB_TDP(EATS(ICFR,1),1.D-10)
	WRITE(IUN,10220)IFIX(ICFR,1),TCTRL,
     &IDEG,ICFG,TCTRM,(IPNF(IREP),IREP=1,NREP)
	ENDDO
	ENDDO
10210	FORMAT('!f',5X,'<X>',4X,'Nel',1X,'Conf',4X,'<E>')
10220	FORMAT(1X,I1,1X,A10,I3,1X,I3,2X,A10,14(1X,I3))
	ELSEIF(NSPIN.EQ.2)THEN
	ICFR=0
	WRITE(IUN,10230)
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	ICNF=IPCF(ICFR) ! find out the configuration in first numbering scheme
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	ICFR1=0
	DO IDEG1=0,NDEG/NSPIN
	DO ICFG1=1,LCFG(IDEG1)
	ICFR1=ICFR1+1
	ICNF1=IPCF(ICFR1) ! find out the configuration in first numbering scheme
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF1,NCNF,IPNFP1)
	TCTRL=LIB_TDP(COMR(ICFR)*COMR(ICFR1)*XHUB(ICFR,ICFR1),1.D-10)
	TCTRM=LIB_TDP(EATS(ICFR,ICFR1),1.D-10)
	WRITE(IUN,10240)IFIX(ICFR,ICFR1),TCTRL,
     &IDEG,IDEG1,ICFG,ICFG1,TCTRM,
     &(IPNF(IREP),IREP=1,NREP),(IPNFP1(IREP),IREP=1,NREP)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
10230	FORMAT('!f',6X,'<X>',2X,'N_up N_dn C_up C_dn',5X,
     &       '<E>',6X,'Sublevels')
10240	FORMAT(1X,I1,1X,A10,I3,2X,I3,2X,I3,2X,I3,2X,A10,28(1X,I3))
	ENDIF
C Print
	DO ISPIN=1,NSPIN
	DO IREP=1,NREP
	WRITE(IUN,10250)IREP,ISPIN,ELOC(IREP,ISPIN)
	WRITE(IUN,10260)DREAL(SIG(0,IREP,ISPIN)),
     &DREAL(SIG(NMSC,IREP,ISPIN)),ZREP(IREP,ISPIN),ZIMP(IREP,ISPIN)
	ENDDO
	ENDDO
10250 FORMAT(1X,'Results for Rep, Spin = ',I3,1X,I3,' Eloc = ',G12.5)
10260	FORMAT(1X,'Sig0= ',G12.5,' SigI= ',G12.5,
     &         ' Zrep= ',G12.5,' Zimp= ',G12.5)
	DO ISPIN=1,NSPIN
	IF(NSPIN.EQ.2)
     &WRITE(IUN,10270)ISPIN
	IF(NSPIN.EQ.1)
     &WRITE(IUN,10275)
	WRITE(IUN,10280)ZHUB(ISPIN),ZSUM(ISPIN)*2.D0/DBLE(NSPIN),
     &                ZTOT(ISPIN)*2.D0/DBLE(NSPIN)
	ENDDO
10270 FORMAT(1X,'Results for Charges, Spin = ',I3)
10275 FORMAT(1X,'Results for Charges')
10280 FORMAT(1X,'Zhub= ',G12.5,' Zsum= ',G12.5,' Ztot= ',G12.5)
C MEMORY DEALLOCATIONS
	DEALLOCATE(IPNFP1,GAT,GDGN,ZHUB,ZTOT,ZSUM,ZLEV)
	DEALLOCATE(MREP,IPNF,LCFG,IPCF,IPRF)
	DEALLOCATE(COMR,COMB,XSTA)
	END










	SUBROUTINE IMP_CRFHUB1_REAL(NDEG,NREP,NSPIN,LREP,NCNF,NCNF0,
     &           EATS,TPAR,UPAR,GPAR,ELOC,ZMSC,DEL,GFF,SIG,
     &           XHUB,YHUB,IERR)	
****************************************************************
* Solves Anderson impurity model, returns Greens function      *
* and Self-energy                                              *
****************************************************************
      USE COM_UNIT    ,ONLY: IUN,IUT,IUR,IUI            !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
C PASSED ARRAYS
	INTEGER LREP(NREP)
	DIMENSION ELOC(NREP,NSPIN),XHUB(NCNF,NCNF0),YHUB(NCNF,NCNF0)
     &         ,EATS(NCNF,NCNF0)
	COMPLEX*16 DEL(NREP,NSPIN),GFF(NREP,NSPIN),SIG(NREP,NSPIN),ZMSC
C ALLOCATABLE ARRAYS
	INTEGER, ALLOCATABLE :: MREP(:)
	INTEGER, ALLOCATABLE :: IPNF(:)
	INTEGER, ALLOCATABLE :: LCFG(:)
	INTEGER, ALLOCATABLE :: IPCF(:)
	INTEGER, ALLOCATABLE :: IPRF(:)
	INTEGER, ALLOCATABLE :: IPNF0(:)
	INTEGER, ALLOCATABLE :: IPNFP1(:)
	REAL*8, ALLOCATABLE :: COMR(:)
	REAL*8, ALLOCATABLE :: COMB(:,:)
	COMPLEX*16, ALLOCATABLE :: GAT(:,:)
	COMPLEX*16, ALLOCATABLE :: GDGN(:,:,:)
	COMPLEX*16 DIM
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/,EPS/1.D-05/
C CONFIGURATIONS, Minimum: NDEG/2+1, Maximum 2**(NDEG/2)
	ALLOCATE(MREP(0:NREP))
	MREP(0)=1
	DO IREP=1,NREP
	MREP(IREP)=MREP(IREP-1)*(LREP(IREP)+1)
	ENDDO
C SORT BY NUMBERS OF ELECTRONS
	ALLOCATE(IPNF(NREP))
	ALLOCATE(IPNF0(NREP))
	ALLOCATE(IPCF(NCNF))
	ALLOCATE(IPRF(NCNF))
	ALLOCATE(LCFG(0:NDEG/NSPIN))
C Looping over total allowed numbers of electrons in all sublevels
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	LCFG(IDEG)=0
C Configurations are numbered in one way from 1 to NCNF.
C Start looping over them
	DO ICNF=1,NCNF
C Find out numbers of electrons at each sublevel for given configuration ICNF
C Return array is IPNF(IREP) for spin channel
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	NELC=0 ! count # of electrons for given configuration
	DO IREP=1,NREP
	NELC=NELC+IPNF(IREP)
	ENDDO
C if this configuration contains right number of electrons, count it
	IF(NELC.NE.IDEG)CYCLE 
	LCFG(IDEG)=LCFG(IDEG)+1
	ICFR=ICFR+1
C this pointer relates second-to-first numbering schemes for configurations
	IPCF(ICFR)=ICNF 
	ENDDO
	ENDDO
C CHECK ALGORITHM
	ICFR=0 ! second way of numbering configurations
C looping over allowed total numbers of electrons of the same spin
	DO IDEG=0,NDEG/NSPIN
C looping over configurations within given number of electrons of the same spin
	DO ICFG=1,LCFG(IDEG) 
	ICFR=ICFR+1
	ICNF=IPCF(ICFR) ! find out the configuration in first numbering scheme
C Find out array showing how many electrons in each sublevel
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF) 
	NELC=0 ! count # of electrons
	DO IREP=1,NREP
	IF(IPNF(IREP).LT.0.OR.IPNF(IREP).GT.LREP(IREP))
     &CALL FULLSTOP('Wrong configuration')
	NELC=NELC+IPNF(IREP)
	ENDDO
	ENDDO
	IF(NELC.NE.IDEG)CALL FULLSTOP('# of electrons mismatch')
	ENDDO
C Set reciprocal pointer which related first-to-second numbering scheme for configirations
	DO ICNF=1,NCNF 
	IPRF(ICNF)=-1
	ENDDO
	DO ICFR=1,NCNF
	ICNF=IPCF(ICFR)
	IPRF(ICNF)=ICFR
	ENDDO
	DO ICNF=1,NCNF
	IF(IPRF(ICNF).EQ.-1)CALL FULLSTOP('Wrong reciprocal pointer')
	ENDDO
C *******************************************
C Compute product of all combinatorics using second scheme of 
C numbering configurations
	ALLOCATE(COMR(NCNF))
	ALLOCATE(COMB(0:NDEG,0:NDEG))
	CALL LIB_GETCOMB(COMB,NDEG)
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	ICNF=IPCF(ICFR)
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	COMR(ICFR)=1.D0 
	DO IREP=1,NREP
	COMR(ICFR)=COMR(ICFR)*COMB(IPNF(IREP),LREP(IREP))
	ENDDO
	ENDDO
	ENDDO
C CHECK COMBINATORICS
	SUS=0.D0
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	SUM=0.D0
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	SUM=SUM+COMR(ICFR)
	SUS=SUS+COMR(ICFR)
	ENDDO
	IF(ABS(SUM-COMB(IDEG,NDEG/NSPIN)).GT.1.D-07)
     &CALL FULLSTOP('Combinatorics mismatch')
	ENDDO
	IF(ABS(SUS-2.D0**(NDEG/NSPIN)).GT.1.D-07)
     &CALL FULLSTOP('Combinatorics mismatch')
C *************************************
C Many-body atomic levels EATS numbered using second scheme of numbering configurations
	IF(NSPIN.EQ.2.AND.NCNF.NE.NCNF0)CALL FULLSTOP('nCnf.ne.nCnf0')
	IF(NSPIN.EQ.1.AND.NCNF0.NE.1)CALL FULLSTOP('nCnf0 should be 1')
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	ICNF=IPCF(ICFR)
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
	ICFR0=0
	DO IDEG0=0,NDEG/NSPIN
	DO ICFG0=1,LCFG(IDEG0)
	ICFR0=ICFR0+1
	IF(NSPIN.EQ.1.AND.ICFR0.GT.1)CYCLE
	ICNF0=IPCF(ICFR0)
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF0,NCNF,IPNF0)
	EATS(ICFR,ICFR0)=0.D0
	NELC =0
	NELC0=0
	DO IREP=1,NREP
	IF(NSPIN.EQ.1.AND.IPNF0(IREP).NE.0)CALL FULLSTOP('Ipnf0.ne.0')
	EATS(ICFR,ICFR0)=EATS(ICFR,ICFR0)+
     &ELOC(IREP,1)*DBLE(IPNF(IREP))+ELOC(IREP,NSPIN)*DBLE(IPNF0(IREP))
	NELC =NELC +IPNF(IREP)
	NELC0=NELC0+IPNF0(IREP)
	ENDDO
	IF(NELC .NE.IDEG )CALL FULLSTOP('# of electrons mismatch')
	IF(NELC0.NE.IDEG0)CALL FULLSTOP('# of electrons mismatch')
	ITOT=IDEG+IDEG0 ! total number of electrons
	IF(NSPIN.EQ.1)DIUP=DBLE(ITOT)/2.D0
	IF(NSPIN.EQ.1)DIDN=DBLE(ITOT)/2.D0
	IF(NSPIN.EQ.2)DIUP=DBLE(IDEG)
	IF(NSPIN.EQ.2)DIDN=DBLE(IDEG0)
	EATS(ICFR,ICFR0)=EATS(ICFR,ICFR0)+
     &UPAR*DBLE(ITOT*(ITOT-1))/2.D0-
     &GPAR*    (DIUP*(DIUP-1))/2.D0-
     &GPAR*    (DIDN*(DIDN-1))/2.D0
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C ***************************************************************************************
C MEMORY ALLOCATIONS
	ALLOCATE(IPNFP1(NREP))
	ALLOCATE(GDGN(NCNF,NREP,NSPIN))
	ALLOCATE(GAT(NREP,NSPIN))
	IERR=0
C Set atomic G22
	DO ISPIN=1,NSPIN            ! looping over spins
	DO IREP=1,NREP              ! looping over sublevels
	ICFR=0
	DO IDEG=0,NDEG/NSPIN        ! looping over all configurations
	DO ICFG=1,LCFG(IDEG)        ! using second numbering scheme
	ICFR=ICFR+1
      GDGN(ICFR,IREP,ISPIN)=(0.D0,0.D0)
	ICNF=IPCF(ICFR)
C Find real occupancies for each sublevel within given configuration
	CALL LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,IPNF)
C Increase occupancy of sublevel IREP by 1
	DO IREP1=1,NREP
	IPNFP1(IREP1)=IPNF(IREP1)
	ENDDO
	IPNFP1(IREP)=IPNFP1(IREP)+1
C Find out corresponding configuration number in the first numbering scheme
	CALL LIB_SETCONF(NREP,LREP,MREP,ICNFP1,NCNF,IPNFP1)
	ICFRP1=0 ! try to understand that we did not exceed the total allowed 
	IF(IPNF(IREP).NE.LREP(IREP))ICFRP1=IPRF(ICNFP1) ! number of electrons
C Set contribution to atomic Green function from this excitation
C the prefactor (N-n)/N comes because combinatorics we are defined as C(n,N) while
C they should be C(n,N-1) for excitation at a given sublevel.
	IF(ICFRP1.NE.0)THEN
	DO JCFR=1,NCNF0
	IF(ISPIN.EQ.1)
     &GDGN(ICFR,IREP,ISPIN)=GDGN(ICFR,IREP,ISPIN)+
     &DBLE(LREP(IREP)-IPNF(IREP))/
     &DBLE(LREP(IREP))*COMR(ICFR)*COMR(JCFR)*
     &(XHUB(ICFR,JCFR)+XHUB(ICFRP1,JCFR))/
     &(ZMSC-(EATS(ICFRP1,JCFR)-EATS(ICFR,JCFR)))
	IF(ISPIN.EQ.2)
     &GDGN(ICFR,IREP,ISPIN)=GDGN(ICFR,IREP,ISPIN)+
     &DBLE(LREP(IREP)-IPNF(IREP))/
     &DBLE(LREP(IREP))*COMR(JCFR)*COMR(ICFR)*
     &(XHUB(JCFR,ICFR)+XHUB(JCFR,ICFRP1))/
     &(ZMSC-(EATS(JCFR,ICFRP1)-EATS(JCFR,ICFR)))
	ENDDO
	ENDIF
	ENDDO
	ENDDO
	ENDDO                  !!! over sublevels
	ENDDO                  !!! over spins
C  SPECTRAL FUNCTIONS
	DO ISPIN=1,NSPIN       !!! over spins
	DO IREP=1,NREP         !!! over representations
C  Atomic Green function
	GAT(IREP,ISPIN)=(0.D0,0.D0)
	ICFR=0
	DO IDEG=0,NDEG/NSPIN
	DO ICFG=1,LCFG(IDEG)
	ICFR=ICFR+1
	GAT(IREP,ISPIN)=GAT(IREP,ISPIN)+GDGN(ICFR,IREP,ISPIN)
	ENDDO
	ENDDO
C  Self energy
      SIG(IREP,ISPIN)=ZMSC-ELOC(IREP,ISPIN)-1.D0/GAT(IREP,ISPIN)
C  Physical Green function
      GFF(IREP,ISPIN)=
     &1.D0/(ZMSC-ELOC(IREP,ISPIN)-DEL(IREP,ISPIN)-SIG(IREP,ISPIN))
	ENDDO                  !!! over representations
	ENDDO                  !!! over spins
C MEMORY DEALLOCATIONS
	DEALLOCATE(IPNFP1,GAT,GDGN)
	DEALLOCATE(MREP,IPNF,IPNF0,LCFG,IPCF,IPRF)
	DEALLOCATE(COMR,COMB)
	END






 





	SUBROUTINE IMP_CRFHUB1_XNOT(NCNF,NCNF0,EATS,TEMP,COMR,XHUB)
*******************************************************
*  Finds correlational functions for an atom          *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION EATS(NCNF,NCNF0),COMR(NCNF)
	INTEGER, ALLOCATABLE :: IATM(:)
	REAL*8, ALLOCATABLE :: EATM(:)
	REAL*8, ALLOCATABLE :: XATM(:)
	REAL*8 XHUB(NCNF,NCNF0)
	DATA EPS/1.D-05/
C Order atomic states in increasing order
	ALLOCATE(EATM(NCNF*NCNF0))
	ALLOCATE(XATM(NCNF*NCNF0))
	ALLOCATE(IATM(NCNF*NCNF0))
	DO ICNF=1,NCNF
	DO JCNF=1,NCNF0
	EATM(ICNF+NCNF*(JCNF-1))=EATS(ICNF,JCNF)
	IATM(ICNF+NCNF*(JCNF-1))=ICNF+NCNF*(JCNF-1)
	ENDDO
	ENDDO
      DO ISTR=1,NCNF*NCNF0
      DO JSTR=ISTR+1,NCNF*NCNF0
	IF(EATM(ISTR).GT.EATM(JSTR))THEN
	S=EATM(ISTR)
	EATM(ISTR)=EATM(JSTR)
	EATM(JSTR)=S
	I=IATM(ISTR)
	IATM(ISTR)=IATM(JSTR)
	IATM(JSTR)=I
	ENDIF
	ENDDO
	ENDDO
C In this ordering scheme we can loop forward only
	IF(TEMP.GT.0.D0)THEN
      XATM(1)=1.D0
      DO ISTR=1,NCNF*NCNF0-1
      E=EATM(ISTR+1)-EATM(ISTR)
      XATM(ISTR+1)=SAFEEXP(-E/TEMP)*XATM(ISTR)
      ENDDO
C If temperature is zero, only first states survive
	ELSEIF(TEMP.EQ.0.D0)THEN
      XATM(1)=1.D0
      DO ISTR=1,NCNF*NCNF0-1
      E=EATM(ISTR+1)-EATM(ISTR)
      IF(ABS(E).LE.EPS)XATM(ISTR+1)=XATM(ISTR)
	IF(ABS(E).GT.EPS)XATM(ISTR+1)=0.D0
      ENDDO
	ENDIF
C Reorder
	DO ISTR=1,NCNF*NCNF0
	JCNF=(IATM(ISTR)-1)/NCNF+1
	ICNF=IATM(ISTR)-NCNF*(JCNF-1)
	XHUB(ICNF,JCNF)=XATM(ISTR)
	ENDDO
C Renormalize 
      XSUM=0.D0
      DO ICNF=1,NCNF
      DO JCNF=1,NCNF0
      XSUM=XSUM+XHUB(ICNF,JCNF)*COMR(ICNF)*COMR(JCNF)
      ENDDO
	ENDDO
      DO ICNF=1,NCNF
      DO JCNF=1,NCNF0
      XHUB(ICNF,JCNF)=XHUB(ICNF,JCNF)/XSUM
      ENDDO
      ENDDO
C Check
	XSUM=0.D0
	DO ICNF=1,NCNF
	DO JCNF=1,NCNF0
	XSUM=XSUM+XHUB(ICNF,JCNF)*COMR(ICNF)*COMR(JCNF)
	ENDDO
	ENDDO
	IF(ABS(XSUM-1.D0).GT.1.D-10)
     &CALL FULLSTOP('<IMP_CRFHUB1_XNOT>:: Normalization error')
	DEALLOCATE(EATM,XATM,IATM)
	RETURN
	END






