       subroutine IMP_QMCFFT(rindata,coutdata,Temp,L,Iwmax)
C========+=========+=========+=========+=========+=========+=========+=$
C       PROGRAM: nfourier
C       TYPE   : subroutine
C       PURPOSE: fourier-transform the natural-spline interpolation
C                of function Green(tau) 
C                calculate function Green(omega)
C       I/O    :
C       VERSION: 2-16-92
C                29-Nov-95 removal of minimal bug concerning 
C                          dimension of rindata
C       COMMENT: cf J. Stoer R. Bulirsch, Introduction to numerical
C                analysis (Springer, New York, 1980)
C========+=========+=========+=========+=========+=========+=========+=$
       implicit real*8 (a-h,o-z)
       dimension rindata(L)
	 complex*16 coutdata(0:Iwmax)
	 real*8, allocatable :: rincopy(:)
	 real*8, allocatable :: a(:)
	 real*8, allocatable :: b(:)
	 real*8, allocatable :: c(:)
	 real*8, allocatable :: d(:)
	 real*8, allocatable :: u(:)
	 real*8, allocatable :: q(:)
	 real*8, allocatable :: xm(:)
       complex*16 xi,cdummy,explus,ex,ci
       data ci/(0.d0,1.d0)/,pi/3.1415926535898d0/
     &     ,zero/0.d0/,one/1.d0/,two/2.d0/
c memory allocations
       allocate(rincopy(L+1))
	 allocate(a(L))
	 allocate(b(L))
	 allocate(c(L))
       allocate(d(L))
	 allocate(u(L+1))
	 allocate(q(L+1))
	 allocate(xm(L+1))
c start code:
	 Beta=1.d0/Temp
       xi=cmplx(Zero,One)
       xpi=acos(-One)
       delta=Beta/L
       do i=1,L
         rincopy(i)=rindata(i)
       end do
       rincopy(L+1)=-1-rindata(1)
       three=Two+One
       six=2*three
cc
cc     spline interpolation:  the spline is given by
cc     G(tau)=a(i) + b(i) (tau-tau_i) + c(i) ( )^2 + d(i) ( )^3
cc     The following formulas are taken directly from  Stoer and
cc     Bulirsch p. 102
cc
       q(1)=Zero
       u(1)=Zero
       do k=2,L
          p=q(k-1)/Two+Two
          q(k)=-One/Two/p
          u(k)=three/delta**2*(rincopy(k+1)+rincopy(k-1)-Two*rincopy(k))
          u(k)=(u(k)-u(k-1)/Two)/p
       end do
       XM(L+1)=0
       do k=L,1,-1
          XM(k)=q(k)*XM(k+1)+u(k)
       end do
cc
cc     The following formulas are taken directly from  Stoer and
cc     Bulirsch p. 98
cc
       do j = 1, L
          a(j)=rincopy(j)
          c(j)= XM(j)/Two
          b(j)=(rincopy(j+1)-rincopy(j))/delta - 
     &       (Two*XM(j)+XM(j+1))*delta/6.
          d(j)=(XM(j+1)-XM(j))/(6.*delta)
       end do
cc
cc     The Spline multiplied by the exponential can now be exlicitely
cc     integrated. The following formulas were obtained using
cc     MATHEMATICA
cc
        do i=0,Iwmax
           om=(Two*i+One)*xpi/Beta
           coutdata(i)=Zero
           do j=1,L
              cdummy=xi*om*delta*j
              explus=exp(cdummy)
              cdummy=xi*om*delta*(j-1)
              ex=exp(cdummy)
              coutdata(i)=coutdata(i) + explus*(
     &         ( -six* d(j) )/om**4 + 
     &         ( Two*xi*c(j) + six*delta*xi*d(j)  )/om**3 +
     &         ( b(j)+ Two*delta*c(j)+ three*delta**2*d(j) )/om**2 +
     &         (- xi*a(j) - delta*xi*b(j) - delta**2*xi*c(j) -
     &         delta**3*xi*d(j))/om)
 
              coutdata(i) = coutdata(i) + ex*(
     &        six*d(j)/om**4 - Two*xi*c(j)/om**3 
     &        -b(j)/om**2 + xi*a(j)/om)
           end do
        end do
c memory deallocations
	  deallocate(rincopy,a,b,c,d,u,q,xm)
        end



        subroutine IMP_QMCFFI(cindata,routdata,Temp,L,Iwmax)
C========+=========+=========+=========+=========+=========+=========+=$
C       PROGRAM: invfourier
C       TYPE   : subroutine
C       PURPOSE: inverse fourier transform
C                Greent, Greenw use physical definition
C                Greent(i)=G((i-1)*deltau) for i=1,...,L
C                Greenw(n)=G(i w_n), for n=0,L/2-1
C                       w_n=(2*n+1)pi/beta
C                Symmetry property: 
C                G(iw_(-n)=G(iw_(n-1))*
C                coupled to the impurity
C       I/O    :
C       VERSION: 6-16-92
C       COMMENT: 
C========+=========+=========+=========+=========+=========+=========+=$
       implicit real*8 (a-h,o-z)
       dimension routdata(L)
       complex*16 cindata(0:Iwmax),cdummy,ci
       data ci/(0.d0,1.d0)/,pi/3.1415926535898d0/
     &     ,zero/0.d0/,one/1.d0/,two/2.d0/
c
	 Beta=1.d0/Temp
       xpi=acos(-One) 
       do 1 i=1,L
       routdata(i)=Zero
          tau=(i-1)*beta/real(L)
          do 2 j=0,Iwmax
               om=mod((2*j+One)*xpi/Beta*tau,2*xpi)
               cdummy=cmplx(Zero,One)*om
               dummy=cindata(j)*exp(-cdummy)
         routdata(i)=routdata(i)+Two/beta*dummy
2         continue
1      continue
cc
cc     special treatment for tau=0
cc
       routdata(1)=-One/Two+routdata(1)
       end


       function wfun(z)
C========+=========+=========+=========+=========+=========+=========+=$
C       PROGRAM: wfun.f
C       TYPE   : subroutine
C       PURPOSE: calculate exponentially  rescaled error function w 
C       I/O    :
C       VERSION: 6-16-92
C       COMMENT: 
C========+=========+=========+=========+=========+=========+=========+=$
       complex*16 z,wfun
       double precision xi,yi,u,v
       logical flag
       xi=dreal(z)
       yi=dimag(z)
       call wofz(xi,yi,u,v,flag)
       wfun=dcmplx(u,v)
       end  



      SUBROUTINE WOFZ (XI, YI, U, V, FLAG)
C========+=========+=========+=========+=========+=========+=========+=$
C      ALGORITHM 680, COLLECTED ALGORITHMS FROM ACM.
C      THIS WORK PUBLISHED IN TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C      VOL. 16, NO. 1, PP. 47.
*
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
*
      LOGICAL A, B, FLAG
      PARAMETER (FACTOR   = 1.12837916709551257388D0,
     *           RMAXREAL = 1.340780792994259D+154,
     *           RMAXEXP  = 709.0895657128241D0,
     *           RMAXGONI = 0.6746518850690209D10)
*
      FLAG = .FALSE.
*
      XABS = DABS(XI)
      YABS = DABS(YI)
      X    = XABS/6.3
      Y    = YABS/4.4
*
C
C     THE FOLLOWING IF-STATEMENT PROTECTS
C     QRHO = (X**2 + Y**2) AGAINST OVERFLOW
C
      IF ((XABS.GT.RMAXREAL).OR.(YABS.GT.RMAXREAL)) GOTO 100
*
      QRHO = X**2 + Y**2
*
      XABSQ = XABS**2
      XQUAD = XABSQ - YABS**2
      YQUAD = 2*XABS*YABS
*
      A     = QRHO.LT.0.085264D0
*
      IF (A) THEN
C
C  IF (QRHO.LT.0.085264D0) THEN THE FADDEEVA-FUNCTION IS EVALUATED
C  USING A POWER-SERIES (ABRAMOWITZ/STEGUN, EQUATION (7.1.5), P.297)
C  N IS THE MINIMUM NUMBER OF TERMS NEEDED TO OBTAIN THE REQUIRED
C  ACCURACY
C
        QRHO  = (1-0.85*Y)*DSQRT(QRHO)
        N     = IDNINT(6 + 72*QRHO)
        J     = 2*N+1
        XSUM  = 1.0/J
        YSUM  = 0.0D0
        DO 10 I=N, 1, -1
          J    = J - 2
          XAUX = (XSUM*XQUAD - YSUM*YQUAD)/I
          YSUM = (XSUM*YQUAD + YSUM*XQUAD)/I
          XSUM = XAUX + 1.0/J
 10     CONTINUE
        U1   = -FACTOR*(XSUM*YABS + YSUM*XABS) + 1.0
        V1   =  FACTOR*(XSUM*XABS - YSUM*YABS)
        DAUX =  DEXP(-XQUAD)
        U2   =  DAUX*DCOS(YQUAD)
        V2   = -DAUX*DSIN(YQUAD)
*
        U    = U1*U2 - V1*V2
        V    = U1*V2 + V1*U2
*
      ELSE
C
C  IF (QRHO.GT.1.O) THEN W(Z) IS EVALUATED USING THE LAPLACE
C  CONTINUED FRACTION
C  NU IS THE MINIMUM NUMBER OF TERMS NEEDED TO OBTAIN THE REQUIRED
C  ACCURACY
C
C  IF ((QRHO.GT.0.085264D0).AND.(QRHO.LT.1.0)) THEN W(Z) IS EVALUATED
C  BY A TRUNCATED TAYLOR EXPANSION, WHERE THE LAPLACE CONTINUED FRACTION
C  IS USED TO CALCULATE THE DERIVATIVES OF W(Z)
C  KAPN IS THE MINIMUM NUMBER OF TERMS IN THE TAYLOR EXPANSION NEEDED
C  TO OBTAIN THE REQUIRED ACCURACY
C  NU IS THE MINIMUM NUMBER OF TERMS OF THE CONTINUED FRACTION NEEDED
C  TO CALCULATE THE DERIVATIVES WITH THE REQUIRED ACCURACY
C
*
        IF (QRHO.GT.1.0) THEN
          H    = 0.0D0
          KAPN = 0
          QRHO = DSQRT(QRHO)
          NU   = IDINT(3 + (1442/(26*QRHO+77)))
        ELSE
          QRHO = (1-Y)*DSQRT(1-QRHO)
          H    = 1.88*QRHO
          H2   = 2*H
          KAPN = IDNINT(7  + 34*QRHO)
          NU   = IDNINT(16 + 26*QRHO)
        ENDIF
*
        B = (H.GT.0.0)
*
        IF (B) QLAMBDA = H2**KAPN
*
        RX = 0.0
        RY = 0.0
        SX = 0.0
        SY = 0.0
*
        DO 11 N=NU, 0, -1
          NP1 = N + 1
          TX  = YABS + H + NP1*RX
          TY  = XABS - NP1*RY
          C   = 0.5/(TX**2 + TY**2)
          RX  = C*TX
          RY  = C*TY
          IF ((B).AND.(N.LE.KAPN)) THEN
            TX = QLAMBDA + SX
            SX = RX*TX - RY*SY
            SY = RY*TX + RX*SY
            QLAMBDA = QLAMBDA/H2
          ENDIF
 11     CONTINUE
*
        IF (H.EQ.0.0) THEN
          U = FACTOR*RX
          V = FACTOR*RY
        ELSE
          U = FACTOR*SX
          V = FACTOR*SY
        END IF
*
        IF (YABS.EQ.0.0) U = DEXP(-XABS**2)
*
      END IF
*
*
C
C  EVALUATION OF W(Z) IN THE OTHER QUADRANTS
C
*
      IF (YI.LT.0.0) THEN
*
        IF (A) THEN
          U2    = 2*U2
          V2    = 2*V2
        ELSE
          XQUAD =  -XQUAD
*
C
C         THE FOLLOWING IF-STATEMENT PROTECTS 2*EXP(-Z**2)
C         AGAINST OVERFLOW
C
          IF ((YQUAD.GT.RMAXGONI).OR.
     *        (XQUAD.GT.RMAXEXP)) GOTO 100
*
          W1 =  2*DEXP(XQUAD)
          U2  =  W1*DCOS(YQUAD)
          V2  = -W1*DSIN(YQUAD)
        END IF
*
        U = U2 - U
        V = V2 - V
        IF (XI.GT.0.0) V = -V
      ELSE
        IF (XI.LT.0.0) V = -V
      END IF
*
      RETURN
*
  100 FLAG = .TRUE.
      RETURN
*
      END
