	SUBROUTINE IMP_SUNHUB1_STAT(NDEG,EATS,TPAR,UPAR,ELOC,DEL,
     &   GFF,SIG,XHUB,YHUB,IFIX,SIGI,VPAR,WPAR,POLE,OVRL,
     &   ZSUM,ZHUB,KFIX,TEMP,NMSC,WMSC,FMSC,DAMP,IERR)
****************************************************************
* Solves Anderson impurity model, returns Greens function      *
* and Self-energy, general Temperature version with            *
* statitstical weight approximation                            *
****************************************************************
      USE COM_UNIT    ,ONLY: IUN,IUT,IUR,IUI                       !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
C Passed parameters
	INTEGER IFIX(0:NDEG)
	DIMENSION XHUB(0:NDEG),YHUB(0:NDEG),FMSC(0:NMSC),EATS(0:NDEG)
	COMPLEX*16 GFF(0:NMSC+1),SIG(0:NMSC+1),DEL(0:NMSC+1),WMSC(0:NMSC)
C Allocatable arrays
	REAL*8, ALLOCATABLE :: ELEV(:)
	REAL*8, ALLOCATABLE :: XSTA(:)
	REAL*8, ALLOCATABLE :: COMB(:,:)
	COMPLEX*16, ALLOCATABLE :: GAT(:)
	COMPLEX*16, ALLOCATABLE :: GDGN(:,:)
	COMPLEX*16, ALLOCATABLE :: GCNV(:,:)
	COMPLEX*16, ALLOCATABLE :: GCNW(:,:)
C Local variables
	COMPLEX*16 DIM,ZMSC
	CHARACTER HELP*40,TCTRL*10,TCTRM*10,LIB_TDP*10
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/
     &    ,EPS/1.D-06/,NITER/300/,KITER/50/
     &    ,NIBRD/15/,ADMIX/0.35D0/
C MEMORY ALLOCATIONS
	ALLOCATE(XSTA(0:NDEG))
	ALLOCATE(ELEV(0:NDEG))
	ALLOCATE(GDGN(0:NMSC,0:NDEG-1))
	ALLOCATE(GCNV(0:NMSC,0:NDEG-1))
	ALLOCATE(GCNW(0:NMSC,0:NDEG-1))
	ALLOCATE(GAT(0:NMSC))
	ALLOCATE(COMB(0:NDEG,0:NDEG))
C LEVELS WITHIN SU(N)
	EDOP=ELOC+DBLE(NDEG-1)/2.D0*UPAR
	DO IDEG=0,NDEG
	ELEV(IDEG)=(DBLE(IDEG)-DBLE(NDEG-1)/2.D0)*UPAR
	EATS(IDEG)=ELOC*DBLE(IDEG)+UPAR*DBLE(IDEG*(IDEG-1))/2.D0
	ENDDO
	CALL LIB_GETCOMB(COMB,NDEG)
C PRINT INPUT DATA
	WRITE(IUN,10010)
	WRITE(IUN,10020)TEMP
	WRITE(IUN,10030)UPAR
	WRITE(IUN,10070)TPAR
	WRITE(IUN,10080)NDEG
	WRITE(IUN,10090)NMSC
10010	FORMAT(1X,'Solving impurity model:')
10020	FORMAT(1X,'Temp = ',G12.5)
10030	FORMAT(1X,'Upar = ',G12.5)
10050	FORMAT(1X,'Z1-1 = ',G12.5)
10060	FORMAT(1X,'Wkon = ',G12.5)
10070	FORMAT(1X,'Tpar = ',G12.5)
10080	FORMAT(1X,'Ndeg = ',I5)
10090	FORMAT(1X,'Nmsc = ',I5)
	IERR=0
C BUILD CORRELATIONAL FUNCTIONS
      CALL IMP_SUNHUB1_XNOT(NDEG,ELEV,EDOP,TEMP,COMB,XSTA)
      DO IDEG=0,NDEG
	IF(KFIX.EQ.0.AND.IFIX(IDEG).EQ.0)
     &XHUB(IDEG)=XSTA(IDEG)
      ENDDO
C RENORMALIZE
      XSUM=0.D0
	CSUM=0.D0
      DO IDEG=0,NDEG
      XSUM=XSUM+COMB(IDEG,NDEG)*XHUB(IDEG)
      CSUM=CSUM+COMB(IDEG,NDEG)
      ENDDO
	IF(XSUM.LE.0.D0)XSUM=CSUM
      DO IDEG=0,NDEG
      XHUB(IDEG)=XHUB(IDEG)/XSUM
	YHUB(IDEG)=0.D0
      ENDDO
C Find # of electrons in case not known
	ZHUB=0.D0
	DO IDEG=0,NDEG
	ZHUB=ZHUB+DBLE(IDEG)*XHUB(IDEG)*COMB(IDEG,NDEG)
	ENDDO
	ZLEV=ZHUB/DBLE(NDEG)
C  Reset total charge if desired
	ZTOT=ZHUB
C  Set self-energy
	SIGI=UPAR*ZTOT-UPAR*ZLEV
	VPAR=UPAR*SQRT(ABS(ZLEV*(1.D0-ZLEV)))
	WPAR=UPAR*SQRT(ABS(ZLEV*(1.D0-ZLEV)))
	POLE=ELOC+UPAR*ZTOT-UPAR*(1.D0-ZLEV)
	OVRL=1.D0
C Set atomic G22
	DO IMSC=0,NMSC
	ZMSC=WMSC(IMSC)
	DO IDEG=0,NDEG-1
	GDGN(IMSC,IDEG)=COMB(IDEG,NDEG-1)*(XHUB(IDEG)+XHUB(IDEG+1))/
     &    (ZMSC-ELEV(IDEG)-EDOP)
	GCNV(IMSC,IDEG)=GDGN(IMSC,IDEG)
	GCNW(IMSC,IDEG)=GDGN(IMSC,IDEG)
	ENDDO
	ENDDO
C  Atomic Green function
	DO IMSC=0,NMSC
	GAT(IMSC)=(0.D0,0.D0)
	DO IDEG=0,NDEG-1
	GAT(IMSC)=GAT(IMSC)+GCNV(IMSC,IDEG)
	ENDDO
	ENDDO
C Self-energy
	DO IMSC=0,NMSC
	ZMSC=WMSC(IMSC)
      SIG(IMSC)=ZMSC-ELOC-1.D0/GAT(IMSC)
	ENDDO
C  Impurity Green function
	DO IMSC=0,NMSC
	ZMSC=WMSC(IMSC)
      GFF(IMSC)=1.D0/(ZMSC-ELOC-SIG(IMSC)-DEL(IMSC))
	ENDDO
C  Find # of electrons from impurity Green function
	ZSUM=0.D0
	DO IMSC=0,NMSC
	IF(ABS(WMSC(IMSC)).LE.1.D-14)CYCLE
      ZSUM=ZSUM+2.D0*TEMP*DREAL(GFF(IMSC)-1.D0/(WMSC(IMSC)))*
     &FMSC(IMSC)
	ENDDO
	ZSUM=ZSUM+0.5D0
	ZSUM=ZSUM*DBLE(NDEG)
C
	XSUM=0.D0
	WRITE(IUN,10120)
	DO IDEG=0,NDEG
	XSUM=XSUM+XHUB(IDEG)*COMB(IDEG,NDEG)
	TCTRL=LIB_TDP(XHUB(IDEG)*COMB(IDEG,NDEG),1.D-10)
	TCTRM=LIB_TDP(EATS(IDEG),1.D-10)
	WRITE(IUN,10130)IFIX(IDEG),TCTRL,IDEG,TCTRM
	ENDDO
	WRITE(IUN,10135)XSUM
	WRITE(IUN,10140)ZHUB,ZSUM,ZTOT
	WRITE(IUN,10160)ELOC,DREAL(SIG(0)),DREAL(SIG(NMSC))
10120	FORMAT(1X,'f     <Xn>     n     <En>')
10130	FORMAT(1X,I1,1X,A10,1X,I3,1X,A10)
10135	FORMAT(1X,'Xsum =',G12.5)
10140	FORMAT(1X,'Zhub = ',G12.5,'; Zsum = ',G12.5,'; Ztot = ',G12.5)
10160 FORMAT(1X,'Eloc = ',G12.5,'; Sig0 = ',G12.5,'; SigI = ',G12.5)
	DEALLOCATE(ELEV,XSTA,GDGN,GCNV,GCNW,GAT)
	END






	SUBROUTINE IMP_SUNHUB1_XNOT(NDEG,ELEV,EDOP,TEMP,COMB,XHUB)
*******************************************************
*  Finds correlational functions for an atom          *
*  in zero configurational width approximation        *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION ELEV(0:NDEG),COMB(0:NDEG,0:NDEG)
     &   ,XHUB(0:NDEG)
      IDOP=0
      DO IDEG=0,NDEG
      IF(ELEV(IDEG)+EDOP.LE.0.D0)IDOP=IDEG
      IF(ELEV(IDEG)+EDOP.GT.0.D0)EXIT
      ENDDO
C Looping backwards
      XHUB(IDOP)=1.D0
      DO IDEG=IDOP,1,-1
      E=ELEV(IDEG-1)+EDOP
      XHUB(IDEG-1)=SAFEEXP(+E/TEMP)*XHUB(IDEG)
      ENDDO
C Looping forwards
      XHUB(IDOP)=1.D0
      DO IDEG=IDOP,NDEG-1
      E=ELEV(IDEG)+EDOP
      XHUB(IDEG+1)=SAFEEXP(-E/TEMP)*XHUB(IDEG)
      ENDDO
C  Renormalize continiously
      XSUM=0.D0
      DO JDEG=0,NDEG
      XSUM=XSUM+XHUB(JDEG)*COMB(JDEG,NDEG)
      ENDDO
      DO JDEG=0,NDEG
      XHUB(JDEG)=XHUB(JDEG)/XSUM
      ENDDO
C  Check Normalizaton
	XSUM=0.D0
	DO IDEG=0,NDEG
	XSUM=XSUM+XHUB(IDEG)*COMB(IDEG,NDEG)
	ENDDO
	IF(ABS(XSUM-1.D0).GT.1.D-10)STOP
	END



	SUBROUTINE IMP_SUNHUB1_REAL(NDEG,EATS,UPAR,ELOC,ZMSC,
     &           DEL,GFF,SIG,XHUB,YHUB,IERR)
*****************************************************************
*  Solution for arbitrary W if Xhub,Yhub are known              *
*****************************************************************
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION XHUB(0:NDEG),YHUB(0:NDEG),EATS(0:NDEG)
	COMPLEX*16 DEL,GFF,SIG,ZMSC
C  Allocatable arrays
	REAL*8, ALLOCATABLE :: ELEV(:)
	REAL*8, ALLOCATABLE :: COMB(:,:)
	COMPLEX*16, ALLOCATABLE :: GDGN(:)
C  Local variables
	COMPLEX*16 DIM,SGM,GAT
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/
C  Memory allocations
	ALLOCATE(COMB(0:NDEG,0:NDEG))
	ALLOCATE(ELEV(0:NDEG))
	ALLOCATE(GDGN(0:NDEG-1))
C LEVELS WITHIN SU(N)
	EDOP=ELOC+DBLE(NDEG-1)/2.D0*UPAR
	DO IDEG=0,NDEG
	ELEV(IDEG)=(DBLE(IDEG)-DBLE(NDEG-1)/2.D0)*UPAR
	EATS(IDEG)=ELOC*DBLE(IDEG)+UPAR*DBLE(IDEG*(IDEG-1))/2.D0
	ENDDO
	CALL LIB_GETCOMB(COMB,NDEG)
	IERR=0
C Set atomic Green function
	DO IDEG=0,NDEG-1
	GDGN(IDEG)=COMB(IDEG,NDEG-1)*(XHUB(IDEG)+XHUB(IDEG+1))/
     &           (ZMSC-ELEV(IDEG)-EDOP)
	ENDDO
	GAT=(0.D0,0.D0)
	DO IDEG=0,NDEG-1
	GAT=GAT+GDGN(IDEG)
	ENDDO
C Self-energy
	IF(ABS(GAT).LE.1.D-10)SIG=(0.D0,0.D0)
	IF(ABS(GAT).GT.1.D-10)SIG=ZMSC-ELOC-1.D0/GAT
C Impurity Green function
	GFF=1.D0/(ZMSC-ELOC-SIG-DEL)
C  MEMORY DEALLOCATIONS
	DEALLOCATE(COMB,ELEV,GDGN)
	END
