	SUBROUTINE INI_GROUPS(A,COORD,IS,LMM,NATOM,NSORT,NSPIN,NORBS,
     &                      AXMAG,AXEXT,EPS)
***************************************************************
*   Program finds crystalline group by checking whether       *
*   atoms go to each other after applying group operations    *
*   generated by the WIGMAT subroutine. If spin-orbit         *
*   coupled calculation is necessary, only operations         *
*   which donot change z-axe direction are left together with *
*   inversional operations. Only those leave spin-orbit       *
*   coupling operator invariant.                              *
*     Input:                                                  *
*       A - translational vectors in units of lat.par.        *
*       COORD - positions of atoms in units of lat.par.       *
*       IS(IATOM) - number of type for given atom             *
*       LMM - lmax i.e 0:s,1:p,2:d,3:f,etc                    *
*       NATOM  - number of atoms in primitive cell            *
*       NSORT  - number of atoms of different kind            *
*       NSPIN  - number of spins                              *
*       NORBS  - 2 if spin and orbit are coupled.             *
*     Output: (in MODULE GROUPS)                              *
*       NGROUP - number of group elements found               *
*       U - Wigner"s matrices                                 *
*       IP - shows new atom positions after rotation          *
*       SHIFT - shifts if group is non-symmorphic one         *
*       SOPHS - phase shifts of spinor components             *
***************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_GROUPS,  ONLY : NGROUP,INV,KELTAB,IKOV,IP       !!! from groups
     &                       ,RDC,SHIFT,SOPHS,U,UJ,SJ
	USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)

	integer :: fh_grp = 7919        ! Variable added by Gunnar 6.21.2000

	
	INTEGER, ALLOCATABLE :: IPT(:)
	REAL*8, ALLOCATABLE :: COORD1(:,:)
	REAL*8, ALLOCATABLE :: COORD2(:,:)
	REAL*8, ALLOCATABLE :: RDCT(:,:)
	REAL*8, ALLOCATABLE :: PSHIFT(:,:)
	REAL*8, ALLOCATABLE :: FAC(:)
	COMPLEX*16, ALLOCATABLE :: SPHASE(:,:)
	INTEGER IS(NATOM),M3(5)
	DIMENSION A(3,3),COORD(3,NATOM),B(3,3),AXMAG(3,NSORT)
     &    ,ORIGIN(3),PR(3,2),PG(3,2),PL(3,2),AXEXT(3,NSORT)
	COMPLEX*16 YL(-1:1),SOM(2,2),SOU(2,2),TRS(2,2)
	DATA M3/1,2,3,1,2/,PI/3.141592653589793D0/
	DATA NONSYM/1/ ! set to zero if Nopr.le.48 is needed
C MEMORY ALLOCATIONS
	ALLOCATE(IPT(NATOMMEM))
	ALLOCATE(COORD1(3,NATOMMEM))
	ALLOCATE(COORD2(3,NATOMMEM))
	ALLOCATE(PSHIFT(3,NATOMMEM))
	ALLOCATE(RDCT(3,NATOMMEM))
	ALLOCATE(FAC(0:4*LMAXTMEM))
	ALLOCATE(SPHASE(2,NSORTMEM))
      FAC(0)=1.D0
      DO L=1,4*LMAXTMEM
      FAC(L)=FAC(L-1)*DBLE(L)
      ENDDO
	DO I=1,3
	B(1,I)=A(1,I)
	DO J=2,3
	B(J,I)=A(J,I)
	ENDDO
	ENDDO
C  A(*,I) IS I-TH TRANSLATION. B'LL BE RECIPROCAL ONES WITHOUT 2PI
	CALL LIB_DET(B,3,3,OMEGA)
	DO J=1,3
	DO I=1,3
	B(I,J)=(A(M3(I+1),M3(J+1))*A(M3(I+2),M3(J+2))-
     &A(M3(I+2),M3(J+1))*A(M3(I+1),M3(J+2)))/OMEGA
	ENDDO
	ENDDO
C  SHIFT THE ORIGIN TO THE FIRST ATOM
	DO I=1,3
	ORIGIN(I)=COORD(I,1)
	ENDDO
	DO IATOM=1,NATOM
	DO I=1,3
	COORD(I,IATOM)=COORD(I,IATOM)-ORIGIN(I)
	ENDDO
	ENDDO
C  REDUCE TO FIRST CELL
	DO IATOM=1,NATOM
	X1=COORD(1,IATOM)+ORIGIN(1)
	Y1=COORD(2,IATOM)+ORIGIN(2)
	Z1=COORD(3,IATOM)+ORIGIN(3)
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	COORD2(1,IATOM)=ABS(P)
	COORD2(2,IATOM)=ABS(Q)
	COORD2(3,IATOM)=ABS(R)
	X1=COORD(1,IATOM)
	Y1=COORD(2,IATOM)
	Z1=COORD(3,IATOM)
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	COORD1(1,IATOM)=ABS(P)
	COORD1(2,IATOM)=ABS(Q)
	COORD1(3,IATOM)=ABS(R)
	IF(P.LE.1.D-5.AND.Q.LE.1.D-5.AND.R.LE.1.D-5)THEN
	K=0
	DO JATOM=1,NATOM
	IF(IS(JATOM).EQ.IS(IATOM))THEN
	K=K+1
	DO L=1,3
	PSHIFT(L,K)=COORD(L,JATOM)  !!! POSSIBLE SHIFTS
	ENDDO
	ENDIF
	ENDDO
	KS=K
	ENDIF
	ENDDO
C *** CHECK LATTICE FOR THE GROUP OPERATIONS ***
	NG=NGROUP
	DO 3000 IRUN=1,2 !!! RUN TWICE
	IR=0
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
C IS MADE ONLY AT THE SECOND RUN
	IF(IRUN.EQ.2)THEN
	IF(ALLOCATED(IP))DEALLOCATE(IP)
	ALLOCATE(IP(NATOMMEM,NOPRMEM))
	IF(ALLOCATED(IKOV))DEALLOCATE(IKOV)
	ALLOCATE(IKOV(NOPRMEM))
	IF(ALLOCATED(RDC))DEALLOCATE(RDC)
	ALLOCATE(RDC(3,NATOMMEM,NOPRMEM))
	IF(ALLOCATED(SHIFT))DEALLOCATE(SHIFT)
	ALLOCATE(SHIFT(3,NOPRMEM))
	IF(ALLOCATED(SOPHS))DEALLOCATE(SOPHS)
	ALLOCATE(SOPHS(2,NOPRMEM))
	IF(ALLOCATED(SJ))DEALLOCATE(SJ)
	ALLOCATE(SJ(2,2,NOPRMEM))
	ENDIF
	DO 2000 IG=1,NG  !!! OVER KOVALEV ROTATIONS
C ROTATE THE ORIGIN
	CALL LIB_ROTATE(ORIGIN(1),ORIGIN(2),ORIGIN(3),OX,OY,OZ,U(2,IG),2)
C 1) CHECK LATTICE TRANSLATIONS
	ILAT=0
	DO I=1,3
	CALL LIB_ROTATE(A(1,I),A(2,I),A(3,I),X1,Y1,Z1,U(2,IG),2)
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	IATOM=1 ! atom in the origin is always the first one.
	CALL GROUPS_CHECK(P,Q,R,COORD1,IPATOM,NATOM,IS,IATOM,EPS)
	IF(IPATOM.NE.0)ILAT=ILAT+1
	ENDDO
	IF(ILAT.LT.3)GOTO2000
C 2) SELECT OPERATIONS WHICH LEAVE SPIN-ORBIT OPERATOR INVARIANT
	IAXIS=0
	IF(NORBS.EQ.1)THEN
	IAXIS=1
	ELSEIF(NORBS.EQ.2)THEN
	DO ISORT=1,NSORT                !!! over sorts
C Set matrix Sigma*Axis
	IAXIS=1
	R=SQRT(AXMAG(1,ISORT)**2+AXMAG(2,ISORT)**2+AXMAG(3,ISORT)**2)
	IF(R.GT.1.D-10)THEN
	CALL LIB_ROTAXES(AXMAG(1,ISORT),TETA,PFI)
	SOM(1,1)=+DCMPLX(COS(TETA),0.D0)
	SOM(2,1)=+SIN(TETA)*EXP((0.D0,+1.D0)*PFI)
	SOM(1,2)=+SIN(TETA)*EXP((0.D0,-1.D0)*PFI)
	SOM(2,2)=-DCMPLX(COS(TETA),0.D0)
C Apply unitary transformation
	DO I=1,2
	DO J=1,2
	SOU(I,J)=(0.D0,0.D0)
	DO M1=1,2
	DO M2=1,2
	SOU(I,J)=SOU(I,J)+DCONJG(UJ(M1,I,IG))*SOM(M1,M2)*UJ(M2,J,IG)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	DO I=1,2
	DO J=1,2
	IF(ABS(SOU(I,J)-SOM(I,J)).GT.1.D-04)IAXIS=0
	ENDDO
	ENDDO
	ENDIF
	IF(IAXIS.EQ.0)EXIT
C Set matrix Sigma*Axis
	IAXIS=1
	R=SQRT(AXEXT(1,ISORT)**2+AXEXT(2,ISORT)**2+AXEXT(3,ISORT)**2)
	IF(R.GT.1.D-10)THEN
	CALL LIB_ROTAXES(AXEXT(1,ISORT),TETA,PFI)
	SOM(1,1)=+DCMPLX(COS(TETA),0.D0)
	SOM(2,1)=+SIN(TETA)*EXP((0.D0,+1.D0)*PFI)
	SOM(1,2)=+SIN(TETA)*EXP((0.D0,-1.D0)*PFI)
	SOM(2,2)=-DCMPLX(COS(TETA),0.D0)
C Apply unitary transformation
	DO I=1,2
	DO J=1,2
	SOU(I,J)=(0.D0,0.D0)
	DO M1=1,2
	DO M2=1,2
	SOU(I,J)=SOU(I,J)+DCONJG(UJ(M1,I,IG))*SOM(M1,M2)*UJ(M2,J,IG)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	DO I=1,2
	DO J=1,2
	IF(ABS(SOU(I,J)-SOM(I,J)).GT.1.D-04)IAXIS=0
	ENDDO
	ENDDO
	ENDIF
	IF(IAXIS.EQ.0)EXIT
C Set transformation matrix, bad choice for teta=phi=0 !!!
C	TRS(1,1)=+COS(0.5D0*TETA)
C	TRS(1,2)=+SIN(0.5D0*TETA)*EXP((0.D0,-1.D0)*PFI)
C	TRS(2,1)=+SIN(0.5D0*TETA)*EXP((0.D0,+1.D0)*PFI)
C	TRS(2,2)=-COS(0.5D0*TETA)
C Set transformation matrix
	TRS(1,1)=+DCMPLX(COS(0.5D0*TETA),0.D0)
	TRS(1,2)=-SIN(0.5D0*TETA)*EXP((0.D0,-1.D0)*PFI)
	TRS(2,1)=+SIN(0.5D0*TETA)*EXP((0.D0,+1.D0)*PFI)
	TRS(2,2)=+DCMPLX(COS(0.5D0*TETA),0.D0)
C  Check that the Matrix Trm*Uj*TrmX should be diagonal for this group element
	DO I=1,2
	DO J=1,2
	SOM(I,J)=(0.D0,0.D0)
	DO M1=1,2
	DO M2=1,2
C	SOM(I,J)=SOM(I,J)+TRS(I,M1)*UJ(M1,M2,IG)*DCONJG(TRS(J,M2))
	SOM(I,J)=SOM(I,J)+DCONJG(TRS(M1,I))*UJ(M1,M2,IG)*TRS(M2,J)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IERR1=0
	IF(ABS(SOM(1,2)).GT.1.D-05)IERR1=1
	IF(ABS(SOM(2,1)).GT.1.D-05)IERR1=1
	SPHASE(1,ISORT)=SOM(1,1)
	SPHASE(2,ISORT)=SOM(2,2)
	IF(IERR1.EQ.1.AND.IAXIS.EQ.1)
     &CALL FULLSTOP('<GROUPS> :: Error in SO element selection')
	ENDDO                     !!! over sorts
	ENDIF
 	IF(IAXIS.EQ.0)GOTO2000
C 3) OPERATIONS WITH THE ATOMS IN BASIS
	DO 1000 K=1,KS
	DX=PSHIFT(1,K)
	DY=PSHIFT(2,K)
	DZ=PSHIFT(3,K)
	CALL LIB_ROTATE(DX,DY,DZ,X2,Y2,Z2,U(2,IG),1)
	DO IATOM=1,NATOM
	X=COORD(1,IATOM)
	Y=COORD(2,IATOM)
	Z=COORD(3,IATOM)
	CALL LIB_ROTATE(X,Y,Z,X1,Y1,Z1,U(2,IG),1)
	X1=X1-X2
	Y1=Y1-Y2
	Z1=Z1-Z2
C  REDUCE TO FIRST CELL
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	CALL GROUPS_CHECK(P,Q,R,COORD1,IPATOM,NATOM,IS,IATOM,EPS)
	IF(IPATOM.EQ.0)GOTO1000
	IPT(IATOM)=IPATOM
	RDCT(1,IATOM)=X1-COORD(1,IPATOM)
	RDCT(2,IATOM)=Y1-COORD(2,IPATOM)
	RDCT(3,IATOM)=Z1-COORD(3,IPATOM)
	ENDDO
C  CHECKING IS DONE, SET UP OPERATION:
	IR=IR+1
C  FILL ARARYS AT THE SECOND TIME ONLY!
	IF(IRUN.EQ.2)THEN              !!! second run only
	DO IATOM=1,NATOM
	IP(IATOM,IR)=IPT(IATOM)
	RDC(1,IATOM,IR)=RDCT(1,IATOM)
	RDC(2,IATOM,IR)=RDCT(2,IATOM)
	RDC(3,IATOM,IR)=RDCT(3,IATOM)
	ENDDO
	IKOV(IR)=IG
	SHIFT(1,IR)=DX-OX+ORIGIN(1)
	SHIFT(2,IR)=DY-OY+ORIGIN(2)
	SHIFT(3,IR)=DZ-OZ+ORIGIN(3)
C  No phases appear if no SO
	IF(NORBS.EQ.1)THEN
	SOPHS(1,IR)=(1.D0,0.D0)
	SOPHS(2,IR)=(1.D0,0.D0)
	SJ(1,1,IR)=(1.D0,0.D0)
	SJ(1,2,IR)=(0.D0,0.D0)
	SJ(2,1,IR)=(0.D0,0.D0)
	SJ(2,2,IR)=(1.D0,0.D0)
C  There are changes in phase only for spinors oriented along local axis
	ELSEIF(NORBS.EQ.2)THEN
	SOPHS(1,IR)=SOM(1,1)
	SOPHS(2,IR)=SOM(2,2)
	DO ISORT=1,NSORT
	IF(ABS(SOPHS(1,IR)-SPHASE(1,ISORT)).GT.1.D-10.
     &OR.ABS(SOPHS(2,IR)-SPHASE(2,ISORT)).GT.1.D-10)
     &CALL GROUPS_MESG04
	SJ(1,1,IR)=UJ(1,1,IG)
	SJ(1,2,IR)=UJ(1,2,IG)
	SJ(2,1,IR)=UJ(2,1,IG)
	SJ(2,2,IR)=UJ(2,2,IG)
	ENDDO
	ENDIF
	ENDIF                         !!! second run only
	IF(NONSYM.EQ.0)GOTO2000       !!! set nonsym=0 if Nopr.le.48 is needed
1000  CONTINUE
2000  CONTINUE                      !!! over Kovalev rotations
	NGROUP=IR  !! TOTAL # OF GROUP ELEMENTS
	NOPRMEM=IR !! SIZE OF ALLOCATABLE ARRAYS IS KNOWN NOW
3000	CONTINUE                      !!! over two RUNS
C  SHIFT THE ORIGIN TO THE OLD ONE
	DO IATOM=1,NATOM
	DO I=1,3
	COORD(I,IATOM)=COORD(I,IATOM)+ORIGIN(I)
	ENDDO
	ENDDO
C  CHECK OUT GENERATION
	DO IATOM=1,NATOM
	X=COORD(1,IATOM)
	Y=COORD(2,IATOM)
	Z=COORD(3,IATOM)
	DO IR=1,NGROUP
	IG=IKOV(IR)
	DX=SHIFT(1,IR)
	DY=SHIFT(2,IR)
	DZ=SHIFT(3,IR)
	CALL LIB_ROTATE(DX,DY,DZ,X2,Y2,Z2,U(2,IG),1)
	CALL LIB_ROTATE(X,Y,Z,X1,Y1,Z1,U(2,IG),1)
	X1=X1-X2
	Y1=Y1-Y2
	Z1=Z1-Z2
C REDUCE TO FIRST CELL
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	CALL GROUPS_CHECK(P,Q,R,COORD2,IPATOM,NATOM,IS,IATOM,EPS)
	IF(IPATOM.EQ.0)CALL GROUPS_MESG01
	IF(IS(IPATOM).NE.IS(IATOM))CALL GROUPS_MESG02
C CHECK RDC
	X2=COORD(1,IPATOM)+RDC(1,IATOM,IR)
	Y2=COORD(2,IPATOM)+RDC(2,IATOM,IR)
	Z2=COORD(3,IPATOM)+RDC(3,IATOM,IR)
	IF(ABS(X2-X1).GT.EPS.OR.ABS(Y2-Y1).GT.EPS.OR.
     &   ABS(Z2-Z1).GT.EPS)CALL GROUPS_MESG03
	ENDDO
	ENDDO


c---------- Output statements added by Gunnar Wed Jun 21 2000 ----------- 

	open(fh_grp,file='grp.dat',status='unknown')

	write(fh_grp,'("##### Number of group elements #####")')
	write(fh_grp,'("##### and atoms in unit cell.  #####")')
	write(fh_grp,'(2i3)') ngroup, natom

	write(fh_grp,'("##### ir-ig group correspondance #####")')
	write(fh_grp,'("##### ir and ig                  #####")')
	do ir = 1,ngroup
	   write(fh_grp,'(6x,2i3)') ir,ikov(ir)
	enddo

	write(fh_grp,'("##### Rotation of atom positions   #####")')
	write(fh_grp,'("##### ir, atom pre/post rot, x,y,z #####")')
	do ir = 1,ngroup
	   do iatom = 1,natom
	      write(fh_grp,'(6x,3i3,3e18.10)') ir,iatom,ip(iatom,ir),
     &        rdc(1,iatom,ir),rdc(2,iatom,ir),rdc(3,iatom,ir)
	   enddo
	enddo

	write(fh_grp,*) ngroup
	write(fh_grp,*) (((SJ(i,j,ir),i=1,2),j=1,2),ir=1,ngroup)
	close(fh_grp)

c---------- End of added output statement ----------

	
C MEMORY DEALLOCATIONS
	DEALLOCATE(IPT,COORD1,COORD2,PSHIFT,RDCT,FAC,SPHASE)
	RETURN
C ERROR HANDLINGS
	CONTAINS
	SUBROUTINE GROUPS_MESG01
	WRITE(IUT,*)'Terminated in <GROUPS>:'
	WRITE(IUT,*)'Failed to generate the crystal group.'
	WRITE(IUT,*)'IPATOM=0 detected during testing.'
	CALL FULLSTOP(' ')
	END SUBROUTINE GROUPS_MESG01
	SUBROUTINE GROUPS_MESG02
	WRITE(IUT,*)'Terminated in <GROUPS>:'
	WRITE(IUT,*)'Failed to generate the crystal group.'
	WRITE(IUT,*)'Wrong IS array detected during testing.'
	CALL FULLSTOP(' ')
	END SUBROUTINE GROUPS_MESG02
	SUBROUTINE GROUPS_MESG03
	WRITE(IUT,*)'Terminated in <GROUPS>:'
	WRITE(IUT,*)'Failed to generate the crystal group.'
	WRITE(IUT,*)'Wrong RDC array detected during testing.'
	CALL FULLSTOP(' ')
	END SUBROUTINE GROUPS_MESG03
	SUBROUTINE GROUPS_MESG04
	WRITE(IUT,*)'Terminated in <GROUPS>:'
	WRITE(IUT,*)'Failed to generate the crystal group.'
	WRITE(IUT,*)'Wrong SPHASE array detected during testing.'
	CALL FULLSTOP(' ')
	END SUBROUTINE GROUPS_MESG04
	END



	SUBROUTINE GROUPS_CHECK(P,Q,R,COORD1,IP,NATOM,IS,JATOM,EPS)
***************************************************************
*  Checks if the atoms transform to each other.               *
***************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION COORD1(3,NATOM),IS(NATOM)
	DO IATOM=1,NATOM
	F1=ABS(P-COORD1(1,IATOM))
	F2=ABS(F1-1.D0)
	IF(F1.LE.EPS.OR.F2.LE.EPS)THEN
	F1=ABS(Q-COORD1(2,IATOM))
	F2=ABS(F1-1.D0)
	IF(F1.LE.EPS.OR.F2.LE.EPS)THEN
	F1=ABS(R-COORD1(3,IATOM))
	F2=ABS(F1-1.D0)
	IF(F1.LE.EPS.OR.F2.LE.EPS)THEN
	IP=IATOM
	IF(IS(IP).NE.IS(JATOM))THEN
	IP=0
	ENDIF
	RETURN
	ENDIF
	ENDIF
	ENDIF
	ENDDO
	IP=0
	END



	SUBROUTINE KELITAB
*******************************************************
*  The preparation of Keli table for a given group    *
*******************************************************
C SET ARRAY-SIZE DATA
        USE DIM_LMTART,  ONLY : NOPRMEM
C SET EXTERNAL CONNECTIONS
        USE COM_GROUPS                                          !!! from groups
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: RK(:,:)
	DATA X/0.1D0/,Y/0.2D0/,Z/0.3D0/
C MEMORY ALLOCATIONS
	ALLOCATE(RK(3,NOPRMEM))
C START CODE:
	DO IR=1,NGROUP
	IG=IKOV(IR)
	CALL LIB_ROTATE(X,Y,Z,RK(1,IR),RK(2,IR),RK(3,IR),U(2,IG),2)
	ENDDO
	DO IR1=1,NGROUP
	IG1=IKOV(IR1)
	DO IR2=1,NGROUP
	IG2=IKOV(IR2)
	CALL LIB_ROTATE(RK(1,IR1),RK(2,IR1),RK(3,IR1),X2,Y2,Z2,U(2,IG2),2)
	KELTAB(IR2,IR1)=0
	DO IR3=1,NGROUP
	IG3=IKOV(IR3)
	IF(ABS(RK(1,IR3)-X2).LE.1.D-04.AND
     &  .ABS(RK(2,IR3)-Y2).LE.1.D-04.AND
     &  .ABS(RK(3,IR3)-Z2).LE.1.D-04)KELTAB(IR2,IR1)=IR3
	ENDDO
	ENDDO
	ENDDO
C MEMORY DEALLOCATIONS
	DEALLOCATE(RK)
	END
