	SUBROUTINE INI_MAKEFFT
************************************************************
*  Prepares data for the fast fourier transform.           *
************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
        USE COM_LMTMEM                                         !!! from setup
	USE COM_CONTROL,  ONLY : IASA,ILDA,IGGA                  !!! from readini
      USE COM_PARAM,    ONLY : NATOM,NSORT,IS                  !!! from readini
      USE COM_LATPAR,   ONLY : PAR,OMEGAC                      !!! from readini
      USE COM_SPHERE,   ONLY : SMT                             !!! from readini
      USE COM_LATTICE,  ONLY : RBAS,TAU                        !!! from readstr
      USE COM_FFTMESH,  ONLY : NFFTG,AD,RD,INTFFT              !!! from fftmesh
      USE COM_SITEGEN,  ONLY : NSITES,SITES,IREFSITES          !!! from sitegen
      USE COM_MASHDEP,  ONLY : LENINT,LENREL,LENCMP            !!! from readini
      USE COM_UNIT,     ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER, INTRINSIC :: SIZE
	DATA PI/3.141592653589793D0/
      WRITE(IUN,1010)
1010  FORMAT(1X,'Start building fast Fourier transform grid:   '/)
C TRY TO ESTIMATE FFT GRID
C Set ratio between total volume and MT spheres
	OMEGAM=0.D0
	DO IATOM=1,NATOM
	OMEGAM=OMEGAM+4.D0/3.D0*PI*SMT(IS(IATOM))**3
	ENDDO
	REL=OMEGAC/OMEGAM
C For LDA try 3000, for GGA twice more.
	FFTATOM=3000.D0*REL
	IF(IGGA.GT.0)FFTATOM=6000.D0*REL
      FFTOPT=FFTATOM*DBLE(NATOM)
      RL1=SQRT(RBAS(1,1)**2+RBAS(2,1)**2+RBAS(3,1)**2)
      RL2=SQRT(RBAS(1,2)**2+RBAS(2,2)**2+RBAS(3,2)**2)
      RL3=SQRT(RBAS(1,3)**2+RBAS(2,3)**2+RBAS(3,3)**2)
	RMAX=MAX(RL1,RL2,RL3)
	FFTLIN=RMAX*(FFTOPT/RL1/RL2/RL3)**(1.D0/3.D0)
      RN1=FFTLIN*RL1/RMAX
      RN2=FFTLIN*RL2/RMAX
      RN3=FFTLIN*RL3/RMAX
	NN1=NINT(RN1)
	NN2=NINT(RN2)
	NN3=NINT(RN3)
	IF(MOD(NN1,2).NE.0)NN1=NN1+1
	IF(MOD(NN2,2).NE.0)NN2=NN2+1
	IF(MOD(NN3,2).NE.0)NN3=NN3+1
	IF(NN1.EQ.0)NN1=1
	IF(NN2.EQ.0)NN2=1
	IF(NN3.EQ.0)NN3=1
C PRINT ESTIMATE
	IF(IGGA.EQ.0)WRITE(IUN,1110)NN1,NN2,NN3
	IF(IGGA.GT.0)WRITE(IUN,1120)NN1,NN2,NN3
1110	FORMAT(1X,'Optimal FFT divsions nfftg(:) for LDA =',3(1X,I4))
1120	FORMAT(1X,'Optimal FFT divsions nfftg(:) for GGA =',3(1X,I4))
      IF(NFFTG(1)+NFFTG(2)+NFFTG(3).NE.0)THEN
	WRITE(IUN,1130)NFFTG
1130	FORMAT(1X,'FFT divisions read from the INIFILE   =',3(1X,I4))
	ELSE
      NFFTG(1)=NN1
      NFFTG(2)=NN2
      NFFTG(3)=NN3
      ENDIF
C SET MEMORY ALLOCATIONS DATA
	IF(IASA.NE.2)NFFTMEM=-1
	IF(IASA.NE.2)RETURN
	NFFTMEM=(NFFTG(1)+1)*(NFFTG(2)+1)*(NFFTG(3)+1)
C SET PRIMITIVE TRANSLATIONS
	DO J=1,3
	AD(J,1)=RBAS(J,1)/DBLE(NFFTG(1))
	AD(J,2)=RBAS(J,2)/DBLE(NFFTG(2))
	AD(J,3)=RBAS(J,3)/DBLE(NFFTG(3))
	ENDDO
C SET TRANSLATIONS WHICH ARE RECIPROCAL TO AD
	CALL INI_MAKEFFT_CROS(RD(1,1),AD(1,2),AD(1,3))
	CALL INI_MAKEFFT_CROS(RD(1,2),AD(1,3),AD(1,1))
	CALL INI_MAKEFFT_CROS(RD(1,3),AD(1,1),AD(1,2))
	WD=AD(1,1)*RD(1,1)+AD(2,1)*RD(2,1)+AD(3,1)*RD(3,1)
	DO I=1,3
	RD(I,1)=RD(I,1)/WD
	RD(I,2)=RD(I,2)/WD
	RD(I,3)=RD(I,3)/WD
	ENDDO
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	IF(NFFTMEM.EQ.-1)RETURN
	ALLOCATE(INTFFT(NFFTMEM))
      SIZE_INTFFT=DBLE(SIZE(INTFFT)*LENINT)/1024.D0**2
      CURMEM=CURMEM+SIZE_INTFFT
C FIND SPHERE RADIUS CIRCUMSCRIBING UNIT CELL
	SMAX=0.D0
	DO ISORT=1,NSORT
	SMAX=MAX(SMAX,SMT(ISORT)/PAR)
	ENDDO
	RMAX=0.D0
	DO IZ=-1,1
	DO JZ=-1,1
	DO KZ=-1,1
	RSUM=0.D0
	DO I=1,3
	RSUM=RSUM+(IZ*RBAS(I,1)+JZ*RBAS(I,2)+KZ*RBAS(I,3))**2
	ENDDO
	RMAX=MAX(RMAX,RSUM)
	ENDDO
	ENDDO
	ENDDO
	RMAX=SQRT(RMAX)+2.D0*SMAX+0.1D0
C GENERATE ALL LATTICE SITES INSIDE SPHERE "RMAX"
	CALL STR_SITEGEN(RMAX,RBAS,TAU,NATOM)
	NVEC=NSITES
C CALCULATE WHICH POINTS ARE IN THE INTERSTITIALS
	IINT=0
	IMTS=0
	DO MFFT3=0,NFFTG(3)-1
	DO MFFT2=0,NFFTG(2)-1
	DO MFFT1=0,NFFTG(1)-1
	IFFT=1+MFFT1+MFFT2*NFFTG(1)+MFFT3*NFFTG(1)*NFFTG(2)
C RADIUS-VECTOR IN THE INTERSTITIAL REGION IS GIVEN BY
	RSX=(MFFT1*AD(1,1)+MFFT2*AD(1,2)+MFFT3*AD(1,3))
	RSY=(MFFT1*AD(2,1)+MFFT2*AD(2,2)+MFFT3*AD(2,3))
	RSZ=(MFFT1*AD(3,1)+MFFT2*AD(3,2)+MFFT3*AD(3,3))
	IATOM=0
	DO IVEC=0,NVEC
	TX=SITES(1,IVEC)
	TY=SITES(2,IVEC)
	TZ=SITES(3,IVEC)
	DIF=SQRT((TX-RSX)**2+(TY-RSY)**2+(TZ-RSZ)**2)
	IF(DIF.LE.SMT(IS(IREFSITES(IVEC)))/PAR)IATOM=IREFSITES(IVEC)
	IF(DIF.LE.SMT(IS(IREFSITES(IVEC)))/PAR)EXIT
	ENDDO
	INTFFT(IFFT)=IATOM
	IF(IATOM.EQ.0)IINT=IINT+1
	IF(IATOM.GT.0)IMTS=IMTS+1
	ENDDO
	ENDDO
	ENDDO
	NINTTOT=IINT
	NMTSTOT=IMTS
	WRITE(IUN,1210)NMTSTOT
	WRITE(IUN,1220)NINTTOT
	WRITE(IUN,1230)NFFTG(1)*NFFTG(2)*NFFTG(3)
	WRITE(IUN,1240)NMTSTOT+NINTTOT
1210	FORMAT(1X,'# of points of the FFT grid within MTS=',I7)
1220	FORMAT(1X,'# of points of the FFT grid in the INT=',I7)
1230	FORMAT(1X,'Total # of FFT points generated       =',I7)
1240	FORMAT(1X,'                         Sum (MTS+INT)=',I7/)
	IF(NINTTOT+NMTSTOT.NE.NFFTG(1)*NFFTG(2)*NFFTG(3))    
     &  CALL MAKEFFT_MESG01
C MEMORY DEALLOCATIONS
      IF(ALLOCATED(SITES))DEALLOCATE(SITES)
      IF(ALLOCATED(IREFSITES))DEALLOCATE(IREFSITES)
C ERROR HANDLINGS
      CONTAINS
      SUBROUTINE MAKEFFT_MESG01
      WRITE(IUT,*)'Message from <MAKEFFT>:'
      WRITE(IUT,*)'Sorting of points to those belonging to '
      WRITE(IUT,*)'MT spheres and interstitials mismatch.'
      WRITE(IUT,*)'Probably MT spheres overlap.'
      END SUBROUTINE MAKEFFT_MESG01
	END SUBROUTINE INI_MAKEFFT




      SUBROUTINE INI_MAKEFFT_CROS(A,B,C)
**********************************************************
*    Cross product (AX,AY,AZ)=(BX,BY,BZ)*(CX,CY,CZ)      *
**********************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(3),B(3),C(3)
      A(1)=B(2)*C(3)-B(3)*C(2)
      A(2)=B(3)*C(1)-B(1)*C(3)
      A(3)=B(1)*C(2)-B(2)*C(1)
      END SUBROUTINE INI_MAKEFFT_CROS
