      SUBROUTINE LIB_CMPEIG1A(NDIM,HMAT,OMAT,E,NSPIN,NORBS,
     &           IVEC,IERR,ICHECK,EPS)
*****************************************************************
*    The solution of the generalized eigenvalue problem         *
*     __                 __                                     *
*     >   H(L"L)*A(L)= E >  O(L"L)A(L)                          *
*     --                 --                                     *
*      L                  L                                     *
*    where H can be non-hermitian matrix and O must be          *
*    positively defined hermitian overlap matrix.               *
*****************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM
      USE COM_MASHDEP
      USE COM_UNIT
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	INTEGER, ALLOCATABLE :: MN(:)
      REAL*8, ALLOCATABLE :: HR(:,:)
      REAL*8, ALLOCATABLE :: HI(:,:)
      REAL*8, ALLOCATABLE :: ZR(:,:)
      REAL*8, ALLOCATABLE :: ZI(:,:)
      REAL*8, ALLOCATABLE :: WR(:)
      REAL*8, ALLOCATABLE :: WI(:)
      REAL*8, ALLOCATABLE :: PR(:)
      REAL*8, ALLOCATABLE :: W1(:)
      REAL*8, ALLOCATABLE :: W2(:)
      REAL*8, ALLOCATABLE :: W3(:)
	COMPLEX*16, ALLOCATABLE :: OC(:,:)
	COMPLEX*16, ALLOCATABLE :: WC(:,:)
	COMPLEX*16, ALLOCATABLE :: WA(:)
      COMPLEX*16 CHECK,HMAT(NDIM,NDIM),OMAT(NDIM,NDIM),E(NDIM),S
      NDIAG=NSPIN-NORBS+1
      IF(MOD(NDIM,NDIAG).NE.0)
     &CALL FULLSTOP('<CMPEIG1A>: error in dimension')
      MDIM=NDIM/NDIAG
C MEMORY ALLOCATIONS
      ALLOCATE(HR(MDIM,MDIM),STAT=IALLOC)
      IF(IALLOC.EQ.1)CALL CMPEIG1A_MESG01('HR',SIZE(HR))
      ALLOCATE(HI(MDIM,MDIM),STAT=IALLOC)
      IF(IALLOC.EQ.1)CALL CMPEIG1A_MESG01('HI',SIZE(HI))
      ALLOCATE(ZR(MDIM,MDIM),STAT=IALLOC)
      IF(IALLOC.EQ.1)CALL CMPEIG1A_MESG01('ZR',SIZE(ZR))
      ALLOCATE(ZI(MDIM,MDIM),STAT=IALLOC)
      IF(IALLOC.EQ.1)CALL CMPEIG1A_MESG01('ZI',SIZE(ZI))
      ALLOCATE(OC(MDIM,MDIM),STAT=IALLOC)
      IF(IALLOC.EQ.1)CALL CMPEIG1A_MESG01('OC',SIZE(OC))
      ALLOCATE(WC(MDIM,MDIM),STAT=IALLOC)
      IF(IALLOC.EQ.1)CALL CMPEIG1A_MESG01('WC',SIZE(WC))
	ALLOCATE(WA(MDIM),STAT=IALLOC)
	ALLOCATE(MN(MDIM),STAT=IALLOC)
	ALLOCATE(WR(MDIM),STAT=IALLOC)
	ALLOCATE(WI(MDIM),STAT=IALLOC)
	ALLOCATE(PR(MDIM),STAT=IALLOC)
	ALLOCATE(W1(MDIM),STAT=IALLOC)
	ALLOCATE(W2(MDIM),STAT=IALLOC)
	ALLOCATE(W3(MDIM),STAT=IALLOC)
      SIZE_HR=DBLE(SIZE(HR)*LENREL)/1024.D0**2
      CURMEM=CURMEM+SIZE_HR*8.D0
      IF(TOTMEM.LT.CURMEM)TXTMEM='<CMPEIG1A> Subroutine'
      if(TOTMEM.LT.CURMEM)TOTMEM=CURMEM
C DO OVER SPINS IF NO SPIN-ORBIT COUPLING
      DO 1000 IDIAG=1,NDIAG
      MSHF=MDIM*(IDIAG-1)
C SAVE OVERLAP MATRIX IN SCRATCH ARRAY
	DO IDIM=1,MDIM
	DO JDIM=1,MDIM
	WC(IDIM,JDIM)=OMAT(IDIM+MSHF,JDIM+MSHF)
	OC(IDIM,JDIM)=OMAT(IDIM+MSHF,JDIM+MSHF)
	ENDDO
	ENDDO
C PERFORM CHOLESKY FACTORIZATION OF THE O-MATRIX
	CALL LIB_CHOLDEC(MDIM,WC,MDIM,PR,IFAIL)
	IF(IFAIL.EQ.1)THEN
	WRITE(IUT,*)'<CMPEIG1A>: OVERLAP MATRIX IS NOT POSITIVE DEFINED'
	ENDIF
C NOW O = U"U AND U-MATRIX IS CONTAINED IN UPPER TRIANGLE OF O-MATRIX
C INVERT U-MATRIX U**(-1)=OC NOW!
	DO JDIM=1,MDIM
	OC(JDIM,JDIM)=DCMPLX(PR(JDIM),0.D0)
	DO IDIM=JDIM-1,1,-1
	S=(0.D0,0.D0)
	DO KDIM=IDIM+1,JDIM
	S=S+WC(IDIM,KDIM)*OC(KDIM,JDIM)
	ENDDO
	OC(IDIM,JDIM)=-S*PR(IDIM)
	OC(JDIM,IDIM)=(0.D0,0.D0)
	ENDDO
	ENDDO
C BUILD NEW H-MATRIX H'=B" H B
	DO IDIM=1,MDIM
	DO JDIM=1,MDIM
	S=(0.D0,0.D0)
	DO KDIM=1,IDIM
	DO LDIM=1,JDIM
	S=S+DCONJG(OC(KDIM,IDIM))*HMAT(KDIM+MSHF,LDIM+MSHF)*OC(LDIM,JDIM)
	ENDDO
	ENDDO
	HR(IDIM,JDIM)=DREAL(S)
	HI(IDIM,JDIM)=DIMAG(S)
	ENDDO
	ENDDO
C SOLVE STANDARD EIGENVALUE PROBLEM H'A'=E A'
      CALL LIB_CMPDIAG(MDIM,MDIM,HR,HI,WR,WI,IVEC,ZR,ZI,W1,W2,W3,IERR)
	IF(IERR.NE.0)
     &WRITE(IUT,*)'<CMPEIG1A>: ERROR IN DIAGONALIZATION PROGRAM'
C SAVE EIGENVALUES
	DO IBND=1,MDIM
	E(IBND+MSHF)=DCMPLX(WR(IBND),WI(IBND))
	ENDDO
	IF(IVEC.EQ.0)GOTO 1000
C RENORMALIZE EIGENVECTORS
      DO IBND=1,MDIM
	ANORM=0.D0
	DO IDIM=1,MDIM
	ANORM=ANORM+DCMPLX(ZR(IDIM,IBND),+ZI(IDIM,IBND))*
     &            DCMPLX(ZR(IDIM,IBND),-ZI(IDIM,IBND))
	ENDDO
      DO IDIM=1,MDIM
	ZR(IDIM,IBND)=ZR(IDIM,IBND)/SQRT(ANORM)
	ZI(IDIM,IBND)=ZI(IDIM,IBND)/SQRT(ANORM)
      ENDDO
      ENDDO
C RETURN TO EIGENVECTORS A=B A'
	DO IBND=1,MDIM
	DO IDIM=1,MDIM
	S=(0.D0,0.D0)
	DO KDIM=IDIM,MDIM
	S=S+OC(IDIM,KDIM)*DCMPLX(ZR(KDIM,IBND),ZI(KDIM,IBND))
	ENDDO
	WC(IDIM,IBND)=S
	ENDDO
	ENDDO
      IF(ICHECK.EQ.1)THEN
C CHECK WHETHER EIGENVECTORS ARE CORRECT
      DO IBND=1,MDIM
      DO IDIM=1,MDIM
      CHECK=(0.D0,0.D0)
      DO JDIM=1,MDIM
      CHECK=CHECK+(
     &HMAT(IDIM+MSHF,JDIM+MSHF)-E(IBND+MSHF)*
     &OMAT(IDIM+MSHF,JDIM+MSHF))*WC(JDIM,IBND)
      ENDDO
      IF(ABS(CHECK).GE.EPS)
     &WRITE(IUT,*)'<CMPEIG1A>: WRONG EIGENVECTORS:',
     &CHECK,IDIM,IBND
      ENDDO
      ENDDO
	ENDIF
C FIND RECIPROCAL EIGENVECTORS
      DO IBND=1,MDIM
      DO IDIM=1,MDIM
      OC(IDIM,IBND)=(0.D0,0.D0)
	DO JDIM=1,MDIM
      OC(IDIM,IBND)=OC(IDIM,IBND)+
     &OMAT(IDIM+MSHF,JDIM+MSHF)*WC(JDIM,IBND)
	ENDDO
      ENDDO
      ENDDO
	CALL LIB_CINV(MDIM,OC,MDIM,WA,IFAIL)
	IF(IFAIL.NE.0)
     &CALL FULLSTOP('<CMPEIG1A>:: Error in inversion program')
      IF(ICHECK.EQ.1)THEN
C CHECK WHETHER RECIPROCAL EIGENVECTORS ARE CORRECT
      DO IBND=1,MDIM
      DO IDIM=1,MDIM
      CHECK=(0.D0,0.D0)
      DO JDIM=1,MDIM
      CHECK=CHECK+OC(IBND,JDIM)*(
     &HMAT(JDIM+MSHF,IDIM+MSHF)-E(IBND+MSHF)*
     &OMAT(JDIM+MSHF,IDIM+MSHF))
      ENDDO
      IF(ABS(CHECK).GE.EPS)
     &WRITE(IUT,*)'<CMPEIG1A>: WRONG EIGENVECTORS:',
     &CHECK,IDIM,IBND
      ENDDO
      ENDDO
	ENDIF
C STORE EIGENVECTORS IN OVERLAP, RECIPROCALS IN THE HAMILTONIAN
      DO IBND=1,MDIM
      DO IDIM=1,MDIM
	OMAT(IDIM+MSHF,IBND+MSHF)=WC(IDIM,IBND)
      HMAT(IDIM+MSHF,IBND+MSHF)=OC(IBND,IDIM) ! index swapped
      ENDDO
      ENDDO
 1000	CONTINUE
C SORT EIGENVALUES BY INCREASING ORDER
	DO IDIAG=1,NDIAG              !!! over idiag
      MSHF=MDIM*(IDIAG-1)
	DO IBND=1,MDIM
	MN(IBND)=IBND
	WA(IBND)=E(IBND+MSHF)
	ENDDO
      DO IBND=1,MDIM
      DO JBND=IBND+1,MDIM
	IF(DREAL(E(IBND+MSHF)).GT.DREAL(E(JBND+MSHF)))THEN
	S=E(IBND+MSHF)
	E(IBND+MSHF)=E(JBND+MSHF)
	E(JBND+MSHF)=S
	I=MN(IBND)
	MN(IBND)=MN(JBND)
	MN(JBND)=I
	ENDIF
	ENDDO
	ENDDO
	IF(IVEC.EQ.0)CYCLE
	DO IBND=1,MDIM
	IF(ABS(E(IBND+MSHF)-WA(MN(IBND))).GT.1.D-10)
     &WRITE(IUT,*)'<CMPEIG1A>: Error in ordering algorithm'
	DO IDIM=1,MDIM
	WC(IDIM,IBND)=OMAT(IDIM+MSHF,MN(IBND)+MSHF)
	OC(IDIM,IBND)=HMAT(IDIM+MSHF,MN(IBND)+MSHF)
	ENDDO
	ENDDO
	DO IBND=1,MDIM
	DO IDIM=1,MDIM
	OMAT(IDIM+MSHF,IBND+MSHF)=WC(IDIM,IBND)
	HMAT(IDIM+MSHF,IBND+MSHF)=OC(IDIM,IBND)
	ENDDO
	ENDDO
	ENDDO                    !!! over idiag
C REORDER EIGENVALUES AND EIGENVECTORS
      IF(NDIAG.EQ.2)THEN
      DO IBNDT=1,MDIM
      WA(IBNDT)=E(IBNDT+MDIM)
      DO IDIM=1,MDIM
	WC(IDIM,IBNDT)=OMAT(IDIM+MDIM,IBNDT+MDIM)
	OC(IDIM,IBNDT)=HMAT(IDIM+MDIM,IBNDT+MDIM)
	ENDDO
	ENDDO
      DO IBNDT=MDIM,1,-1
      IBAND=1+NSPIN*(IBNDT-1)
	E(IBAND)=E(IBNDT)
      DO IDIM=1,MDIM
	OMAT(IDIM,IBAND)=OMAT(IDIM,IBNDT)
	HMAT(IDIM,IBAND)=HMAT(IDIM,IBNDT)
	ENDDO
      DO IDIM=1,MDIM
	OMAT(IDIM+MDIM,IBAND)=(0.D0,0.D0)
	HMAT(IDIM+MDIM,IBAND)=(0.D0,0.D0)
	ENDDO
      IBAND=2+NSPIN*(IBNDT-1)
	E(IBAND)=WA(IBNDT)
	DO IDIM=1,MDIM
	OMAT(IDIM,IBAND)=(0.D0,0.D0)
	HMAT(IDIM,IBAND)=(0.D0,0.D0)
	ENDDO
      DO IDIM=1,MDIM
	OMAT(IDIM+MDIM,IBAND)=WC(IDIM,IBNDT)
	HMAT(IDIM+MDIM,IBAND)=OC(IDIM,IBNDT)
	ENDDO
	ENDDO
      ENDIF
C MEMORY DEALLOCATIONS
      DEALLOCATE(HR,HI,ZR,ZI,WR,WI,OC,WC,PR,W1,W2,W3,WA,MN)
      CURMEM=CURMEM-SIZE_HR*8.D0
      CONTAINS
      SUBROUTINE CMPEIG1A_MESG01(CHARV,LSIZE)
      CHARACTER CHARV*(*)
      WRITE(IUT,*)'Terminated in <CMPEIG1A>::'
      WRITE(IUT,*)'Memory allocation problem detected by'
      WRITE(IUT,*)'the system. Real array '//CHARV//' of the size:'
      WRITE(IUT,*) DBLE(LSIZE*LENREL)/1024.D0**2, ' Mbyte'
      WRITE(IUT,*)'cannot be allocated.'
      CALL FULLSTOP(' ')
      END SUBROUTINE CMPEIG1A_MESG01
      END SUBROUTINE LIB_CMPEIG1A



      SUBROUTINE LIB_CMPDIAG(NM,N,AR,AI,WR,WI,MATZ,ZR,ZI,
     &           FV1,FV2,FV3,IERR)     
	IMPLICIT NONE
C
      INTEGER N,NM,IS1,IS2,IERR,MATZ
      REAL*8 AR(NM,N),AI(NM,N),WR(N),WI(N),ZR(NM,N),ZI(NM,N),
     &       FV1(N),FV2(N),FV3(N)
C
C     THIS SUBROUTINE CALLS THE RECOMMENDED SEQUENCE OF
C     SUBROUTINES FROM THE EIGENSYSTEM SUBROUTINE PACKAGE (EISPACK)
C     TO FIND THE EIGENVALUES AND EIGENVECTORS (IF DESIRED)
C     OF A COMPLEX GENERAL MATRIX.
C
C     ON INPUT
C
C        NM  MUST BE SET TO THE ROW DIMENSION OF THE TWO-DIMENSIONAL
C        ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C        DIMENSION STATEMENT.
C
C        N  IS THE ORDER OF THE MATRIX  A=(AR,AI).
C
C        AR  AND  AI  CONTAIN THE REAL AND IMAGINARY PARTS,
C        RESPECTIVELY, OF THE COMPLEX GENERAL MATRIX.
C
C        MATZ  IS AN INTEGER VARIABLE SET EQUAL TO ZERO IF
C        ONLY EIGENVALUES ARE DESIRED.  OTHERWISE IT IS SET TO
C        ANY NON-ZERO INTEGER FOR BOTH EIGENVALUES AND EIGENVECTORS.
C
C     ON OUTPUT
C
C        WR  AND  WI  CONTAIN THE REAL AND IMAGINARY PARTS,
C        RESPECTIVELY, OF THE EIGENVALUES.
C
C        ZR  AND  ZI  CONTAIN THE REAL AND IMAGINARY PARTS,
C        RESPECTIVELY, OF THE EIGENVECTORS IF MATZ IS NOT ZERO.
C
C        IERR  IS AN INTEGER OUTPUT VARIABLE SET EQUAL TO AN ERROR
C           COMPLETION CODE DESCRIBED IN THE DOCUMENTATION FOR COMQR
C           AND COMQR2.  THE NORMAL COMPLETION CODE IS ZERO.
C
C        FV1, FV2, AND  FV3  ARE TEMPORARY STORAGE ARRAYS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IF (N .LE. NM) GO TO 10
      IERR = 10 * N
      GO TO 50
C
   10 CALL  LIB_CMPDIAG_CBAL(NM,N,AR,AI,IS1,IS2,FV1)
      CALL  LIB_CMPDIAG_CORTH(NM,N,IS1,IS2,AR,AI,FV2,FV3)
      IF (MATZ .NE. 0) GO TO 20
C     .......... FIND EIGENVALUES ONLY ..........
      CALL  LIB_CMPDIAG_COMQR(NM,N,IS1,IS2,AR,AI,WR,WI,IERR)
      GO TO 50
C     .......... FIND BOTH EIGENVALUES AND EIGENVECTORS ..........
   20 CALL  LIB_CMPDIAG_COMQR2(NM,N,IS1,IS2,FV2,FV3,AR,AI,
     &      WR,WI,ZR,ZI,IERR)
      IF (IERR .NE. 0) GO TO 50
      CALL  LIB_CMPDIAG_CBABK2(NM,N,IS1,IS2,FV1,N,ZR,ZI)
   50 RETURN
      END
      
      SUBROUTINE LIB_CMPDIAG_CBAL(NM,N,AR,AI,LOW,IGH,SCALE)
      IMPLICIT NONE
C
      INTEGER I,J,K,L,M,N,JJ,NM,IGH,LOW,IEXC
      REAL*8 AR(NM,N),AI(NM,N),SCALE(N)
      REAL*8 C,F,G,R,S,B2,RADIX
      LOGICAL NOCONV
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE
C     CBALANCE, WHICH IS A COMPLEX VERSION OF BALANCE,
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971).
C
C     THIS SUBROUTINE BALANCES A COMPLEX MATRIX AND ISOLATES
C     EIGENVALUES WHENEVER POSSIBLE.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX MATRIX TO BE BALANCED.
C
C     ON OUTPUT
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE BALANCED MATRIX.
C
C        LOW AND IGH ARE TWO INTEGERS SUCH THAT AR(I,J) AND AI(I,J)
C          ARE EQUAL TO ZERO IF
C           (1) I IS GREATER THAN J AND
C           (2) J=1,...,LOW-1 OR I=IGH+1,...,N.
C
C        SCALE CONTAINS INFORMATION DETERMINING THE
C           PERMUTATIONS AND SCALING FACTORS USED.
C
C     SUPPOSE THAT THE PRINCIPAL SUBMATRIX IN ROWS LOW THROUGH IGH
C     HAS BEEN BALANCED, THAT P(J) DENOTES THE INDEX INTERCHANGED
C     WITH J DURING THE PERMUTATION STEP, AND THAT THE ELEMENTS
C     OF THE DIAGONAL MATRIX USED ARE DENOTED BY D(I,J).  THEN
C        SCALE(J) = P(J),    FOR J = 1,...,LOW-1
C                 = D(J,J)       J = LOW,...,IGH
C                 = P(J)         J = IGH+1,...,N.
C     THE ORDER IN WHICH THE INTERCHANGES ARE MADE IS N TO IGH+1,
C     THEN 1 TO LOW-1.
C
C     NOTE THAT 1 IS RETURNED FOR IGH IF IGH IS ZERO FORMALLY.
C
C     THE ALGOL PROCEDURE EXC CONTAINED IN CBALANCE APPEARS IN
C     CBAL  IN LINE.  (NOTE THAT THE ALGOL ROLES OF IDENTIFIERS
C     K,L HAVE BEEN REVERSED.)
C
C     ARITHMETIC IS REAL THROUGHOUT.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      RADIX = 16.D0
C
      B2 = RADIX * RADIX
      K = 1
      L = N
      GO TO 100
C     .......... IN-LINE PROCEDURE FOR ROW AND
C                COLUMN EXCHANGE ..........
   20 SCALE(M) = J
      IF (J .EQ. M) GO TO 50
C
      DO 30 I = 1, L
         F = AR(I,J)
         AR(I,J) = AR(I,M)
         AR(I,M) = F
         F = AI(I,J)
         AI(I,J) = AI(I,M)
         AI(I,M) = F
   30 CONTINUE
C
      DO 40 I = K, N
         F = AR(J,I)
         AR(J,I) = AR(M,I)
         AR(M,I) = F
         F = AI(J,I)
         AI(J,I) = AI(M,I)
         AI(M,I) = F
   40 CONTINUE
C
   50 GO TO (80,130), IEXC
C     .......... SEARCH FOR ROWS ISOLATING AN EIGENVALUE
C                AND PUSH THEM DOWN ..........
   80 IF (L .EQ. 1) GO TO 280
      L = L - 1
C     .......... FOR J=L STEP -1 UNTIL 1 DO -- ..........
  100 DO 120 JJ = 1, L
         J = L + 1 - JJ
C
         DO 110 I = 1, L
            IF (I .EQ. J) GO TO 110
            IF (AR(J,I) .NE. 0.D0 .OR. AI(J,I) .NE. 0.D0) GO TO 120
  110    CONTINUE
C
         M = L
         IEXC = 1
         GO TO 20
  120 CONTINUE
C
      GO TO 140
C     .......... SEARCH FOR COLUMNS ISOLATING AN EIGENVALUE
C                AND PUSH THEM LEFT ..........
  130 K = K + 1
C
  140 DO 170 J = K, L
C
         DO 150 I = K, L
            IF (I .EQ. J) GO TO 150
            IF (AR(I,J) .NE. 0.D0 .OR. AI(I,J) .NE. 0.D0) GO TO 170
  150    CONTINUE
C
         M = K
         IEXC = 2
         GO TO 20
  170 CONTINUE
C     .......... NOW BALANCE THE SUBMATRIX IN ROWS K TO L ..........
      DO 180 I = K, L
  180 SCALE(I) = 1.D0
C     .......... ITERATIVE LOOP FOR NORM REDUCTION ..........
  190 NOCONV = .FALSE.
C
      DO 270 I = K, L
         C = 0.D0
         R = 0.D0
C
         DO 200 J = K, L
            IF (J .EQ. I) GO TO 200
            C = C + DABS(AR(J,I)) + DABS(AI(J,I))
            R = R + DABS(AR(I,J)) + DABS(AI(I,J))
  200    CONTINUE
C     .......... GUARD AGAINST ZERO C OR R DUE TO UNDERFLOW ..........
         IF (C .EQ. 0.D0 .OR. R .EQ. 0.D0) GO TO 270
         G = R / RADIX
         F = 1.D0
         S = C + R
  210    IF (C .GE. G) GO TO 220
         F = F * RADIX
         C = C * B2
         GO TO 210
  220    G = R * RADIX
  230    IF (C .LT. G) GO TO 240
         F = F / RADIX
         C = C / B2
         GO TO 230
C     .......... NOW BALANCE ..........
  240    IF ((C + R) / F .GE. 0.95D0 * S) GO TO 270
         G = 1.D0 / F
         SCALE(I) = SCALE(I) * F
         NOCONV = .TRUE.
C
         DO 250 J = K, N
            AR(I,J) = AR(I,J) * G
            AI(I,J) = AI(I,J) * G
  250    CONTINUE
C
         DO 260 J = 1, L
            AR(J,I) = AR(J,I) * F
            AI(J,I) = AI(J,I) * F
  260    CONTINUE
C
  270 CONTINUE
C
      IF (NOCONV) GO TO 190
C
  280 LOW = K
      IGH = L
      RETURN
      END
      
      SUBROUTINE LIB_CMPDIAG_CORTH(NM,N,LOW,IGH,AR,AI,ORTR,ORTI)
      IMPLICIT NONE
C
      INTEGER I,J,M,N,II,JJ,LA,MP,NM,IGH,KP1,LOW
      REAL*8 AR(NM,N),AI(NM,N),ORTR(IGH),ORTI(IGH)
      REAL*8 F,G,H,FI,FR,SCALE,PYTHAG
C
C     THIS SUBROUTINE IS A TRANSLATION OF A COMPLEX ANALOGUE OF
C     THE ALGOL PROCEDURE ORTHES, NUM. MATH. 12, 349-368(1968)
C     BY MARTIN AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 339-358(1971).
C
C     GIVEN A COMPLEX GENERAL MATRIX, THIS SUBROUTINE
C     REDUCES A SUBMATRIX SITUATED IN ROWS AND COLUMNS
C     LOW THROUGH IGH TO UPPER HESSENBERG FORM BY
C     UNITARY SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  CBAL.  IF  CBAL  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX INPUT MATRIX.
C
C     ON OUTPUT
C
C        AR AND AI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE HESSENBERG MATRIX.  INFORMATION
C          ABOUT THE UNITARY TRANSFORMATIONS USED IN THE REDUCTION
C          IS STORED IN THE REMAINING TRIANGLES UNDER THE
C          HESSENBERG MATRIX.
C
C        ORTR AND ORTI CONTAIN FURTHER INFORMATION ABOUT THE
C          TRANSFORMATIONS.  ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     CALLS PYTHAG FOR  SQRT(A*A + B*B) .
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      LA = IGH - 1
      KP1 = LOW + 1
      IF (LA .LT. KP1) GO TO 200
C
      DO 180 M = KP1, LA
         H = 0.D0
         ORTR(M) = 0.D0
         ORTI(M) = 0.D0
         SCALE = 0.D0
C     .......... SCALE COLUMN (ALGOL TOL THEN NOT NEEDED) ..........
         DO 90 I = M, IGH
   90    SCALE = SCALE + DABS(AR(I,M-1)) + DABS(AI(I,M-1))
C
         IF (SCALE .EQ. 0.D0) GO TO 180
         MP = M + IGH
C     .......... FOR I=IGH STEP -1 UNTIL M DO -- ..........
         DO 100 II = M, IGH
            I = MP - II
            ORTR(I) = AR(I,M-1) / SCALE
            ORTI(I) = AI(I,M-1) / SCALE
            H = H + ORTR(I) * ORTR(I) + ORTI(I) * ORTI(I)
  100    CONTINUE
C
         G = DSQRT(H)
         F = PYTHAG(ORTR(M),ORTI(M))
         IF (F .EQ. 0.D0) GO TO 103
         H = H + F * G
         G = G / F
         ORTR(M) = (1.D0 + G) * ORTR(M)
         ORTI(M) = (1.D0 + G) * ORTI(M)
         GO TO 105
C
  103    ORTR(M) = G
         AR(M,M-1) = SCALE
C     .......... FORM (I-(U*UT)/H) * A ..........
  105    DO 130 J = M, N
            FR = 0.D0
            FI = 0.D0
C     .......... FOR I=IGH STEP -1 UNTIL M DO -- ..........
            DO 110 II = M, IGH
               I = MP - II
               FR = FR + ORTR(I) * AR(I,J) + ORTI(I) * AI(I,J)
               FI = FI + ORTR(I) * AI(I,J) - ORTI(I) * AR(I,J)
  110       CONTINUE
C
            FR = FR / H
            FI = FI / H
C
            DO 120 I = M, IGH
               AR(I,J) = AR(I,J) - FR * ORTR(I) + FI * ORTI(I)
               AI(I,J) = AI(I,J) - FR * ORTI(I) - FI * ORTR(I)
  120       CONTINUE
C
  130    CONTINUE
C     .......... FORM (I-(U*UT)/H)*A*(I-(U*UT)/H) ..........
         DO 160 I = 1, IGH
            FR = 0.D0
            FI = 0.D0
C     .......... FOR J=IGH STEP -1 UNTIL M DO -- ..........
            DO 140 JJ = M, IGH
               J = MP - JJ
               FR = FR + ORTR(J) * AR(I,J) - ORTI(J) * AI(I,J)
               FI = FI + ORTR(J) * AI(I,J) + ORTI(J) * AR(I,J)
  140       CONTINUE
C
            FR = FR / H
            FI = FI / H
C
            DO 150 J = M, IGH
               AR(I,J) = AR(I,J) - FR * ORTR(J) - FI * ORTI(J)
               AI(I,J) = AI(I,J) + FR * ORTI(J) - FI * ORTR(J)
  150       CONTINUE
C
  160    CONTINUE
C
         ORTR(M) = SCALE * ORTR(M)
         ORTI(M) = SCALE * ORTI(M)
         AR(M,M-1) = -G * AR(M,M-1)
         AI(M,M-1) = -G * AI(M,M-1)
  180 CONTINUE
C
  200 RETURN
      END

      SUBROUTINE LIB_CMPDIAG_COMQR(NM,N,LOW,IGH,HR,HI,WR,WI,IERR)
      IMPLICIT NONE
C
      INTEGER I,J,L,N,EN,LL,NM,IGH,ITN,ITS,LOW,LP1,ENM1,IERR
      REAL*8 HR(NM,N),HI(NM,N),WR(N),WI(N)
      REAL*8 SI,SR,TI,TR,XI,XR,YI,YR,ZZI,ZZR,NORM,TST1,TST2,
     &       PYTHAG
C
C     THIS SUBROUTINE IS A TRANSLATION OF A UNITARY ANALOGUE OF THE
C     ALGOL PROCEDURE  COMLR, NUM. MATH. 12, 369-376(1968) BY MARTIN
C     AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 396-403(1971).
C     THE UNITARY ANALOGUE SUBSTITUTES THE QR ALGORITHM OF FRANCIS
C     (COMP. JOUR. 4, 332-345(1962)) FOR THE LR ALGORITHM.
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES OF A COMPLEX
C     UPPER HESSENBERG MATRIX BY THE QR METHOD.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  CBAL.  IF  CBAL  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        HR AND HI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX UPPER HESSENBERG MATRIX.
C          THEIR LOWER TRIANGLES BELOW THE SUBDIAGONAL CONTAIN
C          INFORMATION ABOUT THE UNITARY TRANSFORMATIONS USED IN
C          THE REDUCTION BY  CORTH, IF PERFORMED.
C
C     ON OUTPUT
C
C        THE UPPER HESSENBERG PORTIONS OF HR AND HI HAVE BEEN
C          DESTROYED.  THEREFORE, THEY MUST BE SAVED BEFORE
C          CALLING  COMQR  IF SUBSEQUENT CALCULATION OF
C          EIGENVECTORS IS TO BE PERFORMED.
C
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVALUES.  IF AN ERROR
C          EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT
C          FOR INDICES IERR+1,...,N.
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT.
C
C     CALLS CDIV FOR COMPLEX DIVISION.
C     CALLS CSROOT FOR COMPLEX SQUARE ROOT.
C     CALLS PYTHAG FOR  SQRT(A*A + B*B) .
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IERR = 0
      IF (LOW .EQ. IGH) GO TO 180
C     .......... CREATE REAL SUBDIAGONAL ELEMENTS ..........
      L = LOW + 1
C
      DO 170 I = L, IGH
         LL = MIN0(I+1,IGH)
         IF (HI(I,I-1) .EQ. 0.D0) GO TO 170
         NORM = PYTHAG(HR(I,I-1),HI(I,I-1))
         YR = HR(I,I-1) / NORM
         YI = HI(I,I-1) / NORM
         HR(I,I-1) = NORM
         HI(I,I-1) = 0.D0
C
         DO 155 J = I, IGH
            SI = YR * HI(I,J) - YI * HR(I,J)
            HR(I,J) = YR * HR(I,J) + YI * HI(I,J)
            HI(I,J) = SI
  155    CONTINUE
C
         DO 160 J = LOW, LL
            SI = YR * HI(J,I) + YI * HR(J,I)
            HR(J,I) = YR * HR(J,I) - YI * HI(J,I)
            HI(J,I) = SI
  160    CONTINUE
C
  170 CONTINUE
C     .......... STORE ROOTS ISOLATED BY CBAL ..........
  180 DO 200 I = 1, N
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 200
         WR(I) = HR(I,I)
         WI(I) = HI(I,I)
  200 CONTINUE
C
      EN = IGH
      TR = 0.D0
      TI = 0.D0
      ITN = 30*N
C     .......... SEARCH FOR NEXT EIGENVALUE ..........
  220 IF (EN .LT. LOW) GO TO 1001
      ITS = 0
      ENM1 = EN - 1
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT
C                FOR L=EN STEP -1 UNTIL LOW E0 -- ..........
  240 DO 260 LL = LOW, EN
         L = EN + LOW - LL
         IF (L .EQ. LOW) GO TO 300
         TST1 = DABS(HR(L-1,L-1)) + DABS(HI(L-1,L-1))
     &            + DABS(HR(L,L)) + DABS(HI(L,L))
         TST2 = TST1 + DABS(HR(L,L-1))
         IF (TST2 .EQ. TST1) GO TO 300
  260 CONTINUE
C     .......... FORM SHIFT ..........
  300 IF (L .EQ. EN) GO TO 660
      IF (ITN .EQ. 0) GO TO 1000
      IF (ITS .EQ. 10 .OR. ITS .EQ. 20) GO TO 320
      SR = HR(EN,EN)
      SI = HI(EN,EN)
      XR = HR(ENM1,EN) * HR(EN,ENM1)
      XI = HI(ENM1,EN) * HR(EN,ENM1)
      IF (XR .EQ. 0.D0 .AND. XI .EQ. 0.D0) GO TO 340
      YR = (HR(ENM1,ENM1) - SR) / 2.D0
      YI = (HI(ENM1,ENM1) - SI) / 2.D0
      CALL LIB_CMPDIAG_CSROOT(YR**2-YI**2+XR,2.D0*YR*YI+XI,ZZR,ZZI)
      IF (YR * ZZR + YI * ZZI .GE. 0.D0) GO TO 310
      ZZR = -ZZR
      ZZI = -ZZI
  310 CALL LIB_CMPDIAG_CDIV(XR,XI,YR+ZZR,YI+ZZI,XR,XI)
      SR = SR - XR
      SI = SI - XI
      GO TO 340
C     .......... FORM EXCEPTIONAL SHIFT ..........
  320 SR = DABS(HR(EN,ENM1)) + DABS(HR(ENM1,EN-2))
      SI = 0.D0
C
  340 DO 360 I = LOW, EN
         HR(I,I) = HR(I,I) - SR
         HI(I,I) = HI(I,I) - SI
  360 CONTINUE
C
      TR = TR + SR
      TI = TI + SI
      ITS = ITS + 1
      ITN = ITN - 1
C     .......... REDUCE TO TRIANGLE (ROWS) ..........
      LP1 = L + 1
C
      DO 500 I = LP1, EN
         SR = HR(I,I-1)
         HR(I,I-1) = 0.D0
         NORM = PYTHAG(PYTHAG(HR(I-1,I-1),HI(I-1,I-1)),SR)
         XR = HR(I-1,I-1) / NORM
         WR(I-1) = XR
         XI = HI(I-1,I-1) / NORM
         WI(I-1) = XI
         HR(I-1,I-1) = NORM
         HI(I-1,I-1) = 0.D0
         HI(I,I-1) = SR / NORM
C
         DO 490 J = I, EN
            YR = HR(I-1,J)
            YI = HI(I-1,J)
            ZZR = HR(I,J)
            ZZI = HI(I,J)
            HR(I-1,J) = XR * YR + XI * YI + HI(I,I-1) * ZZR
            HI(I-1,J) = XR * YI - XI * YR + HI(I,I-1) * ZZI
            HR(I,J) = XR * ZZR - XI * ZZI - HI(I,I-1) * YR
            HI(I,J) = XR * ZZI + XI * ZZR - HI(I,I-1) * YI
  490    CONTINUE
C
  500 CONTINUE
C
      SI = HI(EN,EN)
      IF (SI .EQ. 0.D0) GO TO 540
      NORM = PYTHAG(HR(EN,EN),SI)
      SR = HR(EN,EN) / NORM
      SI = SI / NORM
      HR(EN,EN) = NORM
      HI(EN,EN) = 0.D0
C     .......... INVERSE OPERATION (COLUMNS) ..........
  540 DO 600 J = LP1, EN
         XR = WR(J-1)
         XI = WI(J-1)
C
         DO 580 I = L, J
            YR = HR(I,J-1)
            YI = 0.D0
            ZZR = HR(I,J)
            ZZI = HI(I,J)
            IF (I .EQ. J) GO TO 560
            YI = HI(I,J-1)
            HI(I,J-1) = XR * YI + XI * YR + HI(J,J-1) * ZZI
  560       HR(I,J-1) = XR * YR - XI * YI + HI(J,J-1) * ZZR
            HR(I,J) = XR * ZZR + XI * ZZI - HI(J,J-1) * YR
            HI(I,J) = XR * ZZI - XI * ZZR - HI(J,J-1) * YI
  580    CONTINUE
C
  600 CONTINUE
C
      IF (SI .EQ. 0.D0) GO TO 240
C
      DO 630 I = L, EN
         YR = HR(I,EN)
         YI = HI(I,EN)
         HR(I,EN) = SR * YR - SI * YI
         HI(I,EN) = SR * YI + SI * YR
  630 CONTINUE
C
      GO TO 240
C     .......... A ROOT FOUND ..........
  660 WR(EN) = HR(EN,EN) + TR
      WI(EN) = HI(EN,EN) + TI
      EN = ENM1
      GO TO 220
C     .......... SET ERROR -- ALL EIGENVALUES HAVE NOT
C                CONVERGED AFTER 30*N ITERATIONS ..........
 1000 IERR = EN
 1001 RETURN
      END

      
      SUBROUTINE LIB_CMPDIAG_COMQR2(NM,N,LOW,IGH,ORTR,ORTI,HR,HI,     
     &                          WR,WI,ZR,ZI,IERR)
      IMPLICIT NONE
C
      INTEGER I,J,K,L,M,N,EN,II,JJ,LL,NM,NN,IGH,IP1,
     &        ITN,ITS,LOW,LP1,ENM1,IEND,IERR
      REAL*8 HR(NM,N),HI(NM,N),WR(N),WI(N),ZR(NM,N),ZI(NM,N),
     &       ORTR(IGH),ORTI(IGH)
      REAL*8 SI,SR,TI,TR,XI,XR,YI,YR,ZZI,ZZR,NORM,TST1,TST2,
     &       PYTHAG
C
C     THIS SUBROUTINE IS A TRANSLATION OF A UNITARY ANALOGUE OF THE
C     ALGOL PROCEDURE  COMLR2, NUM. MATH. 16, 181-204(1970) BY PETERS
C     AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971).
C     THE UNITARY ANALOGUE SUBSTITUTES THE QR ALGORITHM OF FRANCIS
C     (COMP. JOUR. 4, 332-345(1962)) FOR THE LR ALGORITHM.
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
C     OF A COMPLEX UPPER HESSENBERG MATRIX BY THE QR
C     METHOD.  THE EIGENVECTORS OF A COMPLEX GENERAL MATRIX
C     CAN ALSO BE FOUND IF  CORTH  HAS BEEN USED TO REDUCE
C     THIS GENERAL MATRIX TO HESSENBERG FORM.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  CBAL.  IF  CBAL  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        ORTR AND ORTI CONTAIN INFORMATION ABOUT THE UNITARY TRANS-
C          FORMATIONS USED IN THE REDUCTION BY  CORTH, IF PERFORMED.
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED.  IF THE EIGENVECTORS
C          OF THE HESSENBERG MATRIX ARE DESIRED, SET ORTR(J) AND
C          ORTI(J) TO 0.D0 FOR THESE ELEMENTS.
C
C        HR AND HI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE COMPLEX UPPER HESSENBERG MATRIX.
C          THEIR LOWER TRIANGLES BELOW THE SUBDIAGONAL CONTAIN FURTHER
C          INFORMATION ABOUT THE TRANSFORMATIONS WHICH WERE USED IN THE
C          REDUCTION BY  CORTH, IF PERFORMED.  IF THE EIGENVECTORS OF
C          THE HESSENBERG MATRIX ARE DESIRED, THESE ELEMENTS MAY BE
C          ARBITRARY.
C
C     ON OUTPUT
C
C        ORTR, ORTI, AND THE UPPER HESSENBERG PORTIONS OF HR AND HI
C          HAVE BEEN DESTROYED.
C
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVALUES.  IF AN ERROR
C          EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT
C          FOR INDICES IERR+1,...,N.
C
C        ZR AND ZI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVECTORS.  THE EIGENVECTORS
C          ARE UNNORMALIZED.  IF AN ERROR EXIT IS MADE, NONE OF
C          THE EIGENVECTORS HAS BEEN FOUND.
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT.
C
C     CALLS CDIV FOR COMPLEX DIVISION.
C     CALLS CSROOT FOR COMPLEX SQUARE ROOT.
C     CALLS PYTHAG FOR  SQRT(A*A + B*B) .
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IERR = 0
C     .......... INITIALIZE EIGENVECTOR MATRIX ..........
      DO 101 J = 1, N
C
         DO 100 I = 1, N
            ZR(I,J) = 0.D0
            ZI(I,J) = 0.D0
  100    CONTINUE
         ZR(J,J) = 1.D0
  101 CONTINUE
C     .......... FORM THE MATRIX OF ACCUMULATED TRANSFORMATIONS
C                FROM THE INFORMATION LEFT BY CORTH ..........
      IEND = IGH - LOW - 1
      IF (IEND) 180, 150, 105
C     .......... FOR I=IGH-1 STEP -1 UNTIL LOW+1 DO -- ..........
  105 DO 140 II = 1, IEND
         I = IGH - II
         IF (ORTR(I) .EQ. 0.D0 .AND. ORTI(I) .EQ. 0.D0) GO TO 140
         IF (HR(I,I-1) .EQ. 0.D0 .AND. HI(I,I-1) .EQ. 0.D0) GO TO 140
C     .......... NORM BELOW IS NEGATIVE OF H FORMED IN CORTH ..........
         NORM = HR(I,I-1) * ORTR(I) + HI(I,I-1) * ORTI(I)
         IP1 = I + 1
C
         DO 110 K = IP1, IGH
            ORTR(K) = HR(K,I-1)
            ORTI(K) = HI(K,I-1)
  110    CONTINUE
C
         DO 130 J = I, IGH
            SR = 0.D0
            SI = 0.D0
C
            DO 115 K = I, IGH
               SR = SR + ORTR(K) * ZR(K,J) + ORTI(K) * ZI(K,J)
               SI = SI + ORTR(K) * ZI(K,J) - ORTI(K) * ZR(K,J)
  115       CONTINUE
C
            SR = SR / NORM
            SI = SI / NORM
C
            DO 120 K = I, IGH
               ZR(K,J) = ZR(K,J) + SR * ORTR(K) - SI * ORTI(K)
               ZI(K,J) = ZI(K,J) + SR * ORTI(K) + SI * ORTR(K)
  120       CONTINUE
C
  130    CONTINUE
C
  140 CONTINUE
C     .......... CREATE REAL SUBDIAGONAL ELEMENTS ..........
  150 L = LOW + 1
C
      DO 170 I = L, IGH
         LL = MIN0(I+1,IGH)
         IF (HI(I,I-1) .EQ. 0.D0) GO TO 170
         NORM = PYTHAG(HR(I,I-1),HI(I,I-1))
         YR = HR(I,I-1) / NORM
         YI = HI(I,I-1) / NORM
         HR(I,I-1) = NORM
         HI(I,I-1) = 0.D0
C
         DO 155 J = I, N
            SI = YR * HI(I,J) - YI * HR(I,J)
            HR(I,J) = YR * HR(I,J) + YI * HI(I,J)
            HI(I,J) = SI
  155    CONTINUE
C
         DO 160 J = 1, LL
            SI = YR * HI(J,I) + YI * HR(J,I)
            HR(J,I) = YR * HR(J,I) - YI * HI(J,I)
            HI(J,I) = SI
  160    CONTINUE
C
         DO 165 J = LOW, IGH
            SI = YR * ZI(J,I) + YI * ZR(J,I)
            ZR(J,I) = YR * ZR(J,I) - YI * ZI(J,I)
            ZI(J,I) = SI
  165    CONTINUE
C
  170 CONTINUE
C     .......... STORE ROOTS ISOLATED BY CBAL ..........
  180 DO 200 I = 1, N
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 200
         WR(I) = HR(I,I)
         WI(I) = HI(I,I)
  200 CONTINUE
C
      EN = IGH
      TR = 0.D0
      TI = 0.D0
      ITN = 30*N
C     .......... SEARCH FOR NEXT EIGENVALUE ..........
  220 IF (EN .LT. LOW) GO TO 680
      ITS = 0
      ENM1 = EN - 1
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT
C                FOR L=EN STEP -1 UNTIL LOW DO -- ..........
  240 DO 260 LL = LOW, EN
         L = EN + LOW - LL
         IF (L .EQ. LOW) GO TO 300
         TST1 = DABS(HR(L-1,L-1)) + DABS(HI(L-1,L-1))
     &            + DABS(HR(L,L)) + DABS(HI(L,L))
         TST2 = TST1 + DABS(HR(L,L-1))
         IF (TST2 .EQ. TST1) GO TO 300
  260 CONTINUE
C     .......... FORM SHIFT ..........
  300 IF (L .EQ. EN) GO TO 660
      IF (ITN .EQ. 0) GO TO 1000
      IF (ITS .EQ. 10 .OR. ITS .EQ. 20) GO TO 320
      SR = HR(EN,EN)
      SI = HI(EN,EN)
      XR = HR(ENM1,EN) * HR(EN,ENM1)
      XI = HI(ENM1,EN) * HR(EN,ENM1)
      IF (XR .EQ. 0.D0 .AND. XI .EQ. 0.D0) GO TO 340
      YR = (HR(ENM1,ENM1) - SR) / 2.D0
      YI = (HI(ENM1,ENM1) - SI) / 2.D0
      CALL LIB_CMPDIAG_CSROOT(YR**2-YI**2+XR,2.D0*YR*YI+XI,ZZR,ZZI)
      IF (YR * ZZR + YI * ZZI .GE. 0.D0) GO TO 310
      ZZR = -ZZR
      ZZI = -ZZI
  310 CALL LIB_CMPDIAG_CDIV(XR,XI,YR+ZZR,YI+ZZI,XR,XI)
      SR = SR - XR
      SI = SI - XI
      GO TO 340
C     .......... FORM EXCEPTIONAL SHIFT ..........
  320 SR = DABS(HR(EN,ENM1)) + DABS(HR(ENM1,EN-2))
      SI = 0.D0
C
  340 DO 360 I = LOW, EN
         HR(I,I) = HR(I,I) - SR
         HI(I,I) = HI(I,I) - SI
  360 CONTINUE
C
      TR = TR + SR
      TI = TI + SI
      ITS = ITS + 1
      ITN = ITN - 1
C     .......... REDUCE TO TRIANGLE (ROWS) ..........
      LP1 = L + 1
C
      DO 500 I = LP1, EN
         SR = HR(I,I-1)
         HR(I,I-1) = 0.D0
         NORM = PYTHAG(PYTHAG(HR(I-1,I-1),HI(I-1,I-1)),SR)
         XR = HR(I-1,I-1) / NORM
         WR(I-1) = XR
         XI = HI(I-1,I-1) / NORM
         WI(I-1) = XI
         HR(I-1,I-1) = NORM
         HI(I-1,I-1) = 0.D0
         HI(I,I-1) = SR / NORM
C
         DO 490 J = I, N
            YR = HR(I-1,J)
            YI = HI(I-1,J)
            ZZR = HR(I,J)
            ZZI = HI(I,J)
            HR(I-1,J) = XR * YR + XI * YI + HI(I,I-1) * ZZR
            HI(I-1,J) = XR * YI - XI * YR + HI(I,I-1) * ZZI
            HR(I,J) = XR * ZZR - XI * ZZI - HI(I,I-1) * YR
            HI(I,J) = XR * ZZI + XI * ZZR - HI(I,I-1) * YI
  490    CONTINUE
C
  500 CONTINUE
C
      SI = HI(EN,EN)
      IF (SI .EQ. 0.D0) GO TO 540
      NORM = PYTHAG(HR(EN,EN),SI)
      SR = HR(EN,EN) / NORM
      SI = SI / NORM
      HR(EN,EN) = NORM
      HI(EN,EN) = 0.D0
      IF (EN .EQ. N) GO TO 540
      IP1 = EN + 1
C
      DO 520 J = IP1, N
         YR = HR(EN,J)
         YI = HI(EN,J)
         HR(EN,J) = SR * YR + SI * YI
         HI(EN,J) = SR * YI - SI * YR
  520 CONTINUE
C     .......... INVERSE OPERATION (COLUMNS) ..........
  540 DO 600 J = LP1, EN
         XR = WR(J-1)
         XI = WI(J-1)
C
         DO 580 I = 1, J
            YR = HR(I,J-1)
            YI = 0.D0
            ZZR = HR(I,J)
            ZZI = HI(I,J)
            IF (I .EQ. J) GO TO 560
            YI = HI(I,J-1)
            HI(I,J-1) = XR * YI + XI * YR + HI(J,J-1) * ZZI
  560       HR(I,J-1) = XR * YR - XI * YI + HI(J,J-1) * ZZR
            HR(I,J) = XR * ZZR + XI * ZZI - HI(J,J-1) * YR
            HI(I,J) = XR * ZZI - XI * ZZR - HI(J,J-1) * YI
  580    CONTINUE
C
         DO 590 I = LOW, IGH
            YR = ZR(I,J-1)
            YI = ZI(I,J-1)
            ZZR = ZR(I,J)
            ZZI = ZI(I,J)
            ZR(I,J-1) = XR * YR - XI * YI + HI(J,J-1) * ZZR
            ZI(I,J-1) = XR * YI + XI * YR + HI(J,J-1) * ZZI
            ZR(I,J) = XR * ZZR + XI * ZZI - HI(J,J-1) * YR
            ZI(I,J) = XR * ZZI - XI * ZZR - HI(J,J-1) * YI
  590    CONTINUE
C
  600 CONTINUE
C
      IF (SI .EQ. 0.D0) GO TO 240
C
      DO 630 I = 1, EN
         YR = HR(I,EN)
         YI = HI(I,EN)
         HR(I,EN) = SR * YR - SI * YI
         HI(I,EN) = SR * YI + SI * YR
  630 CONTINUE
C
      DO 640 I = LOW, IGH
         YR = ZR(I,EN)
         YI = ZI(I,EN)
         ZR(I,EN) = SR * YR - SI * YI
         ZI(I,EN) = SR * YI + SI * YR
  640 CONTINUE
C
      GO TO 240
C     .......... A ROOT FOUND ..........
  660 HR(EN,EN) = HR(EN,EN) + TR
      WR(EN) = HR(EN,EN)
      HI(EN,EN) = HI(EN,EN) + TI
      WI(EN) = HI(EN,EN)
      EN = ENM1
      GO TO 220
C     .......... ALL ROOTS FOUND.  BACKSUBSTITUTE TO FIND
C                VECTORS OF UPPER TRIANGULAR FORM ..........
  680 NORM = 0.D0
C
      DO 720 I = 1, N
C
         DO 720 J = I, N
            TR = DABS(HR(I,J)) + DABS(HI(I,J))
            IF (TR .GT. NORM) NORM = TR
  720 CONTINUE
C
      IF (N .EQ. 1 .OR. NORM .EQ. 0.D0) GO TO 1001
C     .......... FOR EN=N STEP -1 UNTIL 2 DO -- ..........
      DO 800 NN = 2, N
         EN = N + 2 - NN
         XR = WR(EN)
         XI = WI(EN)
         HR(EN,EN) = 1.D0
         HI(EN,EN) = 0.D0
         ENM1 = EN - 1
C     .......... FOR I=EN-1 STEP -1 UNTIL 1 DO -- ..........
         DO 780 II = 1, ENM1
            I = EN - II
            ZZR = 0.D0
            ZZI = 0.D0
            IP1 = I + 1
C
            DO 740 J = IP1, EN
               ZZR = ZZR + HR(I,J) * HR(J,EN) - HI(I,J) * HI(J,EN)
               ZZI = ZZI + HR(I,J) * HI(J,EN) + HI(I,J) * HR(J,EN)
  740       CONTINUE
C
            YR = XR - WR(I)
            YI = XI - WI(I)
            IF (YR .NE. 0.D0 .OR. YI .NE. 0.D0) GO TO 765
               TST1 = NORM
               YR = TST1
  760          YR = 1.D-1 * YR
               TST2 = NORM + YR
               IF (TST2 .GT. TST1) GO TO 760
  765       CONTINUE
            CALL LIB_CMPDIAG_CDIV(ZZR,ZZI,YR,YI,HR(I,EN),HI(I,EN))
C     .......... OVERFLOW CONTROL ..........
            TR = DABS(HR(I,EN)) + DABS(HI(I,EN))
            IF (TR .EQ. 0.D0) GO TO 780
            TST1 = TR
            TST2 = TST1 + 1.D0/TST1
            IF (TST2 .GT. TST1) GO TO 780
            DO 770 J = I, EN
               HR(J,EN) = HR(J,EN)/TR
               HI(J,EN) = HI(J,EN)/TR
  770       CONTINUE
C
  780    CONTINUE
C
  800 CONTINUE
C     .......... END BACKSUBSTITUTION ..........
      ENM1 = N - 1
C     .......... VECTORS OF ISOLATED ROOTS ..........
      DO  840 I = 1, ENM1
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 840
         IP1 = I + 1
C
         DO 820 J = IP1, N
            ZR(I,J) = HR(I,J)
            ZI(I,J) = HI(I,J)
  820    CONTINUE
C
  840 CONTINUE
C     .......... MULTIPLY BY TRANSFORMATION MATRIX TO GIVE
C                VECTORS OF ORIGINAL FULL MATRIX.
C                FOR J=N STEP -1 UNTIL LOW+1 DO -- ..........
      DO 880 JJ = LOW, ENM1
         J = N + LOW - JJ
         M = MIN0(J,IGH)
C
         DO 880 I = LOW, IGH
            ZZR = 0.D0
            ZZI = 0.D0
C
            DO 860 K = LOW, M
               ZZR = ZZR + ZR(I,K) * HR(K,J) - ZI(I,K) * HI(K,J)
               ZZI = ZZI + ZR(I,K) * HI(K,J) + ZI(I,K) * HR(K,J)
  860       CONTINUE
C
            ZR(I,J) = ZZR
            ZI(I,J) = ZZI
  880 CONTINUE
C
      GO TO 1001
C     .......... SET ERROR -- ALL EIGENVALUES HAVE NOT
C                CONVERGED AFTER 30*N ITERATIONS ..........
 1000 IERR = EN
 1001 RETURN
      END

      SUBROUTINE LIB_CMPDIAG_CBABK2(NM,N,LOW,IGH,SCALE,M,ZR,ZI)
      IMPLICIT NONE
C
      INTEGER I,J,K,M,N,II,NM,IGH,LOW
      REAL*8 SCALE(N),ZR(NM,M),ZI(NM,M)
      REAL*8 S
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE
C     CBABK2, WHICH IS A COMPLEX VERSION OF BALBAK,
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971).
C
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A COMPLEX GENERAL
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
C     BALANCED MATRIX DETERMINED BY  CBAL.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY  CBAL.
C
C        SCALE CONTAINS INFORMATION DETERMINING THE PERMUTATIONS
C          AND SCALING FACTORS USED BY  CBAL.
C
C        M IS THE NUMBER OF EIGENVECTORS TO BE BACK TRANSFORMED.
C
C        ZR AND ZI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVECTORS TO BE
C          BACK TRANSFORMED IN THEIR FIRST M COLUMNS.
C
C     ON OUTPUT
C
C        ZR AND ZI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE TRANSFORMED EIGENVECTORS
C          IN THEIR FIRST M COLUMNS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IF (M .EQ. 0) GO TO 200
      IF (IGH .EQ. LOW) GO TO 120
C
      DO 110 I = LOW, IGH
         S = SCALE(I)
C     .......... LEFT HAND EIGENVECTORS ARE BACK TRANSFORMED
C                IF THE FOREGOING STATEMENT IS REPLACED BY
C                S=1.D0/SCALE(I). ..........
         DO 100 J = 1, M
            ZR(I,J) = ZR(I,J) * S
            ZI(I,J) = ZI(I,J) * S
  100    CONTINUE
C
  110 CONTINUE
C     .......... FOR I=LOW-1 STEP -1 UNTIL 1,
C                IGH+1 STEP 1 UNTIL N DO -- ..........
  120 DO 140 II = 1, N
         I = II
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 140
         IF (I .LT. LOW) I = LOW - II
         K = SCALE(I)
         IF (K .EQ. I) GO TO 140
C
         DO 130 J = 1, M
            S = ZR(I,J)
            ZR(I,J) = ZR(K,J)
            ZR(K,J) = S
            S = ZI(I,J)
            ZI(I,J) = ZI(K,J)
            ZI(K,J) = S
  130    CONTINUE
C
  140 CONTINUE
C
  200 RETURN
      END

      REAL*8 FUNCTION PYTHAG(A,B)
      IMPLICIT NONE
      REAL*8 A,B
C
C     FINDS SQRT(A**2+B**2) WITHOUT OVERFLOW OR DESTRUCTIVE UNDERFLOW
C
      REAL*8 P,R,S,T,U
      
      P = DMAX1(DABS(A),DABS(B))
      IF (P .EQ. 0.D0) GO TO 20
      R = (DMIN1(DABS(A),DABS(B))/P)**2
   10 CONTINUE
         T = 4.D0 + R
         IF (T .EQ. 4.D0) GO TO 20
         S = R/T
         U = 1.D0 + 2.D0*S
         P = U*P
         R = (S/U)**2 * R
      GO TO 10
   20 PYTHAG = P
      RETURN
      END
      
      SUBROUTINE LIB_CMPDIAG_CSROOT(XR,XI,YR,YI)
      IMPLICIT NONE
      REAL*8 XR,XI,YR,YI
C
C     (YR,YI) = COMPLEX SQRT(XR,XI)
C     BRANCH CHOSEN SO THAT YR .GE. 0.0 AND SIGN(YI) .EQ. SIGN(XI)
C
      REAL*8 S,TR,TI,PYTHAG
      
      TR = XR
      TI = XI
      S = DSQRT(0.5D0*(PYTHAG(TR,TI) + DABS(TR)))
      IF (TR .GE. 0.D0) YR = S
      IF (TI .LT. 0.D0) S = -S
      IF (TR .LE. 0.D0) YI = S
      IF (TR .LT. 0.D0) YR = 0.5D0*(TI/YI)
      IF (TR .GT. 0.D0) YI = 0.5D0*(TI/YR)
      RETURN
      END
      
      SUBROUTINE LIB_CMPDIAG_CDIV(AR,AI,BR,BI,CR,CI)
      IMPLICIT NONE
      REAL*8 AR,AI,BR,BI,CR,CI
C
C     COMPLEX DIVISION, (CR,CI) = (AR,AI)/(BR,BI)
C
      REAL*8 S,ARS,AIS,BRS,BIS
      
      S = DABS(BR) + DABS(BI)
      ARS = AR/S
      AIS = AI/S
      BRS = BR/S
      BIS = BI/S
      S = BRS**2 + BIS**2
      CR = (ARS*BRS + AIS*BIS)/S
      CI = (AIS*BRS - ARS*BIS)/S
      RETURN
      END










      SUBROUTINE LIB_CHOLDEC(N, A, IA, P, IFAIL)
C     MARK 7 RELEASE. NAG COPYRIGHT 1978.
C
C     DOUBLE PRECISION VERSION WITH COMPLEX
C
C     THE LOWER TRIANGLE OF A POSITIVE DEFINITE HERMITIAN
C     MATRIX A, IS STORED IN THE LOWER TRIANGLE OF AN N*N
C     COMPLEX ARRAY A(I,J), I=1(1)N, J=1(1)I. THE CHOLESKY
C     DECOMPOSITION A=(U**H)*U IS PERFORMED, WHERE U IS A COMPLEX
C     UPPER TRIANGULAR MATRIX WITH REAL DIAGONAL ELEMENTS AND
C     U**H IS THE COMPLEX CONJUGATE TRANSPOSE. U IS STORED IN
C     THE REMAINDER OF THE ARRAY A EXCEPT FOR THE DIAGONAL
C     ELEMENTS AND THE RECIPROCALS OF THESE ARE STORED IN
C     P(I), I=1(1)N. THE MATRIX A IS RETAINED
C     SO THAT THE SOLUTION CAN BE SUBSEQUENTLY IMPROVED.
C     THE SUBROUTINE WILL FAIL WITH IFAIL=1 IF A HAS A
C     COMPLEX DIAGONAL ELEMENT OR WITH IFAIL=2 IF A IS NOT
C     POSITIVE DEFINITE, POSSIBLY DUE TO ROUNDING ERRORS.
C
C     PDK, DNAC, NPL, TEDDINGTON. JUNE 1977.
C     NPL DNAC LIBRARY SUBROUTINE HUHU.
C
C     .. SCALAR ARGUMENTS ..
      INTEGER IA, IFAIL, N
C     .. ARRAY ARGUMENTS ..
      COMPLEX*16 A(IA,N)
      DOUBLE PRECISION P(N)
C     ..
C     .. LOCAL SCALARS ..
      COMPLEX*16 X
      DOUBLE PRECISION ONE, Y, ZERO
      INTEGER I1, I, J, K
C     ..
      DATA ZERO /0.0D0/, ONE /1.0D0/
      DO 120 I=1,N
         I1 = I - 1
         DO 100 J=I,N
            X = CONJG(A(J,I))
            IF (I.EQ.1) GO TO 40
            DO 20 K=1,I1
               X = X - A(K,J)*CONJG(A(K,I))
   20       CONTINUE
   40       IF (J.NE.I) GO TO 80
            Y = DREAL(X)
            IF (Y.GT.ZERO) GO TO 60
            IFAIL = 1
            P(I) = Y
            RETURN
   60       Y = ONE/SQRT(Y)
            P(I) = Y
            GO TO 100
   80       A(I,J) = X*Y
  100    CONTINUE
  120 CONTINUE
      IFAIL = 0
      END