	SUBROUTINE LIB_HUBMAT_L2(IATOM,UMAT,GMAT,UBAR,GBAR)
**********************************************************
*  Computes Hubbard and Exchange matrices for a          *
*  given atom. Knowledge of Slater itegrals is required. *
*                                                        *
*  Notations:                                            *
*  If <ij|1/r|kl> stands for a standard definition,      *
*     <ij|1/r|ij> is a Hubbard-type interaction and      *
*     <ij|1/r|ji> is an exchange interaction, then:      *
*                                                        *
*1)Hubbard matrix in the lm-representation is given by   *
*     <lm,l'm'|1/r|lk,l'k'>=U(lmk,l'm'k')                *
*  and both LMK and L'M'K' can be packed into one index. *
*  Therefore UMAT(IU,IU1)=U(lmk,l'm'k')                  *
*                                                        *
*2)Exchange matrix in the lm-representation              *
*     <lm,l'm'|1/r|l'k',lk>=J(lmk,l'm'k')                *
*  and both LMK and L'M'K' can be packed into one index. *
*  Therefore GMAT(IU,IU1)=J(lmk,l'm'k')                  *
*                                                        *
*  Definitions for Slater integrals:                     *
*  For Hubbard-type interaction it is useful to          *
*  consider      /   l"  l"+1   2     2                  *
*     Fu(l",ll')=\ {r  /r'   } P (r)*P (r')              *
*                /              l     l'                 *
*  For Exchange interactions it is useful to define      *
*                /   l"  l"+1                            *
*     Fj(l",ll')=\ {r  /r'   } P (r)*P (r)*P (r')*P (r') *
*                /              l     l'    l     l'     *
*  Fu(l",ll')=FSLU(L",LL') and                           *
*  Fj(l",ll')=FSLG(L",LL') are input matrices to the     *
*  program.                                              *
*                                                        *
*  Last revised 21.11.96                                 *
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS,ILSB   !!! from readini
      USE COM_PANPAR ,ONLY: LSPAN,LTPAN                       !!! from readini
      USE COM_FSLATR ,ONLY: FSLG,FSLU                         !!! from readhub
      USE COM_GAUNT  ,ONLY: C                                 !!! from gdata
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                       !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION UMAT(LIMB2MEM,LIMB2MEM),GMAT(LIMB2MEM,LIMB2MEM) 
     &   ,UBAR(0:LMAXBMEM,0:LMAXBMEM),GBAR(0:LMAXBMEM,0:LMAXBMEM)
	DATA PI/3.141592653589793D0/
C START CODE
	ISORT=IS(IATOM)
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	DO L1=0,ILSB(ISORT)                       !!! over l1
	DO M1=-L1,L1                              !!! over m1
	DO K1=-L1,L1                              !!! over k1
	L1M1=L1*(L1+1)+M1+1
	L1K1=L1*(L1+1)+K1+1
	IU1=L1*(2*L1-1)*(2*L1+1)/3+(2*L1+1)*(L1+K1)+L1+M1+1  !  (l1,m1 k1) packed
C BUILT HUBBARD  MATRIX "UMAT" FROM THE SLATER INTEGRALS "FSLU"
	UMAT(IU,IU1)=(0.D0,0.D0)
C U(lmk; l'm'k') corresponds to <lm l'm'|1/r|lk l'k'>
	DO L2=0,MIN(2*L,2*L1),2                   !!! over l2
	M2=M-K
	K2=K1-M1
	IF(M2.EQ.K2.AND.ABS(M2).LE.L2)THEN        !!! selection rule
	UMAT(IU,IU1)=UMAT(IU,IU1)+4.D0*PI/DBLE(2*L2+1)*
     &FSLU(L2,L,L1,IATOM)*C(L2/2,LM,LK)*C(L2/2,L1K1,L1M1)
	ENDIF                                     !!! selection rule
	ENDDO                                     !!! over l2
C BUILT EXCHANGE MATRIX "GMAT" FROM THE SLATER INTEGRALS "FSLG"
	GMAT(IU,IU1)=(0.D0,0.D0)
C U(lmk; l'm'k') corresponds to <lm l'm'|1/r|lk l'k'>
	DO L2=ABS(L-L1),L+L1,2                    !!! over l2
	M2=M-K1
	K2=K-M1
	IF(M2.EQ.K2.AND.ABS(M2).LE.L2)THEN        !!! selection rule
	GMAT(IU,IU1)=GMAT(IU,IU1)+4.D0*PI/DBLE(2*L2+1)*
     &FSLG(L2,L,L1,IATOM)*C(L2/2,LM,L1K1)*C(L2/2,LK,L1M1)
	ENDIF                                     !!! selection rule
	ENDDO                                     !!! over l2
	ENDDO                                     !!! over k1
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over l1
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
C SET AVERAGE VALUES <U> AND <J>
	DO L=0,ILSB(ISORT)                        !!! over l
	DO L1=0,ILSB(ISORT)                       !!! over l1
	UBAR(L,L1)=0.D0
	GBAR(L,L1)=0.D0
	DLL1=1.D0/DBLE(2*L+1)/DBLE(2*L1+1)
	DLL=0.D0
	IF(L.NE.0)
     &DLL =1.D0/DBLE(2*L  )/DBLE(2*L1+1) ! not sure, must be checked !
	DO M=-L,L                                 !!! over m
	DO M1=-L1,L1                              !!! over m1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M+1           !  (l,m m) packed
	IU1=L1*(2*L1-1)*(2*L1+1)/3+(2*L1+1)*(L1+M1)+L1+M1+1  !  (l1,m1 m1) packed
	UBAR(L,L1)=UBAR(L,L1)+DLL1*UMAT(IU,IU1)
	IF(L.NE.0)
     &GBAR(L,L1)=GBAR(L,L1)+DLL1*UMAT(IU,IU1)-
     &DLL*(UMAT(IU,IU1)-GMAT(IU,IU1))
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l1
	ENDDO                                     !!! over l
	END





	SUBROUTINE LIB_HUBMAT_L4(L,FSLT,UMAT,UBAR,GBAR)
**************************************************************************
*  Computes Hubbard and Exchange matrices for a                          *
*  given atom. Knowledge of Slater itegrals is required.                 *
*                                                                        *
*  Notations:                                                            *
*  If <ij|1/r|kl> stands for a standard definition,                      *
*     <ij|1/r|ij> is a Hubbard-type interaction and                      *
*     <ij|1/r|ji> is an exchange interaction, then:                      *
*                                                                        *
*1)Hubbard matrix in the lm-representation is given by                   *
*     <ms,m's'|1/r|kt,k't'>=U(ms,kt,m's',k't')=U(mk,m'k')d(s=t)d(s'=t')  *
*                                                                        *
*2)Exchange matrix in the lm-representation                              *
*     <ms,m's'|1/r|k't',kt>=J(ms,kt,'m's'k't')=U(msk't',m's'kt)          *
**************************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_GAUNT  ,ONLY: C                                 !!! from gdata
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                       !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8 FSLT(0:L),UMAT(-L:L,-L:L,-L:L,-L:L)
	DATA PI/3.141592653589793D0/
C BUILT HUBBARD  MATRIX "UMAT" FROM THE SLATER INTEGRALS "FSLU"
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	DO M1=-L,L                                !!! over m1
	DO K1=-L,L                                !!! over k1
	L1M1=L*(L+1)+M1+1
	L1K1=L*(L+1)+K1+1
	UMAT(M,K,M1,K1)=(0.D0,0.D0)
C U(mk m'k') corresponds to <mm'|1/r|kk'>
	DO L2=0,2*L,2                             !!! over l2
	M2=M-K
	K2=K1-M1
	IF(M2.EQ.K2.AND.ABS(M2).LE.L2)THEN        !!! selection rule
	UMAT(M,K,M1,K1)=UMAT(M,K,M1,K1)+4.D0*PI/DBLE(2*L2+1)*
     &  FSLT(L2/2)*C(L2/2,LM,LK)*C(L2/2,L1K1,L1M1)
	ENDIF                                     !!! selection rule
	ENDDO                                     !!! over l2
	ENDDO                                     !!! over k1
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
C SET AVERAGE VALUES <U> AND <J>
	UBAR=0.D0
	GBAR=0.D0
	DLL1=1.D0/DBLE(2*L+1)/DBLE(2*L+1)
	DLL=0.D0
	IF(L.NE.0)
     &DLL =1.D0/DBLE(2*L  )/DBLE(2*L+1) ! not sure, must be checked !
	DO M=-L,L                                 !!! over m
	DO M1=-L,L                                !!! over m1
	UBAR=UBAR+DLL1*UMAT(M,M,M1,M1)
	IF(L.NE.0)
     &GBAR=GBAR+DLL1*UMAT(M,M,M1,M1)-
     &DLL*(UMAT(M,M,M1,M1)-UMAT(M,M1,M1,M))
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over m
C MEMORY DEALLCATIONS
	END






	SUBROUTINE LIB_HUBMAT_T4(L,UINP,UOUT,TMAT)
***************************************************************************
*  Transforms interaction matrix to desired representation                *
*  Indexation of Uinp assumed to be UINP(i',i,j',j)=<i'i|Vc|j'j>          *
***************************************************************************
	REAL*8 UOUT(2*(2*L+1),2*(2*L+1),2*(2*L+1),2*(2*L+1))
     &      ,UINP(-L:L,-L:L,-L:L,-L:L)
      COMPLEX*16 TMAT(2*(2*L+1),2*(2*L+1))
	DO ISPIN1=1,2
	DO I1=-L,L
	IBND1=I1+L+1+(2*L+1)*(ISPIN1-1)
	DO ISPIN =1,2
	DO I =-L,L
	IBND=I+L+1+(2*L+1)*(ISPIN-1)
	DO JSPIN1=1,2
	DO J1=-L,L
	JBND1=J1+L+1+(2*L+1)*(JSPIN1-1)
	DO JSPIN =1,2
	DO J =-L,L
	JBND=J+L+1+(2*L+1)*(JSPIN-1)
	UOUT(IBND1,IBND,JBND1,JBND)=0.D0
C Sum over all indexes
	DO MSPIN1=1,2
	DO M1=-L,L
	MDIM1=M1+L+1+(2*L+1)*(MSPIN1-1)
	DO MSPIN =1,2
	DO M =-L,L
	MDIM=M+L+1+(2*L+1)*(MSPIN-1)
	DO KSPIN1=1,2
	IF(MSPIN1.NE.KSPIN1)CYCLE
	DO K1=-L,L
	KDIM1=K1+L+1+(2*L+1)*(KSPIN1-1)
	DO KSPIN =1,2
	IF(MSPIN .NE.KSPIN )CYCLE
	DO K =-L,L
	KDIM=K+L+1+(2*L+1)*(KSPIN-1)
C UINP(i1,i,j1,j)=<i'i|Vc|j'j>
	UOUT(IBND1,IBND,JBND1,JBND)=UOUT(IBND1,IBND,JBND1,JBND)+
     &DCONJG(TMAT(MDIM1,IBND1)*TMAT(MDIM,IBND))*UINP(M1,M,K1,K)*
     &       TMAT(KDIM1,JBND1)*TMAT(KDIM,JBND)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
      END
      


	SUBROUTINE LIB_HUBMAT_K4(L,UINP,UOUT,KSYS)
***************************************************************************
*  Transforms interaction matrix to desired representation                *
*  Indexation of Uinp assumed to be UINP(i',i,j',j)=<i'i|Vc|j'j>          *
***************************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8, ALLOCATABLE :: TREL(:,:)
      COMPLEX*16, ALLOCATABLE :: RMAT(:,:),TMAT(:,:)
	REAL*8 UOUT(2*(2*L+1),2*(2*L+1),2*(2*L+1),2*(2*L+1))
     &      ,UINP(-L:L,-L:L,-L:L,-L:L)
      COMPLEX*16 TC(-3:3,7,0:3),TS(7,-3:3,0:3)
      NDIM=2*(2*L+1)
	ALLOCATE(TREL(NDIM,NDIM))
	ALLOCATE(TMAT(NDIM,NDIM))
	ALLOCATE(RMAT(NDIM,NDIM))
C Build transformation matrices
	CALL LIB_TRANSC(TC)
	CALL LIB_TRANSS(TS)
	CALL LIB_TRANSREL_LINP(TREL,L)
C SET UP REPRESENTATION
	DO ISPIN1=1,2
	DO ISPIN=1,2
	DO K=-L,L
	IDIM1=K+L+1+(2*L+1)*(ISPIN1-1)
	DO M=-L,L
	IDIM=M+L+1+(2*L+1)*(ISPIN-1)
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
C Cubic Harmonics
	IF(KSYS.EQ.1)THEN
	IF(ISPIN1.EQ.ISPIN)RMAT(IDIM1,IDIM)=TC(K,M+L+1,L)
	IF(ISPIN1.EQ.ISPIN)TMAT(IDIM1,IDIM)=TS(M+L+1,K,L)
	IF(ISPIN1.NE.ISPIN)RMAT(IDIM1,IDIM)=(0.D0,0.D0)
	IF(ISPIN1.NE.ISPIN)TMAT(IDIM1,IDIM)=(0.D0,0.D0)
C Spherical Harmonics
	ELSEIF(KSYS.EQ.2)THEN
	IF(IDIM1.EQ.IDIM)VAL=(1.D0,0.D0)
	IF(IDIM1.NE.IDIM)VAL=(0.D0,0.D0)
	RMAT(IDIM1,IDIM)=VAL
	TMAT(IDIM1,IDIM)=VAL
C Spinors
	ELSEIF(KSYS.EQ.3)THEN
	RMAT(IDIM1,IDIM)=TREL(IDIM,IDIM1) ! reciprocal is simply transposed, but transposition is done later
	TMAT(IDIM1,IDIM)=TREL(IDIM,IDIM1) ! indexation checked, first index in OTRAMAT= sph.harm, 
	ENDIF
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Call transformation routine
	CALL LIB_HUBMAT_T4(L,UINP,UOUT,TMAT)
C Memory deallocations
      DEALLOCATE(TMAT,TREL,RMAT)
      END