      SUBROUTINE LIB_LSQMOM(NGRD,XGRD,FVAL,NMOM,FMOM,IERR)
**********************************************************
*   Return least square fitted moments to the function F *
*   given on a grid of points as follows                 *
*        F(x)=Sum{i=0...N} M_i/x^i)                      *
**********************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 XGRD(NGRD),FMOM(0:NMOM)
      COMPLEX*16 FVAL(NGRD)
      REAL*8, ALLOCATABLE :: RPOL(:,:),FPOL(:)
      COMPLEX*16, ALLOCATABLE :: OMAT(:,:),WORK(:)
      IERR=0
      IF(MOD(NMOM,2).NE.1)IERR=1
      IF(MOD(NMOM,2).NE.1)RETURN
C **********************************     
C REAL PART: EVEN MOMENTS
      ALLOCATE(RPOL(NGRD,0:NMOM/2))
      ALLOCATE(OMAT(0:NMOM/2,0:NMOM/2))
      ALLOCATE(FPOL(0:NMOM/2))
      ALLOCATE(WORK(0:NMOM/2))
C Evaluate reciprocal polynomials      
      DO IMOM=0,NMOM/2
      DO IGRD=1,NGRD
      RPOL(IGRD,IMOM)=1.D0/XGRD(IGRD)**(2*IMOM)
      ENDDO
      ENDDO
C Evaluate overlap matrix for reciprocal polynomilas
      DO IMOM=0,NMOM/2
      DO JMOM=0,NMOM/2
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      DO IGRD=1,NGRD
      OMAT(IMOM,JMOM)=OMAT(IMOM,JMOM)+
     &DCMPLX(RPOL(IGRD,IMOM)*RPOL(IGRD,JMOM),0.D0)
      ENDDO      
      ENDDO      
      ENDDO      
C Evaluate overlap with the function
      DO IMOM=0,NMOM/2
      FPOL(IMOM)=0.D0
      DO IGRD=1,NGRD
      FPOL(IMOM)=FPOL(IMOM)+DREAL(FVAL(IGRD))*RPOL(IGRD,IMOM)
      ENDDO      
      ENDDO      
C Find reciprocal overlap matrix
      CALL LIB_CINV(NMOM/2+1,OMAT,NMOM/2+1,WORK,IFAIL)
      IF(IFAIL.EQ.1)THEN
      IERR=1
      DO IMOM=0,NMOM/2
      DO JMOM=0,NMOM/2
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      ENDDO
      ENDDO      
      ENDIF
C Find moments
      DO IMOM=0,NMOM/2
      FMOM(2*IMOM)=0.D0
      DO JMOM=0,NMOM/2
      FMOM(2*IMOM)=FMOM(2*IMOM)+DREAL(OMAT(IMOM,JMOM))*FPOL(JMOM)
      ENDDO
      ENDDO
      DEALLOCATE(RPOL,OMAT,FPOL,WORK)
C **********************************     
C IMAGINARY PART: ODD MOMENTS
      ALLOCATE(RPOL(NGRD,1:NMOM/2+1))
      ALLOCATE(OMAT(1:NMOM/2+1,1:NMOM/2+1))
      ALLOCATE(FPOL(1:NMOM/2+1))
      ALLOCATE(WORK(1:NMOM/2+1))
C Evaluate reciprocal polynomials      
      DO IMOM=1,NMOM/2+1
      DO IGRD=1,NGRD
      RPOL(IGRD,IMOM)=1.D0/XGRD(IGRD)**(2*IMOM-1)
      ENDDO
      ENDDO
C Evaluate overlap matrix for reciprocal polynomilas
      DO IMOM=1,NMOM/2+1
      DO JMOM=1,NMOM/2+1
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      DO IGRD=1,NGRD
      OMAT(IMOM,JMOM)=OMAT(IMOM,JMOM)+
     &DCMPLX(RPOL(IGRD,IMOM)*RPOL(IGRD,JMOM),0.D0)
      ENDDO      
      ENDDO      
      ENDDO      
C Evaluate overlap with the function
      DO IMOM=1,NMOM/2+1
      FPOL(IMOM)=0.D0
      DO IGRD=1,NGRD
      FPOL(IMOM)=FPOL(IMOM)+DIMAG(FVAL(IGRD))*RPOL(IGRD,IMOM)
      ENDDO      
      ENDDO      
C Find reciprocal overlap matrix
      CALL LIB_CINV(NMOM/2+1,OMAT,NMOM/2+1,WORK,IFAIL)
      IF(IFAIL.EQ.1)THEN
      IERR=1
      DO IMOM=1,NMOM/2+1
      DO JMOM=1,NMOM/2+1
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      ENDDO
      ENDDO      
      ENDIF
C Find moments
      DO IMOM=1,NMOM/2+1
      FMOM(2*IMOM-1)=0.D0
      DO JMOM=1,NMOM/2+1
      FMOM(2*IMOM-1)=FMOM(2*IMOM-1)+DREAL(OMAT(IMOM,JMOM))*FPOL(JMOM)
      ENDDO
      ENDDO
      DEALLOCATE(RPOL,OMAT,FPOL,WORK)
      END
      

      SUBROUTINE LIB_LSQMOM_FIXF(NGRD,XGRD,FVAL,NMOM,FMOM,IERR)
**********************************************************
*   Return least square fitted moments to the function F *
*   given on a grid of points as follows                 *
*        F(x)=Sum{i=0...N} M_i/x^i)                      *
*   assuming that M0 and M1 are given by the input.      *
**********************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 XGRD(NGRD),FMOM(0:NMOM)
      COMPLEX*16 FVAL(NGRD)
      REAL*8, ALLOCATABLE :: RPOL(:,:),FPOL(:),FFUN(:)
      COMPLEX*16, ALLOCATABLE :: OMAT(:,:),WORK(:)
      IERR=0
      IF(NMOM.EQ.1)RETURN
      IF(MOD(NMOM,2).NE.1)IERR=1
      IF(MOD(NMOM,2).NE.1)RETURN
C **********************************     
C REAL PART: EVEN MOMENTS
      ALLOCATE(RPOL(NGRD,1:NMOM/2))
      ALLOCATE(OMAT(1:NMOM/2,1:NMOM/2))
      ALLOCATE(FPOL(1:NMOM/2))
      ALLOCATE(WORK(1:NMOM/2))
      ALLOCATE(FFUN(NGRD))
C Evaluate reciprocal polynomials      
      DO IMOM=1,NMOM/2
      DO IGRD=1,NGRD
      RPOL(IGRD,IMOM)=1.D0/XGRD(IGRD)**(2*IMOM)
      ENDDO
      ENDDO
C Evaluate overlap matrix for reciprocal polynomilas
      DO IMOM=1,NMOM/2
      DO JMOM=1,NMOM/2
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      DO IGRD=1,NGRD
      OMAT(IMOM,JMOM)=OMAT(IMOM,JMOM)+
     &DCMPLX(RPOL(IGRD,IMOM)*RPOL(IGRD,JMOM),0.D0)
      ENDDO      
      ENDDO      
      ENDDO      
C Evaluate function
      DO IGRD=1,NGRD
      FFUN(IGRD)=DREAL(FVAL(IGRD))-FMOM(0)
      ENDDO
C Evaluate overlap with the function
      DO IMOM=1,NMOM/2
      FPOL(IMOM)=0.D0
      DO IGRD=1,NGRD
      FPOL(IMOM)=FPOL(IMOM)+FFUN(IGRD)*RPOL(IGRD,IMOM)
      ENDDO      
      ENDDO      
C Find reciprocal overlap matrix
      CALL LIB_CINV(NMOM/2,OMAT,NMOM/2,WORK,IFAIL)
      IF(IFAIL.EQ.1)THEN
      IERR=1
      DO IMOM=1,NMOM/2
      DO JMOM=1,NMOM/2
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      ENDDO
      ENDDO      
      ENDIF
C Find moments
      DO IMOM=1,NMOM/2
      FMOM(2*IMOM)=0.D0
      DO JMOM=1,NMOM/2
      FMOM(2*IMOM)=FMOM(2*IMOM)+DREAL(OMAT(IMOM,JMOM))*FPOL(JMOM)
      ENDDO
      ENDDO
      DEALLOCATE(RPOL,OMAT,FPOL,WORK,FFUN)
C **********************************     
C IMAGINARY PART: ODD MOMENTS
      ALLOCATE(RPOL(NGRD,2:NMOM/2+1))
      ALLOCATE(OMAT(2:NMOM/2+1,2:NMOM/2+1))
      ALLOCATE(FPOL(2:NMOM/2+1))
      ALLOCATE(WORK(2:NMOM/2+1))
      ALLOCATE(FFUN(NGRD))
C Evaluate reciprocal polynomials      
      DO IMOM=2,NMOM/2+1
      DO IGRD=1,NGRD
      RPOL(IGRD,IMOM)=1.D0/XGRD(IGRD)**(2*IMOM-1)
      ENDDO
      ENDDO
C Evaluate overlap matrix for reciprocal polynomilas
      DO IMOM=2,NMOM/2+1
      DO JMOM=2,NMOM/2+1
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      DO IGRD=1,NGRD
      OMAT(IMOM,JMOM)=OMAT(IMOM,JMOM)+
     &DCMPLX(RPOL(IGRD,IMOM)*RPOL(IGRD,JMOM),0.D0)
      ENDDO      
      ENDDO      
      ENDDO      
C Evaluate function
      DO IGRD=1,NGRD
      FFUN(IGRD)=DIMAG(FVAL(IGRD))-FMOM(1)/XGRD(IGRD)
      ENDDO
C Evaluate overlap with the function
      DO IMOM=2,NMOM/2+1
      FPOL(IMOM)=0.D0
      DO IGRD=1,NGRD
      FPOL(IMOM)=FPOL(IMOM)+FFUN(IGRD)*RPOL(IGRD,IMOM)
      ENDDO      
      ENDDO      
C Find reciprocal overlap matrix
      CALL LIB_CINV(NMOM/2,OMAT,NMOM/2,WORK,IFAIL)
      IF(IFAIL.EQ.1)THEN
      IERR=1
      DO IMOM=2,NMOM/2+1
      DO JMOM=2,NMOM/2+1
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      ENDDO
      ENDDO      
      ENDIF
C Find moments
      DO IMOM=2,NMOM/2+1
      FMOM(2*IMOM-1)=0.D0
      DO JMOM=2,NMOM/2+1
      FMOM(2*IMOM-1)=FMOM(2*IMOM-1)+DREAL(OMAT(IMOM,JMOM))*FPOL(JMOM)
      ENDDO
      ENDDO
      DEALLOCATE(RPOL,OMAT,FPOL,WORK,FFUN)
      END




      SUBROUTINE LIB_LSQMOM_TEST(NGRD,XGRD,FVAL,KMOM,NMOM,FMOM,IERR)
**********************************************************
*   Return least square fitted moments to the function F *
*   given on a grid of points as follows                 *
*        F(x)=Sum{i=k...N} M_i/x^i                       *
**********************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 FVAL(NGRD),FMOM(KMOM:NMOM),XGRD(NGRD)
      COMPLEX*16, ALLOCATABLE :: RPOL(:,:)
      COMPLEX*16, ALLOCATABLE :: OMAT(:,:),FPOL(:),WORK(:)
      ALLOCATE(RPOL(NGRD,KMOM:NMOM))
      ALLOCATE(OMAT(KMOM:NMOM,KMOM:NMOM))
      ALLOCATE(FPOL(KMOM:NMOM))
      ALLOCATE(WORK(KMOM:NMOM))
      IERR=0
C Evaluate reciprocal polynomials      
      DO IMOM=KMOM,NMOM
      DO IGRD=1,NGRD
      RPOL(IGRD,IMOM)=1.D0/XGRD(IGRD)**IMOM
      ENDDO
      ENDDO
C Evaluate overlap matrix for reciprocal polynomilas
      DO IMOM=KMOM,NMOM
      DO JMOM=KMOM,NMOM
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      DO IGRD=1,NGRD
      OMAT(IMOM,JMOM)=OMAT(IMOM,JMOM)+
     &DCONJG(RPOL(IGRD,IMOM))*RPOL(IGRD,JMOM)
      ENDDO      
      ENDDO      
      ENDDO      
C Evaluate overlap with the function
      DO IMOM=KMOM,NMOM
      FPOL(IMOM)=(0.D0,0.D0)
      DO IGRD=1,NGRD
      FPOL(IMOM)=FPOL(IMOM)+FVAL(IGRD)*DCONJG(RPOL(IGRD,IMOM))
      ENDDO      
      ENDDO      
C Find reciprocal overlap matrix
      CALL LIB_CINV(NMOM-KMOM+1,OMAT,NMOM-KMOM+1,WORK,IFAIL)
      IF(IFAIL.EQ.1)THEN
      IERR=1
      DO IMOM=KMOM,NMOM
      DO JMOM=KMOM,NMOM
      OMAT(IMOM,JMOM)=(0.D0,0.D0)
      ENDDO
      ENDDO      
      ENDIF
C Find moments
      DO IMOM=KMOM,NMOM
      FMOM(IMOM)=(0.D0,0.D0)
      DO JMOM=KMOM,NMOM
      FMOM(IMOM)=FMOM(IMOM)+OMAT(IMOM,JMOM)*FPOL(JMOM)
      ENDDO
      ENDDO
      DEALLOCATE(RPOL,OMAT,FPOL,WORK)
      END
