	SUBROUTINE LIB_GETCOMB(COMB,NMAX)
*******************************************************
*  Evaluates combinatoric array C(i,n)=n!/i!/(n-i)!)  *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION COMB(0:NMAX,0:NMAX)
	REAL*8, ALLOCATABLE :: DF(:)
C MEMORY ALLOCATIONS
	ALLOCATE(DF(0:NMAX))
	DF(0)=1.D0
	DO I=1,NMAX
	DF(I)=DF(I-1)*DBLE(I)
	ENDDO
C  N>0
	DO N=0,NMAX
	DO I=0,N
	COMB(I,N)=DF(N)/DF(I)/DF(N-I)
	ENDDO
	DO I=N+1,NMAX
	COMB(I,N)=0.D0
	ENDDO
	ENDDO
C MEMORY DEALLOCATIONS
	DEALLOCATE(DF)
	END	

	SUBROUTINE LIB_GETCONF(NREP,LREP,MREP,ICNF,NCNF,LCNF)
***************************************************************
* Returns configuration LCNF{n1...ng} corresponding to Icnf   *
***************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER LREP(NREP),MREP(0:NREP),LCNF(NREP)
	ICUR=ICNF-1
	DO IREP=NREP,1,-1
	LCNF(IREP)=ICUR/MREP(IREP-1)
	ICUR=ICUR-LCNF(IREP)*MREP(IREP-1)
	ENDDO
	END

	SUBROUTINE LIB_SETCONF(NREP,LREP,MREP,ICNF,NCNF,LCNF)
***************************************************************
* Returns Icnf corresponding to configuration LCNF{n1...ng}   *
***************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER LREP(NREP),MREP(0:NREP),LCNF(NREP)
	ICNF=1
	DO IREP=1,NREP
	ICNF=ICNF+LCNF(IREP)*MREP(IREP-1)
	ENDDO
	END


	REAL*8 FUNCTION FERMIFUN(E,T)
*******************************************************
*  Evaluates Fermi function                           *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	IF(T.GT.0.D0)THEN
	FERMIFUN=1.D0/(SAFEEXP(E/T)+1.D0)
	ELSEIF(T.EQ.0.D0.AND.E.LT.0.D0)THEN
	FERMIFUN=1.D0
	ELSEIF(T.EQ.0.D0.AND.E.EQ.0.D0)THEN
	FERMIFUN=0.5D0
	ELSEIF(T.EQ.0.D0.AND.E.GT.0.D0)THEN
	FERMIFUN=0.D0
	ENDIF
	END




	REAL*8 FUNCTION SAFEEXP(POW)
*******************************************************
*  Safely evaluates expontent e(pow)                  *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DATA POWMAX/60.D0/
	IF(POW.GT.POWMAX)THEN
	SAFEEXP=EXP(POWMAX)	
	ELSEIF(POW.LT.-POWMAX)THEN
	SAFEEXP=EXP(-POWMAX)
	ELSE
	SAFEEXP=EXP(POW)
	ENDIF
	END



	SUBROUTINE LIB_SQUAREROOT(Z,SQUAREROOT)
*******************************************************
*  Evaluates Fermi function                           *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 Z,SQPH,SQUAREROOT
	DATA PI/3.141592653589793D0/
	SQUAREROOT=(0.D0,0.D0)
	ABSZ=ABS(Z)
	IF(ABSZ.LE.1.D-14)RETURN
	SQAZ=SQRT(ABS(Z))
	CPH=DREAL(Z)/ABSZ
	SPH=DIMAG(Z)/ABSZ
	IF(SPH.GE.0.D0)PHS=+ACOS(CPH)
	IF(SPH.LT.0.D0)PHS=-ACOS(CPH)+2.D0*PI
	SQPH=EXP((0.D0,1.D0)*PHS/2.D0)
	IF(DREAL(Z).GE.0.D0.AND.DIMAG(Z).GE.0.D0)SIGN=+1.D0
	IF(DREAL(Z).LT.0.D0.AND.DIMAG(Z).GE.0.D0)SIGN=+1.D0
	IF(DREAL(Z).GE.0.D0.AND.DIMAG(Z).LT.0.D0)SIGN=+1.D0
	IF(DREAL(Z).LT.0.D0.AND.DIMAG(Z).LT.0.D0)SIGN=+1.D0
	SQUAREROOT=SIGN*SQAZ*SQPH
	END



	SUBROUTINE LIB_PARAROOT(F1,F2,F3,X1,X2,X3,RR1,RI1,RR2,RI2)
******************************************************* 
*   Returns zeroes of parabola                        *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 R1,R2
C PARABOLIC INTERPOLATION FORMULA
C	F3FUN(X,F1,F2,F3,X1,X2,X3)=
C    &  (X-X2)*(X-X3)/(X1-X2)/(X1-X3)*F1+
C    &  (X-X1)*(X-X3)/(X2-X1)/(X2-X3)*F2+
C    &  (X-X1)*(X-X2)/(X3-X1)/(X3-X2)*F3
	IF(ABS(X1-X2).GT.1.D-10.AND.
     &   ABS(X1-X3).GT.1.D-10.AND.
     &   ABS(X2-X3).GT.1.D-10)THEN
	A=1.D0       /(X1-X2)/(X1-X3)*F1+
     &  1.D0       /(X2-X1)/(X2-X3)*F2+
     &  1.D0       /(X3-X1)/(X3-X2)*F3
	B=(-X2-X3)   /(X1-X2)/(X1-X3)*F1+
     &  (-X1-X3)   /(X2-X1)/(X2-X3)*F2+
     &  (-X1-X2)   /(X3-X1)/(X3-X2)*F3
	C=(-X2)*(-X3)/(X1-X2)/(X1-X3)*F1+
     &  (-X1)*(-X3)/(X2-X1)/(X2-X3)*F2+
     &  (-X1)*(-X2)/(X3-X1)/(X3-X2)*F3
	ELSEIF(ABS(X1-X2).LE.1.D-10)THEN
	A=0.D0
	B=(F1-F3) ! /(X1-X3)
	C=(F3*X1-F1*X3) ! /(X1-X3)
	ELSEIF(ABS(X1-X3).LE.1.D-10)THEN
	A=0.D0
	B=(F2-F3) ! /(X2-X3)
	C=(F3*X2-F2*X3) ! /(X2-X3)
	ELSEIF(ABS(X2-X3).LE.1.D-10)THEN
	A=0.D0
	B=(F1-F2) ! /(X1-X2)
	C=(F2*X1-F1*X2) ! /(X1-X2)
	ELSE
	CALL FULLSTOP('<LIB_PARAROOT>:: Degeneracy')
	ENDIF
C
	IF(ABS(A).GE.1.D-10)THEN
	R1=(-B+SQRT(DCMPLX(B**2-4.D0*A*C,0.D0)))/2.D0/A
	R2=(-B-SQRT(DCMPLX(B**2-4.D0*A*C,0.D0)))/2.D0/A
	ELSE
	IF(ABS(B).GE.1.D-10)
     &R1=DCMPLX(-C/B,0.D0)
	IF(ABS(B).LT.1.D-10)
     &R1=DCMPLX(1.D+10,0.D0)
	R2=DCMPLX(1.D+10,0.D0)
	ENDIF
	RR1=DREAL(R1)
	RR2=DREAL(R2)
	RI1=DIMAG(R1)
	RI2=DIMAG(R2)
	END


	SUBROUTINE LIB_FINDROOT(ROOT,DIFF,R1,R2,ITER,IEXIT,IUN,EPS,NITER)
********************************************************************
*  Finds approximate root by cubic interpolation at approximately  *
*  given interval from R1 to R2. If no root is found the interval  *
*  is extended until root is located.                              *
********************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	SAVE
	DATA PI/3.141592653589793D0/,NSTEP/0/
C START CODE: 
	IEXIT=0
	IF(ABS(R2-R1).LE.1.D-10)
     &CALL FULLSTOP('<LIB_INTROOT> :: Zero interval set')
  100 STEP=(R2-R1)/2.D0*DBLE(NSTEP)
C Check first
	IF(ITER.EQ.0)THEN
	ROOT=R1-STEP
	RT1=ROOT
	RETURN
	ENDIF
C If iter=1, check doping
	IF(ITER.EQ.1)THEN
	DF1=DIFF
	ROOT=R2+STEP
	RT2=ROOT
	RETURN
	ENDIF
	IF(ITER.EQ.2)DF2=DIFF
C If there is no root between R1 and R2, increase step
	IF(DF2*DF1.GE.0.D0.AND.ITER.EQ.2)THEN
	ITER=0
	NSTEP=NSTEP+1
	IF(NSTEP.GT.NITER)THEN
	WRITE(IUN,10010)R1-STEP,R2+STEP
10010	FORMAT(1X,'Cannot locate root, interval = ',G12.5,1X,G12.5)
	IEXIT=1
	NSTEP=0
	IF(ABS(DF1).LE.ABS(DF2))ROOT=RT1
	IF(ABS(DF1).GT.ABS(DF2))ROOT=RT2
	RETURN
	ENDIF
	GOTO 100
	ENDIF
C Root exists, try to locate it by linear/parabolic interpolations
	IF(ITER.EQ.2)THEN
	RTNEW=(DF2*RT1-DF1*RT2)/(DF2-DF1)
	ELSEIF(ITER.GT.2)THEN
	CALL LIB_PARAROOT(DF1,DIFF,DF2,RT1,ROOT,RT2,RR1,RI1,RR2,RI2)
	IF(RR1.GE.RT1.AND.RR1.LE.RT2.AND.ABS(RI1).LE.1.D-10)THEN
      RTNEW=RR1
	ELSEIF(RR2.GE.RT1.AND.RR2.LE.RT2.AND.ABS(RI2).LE.1.D-10)THEN
      RTNEW=RR2
	ELSE ! exit on error
	RTNEW=ROOT
	ENDIF
	ENDIF
C Save data
      IF(DIFF*DF1.GE.0.D0)THEN
	RT1=ROOT
	DF1=DIFF
	ELSEIF(DIFF*DF2.GE.0.D0)THEN
	RT2=ROOT
	DF2=DIFF
	ENDIF
C  CHECK FOR CONVERGENCY
	IF(ABS(RTNEW-ROOT).GT.EPS.OR .ABS(DIFF).GT.EPS)IEXIT=0
	IF(ABS(RTNEW-ROOT).LE.EPS.AND.ABS(DIFF).LE.EPS)IEXIT=1
	IF(ITER.EQ.NITER)THEN
	WRITE(IUN,10020)RTNEW,ROOT
10020	FORMAT(1X,'No convergency in Root = ',G12.5,1X,G12.5)
	IEXIT=1
	ENDIF
	ROOT=RTNEW
	NSTEP=0
	END
