      SUBROUTINE LIB_ORDER_IMAX(NBND,E1,E2,IO,NORD,NCUT,EPS)
********************************************************************
* Orders complex energies by matching maximal imaginary parts      *
********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER IO(NBND)
      COMPLEX*16 E1(NBND),E2(NBND),S
      INTEGER, ALLOCATABLE :: I1(:),I2(:),K1(:),K2(:),LO(:),KO(:),LB(:)
      COMPLEX*16, ALLOCATABLE :: C1(:),C2(:),O1(:),O2(:),T2(:)
C Returns
      IF(NCUT.LE.0)RETURN
C Memory Allocations
      ALLOCATE(I1(NBND))
      ALLOCATE(I2(NBND))
      ALLOCATE(K1(NBND))
      ALLOCATE(K2(NBND))
      ALLOCATE(C1(NBND))
      ALLOCATE(C2(NBND))
C Sort energies by increasing their imaginary parts
C C1 and C2 are arrays ordered with imaginary parts
C I1 and I2 are ordering pointers
      DO IBND=1,NBND
      I1(IBND)=IBND
      I2(IBND)=IBND
      C1(IBND)=E1(IBND)
      C2(IBND)=E2(IBND)
      ENDDO
      DO IBND=1,NBND
	IF(I1(IBND).EQ.0)CYCLE
      DO JBND=IBND+1,NBND
	IF(I1(JBND).EQ.0)CYCLE
	IF(DIMAG(C1(IBND)).GT.DIMAG(C1(JBND)))THEN
	S=C1(IBND)
	C1(IBND)=C1(JBND)
	C1(JBND)=S
	I=I1(IBND)
	I1(IBND)=I1(JBND)
	I1(JBND)=I
	ENDIF
	ENDDO
	ENDDO
      DO IBND=1,NBND
	IF(I2(IBND).EQ.0)CYCLE
      DO JBND=IBND+1,NBND
	IF(I2(JBND).EQ.0)CYCLE
	IF(DIMAG(C2(IBND)).GT.DIMAG(C2(JBND)))THEN
	S=C2(IBND)
	C2(IBND)=C2(JBND)
	C2(JBND)=S
	I=I2(IBND)
	I2(IBND)=I2(JBND)
	I2(JBND)=I
	ENDIF
	ENDDO
	ENDDO
C Set pointers K1 and K2 reciprocal to I1 and I2
      DO IBND=1,NBND
      K1(I1(IBND))=IBND
      K2(I2(IBND))=IBND
      ENDDO
C All bands below Ncut should be ordered now
      ALLOCATE(O1(NCUT))
      ALLOCATE(O2(NCUT))
      ALLOCATE(LO(NCUT))
      ALLOCATE(LB(NCUT))
      ICUT=0
      DO IBND=1,NBND
      IF(K1(IBND).GT.NCUT)CYCLE
      ICUT=ICUT+1
      O1(ICUT)=E1(IBND)
      ENDDO
      ICUT=0
      DO IBND=1,NBND
      IF(K2(IBND).GT.NCUT)CYCLE
      ICUT=ICUT+1
      O2(ICUT)=E2(IBND)
      LO(ICUT)=ICUT
      LB(ICUT)=IBND
      ENDDO
      CALL LIB_ORDER_FUNC(NCUT,O1,O2,LO,NORD,EPS)
C Place ordered bands into original array
      ALLOCATE(T2(NBND))
      ALLOCATE(KO(NBND))
      MBND=1
      LBND=1
      ICUT2=0
      DO IBND1=1,NBND
      IF(K1(IBND1).GT.NCUT)THEN
      DO IBND2=MBND,NBND
      IF(K2(IBND2).LE.NCUT)CYCLE
      T2(IBND1)=E2(IBND2)
      KO(IBND1)=IO(IBND2)
      MBND=IBND2+1
      EXIT
      ENDDO
      ELSEIF(K1(IBND1).LE.NCUT)THEN
      DO IBND2=LBND,NBND
      IF(K2(IBND2).GT.NCUT)CYCLE
      ICUT2=ICUT2+1
      KCUT2=LO(ICUT2)
      KBND2=LB(KCUT2)
      T2(IBND1)=E2(KBND2)
      KO(IBND1)=IO(KBND2)
      LBND=IBND2+1
      EXIT
      ENDDO
      ENDIF
      ENDDO
      DO IBND=1,NBND
      E2(IBND)=T2(IBND)
      IO(IBND)=KO(IBND)
      ENDDO
C Memory deallocations
      DEALLOCATE(I1,I2,K1,K2,C1,C2,T2,LO,KO,LB)
      END



      SUBROUTINE LIB_ORDER_FUNC(NBND,E1,E2,IO,NORD,EPS)
********************************************************************
* Orders complex energies by minimizing the mean square difference *
* Most general scheme, however makes sense for NORD less than 5-8  *
* since time grows as NORD!                                        *
********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER IO(NBND)
      COMPLEX*16 E1(NBND),E2(NBND)
      INTEGER, ALLOCATABLE :: I2(:),LINK(:,:)
      COMPLEX*16, ALLOCATABLE :: C1(:),C2(:)
      NDEP=NORD
      IF(NDEP.GT.NBND)NDEP=NBND
C Get links
	NCNF=1
	DO I=1,NDEP
	NCNF=NCNF*I
	ENDDO
	ALLOCATE(LINK(NDEP,NCNF))
      CALL LIB_ORDER_LINK(NDEP,NCNF,LINK)
C Start looping over stacks
      ALLOCATE(C1(NDEP))
      ALLOCATE(C2(NDEP))
      ALLOCATE(I2(NDEP))
      DO 1000 IBND=1,NBND-NDEP+1
      DO IDEP=1,NDEP
      C1(IDEP)=E1(IDEP+IBND-1)
      C2(IDEP)=E2(IDEP+IBND-1)
      I2(IDEP)=IO(IDEP+IBND-1)
      ENDDO
C Skip links with similar imaginary parts
      IF(ABS(DIMAG(C1(1)-C2(1))).LE.EPS.AND.IBND.LT.NBND-NDEP+1)CYCLE
C Order arrays by minimization
      CALL LIB_ORDER_DATA(C1,C2,I2,NDEP,NCNF,LINK)
      DO IDEP=1,NDEP
      E1(IDEP+IBND-1)=C1(IDEP)
      E2(IDEP+IBND-1)=C2(IDEP)
      IO(IDEP+IBND-1)=I2(IDEP)
      ENDDO
 1000 CONTINUE
      DEALLOCATE(C1,C2,I2,LINK)
      END
      
      SUBROUTINE LIB_ORDER_LINK(NDEP,NCNF,LINK)
********************************************************************
* Returns all possible links between two arrays                    *
********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER LINK(NDEP,NCNF)
      INTEGER, ALLOCATABLE :: LINK1(:,:), LINK2(:,:)
      ALLOCATE(LINK1(1,1))
      KCNF1=1
      KDEP1=1
      LINK1(1,1)=1
C Start iterating
      DO KDEP2=2,NDEP
      KCNF2=1
      DO IDEP=1,KDEP2
      KCNF2=KCNF2*IDEP
      ENDDO
      ALLOCATE(LINK2(KDEP2,KCNF2))
C Set up link2 from link1
      ICNF2=0
      DO ICNF1=1,KCNF1
      DO IDEP2=1,KDEP2
      ICNF2=ICNF2+1
      LINK2(IDEP2,ICNF2)=KDEP2
      DO IDEP1=1,KDEP1
      IF(IDEP1.LT.IDEP2)I=0
      IF(IDEP1.GE.IDEP2)I=1
      LINK2(IDEP1+I,ICNF2)=LINK1(IDEP1,ICNF1)
      ENDDO
      ENDDO
      ENDDO
C Prepare for next iteration
      DEALLOCATE(LINK1)
      ALLOCATE(LINK1(KDEP2,KCNF2))
      DO ICNF=1,KCNF2
      DO IDEP=1,KDEP2
      LINK1(IDEP,ICNF)=LINK2(IDEP,ICNF)
      ENDDO
      ENDDO
      DEALLOCATE(LINK2)
      KCNF1=KCNF2
      KDEP1=KDEP2
      ENDDO
C Set output array
      ICNF0=0
      DO ICNF=NCNF,1,-1
      ICNF0=ICNF0+1
      DO IDEP=1,NDEP
      LINK(IDEP,ICNF0)=LINK1(IDEP,ICNF)
      ENDDO
      ENDDO
      DEALLOCATE(LINK1)
      END      
      
      
      SUBROUTINE LIB_ORDER_DATA(C1,C2,I2,NDEP,NCNF,LINK)
********************************************************************
* Orders complex energies by minimizing the mean square difference 
********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER LINK(NDEP,NCNF),I2(NDEP)
      COMPLEX*16 C1(NDEP),C2(NDEP)
      INTEGER, ALLOCATABLE :: L2(:)
      REAL*8, ALLOCATABLE :: FUNC(:)
      COMPLEX*16, ALLOCATABLE :: T2(:)
      ALLOCATE(FUNC(NCNF))
      DO ICNF=1,NCNF
C Generate functional data
      FUNC(ICNF)=0.D0
      DO IDEP=1,NDEP
      ILIN=LINK(IDEP,ICNF)
      FUNC(ICNF)=FUNC(ICNF)+DREAL(C1(IDEP)-C2(ILIN))**2
     &                     +DIMAG(C1(IDEP)-C2(ILIN))**2
      ENDDO
      ENDDO
C Find minimum
      FMIN=FUNC(1)
      IMIN=1
      DO ICNF=2,NCNF
      IF(FUNC(ICNF).GT.FMIN)CYCLE
      IMIN=ICNF
      FMIN=FUNC(ICNF)
      ENDDO
C Reorder energies according to configuration
      ALLOCATE(T2(NDEP))
      ALLOCATE(L2(NDEP))
      DO IDEP=1,NDEP
      ILIN=LINK(IDEP,IMIN)
      T2(IDEP)=C2(ILIN)
      L2(IDEP)=I2(ILIN)
      ENDDO
      DO IDEP=1,NDEP
      C2(IDEP)=T2(IDEP)
      I2(IDEP)=L2(IDEP)
      ENDDO
      DEALLOCATE(FUNC,T2,L2)
      END