	SUBROUTINE LIB_PCOEFS(MPOL,RPOL,APOL)
******************************************************************
* This subroutine returns polynomial coefficients Ai             *
* for the polynomial: Sum (i=0,m) Ai X**i which is represented   *
* as (X-R1)(X-R2)...(X-Rm)                                       *
******************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
C Passed arrays
	COMPLEX*16 RPOL(MPOL),APOL(0:MPOL)
C Allocatable arrays
	INTEGER, ALLOCATABLE :: ISTATE(:,:)
	REAL*8, ALLOCATABLE :: COMB(:,:)
	COMPLEX*16 DIM,PROD
	DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/,EPS/1.D-04/
C *************************************
	IF(MPOL.EQ.0)RETURN
C Get combinatorics
	ALLOCATE(COMB(0:MPOL,0:MPOL))
	CALL LIB_GETCOMB(COMB,MPOL)
C CREATE FULL FOCK SPACE OF DIMENSION 2**mPol
	NDIM=2**MPOL
	ALLOCATE(ISTATE(MPOL,0:NDIM-1))
	DO IROT=1,MPOL
	ISTATE(IROT,0)=0
	ENDDO
C Represent IVEC using binary system
	DO IVEC=1,NDIM-1
	ITWO=1
	IMAX=0
	DO WHILE(IVEC.GE.ITWO)
	ITWO=ITWO*2
	IMAX=IMAX+1
	ENDDO
	ISTATE(IMAX,IVEC)=1
	DO IROT=IMAX+1,MPOL
	ISTATE(IROT,IVEC)=0
	ENDDO
	INUM=IVEC
	DO I=IMAX-1,1,-1
	ITWO=ITWO/2
	INUM=INUM-ITWO*ISTATE(I+1,IVEC)
	ISTATE(I,IVEC)=INUM/(ITWO/2)
	ENDDO
	ENDDO
C Check algorithm
	DO IFIL=0,MPOL
	ICHK=NINT(COMB(IFIL,MPOL))
	IFOC=0
	DO IVEC=0,NDIM-1
	IOCC=0
	DO IROT=1,MPOL
	IOCC=IOCC+ISTATE(IROT,IVEC)
	ENDDO
	IF(IOCC.EQ.IFIL)IFOC=IFOC+1
	ENDDO
	IF(IFOC.NE.ICHK)CALL FULLSTOP('iFoc.ne.iChk')
	ENDDO
C Set coefficients to unity
	DO IPOL=0,MPOL
	APOL(IPOL)=(0.D0,0.D0)
	ENDDO
C Looping over the states
	DO IVEC=0,NDIM-1
C Occupancy shows which polynomial coefficient is being computed
	IOCC=0
	DO IROT=1,MPOL
	IOCC=IOCC+ISTATE(IROT,IVEC)
	ENDDO
	IPOL=MPOL-IOCC
	PROD=(1.D0,0.D0)
	DO IROT=1,MPOL
	IF(ISTATE(IROT,IVEC).EQ.0)CYCLE
	PROD=PROD*RPOL(IROT)
	ENDDO
	APOL(IPOL)=APOL(IPOL)+PROD
	ENDDO
C Account for the sign
	DO IPOL=0,MPOL
	APOL(IPOL)=APOL(IPOL)*(-1)**(MPOL-IPOL)
	ENDDO
C Memory deallocations
	DEALLOCATE(COMB,ISTATE)
	END