      SUBROUTINE LIB_PZEROS(MPOL,PCOF,PZER,TOI,IFAIL)    
************************************************************************
*     All roots for the polynomial: Sum(i=0,M) P(i)*x**i=0
************************************************************************
C
C     THIS SUBROUTINE ATTEMPTS TO SOLVE THE COMPLEX POLYNOMIAL  
C     EQUATION  
C     (AR(1)+IAC(1))Z (N-1)+(AR(2)+IAC(2))Z 
C     (N-2)+....AR(N)+IAC(N)=0  
C     USING THE SEARCH ALGORITHM PROPOSED IN GRANT AND HITCHINS 
C     (1971)    
C     TO MAXIMUM MACHINE PRECISION. ON NORMAL ENTRY, THE PARAMETER  
C     IFAIL HAS VALUE 0 (HARD FAIL) OR 1 (SOFT FAIL) AND WILL BE    
C     ZERO  
C     ON SUCCESSFUL EXIT WITH THE CALCULATED ESTIMATES OF THE ROOTS 
C     HELD  IN REZ(K)+IIMZ(K), K=1(1)N-1, IN APPROXIMATE DECREASING 
C     ORDER OF MODULUS.  THE PARAMETER TOL GIVES THE PRECISION OF   
C     THE   
C     FLOATING BINARY ARITHMETIC USED, NORMALLY 2 (-T), WHERE T IS  
C     THE   
C     NUMBER OF BITS IN THE MANTISSA.   
C     ABNORMAL EXITS WILL BE INDICATED BY IFAIL HAVING VALUE    
C     1 OR 2.  THE FORMER IMPLIES THAT EITHER AR(1)=0 AND AC(1)=0,  
C     OR N.LT.2 OR N.GT.100.  FOR IFAIL = 2 A POSSIBLE SADDLE   
C     POINT HAS BEEN DETECTED.  
C     THE NUMBER OF COEFFICIENTS OF THE REDUCED POLYNOMIAL IS   
C     STORED    
C     IN N AND ITS COEFFICIENTS ARE HELD IN AR(K)+IAC(K), K=1(1)N,  
C     THE   
C     ROOTS OBTAINED THUS FAR BEING STORED IN THE ARRAYS REZ AND    
C     IMZ   
C     STARTING WITH REZ(N)+IIMZ(N).  AN IMMEDIATE RE-ENTRY IS   
C     POSSIBLE  
C     WITH IFAIL UNCHANGED AND WITH A NEW STARTING POINT FOR THE    
C     SEARCH    
C     HELD IN REZ(1)+IIMZ(1).   
C     REF - J.I.M.A., VOL.8., PP 122-129 (1971).    
      INTEGER IFAIL, FC, IND, N, I, K, II, JTEMP
      DOUBLE PRECISION A1P5, FAC, FUN, G, XXX, J, JX, NFUN, ONE, P1,  
     *P2Z1, P3Z2, P4Z1, P5, R, RX, S, SCALE, SIG, S1, S2,T, TOL, TOI,
     *TOL2, TWO, X, Y, ZERO    
	COMPLEX*16 PCOF(0:MPOL),PZER(MPOL),CSUM
	REAL*8, ALLOCATABLE :: AC(:)
	REAL*8, ALLOCATABLE :: AR(:)
	REAL*8, ALLOCATABLE :: REZ(:)
	REAL*8, ALLOCATABLE :: IMZ(:)
	REAL*8, ALLOCATABLE :: BC(:)
	REAL*8, ALLOCATABLE :: BR(:)
	REAL*8, ALLOCATABLE :: CC(:)
	REAL*8, ALLOCATABLE :: CR(:)
C     *** IMPLEMENTATION DEPENDENT DECLARATION ***  
C     DOUBLE PRECISION DFLOAT   
      LOGICAL SAT, FIRSTR, FLAG 
      DATA ONE /1.0D0/, A1P5 /1.5D0/, ZERO /0.0D0/, P4Z1 /1.0D-5/   
      DATA TWO /2.0D0/, P5 /0.5D0/, P2Z1 /1.0D-3/, P1 /0.1D0/   
      DATA P3Z2 /2.0D-4/
C TOLerance can be modified
      TOL=TOI   
C Machine accuracy
      DO I=-25,1
      XXX=10.D0**I
      IF(1.D0+XXX.NE.1.D0)EXIT
      ENDDO
      IF (TOL.LT.XXX) TOL = XXX 
C     THE ABOVE TEST WAS ADDED AT 4.5 TO PREVENT TOL BEING TOO  
C     SMALL 
	N=MPOL+1
C Memory Allocations
	ALLOCATE(AR(N))
	ALLOCATE(AC(N))
	ALLOCATE(REZ(N))
	ALLOCATE(IMZ(N))
	ALLOCATE(BC(2*N))
	ALLOCATE(BR(2*N))
	ALLOCATE(CC(2*N))
	ALLOCATE(CR(2*N))
C Real Imaginary parts
	DO I=1,N
	AR(I)=DREAL(PCOF(N-I))
	AC(I)=DIMAG(PCOF(N-I))
	ENDDO
      FAC = ONE 
      FLAG = IFAIL.EQ.2 
      IF (FLAG) IFAIL = 1   
      IND = 0   
      TOL2 = TOL**A1P5  
      IF ((AR(1).NE.ZERO .OR. AC(1).NE.ZERO) .AND. N.GE.2 .AND.N.LE.100)    
     * GO TO 20 
      IND = 1
      GO TO 600 
   20 IF (N.EQ.2) GO TO 580 
      IF (AR(N).NE.ZERO .OR. AC(N).NE.ZERO) GO TO 40    
      REZ(N-1) = ZERO   
      IMZ(N-1) = ZERO   
      N = N - 1 
      GO TO 20  
   40 SCALE = ZERO  
      DO 100 I=1,N  
         IF (AR(I).EQ.ZERO .AND. AC(I).EQ.ZERO) GO TO 100   
         IF (DABS(AR(I)).GT.DABS(AC(I))) GO TO 60   
         FUN = DABS(AC(I))*DSQRT(ONE+(AR(I)/AC(I))**2)  
         GO TO 80   
   60    FUN = DABS(AR(I))*DSQRT(ONE+(AC(I)/AR(I))**2)  
   80    IF (FUN.GE.P4Z1) SCALE = DLOG(FUN) + SCALE 
  100 CONTINUE  
      K = IFIX(SNGL(SCALE/(DFLOAT(N)*DLOG(TWO)+P5)))    
      SCALE = TWO**(-K) 
      DO 120 I=1,N  
         AR(I) = AR(I)*SCALE    
         BR(I) = AR(I)  
         AC(I) = AC(I)*SCALE    
         BC(I) = AC(I)  
  120 CONTINUE  
  140 I = N + 1 
  160 I = I - 1 
      IF (I.LT.2) GO TO 220 
      DO 180 K=2,I  
         II = I - K + 1 
         CR(K-1) = BR(I)*BR(K) + BC(I)*BC(K) - BR(1)*BR(II) -BC(1)* 
     *   BC(II) 
         CC(K-1) = BR(I)*BC(K) - BC(I)*BR(K) + BR(1)*BC(II) -BC(1)* 
     *   BR(II) 
  180 CONTINUE  
      IF (CR(I-1).LT.-TOL) GO TO 260    
      T = ONE   
      IF (DABS(CR(I-1)).GE.ONE) T = ONE/DABS(CR(I-1))   
      JTEMP = I - 1 
      DO 200 K=1,JTEMP  
         BR(K) = CR(K)*T    
         BC(K) = CC(K)*T    
  200 CONTINUE  
      GO TO 160 
  220 FAC = FAC*TWO 
      SCALE = ONE   
      JTEMP = N - 1 
      DO 240 II=1,JTEMP 
         I = N - II 
         SCALE = TWO*SCALE  
         AR(I) = SCALE*AR(I)    
         BR(I) = AR(I)  
         AC(I) = SCALE*AC(I)    
         BC(I) = AC(I)  
  240 CONTINUE  
      GO TO 140 
  260 SAT = .FALSE. 
      FC = 1    
      IF (FLAG) GO TO 280   
      X = P2Z1  
      Y = P1    
      GO TO 300 
  280 X = REZ(1)    
      Y = IMZ(1)    
      FLAG = .FALSE.    
  300 FIRSTR = .TRUE.   
      CALL LIB_PZEROS_EVAL(AR,AC,N,TOL,X,Y,R,RX,J,JX,SAT)   
      FUN = R*R + J*J   
  320 FC = FC + 1   
      SAT = FC.GT.20    
      G = RX*RX + JX*JX 
      IF (G.GE.FUN*TOL2) GO TO 360  
  330 IND = 2
      SCALE = ONE   
      JTEMP = N - 1 
      DO 340 II=1,JTEMP 
         I = N - II 
         SCALE = SCALE*FAC  
         AR(I) = AR(I)/SCALE    
         AC(I) = AC(I)/SCALE    
  340 CONTINUE  
      GO TO 600 
  360 S1 = -(R*RX+J*JX)/G   
      S2 = (R*JX-J*RX)/G    
      SIG = P3Z2    
      S = DSQRT(S1*S1+S2*S2)    
      IF (S.LE.ONE) GO TO 380   
      S1 = S1/S 
      S2 = S2/S 
      SIG = SIG/S   
C     VALID DIRECTION OF SEARCH HAS BEEN DETERMINED, NOW PROCEED    
C     TO DETERMINE SUITABLE STEP    
  380 X = X + S1    
      Y = Y + S2    
  400 FC = FC + 1   
      SAT = (S.LT.P4Z1) .OR. FC.GT.20   
      CALL LIB_PZEROS_EVAL(AR,AC,N,TOL,X,Y,R,RX,J,JX,SAT)
      IF (SAT) GO TO 440    
      NFUN = R*R + J*J  
      IF (FUN-NFUN.GE.SIG*FUN) GO TO 420    
      S1 = S1*P5    
      S2 = S2*P5    
      IF (DABS(S1).LE.XXX*DABS(X) .AND. DABS(S2).LE.XXX*DABS(Y))    
     * GO TO 330    
      S = S*P5  
      SIG = SIG*P5  
      X = X - S1    
      Y = Y - S2    
      GO TO 400 
  420 FUN = NFUN    
      GO TO 320 
  440 REZ(N-1) = X*FAC  
      IMZ(N-1) = Y*FAC  
      N = N - 1 
C     FORWARD AND BACKWARD DEFLATIONS ARE PERFORMED WITH THE    
C     COMPLEX ROOT  
      S = X*X + Y*Y 
      BR(1) = AR(1) 
      BC(1) = AC(1) 
      CC(N) = (Y*AR(N+1)-X*AC(N+1))/S   
      CR(N) = -(X*AR(N+1)+Y*AC(N+1))/S  
      DO 460 I=2,N  
         BR(I) = AR(I) + X*BR(I-1) - Y*BC(I-1)  
         BC(I) = AC(I) + X*BC(I-1) + Y*BR(I-1)  
         II = N - I + 1 
         CR(II) = (-X*(AR(II+1)-CR(II+1))-Y*(AC(II+1)-CC(II+1)))/S  
         CC(II) = (Y*(AR(II+1)-CR(II+1))-X*(AC(II+1)-CC(II+1)))/S   
  460 CONTINUE  
      FUN = ONE/TOL2    
      K = 0 
C     MATCHING POINT FOR COMPOSITE DEFLATION EVALUATED  
      DO 480 I=1,N  
         NFUN = DABS(BR(I)) + DABS(CR(I)) + DABS(BC(I)) + DABS(CC(I))   
         IF (NFUN.LE.TOL) GO TO 480 
         NFUN = (DABS(BR(I)-CR(I))+DABS(BC(I)-CC(I)))/NFUN  
         IF (NFUN.GE.FUN) GO TO 480 
         FUN = NFUN 
         K = I  
  480 CONTINUE  
      IF (K.EQ.1) GO TO 520 
      IF (K.EQ.1) GO TO 520 
      JTEMP = K - 1 
      DO 500 I=1,JTEMP  
         AR(I) = BR(I)  
         AC(I) = BC(I)  
  500 CONTINUE  
  520 AR(K) = P5*(BR(K)+CR(K))  
      AC(K) = P5*(BC(K)+CC(K))  
      IF (K+1.GT.N) GO TO 560   
      JTEMP = K + 1 
      DO 540 I=JTEMP,N  
         AR(I) = CR(I)  
         AC(I) = CC(I)  
  540 CONTINUE  
  560 IF (N.EQ.2) GO TO 580 
      IF (.NOT.FIRSTR) GO TO 40 
C     CHECK FOR COMPLEX CONJUGATE BEING A ROOT  
      Y = -Y    
      FIRSTR = .FALSE.  
      CALL LIB_PZEROS_EVAL(AR,AC,N,TOL,X,Y,R,RX,J,JX,SAT)
      IF (SAT) GO TO 440    
      GO TO 40  
  580 N = 1 
      FUN = AR(1)**2 + AC(1)**2 
      REZ(1) = -(AR(1)*AR(2)+AC(1)*AC(2))*FAC/FUN   
      IMZ(1) = (AR(2)*AC(1)-AR(1)*AC(2))*FAC/FUN    
  600 IFAIL = IND   
C Return in complex format
	DO I=1,MPOL
	PZER(I)=DCMPLX(REZ(I),IMZ(I))
	ENDDO
C Check Zeros
	DO I=1,MPOL
	CSUM=PCOF(0)
	DO IPOL=1,MPOL
	CSUM=CSUM+PCOF(IPOL)*PZER(I)**IPOL
	ENDDO
	IF(ABS(CSUM).GT.1.D-08.AND.IFAIL.EQ.0)IFAIL=3
	ENDDO
C Deallocatons
	DEALLOCATE(AR,AC,REZ,IMZ,BC,BR,CC,CR)
      RETURN    
      END   





      SUBROUTINE LIB_PZEROS_EVAL(AR,AC,N,TOL,X,Y,R,RX,J,JX,SAT) 
C     MARK 4 RELEASE NAG COPYRIGHT 1974.    
C     MARK 4.5 REVISED  
C     MARK 5C REVISED   
C     EVALUATES R,J,RX,JX, AT THE POINT X+IY    
C     THE RUNNING COMPLEX ERROR BOUND IS APPLIED IF THE LOGICAL 
C     VARIABLE  
C     SAT IS TRUE ON ENTRY.  SAT IS TRUE ON EXIT IF THE BOUND   
C     IS SATISIFIED.    
      INTEGER N, I  
      DOUBLE PRECISION AC(N), AR(N), BOUND, J, JX, NC, ND, NE, NF, NG,  
     * NH, OC,OD, OE, OF, OG, OH, ONE, R, RX, T, TOL, TWO, U, V, W, X, Y    
      LOGICAL SAT   
      DATA ONE /1.0D0/, TWO /2.0D0/ 
      OC = AR(1)    
      OE = OC   
      OD = AC(1)    
      OF = OD   
      OH = ONE  
      OG = ONE  
      T = DABS(X)   
      U = DABS(Y)   
      N = N - 1 
      DO 40 I=2,N   
         NC = X*OC - Y*OD + AR(I)   
         ND = Y*OC + X*OD + AC(I)   
         NE = NC + X*OE - Y*OF  
         NF = ND + Y*OE + X*OF  
         IF (.NOT.SAT) GO TO 20 
         V = OG + DABS(OC)  
         W = OH + DABS(OD)  
         NG = T*V + U*W + DABS(AR(I)) + TWO*DABS(NC)    
         NH = U*V + T*W + DABS(AC(I)) + TWO*DABS(ND)    
         OG = NG    
         OH = NH    
   20    OC = NC    
         OD = ND    
         OE = NE    
         OF = NF    
   40 CONTINUE  
      N = N + 1 
      R = X*OC - Y*OD + AR(N)   
      J = Y*OC + X*OD + AC(N)   
      RX = NE   
      JX = NF   
      IF (.NOT.SAT) GO TO 60    
      V = OG + DABS(OC) 
      W = OH + DABS(OD) 
      NG = T*V + U*W + DABS(AR(N)) + TWO*DABS(R)    
      NH = U*V + T*W + DABS(AC(N)) + TWO*DABS(J)    
      BOUND = (ONE+TOL)**(5*N)*TOL  
      SAT = DABS(R).LE.BOUND*NG .AND. DABS(J).LE.BOUND*NH   
   60 RETURN    
      END   
