	SUBROUTINE LIB_QD(H,Y,Z,NDIM)
*****************************************************************
*	THE SUBROUTINE CALCULATES NDIM INTEGRAL                   *
*	BETWEEN X1 AND XN OF FUNCTION Z ( N.LE.NDIM               *
*	 XN=X1+(N-1)*H ) FOR THE SPECIAL CASE:                    *
*	NDIM=2*ND+1 Y(1)=0 . Z(2*I+1) IS CALCULATED               *
*	BY THE SIMPSON'S METHOD AND Z(2*I+2)=Z(2*I+1)+            *
*	DELT(2*I+2) . TO INCREASE A ACCURACY IT IS                *
*	USED DELT(2*I+2)=DELT4(2*I+2)*(Z(2*I+3)-Z(2*I+1))/        *
*	(DELT4(2*I+2)+DELT4(2*I+3)) WHERE DELT4(2*I+2)            *
*	IS CALCULATED BY USING A CUBIC INTERPOLATION              *
*	BETWEEN Y(2*I),Y(2*I+1),Y(2*I+2),Y(2*I+3).                *
* Revised by Yu.A.Uspensky (FIAN)                               *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION Y(NDIM),Z(NDIM)
C Primitive cases
      C2=H/2.D0
 	C3=H/3.D0
	C24=H/24.D0
	Z(1)=0.D0
	IF(NDIM.EQ.1)RETURN
	Z(2)=C2*(Y(1)+Y(2))
	IF(NDIM.EQ.2)RETURN
	Y2=Y(1)
	Y3=Y(2)
	Y4=Y(3)
	Z(2)=C24*(10.D0*Y2+16.D0*Y3-2.D0*Y4)
C	Z(2)=(Y2+Y3)/2.D0*H
	SUM2=Y2+4.D0*Y3+Y4
	Z(3)=C3*SUM2
	IF(NDIM.EQ.3)RETURN
	Y5=Y(4)
C Evaluate integral by Simpson rule
	DO I=4,NDIM-1,2
	Y1=Y3
	Y2=Y4
	Y3=Y5
	Y4=Y(I+1)
	DELT3=Y2+4*Y3+Y4
	SUM2=SUM2+DELT3
	Z(I+1)=C3*SUM2
	DELT4=-Y1+13*Y2+13*Y3-Y4
	IF(I.NE.NDIM-1)THEN
 	Y5=Y(I+2)
	DELT5=-Y1+12*Y2+26*Y3+12*Y4-Y5
	ELSE
	DELT5=DELT3*8.D0
      ENDIF
	IF(ABS(DELT5).GT.1.D-20)THEN
	DELT=8.D0*DELT4*DELT3/DELT5
	Z(I)=Z(I-1)+C24*DELT
	ELSE
	Z(I)=Z(I-1)+C24*DELT4
	ENDIF
      ENDDO
C Last point in case Ndim is even
      IF(MOD(NDIM,2).EQ.0)
     &Z(NDIM)=Z(NDIM-1)+C2*(Y(NDIM-1)+Y(NDIM))
	END


	SUBROUTINE LIB_CQD(H,Y,Z,NDIM)
*****************************************************************
*	THE SUBROUTINE CALCULATES NDIM INTEGRALS		            *
*	BETWEEN X1 AND XN OF FUNCTION Z ( N.LE.NDIM		        *
*	 XN=X1+(N-1)*H ) FOR THE SPECIAL CASE:			        *
*	NDIM=2*ND+1 Y(1)=0 . Z(2*I+1) IS CALCULATED		        *
*	BY THE SIMPSON'S METHOD AND Z(2*I+2)=Z(2*I+1)+		    *
*	DELT(2*I+2) . TO INCREASE A ACCURACY IT IS		        *
*	USED DELT(2*I+2)=DELT4(2*I+2)*(Z(2*I+3)-Z(2*I+1))/	    *
*	(DELT4(2*I+2)+DELT4(2*I+3)) WHERE DELT4(2*I+2)		    *
*	IS CALCULATED BY USING A CUBIC INTERPOLATION		        *
*	BETWEEN Y(2*I),Y(2*I+1),Y(2*I+2),Y(2*I+3).		        *
* Revised by Yu.A.Uspensky (FIAN)                               *
*****************************************************************
	IMPLICIT REAL*8 (C,H),COMPLEX*16 (D,S,Y,Z)
	DIMENSION Y(NDIM),Z(NDIM)
C Primitive cases
      C2=H/2.D0
 	C3=H/3.D0
	C24=H/24.D0
	Z(1)=0.D0
	IF(NDIM.EQ.1)RETURN
	Z(2)=C2*(Y(1)+Y(2))
	IF(NDIM.EQ.2)RETURN
	Y2=Y(1)
	Y3=Y(2)
	Y4=Y(3)
	Z(2)=C24*(10.D0*Y2+16.D0*Y3-2.D0*Y4)
C	Z(2)=(Y2+Y3)/2.D0*H
	SUM2=Y2+4.D0*Y3+Y4
	Z(3)=C3*SUM2
	IF(NDIM.EQ.3)RETURN
	Y5=Y(4)
C Evaluate integral by Simpson rule
	DO I=4,NDIM-1,2
	Y1=Y3
	Y2=Y4
	Y3=Y5
	Y4=Y(I+1)
	DELT3=Y2+4*Y3+Y4
	SUM2=SUM2+DELT3
	Z(I+1)=C3*SUM2
	DELT4=-Y1+13*Y2+13*Y3-Y4
	IF(I.NE.NDIM-1)THEN
 	Y5=Y(I+2)
	DELT5=-Y1+12*Y2+26*Y3+12*Y4-Y5
	ELSE
	DELT5=DELT3*8.D0
      ENDIF
	IF(ABS(DELT5).GT.1.D-20)THEN
	DELT=8.D0*DELT4*DELT3/DELT5
	Z(I)=Z(I-1)+C24*DELT
	ELSE
	Z(I)=Z(I-1)+C24*DELT4
	ENDIF
      ENDDO
C Last point in case Ndim is even
      IF(MOD(NDIM,2).EQ.0)
     &Z(NDIM)=Z(NDIM-1)+C2*(Y(NDIM-1)+Y(NDIM))
	END




	SUBROUTINE LIB_DQD(H,Y,Z,NDIM)
*****************************************************************
*	THE SUBROUTINE CALCULATES NDIM INTEGRALS		            *
*	BETWEEN X1 AND XN OF FUNCTION Z ( N.LE.NDIM		        *
*	 XN=X1+(N-1)*H ) FOR THE SPECIAL CASE:			        *
*	NDIM=2*ND+1 Y(1)=0 . Z(2*I+1) IS CALCULATED		        *
*	BY THE SIMPSON'S METHOD AND Z(2*I+2)=Z(2*I+1)+		    *
*	DELT(2*I+2) . TO INCREASE A ACCURACY IT IS		        *
*	USED DELT(2*I+2)=DELT4(2*I+2)*(Z(2*I+3)-Z(2*I+1))/	    *
*	(DELT4(2*I+2)+DELT4(2*I+3)) WHERE DELT4(2*I+2)		    *
*	IS CALCULATED BY USING A CUBIC INTERPOLATION		        *
*	BETWEEN Y(2*I),Y(2*I+1),Y(2*I+2),Y(2*I+3).		        *
* Revised by Yu.A.Uspensky (FIAN)                               *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8 H
	DIMENSION Y(NDIM),Z(NDIM)
C Primitive cases
      C2=H/2.D0
 	C3=H/3.D0
	C24=H/24.D0
	Z(1)=0.D0
	IF(NDIM.EQ.1)RETURN
	Z(2)=C2*(Y(1)+Y(2))
	IF(NDIM.EQ.2)RETURN
	Y2=Y(1)
	Y3=Y(2)
	Y4=Y(3)
	Z(2)=C24*(10.D0*Y2+16.D0*Y3-2.D0*Y4)
C	Z(2)=(Y2+Y3)/2.D0*H
	SUM2=Y2+4.D0*Y3+Y4
	Z(3)=C3*SUM2
	IF(NDIM.EQ.3)RETURN
	Y5=Y(4)
C Evaluate integral by Simpson rule
	DO I=4,NDIM-1,2
	Y1=Y3
	Y2=Y4
	Y3=Y5
	Y4=Y(I+1)
	DELT3=Y2+4*Y3+Y4
	SUM2=SUM2+DELT3
	Z(I+1)=C3*SUM2
	DELT4=-Y1+13*Y2+13*Y3-Y4
	IF(I.NE.NDIM-1)THEN
 	Y5=Y(I+2)
	DELT5=-Y1+12*Y2+26*Y3+12*Y4-Y5
	ELSE
	DELT5=DELT3*8.D0
      ENDIF
	IF(ABS(DELT5).GT.1.D-20)THEN
	DELT=8.D0*DELT4*DELT3/DELT5
	Z(I)=Z(I-1)+C24*DELT
	ELSE
	Z(I)=Z(I-1)+C24*DELT4
	ENDIF
      ENDDO
C Last point in case Ndim is even
      IF(MOD(NDIM,2).EQ.0)
     &Z(NDIM)=Z(NDIM-1)+C2*(Y(NDIM-1)+Y(NDIM))
	END
