C========+=========+=========+=========+=========+=========+=========+=$
C     PROGRAM: ranw.f
C     TYPE   : real function
C     PURPOSE: produce uniformly distributed random numbers
C              following the algorithm of Mitchell and Moore
C     I/O    :
C     VERSION: 30-Sep-95
C     COMMENT: cf. D. E. Knuth, Seminumerical Algorithms, 2nd edition
C              Vol 2 of  The Art of Computer Programming (Addison-Wesley,
C              1981) pp 26f. (Note: the procedure ran3 in
C              W. H. Press et al,  Numerical
C              Recipes in FORTRAN, 2nd edition (Cambridge University
C              Press 1992)  is based on the same algorithm).
C              I would suggest that you make sure for yourself that
C              the quality of the random number generator is sufficient,
C              or else replace it!
C========+=========+=========+=========+=========+=========+=========+=$
      double precision function lib_ranw(idum)
      Parameter (Mbig=2**30-2, Xinvers=1./Mbig)
      data ibit/ 1/
      Integer IX(55)
      save
      if (ibit.ne.0) then
         ibit=0
cc
cc       fill up the vector ix with some random integers, which are
cc       not all even
cc
         if (idum.eq.0) stop 'use nonzero value of idum'
         idum=abs(mod(idum,Mbig))
         ibit=0
         Ix(1)=871871
         Do i=2,55
            Ix(i)=mod(Ix(i-1)+idum,Ix(i-1))
            Ix(i)=max(mod(Ix(i),Mbig),idum)
         end do
         j=24
         k=55
cc
cc       warm up the generator
cc
         do i=1,1258
            Ix(k)=mod(Ix(k)+Ix(j),Mbig)
            j=j-1
            if (j.eq.0) j=55
            k=k-1
            if (k.eq.0) k=55
         end do
      end if
cc
cc    this is where execution usually starts:
cc
      Ix(k)=mod(Ix(k)+Ix(j),Mbig)
      j=j-1
      if (j.eq.0) j=55
      k=k-1
      if (k.eq.0) k=55
      lib_ranw=Ix(k)*Xinvers
      end

