      SUBROUTINE LIB_RATMOM2(D0,D1,W,P,PDEF)
********************************************************************
* Computes rational interpolation based on two moments
*     G(w)=W/(w-P)
* by solving following system of equations
*     W=D0
*     W*P=D1
********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
	W=D0
	IF(ABS(D0).GT.1.D-10)P=D1/D0
	IF(ABS(D0).LE.1.D-10)P=PDEF
      END


      SUBROUTINE LIB_RATMOM4(D0,D1,D2,R1,W1,W2,P1,P2,PDEF)
********************************************************************
* Computes rational interpolation based on four moments
*     G(w)=W1/(w-P1)+W2/(w-P2)
* by solving following system of equations
*     W1+W2=D0
*     W1*P1+W2*P2=D1
*     W1*P1^2+W2*P2^2=D2
*     W1/P1+W2/P2=R1
********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      IF(ABS(D0).GT.1.D-10)THEN
      IONEP=0
      PAVN1=D1/D0
      PAVN2=D2/D0
      PAVR1=R1/D0
      DENOM=PAVN1*PAVR1-1.D0
      PAV2M=0.D0
      IF(ABS(DENOM).GT.1.D-10)
     &PAV2M=(PAVN2-PAVN1**2)/DENOM
      IF(ABS(DENOM).LE.1.D-10)IONEP=1
      PAV1S=0.D0
      IF(ABS(DENOM).GT.1.D-10)
     &PAV1S=(PAVR1*PAVN2-PAVN1)/DENOM
C Poles
      SPLIT=PAV1S**2-4.D0*PAV2M
      POLE1=PDEF
      POLE2=PDEF
      IF(SPLIT.GE.0.D0)
     &POLE1=0.5D0*PAV1S-0.5D0*SQRT(SPLIT)
      IF(SPLIT.GE.0.D0)
     &POLE2=0.5D0*PAV1S+0.5D0*SQRT(SPLIT)
      IF(SPLIT.LT.0.D0)IONEP=1
C Weights
      DENOM=POLE2-POLE1
      WEIG1=0.D0
      WEIG2=0.D0
      IF(ABS(DENOM).GT.1.D-10)
     &WEIG1=-D0*(PAVN1-POLE2)/DENOM
      IF(ABS(DENOM).GT.1.D-10)
     &WEIG2=+D0*(PAVN1-POLE1)/DENOM
      IF(ABS(DENOM).LE.1.D-10)IONEP=1
      IF(WEIG1.LE.0.D0)IONEP=1
      IF(WEIG2.LE.0.D0)IONEP=1
C Check signs of poles
      IF(POLE1*PDEF.LT.0.D0)IONEP=1 ! sign should be the same
      IF(POLE2*PDEF.LT.0.D0)IONEP=1
C Check if one pole approximation is necessary
      IF(IONEP.EQ.1)THEN
      CALL LIB_RATMOM2(D0,D1,WEIG1,POLE1,PDEF)      
      WEIG2=0.D0
      POLE2=PDEF
      ENDIF
C In case band is empty/filled
      ELSE
      WEIG1=0.D0
      WEIG2=0.D0
      POLE1=PDEF
      POLE2=PDEF
      ENDIF
C *****************
C Set final results
      W1=WEIG1
      W2=WEIG2
      P1=POLE1
      P2=POLE2
      END
      
      
      SUBROUTINE LIB_RATMOM6(D0,D1,D2,D3,R1,R2,W1,W2,W3,P1,P2,P3,PDEF)
********************************************************************
* Computes rational interpolation based on four moments
*     G(w)=W1/(w-P1)+W2/(w-P2)
* by solving following system of equations
*     W1+W2+W3=D0
*     W1*P1+W2*P2+W3*P3=D1
*     W1*P1^2+W2*P2^2+W3*P3^2=D2
*     W1*P1^3+W2*P3^2+W3*P3^2=D3
*     W1/P1+W2/P2+W3/P3=R1
*     W1/P1^2+W2/P2^2+W3/P3^2=R2
********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      CALL FULLSTOP('RATMOM6 not ready')
      IF(ABS(D0).GT.1.D-10)THEN
      ITWOP=0
C ... Add solution here


C Check signs of poles
      IF(POLE1*PDEF.LT.0.D0)ITWOP=1 ! sign should be the same
      IF(POLE2*PDEF.LT.0.D0)ITWOP=1
      IF(POLE3*PDEF.LT.0.D0)ITWOP=1
C Check if two pole approximation is necessary
      IF(ITWOP.EQ.1)THEN
      CALL LIB_RATMOM4(D0,D1,D2,R1,WEIG1,WEIG2,POLE1,POLE2,PDEF)
      WEIG3=0.D0
      POLE3=PDEF
      ENDIF
C In case band is empty/filled
      ELSE
      WEIG1=0.D0
      WEIG2=0.D0
      WEIG3=0.D0
      POLE1=PDEF
      POLE2=PDEF
      POLE3=PDEF
      ENDIF
C *****************
C Set final results
      W1=WEIG1
      W2=WEIG2
      W3=WEIG3
      P1=POLE1
      P2=POLE2
      P3=POLE3
      END