	SUBROUTINE MAN_LMTSCF
************************************************************
*   The calculation of the energy band structure in terms  *
*   of LMTO method (O.K.Andersen,Phys.Rev.B12,3060(1975))  *
************************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                         !!! from setup
      USE COM_LMTINP,  ONLY : TRACINP                        !!! from readini
      USE COM_CONTROL, ONLY : LIFT,LMTO                      !!! from readini
      USE COM_SCFLOOP, ONLY : NITER1                         !!! from readini
      USE COM_CHANNELS,ONLY : IUSCF,IUPOT                    !!! from readini
	USE COM_LMTOUT,  ONLY : ICPOT                          !!! from readini
      USE COM_UNIT,    ONLY : IUN,IUR,IUT,IUI                !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	CHARACTER ST*8,FN*8,PROGN*8
	LOGICAL EXIT,LIB_TRACER
	DATA ST/'started'/,FN/'finished'/
C CALCULATE STRUCTURE CONSTANTS
	progn='Strmsh  '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL STR_VECGEN     !!! generate structure vectors
	CALL STR_STRSNG     !!! check for singularities
	CALL STR_STRMSH     !!! calculate structure constants
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
C CALCULATE SCREENED STRUCTURE CONSTANTS
	progn='Scrcon  '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
      call lib_timel(times)
      write(iun,1110)progn,st,times,curmem,totmem
      CALL HOP_ALPCON     !!! get screening parameters "alpha"
      CALL HOP_SCRCON     !!! screen bare structure constants
      call lib_timel(timef)
      write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	IF(LIFT.LE.1)CALL MAN_ARTOUT('No band structure calculation')
	CALL BND_PLACE(0)   !!! save Dny for admixing
C  *****  THE LOOP OF SELF-CONSISTENCY *****
	DO 100 ITER=1,NITER1
	write(iun,1100)iter
	CALL RHO_RHOFILES(IUSCF,'R','A')! read charge density
C CALCULATE MULTIPOLE FIELD
	CALL POT_MULTRHO    !!! calculate moments of (pseudo)density
	CALL POT_MULTFTR    !!! multipoles as lattice sums
	CALL POT_MULTASA    !!! multipoles as lattice sums
	CALL POT_PRINTC(1)  !!! save input charge for printing
      CALL RHO_MIXROB(ITER,1) !!! initialize Broyden mixing
      CALL HUB_MIXHUB(ITER,1) !!! initialize mixing for occupancies
C CALCULATE FULL POTENTIAL
	progn='FullPOT '
	CALL POT_POTFILES(IUPOT,'!','A')! initialize full potential
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL POT_VCOUL      !!! calculate Coulomb potential within MT-spheres
	CALL POT_VEXCH(1)   !!! calculate ex.-cor.potential within MT-spheres
	CALL POT_VCFTR      !!! calculate Coulomb potential in the int.region
	CALL POT_VXFTR(1)   !!! calculate ex.-cor.potential in the int.region
	CALL POT_ZEROV      !!! set up energy zero
	CALL POT_VPINT      !!! calculate convolutions of pseudopotential
	CALL POT_PRINT2     !!! print necessary information
      CALL HUB_HUBPOT     !!! adds Hubbard correction to the potential
      CALL HUB_WRITEHUB_DATA(IUN,2)  !!! print Hubbard correction
      CALL HUB_WRITEHUB_DATA(IUN,3)  !!! print Double Counting correction
      CALL HUB_WRITEHUB_DATA(IUN,4)  !!! print Constrained potential
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	CALL RHO_RHOFILES(IUSCF,'!','D')! deallocate charge density
	IF(ICPOT.GT.0)
     &CALL POT_POTFILES(IUPOT,'W','!')! store the potential if needed
C CALCULATE BAND STRUCTURE
	progn='Bands   '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL BND_LMTDFT         !!! find band energies and eigenvectors
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
C CALCULATE OPTICAL PRPOERTIES WITHIN DFT
	progn='LmtOpt  '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL OPT_LMTOPT         !!! find optical properties
      CALL ODF_LMTODF         !!! find Optics using DMFT
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
C CALCULATE GREEN FUNCTION WITH DMFT
      progn='DMFT    '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
      call lib_timel(times)
      write(iun,1110)progn,st,times,curmem,totmem
      CALL DMF_LMTREA         !!! real axis DMFT
      CALL DMF_LMTIMA         !!! imag axis DMFT
      CALL DMF_LMTRAT         !!! rational interpolation DMFT
      CALL DMF_LMTQUP         !!! low frequency quasiparticle DMFT
      call lib_timel(timef)
      write(iun,1120)progn,fn,timef,curmem,totmem
      endif
C CALCULATE SUSCEPTIBILITY AND EXCHANGE CONSTANTS
	progn='LmtChi  '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL CHI_LMTCHI         !!! find susceptibility
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
      IF(LIFT.LE.2)CALL MAN_ARTOUT('No SCF loop')
C CALCULATE FULL VALENCE PLUS SEMICORE DENSITY IN UNIT-CELL
	progn='FullRHO '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL RHO_RHOFUL     !!! find new valence charge density within MTS
	CALL RHO_SRODOT     !!! find energy derivative for rho(r)
      CALL RHO_ORBMAG     !!! find orbital contribution to magnetization
	CALL RHO_EMADER     !!! find derivative for EMA
	CALL RHO_RENROV     !!! renormalize valence density
      CALL HUB_RHOHUB     !!! set occupation numbers
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
C CALCULATE DEEP CORE
	progn='RenCOR  '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL RHO_RENCOR     !!! find core charge density destribution in cell
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	CALL POT_POTFILES(IUPOT,'W','D')!! store the potential
C CALCULATE TOTAL ENERGY AND FORCES
      CALL HUB_WRITEHUB_DATA(IUN,1)  !!! print occupation numbers
      CALL HUB_WRITEHUB(2)       !!! store occupation numbers
	progn='Energy  '
      if(lib_tracer(progn,tracinp,iut,iur,iui))then
	call lib_timel(times)
	write(iun,1110)progn,st,times,curmem,totmem
	CALL FRC_ENERGY(EXIT)      !!! calculate Etot in LDA or GGA91
      CALL FRC_FORCES            !!! calculate HF+Pulay - forces 
	call lib_timel(timef)
	write(iun,1120)progn,fn,timef,curmem,totmem
	endif
	CALL POT_PRINTC(2)         !!! print input-output charge transfer
	IF(EXIT)CALL MAN_ARTOUT('Total energy convergence reached')
C ADMIX DENSITY FOR THE NEXT ITERATION
      CALL RHO_MIXROB(ITER,2)    !!! mix densities according to Broyden
      CALL RHO_MIXRO1(ITER,EXIT) !!! mix densities with linear mixing
      CALL HUB_MIXHUB(ITER,2)    !!! mix occupancies
	CALL BND_PLACE(ITER)       !!! place Eny at the centers of gravity
	CALL RHO_RHOFILES(IUSCF,'W','D')!! store charge density at the iteration
	IF(EXIT)CALL MAN_ARTOUT('Charge density convergence reached')
	write(iun,1130)TOTMEM
	write(iun,1140)TXTMEM
 100  CONTINUE
	CALL MAN_ARTOUT('Number of iterations exceeded in the SCF1 loop')
1100	FORMAT(/'                ITERATION ',I3/)
1110  FORMAT('***** ',A8,1X,A8,'; CPU : ',G12.5,
     &       '; CUR/MAX mem.(Mb): ',G10.3,'/',G10.3)
1120  FORMAT('***** ',A8,1X,A8,'; CPU : ',G12.5,
     &       '; CUR/MAX mem.(Mb): ',G10.3,'/',G10.3)
1130  FORMAT(/' Maximum memory allocated (Mbyte) ==',G14.7)
1140  FORMAT( ' Peak memory reached in ',A)
	END
