	MODULE COM_HUBDAT
*************************************************************
* Controls correlated electrons.                            *
*************************************************************
	INTEGER, SAVE :: NHUB                           !!! # of correlated states
	INTEGER, SAVE :: NCLM                           !!! # of correlated basis functions
	INTEGER, SAVE, ALLOCATABLE :: MCRL(:)           !!! correlated atoms pointer
	INTEGER, SAVE, ALLOCATABLE :: LCRL(:,:)         !!! correlated orbitals pointer
	INTEGER, SAVE, ALLOCATABLE :: KSIGFUN(:,:)      !!! Sigma calculation scheme
	INTEGER, SAVE, ALLOCATABLE :: KSIGINT(:,:)      !!! Sigma interpolation method
	INTEGER, SAVE, ALLOCATABLE :: KSIGRAT(:,:)      !!! force rational representation
	INTEGER, SAVE, ALLOCATABLE :: KDELFUN(:,:)      !!! hybridization calculation scheme
	INTEGER, SAVE, ALLOCATABLE :: KDELINT(:,:)      !!! delta interpolation method
	INTEGER, SAVE, ALLOCATABLE :: KDELRAT(:,:)      !!! force rational representation
	INTEGER, SAVE, ALLOCATABLE :: KSOLVER(:,:)      !!! Impurty solver choice
	INTEGER, SAVE, ALLOCATABLE :: KFIXSIG(:,:)      !!! Fix self-energy if needed
	INTEGER, SAVE, ALLOCATABLE :: KDBLCNT(:,:)      !!! Double counting choice
	INTEGER, SAVE, ALLOCATABLE :: KSIGMOO(:,:)      !!! Sigma(oo) choice
	INTEGER, SAVE, ALLOCATABLE :: KIMPLEV(:,:)      !!! Impurity level choice
	REAL*8,  SAVE, ALLOCATABLE :: CDBLCNT(:,:)      !!! double counting constant for U(n-a)
	INTEGER, SAVE :: KHRM                           !!! Use hermitian hamiltonians
	INTEGER, SAVE :: KGRF                           !!! Compute downfolded Green functions
	INTEGER, SAVE :: KSUM                           !!! k-sum for local Green function
	INTEGER, SAVE :: ICUBC                          !!! cubic harmonics input/output
	INTEGER, SAVE :: IRE                            !!! real/complex output
	INTEGER, SAVE :: IWR 
	REAL*8,  SAVE :: PDMP                           !!! pole broadening in self-energy
	REAL*8,  SAVE :: GDMP                           !!! pole broadening in Green function
	END MODULE COM_HUBDAT



      MODULE COM_HUBSYS
*************************************************************
* Containes local coordinate systems for hubbard occupancies*
*************************************************************
      REAL*8, SAVE, ALLOCATABLE :: RINPAXE(:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTAXE(:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: RINPANG(:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTANG(:,:)
      REAL*8, SAVE, ALLOCATABLE :: RINPINV(:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTINV(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHSYS(:,:)
	CHARACTER*80, SAVE, ALLOCATABLE :: CHAXE(:,:)
	CHARACTER*80, SAVE, ALLOCATABLE :: CHANG(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHINV(:,:)
	CHARACTER*80, SAVE, ALLOCATABLE :: CHMAG(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHSPL(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHISP(:,:)
      END MODULE COM_HUBSYS




	MODULE COM_FSLATR
*************************************************************
* Containes Slater integrals.                               *
*************************************************************
      REAL*8, SAVE, ALLOCATABLE :: FSLG(:,:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: FSLU(:,:,:,:)
	END MODULE COM_FSLATR



	MODULE COM_JEYMAT
*************************************************************
* Contains exchange integrals matrix                        *
*************************************************************
      COMPLEX*16, SAVE, ALLOCATABLE :: GINT0(:,:,:,:,:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: AMOML(:,:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: WEISS(:,:,:)
	END MODULE COM_JEYMAT




	MODULE COM_MSBMESH
*************************************************************
* Containes frequency mesh data including Matsubara grid
*************************************************************
	INTEGER, SAVE :: NMSB  ! # of logarithmic Matsubara points
	INTEGER, SAVE :: NMSL  ! # of linear Matsubara points
	INTEGER, SAVE :: NOMG  ! # of real axis points
	INTEGER, SAVE :: NMSC  ! # current grid
	INTEGER, SAVE :: KGRD  ! 1 for real axis, 2 for imaginary axis
	REAL*8,  SAVE :: TEMP  ! temperature
	REAL*8,  SAVE :: OBND  ! effective bandwidth for logarithmic grid
	REAL*8,  SAVE :: WMIN  ! real axis grid limits
	REAL*8,  SAVE :: WMAX  ! real axis grid limits
	REAL*8,  SAVE :: SMSB
	REAL*8,  SAVE :: SMSI
	REAL*8,  SAVE :: SMSL
	REAL*8,  SAVE :: SMSE
	REAL*8, SAVE, ALLOCATABLE :: WMSB(:)  ! logarithmic Matsubara grid
	REAL*8, SAVE, ALLOCATABLE :: DMSB(:)
	REAL*8, SAVE, ALLOCATABLE :: FMSB(:)
	REAL*8, SAVE, ALLOCATABLE :: WMSL(:)  ! linear Matsubara grid
	REAL*8, SAVE, ALLOCATABLE :: DMSL(:)
	REAL*8, SAVE, ALLOCATABLE :: FMSL(:)
	REAL*8, SAVE, ALLOCATABLE :: WOMG(:)  ! real axis grid
	REAL*8, SAVE, ALLOCATABLE :: DOMG(:)
	REAL*8, SAVE, ALLOCATABLE :: FOMG(:)
	COMPLEX*16, SAVE, ALLOCATABLE :: WMSC(:) ! current grid
	REAL*8, SAVE, ALLOCATABLE :: DMSC(:)
	REAL*8, SAVE, ALLOCATABLE :: FMSC(:)  
	END MODULE COM_MSBMESH


	MODULE COM_DMFPAR
*************************************************************
* Containes DMFT data
*************************************************************
	INTEGER, SAVE :: NDIT      !!! # of DMFT iterations
	REAL*8,  SAVE :: EPSDMF    !!! accuracy in DMFT loop
	REAL*8,  SAVE :: ADMIXL    !!! mixing
	REAL*8,  SAVE :: ADMIX1    !!! mixing to start
	LOGICAL, SAVE :: MIXBRD    !!! use broyden
	INTEGER, SAVE :: MIXFUN    !!! 1 for Delta(w), 2 for Sigma(w)
	END MODULE COM_DMFPAR




	MODULE COM_LMTVEC
*************************************************************
* Contains left/right eigenvectors
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: APHR(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: APSR(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: APJR(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: APHL(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: APSL(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: APJL(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AQHR(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AQSR(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AQJR(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AQHL(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AQSL(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AQJL(:,:,:,:,:)
	END MODULE COM_LMTVEC



	MODULE COM_HUBCOM
*************************************************************
* Contains Hubbard parameters for LDA+U.                    *
*************************************************************
      COMPLEX*16, SAVE, ALLOCATABLE :: VHUB(:,:,:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: VCNS(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: VDBC(:,:,:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: DHUB(:,:,:,:)
	END MODULE COM_HUBCOM


	MODULE COM_DMFDAT
*************************************************************
* Containes Green function data
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: SIG0(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SIGI(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SIGW(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: DELW(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: DELI(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: BMAS(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: RMAS(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GLOC(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OIMP(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: EIMP(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ELOC(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AINV(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ALAM(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ELAM(:,:,:,:)
	END MODULE COM_DMFDAT




	MODULE COM_RATDAT
******************************************************************
* Containes Self-energy/Hybridization coefficients for rational interpolation
******************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: VRAT(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: WRAT(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PRAT(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ORAT(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: VRHB(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: WRHB(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PRHB(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ORHB(:,:,:,:,:)
	END MODULE COM_RATDAT





	MODULE COM_GRFDAT
*************************************************************
* Contains Green function data
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: EPOL(:,:,:) !!! Complex energy bands E(kj,iWn)
	COMPLEX*16, SAVE, ALLOCATABLE :: EPOW(:,:,:) !!! Complex energy bands E(kj,iWn) at dense grid
	COMPLEX*16, SAVE, ALLOCATABLE :: GWGT(:,:,:) !!! Integration weights for Green function
	COMPLEX*16, SAVE, ALLOCATABLE :: GWGD(:,:,:) !!! Integration weights for Green function
	COMPLEX*16, SAVE, ALLOCATABLE :: GW0T(:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GW0D(:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: EBND(:,:,:) !!! Matrix elements <kjw|-Nabla2+V|kjw>
	COMPLEX*16, SAVE, ALLOCATABLE :: ECAP(:,:)   !!! Band Capacities <kjw|kjw>
	REAL*8, SAVE, ALLOCATABLE :: GWTS(:,:,:,:)
	END MODULE COM_GRFDAT


	MODULE COM_ORDDAT
*************************************************************
* Contains information how to order complex eigenvalues
*************************************************************
      INTEGER, SAVE :: KORD                        !!! reordering scheme
      INTEGER, SAVE, ALLOCATABLE :: IORD(:,:,:)    !!! reordering pointer
      INTEGER, SAVE :: NORD                        !!! # of nearest bands to scan during ordering
      INTEGER, SAVE :: NSIG                        !!! max. # of bands with imaginary part
      REAL*8, SAVE :: OEPS                         !!! accuracy in reordering
      REAL*8, SAVE :: OSIG                         !!! min.imaginary part to scan during ordering
	END MODULE COM_ORDDAT


	MODULE COM_IMPDAT
*************************************************************
* Containes data for impurity solver
*************************************************************
	INTEGER, SAVE, ALLOCATABLE :: KCRDSYS(:,:)    !!! coordiate system choice
	INTEGER, SAVE, ALLOCATABLE :: NRUNIMP(:,:)    !!! # of runs for impurity solver
	INTEGER, SAVE, ALLOCATABLE :: KSTATES(:,:,:,:)!!! states to include in impurity solver
	INTEGER, SAVE, ALLOCATABLE :: NPOLIMP(:,:)    !!! Rational interpolation for Sigma(w)
	INTEGER, SAVE, ALLOCATABLE :: NPLDIMP(:,:)    !!! Fational interpolation for Delta(w)
	INTEGER, SAVE, ALLOCATABLE :: ISPLIMP(:,:)    !!! splitting behavior during iterations
	INTEGER, SAVE, ALLOCATABLE :: IMPSECT(:,:)    !!! Information array handling
	INTEGER, SAVE, ALLOCATABLE :: IMPINFO(:,:)    !!! Information array length
	REAL*8, SAVE, ALLOCATABLE :: AMAGIMP(:,:,:)   !!! impurity magnetization axis
	REAL*8, SAVE, ALLOCATABLE :: BSPLIMP(:,:)     !!! initial splitting by magnetic field
	COMPLEX*16, SAVE, ALLOCATABLE :: DSYS(:,:,:,:)!!! custom transformation matrix
	COMPLEX*16, SAVE, ALLOCATABLE :: RSYS(:,:,:,:)!!! reverse custom transformation matrix
	CHARACTER*150, SAVE, ALLOCATABLE :: IMPDATA(:,:,:)   !!! Information data
	CHARACTER*150, SAVE, ALLOCATABLE :: IMPWORK(:)!!! Temporary storage
	END MODULE COM_IMPDAT



	MODULE COM_QMCDAT
*************************************************************
* Containes data for QMC impurity solver
*************************************************************
	INTEGER, SAVE :: NTAU
	INTEGER, SAVE :: NSWP
	END MODULE COM_QMCDAT
