      MODULE COM_STREAM
***************************************************************
*  General control keys for the program.                      *
***************************************************************
      INTEGER, SAVE :: ISTREAM     !!! controls computational stream
	END MODULE COM_STREAM


      MODULE COM_CONTROL
***************************************************************
*  General control keys for the program.                      *
***************************************************************
      INTEGER, SAVE :: MODE        !!! controls computational mode
      INTEGER, SAVE :: LIFT        !!! controls SCF looping
      INTEGER, SAVE :: LMTO        !!! controls unscreened/TB LMTO
      INTEGER, SAVE :: IASA        !!! controls ASA or FP
      INTEGER, SAVE :: NOVR        !!! controls Overlap calc
      INTEGER, SAVE :: NPFR        !!! controls Pulay forces
      INTEGER, SAVE :: LRWF        !!! controls radial functions
      INTEGER, SAVE :: LPRO        !!! controls local projector operation
      INTEGER, SAVE :: LGRP        !!! controls crystal group
      INTEGER, SAVE :: ILDA        !!! controls LDA exchange-correlation
      INTEGER, SAVE :: IGGA        !!! controls GGA exchange-correlation
      INTEGER, SAVE :: ILSD        !!! controls LSD exchange-correlation
      INTEGER, SAVE :: IDMF        !!! controls DMF corrections
      INTEGER, SAVE :: ISIC        !!! controls self-interaction corrections
      INTEGER, SAVE :: IECALC      !!! only total energy calculation
      END MODULE COM_CONTROL


      MODULE COM_PARAM
***************************************************************
*  Main parameters of the calculation                         *
***************************************************************
      INTEGER, SAVE :: NATOM                 !!! # of atoms in cell
      INTEGER, SAVE :: NSORT                 !!! # of different sorts
      INTEGER, SAVE :: NSPIN                 !!! # of spins
      INTEGER, SAVE :: NORBS                 !!! spin-orbit coupling
      INTEGER, SAVE, ALLOCATABLE :: IS(:)    !!! atom-to-sort pointer
      INTEGER, SAVE, ALLOCATABLE :: NHSORT(:)!!! # of atoms for given sort
      INTEGER, SAVE, ALLOCATABLE :: ILST(:)  !!! max-l for Psi_k
      INTEGER, SAVE, ALLOCATABLE :: ILSB(:)  !!! max-l for basis
      INTEGER, SAVE, ALLOCATABLE :: ILSV(:)  !!! max-l for potential
      INTEGER, SAVE, ALLOCATABLE :: LRTV(:)  !!! controls relativistic
      INTEGER, SAVE, ALLOCATABLE :: LSCO(:)  !!! controls spin-orbit
      REAL*8,  SAVE :: SORN
      END MODULE COM_PARAM



      MODULE COM_MIXRHO1
*************************************************************
* Containes linear mixing parameters.                       *
*************************************************************
      INTEGER, SAVE :: KEY            !!! controls linear mixing for rho
      INTEGER, SAVE :: KSP            !!! controls linear mixing for m
      REAL*8, SAVE :: ADMIX1          !!! initial linear mixing for rho
      REAL*8, SAVE :: ADMIN           !!! minimal linear mixing for rho
      REAL*8, SAVE :: ADMAX           !!! maximum linear mixing for rho
      REAL*8, SAVE :: ADMOPT          !!! optimal linear mixing for rho
      REAL*8, SAVE :: ADMSPN          !!! initial linear mixing for m(r)
      REAL*8, SAVE :: ASMIN           !!! minimal linear mixing for m(r)
      REAL*8, SAVE :: ASMAX           !!! maximum linear mixing for m(r)
      REAL*8, SAVE :: ADSPIN          !!! optimal linear mixing for m(r)
      END MODULE COM_MIXRHO1


      MODULE COM_MIXBROY
*************************************************************
* Containes Broyden mixing parameters.                      *
*************************************************************
      INTEGER, SAVE :: NBROY        !!! max.# of Broyden iterations
      INTEGER, SAVE :: LBROY        !!! all low l's mixed with Broyden
      INTEGER, SAVE :: IBROY        !!! Broyden is switched after Ibroy iters
      INTEGER, SAVE :: MTER         !!! current Broyden iteration
      REAL*8, SAVE :: ADMIXB        !!! initial Broyden mixing
      REAL*8, SAVE :: ADMIXH        !!! linear mixing for high l's
      COMPLEX*16, SAVE, ALLOCATABLE :: WMIX(:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: WINP(:)
      END MODULE COM_MIXBROY



      MODULE COM_SCFLOOP
*************************************************************
* Containes accuracy parameters and # of iterations.        *
*************************************************************
      INTEGER, SAVE :: NITER1        !!! max.# of iterations in SCF1
      REAL*8, SAVE :: EPSTOT         !!! total energy accuracy
      REAL*8, SAVE :: EPSRHO         !!! density convergency
      REAL*8, SAVE :: EPSMAG         !!! magnetization convergency
      END MODULE COM_SCFLOOP


      MODULE COM_LMTINP
*************************************************************
* Containes pointer to additional input information.        *
*************************************************************
C Project title
      CHARACTER*130, SAVE :: PROJECT
C Input information key
      CHARACTER*130, SAVE :: INPINFO
C Running mode key
      CHARACTER*130, SAVE :: RUNMODE
C Controls and errors
      CHARACTER*130, SAVE :: CTRFILE
      CHARACTER*130, SAVE :: LOGFILE
	CHARACTER*130, SAVE :: ABRFILE
C Main input files
      CHARACTER*130, SAVE :: INIFILE
      CHARACTER*130, SAVE :: SCSFILE
      CHARACTER*130, SAVE :: STRFILE
C Input keys
	LOGICAL, SAVE :: TRACINP  ! tracing the program
	LOGICAL, SAVE :: SCSMADE  ! shows whether input density was built
	LOGICAL, SAVE :: SCFMADE  ! shows whether scf density was built
	LOGICAL, SAVE :: SMTMADE  ! make MT-spheres
C Additional input keys
      INTEGER, SAVE :: ICHUB
      INTEGER, SAVE :: ICHOP
      INTEGER, SAVE :: ICPNT
      INTEGER, SAVE :: ICENR
C Additional input files
      CHARACTER*130, SAVE :: HUBFILE
      CHARACTER*130, SAVE :: HOPFILE
      CHARACTER*130, SAVE :: PNTFILE
      CHARACTER*130, SAVE :: ENRFILE
      END MODULE COM_LMTINP



      MODULE COM_LMTOUT
*************************************************************
* Containes pointers to output information.                 *
*************************************************************
C Scratch files storage
      INTEGER, SAVE :: ISCR1MIN
      INTEGER, SAVE :: ISCR1MAX
      INTEGER, SAVE :: ISCP1MIN
      INTEGER, SAVE :: ISCP1MAX
      INTEGER, SAVE :: ISCP2MIN
      INTEGER, SAVE :: ISCP2MAX
      INTEGER, SAVE :: ISCN1MIN
      INTEGER, SAVE :: ISCN1MAX
      INTEGER, SAVE :: ISCN2MIN
      INTEGER, SAVE :: ISCN2MAX
      INTEGER, SAVE :: ISCN3MIN
      INTEGER, SAVE :: ISCN3MAX
      INTEGER, SAVE :: ISCM1MIN
      INTEGER, SAVE :: ISCM1MAX
      INTEGER, SAVE :: ISCM2MIN
      INTEGER, SAVE :: ISCM2MAX
      INTEGER, SAVE :: ISCM3MIN
      INTEGER, SAVE :: ISCM3MAX
      INTEGER, SAVE :: ISCM4MIN
      INTEGER, SAVE :: ISCM4MAX
      CHARACTER*130, SAVE, ALLOCATABLE :: SCRFILE(:)
	CHARACTER*130, SAVE, ALLOCATABLE :: SCPFILE(:,:)
	CHARACTER*130, SAVE, ALLOCATABLE :: SCNFILE(:,:,:)
	CHARACTER*130, SAVE, ALLOCATABLE :: SCMFILE(:,:,:,:)
C Output keys
      INTEGER, SAVE :: ICCON
      INTEGER, SAVE :: ICFTR
      INTEGER, SAVE :: ICKOV
      INTEGER, SAVE :: ICATM
      INTEGER, SAVE :: ICGRP
      INTEGER, SAVE :: ICPSI
      INTEGER, SAVE :: ICPSK
      INTEGER, SAVE :: ICTMP
      INTEGER, SAVE :: ICBND
      INTEGER, SAVE :: ICFRS
      INTEGER, SAVE :: ICPOT
      INTEGER, SAVE :: ICFAT
      INTEGER, SAVE :: ICDOS
      INTEGER, SAVE :: ICGRF
      INTEGER, SAVE :: ICHYB
      INTEGER, SAVE :: ICSCF
      INTEGER, SAVE :: ICHBR
      INTEGER, SAVE :: ICHPP
      INTEGER, SAVE :: ICOUT
      INTEGER, SAVE :: ICOPT
      INTEGER, SAVE :: ICODF
      INTEGER, SAVE :: ICCHI
C Terminal print-out
      CHARACTER*130, SAVE :: OUTFILE
C Output files
      CHARACTER*130, SAVE :: CONFILE
      CHARACTER*130, SAVE :: FTRFILE
      CHARACTER*130, SAVE :: KOVFILE
      CHARACTER*130, SAVE :: ATMFILE
      CHARACTER*130, SAVE :: GRPFILE
      CHARACTER*130, SAVE :: PSIFILE
      CHARACTER*130, SAVE :: PSKFILE
      CHARACTER*130, SAVE :: TMPFILE
      CHARACTER*130, SAVE :: BNDFILE
      CHARACTER*130, SAVE :: FRSFILE
      CHARACTER*130, SAVE :: POTFILE
      CHARACTER*130, SAVE :: FATFILE
      CHARACTER*130, SAVE :: DOSFILE
      CHARACTER*130, SAVE :: GRFFILE
      CHARACTER*130, SAVE :: HYBFILE
      CHARACTER*130, SAVE :: SCFFILE
      CHARACTER*130, SAVE :: HBRFILE
      CHARACTER*130, SAVE :: HPPFILE
      CHARACTER*130, SAVE :: OPTFILE
      CHARACTER*130, SAVE :: ODFFILE
      CHARACTER*130, SAVE :: CHIFILE
      END MODULE COM_LMTOUT



      MODULE COM_UNIT
*************************************************************
* Containes input and output units.                         *
*************************************************************
      INTEGER, SAVE :: IUN
      INTEGER, SAVE :: IUT
      INTEGER, SAVE :: IUR
      INTEGER, SAVE :: IUI
      END MODULE COM_UNIT



	MODULE COM_EXTENS
*************************************************************
* Contains default extensions
*************************************************************
	INTEGER     , SAVE :: LENEXTN
C Controls
	CHARACTER*60, SAVE :: EXTNCTR
	CHARACTER*60, SAVE :: EXTNLOG
	CHARACTER*60, SAVE :: EXTNABR
	CHARACTER*60, SAVE :: EXTNINF
C Main Inputs
	CHARACTER*60, SAVE :: EXTNINI
	CHARACTER*60, SAVE :: EXTNSTR
	CHARACTER*60, SAVE :: EXTNSCS
C Additional inputs
	CHARACTER*60, SAVE :: EXTNHOP
	CHARACTER*60, SAVE :: EXTNHUB
	CHARACTER*60, SAVE :: EXTNPNT
	CHARACTER*60, SAVE :: EXTNKOV
	CHARACTER*60, SAVE :: EXTNENR
C Outputs
	CHARACTER*60, SAVE :: EXTNCON 
	CHARACTER*60, SAVE :: EXTNFTR
	CHARACTER*60, SAVE :: EXTNATM
	CHARACTER*60, SAVE :: EXTNGRP
	CHARACTER*60, SAVE :: EXTNOUT
	CHARACTER*60, SAVE :: EXTNSCF
	CHARACTER*60, SAVE :: EXTNHBR
	CHARACTER*60, SAVE :: EXTNHPP
	CHARACTER*60, SAVE :: EXTNBND
	CHARACTER*60, SAVE :: EXTNFRS
	CHARACTER*60, SAVE :: EXTNFAT
	CHARACTER*60, SAVE :: EXTNPSI
	CHARACTER*60, SAVE :: EXTNPSK
	CHARACTER*60, SAVE :: EXTNDOS
	CHARACTER*60, SAVE :: EXTNGRF
	CHARACTER*60, SAVE :: EXTNHYB
	CHARACTER*60, SAVE :: EXTNPOT
	CHARACTER*60, SAVE :: EXTNOPT
	CHARACTER*60, SAVE :: EXTNODF
	CHARACTER*60, SAVE :: EXTNCHI
	CHARACTER*60, SAVE :: EXTNTMP
	CHARACTER*60, SAVE :: EXTNSCR
	CHARACTER*60, SAVE :: EXTNSCN
	CHARACTER*60, SAVE :: EXTNSCP
	END MODULE COM_EXTENS


      MODULE COM_LABELS
*************************************************************
* Containes labels for elements and spins                   *
*************************************************************
      CHARACTER*10, SAVE, ALLOCATABLE :: TXTEL(:) !!! Elements labels
      CHARACTER*10, SAVE, ALLOCATABLE :: TXTAT(:) !!! Elements labels
      CHARACTER*2 , SAVE :: TXTSP(2)              !!! Spin labels
      CHARACTER*10, SAVE :: TXTST                 !!! Structure label
      CHARACTER*64, SAVE :: HEADINI               !!! INIFILE head
      CHARACTER*64, SAVE :: HEADSTR               !!! STRFILE head
      CHARACTER*130, SAVE :: PATHSCRT              !!! path to scratch files
      CHARACTER*130, SAVE :: NAMEDENS              !!! name of density file
      CHARACTER*130, SAVE :: NAMELMTO              !!! name of atom LMTO file
      CHARACTER*130, SAVE :: NAMEATOM              !!! name of atomic file
      CHARACTER*130, SAVE :: NAMESTRC              !!! name of structure file
	CHARACTER*2, SAVE :: ELEMENT(0:104)         !!! elements names
      END MODULE COM_LABELS


      MODULE COM_MASHDEP
*************************************************************
* Containes mashine dependent constants                     *
*************************************************************
      REAL*8, SAVE :: EPSMAS       !!! mashine accuracy
      INTEGER, SAVE :: NVAL        !!! direct access parameter
	INTEGER, SAVE :: LENREL      !!! lenght of real #'s (byte)
	INTEGER, SAVE :: LENINT      !!! lenght of integers (byte)
	INTEGER, SAVE :: LENCMP      !!! lenghts of complex (byte)
      END MODULE COM_MASHDEP


      MODULE COM_LMTMEM
***************************************************************
*  Traces dynamical memory usage inside the program.          *
***************************************************************
      REAL*8, SAVE :: CURMEM       !!! current memory usage
      REAL*8, SAVE :: TOTMEM       !!! maximum memory usage
      CHARACTER*60, SAVE :: TXTMEM !!! peak memory text
      END MODULE COM_LMTMEM





      MODULE COM_LATPAR
***************************************************************
*  Containes lattice parameter data.                          *
***************************************************************
      REAL*8, SAVE :: PAR           !!! lattice parameter
      REAL*8, SAVE :: PAR0          !!! lattice parameter (uncompressed)
      REAL*8, SAVE :: VV0           !!! volume compression
      REAL*8, SAVE :: ORTH(2:3)     !!! orthorombicity
      REAL*8, SAVE :: OMEGAC        !!! cell volume
      REAL*8, SAVE :: STRAIN(3,3,2) !!! applied strain
      END MODULE COM_LATPAR


      MODULE COM_SPHERE
***************************************************************
*  Containes MT-spheres, atomic spheres, etc                  *
***************************************************************
      REAL*8, SAVE :: SWZ                    !!! Wigner-Seitz sphere
	REAL*8, SAVE :: OVRMAX                 !!! Maximum allowed overlap
	REAL*8, SAVE :: CLSMAX                 !!! Cluster size
      REAL*8, SAVE, ALLOCATABLE :: SMT(:)    !!! Augmentation sphere
      REAL*8, SAVE, ALLOCATABLE :: SMTS(:)   !!! non-touching MT-sphere
      REAL*8, SAVE, ALLOCATABLE :: SROU(:)   !!! Circumscribed sphere
      REAL*8, SAVE, ALLOCATABLE :: SASA(:)   !!! Atomic sphere
      REAL*8, SAVE, ALLOCATABLE :: ZNUC(:)   !!! Nuclear charge
      REAL*8, SAVE, ALLOCATABLE :: ZVAL(:)   !!! # of valence electrons
      REAL*8, SAVE, ALLOCATABLE :: ZSEM(:)   !!! # of semicore electrons
      REAL*8, SAVE, ALLOCATABLE :: ZCOR(:)   !!! Core charge
      REAL*8, SAVE, ALLOCATABLE :: AMAS(:)   !!! Atomic mass
      END MODULE COM_SPHERE




      MODULE COM_KAPPAR
***************************************************************
*  Containes # of tails and their energies.                   *
***************************************************************
      INTEGER, SAVE :: NPAN                       !!! # of panels
      INTEGER, SAVE, ALLOCATABLE :: NKAP(:)       !!! # of kappa's
      COMPLEX*16, SAVE, ALLOCATABLE :: EKAP(:,:)  !!! tail energies
      COMPLEX*16, SAVE, ALLOCATABLE :: EK(:,:)    !!! tail energies
      REAL*8, SAVE :: EDGE                        !!! Lowest Energy for Evald
      END MODULE COM_KAPPAR





      MODULE COM_PANPAR
***************************************************************
*  Containes basis set data.                                  *
***************************************************************
      LOGICAL, SAVE, ALLOCATABLE :: LTPAN(:,:,:,:)
      LOGICAL, SAVE, ALLOCATABLE :: LSPAN(:,:,:,:)
      INTEGER, SAVE, ALLOCATABLE :: IATS(:,:)
      INTEGER, SAVE, ALLOCATABLE :: ISTS(:,:)
      END MODULE COM_PANPAR




      MODULE COM_NBANDS
**************************************************************
*  Containes # of bands.                                     *
**************************************************************
      INTEGER, SAVE, ALLOCATABLE :: NBNDT(:)  !!! total # of bands
      INTEGER, SAVE, ALLOCATABLE :: NBNDF(:)  !!! input # of filled bands
      INTEGER, SAVE, ALLOCATABLE :: NBNDE(:)  !!! input # of empty bands
      INTEGER, SAVE, ALLOCATABLE :: NBNDC(:)  !!! input # of bands at Ef
      INTEGER, SAVE, ALLOCATABLE :: NBNDH(:)  !!! hamiltonian expansion
	INTEGER, SAVE :: MBNDF     ! # of filled bands determined 
	INTEGER, SAVE :: MBNDE     ! # of empty bands determined 
	INTEGER, SAVE :: MBNDC     ! # of crossing bands determined 
      REAL*8, SAVE :: POLUS                   !!! broadening parameter
	REAL*8, SAVE :: WINDOW     ! window around Ef
      END MODULE COM_NBANDS




      MODULE COM_MAGDAT
**************************************************************
*  Controls spin splitting                                   *
**************************************************************
      INTEGER, SAVE, ALLOCATABLE :: ISPL(:)    !!! control key
      REAL*8,  SAVE, ALLOCATABLE :: BEXT(:)    !!! value of external field
      REAL*8,  SAVE, ALLOCATABLE :: BSPL(:)    !!! value of spin splitting
	REAL*8,  SAVE, ALLOCATABLE :: AXMAG(:,:) !!! local magnetization axis
	REAL*8,  SAVE, ALLOCATABLE :: AXEXT(:,:) !!! external field axis
      END MODULE COM_MAGDAT




      MODULE COM_INPPAR
**************************************************************
*  Containes Enu's, Dnu's, and related arrays.               *
**************************************************************
      REAL*8, SAVE, ALLOCATABLE :: ENY(:,:,:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: DNY(:,:,:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: TNY(:,:,:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: DHLOG(:,:,:,:,:)
      INTEGER, SAVE, ALLOCATABLE :: MNY(:,:,:,:)
      INTEGER, SAVE, ALLOCATABLE :: LNY(:,:,:,:)
      INTEGER, SAVE, ALLOCATABLE :: ITS(:,:,:,:)
      INTEGER, SAVE, ALLOCATABLE :: NODES(:,:,:,:)
      INTEGER, SAVE, ALLOCATABLE :: NCLS(:,:,:)
      END MODULE COM_INPPAR



      MODULE COM_LATTICE
**************************************************************
*  Containes direct and reciprocal lattice vectors.          *
**************************************************************
      INTEGER, SAVE :: ISTRAIN               !!! distort plane wave sphere
      INTEGER, SAVE :: IBZCUBE               !!! generate k-mesh in cube
      REAL*8, SAVE :: RBAS(3,3)              !!! direct translations
      REAL*8, SAVE :: GBAS(3,3)              !!! reciprocal translations
      REAL*8, SAVE :: BBAS(3,3)              !!! Brillouin Zone
      REAL*8, SAVE :: ORIGIN(3)              !!! Origin in k-space
      REAL*8, SAVE :: AKBAS(3,3,3)           !!! FFT-grid in BZ
      REAL*8, SAVE :: RKBAS(3,3,3)           !!! reciprocal FFT grid in BZ
      REAL*8, SAVE, ALLOCATABLE :: TAU(:,:)  !!! basis atoms in distorted cell
      REAL*8, SAVE, ALLOCATABLE :: TAO(:,:)  !!! basis atoms in original cell
      END MODULE COM_LATTICE





      MODULE COM_INPTTR
*************************************************************
* Contains k-mesh sampling parameters.                      *
*************************************************************
      INTEGER, SAVE :: KEYTTR
      INTEGER, SAVE :: NDIV(3)
      INTEGER, SAVE :: NDIW(3)
      INTEGER, SAVE :: NDIC(3)
      INTEGER, SAVE :: NDIR
      INTEGER, SAVE :: NDIVDIR
	REAL*8, SAVE :: TFERMI
	REAL*8, SAVE :: GBROAD
      REAL*8, SAVE, ALLOCATABLE :: SH(:,:)
      REAL*8, SAVE, ALLOCATABLE :: HSPNT(:,:,:)
      CHARACTER*1, SAVE, ALLOCATABLE :: PNTNOT(:,:)
      END MODULE COM_INPTTR



	MODULE COM_SEMICOR
*************************************************************
* Contains semicore electrons arrays.                       *
*************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NSEM(:)    !
	INTEGER, SAVE, ALLOCATABLE :: NDS(:,:)
	INTEGER, SAVE, ALLOCATABLE :: LBS(:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ESEM(:,:)
	END MODULE COM_SEMICOR


      MODULE COM_CORELEC
*************************************************************
* Containes core electrons data.                            *
*************************************************************
      INTEGER, SAVE, ALLOCATABLE :: ICOR(:)    ! switch for soft core
      INTEGER, SAVE, ALLOCATABLE :: NLEVC(:)   ! # of deep core levels
      INTEGER, SAVE, ALLOCATABLE :: NLEVT(:)   ! total # of levels
      INTEGER, SAVE, ALLOCATABLE :: LNUM(:,:)  ! packed n,l,j
      INTEGER, SAVE, ALLOCATABLE :: LEVC(:,:)  ! shows core/valence level
      INTEGER, SAVE, ALLOCATABLE :: LEVN(:,:)  ! main quantum number 
      INTEGER, SAVE, ALLOCATABLE :: LEVL(:,:)  ! orbital quantum number
      INTEGER, SAVE, ALLOCATABLE :: LEVJ(:,:)  ! total moment (times 2)
      REAL*8,  SAVE, ALLOCATABLE :: ELEV(:,:)  ! level energies (Ry)
      REAL*8,  SAVE, ALLOCATABLE :: ZLEV(:,:)  ! level occupancies
      END MODULE COM_CORELEC




	MODULE COM_CHANNELS
*************************************************************
* Containes channels data                                   *
*************************************************************
C  CONTROLS
	INTEGER, SAVE :: IUCTR  !   Control information                Permanent
	INTEGER, SAVE :: IULOG  !   Log     information                Permanent
	INTEGER, SAVE :: IUABR  !   Abort   information                Permanent
C  ELECTRONS FILES
	INTEGER, SAVE :: IUINP  !   Read all input files                                  
      INTEGER, SAVE :: IUOUT  !   Standard output to OUTFILE         Permanent          
      INTEGER, SAVE :: IUAIN  !   Additional output channel          Permanent          
      INTEGER, SAVE :: IUSCF  !   Charge density SCFFILE        Permanent/Temporary     
      INTEGER, SAVE :: IUTIN  !   Terminal Input                                        
      INTEGER, SAVE :: IUTOU  !   Terminal Output                                       
      INTEGER, SAVE :: IUMAD  !   Madelung's constants               Temporary          
      INTEGER, SAVE :: IUCON  !   Structure constants CONFILE    Permanent/Temporary    
      INTEGER, SAVE :: IUCEL  !   Cell description                   Temporary          
      INTEGER, SAVE :: IUBND  !   Energy bands BNDFILE           Permanent/Temporary    
      INTEGER, SAVE :: IUPSK  !   Wave functions PSKFILE         Permanent/Temporary    
      INTEGER, SAVE :: IUPOT  !   Full potential POTFILE         Permanent/Temporary    
      INTEGER, SAVE :: IUNMT  !   Potential parameters NMT           Temporary          
      INTEGER, SAVE :: IUWFK  !   Wave functions for one k-point     Temporary          
      INTEGER, SAVE :: IUPSI  !   Wave functions PSIFILE         Permanent/Temporary    
      INTEGER, SAVE :: IUPUL  !   Pulay forces                       Temporary          
      INTEGER, SAVE :: IUDOS  !   Density of states data             Temporary          
      INTEGER, SAVE :: IUPIN  !   Potential parameters INT           Temporary          
      INTEGER, SAVE :: IUPDS  !   Pseudodensity parameters           Temporary          
      INTEGER, SAVE :: IUPFC  !   Pulay force parameters             Temporary          
      INTEGER, SAVE :: IUTOS  !   Number of states matrix            Temporary          
      INTEGER, SAVE :: IUBMF  !   Broyden mixing in FTB              Permanent          
      INTEGER, SAVE :: IUBRO  !   Broyden mixing in LMT              Temporary          
      INTEGER, SAVE :: IUSSC  !   Screened structure constants                          
C RESPONSE FILES      
      INTEGER, SAVE :: IUOPT  !   Optical properties                 Temporary
	INTEGER, SAVE :: IUCHI  !   Dyanmical susceptibility           Temporary
C DYNAMICAL MEAN FIELD FILES
	INTEGER, SAVE :: IUTDS  !   Greens function in DMFT            Temporary
	INTEGER, SAVE :: IUCEW  !   Eigenvectors of H+Sigma(iWn)       Temporary
	INTEGER, SAVE :: IUSGM  !   Self-energy/Hybridization          Temporary
	INTEGER, SAVE :: IUBIM  !   Broyden mixing for the impurity    Temporary
	INTEGER, SAVE :: IUBSG  !   Broyden mixing for the self-energy Temporary
	INTEGER, SAVE :: IUHAM  !   Hamiltonian and overlap matrices   Temporary
	INTEGER, SAVE :: IUGRF  !   Greens function on the real axis   Temporary
	INTEGER, SAVE :: IUHYB  !   Hybridization on the real axis     Temporary
	END

