        MODULE COM_QMESH
*************************************************************
*  Containes q-vectors
*************************************************************
        INTEGER, SAVE :: NQPNT                  !!! # of q-points
        INTEGER, SAVE :: NQDIV(3)               !!! divisions
        REAL*8, SAVE, ALLOCATABLE :: QPNT(:,:)  !!! q-points
        END MODULE COM_QMESH



        MODULE COM_LRTCOM
***************************************************************
*  General control keys for the program.                      *
***************************************************************
        INTEGER, SAVE :: LWHI                             !!! weight key
        INTEGER, SAVE :: LDLF                             !!! delrad key
        INTEGER, SAVE :: LWAV                             !!! epi spd-wave key
        INTEGER, SAVE :: IRMT                             !!! rmta key
	  CHARACTER*10, SAVE :: QVECTOR                     !!! q-vector
        CHARACTER*10, SAVE :: MOVTASK                     !!! polarizations
	  CHARACTER*60, SAVE :: RUNTASK                     !!! polarizations
        END MODULE COM_LRTCOM


        MODULE COM_FREQU
*************************************************************
*  Containes frequency/energy intervals
*************************************************************
        INTEGER, SAVE :: NOMGCHI                !!! # of frequencies for chi(q,w)
        INTEGER, SAVE :: NOMGOPT                !!! # of frequencies for optics
        INTEGER, SAVE :: NOMGGRF                !!! # of frequencies for G(w)
        INTEGER, SAVE :: NENRDOS                !!! # of energy points for DOS
        INTEGER, SAVE :: NENRADS              !!! # of energy points for G(E)
        INTEGER, SAVE :: NENROPT                !!! # of energy points for sigma in DMFT
        REAL*8, SAVE, ALLOCATABLE :: OMGCHI(:)  !!! frequencies
        REAL*8, SAVE, ALLOCATABLE :: OMGOPT(:)  !!! frequencies
        REAL*8, SAVE, ALLOCATABLE :: OMGGRF(:)  !!! frequencies
        REAL*8, SAVE, ALLOCATABLE :: ENRDOS(:)  !!! energies
        REAL*8, SAVE, ALLOCATABLE :: ENRADS(:)  !!! energies
        END MODULE COM_FREQU


        MODULE COM_SUBGROUP
*************************************************************
*  Containes group of q-vector
*************************************************************
        INTEGER, SAVE :: NSGQ                   !!! group of q-vector
        INTEGER, SAVE :: NGEQ                   !!! # of equivalent operations
        INTEGER, SAVE, ALLOCATABLE :: ISG(:,:)
        INTEGER, SAVE, ALLOCATABLE :: KSYMPN(:,:)
        INTEGER, SAVE, ALLOCATABLE :: MSYMPN(:,:)
        INTEGER, SAVE, ALLOCATABLE :: IGS(:,:,:)
        END MODULE COM_SUBGROUP


        MODULE COM_SYMMESH
*************************************************************
*  Containes group of q-vector
*************************************************************
        INTEGER, SAVE, ALLOCATABLE :: IG0IG1(:,:)
        INTEGER, SAVE, ALLOCATABLE :: IK0IK1(:,:)
        END MODULE COM_SYMMESH


        MODULE COM_SYMMESW
*************************************************************
*  Containes group of q-vector
*************************************************************
        INTEGER, SAVE, ALLOCATABLE :: IG0IG1(:,:)
        INTEGER, SAVE, ALLOCATABLE :: IK0IK1(:,:)
        END MODULE COM_SYMMESW


        MODULE COM_EIGVEC
*************************************************************
*  Containes eigenvectors for k and k+q
*************************************************************
        COMPLEX*16, SAVE, ALLOCATABLE :: APHK(:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: APHQ(:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: APJK(:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: APJQ(:,:,:,:,:)
        END MODULE COM_EIGVEC


        MODULE COM_CHIDAT
*************************************************************
*  Containes eigenvectors for k and k+q
*************************************************************
        INTEGER, SAVE :: NCHI      !!! dimension
	  INTEGER, SAVE :: ICUBS     !!! output spins
	  INTEGER, SAVE :: IMATR     !!! matrix elements
        INTEGER, SAVE, ALLOCATABLE :: INCHI(:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: CHI0(:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: CHIT(:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: CHIM(:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: GMAT(:,:,:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: GINT(:,:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: CHH(:,:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: CHJ(:,:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: CJH(:,:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: CJJ(:,:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: BHH(:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: BHJ(:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: BJH(:,:,:,:,:)
        COMPLEX*16, SAVE, ALLOCATABLE :: BJJ(:,:,:,:,:)
        END MODULE COM_CHIDAT


	  MODULE COM_DHAMPAR
**************************************************************
* Contains potential parameters of the hamiltonian.          *
* Spontaneoulsy contains TOS matrix in Hpars.                *
**************************************************************
	  COMPLEX*16, SAVE, ALLOCATABLE :: HPARHH(:,:,:,:,:)
	  COMPLEX*16, SAVE, ALLOCATABLE :: HPARJH(:,:,:,:,:)
	  COMPLEX*16, SAVE, ALLOCATABLE :: HPARHJ(:,:,:,:,:)
	  COMPLEX*16, SAVE, ALLOCATABLE :: HPARJJ(:,:,:,:,:)
	  COMPLEX*16, SAVE, ALLOCATABLE :: DELHHH(:,:,:,:,:)
	  COMPLEX*16, SAVE, ALLOCATABLE :: DELHJH(:,:,:,:,:)
	  COMPLEX*16, SAVE, ALLOCATABLE :: DELHHJ(:,:,:,:,:)
	  COMPLEX*16, SAVE, ALLOCATABLE :: DELHJJ(:,:,:,:,:)
        END MODULE COM_DHAMPAR



        MODULE COM_OPTMAT
*************************************************************
*  Containes dielectric function matrix                   
*************************************************************
	  REAL*8, SAVE, ALLOCATABLE :: WGTOPT(:,:)     ! Optical weights
	  REAL*8, SAVE, ALLOCATABLE :: WGTDOS(:  )     ! DOS     weights
	  COMPLEX*16, SAVE, ALLOCATABLE :: E2W(:,:,:)  ! Imaginary susceptibility
	  COMPLEX*16, SAVE, ALLOCATABLE :: V2A(:,:)    ! Average velocities
	  COMPLEX*16, SAVE, ALLOCATABLE :: OPTN(:,:,:) ! Optical matrix el.
        END MODULE COM_OPTMAT



        MODULE COM_ODFMAT
*************************************************************
*  Containes optical conductivity within DMFT
*************************************************************
	  COMPLEX*16, SAVE, ALLOCATABLE :: SIGMAW(:,:,:)     ! optical conductivity
	  COMPLEX*16, SAVE, ALLOCATABLE :: WGTODF(:,:,:,:)   ! Optical weights
	  COMPLEX*16, SAVE, ALLOCATABLE :: OPLL(:,:,:,:)     ! Optical matrix el. LL
	  COMPLEX*16, SAVE, ALLOCATABLE :: OPRL(:,:,:,:)     ! Optical matrix el. RL
	  COMPLEX*16, SAVE, ALLOCATABLE :: OPLR(:,:,:,:)     ! Optical matrix el. LR
	  COMPLEX*16, SAVE, ALLOCATABLE :: OPRR(:,:,:,:)     ! Optical matrix el. RR
        END MODULE COM_ODFMAT



