	SUBROUTINE PHN_SPECTRA(IQPNT)
************************************************************
*  Diagonalization of the dynamical matrix and calculation *
*  of the phonon frequences, polarizations.                *
*  Written by S.Yu.Savrasov in Max-Planck-Institute, FRG.  *
************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                     !!! from setup
      USE COM_CONTROL, ONLY : LIFT,LMTO,NPFR,MODE                        !!! from readini
      USE COM_DYNMAT,  ONLY : DYNMAT,OM,DPOL                             !!! from dynmat
	USE COM_PARAM,   ONLY : NSORT,IS,ILST,ILSB,ILSV,NATOM,NORBS,NSPIN  !!! from readini
	USE COM_SPHERE,  ONLY : AMAS                                       !!! from init
      USE COM_UNIT,    ONLY : IUN,IUR,IUT,IUI                            !!! from setup
	USE COM_PHNOUT,  ONLY : ICPHN                                      !!! from init
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16, ALLOCATABLE :: HMAT(:,:)
	COMPLEX*16, ALLOCATABLE :: OMAT(:,:)
	REAL*8, ALLOCATABLE :: E(:)
C  SET FORCE THEOREM
	IFT=1    !!! first force formula for dynamical matrix is used
	IF(MODE.NE.4)RETURN
	IF(ICPHN.EQ.0)RETURN
C  Memory allocations
	ALLOCATE(HMAT(3*NATOMMEM,3*NATOMMEM))
	ALLOCATE(OMAT(3*NATOMMEM,3*NATOMMEM))
	ALLOCATE(E(3*NATOMMEM))
C  MAKE MATRIX FOR DIAGONALIZATION
	IDIM1=0
	DO 40 INUC1=1,NATOM                      !!! over nuclear shifts
	DO 30 I=1,3                              !!! over x,y,z
	IDIM1=IDIM1+1
	IDIM=0
	DO 20 INUC=1,NATOM                       !!! over nuclear shifts
	DO 10 J=1,3                              !!! over x,y,z
	IDIM=IDIM+1
	AMS=1.D0/SQRT(AMAS(IS(INUC1))*AMAS(IS(INUC)))
	AMS=AMS*109.752                          !!! since Me=5.4876D-04 m.a.u.
	DMRE=DREAL(DYNMAT(I,INUC1,J,INUC,1))     !!! set up zeroes where necess.
	DMIM=DIMAG(DYNMAT(I,INUC1,J,INUC,1))
 	IF(ABS(DMRE).LE.1.D-05)DMRE=0.D0
 	IF(ABS(DMIM).LE.1.D-05)DMIM=0.D0
	IF(IDIM.EQ.IDIM1)DMIM=0.D0
	HMAT(IDIM1,IDIM)=DCMPLX(DMRE,DMIM)*AMS
	OMAT(IDIM1,IDIM)=(0.D0,0.D0)
	IF(IDIM1.EQ.IDIM)OMAT(IDIM1,IDIM)=(1.D0,0.D0)
  10  CONTINUE                                 !!! over x,y,z
  20  CONTINUE                                 !!! over nuclear shifts
  30  CONTINUE                                 !!! over x,y,z
  40  CONTINUE                                 !!! over nuclear shifts
C  SYMMETRIZE DYNAMICAL MATRIX, ENFORCE HERMITIANNESS
	DO 60 IDIM1=1,3*NATOM
	DO 50 IDIM=1,IDIM1
	DMRE10=+DREAL(HMAT(IDIM1,IDIM))
	DMRE01=+DREAL(HMAT(IDIM,IDIM1))
	DMRE=0.5D0*(DMRE10+DMRE01)
	DMIM10=+DIMAG(HMAT(IDIM1,IDIM))
	DMIM01=-DIMAG(HMAT(IDIM,IDIM1))
	DMIM=0.5D0*(DMIM10+DMIM01)
	HMAT(IDIM1,IDIM)=DCMPLX(DMRE,+DMIM)
	HMAT(IDIM,IDIM1)=DCMPLX(DMRE,-DMIM)
  50	CONTINUE
  60	CONTINUE
C  EIGENVALUE PROBLEM
	CALL LIB_EIGEN1C(3*NATOM,HMAT,OMAT,E,1,1,1,IERR,0,1.D-07)
C  FREQUENCES AND POLARIZATIONS
	DO 100 IOMG=1,3*NATOM                    !!! over phonon branches
      IF(E(IOMG).GE.0.D0)OM(IOMG)=+SQRT(E(IOMG))*
     &  13.6058D0*0.3162278D-02                  !!! units are eV
      IF(E(IOMG).LT.0.D0)OM(IOMG)=-SQRT(-E(IOMG))*
     &  13.6058D0*0.3162278D-02                  !!! units are eV
	IDIM=0
	DO 90 INUC=1,NATOM                       !!! over nuclear shifts
	DO 80 I=1,3                              !!! over x,y,z
	IDIM=IDIM+1
	DPOL(I,INUC,IOMG)=OMAT(IDIM,IOMG)
  80  CONTINUE                                 !!! over x,y,z
  90  CONTINUE                                 !!! over nuclear shifts
 100  CONTINUE                                 !!! over phonon branches
C  Memory deallocations
	DEALLOCATE(HMAT,OMAT,E)
	END
