	SUBROUTINE PLZ_POLARZ(IQPNT)
******************************************************************
*   Control module for package of programs finding polarizabi-   *
*   lity function for a crystal. The polarizability is defined   *
*   here as a response to exp^i(q+G)r                            *
*   Written by S.Savrasov.                                       *
*   Created 01.08.95, Last revised 09.08.95.                     *
******************************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                     !!! from setup
      USE COM_CONTROL, ONLY : LIFT,LMTO,NPFR,MODE                        !!! from readini
      USE COM_PHNCOM,  ONLY : ITERMAX,NITER                              !!! from readini
	USE COM_PARAM,   ONLY : NSORT,IS,ILST,ILSB,ILSV,NATOM,NORBS,NSPIN  !!! from readini
	USE COM_KMESH,   ONLY : NPNT                                       !!! from makettr
	USE COM_KAPPAR,  ONLY : NPAN                                       !!! from readini
	USE COM_SUBGROUP,ONLY : NSGQ,NGEQ,ISG,KSYMPN                       !!! from delmesh
      USE COM_PLZCOM,  ONLY : NPLZ,LPLZ,IPLZ,NPTB                        !!! from readlrt
	USE COM_PHNCHN  ,ONLY : IUPLY,IUPSY,IUDMT,IUDTS,IUDIN,IUPDD        !!! from channels
	USE COM_PHNOUT,  ONLY : ICPLZ                                      !!! from storage
      USE COM_UNIT,    ONLY : IUN,IUR,IUT,IUI                            !!! from setup
	IMPLICIT REAL*8 (A-H,O-Z)
	IF(LPLZ.EQ.0)RETURN
	IF(ICPLZ.EQ.2)RETURN
	CALL PLZ_ALLOCATE(IQPNT,'A')
	DO 3000 IPTB=1,NPTB                        !!! over perturbation vectors
	CALL PLZ_PLZFILES(IPTB,IQPNT,IUPLY,'I',' ')!!! initialize polarizability in MTS
	CALL PLZ_PLSFILES(IPTB,IQPNT,IUPSY,'I',' ')!!! initialize pseudopolarizability
	DO 2000 IPAN=1,1                           !!! DO over panel #1
	CALL BND_POTPAR(IPAN)                      !!! different potential parameters
	CALL PLZ_SPLZPAR(IPTB,IPAN,IQPNT)          !!! matrix elements of e^iqr from MTS
	CALL PLZ_SPLZPIN(IPTB,IPAN,IQPNT)          !!! matrix elements of e^iqr from INT
	CALL PLZ_SPLIPAR(IPTB,IPAN,IQPNT)          !!! interstitial potential parameters
	CALL PLZ_SNULPLZ(IPTB,IPAN,IQPNT)          !!! set zeros in polarizability
   	CALL PLZ_SNULPLS(IPTB,IPAN,IQPNT)          !!! set zeros in polarizability
	DO ISGQ=1,NSGQ                             !!! DO over star of the q-vector 
	DO IPNT=1,NPNT(IPAN)                       !!! DO over k-points
	IF(KSYMPN(ISGQ,IPNT).GE.0)THEN
C COMPUTE POLARIZABILITY CONTRIBUTION FROM A GIVEN POINT
	CALL DEL_SLMTVEC(IPNT,ISGQ,IPAN,IQPNT)     !!! rotate variational coef
	CALL PLZ_SPLZMTO(IPNT,ISGQ,IPAN,IQPNT)     !!! get <k+qj'|e^iqr|kj>_mts
      CALL DEL_DPSIVEC(-1,1,IPNT,ISGQ,IPAN,IQPNT)!!! pseudo |kj>
      CALL PLZ_SPLZINT(IPNT,ISGQ,IPTB,IPAN,IQPNT)!!! get <k+qj'|e^iqr|kj>_int
      CALL PLZ_SPLZINL(IPNT,ISGQ,IPTB,IPAN,IQPNT)!!! get <k+qj'|e^iqr|kj>_int
	CALL PLZ_SPLZBND(IPNT,ISGQ,IPTB,IPAN,IQPNT)!!! integrate polariz. in MTS
 	CALL PLZ_SPLSBND(IPNT,ISGQ,IPTB,IPAN,IQPNT)!!! integrate polariz. in INT
	ENDIF
      ENDDO                                      !!! over k-points
      ENDDO                                      !!! over star of the q-vector
C COMPUTE POLARIZABILITY FUNCTION
	CALL PLZ_SPLZMTS(IPTB,IPAN,IQPNT)          !!! pi_q(r) within MTs 
      CALL PLZ_SPLZEND(IPTB,IPAN,IQPNT)          !!! pi_q(r) in the INT
	CLOSE(IUDMT,STATUS='DELETE')               !!! close file with dHpar
	CLOSE(IUDTS,STATUS='DELETE')               !!! close file with dTos 
	CLOSE(IUDIN,STATUS='DELETE')               !!! close file with Rfun
	CLOSE(IUPDD,STATUS='DELETE')               !!! close file with Xfun
 2000 CONTINUE                                   !!! over energy panels
 3000 CONTINUE                                   !!! over perturbation vectors
C SYMMETRIZATION OF POLARIZABILITY 
   	CALL PLZ_SPLZSYM(IQPNT)                    !!! symmterize pi_q(r) in MTS
   	CALL PLZ_SPLSSYM(IQPNT)                    !!! symmterize pseudo-pi_q(r) 
	CALL PLZ_ALLOCATE(IQPNT,'D')               !!! deallocate arrays
	END
