      SUBROUTINE POT_GGA96EXCH(RHO,GRHO,G2RHO,ALRHO,EXLDA,VXLDA,
     &                       EXGGA,VXGGA,DVX2GGA,DVXLGGA)
******************************************************************
*  Returns GGA96 correction to the exchange energy/potential.    *
*  for non-spin polarized system. For both spins call first      *
*  for rho=2*rho_up, then for rho=2*rho_dn.                      *
****************************************************************** 
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 GRHO(-1:1),G2RHO(-1:1,-1:1),DVX2GGA(-1:1,-1:1)
C  Fundamential constants
	DATA A0/1.D0/ ! Bohr radius
     &      ,E2/2.D0/ ! El.charge^2
     &      ,PI/3.141592653589793D0/
C  GGA constants
	DATA CGAM/0.310906908696549009E-01/  ! (1-ln(2))/pi^2
     &      ,CBET/0.066725000000000000D+00/  !
     &      ,CKAP/0.804000000000000000D+00/  !
     &      ,CMUU/0.219516451220895831D+00/  !  betta*pi^2/3
	EXGGA=0.D0
	VXGGA=0.D0
C  Check case rho=0
      IF(RHO.LE.1.D-09)RETURN
C Set |Grad(Rho)|
      ABSG=DCONJG(GRHO(-1))*(GRHO(-1))+
     &     DCONJG(GRHO( 0))*(GRHO( 0))+
     &     DCONJG(GRHO(+1))*(GRHO(+1))
      ABSG=SQRT(ABSG)
	IF(ABSG.LE.1.D-09)RETURN
C Set Grad(Rho)*Grad(|Grad(Rho)|)
      ABSG2=0.D0
      DO MY=-1,1
      ABSG2=ABSG2+DCONJG(GRHO(MY))*(
     &            G2RHO(MY,-1)*DCONJG(GRHO(-1))+
     &            G2RHO(MY, 0)*DCONJG(GRHO( 0))+
     &            G2RHO(MY,+1)*DCONJG(GRHO(+1)))
      ENDDO
      ABSG2=ABSG2/ABSG
C  Set electron gas data
      RS=(3.D0/(4.D0*PI*RHO))**(1.D0/3.D0)
	FK=(3.D0*PI*PI*RHO)**(1.D0/3.D0)
	SK=SQRT(4.D0*FK/PI/A0)
	SS=ABSG/(2.D0*FK*RHO)
C  Set derivatives
	DFK=(3.D0*PI*PI)**(1.D0/3.D0)*1.D0/3.D0*RHO**(-2.D0/3.D0)
	DSK=DFK/SQRT(PI*FK*A0)
	DSS=-2.D0*ABSG/(3.D0*FK*RHO*RHO)
	GSS=1.D0/(2.D0*FK*RHO)
	DGSS=-2.D0/(3.D0*FK*RHO*RHO)
C  Compute GGA96 to exchange
	FXS1=CKAP-CKAP/(1.D0+CMUU*SS*SS/CKAP)
	DFXS=2.D0*CMUU*SS/(1.D0+CMUU*SS*SS/CKAP)**2*DSS
	GFXS=2.D0*CMUU*SS/(1.D0+CMUU*SS*SS/CKAP)**2*GSS
	DGFXS=2.D0*CMUU*SS/(1.D0+CMUU*SS*SS/CKAP)**2*DGSS+
     &2.D0*CMUU*DSS/(1.D0+CMUU*SS*SS/CKAP)**2*GSS-8.D0*
     &CMUU**2*SS**2*DSS/CKAP/(1.D0+CMUU*SS*SS/CKAP)**3*GSS
      G2FXS=
     &2.D0*CMUU*GSS/(1.D0+CMUU*SS*SS/CKAP)**2*GSS-8.D0*
     &CMUU**2*SS**2*GSS/CKAP/(1.D0+CMUU*SS*SS/CKAP)**3*GSS
	EXGGA=EXLDA*FXS1
	VSURF=VXLDA*GFXS*ABSG+EXLDA*DGFXS*RHO*ABSG+
     &EXLDA*G2FXS*RHO*ABSG2/ABSG+EXLDA*GFXS*
     &RHO/ABSG*(ALRHO-ABSG2/ABSG)
	VXGGA=VXLDA*FXS1+RHO*EXLDA*DFXS-VSURF
C Set contributions from the second gradeints
      DVXLGGA=-EXLDA*GFXS*RHO/ABSG
	DVX2FAC=-EXLDA*G2FXS*RHO/ABSG/ABSG+EXLDA*GFXS*
     &RHO/ABSG/ABSG/ABSG
      DO MY1=-1,1
      DO MY =-1,1
      DVX2GGA(MY1,MY)=
     &DCONJG(GRHO(MY1))*DVX2FAC*DCONJG(GRHO(MY))
      ENDDO
      ENDDO
	END



      SUBROUTINE POT_GGA96CORR(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,
     &ALUP,ALDN,ECLDA,VCLDAUP,VCLDADN,ECGGA,VCGGAUP,VCGGADN,
     &DVC2GGAUP,DVC2GGADN,DVCLGGAUP,DVCLGGADN)
******************************************************************
*  Returns GGA96 correction to the correlation energy/potential. *
******************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 
     &    GRUP(-1:1),G2UP(-1:1,-1:1),DVC2GGAUP(-1:1,-1:1)
     &   ,GRDN(-1:1),G2DN(-1:1,-1:1),DVC2GGADN(-1:1,-1:1)
C  Fundamential constants
	DATA A0/1.D0/ ! Bohr radius
     &    ,E2/2.D0/ ! El.charge^2
     &    ,PI/3.141592653589793D0/
C  GGA constants
	DATA CGAM/0.310906908696549009E-01/  ! (1-ln(2))/pi^2
     &    ,CBET/0.066725000000000000D+00/  !
     &    ,CKAP/0.804000000000000000D+00/  !
     &    ,CMUU/0.219516451220895831D+00/  !  betta*pi^2/3
	ECGGA=0.D0
	VCGGAUP=0.D0
	VCGGADN=0.D0
C  Check case rho=0
      RHO=ROUP+RODN
      IF(RHO.LE.1.D-09)RETURN
C  Set |Grad(Rho)|
      ABSG=DCONJG(GRUP(-1)+GRDN(-1))*(GRUP(-1)+GRDN(-1))+
     &     DCONJG(GRUP( 0)+GRDN( 0))*(GRUP( 0)+GRDN( 0))+
     &     DCONJG(GRUP(+1)+GRDN(+1))*(GRUP(+1)+GRDN(+1))
      ABSG=SQRT(ABSG)
	IF(ABSG.LE.1.D-09)RETURN
C  Set Grad(Rho)*Grad(Rho_up)
      GRRUP=DCONJG(GRUP(-1)+GRDN(-1))*GRUP(-1)+
     &      DCONJG(GRUP( 0)+GRDN( 0))*GRUP( 0)+
     &      DCONJG(GRUP(+1)+GRDN(+1))*GRUP(+1)
C  Set Grad(Rho)*Grad(Rho_up)
      GRRDN=DCONJG(GRUP(-1)+GRDN(-1))*GRDN(-1)+
     &      DCONJG(GRUP( 0)+GRDN( 0))*GRDN( 0)+
     &      DCONJG(GRUP(+1)+GRDN(+1))*GRDN(+1)
C  Set Grad(Rho)*Grad(|Grad(Rho)|)
      ABSG2=0.D0
      DO MY=-1,1
      ABSG2=ABSG2+DCONJG(GRUP(MY)+GRDN(MY))*
     &((G2UP(MY,-1)+G2DN(MY,-1))*DCONJG(GRUP(-1)+GRDN(-1))+
     & (G2UP(MY, 0)+G2DN(MY, 0))*DCONJG(GRUP( 0)+GRDN( 0))+
     & (G2UP(MY,+1)+G2DN(MY,+1))*DCONJG(GRUP(+1)+GRDN(+1)))
      ENDDO
      ABSG2=ABSG2/ABSG
C  Total Laplasian
	ALPS=ALUP+ALDN
C  Set electron gas data
      RS=(3.D0/(4.D0*PI*RHO))**(1.D0/3.D0)
      YY=(ROUP-RODN)/RHO
	FK=(3.D0*PI*PI*RHO)**(1.D0/3.D0)
	SK=SQRT(4.D0*FK/PI/A0)
	SS=ABSG/(2.D0*FK*RHO)
	PH=((1.D0+YY)**(2.D0/3.D0)+(1.D0-YY)**(2.D0/3.D0))/2.D0
	TT=ABSG/(2.D0*PH*SK*RHO)
	VCGGAUP=0.D0
	VCGGADN=0.D0
	DO ISPIN=1,2                                   !!! over spins
C  Set derivatives
	IF(ISPIN.EQ.1)DYY=-(ROUP-RODN)/RHO/RHO+1.D0/RHO
	IF(ISPIN.EQ.2)DYY=-(ROUP-RODN)/RHO/RHO-1.D0/RHO
	DFK=(3.D0*PI*PI)**(1.D0/3.D0)*1.D0/3.D0*RHO**(-2.D0/3.D0)
	DSK=DFK/SQRT(PI*FK*A0)
	DSS=-2.D0*ABSG/(3.D0*FK*RHO*RHO)
	GSS=1.D0/(2.D0*FK*RHO)
	DGSS=-2.D0/(3.D0*FK*RHO*RHO)
	DPH=((1.D0+YY)**(-1.D0/3.D0)-(1.D0-YY)**(-1.D0/3.D0))/
     &  3.D0*DYY
	DTT=-ABSG/(2.D0*PH*PH*SK*SK*RHO*RHO)*
     &  (SK*RHO*DPH+SK*PH+PH*RHO*DSK)
	GTT=1.D0/(2.D0*PH*SK*RHO)
	DGTT=-1.D0/(2.D0*PH*PH*SK*SK*RHO*RHO)*
     &  (SK*RHO*DPH+SK*PH+PH*RHO*DSK)
C  Compute GGA96 to correlation
C  1)
	CD=-1.D0/(CGAM*E2/A0)*ECLDA/PH**3
	IF(ISPIN.EQ.1)DECLDA=(VCLDAUP-ECLDA)/RHO
	IF(ISPIN.EQ.2)DECLDA=(VCLDADN-ECLDA)/RHO
	DCD=-1.D0/(CGAM*E2/A0)*(DECLDA*PH-ECLDA*3.D0*DPH)/PH**4
C  2)
	CA=CBET/CGAM/(EXP(CD)-1.D0)
	DCA=-CBET/CGAM/(EXP(CD)-1.D0)**2*EXP(CD)*DCD
C  3)
	CT1=1.D0+CA*TT**2
	DCT1=2.D0*TT*DTT*CA+DCA*TT*TT
	GCT1=2.D0*TT*GTT*CA
	DGCT1=2.D0*DTT*GTT*CA+2.D0*TT*DGTT*CA+2.D0*TT*GTT*DCA
	G2CT1=2.D0*GTT*GTT*CA
	CT2=CT1+CA**2*TT**4
	DCT2=DCT1+4.D0*TT**3*DTT*CA**2+2.D0*CA*DCA*TT**4
	GCT2=GCT1+4.D0*TT**3*GTT*CA**2
	DGCT2=DGCT1+4.D0*3.D0*TT**2*DTT*GTT*CA**2+
     &  4.D0*TT**3*DGTT*CA**2+4.D0*TT**3*GTT*2.D0*CA*DCA
	G2CT2=G2CT1+4.D0*3.D0*TT**2*GTT*GTT*CA**2
	CT=CT1/CT2
	DCT=(CT2*DCT1-DCT2*CT1)/(CT2*CT2)
	GCT=(CT2*GCT1-GCT2*CT1)/(CT2*CT2)
	DGCT=(DCT2*GCT1+CT2*DGCT1-DGCT2*CT1-GCT2*DCT1)/(CT2*CT2)-
     &  2.D0*(CT2*GCT1-GCT2*CT1)/CT2**3*DCT2
	G2CT=(GCT2*GCT1+CT2*G2CT1-G2CT2*CT1-GCT2*GCT1)/(CT2*CT2)-
     &  2.D0*(CT2*GCT1-GCT2*CT1)/CT2**3*GCT2
C  4)
	CL=1.D0+CBET/CGAM*TT*TT*CT
	DCL=CBET/CGAM*(2.D0*TT*DTT*CT+TT*TT*DCT)
	GCL=CBET/CGAM*(2.D0*TT*GTT*CT+TT*TT*GCT)
	DGCL=CBET/CGAM*(2.D0*DTT*GTT*CT+2.D0*TT*DGTT*CT+
     &  2.D0*TT*GTT*DCT+2.D0*TT*DTT*GCT+TT*TT*DGCT)
	G2CL=CBET/CGAM*(2.D0*GTT*GTT*CT+2.D0*TT*GTT*GCT+
     &  2.D0*TT*GTT*GCT+TT*TT*G2CT)
C  Answer
	ECGGA=E2/A0*CGAM*PH**3*LOG(CL)
	DECGGA=E2/A0*CGAM*(3.D0*PH**2*DPH*LOG(CL)+PH**3/CL*DCL)
	GECGGA=E2/A0*CGAM*PH**3/CL*GCL
	DGECGGA=E2/A0*CGAM*(3.D0*PH**2*DPH/CL*GCL-
     &  PH**3/CL**2*DCL*GCL+PH**3/CL*DGCL)
	G2ECGGA=E2/A0*CGAM*PH**3*(G2CL/CL-GCL*GCL/CL**2)
	IF(ISPIN.EQ.1)
     &VCGGAUP=VCGGAUP+ECGGA+RHO*DECGGA-
     &ABSG*GECGGA-RHO*ABSG2/ABSG*G2ECGGA-
     &RHO/ABSG*(ALPS-ABSG2/ABSG)*GECGGA
	IF(ISPIN.EQ.2)
     &VCGGADN=VCGGADN+ECGGA+RHO*DECGGA-
     &ABSG*GECGGA-RHO*ABSG2/ABSG*G2ECGGA-
     &RHO/ABSG*(ALPS-ABSG2/ABSG)*GECGGA
	IF(ISPIN.EQ.1)
     &VCGGAUP=VCGGAUP-RHO/ABSG*GRRUP*DGECGGA
	IF(ISPIN.EQ.1)
     &VCGGADN=VCGGADN-RHO/ABSG*GRRUP*DGECGGA
	IF(ISPIN.EQ.2)
     &VCGGAUP=VCGGAUP-RHO/ABSG*GRRDN*DGECGGA
	IF(ISPIN.EQ.2)
     &VCGGADN=VCGGADN-RHO/ABSG*GRRDN*DGECGGA
C Set contributions from the second gradeints
	IF(ISPIN.EQ.1)DVCLGGAUP=-RHO/ABSG*GECGGA
	IF(ISPIN.EQ.2)DVCLGGADN=-RHO/ABSG*GECGGA
	IF(ISPIN.EQ.1)DVC2FACUP=-RHO/ABSG/ABSG*G2ECGGA+
     &                         RHO/ABSG/ABSG/ABSG*GECGGA
	IF(ISPIN.EQ.2)DVC2FACDN=-RHO/ABSG/ABSG*G2ECGGA+
     &                         RHO/ABSG/ABSG/ABSG*GECGGA
      DO MY1=-1,1
      DO MY =-1,1
	IF(ISPIN.EQ.1)
     &DVC2GGAUP(MY1,MY)=DCONJG(GRUP(MY1)+GRDN(MY1))*
     &        DVC2FACUP*DCONJG(GRUP(MY )+GRDN(MY ))
	IF(ISPIN.EQ.2)
     &DVC2GGADN(MY1,MY)=DCONJG(GRUP(MY1)+GRDN(MY1))*
     &        DVC2FACDN*DCONJG(GRUP(MY )+GRDN(MY ))
	ENDDO
	ENDDO
	ENDDO                                          !!! over spins
	END
