	SUBROUTINE POT_MULTFTR
*********************************************************
* Calculate multipoles as reciprocal lattice sums.      *
*********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,IS,ILSV        !!! from readini
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
      USE COM_SPHERE  ,ONLY: ZNUC,ZCOR,SWZ,SMT                !!! from readini
      USE COM_LATTICE ,ONLY: TAU                              !!! from readstr
      USE COM_SYMMLM  ,ONLY: SYM,LMSYM                        !!! from symlmm
      USE COM_POTDAT  ,ONLY: POTPS                            !!! from potfull
      USE COM_DENDAT  ,ONLY: RHOPS,RHOPS1                     !!! from rhofull
      USE COM_MULTRO  ,ONLY: AMTOT,PSMOM,PSMOM1,QMAD          !!! from multftr
      USE COM_PLWMESH ,ONLY: NPLW,GPLW                        !!! from plwmesh
      USE COM_ACCURG  ,ONLY: BET                              !!! from plwmesh
      USE COM_MASHDEP ,ONLY: LENCMP                           !!! from setup
	USE COM_UNIT    ,ONLY: IUT                              !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER, INTRINSIC :: SIZE
	REAL*8, ALLOCATABLE :: BES(:)
	REAL*8, ALLOCATABLE :: BESD(:)
	REAL*8, ALLOCATABLE :: COFL(:,:)
	COMPLEX*16, ALLOCATABLE :: YLM(:)
	COMPLEX*16, ALLOCATABLE :: GAUS(:,:)
	COMPLEX*16 EXPT
	DATA PI/3.141592653589793D0/
      CALL ABORTNOW()
	IF(NPLW.EQ.-1)RETURN
C MEMORY ALLOCATIONS FOR SCRATCH ARRAYS
	ALLOCATE(BES(-1:LMAXVMEM))
	ALLOCATE(BESD(0:LMAXVMEM))
	ALLOCATE(COFL(0:LMAXVMEM,NSORTMEM))
	ALLOCATE(YLM(LIMVMEM))
	ALLOCATE(GAUS(0:LMAXVMEM,NATOMMEM))
C CALCULATE AUXILARY CHARGE DENSITY IN FOURIER REPRESENTATION
C ALSO CALCULATE MULTIPOLE CHARGE FROM WHOLE SPACE
	LMM=0
	ZAUX=0.D0
	ZPSD=0.D0
	DO ISPIN=1,NSPIN
	ZPSD=ZPSD+OMEGAC*RHOPS(0,ISPIN,1)
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	LMM=MAX(LMM,ILSV(ISORT))
	ZAUX=ZAUX-DREAL(PSMOM1(1,IATOM,ISPIN))*SQRT(4.D0*PI)*SMT(ISORT)
	DO L=0,ILSV(ISORT)
	COFL(L,ISORT)=4.D0*PI/OMEGAC*SMT(ISORT)**(L+1)/FAC2(2*L-1)
	DO M=-L,L
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	QMAD(1,ISYM,IATOM,ISPIN)=(0.D0,0.D0)
	QMAD(2,ISYM,IATOM,ISPIN)=(0.D0,0.D0)
	ENDIF
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	REN1=ZPSD/ZAUX
	DO 1000 IPLW=0,NPLW                  !!! over plane waves
	GX=GPLW(1,IPLW)*2.D0*PI/PAR
	GY=GPLW(2,IPLW)*2.D0*PI/PAR
	GZ=GPLW(3,IPLW)*2.D0*PI/PAR
	AG=SQRT(GX**2+GY**2+GZ**2)
	AG2=AG*AG
C SET SPHERICAL HARMONICS
	IF(AG.GE.1.D-10)THEN
	CALL LIB_SPHHARM(GX/AG,GY/AG,GZ/AG,LMM,YLM)
	ELSE
	YLM(1)=1.D0/SQRT(4.D0*PI)
	DO LM=2,(LMM+1)**2
	YLM(LM)=(0.D0,0.D0)
	ENDDO
	ENDIF
C SET AUXILARY PSEUDODENSITY PSRHO1
	DO ISPIN=1,NSPIN                     !!! over spins
	RHOPS1(IPLW,ISPIN,1)=(0.D0,0.D0)
	DO IATOM=1,NATOM                     !!! over atoms
	ISORT=IS(IATOM)
C CALCULATE EXPONENTIAL FACTOR
	SCP=GPLW(1,IPLW)*TAU(1,IATOM)+
     &      GPLW(2,IPLW)*TAU(2,IATOM)+
     &      GPLW(3,IPLW)*TAU(3,IATOM)
	EXPT=EXP((0.D0,-1.D0)*2.D0*PI*SCP)
	DO L=0,ILSV(ISORT)
	IF(L.EQ.0)AGL=1.D0
	IF(L.NE.0)AGL=AG**L
	EXPG=EXP(-0.25D0*AG**2/BET(L,ISORT)**2)
	GAUS(L,IATOM)=COFL(L,ISORT)*EXPT*AGL*EXPG
	DO M=-L,L
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	RHOPS1(IPLW,ISPIN,1)=RHOPS1(IPLW,ISPIN,1)+
     &GAUS(L,IATOM)*YLM(LM)*PSMOM1(ISYM,IATOM,ISPIN)*REN1
	ENDIF
      ENDDO   
      ENDDO   
      ENDDO                                !!! over atoms
      ENDDO                                !!! over spins
C SET MULTIPOLE CHARGE
	IF(AG.GE.1.D-10)THEN                 !!! not for G=0 !
	DO IATOM=1,NATOM                     !!! over atoms
	ISORT=IS(IATOM)
	CALL LIB_BESSR(AG*SMT(ISORT),BES(0),BESD(0),ILSV(ISORT))
	BES(-1)=COS(AG*SMT(ISORT))/(AG*SMT(ISORT))
	SCP=GPLW(1,IPLW)*TAU(1,IATOM)+
     &    GPLW(2,IPLW)*TAU(2,IATOM)+
     &    GPLW(3,IPLW)*TAU(3,IATOM)
	EXPT=EXP((0.D0,-1.D0)*2.D0*PI*SCP)
	DO ISPIN=1,NSPIN                     !!! over spins
	DO L=0,ILSV(ISORT)
	DO M=-L,L
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	QMAD(1,ISYM,IATOM,ISPIN)=QMAD(1,ISYM,IATOM,ISPIN)+
     &4.D0*PI*SMT(ISORT)/DBLE(2*L+1)*RHOPS(IPLW,ISPIN,1)*
     &4.D0*PI*DCONJG(EXPT*YLM(LM))/AG*BES(L-1)
	QMAD(2,ISYM,IATOM,ISPIN)=QMAD(2,ISYM,IATOM,ISPIN)+
     &4.D0*PI*SMT(ISORT)/DBLE(2*L+1)*RHOPS1(IPLW,ISPIN,1)*
     &4.D0*PI*DCONJG(EXPT*YLM(LM))/AG*BES(L-1)
	ENDIF
	ENDDO            
      ENDDO   
      ENDDO                                !!! over spins
      ENDDO                                !!! over atoms
	ENDIF
1000  CONTINUE                             !!! over plane waves
C CHECK OUT PSEUDOCHARGE
	ZPS0=OMEGAC*(RHOPS (0,1,1)+RHOPS (0,NSPIN,1))/2.D0*DBLE(NSPIN)
	ZPS1=OMEGAC*(RHOPS1(0,1,1)+RHOPS1(0,NSPIN,1))/2.D0*DBLE(NSPIN)
	IF(ABS(ZPS0+ZPS1).GT.1.D-08)THEN
	WRITE(IUT,*)'Warning from <MULTFTR>:'
	WRITE(IUT,*)'Renormalization error in auxilary charge:'
	WRITE(IUT,*)ZPS0,ZPS1
	ENDIF
C MEMORY DEALLOCATIONS
	DEALLOCATE(BES)
	DEALLOCATE(BESD)
	DEALLOCATE(COFL)
	DEALLOCATE(YLM)
	DEALLOCATE(GAUS)
	END
