	SUBROUTINE POT_VCFTR
*********************************************************
* Calculates coulomb part of pseudopotential which is   *
* true Coulomb potential in the interstitial region.    *
*********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS           !!! from readini
      USE COM_LATPAR ,ONLY: PAR,OMEGAC                        !!! from readini
      USE COM_POTDAT ,ONLY: POTPS                             !!! from potfull
      USE COM_DENDAT ,ONLY: RHOPS,RHOPS1                      !!! from rhofull
      USE COM_PLWMESH,ONLY: NPLW,GPLW                         !!! from plwmesh
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 PSTOT,PSTOT1
	DATA PI/3.141592653589793D0/
      CALL ABORTNOW()
	IF(NPLW.EQ.-1)RETURN
	DO ISPIN=1,NSPIN                     !!! over spins
	DO IPLW=0,NPLW                       !!! over plane waves
	GX=GPLW(1,IPLW)*2.D0*PI/PAR
	GY=GPLW(2,IPLW)*2.D0*PI/PAR
	GZ=GPLW(3,IPLW)*2.D0*PI/PAR
	G2=GX**2+GY**2+GZ**2
	PSTOT =(RHOPS (IPLW,1,1)+RHOPS (IPLW,NSPIN,1))/2.D0*DBLE(NSPIN)
	PSTOT1=(RHOPS1(IPLW,1,1)+RHOPS1(IPLW,NSPIN,1))/2.D0*DBLE(NSPIN)
	IF(G2.GE.1.D-10)POTPS(IPLW,ISPIN,1)=8.D0*PI*(PSTOT+PSTOT1)/G2
	IF(G2.LT.1.D-10)POTPS(IPLW,ISPIN,1)=(0.D0,0.D0) ! for G=0
	ENDDO                                !!! over spins
      ENDDO                                !!! over plane waves
	END



	SUBROUTINE POT_VPINT
*********************************************************
* Calculates convolution of pseudopotential with inter- *
* stitial plane-wave integral.                          *
*********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_PARAM    ,ONLY: NATOM,NSORT,NSPIN,NORBS         !!! from readini
      USE COM_LATPAR   ,ONLY: PAR,OMEGAC                      !!! from readini
      USE COM_POTDAT   ,ONLY: POTPS,POTPS1                    !!! from potfull
      USE COM_PLWMESH  ,ONLY: NPLW,GPLW,KVEC,KGMIN,KGMAX      !!! from plwmesh
      USE COM_PLW2MSH  ,ONLY: N2PLW,G2PLW,INDG2               !!! from plw2msh
      USE COM_MASHDEP  ,ONLY: NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER, INTRINSIC :: SIZE
	COMPLEX*16, ALLOCATABLE :: GINT(:)
	COMPLEX*16 GVAL
	DATA PI/3.141592653589793D0/
	IF(NPLW.EQ.-1)RETURN
	ALLOCATE(GINT(0:9*NPLWMEM))
C START CODE
	M1D=2*KGMAX(1)-2*KGMIN(1)+1
	M2D=2*KGMAX(2)-2*KGMIN(2)+1
	M3D=2*KGMAX(3)-2*KGMIN(3)+1
	M12D=M1D*M2D
	M123D=M1D*M2D*M3D
	CALL MAKEPLW_WINT(GINT)              !!! take the integral for PWs
	DO IORBS=1,NORBS                     !!! over iorbs
	DO IUPDN=1,NSPIN                     !!! over spins
	DO IPLW=0,NPLW                       !!! over plane waves
	POTPS1(IPLW,IUPDN,IORBS)=(0.D0,0.D0)
	M1I=KVEC(1,IPLW)+2*KGMIN(1)
	M2I=KVEC(2,IPLW)+2*KGMIN(2)
	M3I=KVEC(3,IPLW)+2*KGMIN(3)
	DO JPLW=0,NPLW                       !!! over plane waves
      M1=KVEC(1,JPLW)-M1I
      M2=KVEC(2,JPLW)-M2I
      M3=KVEC(3,JPLW)-M3I
      IDIM=1+M1+M1D*M2+M12D*M3
	I2PLW=INDG2(IDIM)
	POTPS1(IPLW,IUPDN,IORBS)=POTPS1(IPLW,IUPDN,IORBS)+
     &POTPS(JPLW,IUPDN,IORBS)*GINT(I2PLW)
      ENDDO                                !!! over plane waves
      ENDDO                                !!! over plane waves
      ENDDO                                !!! over spins
      ENDDO                                !!! over iorbs
C MEMORY DEALLOCATIONS
	DEALLOCATE(GINT)
	END
