	SUBROUTINE POT_VCOUL
************************************************************
*  The calculation of the Coulomb potential within MTS     *
*  without nuclear part.                                   * 
************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,IS,ILSV         !!! from readini
      USE COM_SPHERE ,ONLY: SMT                               !!! from readini
      USE COM_SYMMLM ,ONLY: SYM,LMSYM                         !!! from symlmm
      USE COM_POTDAT ,ONLY: POT                               !!! from potfull
      USE COM_DENDAT ,ONLY: RHO                               !!! from rhofull
      USE COM_MULTRO ,ONLY: QMAD                              !!! from multrho
      USE COM_RADMESH,ONLY: NRAD,H,R,DR,WEIGHT,RL             !!! from radmesh
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16, ALLOCATABLE :: VC1(:)
	COMPLEX*16, ALLOCATABLE :: VC2(:)
	COMPLEX*16, ALLOCATABLE :: WORK1(:)
	COMPLEX*16, ALLOCATABLE :: WORK2(:)
	COMPLEX*16, ALLOCATABLE :: ROTOT(:)
	COMPLEX*16  CMAD
	DATA PI/3.141592653589793D0/
      CALL ABORTNOW()
	SQPI4=2.D0*SQRT(PI)
C MEMORY ALLOCATIONS FOR SCRATCH ARRAYS
	ALLOCATE(VC1(0:NRADMEM))
	ALLOCATE(VC2(0:NRADMEM))
	ALLOCATE(WORK1(0:NRADMEM))
	ALLOCATE(WORK2(0:NRADMEM))
	ALLOCATE(ROTOT(0:NRADMEM))
C **** CALCULATE COULOMB CONTRIBUTION TO THE POTENTIAL **** 
	DO IATOM=1,NATOM                    !!! over all atoms in cell
	ISORT=IS(IATOM)
	DO ISPIN=1,NSPIN                    !!! over spins
C DO OVER SPHERICAL HARMONICS EXPANSIONS
	DO L=0,ILSV(ISORT)                  !!! over l
	DO M=-L,L                           !!! over m
	LM=L*(L+1)+M+1
	ISYM=LMSYM(LM,IATOM)
	SL=SMT(ISORT)**L
	IF(SYM(LM,IATOM))THEN               !!! for only non-zero components
C COULOMB CONTRIBUTION  (EXTERNAL PART FROM MTS)
	CMAD=(QMAD(1,ISYM,IATOM,1)+QMAD(1,ISYM,IATOM,NSPIN)+
     &QMAD(2,ISYM,IATOM,1)+QMAD(2,ISYM,IATOM,NSPIN))/2.D0*
     &DBLE(NSPIN)
	DO IRAD=0,NRAD(ISORT)               !!! over radial mesh
	POT(IRAD,ISYM,IATOM,ISPIN,1)=POT(IRAD,ISYM,IATOM,ISPIN,1)+
     &2.D0*CMAD*RL(IRAD,L,ISORT)/SL
      ENDDO                               !!! over radial mesh
C COULOMB CONTRIBUTION  (INTERNAL PART FROM CHARGE DENSITY)
	DO IRAD=0,NRAD(ISORT)
	ROTOT(IRAD)=(RHO(IRAD,ISYM,IATOM,1,1)+
     &RHO(IRAD,ISYM,IATOM,NSPIN,1))/2.D0*DBLE(NSPIN)
	ENDDO
C SOLUTION OF THE POISSON EQUATION
	DO IRAD=0,NRAD(ISORT)
	WORK1(IRAD)=ROTOT(IRAD)*WEIGHT(IRAD,ISORT)*RL(IRAD,L,ISORT)
	ENDDO 
	CALL LIB_CQD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	VC1(0)=0.0
	DO IRAD=1,NRAD(ISORT)
	VC1(IRAD)=WORK2(IRAD)
	DO L1=1,L+1
	VC1(IRAD)=VC1(IRAD)/R(IRAD,ISORT)
      ENDDO   
      ENDDO   
	WORK1(0)=0.0
	DO IRAD=1,NRAD(ISORT)
	WORK1(IRAD)=ROTOT(IRAD)*WEIGHT(IRAD,ISORT)
	DO L1=1,L+1
	WORK1(IRAD)=WORK1(IRAD)/R(IRAD,ISORT)
      ENDDO   
      ENDDO   
	IF(L.LE.1)THEN
	CALL LIB_CQD(H(ISORT),WORK1(0),WORK2(0),NRAD(ISORT)+1)
	ELSE
	CALL LIB_CQD(H(ISORT),WORK1(2),WORK2(2),NRAD(ISORT)-1)
	WORK2(1)=-(WORK1(1)+WORK1(2))*0.5D0*H(ISORT)
	ENDIF
	VC2(0)=0.0
	DO IRAD=1,NRAD(ISORT)
	VC2(IRAD)=(WORK2(NRAD(ISORT))-WORK2(IRAD))*RL(IRAD,L,ISORT)
      ENDDO   
	IF(L.EQ.0)VC2(0)=WORK2(NRAD(ISORT))
C COULOMB POTENTIAL READY 
	PL=2.D0/(2.D0*L+1.D0)
	DO IRAD=0,NRAD(ISORT)               !!! over radial mesh
	POT(IRAD,ISYM,IATOM,ISPIN,1)=POT(IRAD,ISYM,IATOM,ISPIN,1)+
     &PL*(VC1(IRAD)+VC2(IRAD))
      ENDDO                               !!! over radial mesh
	ENDIF                               !!! for only non-zero components
      ENDDO                               !!! over m
      ENDDO                               !!! over l
      ENDDO                               !!! over spins
      ENDDO                               !!! over all atoms in cell
C MEMORY DEALLOCATIONS FOR SCRATCH ARRAYS
	DEALLOCATE(VC1)
	DEALLOCATE(VC2)
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	DEALLOCATE(ROTOT)
	END
