        subroutine qmc_phase(nfs,nts,beta,w,kern)
        implicit none

        integer nfs,nts,k,j
        real*8 w(nfs),beta,dt,pi
        complex*16 kern(nts+1,nfs),ii

        pi=acos(-1.d0)
        ii=cmplx(0.d0,1.d0)
        dt=beta/dble(nts)

        do k=1,nfs
            w(k) = (-nfs-1+2*k)*pi/dble(nts)
            do j=1,nts+1
                kern(j,k) = cos(w(k)*(j-1))-ii*sin(w(k)*(j-1))
            end do
            w(k)=w(k)/dt
        end do

        end subroutine qmc_phase
        
        subroutine qmc_w2t(nts,nfs,beta,a0,c,info)
        implicit none

! a0(nfs): complex, input, G(iw_n)
! c(nfs): complex, output, G(\tau)
! info: 1 for orbital diagonal, zero for off diagonal.

        integer nts,nfs,info
        real*8 beta
        complex*16 a0(nfs),c(nts)

        integer i,j,k
        real*8 bu,bd,b,ed,dt,pi
        complex*16 ii

        real*8, allocatable :: w(:)
        complex*16, allocatable :: a(:),kern(:,:)
        
        allocate(w(nfs),a(nfs),kern(nts+1,nfs))

        pi=acos(-1.d0)
        ii=cmplx(0.d0,1.d0)
        dt=beta/dble(nts)

        call qmc_phase(nfs,nts,beta,w,kern)

        ed=0.d0
        if (info.eq.1) then
            ed=-(1.d0/a0(nfs)-ii*w(nfs))
            do j=1,nfs
                a(j)=a0(j)-1.d0/(ii*w(j)-ed)
            end do
        else
            a(:)=a0(:) 
        end if

        bu=0.0
        bd=0.0
        do i=1,nfs
            bu=bu-real(a(i))/w(i)**2.0
            bd=bd+1.d0/w(i)**4.0
        end do
        b=bu/bd

        do j=1,nfs
            a(j)=a(j)+b/w(j)**2.0
        end do

        c(1:nts)=0.0
        do i=1,nts
            do j=1,nfs
                c(i) = c(i)+kern(i,j)*a(j)
            end do
        end do

        c(1:nts)=c(1:nts)/beta

        !here G(0):=-<cc^{+}>, namely G(0)=G(0^+), so -0.5 for each orbital.

        do i=1,nts
            c(i)=c(i)-info*dexp(-ed*dt*(i-1))/(dexp(-beta*ed)+1.d0)
        end do
        do i=1,nts
            c(i)=c(i)+b*(i-1.d0)*dt/2.0-b*beta/4.0
        enddo

        ! The F-trans gives Gt=-<Tcc^+>, changes to <Tcc^+>.
        c(1:nts)=-c(1:nts)

        deallocate(w,a,kern)

        end subroutine qmc_w2t

        subroutine qmc_t2w(nts,nfs,beta,a0,c0,info,FDsum,SDsum)
        implicit none

! a0(nts,nts): real, input, G(\tau_1,\tau_2)
! c0(nfs): complex, output, G(iw_n)
! info: 1 for orbital diagonal, zero for orbital off diagonal
! FDsum: sum of first derivative of G(\tau) at two ends
! SDsum: sum of second derivative of G(\tau) at two ends

        integer nts,nfs,info
        real*8 beta,FDsum,SDsum
        complex*16 a0(nts),c0(nfs)

        integer i,j,l1,l2,k
        real*8 dt,om,pi,yp0,ypn,ypn2,yp02
        complex*16 sum, sum1, cf,ii,x1,x2
        
        real*8, allocatable :: w(:)
        complex*16, allocatable :: rincopy(:), kern(:,:),XM(:),xmw(:)
        complex*16, allocatable :: a(:),b(:),c(:),d(:) 
        complex*16, allocatable :: u(:),du(:),dn(:),r(:),q(:),p(:)
        
        allocate(w(nfs))
        allocate(rincopy(0:nts), kern(nts+1,nfs),XM(0:nts),xmw(0:nts))
        allocate(a(0:nts),b(0:nts),c(0:nts),d(0:nts))
        allocate(u(0:nts),du(0:nts),dn(0:nts),r(0:nts),q(0:nts),p(0:nts))

        pi=acos(-1.d0)
        ii=cmplx(0.d0,1.d0)
        dt=beta/dble(nts)

        call qmc_phase(nfs,nts,beta,w,kern)
               
        rincopy(0:nts-1)=-a0(1:nts)

        ! G(i,j)=-<T(c(i)c(j)^+)>, G(i,i)=-<c(i)c(i)^+>=G(0^+)
        ! G(beta)=-G(0^-), -G(0^+)=-G(0^-)+\delta(m1,m2)
        ! G(0^+) is used here for the spline interpolation.

        rincopy(nts)=-rincopy(0)-info

        !
        ! spline interpolation:  the spline is given by
        ! G(tau)=a(i) + b(i) (tau-tau_i) + c(i) ( )^2 + d(i) ( )^3
        !
        
        yp0=rincopy(1)-rincopy(0)
        ypn=rincopy(nts)-rincopy(nts-1)
        yp02=rincopy(0)+rincopy(2)-2.d0*rincopy(1)
        ypn2=rincopy(nts)+rincopy(nts-2)-2.d0*rincopy(nts-1)
        
        write(*,*) 
        write(*,*) '1st moment:', (yp0+ypn)/dt, FDsum
        write(*,*) '2nd moment:',(yp02+ypn2)/dt**2, SDsum
        write(*,*) 

        q(0)=4.d0
        q(1)=2.d0-0.5d0/q(0)
        r(0)=0.5d0/q(0)
        p(0)=2.d0
        p(1)=2.d0-0.5d0/q(0)
        u(0)=(6.d0/dt)*((yp0+ypn)/dt-FDsum)+2.d0*SDsum
        du(0)=u(0)
        dn(1)=3.d0*ypn2/dt**2-0.5d0*SDsum+0.5d0*u(0)/q(0)

        do k=1,nts-3
            q(k+1)=2.d0-0.25d0/q(k)
            r(k)=0.5d0*r(k-1)/q(k)
            u(k)=(3.d0/dt**2)*(rincopy(k+1)+rincopy(k-1)-2.0d0*rincopy(k))
            du(k)=u(k)-0.5d0*du(k-1)/q(k-1)
            p(k+1)=p(k)-2.d0*r(k)*r(k-1)
            dn(k+1)=dn(k)+(-1)**k*du(k)*2.d0*r(k)
        end do

        u(nts-2)=(3.d0/dt**2)*(rincopy(nts-2+1)+rincopy(nts-2-1)-2.0d0*rincopy(nts-2))
        du(nts-2)=u(nts-2)-0.5d0*du(nts-3)/q(nts-3)
        r(nts-2)=0.5d0*r(nts-3)/q(nts-2)

        xmw(nts-1)=dn(nts-2)-du(nts-2)*(0.5d0-r(nts-3))/q(nts-2)
        xmw(nts-1)=xmw(nts-1)/( p(nts-2)-(0.5d0-r(nts-3))**2/q(nts-2) )
        xmw(nts-2)=( du(nts-2)-(0.5d0-r(nts-3))*xmw(nts-1) )/q(nts-2)

        do k=nts-3,1,-1
            xmw(k)=( du(k)+((-1)**k)*r(k-1)*xmw(nts-1)-0.5d0*xmw(k+1) )/q(k)
        end do
        xmw(0)=(  u(0) +xmw(nts-1)-xmw(1)  )/4.d0
        xmw(nts)= SDsum-xmw(0)

        xm(:)=xmw(:)

        !
        ! The following formulas are taken directly from  Stoer and
        ! Bulirsch p. 98
        !
        
        do j = 0, nts-1
            a(j)=rincopy(j)
            c(j)= XM(j)/2.0d0
            b(j)=(rincopy(j+1)-rincopy(j))/dt - &
            (2.0d0*XM(j)+XM(j+1))*dt/6.d0
            d(j)=(XM(j+1)-XM(j))/(6.d0*dt)
        end do

        !
        ! The Spline multiplied by the exponential can now be exlicitely
        ! integrated. The following formulas were obtained using
        ! MATHEMATICA
        ! note: j=1 corresponds to t=0 in Fcoef, while t=dt in a,b,c,d. 
        ! note: Sum of j is from t=0 to t=(L-1)*dt<beta

        do i=1,nfs
            om=w(i)
            sum=0.d0

            do j=1,nts

                sum1 = 0.d0
                cf=1.0d0/kern(j,i)
            
                sum1 = sum1 + 6.0d0*d(j-1)/om**4.0d0
                sum1 = sum1 - 2.0d0*ii*c(j-1)/om**3.0d0
                sum1 = sum1 - b(j-1)/om**2.0d0
                sum1 = sum1 + ii*a(j-1)/om

                sum = sum + sum1*cf

                sum1=0.0d0
                cf=1.0d0/kern(j+1,i)
            
                sum1 = sum1 -  6.0d0*d(j-1)/om**4.0d0
                sum1 = sum1 + (2.0d0*ii*c(j-1)+6.0d0*dt*ii*d(j-1))/om**3.0d0
                sum1 = sum1 + (b(j-1)+2.0d0*dt*c(j-1)+3.0d0*dt**2*d(j-1))/om**2.0d0
                sum1 = sum1 + (-ii*a(j-1)-dt*ii*b(j-1)-dt**2*ii*c(j-1)-dt**3*ii*d(j-1))/om

                sum = sum +sum1 * cf
            end do

        ! the above F-trans is for G(i,j)=-<Tc(i)c(j)^+>.
        ! no minus sign required to keep the form of the Dyson equation.            
         c0(i)=sum

        end do

        deallocate(w,rincopy,kern,XM,xmw,a,b,c,d,u,du,dn,r,q,p)

        end subroutine qmc_t2w



        subroutine qmc_getmoment(ndeg,nfs,gwinv,u,rn,nn,w,mom1,mom2,orb)
        implicit none

! gwinv(nfs): complex, input, 1/gw0(iw_n)
! rn(ndeg): real, input, <n>
! nn(ndeg,ndeg), real, input, <nn>
! mom1: output, first moment
! mom2: output, second moment
! orb: integer, input, orbital index
! 

        integer ndeg,nfs,info,orb
        real*8 u(ndeg,ndeg),rn(ndeg),nn(ndeg,ndeg),mom1,mom2,w(nfs)
        complex*16 gwinv(nfs)

        integer i,j
        real*8 x1,x2

        x1= -real(gwinv(nfs))
        x2= w(nfs)*(aimag(gwinv(nfs))-w(nfs))

        mom1=x1
        do i=1,ndeg
            mom1=mom1+u(orb,i)*(rn(i)-0.5d0)
        end do 

        mom2=x1**2+x2
        do i=1,ndeg
            mom2=mom2+2.d0*x1*u(orb,i)*(rn(i)-0.5d0)
            do j=1,ndeg
                mom2=mom2+u(orb,i)*u(orb,j)*( nn(i,j)-( rn(i)+rn(j) )/2.d0+1.d0/4.d0 ) 
            end do
        end do
        mom2=-mom2
         
        end subroutine qmc_getmoment