	SUBROUTINE QMC_READQMC
************************************************************
*  <READQMC> controls reading the data from QMCFILE.       *
************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTINP  ,ONLY: PROJECT,INPINFO                 !!! from artinp
      USE COM_QMCOUT  ,ONLY: ICQMC,QMCFILE                   !!! from artinp
      USE COM_LSHELL  ,ONLY: LORB                            !!! frin readqmc
	USE COM_EXTENS  ,ONLY: LENEXTN                         !!! from setup
	USE COM_QMCEXT  ,ONLY: EXTNQMC                         !!! from setup
      USE COM_CHANNELS,ONLY: IUINP                           !!! from channels
	USE COM_UNIT    ,ONLY: IUN,IUT,IUR                     !!! from setup
	IMPLICIT REAL*8 (A-H,O-Z)
	CHARACTER STRING*150,INF*130
	CHARACTER*150, ALLOCATABLE :: QMCDATA(:)
	LOGICAL EXIST,MODIFIED,PRESENT
	DATA PI/3.141592653589793D0/
	IF(ICQMC.LE.0)RETURN
C CHECK INPINFO
	INF=INPINFO
	CALL SUBSTR_IN_STR(PRESENT,INF,130,EXTNQMC,LENEXTN)
	IF(.NOT.PRESENT)CALL FULLSTOP('<QMCFILE> identifier not found!')
C SET FILE NAME
      CALL FIND_LBLANK(PROJECT,130,NBP)
      CALL FIND_LBLANK(EXTNQMC,60,NBE)
	QMCFILE=PROJECT(:NBP)//'.'//EXTNQMC(:NBE)
	INQUIRE(FILE=QMCFILE,EXIST=EXIST)
	IF(.NOT.EXIST)CALL FULLSTOP('<QMCFILE> not found!')
C ********************************
C UNDERSTAND THE LENGHT OF INIFILE
	CALL FILE_LENGHT(QMCFILE,NLEN,IUINP)
	ALLOCATE(QMCDATA(NLEN))
C ********************************
C READ QMCFILE TO INTERNAL FILE
	OPEN(UNIT=IUINP,FILE=QMCFILE,STATUS='OLD',FORM='FORMATTED')
	DO ILEN=1,NLEN
	READ(IUINP,FMT='(A150)')QMCDATA(ILEN)
	ENDDO
	CLOSE(IUINP)
C ********************************
C UNDERSTAND INPUT: CLASSIC or MODERN
	STRING=QMCDATA(1)
	CALL LIB_CONVLOW(STRING,150)
	CALL LIB_RMBLANK(STRING,150)
	MODERN=0
	DO IPOS=1,80
	IF(STRING(IPOS:IPOS+11).EQ.'input=modern')MODERN=2
	IF(MODERN.NE.0)EXIT
	ENDDO
	IF(MODERN.EQ.2)MODIFIED=.TRUE. 
	IF(MODERN.NE.2)CALL FULLSTOP('<QMCFILE> format not recognized!')
C READ CONTROL DATA
	CALL QMC_READQMC_CTRL(QMCDATA,NLEN)
C READ DMFT SETTINGS
	CALL QMC_READQMC_DMFT(QMCDATA,NLEN)
C READ SHELL INFORMATION
	CALL QMC_READQMC_SHEL(QMCDATA,NLEN)
C READ QMC SETUP
	CALL QMC_READQMC_SETS(QMCDATA,NLEN)
C ********************************
C QMCFILE HAS BEEN READ, OVERWRITE IT
	IF(MODIFIED)CALL READ_OVWR(QMCFILE,QMCDATA,NLEN)
C MEMORY DEALLOCATIONS
	DEALLOCATE(QMCDATA)
C Compute Matsubara grid
      CALL HUB_MSBMESH
C Compute Gaunt coefficients
	LMAXBMEM=LORB
	LMAXCMEM=3
	LIMCMEM=(LMAXCMEM+1)**2
	CALL INI_MAKEGNT
C ERROR HANDLINGS
C	none
C *****************************************************************
	END SUBROUTINE QMC_READQMC




	SUBROUTINE QMC_READQMC_CTRL(QMCDATA,NLEN)
*******************************************************
* Reads control                                       *
*******************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_QMCPAR  ,ONLY : MODR
      USE COM_IMPGRF  ,ONLY : KVRB                !!! from readqmc
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: QMCCOPY(:)
      CHARACTER*150, QMCDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      LOGICAL DEF,ASK,VAL,NTH,MDF
      DATA PI/3.141592653589793D0/
C **************************************
C READING CONTROLS
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=ctrl',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(QMCDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)QMCDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! CONTROL PARAMETERS:                   '
      IF(MSEC.NE.0)QMCDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(QMCCOPY(NSEC))
      DO ILIN=1,MSEC
      QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
      ENDDO
C **************************************
C READING THE DATA: SELECT CONTROLS
	TCTRL='          '
	HELP='! set: QMCYY1                          '
	LHLP=40
	CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &  'method',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	MODR=-1
      IF(TCTRL(1:6).EQ.'qmcyy1')MODR=11
      IF(TCTRL(1:6).EQ.'qmcyy2')MODR=12
	IF(MODR.EQ.-1)THEN
	WRITE(IUT,*)'Terminated in <READQMC>:'
	WRITE(IUT,*)'Syntax error in control line METHOD'
	CALL FULLSTOP(' ')
	ENDIF
C ******************************************
C READING THE DATA: SELECT VERBOCITY SWITCH
	TCTRL='Full      '
	HELP='! set: Full/None                       '
	LHLP=40
	CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &  'verboc',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	KVRB=-1
      IF(TCTRL(1:6).EQ.'none  ')KVRB=0
      IF(TCTRL(1:6).EQ.'full  ')KVRB=1
      IF(TCTRL(1:6).EQ.'auto  ')KVRB=1
	IF(KVRB.EQ.-1)THEN
	WRITE(IUT,*)'Terminated in <READQMC>:'
	WRITE(IUT,*)'Syntax error in control line VERBOC'
	CALL FULLSTOP(' ')
	ENDIF
C **************************************
      CALL READ_ERRR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,'CTRL')
      DEALLOCATE(QMCCOPY)
C Memory deallocations
C     none
	END SUBROUTINE QMC_READQMC_CTRL


	SUBROUTINE QMC_READQMC_DMFT(QMCDATA,NLEN)
*******************************************************
* Reads DMFT settings                                 *
*******************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_IMPGRF  ,ONLY : CHEM,FILL,KFRM
	USE COM_MSBMESH ,ONLY : TEMP,OBND,NMSB,NMSL,WMSB,KGRD
     &                       ,WMSL,DMSB,DMSL,SMSB,SMSL
     &                       ,FMSB,NOMG,WOMG,WMIN,WMAX
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: QMCCOPY(:)
      LOGICAL DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, QMCDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C **************************************
C READING THE DATA: SELECT DMFT SETTINGS
C NMSB - # of Matsubara energies
C TEMP - # Temperature in K
C NDIT - # of DMFT iterations
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dmft',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(QMCDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)QMCDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! DMFT SETTINGS                         '
      IF(MSEC.NE.0)QMCDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(QMCCOPY(NSEC))
      DO ILIN=1,MSEC
      QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
      ENDDO
C Process NMSB
      NMSB=50
      HELP='! # of Matsubara frequencies            '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nmsb',4,NMSB,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	NMSBMEM=NMSB
	NMSCMEM=NMSB
C Process OBND
      OBND=1.D0
      HELP='! Effective bandwidth for the grid      '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'obnd',4,IN,1,0,OBND,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process NMSL
      NMSL=1024
      HELP='! # of linear Matsubara frequencies     '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nmsl',4,NMSL,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      NMSLMEM=NMSL
C Process NOMG
      NOMG=100
      HELP='! # of frequencies at real axis         '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nomg',4,NOMG,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      NOMGMEM=NOMG
C Process WMIN
      WMIN=-10.D0
      HELP='! Minimal frequency for the real axis   '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'wmin',4,IN,1,0,WMIN,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process WMAX
      WMAX=+10.D0
      HELP='! Maximal frequency for the real axis   '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'wmax',4,IN,1,0,WMAX,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process KGRD
      KGRD=2 ! default is imaginary axis
      HELP='! set current grid to real/imag         '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'grid',4,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	IF(TCTRL(1:4).EQ.'real')KGRD=1
	IF(TCTRL(1:4).EQ.'imag')KGRD=2
C Process Fill
      FILL=0.D0
      HELP='! Mean Filling                          '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'fill',4,IN,1,0,FILL,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Chem
      CHEM=0.D0
      HELP='! Chemical potential                    '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'chem',4,IN,1,0,CHEM,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Fermi Search options
      TCTRL='auto      '
      HELP='! Fermi search options                  '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'kfrm',4,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	KFRM=-1
	IF(TCTRL(1:4).EQ.'auto')KFRM=0
	IF(TCTRL(1:4).EQ.'fill')KFRM=0
	IF(TCTRL(1:4).EQ.'find')KFRM=0
	IF(TCTRL(1:4).EQ.'keep')KFRM=1
	IF(KFRM.EQ.-1)
     &CALL FULLSTOP('Keyword KFRM not recognized in QMCFILE')
C Process temperature
      TEMP=0.001D0
      HELP='! Temperature of the calculation        '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'temp',4,IN,1,0,TEMP,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	IF(ABS(TEMP).LE.1.D-10.AND.KGRD.EQ.2)
     &CALL FULLSTOP('T should not be 0 if grid=IMAG')
C **************************************
      CALL READ_ERRR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,'DMFT')
      DEALLOCATE(QMCCOPY)
	END SUBROUTINE QMC_READQMC_DMFT




	SUBROUTINE QMC_READQMC_SHEL(QMCDATA,NLEN)
*******************************************************
* Reads control                                       *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_QMCPAR  ,ONLY : MODR                                    !!! from readqmc
      USE COM_LSHELL  ,ONLY : NORB,LORB,LOCCIMP,HMAG,KSYS,CSTA,
     &                        HORB,ALSC,ELEV,FSLA,NFILSHL,NFLUSHL,    !!! from readqmc
     &                        DAMP,NRUNSHL,KSTSHL
      USE COM_IMPGRF  ,ONLY : CHEM,FILL,KINT,KRAT,NMOM,NPOL,NRAT,     !!! from readqmc
     &                        NPLD,KDIN,KDRT
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from readqmc
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*150, ALLOCATABLE :: QMCCOPY(:)
      INTEGER KST(14)
	REAL*8 ELOC(14)
      CHARACTER*150, QMCDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
     &   ,CTHUB*2,CLHUB*1,TORB*2
      LOGICAL DEF,ASK,VAL,NTH,MDF
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS FOR MAIN ARRAYS
C None
	DO L=0,3
	FSLA(L)=0.D0
	ENDDO
	DO IDEG=1,14
	ELOC(IDEG)=0.D0
	ENDDO
C **************************************
C READING CONTROLS
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=shel',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(QMCDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)QMCDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! SHELL DATA                            '
      IF(MSEC.NE.0)QMCDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(QMCCOPY(NSEC))
      DO ILIN=1,MSEC
      QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
      ENDDO
C Process cState (cannot be blank)
      TCTRL='El@0::1s  '
      HELP='! Correlated State pointer              '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'cstate',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CSTA=TCTRL
	CALL GET_STATE(TCTRL,10,IAORB,ISORB,MQ,LORB,M,1)
	IF(LORB.LT.0.OR.LORB.GT.3)
     &CALL FULLSTOP('Wrong correlated state pointer') 
	NORB=MQ
C Process Fill
      NFILSHL=0
      HELP='! Shell Mean Filling                    '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nfill',5,NFILSHL,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Fluc
      NFLUSHL=0
      HELP='! Fluctuations around mean filling      '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nfluc',5,NFLUSHL,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Hmag
      HMAG=0.D0
      HELP='! External magnetic spin field          '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'hmag',4,IN,1,0,HMAG,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Horb
      HORB=0.D0
      HELP='! External magnetic orbtial field       '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'horb',4,IN,1,0,HORB,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Alsc
      ALSC=0.D0
      HELP='! Spin orbit coupling strength          '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'alsc',4,IN,1,0,ALSC,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Damp
      DAMP=0.D0
      HELP='! Damping parameter                     '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'damp',4,IN,1,0,DAMP,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process coordinate system for local levels
      TCTRL='spherical '
      HELP='! system: cubic/spherical/spinor        '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'crdsys',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KSYS=-1
	IF(TCTRL(:NB-1).EQ.'cubic'    )KSYS=1
	IF(TCTRL(:NB-1).EQ.'spherical')KSYS=2
	IF(TCTRL(:NB-1).EQ.'spinor'   )KSYS=3
C	IF(TCTRL(:NB-1).EQ.'diagonal' )KSYS=4
C	IF(TCTRL(:NB-1).EQ.'custom'   )KSYS=5
	IF(KSYS.EQ.-1)
     &CALL FULLSTOP('Coordinate system not recognized')
C Process Eloc
	IF(LORB.LE.1)HELP='! Local level energies                  '
	IF(LORB.GT.1)HELP='                                        '
      LHLP=40
      DO I=1,14
      ELOC(I)=0.D0
      ENDDO
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'eloc(:)',7,IN,1,0,ELOC,2*(2*LORB+1),1,CM,1,0,' ',1,0,
     &HELP,LHLP,DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C **************************************
C Process # of runs for impurity solver
	NRUN=1
      HELP='! # of runs for impurity solver         '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'imprun',6,NRUN,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	NRUNSHL=NRUN
C **************************************
C For each run process states which participate
	NDIM=2*(2*LORB+1)
	DO IRUN=1,NRUNSHL          !!! ovr runs
	DO IDIM=1,NDIM
	KST(IDIM)=IDIM
	ENDDO
      HELP='! states pointer in each run            '
	CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'kstates(:)',10,KST,NDIM,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
	DO IDIM=1,NDIM
	KSTSHL(IDIM,IRUN)=KST(IDIM)
	ENDDO
C Count degeneracies
	NDEG=0
	DO IDIM=1,NDIM
	IF(KSTSHL(IDIM,IRUN).NE.0)NDEG=NDEG+1
	ENDDO
	IF(MOD(NDEG,2).NE.0)
     &CALL FULLSTOP('Wrong kstates(:) pointer')
	ENDDO                          !!! over runs
C Process Rational Interpolation Method
      TCTRL='mom4      '
      HELP='! Rational interpolation method         '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'sigint',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KINT=-1
	IF(TCTRL(:NB-1).EQ.'none')KINT=0
	IF(TCTRL(:NB-1).EQ.'mom2')KINT=1
	IF(TCTRL(:NB-1).EQ.'mom4')KINT=2
	IF(TCTRL(:NB-1).EQ.'mom6')KINT=3
	IF(TCTRL(:NB-1).EQ.'momn')KINT=4
	IF(KINT.EQ.-1)
     &CALL FULLSTOP('<QMC_READQMC>Wrong <<kint>> keyword')
C Process NPOL for impurity self-energy
      IF(KINT.EQ.0)NPOL=0
      IF(KINT.EQ.1)NPOL=1
      IF(KINT.EQ.2)NPOL=3
      IF(KINT.EQ.3)NPOL=5
      IF(KINT.EQ.4)NPOL=5
      HELP='! # of poles in Sigma(w) fit           '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'npol',4,NPOL,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
	IF(MOD(NPOL+1,2).NE.0.AND.NPOL.GT.0)
     &CALL FULLSTOP('<GETGRF>:: Npol must be Even')
      IF(KINT.EQ.0)NRAT=0
      IF(KINT.EQ.1)NRAT=2
      IF(KINT.EQ.2)NRAT=4
      IF(KINT.EQ.3)NRAT=6
      IF(KINT.EQ.4)NRAT=NPOL+1
      IF(NRAT.NE.NPOL+1.AND.NPOL.GT.0)
     &CALL FULLSTOP('<QMC_READQMC>::NRAT.NE.NPOL+1')
	IF(MOD(NRAT,2).NE.0)
     &CALL FULLSTOP('<QMC_READQMC>:: Nrat must be ODD')
	NMOM=MAX(NRAT,5)
C Force rational interpolation for Self-Energy
      TCTRL='sigmaw    '
      HELP='! Force rational interpolation for Sigma'
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'sigrat',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KRAT=-1
	IF(TCTRL(:NB-1).EQ.'sigmaw')KRAT=0
	IF(TCTRL(:NB-1).EQ.'ratint')KRAT=1
	IF(KRAT.EQ.-1)
     &CALL FULLSTOP('<QMC_READQMC>:Wrong <<krat>> keyword in QMCFILE')
C **************************************
C Process Delta Interpolation method
      TCTRL='none      '
      HELP='! Delta Interpolation Method            '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'delint',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDIN=-1
	IF(TCTRL(:NB-1).EQ.'none')KDIN=0
	IF(TCTRL(:NB-1).EQ.'mom2')KDIN=1
	IF(TCTRL(:NB-1).EQ.'mom4')KDIN=2
	IF(TCTRL(:NB-1).EQ.'mom6')KDIN=3
	IF(TCTRL(:NB-1).EQ.'momn')KDIN=4
	IF(TCTRL(:NB-1).EQ.'grid')KDIN=5
	IF(TCTRL(:NB-1).EQ.'conj')KDIN=6
	IF(TCTRL(:NB-1).EQ.'pade')KDIN=7
	IF(KDIN.EQ.-1)
     &CALL FULLSTOP('<HUB_READHUB>::Wrong <<delint>> keyword')
	IF(KDIN.EQ.1.OR.KDIN.EQ.2.OR.KDIN.EQ.3)WRITE(IUT,*)
     &'<RAT_READRAT>:: Reverse moments for metallic Delta(w) diverge!'
C **************************************
C Process Delta Representation
      TCTRL='deltaw    '
      HELP='! Delta Represenation: DeltaW/RatInt    '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'delrat',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDRT=-1
	IF(TCTRL(:NB-1).EQ.'deltaw')KDRT=0
	IF(TCTRL(:NB-1).EQ.'ratint')KDRT=1
C Process NPLD
      NPLD=0
      IF(KDIN.EQ.1)NPLD=2
      IF(KDIN.EQ.2)NPLD=4
      IF(KDIN.EQ.3)NPLD=6
      HELP='! # of poles for hybridization          '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'npld',4,NPLD,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C **************************************
C Process Slater Integrals, F(0,ll),F(2,ll),F(4,ll), etc.
	DO L=0,LORB
	FSLT=0.D0
      HELP='! Slater integrals                      '
      LHLP=40
	WRITE(CLHUB,FMT='(I1)')2*L
	CTHUB='f'//CLHUB
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &CTHUB,2,IN,1,0,FSLT,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	FSLA(L)=FSLT
	ENDDO
C **************************************
      CALL READ_ERRR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,'SHEL')
      DEALLOCATE(QMCCOPY)
	L=LORB
	NDIM=2*(2*LORB+1)
C Set up occupancies to be calculated
      NDIM=2*(2*LORB+1)
      IF(.NOT.ALLOCATED(LOCCIMP))
     &ALLOCATE(LOCCIMP(0:NDIM))
      DO IOCC=0,NDIM
      LOCCIMP(IOCC)=0
      ENDDO
      DO IOCC=0,NDIM
      IF(IOCC.GE.NFILSHL-NFLUSHL.AND.IOCC.LE.NFILSHL+NFLUSHL)
     &LOCCIMP(IOCC)=1
      ENDDO
C Matrix of local levels
	IF(.NOT.ALLOCATED(ELEV))
     &ALLOCATE(ELEV(NDIM,NDIM))
	DO IDIM1=1,NDIM
	DO IDIM=1,NDIM
	ELEV(IDIM1,IDIM)=(0.D0,0.D0)
	ENDDO
	ENDDO
	DO IDIM=1,NDIM
	ELEV(IDIM,IDIM)=DCMPLX(ELOC(IDIM),0.D0)
	ENDDO
C Transform it to spherical harmonics	
      CALL LIB_TRANSCUB(NDIM,-KSYS,NDIM,-KSYS,ELEV)
C Memory deallocations
C none
	END SUBROUTINE QMC_READQMC_SHEL







	SUBROUTINE QMC_READQMC_SETS(QMCDATA,NLEN)
*******************************************************
* Reads control                                       *
*******************************************************
C SET EXTERNAL CONNECTIONS
      USE COM_QMCPAR  ,ONLY : MODR,NTAU,NSWP,NWUP,NSMP,BETA     !!! from readqmc
     &                       ,MOMT,OMGT,EPSDEG,EPSZIP
      USE COM_MSBMESH ,ONLY : TEMP,OBND
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: QMCCOPY(:)
      CHARACTER*150, QMCDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      LOGICAL DEF,ASK,VAL,NTH,MDF
      DATA PI/3.141592653589793D0/
C **************************************
C READING CONTROLS
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sets',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(QMCDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)QMCDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! QMC SETTINGS:                         '
      IF(MSEC.NE.0)QMCDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(QMCCOPY(NSEC))
      DO ILIN=1,MSEC
      QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
      ENDDO
C **************************************
C READING THE DATA: SELECT NTAU
      NTAU=8
      HELP='! # of time slices                      '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'ntau',4,NTAU,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C ******************************************
C READING THE DATA: SELECT NWUP
      NWUP=5000
      HELP='! # of QMC sweeps for warm-up           '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nwup',4,NWUP,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C ******************************************
C READING THE DATA: SELECT NSMP
      NSWP=10000
      HELP='! # of QMC sweeps for samplings         '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nswp',4,NSWP,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C ******************************************
C READING THE DATA: SELECT NSWP
      NSMP=1
      HELP='! # of QMC sweeps per sampling          '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'nsmp',4,NSMP,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C ******************************************
C READING THE DATA: SELECT MOMT
      MOMT=-1
      HELP='! # of moments to fit self-energy tail  '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'momt',4,MOMT,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      IF(MOD(MOMT,2).NE.1.AND.MOMT.GE.0)
     &CALL FULLSTOP('<QMC_QMCYIF>:: MomT must be ODD')
C ******************************************
C READING THE DATA: SELECT MOMT
      OMGT=10.D0*OBND
      HELP='! frequency to start tail moment fit    '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'omgt',4,IN,1,0,OMGT,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C SET INVERSE TEMPERATURE
      BETA=1.D0/TEMP
      IF(ABS(DBLE(NINT(BETA))-BETA).GT.1.D-15)
     &WRITE(IUN,*)'<QMC_READQMC> :: Inverse temperature not interger'
C Process Degeneracy Threshold
      EPSDEG=1.D-06
      HELP='! Degeneracy threshold                  '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'epsdeg',6,IN,1,0,EPSDEG,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Storage Threshold
      EPSZIP=1.D-06
      HELP='! Storage Threshold                     '
      LHLP=40
      CALL READ_CHAR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,
     &'epszip',6,IN,1,0,EPSZIP,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C ************************************************************
      CALL READ_ERRR(QMCDATA(ISEC),QMCCOPY(1),NSEC,MSEC,'CTRL')
      DEALLOCATE(QMCCOPY)
C Memory deallocations
C     none
	END SUBROUTINE QMC_READQMC_SETS


	SUBROUTINE QMC_WRITEQMC(KW)
*******************************************************
*  Updates QMCFILE.                                   * 
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_QMCPAR  ,ONLY : MODR
      USE COM_QMCOUT  ,ONLY : ICQMC,QMCFILE,ICQMS,QMSFILE     !!! from readqmc
      USE COM_CHANNELS,ONLY : IUINP                           !!! from setup
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*150, ALLOCATABLE :: QMCDATA(:)
      CHARACTER*150, ALLOCATABLE :: QMCCOPY(:)
      LOGICAL EXIST,DEF,ASK,VAL,NTH,MDF
      CHARACTER STRING*150,FILE*130,HELP*40,TCTRL*10
     &   ,TLEV(8,0:4,8)*19,THED*19
      DATA PI/3.141592653589793D0/
C RETURN KEYS
	IF(ICQMC.EQ.0.AND.KW.EQ.1)RETURN
	IF(ICQMS.EQ.0.AND.KW.EQ.2)RETURN
C ********************************
C UNDERSTAND THE LENGHT OF HUBFILE
      CALL FILE_LENGHT(QMCFILE,NLEN,IUINP)
      ALLOCATE(QMCDATA(NLEN))
C ********************************
C READ QMCFILE TO INTERNAL FILE
      OPEN(UNIT=IUINP,FILE=QMCFILE,STATUS='OLD',FORM='FORMATTED')
      DO ILEN=1,NLEN
      READ(IUINP,FMT='(A150)')QMCDATA(ILEN)
      ENDDO
      CLOSE(IUINP)
C OPEN BLANK FILE
	IF(KW.EQ.1)FILE=QMCFILE
	IF(KW.EQ.2)FILE=QMSFILE
      INQUIRE(FILE=FILE,EXIST=EXIST)
      IF(EXIST)THEN
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='OLD',FORM='FORMATTED')
	CLOSE(UNIT=IUINP,STATUS='DELETE')
	ENDIF
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='NEW',FORM='FORMATTED')
C START STORAGE
	WRITE(IUINP,10100)
	WRITE(IUINP,10110)
10100	FORMAT(1X,'<FILE=QMCFILE,INPUT=MODERN,TRACE=TRUE>')
10110 FORMAT('****************************************************')
10120	FORMAT(150(A1))
C PRINT CONTROL PARAMETERS
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=ctrl',13,'<section=',9,'ok',2)
	ALLOCATE(QMCCOPY(NSEC))
	DO ILIN=1,MSEC
	QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=QMCDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(QMCCOPY)
C PRINT CTRL DATA
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=ctrl',13,'<section=',9,'ok',2)
	ALLOCATE(QMCCOPY(NSEC))
	DO ILIN=1,MSEC
	QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=QMCDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(QMCCOPY)
C PRINT DMFT DATA
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dmft',13,'<section=',9,'ok',2)
	ALLOCATE(QMCCOPY(NSEC))
	DO ILIN=1,MSEC
	QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=QMCDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(QMCCOPY)
C PRINT SHEL DATA
      CALL READ_SECT(QMCDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=shel',13,'<section=',9,'ok',2)
	ALLOCATE(QMCCOPY(NSEC))
	DO ILIN=1,MSEC
	QMCCOPY(ILIN)=QMCDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=QMCDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(QMCCOPY)
C Done
	CLOSE(IUINP)
C REFORMAT FILE
	DEALLOCATE(QMCDATA)
      CALL FILE_LENGHT(FILE,NLEN,IUINP)
      ALLOCATE(QMCDATA(NLEN))
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='OLD',FORM='FORMATTED')
      DO ILEN=1,NLEN
      READ(IUINP,FMT='(A150)')QMCDATA(ILEN)
      ENDDO
      CLOSE(IUINP)
      CALL READ_OVWR(FILE,QMCDATA,NLEN)
C MEMORY DEALLOCATIONS
	DEALLOCATE(QMCDATA)
	END
