        subroutine qmc_couple(ndeg,nts,beta,u,IC,xLambdaC)
        implicit none

        integer ndeg,nts,IC(ndeg,ndeg)
        real*8 u(ndeg,ndeg),xLambdaC(ndeg,ndeg),beta

        integer ict,i,j
        real*8 temp,dt

        dt=beta/dble(nts)

        IC(:,:)=0.d0
        ict = 0
        do i=1,ndeg
            do j=i+1,ndeg
                ict = ict +1
                IC(i,j) = ict
                IC(j,i) = ict
            end do
        end do

        xLambdaC(:,:)=0.d0
        do i=1,ndeg
            do j=i+1,ndeg
                temp = dexp(u(i,j)*dt/2.d0)
                xLambdaC(i,j) = dlog(temp+dsqrt(temp**2-1.d0))
                xLambdaC(j,i) = dlog(temp+dsqrt(temp**2-1.d0))
            end do
        end do

        end subroutine qmc_couple

        subroutine qmc_sampling(nts,ndeg,ndnt,nwu,nmc,nsweep,dseed,beta,u,gt0,sgn,gt,dens,nn)
        implicit none

        integer nts,ndeg,ndnt,nwu,nmc,nsweep,dseed,IC(ndeg,ndeg)
        real*8 beta,xLambdaC(ndeg,ndeg),gt0(ndeg*nts,ndeg*nts),gt(ndeg*nts,ndeg*nts)
        real*8 sgn,nn(ndeg,ndeg),u(ndeg,ndeg),dens(ndeg,ndeg)

        integer i,iM,imc,nisC,nconfC,im1,im2
        real*8 dt,qmc_ran2,x,rges
        
        integer, allocatable :: iranC(:)
        real*8, allocatable :: edv(:,:,:),edv1(:,:,:),gg(:,:)

        allocate(iranC(nts))
        allocate(edv(ndeg,ndeg,nts),edv1(ndeg,ndeg,nts),gg(ndnt,ndnt))

        nisC=ndeg*(ndeg-1)/2
        nconfC=2**nisC
        dt=beta/dble(nts)

        call qmc_couple(ndeg,nts,beta,u,IC,xLambdaC)

        !generate initial configuration

        do i=1,nts
            iranC(i) = int(nconfC*qmc_ran2(dseed))
        end do 

        call qmc_reconfig(nts,ndeg,ndnt,nisC,dt,xLambdaC,IC,sgn,iranC,gt,gt0)

        do i=1,nts
            call qmc_dvnm(ndeg,nisC,dt,xLambdaC,IC,iranC(i),edv(1,1,i),edv1(1,1,i))
        end do

        call qmc_heatbath(nts,ndeg,ndnt,dseed,nwu,IC,iranC,dt,sgn,xLambdaC,edv1,gt)

        rges=0.d0
        dens(:,:)=0.d0
        gg(:,:)=0.d0
        nn(:,:)=0.d0
        do imc=1,nmc

            call qmc_heatbath(nts,ndeg,ndnt,dseed,nsweep,IC,iranC,dt,sgn,xLambdaC,edv1,gt)

            rges=rges+sgn

            gg(:,:)=gg(:,:)+gt(:,:)*sgn

            do iM1=1,ndeg
                do iM2=1,ndeg
                    x=0.d0  
                    do i=1,nts
                        x = x-gt(ndeg*(i-1)+iM2,ndeg*(i-1)+iM1)
                        if(iM1.eq.iM2) x=x+1.d0
                    end do
                    dens(iM1,iM2) = dens(iM1,iM2)+(x/nts)*sgn
                 end do
            end do

            do iM1=1,ndeg
                do iM2=1,ndeg
                     x=0.d0
                     do i=1,nts
                        x=x+(1.d0-gt(ndeg*(i-1)+iM1,ndeg*(i-1)+iM1))*(1.d0-gt(ndeg*(i-1)+iM2,ndeg*(i-1)+iM2))
                     end do 
                     nn(iM1,iM2)=nn(iM1,iM2)+x/nts*sgn
                end do
            end do  

        end do

        gt(:,:)=gg(:,:)/rges
        dens(:,:)=dens(:,:)/rges
        nn(:,:)=nn(:,:)/rges

        sgn=rges/nmc

        deallocate(iranC,edv,edv1,gg)

        end subroutine qmc_sampling

        subroutine qmc_heatbath(nts,ndeg,ndnt,idummy,nsweep,IC,iranC,del_tau,sgn,xLambdaC,edv1,gt) 
        implicit none

        integer nts,ndeg,ndnt,nsweep,idummy
        integer IC(ndeg,ndeg),iranC(nts)
        real*8 del_tau,sgn,edv1(ndeg,ndeg,nts),xLambdaC(ndeg,ndeg),gt(ndnt,ndnt)

        integer nisC,nconfC
        integer	ls,loop,m,n,i,j,l,ns,l1,l2,i2C
        real*8 qmc_ran2,det,R

        real*8, allocatable :: dv1(:,:),dv(:,:),tmat(:,:),vmat(:,:)

        allocate(dv1(ndeg,ndeg),dv(ndeg,ndeg),tmat(ndeg,ndeg),vmat(ndeg,ndeg))
        
        nisC=ndeg*(ndeg-1)/2
        nconfC=2**nisC
        do loop=1,nsweep
            do 100 ls=1,nts

	            i2C = int(nconfC*qmc_ran2(Idummy))
	            call qmc_dvnm(ndeg,nisC,del_tau,xLambdaC,IC,i2C,dv,dv1)                         
	            call qmc_get_det(ls,ndeg,ndnt,dv,edv1(1,1,ls),gt,vmat,tmat,det)
	            R = det
	            if( qmc_ran2(idummy).gt.abs(R) ) goto 100
	            call qmc_update(ndeg,ndnt,nts,ls,tmat,vmat,det,gt)
	            iranC(ls) = i2C
	            edv1(1:ndeg,1:ndeg,ls)=dv1(1:ndeg,1:ndeg)
        !	    call dcopy(ndeg*ndeg,dv1,1,edv1(1,1,ls),1)
	            if ( R.lt.0.d0 ) then
	               sgn = -sgn
	            end if
        	    
        100	  continue

        end do
        
        deallocate(dv1,dv,tmat,vmat)
        
        return
        end


        subroutine qmc_dvnm(ndeg,nisC,del_tau,xLambdaC,IC,irC,dv,dv1)
        implicit none

        integer ndeg,nisC,IC(ndeg,ndeg),irC
        real*8 del_tau,xLambdaC(ndeg,ndeg),dv(ndeg,ndeg),dv1(ndeg,ndeg)
                                              
        integer	i,j,k,l,info
        
        integer, allocatable :: isc(:)
        real*8, allocatable :: dd(:)
        	
        allocate(isc(0:nisC),dd(ndeg))
        	
        	
        call qmc_decimal_to_binary(irC,nisC,isC)

        dv(:,:)=0.d0
        dv1(:,:)=0.d0
        dd(:)=0.d0
        do i=1,ndeg
            do j=i+1,ndeg
	            dd(i)=dd(i)+xLambdaC(i,j)*isC(IC(i,j))
	            dd(j)=dd(j)-xLambdaC(i,j)*isC(IC(i,j))
            end do
        dv(i,i) = dexp(dd(i))
        dv1(i,i)=dexp(-dd(i)) 
        end do    

        deallocate(isc,dd)
        
        end subroutine qmc_dvnm

           
        subroutine qmc_reconfig(nts,ndeg,ndnt,nisC,del_tau,xLambdaC,IC,sgn,iranC,gt,gt0)
        implicit none

        Integer nts,ndeg,ndnt,nisC
        integer IC(ndeg,ndeg),iranC(nts)
        real*8 sgn,del_tau,xLambdaC(ndeg,ndeg),gt(ndnt,ndnt),gt0(ndnt,ndnt)

        integer ls
        real*8 det,arg

        real*8, allocatable :: dv(:,:),dv1(:,:),aI(:,:)
        real*8, allocatable :: tmat(:,:),vmat(:,:)

        allocate(dv(ndeg,ndeg),dv1(ndeg,ndeg),aI(ndeg,ndeg))
        allocate(tmat(ndeg,ndeg),vmat(ndeg,ndeg))

        gt(:,:)=gt0(:,:)
        aI(:,:)=0.d0
        do ls=1,ndeg
            aI(ls,ls)=1.d0
        end do 

        !*
        !*  'potmat = exp(V')-I
        !*
        sgn = 1.d0
        do ls=1,nts
            call qmc_dvnm(ndeg,nisC,del_tau,xLambdaC,IC,iranC(ls),dv,dv1)
	        call qmc_get_det(ls,ndeg,ndnt,dv,aI,gt,vmat,tmat,det)
	        if (det.lt.0.d0) sgn = -sgn
	        call qmc_update(ndeg,ndnt,nts,ls,tmat,vmat,det,gt)
        end do

        deallocate(dv,dv1,aI,tmat,vmat)

        end subroutine qmc_reconfig

        subroutine qmc_get_det(ls,nd,ndnt,dv,edv,g,vmat,tmat,det)
        !*
        !*  'tmat = I-(g-I)*{exp(-V)exp(V')-I}
        !*
        implicit none
        integer nd,ndnt,n,m,ls
        real*8 dv(nd,nd),edv(nd,nd),g(ndnt,ndnt),vmat(nd,nd)
        real*8 tmat(nd,nd),det
        
        real*8, allocatable :: aI(:,:),temp(:,:)
        
        allocate(aI(nd,nd),temp(nd,nd))
        
        call lib_dgemm('N','N',nd,nd,nd,1.d0,edv,nd,dv,nd,0.d0,vmat,nd)

        do n=1,nd
            vmat(n,n) = vmat(n,n)-1.d0
        end do

        do n=1,nd
            do m=1,nd
                temp(m,n) = g(nd*(ls-1)+m,nd*(ls-1)+n)
	        end do
	        temp(n,n)=temp(n,n)-1.d0
        end do

        call lib_dgemm('N','N',nd,nd,nd,1.d0,temp,nd,vmat,nd,0.d0,tmat,nd)

        tmat(:,:)=-tmat(:,:)
        do n=1,nd
            tmat(n,n) = 1.d0+tmat(n,n)
        end do

        call LIB_DET(tmat,nd,nd,det)
        !call determinant(tmat,nd,nd,det)

        deallocate(aI,temp)

        end subroutine qmc_get_det


        subroutine qmc_update(nd,ndnt,nts,ls,tmat,vmat,det,g)
        !*
        !*	UPDATE G :
        !*
        !*'	g' = g + (g-I)*(exp(V'-V)-I)*[I-(g-I)(exp(V'-V)-I)]^-1*g
        !*
        !*
        implicit none
        integer nd,nts,ndnt,l,ls,m,n,i,j,lsd,ipiv(nd)
        real*8 tmat(nd,nd),vmat(nd,nd),g(ndnt,ndnt),det,one
        real*8, allocatable :: t1(:,:),t2(:,:),row(:,:),col(:,:),work(:)
        
        allocate(t1(nd,nd),t2(nd,nd),row(nd,ndnt),col(ndnt,nd),work(nd))
        	
        lsd = nd*(ls-1)
        one = 1.d0

        t1(:,:)=tmat(:,:)
        !call dgetrf(nd,nd,t1,nd,ipiv,info)
        !call dgetri(nd,t1,nd,ipiv,work,3*nd,info)
        call lib_inverse(t1,t1,ipiv,work,nd)
        call lib_dgemm('N','N',nd,nd,nd,1.d0,vmat,nd,t1,nd,0.d0,t2,nd)

        row(:,:) = 0.0
        do l=1,ndnt
            do n=1,nd
	            do m=1,nd
	                row(n,l) = row(n,l) + t2(n,m)*g(lsd+m,l)
	            end do
	        end do
        end do

        do n=1,nd
            do l=1,ndnt
	            col(l,n) = g(l,lsd+n)
	        end do
        end do

        do n=1,nd
            col(lsd+n,n) = col(lsd+n,n) - one
        end do

        call lib_dgemm('N','N',ndnt,ndnt,nd,one,col,ndnt,row,nd,one,g,ndnt)

        deallocate(t1,t2,row,col,work)

        end subroutine qmc_update


        function qmc_ran2(idum)
        implicit none
        integer idum,IM1,IM2,IMM1,IA1,IA2,IQ1,IQ2,IR1,IR2,NTAB,NDIV
        real*8 qmc_ran2,AM,EPS,RNMX
        parameter (IM1=2147483563,IM2=2147483399,AM=1./IM1,IMM1=IM1-1,    &
                     IA1=40014,IA2=40692,IQ1=53668,IQ2=52774,IR1=12211,           &
                     IR2=3791,NTAB=32,NDIV=1+IMM1/NTAB,EPS=1.2e-7,RNMX=1.-EPS)
        integer idum2,j,k,iv(NTAB),iy
        save iv,iy,idum2
        data idum2/123456789/, iv/NTAB*0/, iy/0/
        	
            if (idum.le.0) then
	            idum=max(-idum,1)
	            idum2=idum
	            do j=NTAB+8,1,-1
	               k=idum/IQ1
	               idum=IA1*(idum-k*IQ1)-k*IR1
	               if (idum.lt.0) idum=idum+IM1
	               if (j.le.NTAB) iv(j)=idum
	            end do
	            iy=iv(1)
	        end if
	        k=idum/IQ1
	        idum=IA1*(idum-k*IQ1)-k*IR1
	        if (idum.lt.0) idum=idum+IM1
	        k=idum2/IQ2
	        idum2=IA2*(idum2-k*IQ2)-k*IR2
	        if (idum2.lt.0) idum2=idum2+IM2
	        j=1+iy/NDIV
	        iy=iv(j)-idum2
	        iv(j)=idum
	        if (iy.lt.1) iy=iy+IMM1
	        qmc_ran2=min(AM*iy,RNMX)
	        return

        end

        subroutine qmc_decimal_to_binary(ir,nis,is)
        !*
        !*	expand the integer ir in terms of power series
        !*	with base nbase. Integer coefficients are stored
        !*	in iarray(). (ir < nbase**npower, npower=nis here )
        !*

        implicit none
        integer ir,nis,is(0:nis),k,np,n

        k = ir
        do np=1,nis
            n = k/(2**(nis-np))
	        is(nis-np+1) = 1-2*n 
	        k = k - n*2**(nis-np)
        end do
        return

        end

