	SUBROUTINE RAT_ENERGY(IEXIT)
**********************************************************
*  Calculate total energy                                &
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
	USE COM_RATSCF ,ONLY: EPSEN                        !!! from readrat
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                  !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	SAVE ETOT0
	DATA ETOT0/0.D0/
      CALL ABORTNOW()
C START CALCULATING ENERGIES
	CALL RAT_ENERGY_ATOM(ELEV)      !!! find levels energy
C POTENTIAL PART
	CALL RAT_ENERGY_PTNL(EPTN)      !!! find potential energy
C COULOMB PART
	CALL RAT_ENERGY_COUL(ECOU)      !!! find Coulomb energy
C EXCHANGE-CORRELATION IN LDA/GGA
	CALL RAT_ENERGY_EXCH(EXCH)      !!! find exchange-correlation energy
C PRINT ENERGIES
	EKIN=ELEV-EPTN
	ETOT=EKIN+ECOU+EXCH
	WRITE(IUN,1000)
	WRITE(IUN,1020)ELEV
	WRITE(IUN,1030)EPTN
	WRITE(IUN,1040)EKIN
	WRITE(IUN,1050)ECOU
	WRITE(IUN,1060)EXCH
	WRITE(IUN,1090)ETOT
	IF(IUN.NE.IUT)
     &WRITE(IUT,1090)ETOT
	WRITE(IUN,1000)
	IF(ABS(ETOT-ETOT0).LE.EPSEN)IEXIT=1
	IF(ABS(ETOT-ETOT0).GT.EPSEN)IEXIT=0
	ETOT0=ETOT
1000  FORMAT(1X,'***************************************************')
1020  FORMAT(1X,'*       LEVELS ENERGY   =',G20.14,'     *')
1030  FORMAT(1X,'*       POTENTIAL ENERGY=',G20.14,'     *')
1040  FORMAT(1X,'*       KINETIC ENERGY  =',G20.14,'     *')
1050  FORMAT(1X,'*       COULOMB ENERGY  =',G20.14,'     *')
1060  FORMAT(1X,'*       EXCH-CORR ENERGY=',G20.14,'     *')
1090  FORMAT(1X,'*       TOTAL ENERGY    =',G20.14,'     *')
	END







	SUBROUTINE RAT_ENERGY_ATOM(ERAT)
*************************************************************
*  Calculate one-electron contribution to the total energy  *
*************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_RATLEV  ,ONLY: NLEVT,ELEV,ZLEV                  !!! from readrat
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DATA PI/3.141592653589793D0/
C  CALCULATE ONE-ELECTRON CORE CONTRIBUTION
	ERAT=0.D0
	DO ILEV=1,NLEVT
      ERAT=ERAT+ELEV(ILEV)*ZLEV(ILEV)
	ENDDO
	END


	SUBROUTINE RAT_ENERGY_PTNL(EPTN)
*****************************************************************
*  Calculate potential energy contribution to the total energy  *
*****************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
	USE COM_CONTROL ,ONLY: ISIC                             !!! from ratpot
	USE COM_RATSCF  ,ONLY: RATRHO,RATPOT                    !!! from ratpot
      USE COM_RATMESH ,ONLY: NRAT,HRAT,RRAT,DRAT,WRAT         !!! from ratmesh
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: WORK1(:)
	REAL*8, ALLOCATABLE :: WORK2(:)
	DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
C MEMORY ALLOCATIONS
	ALLOCATE(WORK1(0:NRATMEM))
	ALLOCATE(WORK2(0:NRATMEM))
C CALCULATE POTENTIAL ENERGY
	EPTN=0.D0
	DO IRAT=0,NRAT
	WORK1(IRAT)=RATRHO(IRAT)*RATPOT(IRAT)*WRAT(IRAT)
      ENDDO
	CALL LIB_QD(HRAT,WORK1,WORK2,NRAT+1)
	EPTN=EPTN+WORK2(NRAT)
C MEMORY DEALLCATIONS
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	END




	SUBROUTINE RAT_ENERGY_COUL(ECOU)
**************************************************************
*  Calculate electrostatic contribution to the total energy  *
**************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
	USE COM_CONTROL ,ONLY: ISIC                             !!! from ratpot
	USE COM_RATSCF  ,ONLY: ZRAT,RATRHO,RATPOT               !!! from ratpot
      USE COM_RATMESH ,ONLY: NRAT,HRAT,RRAT,DRAT,WRAT         !!! from ratmesh
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8, ALLOCATABLE :: WORK1(:)
      REAL*8, ALLOCATABLE :: WORK2(:)
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS
      ALLOCATE(WORK1(0:NRATMEM))
      ALLOCATE(WORK2(0:NRATMEM))
C CALCULATE COULOMB POTENTIAL WITHOUT NUCLEAR PART
	DO IRAT=0,NRAT                                      !!! over irat
	RATPOT(IRAT)=0.D0
      ENDDO                                               !!! over irat
	CALL RAT_VCOUL(2)       !!! calculate Coulomb potential whithin MTs
C CALCULATE MT-CONTRIBUTION TO THE COULOMB ENERGY
	ECOU=0.D0
	DO IRAT=0,NRAT                                       !!! over irat
	WORK1(IRAT)=RATPOT(IRAT)*RATRHO(IRAT)*WRAT(IRAT)
      ENDDO                                                !!! over irat
	CALL LIB_QD(HRAT,WORK1,WORK2,NRAT+1)
	ECOU=ECOU+0.5D0*WORK2(NRAT)
	DO IRAT=0,NRAT                                       !!! over irat
	WORK1(IRAT)=-4.D0*PI*ZRAT*RATRHO(IRAT)*RRAT(IRAT)*DRAT(IRAT)
      ENDDO                                                !!! over irat
	CALL LIB_QD(HRAT,WORK1,WORK2,NRAT+1)
	ECOU=ECOU+WORK2(NRAT)-0.5D0*ZRAT*RATPOT(0)
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	END




	SUBROUTINE RAT_ENERGY_EXCH(EXCH)
*********************************************************
*  Calculate exchange-correlation contribution to the   *
*  total energy.                                        *
*********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
	USE COM_CONTROL ,ONLY: ISIC                             !!! from ratpot
	USE COM_RATSCF  ,ONLY: RATRHO,RATPOT                    !!! from ratpot
      USE COM_RATMESH ,ONLY: NRAT,HRAT,RRAT,DRAT,WRAT         !!! from ratmesh
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8, ALLOCATABLE :: WORK1(:)
      REAL*8, ALLOCATABLE :: WORK2(:)
      DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
C MEMORY ALLOCATIONS
      ALLOCATE(WORK1(0:NRATMEM))
      ALLOCATE(WORK2(0:NRATMEM))
C CALCULATE DENSITY OF THE EXCHANGE-CORRELATION ENERGY
	DO IRAT=0,NRAT                                      !!! over irat
	RATPOT(IRAT)=0.D0
      ENDDO                                               !!! over irat
	CALL RAT_VEXCH(2)       !!! exchange-correlation within MTs
C CALCULATE MT-CONTRIBUTION TO THE EXCHANGE-CORRELATION ENERGY
	EXCH=0.D0
	DO IRAT=0,NRAT                                      !!! over irat
	WORK1(IRAT)=RATRHO(IRAT)*RATPOT(IRAT)*WRAT(IRAT)
      ENDDO                                               !!! over irat
	CALL LIB_QD(HRAT,WORK1,WORK2,NRAT+1)
	EXCH=EXCH+WORK2(NRAT)
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	END
