	SUBROUTINE TTR_MCTTRINT0(E0,EF,OT,AT,AD,TS,DS,TBLO,TRET)
************************************************************
*  Finds weights for integrating over microtetrahedron:    *
*                                                          *
*        /  3                                              *
*        \ d k  R(k)*theta(Ef-E(k))  - TOS like            *
*        /                                                 *
*       ttr                                                *
*                                                          *
*  or                                                      *
*                                                          *
*        /  3                                              *
*        \ d k  R(k)*delta(Ef-E(k))  - DOS like            *
*        /                                                 *
*       ttr                                                *
*                                                          *
*  where R(k) are 1,kx,ky,kz (for the matrix elements      *
*  evaluation). The linear interpolation is supposed.      *
*  Inputs:                                                 *
*   E0(I)  - energies E(k) at the corners.                 *
*   EF     - Fermi energy.                                 *
*   OT     - Volume of tetrahedron.                        *
*  Keys:                                                   *
*   Tblo=T - standard tetrahedron method.                  *
*   Tblo=B - with Bloechl corrections.                     *
*   Tret=W - returns weights for the corners  (i=1...4).   *
*   Tret=I - returns integrals for 1,kx,ky,kz (i=1...4).   *
*  Outputs:                                                *
*   AT(I)  - TOS weights/integrals for Tret=W/I.           *
*   AD(I)  - DOS weights/integrals for Tret=W/I.           *
*   TS     - total number of states.                       *
*   DS     - total density of states.                      *
*  Comments:                                               *
*   Option Tret=I is not curreantly avaiable.              *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION ET(4),E0(4),IND(4),L(4),AT(4),AD(4)
	CHARACTER TBLO*1,TRET*1
	DATA EPS/1.D-07/
	DO I=1,4
      IND(I)=1
	ENDDO
C ORDER E0
	DO I=1,3
	I1=I+1
	DO J=I1,4
	IF(E0(I).GT.E0(J))IND(J)=IND(J)+1
	IF(E0(I).LE.E0(J))IND(I)=IND(I)+1
	ENDDO
	ENDDO
	DO I=1,4
	JJ=IND(I)
	L(JJ)=I
      ET(JJ)=E0(I)
	ENDDO
C REMOVE DEGENERACIES
	DO I=1,3
	IF(ABS(ET(I)-ET(I+1)).LT.EPS)
     &ET(I)=ET(I)+EPS/DBLE(I)
	ENDDO
	DO I=1,4
	IF(ABS(ET(I)-EF).LT.EPS/10.D0)
     &ET(I)=ET(I)+EPS/10.D0/DBLE(I)
	ENDDO
C FIND CASE 
	IF(EF.LE.ET(4))THEN
C Tetrahedron is empty
	TS=0.D0
	DS=0.D0
	DDS=0.D0
	DO I=1,4
	AT(I)=0.D0
	AD(I)=0.D0
	ENDDO
      ELSEIF(EF.GT.ET(1))THEN
C Tetrahedron is fully occupied
	TS=OT
	DS=0.D0
	DDS=0.D0
	DO I=1,4
	AD(I)=0.D0
	AT(I)=OT/4.D0
	ENDDO
      ELSEIF(EF.LE.ET(3))THEN
C Tetrahedron is partially occupied
	EF4=EF-ET(4)
	E24=EF4/(ET(2)-ET(4))
	E14=EF4/(ET(1)-ET(4))
	E34=EF4/(ET(3)-ET(4))
	TMP=E24*E14*E34*OT
	DS=TMP*3.D0/EF4
	DDS=DS*2.D0/EF4       !!! dD(Ef)/dEf for Bloechl correction
	TS=TMP
	AD(L(1))=DS*E14/3.D0
	AD(L(2))=DS*E24/3.D0
	AD(L(3))=DS*E34/3.D0
	AD(L(4))=DS*(1.D0-(E14+E24+E34)/3.D0)
	AT(L(1))=TS*E14/4.D0
	AT(L(2))=TS*E24/4.D0
	AT(L(3))=TS*E34/4.D0
	AT(L(4))=TS*(1.D0-(E14+E24+E34)/4.D0)
	ELSEIF(EF.GT.ET(3).AND.EF.LE.ET(2))THEN
C Tetrahedron is partially occupied
C SECTION IS A QUADRANGLE
	OM23=ET(2)-ET(3)
C DIVIDING INTO TWO TETRAHEDRA
C THE FIRST TETRAHEDRON
	DOM1=EF-ET(4)
	DOM11=ET(1)-ET(4)
	DOM12=ET(2)-ET(4)
	DOM13=DOM1
	F1=1.D0/DOM11/DOM12*(ET(2)-EF)/OM23
C THE SECOND TETRAHEDRON
	DOM2=ET(1)-EF
	DOM21=DOM2
	DOM22=ET(1)-ET(3)
	DOM23=ET(1)-ET(4)
	F2=1.D0/DOM22/DOM23
	G2=(EF-ET(3))/OM23
	DS=(DOM1*F1+DOM2*F2*G2)*3.D0*OT
	DDS=(DOM2/DOM22-DOM1/DOM12)/DOM11/OM23*6.D0*OT
	TS=(DOM1**2*F1+(1.D0-DOM2**2*F2)*G2)*OT
	DF1=3.D0*OT*DOM1*F1
	TF1=OT*DOM1*F1*DOM1
	DF2=3.D0*OT*DOM2*F2*G2
	TF2=OT*DOM2*F2*G2*DOM2
	DFF=(EF-ET(3))/(ET(2)-ET(3))
	AD(L(1))=DF2*(1.D0-DOM2/3.D0*(1.D0/DOM21+1.D0/DOM22+
     &  1.D0/DOM23))+DF1*DOM1/DOM11/3.D0
	AD(L(2))=DF1*DOM1/3.D0*(1.D0/DOM12+1.D0/DOM13*DFF)+
     &  DF2*DOM2/DOM21/3.D0*DFF
	AD(L(3))=DF2*DOM2/3.D0*(1.D0/DOM21+1.D0/DOM22-
     &  1.D0/DOM21*DFF)+DF1*DOM1/DOM13/3.D0*(1.D0-DFF)
	AD(L(4))=DF1*(1.D0-DOM1/3.D0*(1.D0/DOM11+1.D0/DOM12+
     &  1.D0/DOM13))+DF2*DOM2/DOM23/3.D0
	AT(L(1))=-TF2*(1.D0-DOM2/4.D0*(1.D0/DOM21+1.D0/DOM22+
     &  1.D0/DOM23))+TF1*DOM1/DOM11/4.D0+OT*G2/4.D0
	AT(L(2))=TF1*DOM1/4.D0*(1.D0/DOM12+1.D0/DOM13*DFF)-
     &  TF2*DOM2/DOM21/4.D0*DFF+OT*G2/4.D0*DFF
	AT(L(3))=-TF2*DOM2/4.D0*(1.D0/DOM21+1.D0/DOM22-
     &  1.D0/DOM21*DFF)+TF1*DOM1/DOM13/4.D0*(1.D0-DFF)+
     &  OT*G2/4.D0*(2.D0-DFF)
	AT(L(4))=TF1*(1.D0-DOM1/4.D0*(1.D0/DOM11+1.D0/DOM12+
     &  1.D0/DOM13))-TF2*DOM2/DOM23/4.D0+OT*G2/4.D0
	ELSEIF(EF.GT.ET(2))THEN
C Tetrahedron is partially occupied
	EF1=(EF-ET(1))
	E14=EF1/(ET(1)-ET(4))
	E12=EF1/(ET(1)-ET(2))
	E13=EF1/(ET(1)-ET(3))
	TMP=E12*E13*E14*OT
C   !NB TMP<0,EF1<0
	DS=TMP*3.D0/EF1
	DDS=DS*2.D0/EF1
	TS=OT+TMP
	AD(L(1))=DS*(1.D0+(E12+E13+E14)/3.D0)
	AD(L(2))=-DS*E12/3.D0
	AD(L(3))=-DS*E13/3.D0
	AD(L(4))=-DS*E14/3.D0
	AT(L(1))=OT/4.D0+TMP*(1.D0+(E12+E13+E14)/4.D0)
	AT(L(2))=OT/4.D0-TMP*E12/4.D0
	AT(L(3))=OT/4.D0-TMP*E13/4.D0
	AT(L(4))=OT/4.D0-TMP*E14/4.D0
	ENDIF
	IF(TBLO.EQ.'T')THEN
C DO NOT ADD BLOECHL CORRECTION
	ELSEIF(TBLO.EQ.'B')THEN
C ADD BLOECHL CORRECTION
	DO I=1,4
	DO J=1,4
	AT(L(I))=AT(L(I))+(ET(J)-ET(I))*DS*0.025D0
	AD(L(I))=AD(L(I))+(ET(J)-ET(I))*DDS*0.025D0
	ENDDO
	ENDDO
	ENDIF
C SET WEIGHTS/INTEGRALS ACCORDING TO TRET
	IF(TRET.EQ.'W')THEN
C WEIGHTS ARE READY
	CONTINUE
	ELSEIF(TRET.EQ.'I')THEN
  	CALL FULLSTOP('INTEGRALS NOT AVAIABLE')
	ENDIF
	RETURN
	END




	SUBROUTINE TTR_MCTTRINTG(E0,EF,DE,OT,AT,AD,TS,DS)
************************************************************
*  Returns weights associated with gaussian broadening     *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION E0(4),AT(4),AD(4)
      DATA PI/3.141592653589793D0/
	DO I=1,4
	XX=(E0(I)-EF)/DE
      AT(I)=OT/4.D0*0.5D0*(1.D0-ERF(XX))
      AD(I)=OT/4.D0*EXP(-XX**2)/SQRT(PI)/DE
	ENDDO
	TS=AT(1)+AT(2)+AT(3)+AT(4)
	DS=AD(1)+AD(2)+AD(3)+AD(4)
	RETURN
	END





	SUBROUTINE TTR_MCTTRINT00(E0,EF,DE,OT,AT,AD,TS,DS)
************************************************************
*  Returns weights associated for DOS only, formulae only. *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION ET(4),E0(4),IND(4),L(4),AT(4),AD(4)
     &    ,BT(4),BD(4)
	DATA EPS/1.D-07/
	DO I=1,4
      IND(I)=1
	ENDDO
C ORDER E0
	DO I=1,3
	I1=I+1
	DO J=I1,4
	IF(E0(I).GT.E0(J))IND(J)=IND(J)+1
	IF(E0(I).LE.E0(J))IND(I)=IND(I)+1
	ENDDO
	ENDDO
	DO I=1,4
	JJ=IND(I)
	L(JJ)=I
      ET(JJ)=E0(I)
	ENDDO
C REMOVE DEGENERACIES
	DO I=1,3
	IF(ABS(ET(I)-ET(I+1)).LT.EPS)
     &ET(I)=ET(I)+EPS/DBLE(I)
	ENDDO
	DO I=1,4
	IF(ABS(ET(I)-EF).LT.EPS/10.D0)
     &ET(I)=ET(I)+EPS/10.D0/DBLE(I)
	ENDDO
C FIND CASE 
	IF(EF.LE.ET(4))THEN
C Tetrahedron is empty
	DS=0.D0
	DO I=1,4
	AD(I)=0.D0
	ENDDO
      ELSEIF(EF.GT.ET(1))THEN
C Tetrahedron is fully occupied
	DS=0.D0
	DO I=1,4
	AD(I)=0.D0
	ENDDO
      ELSEIF(EF.LE.ET(3))THEN
C Tetrahedron is partially occupied
	EF4=EF-ET(4)
	E14=ET(1)-ET(4)
	E24=ET(2)-ET(4)
	E34=ET(3)-ET(4)
	DS=EF4*EF4/(E14*E24*E34)*3.D0
	AD(L(1))=EF4*EF4*EF4/(E14*E24*E34*E14)
	AD(L(2))=EF4*EF4*EF4/(E14*E24*E34*E24)
	AD(L(3))=EF4*EF4*EF4/(E14*E24*E34*E34)
	AD(L(4))=EF4*EF4/(E14*E24*E34)*3.D0
     &        -EF4*EF4*EF4/(E14*E24*E34*E14)
     &        -EF4*EF4*EF4/(E14*E24*E34*E24)
     &        -EF4*EF4*EF4/(E14*E24*E34*E34)
	ELSEIF(EF.GT.ET(3).AND.EF.LE.ET(2))THEN
C Tetrahedron is partially occupied
	EF1=EF-ET(1)
	EF2=EF-ET(2)
	EF3=EF-ET(3)
	EF4=EF-ET(4)
	E14=ET(1)-ET(4)
	E24=ET(2)-ET(4)
	E13=ET(1)-ET(3)
	E23=ET(2)-ET(3)
	DS=-EF4*EF2*3.D0/(E14*E24*E23)-EF1*EF3*3.D0/(E13*E14*E23)
	AD(L(1))=-EF1*EF3*2.D0/(E13*E14*E23)
     &         -EF1*EF1*EF3/(E13*E14*E23*E13) !c3
     &         -EF1*EF1*EF3/(E13*E14*E23*E14) !c1
     &         -EF2*EF4*EF4/(E14*E24*E23*E14) !c2
	AD(L(2))=-EF2*EF4*EF4/(E14*E24*E23*E24) !c4
     &         -EF2*EF3*EF4/(E14*E24*E23*E23) !c6
     &         -EF1*EF3*EF3/(E13*E14*E23*E23) !c5
	AD(L(3))=-EF1*EF3/(E13*E14*E23)
     &         -EF2*EF4/(E14*E24*E23)
     &         +EF1*EF1*EF3/(E13*E14*E23*E13) !c3
     &         +EF1*EF3*EF3/(E13*E14*E23*E23) !c5
     &         +EF2*EF3*EF4/(E14*E24*E23*E23) !c6
	AD(L(4))=-EF2*EF4*2.D0/(E14*E24*E23)
     &         +EF2*EF4*EF4/(E14*E24*E23*E14) !c2
     &         +EF2*EF4*EF4/(E14*E24*E23*E24) !c4
     &         +EF1*EF1*EF3/(E13*E14*E23*E14) !c1
	ELSEIF(EF.GT.ET(2))THEN
C Tetrahedron is partially occupied
	EF1=EF-ET(1)
	E12=ET(1)-ET(2)
	E13=ET(1)-ET(3)
	E14=ET(1)-ET(4)
	DS=EF1*EF1/(E12*E13*E14)*3.D0
	AD(L(1))=EF1*EF1/(E12*E13*E14)*3.D0
     &        +EF1*EF1*EF1/(E12*E13*E14*E12)
     &        +EF1*EF1*EF1/(E12*E13*E14*E13)
     &        +EF1*EF1*EF1/(E12*E13*E14*E14)
	AD(L(2))=-EF1*EF1*EF1/(E12*E13*E14*E12)
	AD(L(3))=-EF1*EF1*EF1/(E12*E13*E14*E13)
	AD(L(4))=-EF1*EF1*EF1/(E12*E13*E14*E14)
	ENDIF
	DS=DS*OT
	DO I=1,4
	AD(I)=AD(I)*OT
	ENDDO
	RETURN
	END



	SUBROUTINE TTR_MCTTRINTF(E0,EF,DE,FL,OT,AT,AD,TS,DS)
************************************************************
*  Returns weights associated with temperature dependent   *
*  Fermi distribution. DE is reciprocal temperature in     *
*  energy units. Weights are defined without T=0 term.     *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION ET(4),XT(4),E0(4),IND(4),L(4),AT(4),AD(4)
     &         ,FL(0:4,4),GL(0:4,4),GD(0:4,4)
	DATA EPS/1.D-05/
	DO I=1,4
      IND(I)=1
	ENDDO
C Order E0
	DO I=1,3
	I1=I+1
	DO J=I1,4
	IF(E0(I).GT.E0(J))IND(J)=IND(J)+1
	IF(E0(I).LE.E0(J))IND(I)=IND(I)+1
	ENDDO
	ENDDO
	DO I=1,4
	JJ=IND(I)
	L(JJ)=I
      ET(JJ)=E0(I)
	DO LD=0,4
	GL(LD,JJ)=FL(LD,I)
	ENDDO
	ENDDO
C Do NOT remove degeneracies here!
C Prepare X values
	DO I=1,4
	XT(I)=(ET(I)-EF)/DE
	GD(0,I)=0.D0
	DO LD=1,4
	GD(LD,I)=GL(0,I)*XT(I)**(LD-1)
	ENDDO
	ENDDO
	X12=XT(1)-XT(2)
	X13=XT(1)-XT(3)
	X14=XT(1)-XT(4)
	X23=XT(2)-XT(3)
	X24=XT(2)-XT(4)
	X34=XT(3)-XT(4)
C Zeroize
	TS=0.D0
	DS=0.D0
	DO I=1,4
	AT(I)=0.D0
	AD(I)=0.D0
	ENDDO
C Integrate
C Complete degeneracy case
	IF(ABS(ET(4)-ET(3)).LE.EPS.AND.
     &   ABS(ET(3)-ET(2)).LE.EPS.AND.
     &   ABS(ET(2)-ET(1)).LE.EPS)THEN
	IF(XT(1).LE.0.D0)THEN
	YT=EXP(+XT(1))
	TS=1.D0/(YT+1.D0)-1.D0 !!! -1 removes theta(x) !!!
	DS=YT/(YT+1.D0)**2
	ELSEIF(XT(1).GT.0.D0)THEN
	YT=EXP(-XT(1))
	TS=1.D0-1.D0/(YT+1.D0)
	DS=YT/(YT+1.D0)**2
	ENDIF
	AT(L(1))=TS/4.D0
	AT(L(2))=TS/4.D0
	AT(L(3))=TS/4.D0
	AT(L(4))=TS/4.D0
	AD(L(1))=DS/4.D0
	AD(L(2))=DS/4.D0
	AD(L(3))=DS/4.D0
	AD(L(4))=DS/4.D0
	ENDIF
C 1. From -inf to XT(4)
	! nothing
C 2. From XT(4) to XT(3)
	IF(ABS(ET(3)-ET(4)).GT.EPS)THEN
	XF440INT= (GL(3,3)-GL(3,4))
     &         -(GL(2,3)-GL(2,4))*(XT(4)+XT(4))
     &         +(GL(1,3)-GL(1,4))*(XT(4)*XT(4))
	XF444INT= (GL(4,3)-GL(4,4))
     &         -(GL(3,3)-GL(3,4))*(XT(4)+XT(4)+XT(4))
     &         +(GL(2,3)-GL(2,4))*(XT(4)*XT(4)+XT(4)*XT(4)+XT(4)*XT(4))
     &         -(GL(1,3)-GL(1,4))*(XT(4)*XT(4)*XT(4))
	XF440DER=-(GD(3,3)-GD(3,4))
     &         +(GD(2,3)-GD(2,4))*(2.D0*XT(4))
     &         -(GD(1,3)-GD(1,4))*(XT(4)*XT(4))
     &         -(GL(2,3)-GL(2,4))*(-2.D0)
     &         +(GL(1,3)-GL(1,4))*(-2.D0*XT(4))
	XF444DER=-(GD(4,3)-GD(4,4))
     &         +(GD(3,3)-GD(3,4))*(3.D0*XT(4))
     &         -(GD(2,3)-GD(2,4))*(3.D0*XT(4)*XT(4))
     &         +(GD(1,3)-GD(1,4))*(XT(4)*XT(4)*XT(4))
     &         -(GL(3,3)-GL(3,4))*(-3.D0)
     &         +(GL(2,3)-GL(2,4))*(-6.D0*XT(4))
     &         -(GL(1,3)-GL(1,4))*(-3.D0*XT(4)*XT(4))
	TS=TS+XF440INT/(X14*X24*X34)*3.D0
	DS=DS+XF440DER/(X14*X24*X34)*3.D0
	AT(L(1))=AT(L(1))+XF444INT/(X14*X24*X34*X14)
	AT(L(2))=AT(L(2))+XF444INT/(X14*X24*X34*X24)
	AT(L(3))=AT(L(3))+XF444INT/(X14*X24*X34*X34)
	AT(L(4))=AT(L(4))+XF440INT/(X14*X24*X34)*3.D0
     &                 -XF444INT/(X14*X24*X34*X14)
     &                 -XF444INT/(X14*X24*X34*X24)
     &                 -XF444INT/(X14*X24*X34*X34)
	AD(L(1))=AD(L(1))+XF444DER/(X14*X24*X34*X14)
	AD(L(2))=AD(L(2))+XF444DER/(X14*X24*X34*X24)
	AD(L(3))=AD(L(3))+XF444DER/(X14*X24*X34*X34)
	AD(L(4))=AD(L(4))+XF440DER/(X14*X24*X34)*3.D0
     &                 -XF444DER/(X14*X24*X34*X14)
     &                 -XF444DER/(X14*X24*X34*X24)
     &                 -XF444DER/(X14*X24*X34*X34)
	ENDIF
C 3. From XT(3) to XT(2)
	IF(ABS(ET(2)-ET(3)).GT.EPS)THEN
	XF130INT= (GL(3,2)-GL(3,3))
     &         -(GL(2,2)-GL(2,3))*(XT(1)+XT(3))
     &         +(GL(1,2)-GL(1,3))*(XT(1)*XT(3))
	XF240INT= (GL(3,2)-GL(3,3))
     &         -(GL(2,2)-GL(2,3))*(XT(2)+XT(4))
     &         +(GL(1,2)-GL(1,3))*(XT(2)*XT(4))
	XF113INT= (GL(4,2)-GL(4,3))
     &         -(GL(3,2)-GL(3,3))*(XT(1)+XT(1)+XT(3))
     &         +(GL(2,2)-GL(2,3))*(XT(1)*XT(3)+XT(1)*XT(1)+XT(1)*XT(3))
     &         -(GL(1,2)-GL(1,3))*(XT(1)*XT(1)*XT(3))
	XF244INT= (GL(4,2)-GL(4,3))
     &         -(GL(3,2)-GL(3,3))*(XT(2)+XT(4)+XT(4))
     &         +(GL(2,2)-GL(2,3))*(XT(2)*XT(4)+XT(4)*XT(4)+XT(2)*XT(4))
     &         -(GL(1,2)-GL(1,3))*(XT(2)*XT(4)*XT(4))
	XF234INT= (GL(4,2)-GL(4,3))
     &         -(GL(3,2)-GL(3,3))*(XT(2)+XT(3)+XT(4))
     &         +(GL(2,2)-GL(2,3))*(XT(2)*XT(4)+XT(3)*XT(4)+XT(2)*XT(3))
     &         -(GL(1,2)-GL(1,3))*(XT(2)*XT(3)*XT(4))
	XF133INT= (GL(4,2)-GL(4,3))
     &         -(GL(3,2)-GL(3,3))*(XT(1)+XT(3)+XT(3))
     &         +(GL(2,2)-GL(2,3))*(XT(1)*XT(3)+XT(1)*XT(3)+XT(3)*XT(3))
     &         -(GL(1,2)-GL(1,3))*(XT(1)*XT(3)*XT(3))
	XF130DER=-(GD(3,2)-GD(3,3))
     &         +(GD(2,2)-GD(2,3))*(XT(1)+XT(3))
     &         -(GD(1,2)-GD(1,3))*(XT(1)*XT(3))
     &         -(GL(2,2)-GL(2,3))*(-2.D0)
     &         +(GL(1,2)-GL(1,3))*(-XT(1)-XT(3))
	XF240DER=-(GD(3,2)-GD(3,3))
     &         +(GD(2,2)-GD(2,3))*(XT(2)+XT(4))
     &         -(GD(1,2)-GD(1,3))*(XT(2)*XT(4))
     &         -(GL(2,2)-GL(2,3))*(-2.D0)
     &         +(GL(1,2)-GL(1,3))*(-XT(2)-XT(4))
	XF113DER=-(GD(4,2)-GD(4,3))
     &         +(GD(3,2)-GD(3,3))*(XT(1)+XT(1)+XT(3))
     &         -(GD(2,2)-GD(2,3))*(XT(1)*XT(3)+XT(1)*XT(1)+XT(1)*XT(3))
     &         +(GD(1,2)-GD(1,3))*(XT(1)*XT(1)*XT(3))
     &         -(GL(3,2)-GL(3,3))*(-3.D0)
     &         +(GL(2,2)-GL(2,3))*(-2.D0*XT(1)-2.D0*XT(1)-2.D0*XT(3))
     &         -(GL(1,2)-GL(1,3))*(-XT(1)*XT(3)-XT(1)*XT(1)-XT(1)*XT(3))
	XF244DER=-(GD(4,2)-GD(4,3))
     &         +(GD(3,2)-GD(3,3))*(XT(2)+XT(4)+XT(4))
     &         -(GD(2,2)-GD(2,3))*(XT(2)*XT(4)+XT(4)*XT(4)+XT(2)*XT(4))
     &         +(GD(1,2)-GD(1,3))*(XT(2)*XT(4)*XT(4))
     &         -(GL(3,2)-GL(3,3))*(-3.D0)
     &         +(GL(2,2)-GL(2,3))*(-2.D0*XT(2)-2.D0*XT(4)-2.D0*XT(4))
     &         -(GL(1,2)-GL(1,3))*(-XT(2)*XT(4)-XT(4)*XT(4)-XT(2)*XT(4))
	XF234DER=-(GD(4,2)-GD(4,3))
     &         +(GD(3,2)-GD(3,3))*(XT(2)+XT(3)+XT(4))
     &         -(GD(2,2)-GD(2,3))*(XT(2)*XT(4)+XT(3)*XT(4)+XT(2)*XT(3))
     &         +(GD(1,2)-GD(1,3))*(XT(2)*XT(3)*XT(4))
     &         -(GL(3,2)-GL(3,3))*(-3.D0)
     &         +(GL(2,2)-GL(2,3))*(-2.D0*XT(2)-2.D0*XT(3)-2.D0*XT(4))
     &         -(GL(1,2)-GL(1,3))*(-XT(2)*XT(4)-XT(3)*XT(4)-XT(2)*XT(3))
	XF133DER=-(GD(4,2)-GD(4,3))
     &         +(GD(3,2)-GD(3,3))*(XT(1)+XT(3)+XT(3))
     &         -(GD(2,2)-GD(2,3))*(XT(1)*XT(3)+XT(1)*XT(3)+XT(3)*XT(3))
     &         +(GD(1,2)-GD(1,3))*(XT(1)*XT(3)*XT(3))
     &         -(GL(3,2)-GL(3,3))*(-3.D0)
     &         +(GL(2,2)-GL(2,3))*(-2.D0*XT(1)-2.D0*XT(3)-2.D0*XT(3))
     &         -(GL(1,2)-GL(1,3))*(-XT(1)*XT(3)-XT(1)*XT(3)-XT(3)*XT(3))
	TS=TS-XF240INT*3.D0/(X14*X24*X23)-XF130INT*3.D0/(X13*X14*X23)
	DS=DS-XF240DER*3.D0/(X14*X24*X23)-XF130DER*3.D0/(X13*X14*X23)
	AT(L(1))=AT(L(1))-XF130INT*2.D0/(X13*X14*X23)
     &                 -XF113INT/(X13*X14*X23*X13) !c3
     &                 -XF113INT/(X13*X14*X23*X14) !c1
     &                 -XF244INT/(X14*X24*X23*X14) !c2
	AT(L(2))=AT(L(2))-XF244INT/(X14*X24*X23*X24) !c4
     &                 -XF234INT/(X14*X24*X23*X23) !c6
     &                 -XF133INT/(X13*X14*X23*X23) !c5
	AT(L(3))=AT(L(3))-XF130INT/(X13*X14*X23)
     &                 -XF240INT/(X14*X24*X23)
     &                 +XF113INT/(X13*X14*X23*X13) !c3
     &                 +XF133INT/(X13*X14*X23*X23) !c5
     &                 +XF234INT/(X14*X24*X23*X23) !c6
	AT(L(4))=AT(L(4))-XF240INT*2.D0/(X14*X24*X23)
     &                 +XF244INT/(X14*X24*X23*X14) !c2
     &                 +XF244INT/(X14*X24*X23*X24) !c4
     &                 +XF113INT/(X13*X14*X23*X14) !c1
	AD(L(1))=AD(L(1))-XF130DER*2.D0/(X13*X14*X23)
     &                 -XF113DER/(X13*X14*X23*X13) !c3
     &                 -XF113DER/(X13*X14*X23*X14) !c1
     &                 -XF244DER/(X14*X24*X23*X14) !c2
	AD(L(2))=AD(L(2))-XF244DER/(X14*X24*X23*X24) !c4
     &                 -XF234DER/(X14*X24*X23*X23) !c6
     &                 -XF133DER/(X13*X14*X23*X23) !c5
	AD(L(3))=AD(L(3))-XF130DER/(X13*X14*X23)
     &                 -XF240DER/(X14*X24*X23)
     &                 +XF113DER/(X13*X14*X23*X13) !c3
     &                 +XF133DER/(X13*X14*X23*X23) !c5
     &                 +XF234DER/(X14*X24*X23*X23) !c6
	AD(L(4))=AD(L(4))-XF240DER*2.D0/(X14*X24*X23)
     &                 +XF244DER/(X14*X24*X23*X14) !c2
     &                 +XF244DER/(X14*X24*X23*X24) !c4
     &                 +XF113DER/(X13*X14*X23*X14) !c1
	ENDIF
C 4. From XT(2) to XT(1)
	IF(ABS(ET(1)-ET(2)).GT.EPS)THEN
	XF110INT= (GL(3,1)-GL(3,2))
     &         -(GL(2,1)-GL(2,2))*(XT(1)+XT(1))
     &         +(GL(1,1)-GL(1,2))*(XT(1)*XT(1))
	XF111INT= (GL(4,1)-GL(4,2))
     &         -(GL(3,1)-GL(3,2))*(XT(1)+XT(1)+XT(1))
     &         +(GL(2,1)-GL(2,2))*(XT(1)*XT(1)+XT(1)*XT(1)+XT(1)*XT(1))
     &         -(GL(1,1)-GL(1,2))*(XT(1)*XT(1)*XT(1))
	XF110DER=-(GD(3,1)-GD(3,2))
     &         +(GD(2,1)-GD(2,2))*(2.D0*XT(1))
     &         -(GD(1,1)-GD(1,2))*(XT(1)*XT(1))
     &         -(GL(2,1)-GL(2,2))*(-2.D0)
     &         +(GL(1,1)-GL(1,2))*(-2.D0*XT(1))
	XF111DER=-(GD(4,1)-GD(4,2))
     &         +(GD(3,1)-GD(3,2))*(3.D0*XT(1))
     &         -(GD(2,1)-GD(2,2))*(3.D0*XT(1)*XT(1))
     &         +(GD(1,1)-GD(1,2))*(XT(1)*XT(1)*XT(1))
     &         -(GL(3,1)-GL(3,2))*(-3.D0)
     &         +(GL(2,1)-GL(2,2))*(-6.D0*XT(1))
     &         -(GL(1,1)-GL(1,2))*(-3.D0*XT(1)*XT(1))
	TS=TS+XF110INT/(X12*X13*X14)*3.D0
      DS=DS+XF110DER/(X12*X13*X14)*3.D0
	AT(L(1))=AT(L(1))+XF110INT/(X12*X13*X14)*3.D0
     &                 +XF111INT/(X12*X13*X14*X12)
     &                 +XF111INT/(X12*X13*X14*X13)
     &                 +XF111INT/(X12*X13*X14*X14)
	AT(L(2))=AT(L(2))-XF111INT/(X12*X13*X14*X12)
	AT(L(3))=AT(L(3))-XF111INT/(X12*X13*X14*X13)
	AT(L(4))=AT(L(4))-XF111INT/(X12*X13*X14*X14)
	AD(L(1))=AD(L(1))+XF110DER/(X12*X13*X14)*3.D0
     &                 +XF111DER/(X12*X13*X14*X12)
     &                 +XF111DER/(X12*X13*X14*X13)
     &                 +XF111DER/(X12*X13*X14*X14)
	AD(L(2))=AD(L(2))-XF111DER/(X12*X13*X14*X12)
	AD(L(3))=AD(L(3))-XF111DER/(X12*X13*X14*X13)
	AD(L(4))=AD(L(4))-XF111DER/(X12*X13*X14*X14)
	ENDIF
C From XT(1) to +inf
	! nothing
C Normalize
	TS=TS*OT
	DS=DS*OT/DE
	DO I=1,4
	AT(I)=AT(I)*OT
	AD(I)=AD(I)*OT/DE
	ENDDO
C That's it!
	RETURN
	END







	SUBROUTINE TTR_MCTTRINT1(VINP,VT,A)
*****************************************************************
*    The calculation of the integrals over tetrahedron          *
*                                                               *
*        /  3     R(k)                                          *
*    A=  \ d k   ------                                         *
*        /        V(k)                                          *
*       ttr                                                     *
*                                                               *
*    (For instance V(k)=E(k)-E(k+q)+omega in the linear         *
*                                             response theory.) *
*    where V(k) is given at the vertexes of tetrahedron,        *
*    R(k) is 1,kx,ky,kz (for the matrix elements evaluation)    *
*    The linear interpolation is supposed.                      *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8 VINP(4),VT,A(4)
	DIMENSION V(4),D(4),ALN(4),ICON(6),LL(6),MM(6)
     &   ,T(2:4),KK(2),KP(5),NN2(2:4,6),NN3(2:4,3)
     &   ,NN4(2:4,5)
	DATA LL/1,1,1,2,2,3/,MM/2,3,4,3,4,4/,KP/4,3,2,0,1/
     &      ,NN2/4,3,2,3,4,2,3,2,4,4,4,2,4,2,4,2,4,4/
     &      ,NN3/2,3,3,3,2,3,3,3,2/
     &      ,NN4/3,3,2,3,2,3,2,3,3,3,3,3,3,3,3/
	DATA EPS/1.D-04/
C   INITIALIZATION
	DO 1 I=1,4 
	V(I)=VINP(I)
	IF(ABS(V(I)).LE.1.D-08)V(I)=1.D-08
   1	CONTINUE
C   DEFINE CASE OF COINCIDING
	DO 20 I=1,4
	DO 10 J=I+1,4
	IF(ABS(V(I)-V(J)).LE.EPS)V(J)=V(I)
  10  CONTINUE
  20  CONTINUE
	ICASE=0
	K=0
	DO 40 I=1,4
	DO 30 J=I+1,4
	K=K+1
	ICON(K)=0
	IF(ABS(V(I)-V(J)).LE.1.D-05)ICON(K)=1
	ICASE=ICASE+ICON(K)
  30  CONTINUE
  40  CONTINUE
C
	IF(ICASE.EQ.0)THEN !! ALL ELEMENTS ARE DIFFERENT
	D(1)=V(1)**2/((V(1)-V(2))*(V(1)-V(3))*(V(1)-V(4)))
	D(2)=V(2)**2/((V(2)-V(1))*(V(2)-V(3))*(V(2)-V(4)))
	D(3)=V(3)**2/((V(3)-V(1))*(V(3)-V(2))*(V(3)-V(4)))
	D(4)=-D(1)-D(2)-D(3)
	A(1)=0.D0
	DO 50 I=1,4
	ALN(I)=LOG(ABS(V(I)))
	A(1)=A(1)+3.D0*D(I)*ALN(I)
  50  CONTINUE
	DO 70 J=2,4
	A(J)=0.D0
	DO 60 I=1,4
	IF(I.NE.J)THEN
	A(J)=A(J)+V(I)*D(I)*(ALN(I)-ALN(J))/(V(I)-V(J))
	ELSE
	A(J)=A(J)+D(J)
	ENDIF     
  60  CONTINUE
  70  CONTINUE
C
	ELSEIF(ICASE.EQ.1)THEN !! TWO ELEMENTS ARE EQUAL
	DO 80 K=1,6
	IF(ICON(K).EQ.1)K0=K
  80  CONTINUE
	VV=V(LL(K0))          !! EQUAL ELEMENTS
	VV1=V(LL(7-K0))
	VV2=V(MM(7-K0))
	ALNV=LOG(ABS(VV))
	ALNV1=LOG(ABS(VV1))
	ALNV2=LOG(ABS(VV2))
	DD1=1.D0/((VV1-VV)*(VV1-VV)*(VV1-VV2))
	DD2=1.D0/((VV2-VV)*(VV2-VV)*(VV2-VV1))
	DD3=1.D0/((VV-VV1)*(VV-VV2))
	A(1)=3.D0*VV1**2*DD1*(ALNV1-ALNV)+3.D0*VV2**2*DD2*(ALNV2-ALNV)+
     &     3.D0*VV*DD3
	SS1=DD3/(VV-VV1)
	SS2=DD3/(VV-VV2)
	OO2=3.D0-VV/(VV-VV1)-2.D0*VV/(VV-VV2)
	OO3=3.D0-VV/(VV-VV2)-2.D0*VV/(VV-VV1)
	T(2)=VV**2*SS2*OO2*(ALNV-ALNV2)+VV1**3*DD1/(VV1-VV2)*
     &   (ALNV1-ALNV2)+VV**2*SS2+VV2**2*DD2
	T(3)=VV**2*SS1*OO3*(ALNV-ALNV1)+VV2**3*DD2/(VV2-VV1)*
     &   (ALNV2-ALNV1)+VV**2*SS1+VV1**2*DD1
	T(4)=VV1**3*DD1/(VV1-VV)*(ALNV1-ALNV)+VV2**3*DD2/(VV2-VV)*
     &   (ALNV2-ALNV)+2.5D0*VV*DD3-VV**2*(SS1+SS2)
	DO 90 I=2,4
	A(I)=T(NN2(I,K0))
  90  CONTINUE
C
	ELSEIF(ICASE.EQ.2)THEN  !! TWO PAIRS ARE EQUAL
	K=0
	DO 100 I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+1
	KK(K)=I
	ENDIF
 100  CONTINUE
	VV=V(LL(KK(1)))
	VV1=V(LL(KK(2)))
C  NEW FORMULAE
	XX=VV1/VV
	XX1=VV/VV1
	ALOGXX=LOG(ABS(XX))
	A(1)=VV*VV1/(VV-VV1)**3*
     &   (6.D0*ALOGXX+3.D0*(XX1-XX))
	T(2)=VV*VV1/(VV-VV1)**3*
     &   (3.D0/(1.D0-XX1)*ALOGXX+2.5D0*(XX1-1.D0)-2.D0*XX1-XX)
	T(3)=VV*VV1/(VV-VV1)**3*
     &   (3.D0/(1.D0-XX)*ALOGXX+2.5D0*(1.D0-XX)+2.D0*XX+XX1)
C  OLD FORMULAE
C	ALNV=LOG(ABS(VV))
C	ALNV1=LOG(ABS(VV1))
C	DD=1.D0/(VV-VV1)**2
C	DD2=DD*DD
C	A(1)=6.D0*VV*VV1*DD2*(VV-VV1)*(ALNV1-ALNV)+3.D0*(VV+VV1)*DD
C	T(2)=3.D0*VV*VV1**2*DD2*(ALNV-ALNV1)+2.5D0*VV*DD+(2.D0*VV**2+
C    &   VV1**2)*DD2*(VV1-VV)
C	T(3)=3.D0*VV1*VV**2*DD2*(ALNV1-ALNV)+2.5D0*VV1*DD+(2.D0*VV1**2+
C    &   VV**2)*DD2*(VV-VV1)
 	DO 110 I=2,4
 	A(I)=T(NN3(I,KK(1)))
 110  CONTINUE
C
	ELSEIF(ICASE.EQ.3)THEN   !! THREE ELEMENTS ARE EQUAL
	K=0
	DO 120 I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+I
	K0=I
	ENDIF
 120  CONTINUE
	M0=(K-5)/2
	VV=V(LL(K0))
	VV1=V(KP(M0))
C  NEW FORMULAE
	XX=VV1/VV
	XX1=VV/VV1
	ALOGXX=LOG(ABS(XX))
	A(1)=VV1*VV/(VV1-VV)**3*
     &   (3.D0*XX*ALOGXX+6.D0-4.5D0*XX-1.5D0*XX1)
	T(2)=VV1*VV/(VV1-VV)**3*
     &   (3.D0*XX/(1.D0-XX)*ALOGXX+2.5D0-0.5D0*XX1+XX)
	T(3)=VV1*VV/(VV1-VV)**3*
     &   (XX/(1.D0-XX1)*ALOGXX-11.D0/6.D0*XX*(1.D0-XX1)**2+
     &   1.5D0*XX1-2.5D0)
C  OLD FORMULAE
C     DD=1.D0/(VV1-VV)**2
C     DD1=DD/(VV1-VV)
C     DD2=DD*DD
C     ALNV=LOG(ABS(VV))
C     ALNV1=LOG(ABS(VV1))
C     A(1)=3*VV1**2*DD1*(ALNV1-ALNV)+DD1*(6.D0*VV*VV1-
C    &   4.5D0*VV1**2-1.5D0*VV**2)
C     T(2)=3.D0*VV*VV1**2*DD2*(ALNV-ALNV1)+DD*2.5D0*VV+
C    &   DD1*(2.D0*VV**2+VV1**2)
C     T(3)=VV1**3*DD2*(ALNV1-ALNV)+11.D0/(VV-VV1)/6.D0-2.5D0*VV*DD-
C    &   VV**2*DD1
	DO 130 I=2,4
	A(I)=T(NN4(I,M0))
 130  CONTINUE
C
	ELSEIF(ICASE.EQ.6)THEN !! ALL ELEMENTS ARE EQUAL
	A(1)=1.D0/V(1)
	A(2)=0.25D0*A(1)
	A(3)=A(2)
	A(4)=A(3)
	ENDIF
C  NORMALIZATION
    	DO 140 J=1,4
	A(J)=A(J)*VT
 140  CONTINUE
	END



      SUBROUTINE TTR_MCTTRINT2(VT,A)
************************************************************
*    The calculation of the integrals over tetrahedron     *
*                                                          *
*        /  3                                              *
*    A=  \ d k  R(k)                                       *
*        /                                                 *
*       ttr                                                *
*                                                          *
*    where R(k) are 1,kx,ky,kz (for the matrix elements    *
*    evaluation). The linear interpolation is supposed.    *
*    Written by S.Yu.Savrasov (Lebedev Physical Institute) *
************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(4)
      A(1)=VT
      DO 10 I=2,4
      A(I)=VT*0.25D0
  10  CONTINUE
      END



	SUBROUTINE TTR_MCTTRINT3(VINP,VT,A)
*****************************************************************
*    The calculation of the integrals over tetrahedron          *
*                                                               *
*        /  3     R(k)                                          *
*    A=  \ d k   ------                                         *
*        /        V(k)                                          *
*       ttr                                                     *
*                                                               *
*    (For instance V(k)=E(k)-E(k+q)+omega in the linear         *
*                                             response theory.) *
*    where V(k) is given at the vertexes of tetrahedron,        *
*    R(k) is 1,kx,ky,kz (for the matrix elements evaluation)    *
*    The linear interpolation is supposed.                      *
*    Complex version
*****************************************************************
	IMPLICIT COMPLEX*16 (A-H,O-Z)
	INTEGER ICON(6),LL(6),MM(6),KK(2),KP(5),NN2(2:4,6),NN3(2:4,3)
     &   ,NN4(2:4,5)
	REAL*8 VT,EPS
	COMPLEX*16 VINP(4),A(4),V(4),D(4),ALN(4),T(2:4)
	DATA LL/1,1,1,2,2,3/,MM/2,3,4,3,4,4/,KP/4,3,2,0,1/
     &      ,NN2/4,3,2,3,4,2,3,2,4,4,4,2,4,2,4,2,4,4/
     &      ,NN3/2,3,3,3,2,3,3,3,2/
     &      ,NN4/3,3,2,3,2,3,2,3,3,3,3,3,3,3,3/
	DATA EPS/1.D-03/
C   INITIALIZATION
	DO 1 I=1,4 
	V(I)=VINP(I)
	IF(ABS(V(I)).LE.1.D-08)V(I)=1.D-08
   1	CONTINUE
C   DEFINE CASE OF COINCIDING
	DO 20 I=1,4
	DO 10 J=I+1,4
	IF(ABS(V(I)-V(J)).LE.EPS)V(J)=V(I)
	IF(ABS(V(I)).GT.1.D-07.AND.ABS(V(J)).GT.1.D-07)THEN
      IF(ABS(V(I)-V(J))/ABS(V(I)).LE.EPS.AND.
     &   ABS(V(I)-V(J))/ABS(V(J)).LE.EPS)V(J)=V(I)
	ENDIF
  10  CONTINUE
  20  CONTINUE
	ICASE=0
	K=0
	DO 40 I=1,4
	DO 30 J=I+1,4
	K=K+1
	ICON(K)=0
	IF(V(I).EQ.V(J))ICON(K)=1
	ICASE=ICASE+ICON(K)
  30  CONTINUE
  40  CONTINUE
C
	IF(ICASE.EQ.0)THEN !! ALL ELEMENTS ARE DIFFERENT
	D(1)=V(1)**2/((V(1)-V(2))*(V(1)-V(3))*(V(1)-V(4)))
	D(2)=V(2)**2/((V(2)-V(1))*(V(2)-V(3))*(V(2)-V(4)))
	D(3)=V(3)**2/((V(3)-V(1))*(V(3)-V(2))*(V(3)-V(4)))
	D(4)=-D(1)-D(2)-D(3)
	A(1)=(0.D0,0.D0)
	DO 50 I=1,4
	ALN(I)=LOG(V(I)) ! was LOG(ABS(V(I))
	A(1)=A(1)+3.D0*D(I)*ALN(I)
  50  CONTINUE
	DO 70 J=2,4
	A(J)=(0.D0,0.D0)
	DO 60 I=1,4
	IF(I.NE.J)THEN
	A(J)=A(J)+V(I)*D(I)*(ALN(I)-ALN(J))/(V(I)-V(J))
	ELSE
	A(J)=A(J)+D(J)
	ENDIF     
  60  CONTINUE
  70  CONTINUE
C
	ELSEIF(ICASE.EQ.1)THEN !! TWO ELEMENTS ARE EQUAL
	DO 80 K=1,6
	IF(ICON(K).EQ.1)K0=K
  80  CONTINUE
	VV=V(LL(K0))          !! EQUAL ELEMENTS
	VV1=V(LL(7-K0))
	VV2=V(MM(7-K0))
	ALNV=LOG(VV)   ! was LOG(ABS(VV))
	ALNV1=LOG(VV1) ! was LOG(ABS(VV1))
	ALNV2=LOG(VV2) ! was LOG(ABS(VV2))
	DD1=1.D0/((VV1-VV)*(VV1-VV)*(VV1-VV2))
	DD2=1.D0/((VV2-VV)*(VV2-VV)*(VV2-VV1))
	DD3=1.D0/((VV-VV1)*(VV-VV2))
	A(1)=3.D0*VV1**2*DD1*(ALNV1-ALNV)+3.D0*VV2**2*DD2*(ALNV2-ALNV)+
     &     3.D0*VV*DD3
	SS1=DD3/(VV-VV1)
	SS2=DD3/(VV-VV2)
	OO2=3.D0-VV/(VV-VV1)-2.D0*VV/(VV-VV2)
	OO3=3.D0-VV/(VV-VV2)-2.D0*VV/(VV-VV1)
	T(2)=VV**2*SS2*OO2*(ALNV-ALNV2)+VV1**3*DD1/(VV1-VV2)*
     &   (ALNV1-ALNV2)+VV**2*SS2+VV2**2*DD2
	T(3)=VV**2*SS1*OO3*(ALNV-ALNV1)+VV2**3*DD2/(VV2-VV1)*
     &   (ALNV2-ALNV1)+VV**2*SS1+VV1**2*DD1
	T(4)=VV1**3*DD1/(VV1-VV)*(ALNV1-ALNV)+VV2**3*DD2/(VV2-VV)*
     &   (ALNV2-ALNV)+2.5D0*VV*DD3-VV**2*(SS1+SS2)
	DO 90 I=2,4
	A(I)=T(NN2(I,K0))
  90  CONTINUE
C
	ELSEIF(ICASE.EQ.2)THEN  !! TWO PAIRS ARE EQUAL
	K=0
	DO 100 I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+1
	KK(K)=I
	ENDIF
 100  CONTINUE
	VV=V(LL(KK(1)))
	VV1=V(LL(KK(2)))
C  NEW FORMULAE
	XX=VV1/VV
	XX1=VV/VV1
	ALOGXX=LOG(XX) !was LOG(ABS(XX))
	A(1)=VV*VV1/(VV-VV1)**3*
     &   (6.D0*ALOGXX+3.D0*(XX1-XX))
	T(2)=VV*VV1/(VV-VV1)**3*
     &   (3.D0/(1.D0-XX1)*ALOGXX+2.5D0*(XX1-1.D0)-2.D0*XX1-XX)
	T(3)=VV*VV1/(VV-VV1)**3*
     &   (3.D0/(1.D0-XX)*ALOGXX+2.5D0*(1.D0-XX)+2.D0*XX+XX1)
C  OLD FORMULAE
C	ALNV=LOG(ABS(VV))
C	ALNV1=LOG(ABS(VV1))
C	DD=1.D0/(VV-VV1)**2
C	DD2=DD*DD
C	A(1)=6.D0*VV*VV1*DD2*(VV-VV1)*(ALNV1-ALNV)+3.D0*(VV+VV1)*DD
C	T(2)=3.D0*VV*VV1**2*DD2*(ALNV-ALNV1)+2.5D0*VV*DD+(2.D0*VV**2+
C    &   VV1**2)*DD2*(VV1-VV)
C	T(3)=3.D0*VV1*VV**2*DD2*(ALNV1-ALNV)+2.5D0*VV1*DD+(2.D0*VV1**2+
C    &   VV**2)*DD2*(VV-VV1)
 	DO 110 I=2,4
 	A(I)=T(NN3(I,KK(1)))
 110  CONTINUE
C
	ELSEIF(ICASE.EQ.3)THEN   !! THREE ELEMENTS ARE EQUAL
	K=0
	DO 120 I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+I
	K0=I
	ENDIF
 120  CONTINUE
	M0=(K-5)/2
	VV=V(LL(K0))
	VV1=V(KP(M0))
C  NEW FORMULAE
	XX=VV1/VV
	XX1=VV/VV1
	ALOGXX=LOG(XX) !was LOG(ABS(XX))
	A(1)=VV1*VV/(VV1-VV)**3*
     &   (3.D0*XX*ALOGXX+6.D0-4.5D0*XX-1.5D0*XX1)
	T(2)=VV1*VV/(VV1-VV)**3*
     &   (3.D0*XX/(1.D0-XX)*ALOGXX+2.5D0-0.5D0*XX1+XX)
	T(3)=VV1*VV/(VV1-VV)**3*
     &   (XX/(1.D0-XX1)*ALOGXX-11.D0/6.D0*XX*(1.D0-XX1)**2+
     &   1.5D0*XX1-2.5D0)
C  OLD FORMULAE
C     DD=1.D0/(VV1-VV)**2
C     DD1=DD/(VV1-VV)
C     DD2=DD*DD
C     ALNV=LOG(ABS(VV))
C     ALNV1=LOG(ABS(VV1))
C     A(1)=3*VV1**2*DD1*(ALNV1-ALNV)+DD1*(6.D0*VV*VV1-
C    &   4.5D0*VV1**2-1.5D0*VV**2)
C     T(2)=3.D0*VV*VV1**2*DD2*(ALNV-ALNV1)+DD*2.5D0*VV+
C    &   DD1*(2.D0*VV**2+VV1**2)
C     T(3)=VV1**3*DD2*(ALNV1-ALNV)+11.D0/(VV-VV1)/6.D0-2.5D0*VV*DD-
C    &   VV**2*DD1
	DO 130 I=2,4
	A(I)=T(NN4(I,M0))
 130  CONTINUE
C
	ELSEIF(ICASE.EQ.6)THEN !! ALL ELEMENTS ARE EQUAL
	A(1)=1.D0/V(1)
	A(2)=0.25D0*A(1)
	A(3)=A(2)
	A(4)=A(3)
	ENDIF
C  NORMALIZATION
    	DO 140 J=1,4
	A(J)=A(J)*VT
 140  CONTINUE
	END



	SUBROUTINE TTR_MCTTRINT4(VINP,VT,Z,A,DA,KEY)
*****************************************************************
*    The calculation of the integrals over tetrahedron          *
*                                                               *
*        /  3     R(k)                                          *
*    A=  \ d k   ------                                         *
*        /        V(k)                                          *
*       ttr                                                     *
*                                                               *
*    (For instance V(k)=E(k)-E(k+q)+omega in the linear         *
*                                             response theory.) *
*    where V(k) is given at the vertexes of tetrahedron,        *
*    R(k) is 1,kx,ky,kz (for the matrix elements evaluation)    *
*    The linear interpolation is supposed.                      *
*                                                               *
*    Complex version, adapted from Poteryaev's programs         *
*****************************************************************
	IMPLICIT COMPLEX*16 (A-H,O-Z)
	INTEGER ICON(6),LL(6),MM(6),KK(2),KP(5),MM2(2:4,6),MM3(2:4,3)
     &   ,MM4(2:4,5)
	REAL*8 VT,EPS
	COMPLEX*16 VINP(4),A(4),V(4),RES(4),Z,E(4),DA(4)
	DATA EPS/1.D-05/
	DATA LL/1,1,1,2,2,3/,MM/2,3,4,3,4,4/,KP/4,3,2,0,1/
     &      ,MM2/2,3,4,3,2,4,3,4,2,2,2,4,2,4,2,4,2,2/
     &      ,MM3/2,3,3,3,2,3,3,3,2/
     &      ,MM4/2,2,4,2,4,2,4,2,2,2,2,2,2,2,2/
C   INITIALIZATION
	DO I=1,4 
	V(I)=VINP(I)
	IF(ABS(V(I)).LE.1.D-08)V(I)=1.D-08
	ENDDO
C   DEFINE CASE OF COINCIDING
	DO I=1,4
	DO J=I+1,4
	IF(ABS(V(I)-V(J)).LE.EPS)V(J)=V(I)
	IF(ABS(V(I)).GT.1.D-07.AND.ABS(V(J)).GT.1.D-07)THEN
      IF(ABS(V(I)-V(J))/ABS(V(I)).LE.EPS.AND.
     &   ABS(V(I)-V(J))/ABS(V(J)).LE.EPS)V(J)=V(I)
	ENDIF
	ENDDO
	ENDDO
	ICASE=0
	K=0
	DO I=1,4
	DO J=I+1,4
	K=K+1
	ICON(K)=0
	IF(V(I).EQ.V(J))ICON(K)=1
	ICASE=ICASE+ICON(K)
	ENDDO
	ENDDO
C
	IF(ICASE.EQ.0)THEN !! ALL ELEMENTS ARE DIFFERENT
C
	E(1)=V(1)
	E(2)=V(2)
	E(3)=V(3)
	E(4)=V(4)
	CALL TTR_MCTTRINT4_ARBEK(Z,E,RES)
	A(1)=-RES(1)-RES(2)-RES(3)-RES(4)
	A(2)=-RES(2)
	A(3)=-RES(3)
	A(4)=-RES(4)

	ELSEIF(ICASE.EQ.1)THEN !! TWO ELEMENTS ARE EQUAL
	DO K=1,6
	IF(ICON(K).EQ.1)K0=K
	ENDDO
	E(1)=V(LL(K0))          !! EQUAL ELEMENTS
	E(2)=V(LL(K0))
	E(3)=V(LL(7-K0))
	E(4)=V(MM(7-K0))
	CALL TTR_MCTTRINT4_EK2IDN(Z,E,RES)
	A(1)=-RES(1)-RES(2)-RES(3)-RES(4)
	DO I=2,4
	A(I)=-RES(MM2(I,K0))
	ENDDO
C
	ELSEIF(ICASE.EQ.2)THEN  !! TWO PAIRS ARE EQUAL
	K=0
	DO I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+1
	KK(K)=I
	ENDIF
	ENDDO
	E(1)=V(LL(KK(1)))
	E(2)=V(LL(KK(1)))
	E(3)=V(LL(KK(2)))
	E(4)=V(LL(KK(2)))
	CALL TTR_MCTTRINT4_EK22IDN(Z,E,RES)
	A(1)=-RES(1)-RES(2)-RES(3)-RES(4)
	DO I=2,4
	A(I)=-RES(MM3(I,KK(1)))
	ENDDO
C
	ELSEIF(ICASE.EQ.3)THEN   !! THREE ELEMENTS ARE EQUAL
	K=0
	DO I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+I
	K0=I
	ENDIF
	ENDDO
	M0=(K-5)/2

	E(1)=V(LL(K0))
	E(2)=V(LL(K0))
	E(3)=V(LL(K0))
	E(4)=V(KP(M0))
	CALL TTR_MCTTRINT4_EK3IDN(Z,E,RES)
	A(1)=-RES(1)-RES(2)-RES(3)-RES(4)
	DO I=2,4
	A(I)=-RES(MM4(I,M0))
	ENDDO
C
	ELSEIF(ICASE.EQ.6)THEN !! ALL ELEMENTS ARE EQUAL
	A(1)=1.D0/V(1)
	A(2)=0.25D0*A(1)
	A(3)=A(2)
	A(4)=A(3)
	ENDIF

C  NORMALIZATION
    	DO J=1,4
	A(J)=A(J)*VT
	DA(J)=(0.D0,0.D0)
	ENDDO

	CONTAINS

      subroutine TTR_mcttrint4_arbek( z,e,res )
c|--------------------------------------------------------------------|
c|                                                                    |
c|       Calculation of complex weight factor for different           |
c|                   complex eigenvalues                              |
c|                                                                    |
c|   Inputs:                                                          |
ci     z - complex energy                                             |
ci     e - eigenvalues                                                |
c|   Outputs:                                                         |
co     res  - weight factor of Green function of one orbital          |
c|   Remarks:                                                         |
cr     PRB 29,6,3430 (1984); (formulas 7,8)                           |
c|                                                                    |
c|--------------------------------------------------------------------|      
      implicit none
c---------- Passed variables ----------
      complex*16 z,res(4),e(4)
c---------- Local variables ----------
      integer i,j,k
      real*8 rezi,imzi,rezj,imzj
      complex*16 r,y,y1,t,zi,zj,zij,zero,one
      complex*16 ttr_mcttrint4_cdln1
c---------- Data constructions ----------
      data zero /(0.d0,0.d0)/, one /(1.d0,0.d0)/
c---------- Note that the line numbers are essential ----------
      do 10 i = 1,4
         r = zero
         do 12 j = 1,4
          if( j.eq.i ) goto 12
          zj = z - e(j)
          zij = e(i) - e(j)
          y = zij / zj
          if( cdabs( y ).gt.2.5d-2 )then
             t = zij
             do 13 k = 1,4
                if( k.eq.j ) goto 13
                t = t*( e(k)-e(j) )
 13          continue
            y1 = zj**3 / t 
            r  = r - y1 * ( y + 5.d-1*y*y + y*y*y / 3.d0 + 
     &           cdlog( 1.d0 - y ) )
         else
            t = one
            do 11 k = 1,4
               if( k.eq.j.or.k.eq.i ) goto 11
               t = t*( e(k)-e(j) )
 11         continue
            y1 = zij / t
            r  = r - y1*ttr_mcttrint4_cdln1( y,4,25,3 )
         end if 
 12   continue
      res(i) = r
 10   continue 
      end subroutine TTR_mcttrint4_arbek

      subroutine TTR_mcttrint4_ek22idn( z,e,res )
c|--------------------------------------------------------------------|
c|                                                                    |
c|       Calculation of complex weight factor in case when            |
c|              three corner energies are identical                   |
c|                                                                    |
c|   Inputs:                                                          |
ci     z - complex energy                                             |
ci     e - eigenvalues                                                |
c|   Outputs:                                                         |
co     res  - weight factor of Green function of one orbital          |
c|   Remarks:                                                         |
cr     PRB 29,6,3430 (1984); (formula A4)                             |
c|                                                                    |
c|--------------------------------------------------------------------|      
      implicit none
c---------- Passed variables ----------
      complex*16 z,res(4),e(4)
c---------- Local variables ----------
      complex*16 z2,z3,z32,y,one,two,three,r
      complex*16 ttr_mcttrint4_cdln1,ttr_mcttrint4_cdln2
c---------- Data constructions ----------
      data one /(1.d0,0.d0)/, two /(2.d0,0.d0)/, three /(3.d0,0.d0)/
      z2 = z - e(2)
      z3 = z - e(3)
      z32 = e(3) - e(2)

      if(abs(z2).le.1.d-08)z2 = (1.d-08,0.d0)
      if(abs(z3).le.1.d-08)z3 = (1.d-08,0.d0)

      if( cdabs( z32/z3 ).gt.2.5d-2 )then
         y = - three*z2*( two*z3 - z32 ) / ( two*z32**3 ) -
     &        one / z32
         r = three*cdlog( z2/z3 )*z3**2*z2 / z32**4 
         r = r + y
         res(1) = r
         res(2) = r
      else
         y = z32/z3
         r = three*( ttr_mcttrint4_cdln2( y,4,25,3 ) + 
     &               ttr_mcttrint4_cdln2( y,3,25,2 ) ) / z32 
         res(1) = r
         res(2) = r

      end if
      if( cdabs( z32/z2 ).gt.2.5d-2 )then
         y = three*z3*( two*z2 + z32 ) / ( two*z32**3 ) +
     &        one / z32
         r = three*cdlog( z3/z2 )*z2**2*z3 / z32**4 
         r = r + y
         res(3) = r
         res(4) = r
      else
         y = z32/z2
         r = three*( ttr_mcttrint4_cdln1( y,4,25,3 ) - 
     &               ttr_mcttrint4_cdln1( y,3,25,2 ) ) / z32 
         res(3) = r
         res(4) = r
      end if

      end subroutine TTR_mcttrint4_ek22idn

      subroutine TTR_mcttrint4_ek2idn( z,e,res )
c|--------------------------------------------------------------------|
c|                                                                    |
c|       Calculation of complex weight factor in case when            |
c|              two corner energies are identical                     |
c|                                                                    |
c|   Inputs:                                                          |
ci     z - complex energy                                             |
ci     e - eigenvalues                                                |
c|   Outputs:                                                         |
co     res  - weight factor of Green function of one orbital          |
c|   Remarks:                                                         |
cr     PRB 29,6,3430 (1984); (formulas A1,A2)                         |
c|                                                                    |
c|--------------------------------------------------------------------|
      implicit none
c---------- Passed variables ----------
      complex*16 z,res(4),e(4)
c---------- Local variables ----------
      complex*16 zm,zi,zk,zik,zim,zkm,y1,y2,zero,half,unit,two,three,r
      complex*16 ttr_mcttrint4_cdln2,ttr_mcttrint4_cda1
c---------- Data constructions ----------
      data zero /(0.d0,0.d0)/, half /(5.d-1,0.d0)/, unit /(1.d0,0.d0)/,
     &     two /(2.d0,0.d0)/, three /(3.d0,0.d0)/
      zm = z - e(1)
      zi = z - e(3)
      zk = z - e(4)
      zik = e(3) - e(4)
      zim = e(3) - e(1)
      zkm = e(4) - e(1)
      y1 = zim/zi
      if( cdabs( y1 ).gt.2.5d-2 )then
         y2 = cdlog( unit + y1 ) / ( y1**3 ) - unit / ( y1*y1 ) + 
     &        half / y1 - unit / three
         r  = - y2 / zik
      else
         r = - ttr_mcttrint4_cdln2( y1,4,25,3 ) / zik
      end if
      y1 = zkm/zk
      if( cdabs( y1 ).gt.2.5d-2 )then
         y2 = cdlog( unit + y1 ) / ( y1**3 ) - unit / ( y1*y1 ) + 
     &        half / y1 - unit / three
         r  = r + y2 / zik
      else
         r = r + ttr_mcttrint4_cdln2( y1,4,25,3 ) / zik
      end if
      res(1) = r
      res(2) = r
      res(3) = ttr_mcttrint4_cda1( z,e,3,4,1 )
      res(4) = ttr_mcttrint4_cda1( z,e,4,3,1 )
      end subroutine TTR_mcttrint4_ek2idn

      


      subroutine TTR_mcttrint4_ek3idn( z,e,res )
c|--------------------------------------------------------------------|
c|                                                                    |
c|       Calculation of complex weight factor in case when            |
c|              three corner energies are identical                   |
c|                                                                    |
c|   Inputs:                                                          |
ci     z - complex energy                                             |
ci     e - eigenvalues                                                |
c|   Outputs:                                                         |
co     res  - weight factor of Green function of one orbital          |
c|   Remarks:                                                         |
cr     PRB 29,6,3430 (1984); (formula A3)                             |
c|                                                                    |
c|--------------------------------------------------------------------|
      implicit none
c---------- Passed variables ----------
      complex*16 z,res(4),e(4)
c---------- Local variables ----------
      complex*16 z3,z4,z43,y,zero,one,two,three,six,r
      complex*16 ttr_mcttrint4_cdln2
c---------- Data constructions ----------
      data zero /(0.d0,0.d0)/, one /(1.d0,0.d0)/, two /(2.d0,0.d0)/,
     &     three /(3.d0,0.d0)/, six /(6.d0,0.d0)/
      z3 = z - e(3)
      z4 = z - e(4)
      z43 = e(4) - e(3)

      if(abs(z3).le.1.d-08)z3 = (1.d-08,0.d0)
      if(abs(z4).le.1.d-08)z4 = (1.d-08,0.d0)

      if( cdabs( z43/z4 ) .gt.2.5d-2 )then
         r = ( six*z4**2 - three*z43*z4 + two*z43**2 ) /
     &        ( six*z43**3 ) 
         y = zero
         if( abs(z4).ge.1.d-08 )
     &        y = cdlog( z4/z3 )*z4**3 / z43**4 
         r = r + y
         res(1) = r
         res(2) = r
         res(3) = r
         r = - three*z3*( two*z4 - z43 ) / ( two*z43**3 ) -
     &        one / z43
         y = three*cdlog( z3/z4 )*z4**2*z3 / z43**4 
         r = r + y
         res(4) = r
      else
         y = z43/z4
         r = -ttr_mcttrint4_cdln2( y,4,25,3 ) / z43
         res(1) = r
         res(2) = r
         res(3) = r
         res(4) = three*( ttr_mcttrint4_cdln2( y,3,25,2 ) + 
     &                    ttr_mcttrint4_cdln2( y,4,25,3 ) ) / z43 
      end if

      end subroutine TTR_mcttrint4_ek3idn
      
	END SUBROUTINE TTR_MCTTRINT4


      complex*16 function ttr_mcttrint4_cdln1( z,nmin,nmax,n )
c|--------------------------------------------------------------|
c|                                                              |
c|                 Evaluation of sum -z**(i-n)/i                |
c|   Inputs:                                                    |
ci    nmin  - lower bound of sum (must be larger zero)          |
ci    nmax  - upper bound of sum (must be larger nmin)          |
ci    n     - n <=nmin )                                        |
ci    z     - variable (must be in the range  | z | < 1 )       |
c|   Outputs:                                                   |
co    cdln1 - magnitude of sum                                  |
c|                                                              |
cr    If  nmin = 1,  n = 0 and  nmax --> inf then               |
cr        cdln1 = ln( 1 - z )                                   |
c|                                                              |
c|--------------------------------------------------------------| 
      implicit none
c---------- Passed variables ----------
      integer nmin,nmax,n
      complex*16 z
c---------- Local variables ----------
      integer i
      complex*16 a,b
c     if( nmin.le.0 )then
c     write(6,10)nmin
c     stop
c     end if
c     if( nmin.ge.nmax )then
c        write(6,20)nmin,nmax
c     stop
c     end if
c     if( cdabs( z ).ge.1.d0 )then
c     write(6,30)cdabs( z )
c     stop
c     end if
c     if( n.gt.nmin )then
c     write(6,40)n,nmin
c     stop
c     end if
      a = dcmplx( 0.d0,0.d0 )
      b = z**( nmin - n )
      do i = nmin,nmax
         a = a + b / dfloat( i )
         b = b*z
      end do
      ttr_mcttrint4_cdln1 = - a
 10   format(' Error in function ttr_mcttrint4_cdln1:',/,
     &     ' lower bound is wrong ',3x,i10 )
 20   format(' Error in function ttr_mcttrint4_cdln1:',/,
     &     ' lower bound',2x,i10,' is larger than upper bound',2x,i10)
 30   format(' Error in function ttr_mcttrint4_cdln1:',/,
     &     ' Wrong value of z',3x,f20.12)
 40   format(' Error in function ttr_mcttrint4_cdln1:',/,
     &     ' value of n',2x,i10,' is larger than lower bound',2x,i10)
      end function ttr_mcttrint4_cdln1

      complex*16 function ttr_mcttrint4_cdln2( z,nmin,nmax,n )
c|--------------------------------------------------------------|
c|                                                              |
c|                 Evaluation of sum  -(-z)**i/i                |
c|   Inputs:                                                    |
ci    nmin  - lower bound of sum (must be larger zero)          |
ci    nmax  - upper bound of sum (must be larger nmin)          |
ci    n     - n <= nmin                                         |
ci    z     - variable (must be in the range  | z | < 1 )       |
c|   Outputs:                                                   |
co    cdln2 - magnitude of sum                                  |
c|                                                              |
cr    If  nmin = 1, n = 0  and nmax --> inf then                |
cr        cdln2 = ln( 1 + z )                                   |
c|                                                              |
c|--------------------------------------------------------------| 
      implicit none
c---------- Passed variables ----------
      integer nmin,nmax,n
      complex*16 z
c---------- Local variables ----------
      integer i
      complex*16 a,b
c     if( nmin.le.0 )then
c     write(6,10)nmin
c     stop
c     end if
c     if( nmin.ge.nmax )then
c     write(6,20)nmin,nmax
c     stop
c     end if
c     if( cdabs( z ).ge.1.d0 )then
c     write(6,30)cdabs( z )
c     stop
c     end if
c     if( n.gt.nmin )then
c     write(6,40)n,nmin
c     stop
c     end if
      a = dcmplx( 0.d0,0.d0 )
      b = z**( nmin - n )
      b = (-1)**nmin*b
      do i = nmin,nmax
         a = a + b / dfloat( i )
         b = -b*z
      end do
      ttr_mcttrint4_cdln2 = - a
 10   format(' Error in function ttr_mcttrint4_cdln2:',/,
     &     ' lower bound is wrong ',3x,i10 )
 20   format(' Error in function ttr_mcttrint4_cdln2:',/,
     &     ' lower bound',2x,i10,' is larger than upper bound',2x,i10)
 30   format(' Error in function ttr_mcttrint4_cdln2:',/,
     &     ' Wrong value of z',3x,f20.12)
 40   format(' Error in function ttr_mcttrint4_cdln2:',/,
     &     ' value of n',2x,i10,' is larger than lower bound',2x,i10)
      end function ttr_mcttrint4_cdln2


      complex*16 function ttr_mcttrint4_cda1( z,e,i,k,m )
c|--------------------------------------------------------------------|
c|                                                                    |
c|       Formula A1 from PRB 29,6,3430 (1984)                         |
c|                                                                    |
c|   Inputs:                                                          |
ci     z - complex energy                                             |
ci     e - eigenvalues                                                |
ci     i - number of corner in which complex weight factor is calcul. |
ci     k - number of unequivalent corner                              |
ci     m - number of corner in which energies are identical           |
c|   Outputs:                                                         |
co     A1  - weight factor                                            |
c|                                                                    |
c|--------------------------------------------------------------------|
      implicit none
c---------- Passed variables ----------
      integer i,k,m
      complex*16 z,e(4)
c---------- Local variables ----------
      complex*16 zi,zk,zm,zkm,zik,zim,
     &           y1,y2,r,one,half,unit,two,three
      complex*16 ttr_mcttrint4_cdln1,ttr_mcttrint4_cdln2
c---------- Data constructions ----------
      data one /(1.d0,0.d0)/, half /(5.d-1,0.d0)/, unit /(1.d0,0.d0)/,
     &     two /(2.d0,0.d0)/, three /(3.d0,0.d0)/
      zi = z - e(i)
      zk = z - e(k)
      zm = z - e(m)
      zkm = e(k) - e(m) 
      zik = e(i) - e(k) 
      zim = e(i) - e(m) 
      r = zim / ( zm * zkm )
      y1 = zim / zm
      if( cdabs( y1 ).gt.2.5d-2 )then 
         y2 = ( two*zi/zim + zk/zkm ) / ( zkm*y1**2 )
         r  = r + y2*( cdlog( unit - y1 ) + y1 + half*y1*y1 
     &        + y1**3/three )
      else
         y2 = three*ttr_mcttrint4_cdln1( y1,4,30,2 ) / zkm
         r  = r - y2
         y2 = ( two + zim/zkm )*ttr_mcttrint4_cdln1( y1,4,30,3 ) / zkm
         r  = r + y2
      end if
      y1 = zik / zk
      if( cdabs( y1 ).gt.2.5d-2 )then 
         y2 = zik / ( zkm**2 * y1**3 )
         r  = r - y2*( cdlog( unit - y1 ) + y1 + 5.d-1*y1*y1 
     &        + y1*y1*y1/three )
      else
         y2 = zik*ttr_mcttrint4_cdln1( y1,4,30,3 ) / ( zkm*zkm )
         r  = r - y2
      end if
      ttr_mcttrint4_cda1 = r
      end function ttr_mcttrint4_cda1





	SUBROUTINE TTR_MCTTRINT5(VINP,VT,Z,A,DA,KEY)
*****************************************************************
*    The calculation of the integrals over tetrahedron          *
*                                                               *
*        /  3     R(k)                                          *
*    A=  \ d k   ------         (key=0 or key=1)                *
*        /       z-V(k)                                         *
*       ttr                                                     *
*                                                               *
*                                                               *
*   dA   /  3      -R(k)                                        *
*   --=  \ d k   ----------     (key=1 only)                    *
*   dz   /        (z-V(k))**2                                   *
*       ttr                                                     *
*                                                               *
*    (For instance V(k)=E(k)-E(k+q)+omega in the linear         *
*                                             response theory.) *
*    where V(k) is given at the vertexes of tetrahedron,        *
*    R(k) is 1,kx,ky,kz (for the matrix elements evaluation)    *
*    The linear interpolation is supposed.                      *
*                                                               *
*    Complex version, adapted from Poteryaev's programs         *
*****************************************************************
	IMPLICIT COMPLEX*16 (A-H,O-Z)
	INTEGER ICON(6),LL(6),MM(6),KK(2),KP(5),MM2(2:4,6),MM3(2:4,3)
     &   ,MM4(2:4,5)
	REAL*8 VT,EPS
	COMPLEX*16 VINP(4),A(4),DA(4),V(4),RES(4),DRES(4),Z,E(4)
	DATA EPS/1.D-05/
	DATA LL/1,1,1,2,2,3/,MM/2,3,4,3,4,4/,KP/4,3,2,0,1/
     &      ,MM2/2,3,4,3,2,4,3,4,2,2,2,4,2,4,2,4,2,2/
     &      ,MM3/2,3,3,3,2,3,3,3,2/
     &      ,MM4/2,2,4,2,4,2,4,2,2,2,2,2,2,2,2/
C   INITIALIZATION
	DO I=1,4 
	V(I)=VINP(I)
	IF(ABS(V(I)).LE.1.D-08)V(I)=1.D-08
	ENDDO
C   DEFINE CASE OF COINCIDING
	DO I=1,4
	DO J=I+1,4
	IF(ABS(V(I)-V(J)).LE.EPS)V(J)=V(I)
	IF(ABS(V(I)).GT.1.D-07.AND.ABS(V(J)).GT.1.D-07)THEN
      IF(ABS(V(I)-V(J))/ABS(V(I)).LE.EPS.AND.
     &   ABS(V(I)-V(J))/ABS(V(J)).LE.EPS)V(J)=V(I)
	ENDIF
	ENDDO
	ENDDO
	ICASE=0
	K=0
	DO I=1,4
	DO J=I+1,4
	K=K+1
	ICON(K)=0
	IF(V(I).EQ.V(J))ICON(K)=1
	ICASE=ICASE+ICON(K)
	ENDDO
	ENDDO
	DO I=1,4
	A(I)=(0.D0,0.D0)
	DA(I)=(0.D0,0.D0)
	ENDDO
C CASE:
	IF(ICASE.EQ.0)THEN !! ALL ELEMENTS ARE DIFFERENT
	E(1)=V(1)
	E(2)=V(2)
	E(3)=V(3)
	E(4)=V(4)
	IF(KEY.EQ.0)THEN ! Integrals
	CALL TTR_MCTTRINT5_AEKNIDN(Z,E,RES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	A(2)=RES(2)
	A(3)=RES(3)
	A(4)=RES(4)
	ELSEIF(KEY.EQ.1)THEN ! Integrals plus derivatives
	CALL TTR_MCTTRINT5_DEKNIDN(Z,V,RES,DRES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	A(2)=RES(2)
	A(3)=RES(3)
	A(4)=RES(4)
	DA(1)=DRES(1)+DRES(2)+DRES(3)+DRES(4)
	DA(2)=DRES(2)
	DA(3)=DRES(3)
	DA(4)=DRES(4)
	ENDIF
	ELSEIF(ICASE.EQ.1)THEN !! TWO ELEMENTS ARE EQUAL
	DO K=1,6
	IF(ICON(K).EQ.1)K0=K
	ENDDO
	E(1)=V(LL(K0))          !! EQUAL ELEMENTS
	E(2)=V(LL(K0))
	E(3)=V(LL(7-K0))
	E(4)=V(MM(7-K0))
	IF(KEY.EQ.0)THEN ! Integrals
	CALL TTR_MCTTRINT5_AEK2IDN(Z,E,RES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	DO I=2,4
	A(I)=RES(MM2(I,K0))
	ENDDO
	ELSEIF(KEY.EQ.1)THEN ! Integrals plus derivatives
	CALL TTR_MCTTRINT5_DEK2IDN(Z,E,RES,DRES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	DA(1)=DRES(1)+DRES(2)+DRES(3)+DRES(4)
	DO I=2,4
	A(I)=RES(MM2(I,K0))
	DA(I)=DRES(MM2(I,K0))
	ENDDO
	ENDIF
	ELSEIF(ICASE.EQ.2)THEN  !! TWO PAIRS ARE EQUAL
	K=0
	DO I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+1
	KK(K)=I
	ENDIF
	ENDDO
	E(1)=V(LL(KK(1)))
	E(2)=V(LL(KK(1)))
	E(3)=V(LL(KK(2)))
	E(4)=V(LL(KK(2)))
	IF(KEY.EQ.0)THEN ! Integrals
	CALL TTR_MCTTRINT5_AEK22IDN(Z,E,RES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	DO I=2,4
	A(I)=RES(MM3(I,KK(1)))
	ENDDO
	ELSEIF(KEY.EQ.1)THEN ! Integrals plus derivatives
	CALL TTR_MCTTRINT5_DEK22IDN(Z,E,RES,DRES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	DA(1)=DRES(1)+DRES(2)+DRES(3)+DRES(4)
	DO I=2,4
	A(I)=RES(MM3(I,KK(1)))
	DA(I)=DRES(MM3(I,KK(1)))
	ENDDO
	ENDIF
	ELSEIF(ICASE.EQ.3)THEN   !! THREE ELEMENTS ARE EQUAL
	K=0
	DO I=1,6
	IF(ICON(I).EQ.1)THEN
	K=K+I
	K0=I
	ENDIF
	ENDDO
	M0=(K-5)/2
	E(1)=V(LL(K0))
	E(2)=V(LL(K0))
	E(3)=V(LL(K0))
	E(4)=V(KP(M0))
	IF(KEY.EQ.0)THEN ! Integrals
	CALL TTR_MCTTRINT5_AEK3IDN(Z,E,RES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	DA(1)=(0.D0,0.D0)
	DO I=2,4
	A(I)=RES(MM4(I,M0))
	DA(I)=(0.D0,0.D0)
	ENDDO
	ELSEIF(KEY.EQ.1)THEN ! Integrals plus derivatives
	CALL TTR_MCTTRINT5_DEK3IDN(Z,E,RES,DRES)
	A(1)=RES(1)+RES(2)+RES(3)+RES(4)
	DA(1)=DRES(1)+DRES(2)+DRES(3)+DRES(4)
	DO I=2,4
	A(I)=RES(MM4(I,M0))
	DA(I)=DRES(MM4(I,M0))
	ENDDO
	ENDIF
	ELSEIF(ICASE.EQ.6)THEN !! ALL ELEMENTS ARE EQUAL
	A(1)=1.D0/(Z-V(1))
	A(2)=0.25D0*A(1)
	A(3)=A(2)
	A(4)=A(3)
	IF(KEY.EQ.1)THEN ! Integrals plus derivatives
	DA(1)=-1.D0/(Z-V(1))**2
	DA(2)=0.25D0*DA(1)
	DA(3)=DA(2)
	DA(4)=DA(3)
	ENDIF
	ENDIF
C  NORMALIZATION
    	DO J=1,4
	A(J)=A(J)*VT
	IF(KEY.EQ.1)
     &DA(J)=DA(J)*VT
	ENDDO

	CONTAINS


      SUBROUTINE TTR_MCTTRINT5_AEKNIDN(Z,E,RES)
***********************************************************************
*     Calculation of complex weight factor for different              *
*     complex eigenvalues                                             *
*     PRB 29,6,3430 (1984); (formulas 7,8)                            *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),E(4)
      INTEGER I,J,K
      REAL*8 REZI,IMZI,REZJ,IMZJ
      COMPLEX*16 R,Y,Y1,T,ZI,ZJ,ZIJ
      COMPLEX*16 TTR_MCTTRINT5_ACDLN1
C START CODE
      DO I=1,4
      R=(0.D0,0.D0)
      DO J=1,4
      IF(J.EQ.I)CYCLE
      ZJ=Z-E(J)
      ZIJ=E(I)-E(J)
      Y=ZIJ/ZJ
      IF(ABS(Y).GT.2.5D-02)THEN
      T=ZIJ
      DO K=1,4
      IF(K.EQ.J)CYCLE
      T=T*(E(K)-E(J))
      ENDDO
      Y1=ZJ**3/T 
      R=R-Y1*(Y+5.D-01*Y*Y+Y*Y*Y/3.D0+LOG(1.D0-Y))
      ELSE
      T=(1.D0,0.D0)
      DO K=1,4
      IF(K.EQ.J.OR.K.EQ.I)CYCLE
      T=T*(E(K)-E(J))
      ENDDO
      Y1=ZIJ/T
      R=R-Y1*TTR_MCTTRINT5_ACDLN1(Y,4,25,3)
      ENDIF 
	ENDDO
      RES(I)=R
	ENDDO
      END SUBROUTINE TTR_MCTTRINT5_AEKNIDN




      SUBROUTINE TTR_MCTTRINT5_DEKNIDN(Z,E,RES,DRES)
***********************************************************************
*     Calculation of complex weight factor for different              *
*     complex eigenvalues                                             *
*     PRB 29,6,3430 (1984); (formulas 7,8)                            *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),DRES(4),E(4)
      INTEGER I,J,K
      REAL*8 REZI,IMZI,REZJ,IMZJ
      COMPLEX*16 R,DR,Y,DY,Y1,DY1,T,ZI,ZJ,ZIJ,T1
      COMPLEX*16 TTR_MCTTRINT5_ACDLN1,TTR_MCTTRINT5_DCDLN1
C START CODE
      DO I=1,4
      R=(0.D0,0.D0)
      DR=(0.D0,0.D0)
      DO J=1,4
      IF(J.EQ.I)CYCLE
      ZJ=Z-E(J)
      ZIJ=E(I)-E(J)
      Y=ZIJ/ZJ
	DY=-Y/ZJ
      IF(ABS(Y).GT.2.5D-02)THEN
      T=ZIJ
      DO K=1,4
      IF(K.EQ.J)CYCLE
      T=T*(E(K)-E(J))
      ENDDO
      Y1=ZJ**3/T 
	DY1=3.D0*ZJ**2/T
	T1=(Y+0.5D0*Y*Y+Y*Y*Y/3.D0+LOG(1.D0-Y))
      R=R-Y1*T1
	DR=DR-DY1*T1-Y1*(DY+Y*DY+Y*Y*DY-DY/(1.D0-Y))
      ELSE
      T=(1.D0,0.D0)
      DO K=1,4
      IF(K.EQ.J.OR.K.EQ.I)CYCLE
      T=T*(E(K)-E(J))
      ENDDO
      Y1=ZIJ/T
      R=R-Y1*TTR_MCTTRINT5_ACDLN1(Y,4,25,3)
	DR=DR-Y1*TTR_MCTTRINT5_DCDLN1(Y,4,25,3)*DY
      ENDIF 
	ENDDO
      RES(I)=R
	DRES(I)=DR
	ENDDO
      END SUBROUTINE TTR_MCTTRINT5_DEKNIDN







      SUBROUTINE TTR_MCTTRINT5_AEK22IDN(Z,E,RES)
***********************************************************************
*        Calculation of complex weight factor in case when            *
*               two pairs of energies are identical                   *
*    Inputs:                                                          *
*      z - complex energy                                             *
*      e - eigenvalues                                                *
*    Outputs:                                                         *
*      res  - weight factor of Green function of one orbital          *
*    Remarks:                                                         *
*      PRB 29,6,3430 (1984); (formula A4)                             *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),E(4)
      COMPLEX*16 Z2,Z3,Z32,Y,R
      COMPLEX*16 TTR_MCTTRINT5_ACDLN1,TTR_MCTTRINT5_ACDLN2
      Z2=Z-E(2)
      Z3=Z-E(3)
      Z32=E(3)-E(2)
      IF(ABS(Z2).LE.1.D-08)Z2=(1.D-08,0.D0)
      IF(ABS(Z3).LE.1.D-08)Z3=(1.D-08,0.D0)
      IF(ABS(Z32/Z3).GT.2.5D-2)THEN
      Y=-3.D0*Z2*(2.D0*Z3-Z32)/(2.D0*Z32**3)-1.D0/Z32
      R=3.D0*LOG(Z2/Z3)*Z3**2*Z2/Z32**4 
      R=R+Y
      RES(1)=R
      RES(2)=R
      ELSE
      Y=Z32/Z3
      R=3.D0*(TTR_MCTTRINT5_ACDLN2(Y,4,25,3)+
     &        TTR_MCTTRINT5_ACDLN2(Y,3,25,2))/Z32 
      RES(1)=R
      RES(2)=R
      ENDIF
      IF(ABS(Z32/Z2).GT.2.5D-2)THEN
      Y=3.D0*Z3*(2.D0*Z2+Z32)/(2.D0*Z32**3)+1.D0/Z32
      R=3.D0*LOG(Z3/Z2)*Z2**2*Z3/Z32**4 
      R=R+Y
      RES(3)=R
      RES(4)=R
      ELSE
      Y=Z32/Z2
      R=3.D0*(TTR_MCTTRINT5_ACDLN1(Y,4,25,3 )-
     &        TTR_MCTTRINT5_ACDLN1(Y,3,25,2 ))/Z32 
      RES(3)=R
      RES(4)=R
      ENDIF
      END SUBROUTINE TTR_MCTTRINT5_AEK22IDN




      SUBROUTINE TTR_MCTTRINT5_DEK22IDN(Z,E,RES,DRES)
***********************************************************************
*        Calculation of complex weight factor in case when            *
*               two pairs of energies are identical                   *
*    Inputs:                                                          *
*      z - complex energy                                             *
*      e - eigenvalues                                                *
*    Outputs:                                                         *
*      res  - weight factor of Green function of one orbital          *
*    Remarks:                                                         *
*      PRB 29,6,3430 (1984); (formula A4)                             *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),DRES(4),E(4)
      COMPLEX*16 Z2,Z3,Z32,Y,DY,R,DR
      COMPLEX*16 TTR_MCTTRINT5_ACDLN1,TTR_MCTTRINT5_ACDLN2,
     &           TTR_MCTTRINT5_DCDLN1,TTR_MCTTRINT5_DCDLN2
      Z2=Z-E(2)
      Z3=Z-E(3)
      Z32=E(3)-E(2)
      IF(ABS(Z2).LE.1.D-08)Z2=(1.D-08,0.D0)
      IF(ABS(Z3).LE.1.D-08)Z3=(1.D-08,0.D0)
      IF(ABS(Z32/Z3).GT.2.5D-2)THEN
      Y=-3.D0*Z2*(2.D0*Z3-Z32)/(2.D0*Z32**3)-1.D0/Z32
	DY=-3.D0*(2.D0*Z3-Z32)/(2.D0*Z32**3)
     &   -3.D0*Z2*2.D0/(2.D0*Z32**3)
      R=3.D0*LOG(Z2/Z3)*Z3**2*Z2/Z32**4 
	DR=3.D0*(1.D0/Z2-1.D0/Z3)*Z3**2*Z2/Z32**4+
     &   3.D0*LOG(Z2/Z3)*2.D0*Z3*Z2/Z32**4+
     &   3.D0*LOG(Z2/Z3)*Z3**2/Z32**4 
      R=R+Y
	DR=DR+DY
      RES(1)=R
      RES(2)=R
	DRES(1)=DR
	DRES(2)=DR
      ELSE
      Y=Z32/Z3
	DY=-Z32/Z3/Z3
      R=3.D0*(TTR_MCTTRINT5_ACDLN2(Y,4,25,3)+
     &        TTR_MCTTRINT5_ACDLN2(Y,3,25,2))/Z32
	DR=3.D0*(TTR_MCTTRINT5_DCDLN2(Y,4,25,3)+
     &         TTR_MCTTRINT5_DCDLN2(Y,3,25,2))/Z32*DY
      RES(1)=R
      RES(2)=R
      DRES(1)=DR
      DRES(2)=DR
      ENDIF
      IF(ABS(Z32/Z2).GT.2.5D-2)THEN
      Y=3.D0*Z3*(2.D0*Z2+Z32)/(2.D0*Z32**3)+1.D0/Z32
      DY=3.D0*(2.D0*Z2+Z32)/(2.D0*Z32**3)
     &  +3.D0*Z3*2.D0/(2.D0*Z32**3)
      R=3.D0*LOG(Z3/Z2)*Z2**2*Z3/Z32**4
	DR=3.D0*(1.D0/Z3-1.D0/Z2)*Z2**2*Z3/Z32**4+
     &   3.D0*LOG(Z3/Z2)*2.D0*Z2*Z3/Z32**4+
     &   3.D0*LOG(Z3/Z2)*Z2**2/Z32**4 
      R=R+Y
      DR=DR+DY
      RES(3)=R
      RES(4)=R
      DRES(3)=DR
      DRES(4)=DR
      ELSE
      Y=Z32/Z2
      DY=-Z32/Z2/Z2
      R=3.D0*(TTR_MCTTRINT5_ACDLN1(Y,4,25,3 )-
     &        TTR_MCTTRINT5_ACDLN1(Y,3,25,2 ))/Z32 
      DR=3.D0*(TTR_MCTTRINT5_DCDLN1(Y,4,25,3 )-
     &         TTR_MCTTRINT5_DCDLN1(Y,3,25,2 ))*DY/Z32 
      RES(3)=R
      RES(4)=R
      DRES(3)=DR
      DRES(4)=DR
      ENDIF
      END SUBROUTINE TTR_MCTTRINT5_DEK22IDN





      SUBROUTINE TTR_MCTTRINT5_AEK2IDN(Z,E,RES)
***********************************************************************
*       Calculation of complex weight factor in case when             *
*               two corner energies are identical                     *
*                                                                     *
*    Inputs:                                                          *
*      z - complex energy                                             *
*      e - eigenvalues                                                *
*    Outputs:                                                         *
*      res  - weight factor of Green function of one orbital          *
*    Remarks:                                                         *
*      PRB 29,6,3430 (1984); (formulas A1,A2)                         *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),E(4)
      COMPLEX*16 ZM,ZI,ZK,ZIK,ZIM,ZKM,Y1,Y2,R
      COMPLEX*16 TTR_MCTTRINT5_ACDA1,TTR_MCTTRINT5_ACDLN2
      ZM=Z-E(1)
      ZI=Z-E(3)
      ZK=Z-E(4)
      ZIK=E(3)-E(4)
      ZIM=E(3)-E(1)
      ZKM=E(4)-E(1)
      Y1=ZIM/ZI
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=LOG(1.D0+Y1)/(Y1**3)-1.D0/(Y1*Y1)+0.5D0/Y1-1.D0/3.D0
      R=-Y2/ZIK
      ELSE
      R=-TTR_MCTTRINT5_ACDLN2(Y1,4,25,3)/ZIK
      ENDIF
      Y1=ZKM/ZK
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=LOG(1.D0+Y1)/(Y1**3)-1.D0/(Y1*Y1)+0.5D0/Y1-1.D0/3.D0
      R=R+Y2/ZIK
      ELSE
      R=R+TTR_MCTTRINT5_ACDLN2(Y1,4,25,3)/ZIK
      ENDIF
      RES(1)=R
      RES(2)=R
      RES(3)=TTR_MCTTRINT5_ACDA1(Z,E,3,4,1)
      RES(4)=TTR_MCTTRINT5_ACDA1(Z,E,4,3,1)
      END SUBROUTINE TTR_MCTTRINT5_AEK2IDN





      SUBROUTINE TTR_MCTTRINT5_DEK2IDN(Z,E,RES,DRES)
***********************************************************************
*       Calculation of complex weight factor in case when             *
*               two corner energies are identical                     *
*                                                                     *
*    Inputs:                                                          *
*      z - complex energy                                             *
*      e - eigenvalues                                                *
*    Outputs:                                                         *
*      res  - weight factor of Green function of one orbital          *
*    Remarks:                                                         *
*      PRB 29,6,3430 (1984); (formulas A1,A2)                         *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),DRES(4),E(4)
      COMPLEX*16 ZM,ZI,ZK,ZIK,ZIM,ZKM,Y1,DY1,Y2,DY2,R,DR
      COMPLEX*16 TTR_MCTTRINT5_ACDA1,TTR_MCTTRINT5_DCDA1
     &          ,TTR_MCTTRINT5_ACDLN2,TTR_MCTTRINT5_DCDLN2
      ZM=Z-E(1)
      ZI=Z-E(3)
      ZK=Z-E(4)
      ZIK=E(3)-E(4)
      ZIM=E(3)-E(1)
      ZKM=E(4)-E(1)
      Y1=ZIM/ZI
      DY1=-Y1/ZI
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=LOG(1.D0+Y1)/(Y1**3)-1.D0/(Y1*Y1)+0.5D0/Y1-1.D0/3.D0
      DY2=DY1/(1.D0+Y1)/(Y1**3)-LOG(1.D0+Y1)*3.D0*DY1/(Y1**4)
     &   +2.D0*DY1/(Y1*Y1*Y1)-0.5D0*DY1/Y1/Y1
      R=-Y2/ZIK
	DR=-DY2/ZIK
      ELSE
      R=-TTR_MCTTRINT5_ACDLN2(Y1,4,25,3)/ZIK
      DR=-TTR_MCTTRINT5_DCDLN2(Y1,4,25,3)*DY1/ZIK
      ENDIF
      Y1=ZKM/ZK
	DY1=-Y1/ZK
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=LOG(1.D0+Y1)/(Y1**3)-1.D0/(Y1*Y1)+0.5D0/Y1-1.D0/3.D0
      DY2=DY1/(1.D0+Y1)/(Y1**3)-LOG(1.D0+Y1)*3.D0*DY1/(Y1**4)
     &   +2.D0*DY1/(Y1*Y1*Y1)-0.5D0*DY1/Y1/Y1
      R=R+Y2/ZIK
      DR=DR+DY2/ZIK
      ELSE
      R=R+TTR_MCTTRINT5_ACDLN2(Y1,4,25,3)/ZIK
      DR=DR+TTR_MCTTRINT5_DCDLN2(Y1,4,25,3)*DY1/ZIK
      ENDIF
      RES(1)=R
      RES(2)=R
      RES(3)=TTR_MCTTRINT5_ACDA1(Z,E,3,4,1)
      RES(4)=TTR_MCTTRINT5_ACDA1(Z,E,4,3,1)
      DRES(1)=DR
      DRES(2)=DR
      DRES(3)=TTR_MCTTRINT5_DCDA1(Z,E,3,4,1)
      DRES(4)=TTR_MCTTRINT5_DCDA1(Z,E,4,3,1)
      END SUBROUTINE TTR_MCTTRINT5_DEK2IDN




      


      SUBROUTINE TTR_MCTTRINT5_AEK3IDN(Z,E,RES)
***********************************************************************
*       Calculation of complex weight factor in case when             *
*              three corner energies are identical                    *
*   Inputs:                                                           *
*     z - complex energy                                              *
*     e - eigenvalues                                                 *
*   Outputs:                                                          *
*     res  - weight factor of Green function of one orbital           *
*   Remarks:                                                          *
*     PRB 29,6,3430 (1984); (formula A3)                              *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),E(4)
      COMPLEX*16 Z3,Z4,Z43,Y,R,TTR_MCTTRINT5_ACDLN2
      Z3=Z-E(3)
      Z4=Z-E(4)
      Z43=E(4)-E(3)
      IF(ABS(Z3).LE.1.D-08)Z3=(1.D-08,0.D0)
      IF(ABS(Z4).LE.1.D-08)Z4=(1.D-08,0.D0)
      IF(ABS(Z43/Z4).GT.2.5D-2)THEN
      R=(6.D0*Z4**2-3.D0*Z43*Z4+2.D0*Z43**2)/(6.D0*Z43**3) 
      Y=(0.D0,0.D0)
      IF(ABS(Z4).GE.1.D-08)
     &Y=LOG(Z4/Z3)*Z4**3/z43**4 
      R=R+Y
      RES(1)=R
      RES(2)=R
      RES(3)=R
      R=-3.D0*Z3*(2.D0*Z4-Z43)/(2.D0*Z43**3)-1.D0/Z43
      Y=3.D0*LOG(Z3/Z4)*Z4**2*Z3/Z43**4 
      R=R+Y
      RES(4)=R
      ELSE
      Y=Z43/Z4
      R=-TTR_MCTTRINT5_ACDLN2(Y,4,25,3)/Z43
      RES(1)=R
      RES(2)=R
      RES(3)=R
      RES(4)=3.D0*(TTR_MCTTRINT5_ACDLN2(Y,3,25,2)+
     &             TTR_MCTTRINT5_ACDLN2(Y,4,25,3))/Z43 
      ENDIF
      END SUBROUTINE TTR_MCTTRINT5_AEK3IDN


      SUBROUTINE TTR_MCTTRINT5_DEK3IDN(Z,E,RES,DRES)
***********************************************************************
*       Calculation of complex weight factor in case when             *
*              three corner energies are identical                    *
*   Inputs:                                                           *
*     z - complex energy                                              *
*     e - eigenvalues                                                 *
*   Outputs:                                                          *
*     res  - weight factor of Green function of one orbital           *
*   Remarks:                                                          *
*     PRB 29,6,3430 (1984); (formula A3)                              *
***********************************************************************
      IMPLICIT NONE
      COMPLEX*16 Z,RES(4),DRES(4),E(4)
      COMPLEX*16 Z3,Z4,Z43,Y,DY,R,DR
      COMPLEX*16 TTR_MCTTRINT5_ACDLN2,TTR_MCTTRINT5_DCDLN2
      Z3=Z-E(3)
      Z4=Z-E(4)
      Z43=E(4)-E(3)
      IF(ABS(Z3).LE.1.D-08)Z3=(1.D-08,0.D0)
      IF(ABS(Z4).LE.1.D-08)Z4=(1.D-08,0.D0)
      IF(ABS(Z43/Z4).GT.2.5D-2)THEN
      R=(6.D0*Z4**2-3.D0*Z43*Z4+2.D0*Z43**2)/(6.D0*Z43**3) 
      DR=(6.D0*2.D0*Z4-3.D0*Z43)/(6.D0*Z43**3) 
      Y=(0.D0,0.D0)
      DY=(0.D0,0.D0)
      IF(ABS(Z4).GE.1.D-08)
     &Y=LOG(Z4/Z3)*Z4**3/Z43**4 
      IF(ABS(Z4).GE.1.D-08)
     &DY=(1.D0/Z4-1.D0/Z3)*Z4**3/Z43**4+
     &   LOG(Z4/Z3)*3.D0*Z4**2/Z43**4 
      R=R+Y
      DR=DR+DY
      RES(1)=R
      RES(2)=R
      RES(3)=R
      DRES(1)=DR
      DRES(2)=DR
      DRES(3)=DR
      R=-3.D0*Z3*(2.D0*Z4-Z43)/(2.D0*Z43**3)-1.D0/Z43
      DR=-3.D0*(2.D0*Z4-Z43)/(2.D0*Z43**3)
     &   -3.D0*Z3*2.D0/(2.D0*Z43**3)
      Y=3.D0*LOG(Z3/Z4)*Z4**2*Z3/Z43**4 
      DY=3.D0*(1.D0/Z3-1.D0/Z4)*Z4**2*Z3/Z43**4+
     &   3.D0*LOG(Z3/Z4)*2.D0*Z4*Z3/Z43**4+
     &   3.D0*LOG(Z3/Z4)*Z4**2/Z43**4 
      R=R+Y
      DR=DR+DY
      RES(4)=R
      DRES(4)=DR
      ELSE
      Y=Z43/Z4
      DY=-Z43/Z4/Z4
      R=-TTR_MCTTRINT5_ACDLN2(Y,4,25,3)/Z43
      DR=-TTR_MCTTRINT5_DCDLN2(Y,4,25,3)*DY/Z43
      RES(1)=R
      RES(2)=R
      RES(3)=R
      RES(4)=3.D0*(TTR_MCTTRINT5_ACDLN2(Y,3,25,2)+
     &             TTR_MCTTRINT5_ACDLN2(Y,4,25,3))/Z43 
      DRES(1)=DR
      DRES(2)=DR
      DRES(3)=DR
      DRES(4)=3.D0*(TTR_MCTTRINT5_DCDLN2(Y,3,25,2)+
     &              TTR_MCTTRINT5_DCDLN2(Y,4,25,3))/Z43*DY 
      ENDIF
      END SUBROUTINE TTR_MCTTRINT5_DEK3IDN
	END SUBROUTINE TTR_MCTTRINT5






      COMPLEX*16 FUNCTION TTR_MCTTRINT5_ACDLN1(Z,NMIN,NMAX,N)
*****************************************************************
*                 Evaluation of sum -z**(i-n)/i                 *
*   Inputs:                                                     *
*    nmin  - lower bound of sum (must be larger zero)           *
*    nmax  - upper bound of sum (must be larger nmin)           *
*    n     - n <=nmin )                                         *
*    z     - variable (must be in the range  | z | < 1 )        *
*   Outputs:                                                    *
*    cdln1 - magnitude of sum                                   *
*                                                               *
*    If  nmin = 1,  n = 0 and  nmax --> inf then                *
*        cdln1 = ln( 1 - z )                                    *
*****************************************************************
      IMPLICIT NONE
      INTEGER NMIN,NMAX,N,I
      COMPLEX*16 A,B,Z
      A=(0.D0,0.D0)
      B=Z**(NMIN-N)
      DO I=NMIN,NMAX
      A=A+B/DBLE(I)
      B=B*Z
      ENDDO
      TTR_MCTTRINT5_ACDLN1=-A
      END FUNCTION TTR_MCTTRINT5_ACDLN1




      COMPLEX*16 FUNCTION TTR_MCTTRINT5_DCDLN1(Z,NMIN,NMAX,N)
*****************************************************************
*         Evaluation of sum d/dz(-z**(i-n)/i)                   *
*   Inputs:                                                     *
*    nmin  - lower bound of sum (must be larger zero)           *
*    nmax  - upper bound of sum (must be larger nmin)           *
*    n     - n <=nmin )                                         *
*    z     - variable (must be in the range  | z | < 1 )        *
*   Outputs:                                                    *
*    cdln1 - magnitude of sum                                   *
*                                                               *
*    If  nmin = 1,  n = 0 and  nmax --> inf then                *
*        cdln1 = ln( 1 - z )                                    *
*****************************************************************
      IMPLICIT NONE
      INTEGER NMIN,NMAX,N,I
      COMPLEX*16 A,DA,B,DB,Z,TTR_MCTTRINT5_ACDLN1
C     A=(0.D0,0.D0)
      DA=(0.D0,0.D0)
C     B=Z**(NMIN-N)
	DB=Z**(NMIN-N-1)
      DO I=NMIN,NMAX
C     A=A+B/DBLE(I)
	DA=DA+DB*DBLE(I-N)/DBLE(I)
C     B=B*Z
	DB=DB*Z
      ENDDO
C     TTR_MCTTRINT5_ACDLN1=-A
      TTR_MCTTRINT5_DCDLN1=-DA
      END FUNCTION TTR_MCTTRINT5_DCDLN1




      COMPLEX*16 FUNCTION TTR_MCTTRINT5_ACDLN2(Z,NMIN,NMAX,N)
*****************************************************************
*             Evaluation of sum  -(-z)**i/i                     *
*    Inputs:                                                    *
*     nmin  - lower bound of sum (must be larger zero)          *
*     nmax  - upper bound of sum (must be larger nmin)          *
*     n     - n <= nmin                                         *
*     z     - variable (must be in the range  | z | < 1 )       *
*    Outputs:                                                   *
*     cdln2 - magnitude of sum                                  *
*                                                               *
*     If  nmin = 1, n = 0  and nmax --> inf then                *
*         cdln2 = ln( 1 + z )                                   *
*****************************************************************
      IMPLICIT NONE
      INTEGER NMIN,NMAX,N,I
      COMPLEX*16 A,B,Z
      A=(0.D0,0.D0)
      B=Z**(NMIN-N)
      B=(-1)**NMIN*B
      DO I=NMIN,NMAX
      A=A+B/DBLE(I)
      B=-B*Z
      ENDDO
      TTR_MCTTRINT5_ACDLN2=-A
      END FUNCTION TTR_MCTTRINT5_ACDLN2


      COMPLEX*16 FUNCTION TTR_MCTTRINT5_DCDLN2(Z,NMIN,NMAX,N)
*****************************************************************
*        Evaluation of sum d/dz[-(-z)**i/i]                     *
*    Inputs:                                                    *
*     nmin  - lower bound of sum (must be larger zero)          *
*     nmax  - upper bound of sum (must be larger nmin)          *
*     n     - n <= nmin                                         *
*     z     - variable (must be in the range  | z | < 1 )       *
*    Outputs:                                                   *
*     cdln2 - magnitude of sum                                  *
*                                                               *
*     If  nmin = 1, n = 0  and nmax --> inf then                *
*         cdln2 = ln( 1 + z )                                   *
*****************************************************************
      IMPLICIT NONE
      INTEGER NMIN,NMAX,N,I
      COMPLEX*16 A,DA,B,DB,Z
      COMPLEX*16 TTR_MCTTRINT5_ACDLN2
C     A=(0.D0,0.D0)
      DA=(0.D0,0.D0)
C     B=Z**(NMIN-N)
	DB=Z**(NMIN-N-1)
C     B=(-1)**NMIN*B
	DB=(-1)**NMIN*DB
      DO I=NMIN,NMAX
C     A=A+B/DBLE(I)
      DA=DA+DB*DBLE(I-N)/DBLE(I)
C     B=-B*Z
      DB=-DB*Z
      ENDDO
C     TTR_MCTTRINT5_ACDLN2=-A
      TTR_MCTTRINT5_DCDLN2=-DA
      END FUNCTION TTR_MCTTRINT5_DCDLN2



      COMPLEX*16 FUNCTION TTR_MCTTRINT5_ACDA1(Z,E,I,K,M)
***********************************************************************
*       Formula A1 from PRB 29,6,3430 (1984)                          *
*    Inputs:                                                          *
*      z - complex energy                                             *
*      e - eigenvalues                                                *
*      i - number of corner in which complex weight factor is calcul. *
*      k - number of unequivalent corner                              *
*      m - number of corner in which energies are identical           *
*    Outputs:                                                         *
*      A1  - weight factor                                            *
***********************************************************************
      IMPLICIT NONE
      INTEGER I,K,M
      COMPLEX*16 Z,E(4)
      COMPLEX*16 ZI,ZK,ZM,ZKM,ZIK,ZIM,Y1,Y2,R
      COMPLEX*16 TTR_MCTTRINT5_ACDLN1,TTR_MCTTRINT5_ACDLN2
      ZI=Z-E(I)
      ZK=Z-E(K)
      ZM=Z-E(M)
      ZKM=E(K)-E(M) 
      ZIK=E(I)-E(K) 
      ZIM=E(I)-E(M) 
      R=ZIM/(ZM*ZKM)
      Y1=ZIM/ZM
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=(2.D0*ZI/ZIM+ZK/ZKM)/(ZKM*Y1**2)
      R=R+Y2*(LOG(1.D0-Y1)+Y1+0.5D0*Y1*Y1+Y1**3/3.D0)
      ELSE
      Y2=3.D0*TTR_MCTTRINT5_ACDLN1(Y1,4,30,2)/ZKM
      R=R-Y2
      Y2=(2.D0+ZIM/ZKM)*TTR_MCTTRINT5_ACDLN1(Y1,4,30,3)/ZKM
      R=R+Y2
      ENDIF
      Y1=ZIK/ZK
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=ZIK/(ZKM**2*Y1**3)
      R=R-Y2*(LOG(1.D0-Y1)+Y1+0.5D0*Y1*Y1+Y1**3/3.D0)
      ELSE
      Y2=ZIK*TTR_MCTTRINT5_ACDLN1(Y1,4,30,3)/(ZKM*ZKM)
      R=R-Y2
      ENDIF
      TTR_MCTTRINT5_ACDA1=R
      END FUNCTION TTR_MCTTRINT5_ACDA1



      COMPLEX*16 FUNCTION TTR_MCTTRINT5_DCDA1(Z,E,I,K,M)
***********************************************************************
*       Formula A1 from PRB 29,6,3430 (1984)                          *
*    Inputs:                                                          *
*      z - complex energy                                             *
*      e - eigenvalues                                                *
*      i - number of corner in which complex weight factor is calcul. *
*      k - number of unequivalent corner                              *
*      m - number of corner in which energies are identical           *
*    Outputs:                                                         *
*      A1  - weight factor                                            *
***********************************************************************
      IMPLICIT NONE
      INTEGER I,K,M
      COMPLEX*16 Z,E(4)
      COMPLEX*16 ZI,ZK,ZM,ZKM,ZIK,ZIM,T1,Y1,Y2,DY1,DY2,R,DR
      COMPLEX*16 TTR_MCTTRINT5_ACDLN1,TTR_MCTTRINT5_ACDLN2
     &          ,TTR_MCTTRINT5_DCDLN1,TTR_MCTTRINT5_DCDLN2
     &          ,TTR_MCTTRINT5_ACDA1
      ZI=Z-E(I)
      ZK=Z-E(K)
      ZM=Z-E(M)
      ZKM=E(K)-E(M) 
      ZIK=E(I)-E(K) 
      ZIM=E(I)-E(M) 
      R=ZIM/(ZM*ZKM)
	DR=-R/ZM
      Y1=ZIM/ZM
      DY1=-Y1/ZM
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=(2.D0*ZI/ZIM+ZK/ZKM)/(ZKM*Y1**2)
      DY2=(2.D0/ZIM+1.D0/ZKM)/(ZKM*Y1**2)-
     &2.D0*(2.D0*ZI/ZIM+ZK/ZKM)/(ZKM*Y1**3)*DY1
	T1=(LOG(1.D0-Y1)+Y1+0.5D0*Y1*Y1+Y1**3/3.D0)
      R=R+Y2*T1
      DR=DR+DY2*T1+Y2*(-DY1/(1.D0-Y1)+DY1+Y1*DY1+Y1**2*DY1)
      ELSE
      Y2=3.D0*TTR_MCTTRINT5_ACDLN1(Y1,4,30,2)/ZKM
      DY2=3.D0*TTR_MCTTRINT5_DCDLN1(Y1,4,30,2)*DY1/ZKM
      R=R-Y2
      DR=DR-DY2
      Y2=(2.D0+ZIM/ZKM)*TTR_MCTTRINT5_ACDLN1(Y1,4,30,3)/ZKM
      DY2=(2.D0+ZIM/ZKM)*TTR_MCTTRINT5_DCDLN1(Y1,4,30,3)*DY1/ZKM
      R=R+Y2
      DR=DR+DY2
      ENDIF
      Y1=ZIK/ZK
      DY1=-Y1/ZK
      IF(ABS(Y1).GT.2.5D-2)THEN
      Y2=ZIK/(ZKM**2*Y1**3)
      DY2=-3.D0*DY1*ZIK/(ZKM**2*Y1**4)
	T1=(LOG(1.D0-Y1)+Y1+0.5D0*Y1*Y1+Y1**3/3.D0)
      R=R-Y2*T1
      DR=DR-DY2*T1-Y2*(-DY1/(1.D0-Y1)+DY1+Y1*DY1+Y1**2*DY1)
      ELSE
      Y2=ZIK*TTR_MCTTRINT5_ACDLN1(Y1,4,30,3)/(ZKM*ZKM)
      DY2=ZIK*TTR_MCTTRINT5_DCDLN1(Y1,4,30,3)*DY1/(ZKM*ZKM)
      R=R-Y2
      DR=DR-DY2
      ENDIF
      TTR_MCTTRINT5_ACDA1=R
      TTR_MCTTRINT5_DCDA1=DR
      END FUNCTION TTR_MCTTRINT5_DCDA1



	SUBROUTINE TTR_MCTTRINT6(VINP,VT,Z,A,DA,KEY)
*****************************************************************
*    The calculation of the integrals over tetrahedron          *
*                                                               *
*        /  3     R(k)                                          *
*    A=  \ d k   ------                                         *
*        /       V(k)+id                                        *
*       ttr                                                     *
*                                                               *
*                                                               *
*    (For instance V(k)=E(k)-E(k+q)+omega in the linear         *
*                                             response theory.) *
*    where V(k) is given at the vertexes of tetrahedron,        *
*    R(k) is 1,kx,ky,kz (for the matrix elements evaluation)    *
*    The linear interpolation is supposed.                      *
*                                                               *
*    Complex version, returns both real and imaginary parts     *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 VINP(4),Z,A(4),DA(4)
	REAL*8 VINPRE(4),ARE(4),AIM(4),AT(4),AD(4)
	DATA PI/3.141592653589793D0/
C Assign to real numbers and take real part of the integral
      DO I=1,4
      VINPRE(I)=DREAL(Z-VINP(I))
      ENDDO
      CALL TTR_MCTTRINT1(VINPRE,VT,ARE)
C Now take imaginary part of the integral
C (Do not use Bloechl correction, it may result in negative DOS!)
      CALL TTR_MCTTRINT0(VINPRE,0.D0,VT,AT,AD,TS,DS,'T','W')
      AIM(1)=AD(1)+AD(2)+AD(3)+AD(4)
      AIM(2)=AD(2)
      AIM(3)=AD(3)
      AIM(4)=AD(4)
C Returning array
      DO I=1,4
      A(I)=DCMPLX(ARE(I),-PI*AIM(I))
      DA(I)=(0.D0,0.D0)
      ENDDO
      END