from scipy import *
###
def flatten(a):
    """Flatten a list."""
    return bounce(flatten_k(a, lambda x: x))


def bounce(thing):
    """Bounce the 'thing' until it stops being a callable."""
    while callable(thing):
        thing = thing()
    return thing


def flatten_k(a, k):
    """CPS/trampolined version of the flatten function.  The original
    function, before the CPS transform, looked like this:

    def flatten(a):
        if not isinstance(a,(tuple,list)): return [a]
        if len(a)==0: return []
        return flatten(a[0])+flatten(a[1:])

    The following code is not meant for human consumption.
    """
    if not isinstance(a,(tuple,list)):
        return lambda: k([a])
    if len(a)==0:
        return lambda: k([])
    def k1(v1):
        def k2(v2):
            return lambda: k(v1 + v2)
        return lambda: flatten_k(a[1:], k2)
    return lambda: flatten_k(a[0], k1)
###

def ReadFirstLine(filename):
    "Read first line of Ham or overlap to get dimensions"
    fh = open(filename,'r')
    first_line = fh.readline()
    fh.close()
    field = first_line.split()
    return [int(x) for x in first_line.split()]


def union(data):
    " Takes a union of array or list"
    c = []
    for d in data:
        if d not in c:
            c.append(d)
    return c

def repeat(data, fields):
    """ gives number of occurances of field[i] in data.
    If data is array of energies than this gives degeneracy of
    each field"""
    ldata = data.tolist()
    deg=[]
    for f in fields:
        deg.append(ldata.count(f))
    return deg    

def read_data(file, Number=10000):
    """reads a list of size Number of fortran data 
    Input:
          file   -- file object which was opened with open command
          Number -- maximum number of data to read
    """
    res=[]
    for line in file:
        res.extend(line.split())
        if len(res)>=Number : break
    return res

def read_type(file, rtype, Number=10000):
    """reads a list of fortran data of specified type (rtype)
    Input:
          file   -- file object which was opened with open command
          rtpe   -- type of data
          Number -- maximum number of data to read
    """
    return [rtype(x) for x in read_data(file, Number)]

import re
def read_complex(file, Number=10000):
    """reads a list of fortran data of complex type
    Input:
          file   -- file object which was opened with open command
          Number -- maximum number of complex numbers to read
    """
    res=[]
    for line in file:
        cline = line.split()
        for number in cline:
            m = re.match('\((.*)\,(.*)\)', number)
            if m is not None:
                z = float(m.group(1))+float(m.group(2))*1j
                res.append(z)

        if len(res)>=Number : break
    return res

def fort_bool(ch):
    "this function converts fortran boolean 'T'/'F' into python True/False"
    if (ch=='T'): return True
    else: return False
    
def data_reshape(data, shapes):
    vdata=[]
    ofset=0
    for shape in shapes:
        dim = reduce(lambda a,b: a*b, shape)
        vdata.append(reshape(data[ofset:ofset+dim], shape))
        ofset += dim
    return vdata

def total_dimension(shapes):
    ofset=0
    for shape in shapes:
        dim = reduce(lambda a,b: a*b, shape)
        ofset += dim
    return ofset

def smix(A, B, admix):
    """ Mixes vaious arrays arrays of type
    C[n][:,....] = A[n][:,...]*(1-admix) + B[n][:,...]*admix
        Examples of use are:
           rho_tot[natom][nrad_isort+1, lmsym_iatom, nspin, norbs]    -- total_charge
           rho_cor[nsort][nrad_isort+1, nspin]                        -- core charge
           pot_frz0[nsort][nrad_isort+1,nspin]                        -- KS-potential
    """
    n0 = len(A)
    C = []
    for i in range(n0):
        sh = shape(A[i])
        C.append(A[i]*(1-admix) + B[i]*admix)
    return C

def read_next(f, size, cnt=[]):
    "Read file until long enough list is found (of size size)."
    while(len(cnt)<size):
        line = re.sub(r'(.*)\#.*', r"\1", f.next().strip()) # get line and remove comments
        cnt = line.split() # make a list of data
    return cnt
