	SUBROUTINE HUB_READHUB
*******************************************************
*  Reads HUBFILE which contains Slater integrals      *
*  necessary for constructing U-matrix U(m' k'; m k)  *
*  in the LDA+U,LDA+DMFT methods.                     *
*  Also reads occupation numbers for correlated       *
*  states if avaiable.                                *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_CHANNELS,ONLY : IUINP                           !!! from channels
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBDATA(:)
      CHARACTER STRING*150
      LOGICAL EXIST,MODIFIED
      DATA PI/3.141592653589793D0/
	IDMF=0
	IF(ICHUB.LE.0)RETURN
C CHECK EXISTENCE
      INQUIRE(FILE=HUBFILE,EXIST=EXIST)
	IF(.NOT.EXIST)CALL FULLSTOP('Hubfile not found!')
C ********************************
C UNDERSTAND THE LENGHT OF HUBFILE
      CALL FILE_LENGHT(HUBFILE,NLEN,IUINP)
      ALLOCATE(HUBDATA(NLEN))
C ********************************
C READ HUBFILE TO INTERNAL FILE
      OPEN(UNIT=IUINP,FILE=HUBFILE,STATUS='OLD',FORM='FORMATTED')
      DO ILEN=1,NLEN
      READ(IUINP,FMT='(A150)')HUBDATA(ILEN)
      ENDDO
      CLOSE(IUINP)
C ********************************
C UNDERSTAND INPUT: CLASSIC or MODERN
      STRING=HUBDATA(1)
      CALL LIB_CONVLOW(STRING,150)
      MODERN=0
      DO IPOS=1,80
      IF(STRING(IPOS:IPOS+12).EQ.'input=classic')MODERN=1
      IF(STRING(IPOS:IPOS+11).EQ.'input=modern' )MODERN=2
      IF(MODERN.NE.0)EXIT
      ENDDO
      IF(MODERN.EQ.0)ISTART=1
      IF(MODERN.EQ.1)ISTART=2
      IF(MODERN.EQ.0)MODERN=1
      IF(MODERN.EQ.1)MODIFIED=.FALSE.
      IF(MODERN.EQ.2)MODIFIED=.TRUE.
C NO FURTHER SUPPORT FOR "CLASSIC"
      IF(MODERN.EQ.1)
     &CALL FULLSTOP('Classic input not supported in HUBFILE')
C READ CONTROL DATA
	CALL HUB_READHUB_CTRL(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ DMFT SETTINGS
	CALL HUB_READHUB_DMFT(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ DMFT SETTINGS
	CALL HUB_READHUB_ITER(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ GLOC SETTINGS
	CALL HUB_READHUB_GLOC(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ CORRELATED ORBITALS DATA
	CALL HUB_READHUB_CORR(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ OCCUPATION NUMBERS, POTENTIAL
	CALL HUB_READHUB_DATA(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ CUSTOM TRANSFORMATION MATRICES
	CALL HUB_READHUB_CSYS(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ RATIONAL INTERPOLATION FOR SELF-ENERGY AND HYBRDIZATION
	IF(IDMF.EQ.3.OR.IDMF.EQ.4.OR.IDMF.EQ.5.OR.IDMF.EQ.6)
     &CALL HUB_READHUB_RATP(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ SELF-ENERGY, HYBRIDIZATION ON IMAGINARY AXIS
	IF(IDMF.EQ.3)
     &CALL HUB_READHUB_SIGW(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ SELF-ENERGY, HYBRIDIZATION ON REAL AXIS
	IF(IDMF.EQ.5)
     &CALL HUB_READHUB_SIGR(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C ********************************************************
C HUBFILE HAS BEEN READ, REFORMAT IT
      IF(MODIFIED)CALL READ_OVWR(HUBFILE,HUBDATA,NLEN)
C WRITE OCCUPANCIES WITH THE NEW COORDINATES
	CALL HUB_WRITEHUB(1)
C MEMORY DEALLOCATIONS
	DEALLOCATE(HUBDATA)
C ERROR HANDLINGS
      END SUBROUTINE HUB_READHUB






	SUBROUTINE HUB_READHUB_CTRL(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads control                                       *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                          !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                                  !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE                !!! from readhub
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL                       !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      LOGICAL DEF,ASK,VAL,NTH,MDF
      DATA PI/3.141592653589793D0/
C **************************************
C READING THE DATA: SELECT CONTROLS 
C  SCHM='LDA       ' - simple LDA
C       'LDA+C     ' - constrained LDA, potential must be given.
C       'LDA+U     ' - LDA+U scheme
C       'IMA-DMFT  ' - LDA+DMFT scheme with self-energy at imaginary axis
C       'RAT-DMFT  ' - LDA+DMFT with Self-energy as rational interpolation
C       'REA-DMFT  ' - LDA+DMFT scheme with self-energy at real axis
C       'QUP-DMFT  ' - LDA+DMFT scheme with quasiparticle approximation
C  RYEV='Ry' or 'eV' - input/output units
C  CUBC='Cubic' or 'Spherical' harmonics
C  REIM='Real' or 'Complex' - input/output
C MODERN INPUT IS USED, DEFAULTS ARE AVAIABLE
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=ctrl',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! CONTROL PARAMETERS:                   '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process SCHM
      TCTRL='LDA'
      HELP='! LDA+U/LDA+C/IMA-DMFT/RAT-DMFT/REA-DMFT'
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'scheme',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Transform to integer keys
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	IDMF=-1
	IF(TCTRL(:NB-1).EQ.'lda'     )IDMF=0
	IF(TCTRL(:NB-1).EQ.'lda+c'   )IDMF=1
	IF(TCTRL(:NB-1).EQ.'lda+u'   )IDMF=2
C Old keys	
	IF(TCTRL(:NB-1).EQ.'lda+ima' )IDMF=3
	IF(TCTRL(:NB-1).EQ.'lda+rat' )IDMF=4
	IF(TCTRL(:NB-1).EQ.'lda+rea' )IDMF=5
	IF(TCTRL(:NB-1).EQ.'lda+qup' )IDMF=6
C New keys	
	IF(TCTRL(:NB-1).EQ.'ima-dmft')IDMF=3
	IF(TCTRL(:NB-1).EQ.'rat-dmft')IDMF=4
	IF(TCTRL(:NB-1).EQ.'rea-dmft')IDMF=5
	IF(TCTRL(:NB-1).EQ.'qup-dmft')IDMF=6
	IF(IDMF.EQ.-1)
     &CALL FULLSTOP('Wrong <<scheme>> keyword in HUBFILE')
C Process CUBC
      TCTRL='Spherical '
      HELP='! Cubic/Spherical harmonics (output)    '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'yharm',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	ICUBC=0
	IF(TCTRL(:NB-1).EQ.'cubic')ICUBC=1
	IF(TCTRL(:NB-1).EQ.'spherical')ICUBC=2
	IF(TCTRL(:NB-1).EQ.'spinor')ICUBC=3
	IF(ICUBC.EQ.0)
     &CALL FULLSTOP('Wrong <<Yharm>> keyword in HUBFILE')
C Process CUBI
      TCTRL=TCTRL
      HELP='! Cubic/Spherical harmonics (input)     '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'iharm',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	ICUBI=0
	IF(TCTRL(:NB-1).EQ.'cubic')ICUBI=1
	IF(TCTRL(:NB-1).EQ.'spherical')ICUBI=2
	IF(TCTRL(:NB-1).EQ.'spinor')ICUBI=3
	IF(ICUBI.EQ.0)
     &CALL FULLSTOP('Wrong <<Iharm>> keyword in HUBFILE')
C Process NHUB
      NHUB=0
      HELP='! # of correlated states                '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ncrl',4,NHUB,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process NROR
      IF(NORBS.EQ.1)TCTRL='One       '
      IF(NORBS.EQ.2)TCTRL='Both      '
      HELP='! One/Both orbits to read               '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'rorbs',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	NROR=0
	IF(TCTRL(:NB-1).EQ.'one')NROR=1
	IF(TCTRL(:NB-1).EQ.'both')NROR=2
	IF(NROR.EQ.0)
     &CALL FULLSTOP('Wrong <<Rorbs>> keyword in HUBFILE')
C Process NRSP
      IF(NSPIN.EQ.1)TCTRL='One       '
      IF(NSPIN.EQ.2)TCTRL='Both      '
      HELP='! One/Both spins  to read               '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'rspin',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	NRSP=0
	IF(TCTRL(:NB-1).EQ.'one')NRSP=1
	IF(TCTRL(:NB-1).EQ.'both')NRSP=2
	IF(NRSP.EQ.0)
     &CALL FULLSTOP('Wrong <<Rspin>> keyword in HUBFILE')
C Process REIM
      TCTRL='Complex   '
      HELP='! Real/Complex input/output             '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'format',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL LIB_CONVLOW(TCTRL,10)
	CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	IRE=0
	IF(TCTRL(:NB-1).EQ.'real')IRE=1
	IF(TCTRL(:NB-1).EQ.'complex')IRE=2
	IF(IRE.EQ.0)
     &CALL FULLSTOP('Wrong <<format>> keyword in HUBFILE')
C **************************************
      CALL READ_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'CTRL')
      DEALLOCATE(HUBCOPY)
	END SUBROUTINE HUB_READHUB_CTRL








	SUBROUTINE HUB_READHUB_DMFT(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads DMFT settings                                 *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
	USE COM_STREAM  ,ONLY : ISTREAM
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                          !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                                  !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE                !!! from readhub
	USE COM_MSBMESH ,ONLY : TEMP,OBND,KGRD
     &                       ,NMSL,WMSL,FMSL,DMSL,SMSL
     &                       ,NMSB,WMSB,FMSB,DMSB,SMSB
     &                       ,NOMG,WOMG,FOMG,DOMG,WMIN,WMAX
     &                       ,NMSC,WMSC,FMSC,DMSC
      USE COM_FREQU   ,ONLY : NOMGGRF,OMGGRF                          !!! from readini
      USE COM_LMTOUT  ,ONLY : SCRFILE,ICGRF,ICODF,ICHYB               !!! from readini
	USE COM_FERMI   ,ONLY : EF,ZBND,KFRM                            !!! from ttrints
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C **************************************
C READING THE DATA: SELECT DMFT SETTINGS
C NMSB - # of Matsubara energies
C TEMP - # Temperature in K
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dmft',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! DMFT SETTINGS                         '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process KGRD
      KGRD=2 ! default is imaginary axis
      IF(IDMF.EQ.3)KGRD=2
      IF(IDMF.EQ.4)KGRD=2
      IF(IDMF.EQ.5)KGRD=1
      IF(IDMF.EQ.6)KGRD=2
      HELP='! set current grid to real/imag         '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'grid',4,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	IF(TCTRL(1:4).EQ.'auto')KGRD=KGRD
	IF(TCTRL(1:4).EQ.'real')KGRD=1
	IF(TCTRL(1:4).EQ.'imag')KGRD=2
      IF(IDMF.EQ.3.AND.KGRD.NE.2)THEN
      KGRD=2
      WRITE(IUT,*)'Message from <READHUB>::',
     &'Current grid set to IMAG for IMA-DMFT mode'
      ENDIF
      IF(IDMF.EQ.5.AND.KGRD.NE.1)THEN
      KGRD=1
      WRITE(IUT,*)'Message from <READHUB>::',
     &'Current grid set to REAL for REA-DMFT mode'
      ENDIF
C Process NMSB
      NMSB=10
      HELP='! # of Matsubara frequencies            '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmsb',4,NMSB,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process OBND
      OBND=1.D0
      HELP='! Effective bandwidth for the grid      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'obnd',4,IN,1,0,OBND,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process NMSL
      NMSL=1024
      HELP='! # of linear Matsubara frequencies     '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmsl',4,NMSL,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process NOMG
      NOMG=50
      HELP='! # of frequency points on the real axis'
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nomg',4,NOMG,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      IF(MOD(NOMG,2).NE.0)
     &CALL FULLSTOP('<READHUB>: NOMG (frequency points) must be even')
C Process WMIN
      WMIN=-2.D0
      HELP='! Leftmost real axis frequency          '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'wmin',4,IN,1,0,WMIN,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process WMAX
      WMAX=+2.D0
      HELP='! Rightmost real axis frequency         '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'wmax',4,IN,1,0,WMAX,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process temperature
      TEMP=TFERMI
      HELP='! Temeprature of the calculation        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'temp',4,IN,1,0,TEMP,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      IF(ABS(TEMP-TFERMI).GT.1.D-07)
     &CALL FULLSTOP('<HUBFILE> :: Temperature mismatch')
	IF(ABS(TEMP).LE.1.D-10.AND.KGRD.EQ.2.AND.IDMF.GE.3)
     &CALL FULLSTOP('T should not be 0 if grid=IMAG')
	TEV=TEMP*13.6058D0 ! eV
	TK=TEV*11605.D0    ! K
C Process Fermi energy
      EFRM=EF ! not to overwrite the value defined above
      HELP='! Fermi energy (Ry)                     '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'chem',4,IN,1,0,EFRM,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	EF=EFRM
C Process Fermi Search options
      TCTRL='auto      '
      HELP='! Fermi search options                  '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'kfrm',4,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	KFRM=-1
	IF(TCTRL(1:4).EQ.'auto')KFRM=0
	IF(TCTRL(1:4).EQ.'find')KFRM=0
	IF(TCTRL(1:4).EQ.'band')KFRM=1
	IF(TCTRL(1:4).EQ.'keep')KFRM=2
	IF(KFRM.EQ.-1)
     &CALL FULLSTOP('Keyword KFERMI not recognized in HUBFILE')
	IF((KFRM.EQ.1.AND.IDMF.EQ.3).OR.(KFRM.EQ.1.AND.IDMF.EQ.5))
     &CALL FULLSTOP('Kfermi=Band is impossible in IMA/REA mode')
C Process number of particles
      ZBND=0.D0
      HELP='! # of particles to find Ef             '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'fill',4,IN,1,0,ZBND,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	IF(KFRM.EQ.1.AND.ZBND.EQ.0.D0)
     &CALL FULLSTOP('Zbands should be set in HUBFILE')
C **************************************
      CALL READ_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'DMFT')
      DEALLOCATE(HUBCOPY)
C Set Matsubara grid
      CALL HUB_MSBMESH
C Overwrite real axis grid for Green function/Hybridization calculation
	EFRM=EF
	IF((ICGRF.GT.0.OR.ICHYB.GT.0).AND.KGRD.EQ.1.AND.NOMGGRF.GT.0)THEN
	NOMG=NOMGGRF
	NMSC=NOMGGRF
	NMSCMEM=NMSC
	IF(ALLOCATED(WMSC))DEALLOCATE(WMSC)
	IF(ALLOCATED(FMSC))DEALLOCATE(FMSC)
	IF(ALLOCATED(DMSC))DEALLOCATE(DMSC)
	IF(ALLOCATED(WOMG))DEALLOCATE(WOMG)
	IF(ALLOCATED(FOMG))DEALLOCATE(FOMG)
	IF(ALLOCATED(DOMG))DEALLOCATE(DOMG)
	ALLOCATE(WMSC(0:NMSC))
	ALLOCATE(FMSC(0:NMSC))
	ALLOCATE(DMSC(0:NMSC))
	ALLOCATE(WOMG(0:NOMG))
	ALLOCATE(FOMG(0:NOMG))
	ALLOCATE(DOMG(0:NOMG))
	DO IMSC=0,NMSC
	WMSC(IMSC)=DCMPLX(OMGGRF(IMSC),0.D0)
	FMSC(IMSC)=(OMGGRF(NMSC)-OMGGRF(0))/DBLE(NMSC+1)
	DMSC(IMSC)=0.D0
	WOMG(IMSC)=DREAL(WMSC(IMSC))
	FOMG(IMSC)=FMSC(IMSC)
	DOMG(IMSC)=0.D0
	ENDDO
	ENDIF
	END SUBROUTINE HUB_READHUB_DMFT










	SUBROUTINE HUB_READHUB_ITER(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads DMFT settings                                 *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
	USE COM_STREAM  ,ONLY : ISTREAM
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                          !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                                  !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE                !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT,EPSDMF,MIXBRD,
     &                        ADMIXL,ADMIX1,MIXFUN                    !!! from readhub
      USE COM_FREQU   ,ONLY : NOMGGRF,OMGGRF                          !!! from readini
      USE COM_LMTOUT  ,ONLY : SCRFILE,ICGRF,ICODF,ICHYB               !!! from readini
	USE COM_FERMI   ,ONLY : EF,ZBND,KFRM                            !!! from ttrints
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C **************************************
C READING THE DATA: SELECT DMFT SETTINGS
C NMSB - # of Matsubara energies
C TEMP - # Temperature in K
C NDIT - # of DMFT iterations
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=iter',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! ITERATIVE LOOP SETTINGS               '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process NDIT
      NDIT=20
      HELP='! # of DMFT iterations                  '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ndit',4,NDIT,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      IF(ISTREAM.EQ.4)NDIT=1 ! switch off self-consistency for phonon runs
C Use Broyden
      TCTRL='yes       '
      HELP='! Use Broyden mixing for DMFT loop      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'broy',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	MIXBRD=.TRUE.
	IF(TCTRL(1:3).EQ.'yes')MIXBRD=.TRUE.
	IF(TCTRL(1:2).EQ.'no' )MIXBRD=.FALSE.
C Mix Delta(w)/Sigma(w)
      TCTRL='Delta     '
      HELP='! Mix Delta(w)/Sigma(w) in DMFT loop    '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'mixfun',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	MIXFUN=0
	IF(TCTRL(1:5).EQ.'delta')MIXFUN=1
	IF(TCTRL(1:5).EQ.'sigma')MIXFUN=2
	IF(MIXFUN.EQ.0)
     &CALL FULLSTOP('Mixing function not recognized')
C Process AdmixL
      ADMIXL=0.35D0
      HELP='! Mixing parameter                      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'mixing',6,IN,1,0,ADMIXL,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Admix1
      ADMIX1=1.D0
      HELP='! Mixing parameter to start             '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'admix1',6,IN,1,0,ADMIX1,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Accuracy
      EPSDMF=1.D-05
      HELP='! Accuracy                              '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'epsdmf',6,IN,1,0,EPSDMF,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C **************************************
      CALL READ_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'ITER')
      DEALLOCATE(HUBCOPY)
	END SUBROUTINE HUB_READHUB_ITER




	SUBROUTINE HUB_READHUB_GLOC(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads Complex Green function settings               *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                          !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
      USE COM_ORDDAT  ,ONLY : KORD,NORD,OEPS,NSIG,OSIG                !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,KHRM,KSUM,PDMP,GDMP      !!! from readhub
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C **************************************
C READING THE DATA: SELECT ORDE SETTINGS
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=gloc',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! GREEN FUNCTIONS K-SUMS                '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process Hermitianness
      TCTRL='auto      '
      HELP='! Use hermitian hamiltonian: none/true  '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'hermit',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	KHRM=-1
	IF(TCTRL(1:4).EQ.'none')KHRM=0
	IF(TCTRL(1:4).EQ.'true')KHRM=1
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.LT.3)KHRM=1
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.EQ.3)KHRM=0
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.EQ.4)KHRM=1
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.EQ.5)KHRM=1
	IF(KHRM.EQ.-1)
     &CALL FULLSTOP('Keyword HERMIT is not recognized in HUBFILE')
	IF(KHRM.EQ.1.AND.IDMF.EQ.3)
     &WRITE(IUT,*)'Warning from <READHUB>::',
     &'Hermitian Hamiltonian is ON for IMA-DMFT mode'
C Process Tetrahedron integration
      TCTRL='auto      '
      HELP='! K-sums for Green function: ttrs/sums  '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'keysum',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	KSUM=-1
	IF(TCTRL(1:4).EQ.'ttrs')KSUM=0
	IF(TCTRL(1:4).EQ.'sums')KSUM=1
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.LT.3)KSUM=0
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.EQ.3)KSUM=0
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.EQ.4)KSUM=0
	IF(TCTRL(1:4).EQ.'auto'.AND.IDMF.EQ.5)KSUM=0
	IF(KSUM.EQ.-1)
     &CALL FULLSTOP('Keyword K-SUMS is not recognized in HUBFILE')
	IF(KSUM.EQ.1.AND.IDMF.EQ.3)
     &WRITE(IUT,*)'Warning from <READHUB>::',
     &'Hermitian Hamiltonian is ON for IMA-DMFT mode'
C Process Pole Damping
      PDMP=0.D0
      HELP='! Damping in pole expansion for Sigma(w)'
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'damsig',6,IN,1,0,PDMP,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process Green function damping
      GDMP=0.D0
      HELP='! Damping in Green function k-sum       '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'damgrf',6,IN,1,0,GDMP,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process KORD
      KORD=2         
      HELP='! set ordering scheme to func/imax      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'korder',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	IF(TCTRL(1:4).EQ.'auto')KORD=2
	IF(TCTRL(1:4).EQ.'func')KORD=1
	IF(TCTRL(1:4).EQ.'imax')KORD=2
C Process NORD
      NORD=5
      HELP='! # of nearest bands to consider        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'norder',6,NORD,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process NSIG
      NSIG=-1
      HELP='! # of correlated bands                 '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ordbnd',6,NSIG,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process OSIG
      OSIG=-0.1D0
      HELP='! Max. imaginary self-energy            '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ordsig',6,IN,1,0,OSIG,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Process OEPS
      OEPS=1.D-05
      HELP='! Ordering accuracy                     '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ordeps',6,IN,1,0,OEPS,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C **************************************
      CALL READ_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'GLOC')
      DEALLOCATE(HUBCOPY)
	END SUBROUTINE HUB_READHUB_GLOC








	SUBROUTINE HUB_READHUB_CORR(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads correlated states data                        *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF,ILSD                     !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                               !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                             !!! from readini
	USE COM_NBANDS  ,ONLY : NBNDT,NBNDH                             !!! from readhub
      USE COM_INDORB  ,ONLY : INDDMF                                  !!! from grfpar
      USE COM_MAGDAT  ,ONLY : AXMAG,ISPL,BSPL                         !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,NCLM,ICUBC,IRE 
     &                       ,KSOLVER,KDBLCNT,CDBLCNT,KSIGMOO         !!! from readhub
     &                       ,KDELFUN,KDELINT,KDELRAT,KIMPLEV,KFIXSIG
     &                       ,KSIGFUN,KSIGINT,KSIGRAT
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG,RINPINV
     &                       ,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
     &                       ,CHMAG,CHSPL,CHISP                       !!! from readhub
	USE COM_IMPDAT  ,ONLY : KCRDSYS,NRUNIMP,KSTATES,NPOLIMP,NPLDIMP,
     &                        AMAGIMP,BSPLIMP,ISPLIMP,
     &                        IMPSECT,IMPINFO,IMPDATA                 !!! from readhub
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                               !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT                                    !!! from readhub
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL                       !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	INTEGER, ALLOCATABLE :: KST(:)
	REAL*8, ALLOCATABLE :: FHUB(:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
     &   ,RSYS*10,RAXE*80,RANG*80,RINV*10,STRARR(80)*1
     &   ,CTHUB*2,CNHUB*1,CLHUB*1
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS
	ALLOCATE(FSLG(0:2*LMAXBMEM,0:LMAXBMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(FSLU(0:2*LMAXBMEM,0:LMAXBMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KDELFUN(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KDELINT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KDELRAT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSIGFUN(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSIGINT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSIGRAT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSOLVER(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KFIXSIG(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSIGMOO(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KIMPLEV(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KDBLCNT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CDBLCNT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KCRDSYS(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(NRUNIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSTATES(2*LIMBMEM,2*LIMBMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(NPOLIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(NPLDIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(IMPSECT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(IMPINFO(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(IMPDATA(1,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(AMAGIMP(3,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(BSPLIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(ISPLIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KST(2*LIMBMEM))
	ALLOCATE(FHUB(0:LMAXBMEM))
      ALLOCATE(RINPAXE(3,0:LMAXBMEM,NATOMMEM))
      ALLOCATE(ROUTAXE(3,0:LMAXBMEM,NATOMMEM))
      ALLOCATE(RINPANG(0:LMAXBMEM,NATOMMEM))
      ALLOCATE(ROUTANG(0:LMAXBMEM,NATOMMEM))
      ALLOCATE(RINPINV(0:LMAXBMEM,NATOMMEM))
      ALLOCATE(ROUTINV(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHSYS(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHAXE(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHANG(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHINV(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHSPL(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHISP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHMAG(0:LMAXBMEM,NATOMMEM))
C INITIALIZE CORRELATION POINTERS
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
C Correlated state pointers
	MCRL(IATOM)=0 
	DO L=0,LMAXTMEM
	LCRL(L,IATOM)=0
	ENDDO
      DO L=0,LMAXBMEM
C Delta(w) calculation control
	KDELFUN(L,IATOM)=0
	KDELINT(L,IATOM)=0
	KDELRAT(L,IATOM)=0
C Sigma(w) calculation control
	KSIGFUN(L,IATOM)=0
	KSIGINT(L,IATOM)=0
	KSIGRAT(L,IATOM)=0
C Impurity solver control
	KSOLVER(L,IATOM)=0
	KFIXSIG(L,IATOM)=0
	KSIGMOO(L,IATOM)=0
	KIMPLEV(L,IATOM)=0
	KDBLCNT(L,IATOM)=0
	CDBLCNT(L,IATOM)=0.D0
	KCRDSYS(L,IATOM)=0
	NRUNIMP(L,IATOM)=0
	NPOLIMP(L,IATOM)=0
	NPLDIMP(L,IATOM)=0
	DO IRUN=1,2*LIMBMEM
	DO ILMS=1,2*LIMBMEM
	KSTATES(ILMS,IRUN,L,IATOM)=0
	ENDDO
	ENDDO
C Additional information
	IMPINFO(L,IATOM)=0
	IMPSECT(L,IATOM)=0
	DO I=1,150
	IMPDATA(1,L,IATOM)(I:I)=' '
	ENDDO
C Magnetic filed
	BSPLIMP(L,IATOM)=0
	ISPLIMP(L,IATOM)=0
	AMAGIMP(1,L,IATOM)=0.D0
	AMAGIMP(2,L,IATOM)=0.D0
	AMAGIMP(3,L,IATOM)=1.D0
	ENDDO
	ENDDO 
C INITIALIZE SLATER INTEGRALS
	DO IATOM=1,NATOM
	DO L1=0,LMAXBMEM
	DO L=0,LMAXBMEM
	DO L2=0,2*LMAXBMEM
	FSLU(L2,L1,L,IATOM)=0.D0
	FSLG(L2,L1,L,IATOM)=0.D0
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C LOCAL COORDINATE SYSTEMS ASSUME GLOBAL SYSTEM BY DEFAULT
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      DO L=0,ILSB(ISORT)
      RINPAXE(1,L,IATOM)=0.D0
      RINPAXE(2,L,IATOM)=0.D0
      RINPAXE(3,L,IATOM)=1.D0
      ROUTAXE(1,L,IATOM)=0.D0
      ROUTAXE(2,L,IATOM)=0.D0
      ROUTAXE(3,L,IATOM)=1.D0
      RINPANG(L,IATOM)=0.D0
      ROUTANG(L,IATOM)=0.D0
      RINPINV(L,IATOM)=0.D0
      ROUTINV(L,IATOM)=0.D0
      ENDDO
      ENDDO
C **************************************
C READING THE DATA: SELECT CORRELATED STATES
C IAHUB - atom #
C CTHUB - <nl> for correlated state, e.g. '3d'
	ILOC=1
	DO 1000 IHUB=1,NHUB                       !!! over correlated states
      CALL READ_SECT(HUBDATA(ILOC),NLEN-ILOC+1,KSEC,NSEC,MSEC,
     &'<section=corr',13,'<section=',9,'ok',2)
	ISEC=KSEC+ILOC-1
	ILOC=ISEC+1
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! CORRELATED STATES                     '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process State
      SNOT='          '
      HELP='! Correlated state pointer              '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'cstate',6,IN,1,0,RE,1,0,CM,1,0,SNOT,20,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL GET_STATE(SNOT,20,IAHUB,ISPIN,MQ,LHUB,M,ICUBC)
	IF(LHUB.GT.LMAXBMEM)
     &CALL FULLSTOP('Wrong l for correlated state') 
	MCRL(IAHUB)=1
      LCRL(LHUB,IAHUB)=MQ
C ***********************
C Input coordinate system
C System
      RSYS='          '
      HELP='! global/local coordinate system        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpsys',6,IN,1,0,RE,1,0,CM,1,0,RSYS,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(RSYS,10)
      CALL LIB_RMBLANK(RSYS,10)
C Rotation axis
      DO I=1,80
      RAXE(I:I)=' '
      ENDDO
      HELP='! rotational axe                        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpaxis',7,IN,1,0,RE,1,0,CM,1,0,RAXE,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(RAXE,80)
      CALL LIB_RMBLANK(RAXE,80)
C Rotation angle
      DO I=1,80
      RANG(I:I)=' '
      ENDDO
      HELP='! rotational angle                      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpangle',8,IN,1,0,RE,1,0,CM,1,0,RANG,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(RANG,80)
      CALL LIB_RMBLANK(RANG,80)
C Apply inversion
      DO I=1,10
      RINV(I:I)=' '
      ENDDO
      HELP='! apply inversion after rotation        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpinv',6,IN,1,0,RE,1,0,CM,1,0,RINV,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(RINV,10)
      CALL LIB_RMBLANK(RINV,10)
C Set input-system arrays
	IF(RSYS.EQ.'local')THEN
      DO I=1,80
      STRARR(I)=RAXE(I:I)
      ENDDO
      CALL LIB_COMPVEC(STRARR,RINPAXE(1,LHUB,IAHUB),3)
      DO I=1,80
      STRARR(I)=RANG(I:I)
      ENDDO
      CALL LIB_COMPVEC(STRARR,RINPANG(LHUB,IAHUB),1)
      IF(RINV(1:1).EQ.'n')RINPINV(LHUB,IAHUB)=0.D0
      IF(RINV(1:1).EQ.'y')RINPINV(LHUB,IAHUB)=1.D0
	ENDIF
C ************************
C Output coordinate system
C System
      RSYS='          '
      HELP='! global/local coordinate system        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outsys',6,IN,1,0,RE,1,0,CM,1,0,RSYS,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CHSYS(LHUB,IAHUB)=RSYS
      CALL LIB_CONVLOW(RSYS,10)
      CALL LIB_RMBLANK(RSYS,10)
C Rotation axis
      DO I=1,80
      RAXE(I:I)=' '
      ENDDO
      HELP='! rotational axis                       '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outaxis',7,IN,1,0,RE,1,0,CM,1,0,RAXE,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CHAXE(LHUB,IAHUB)=RAXE
      CALL LIB_CONVLOW(RAXE,80)
      CALL LIB_RMBLANK(RAXE,80)
C Rotation angle
      DO I=1,80
      RANG(I:I)=' '
      ENDDO
      HELP='! rotational angle                      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outangle',8,IN,1,0,RE,1,0,CM,1,0,RANG,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CHANG(LHUB,IAHUB)=RANG
      CALL LIB_CONVLOW(RANG,80)
      CALL LIB_RMBLANK(RANG,80)
C Apply inversion
      DO I=1,10
      RINV(I:I)=' '
      ENDDO
      HELP='! apply inversion after rotation        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outinv',6,IN,1,0,RE,1,0,CM,1,0,RINV,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CHINV(LHUB,IAHUB)=RINV
      CALL LIB_CONVLOW(RINV,10)
      CALL LIB_RMBLANK(RINV,10)
C Set output-system arrays
	IF(RSYS.EQ.'local')THEN
      DO I=1,80
      STRARR(I)=RAXE(I:I)
      ENDDO
      CALL LIB_COMPVEC(STRARR,ROUTAXE(1,LHUB,IAHUB),3)
      DO I=1,80
      STRARR(I)=RANG(I:I)
      ENDDO
      CALL LIB_COMPVEC(STRARR,ROUTANG(LHUB,IAHUB),1)
      IF(RINV(1:2).EQ.'no' )ROUTINV(LHUB,IAHUB)=0.D0
      IF(RINV(1:3).EQ.'yes')ROUTINV(LHUB,IAHUB)=1.D0
	ENDIF
C **************************************************
C Process ISPLIMP
      DO I=1,10
	CHISP(LHUB,IAHUB)(I:I)=' '
      ENDDO
	TCTRL='none      '
	HELP='! splitting key: auto/always/kickup    '
	LHLP=40
	CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &  'ispl',4,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	ISPLIMP(LHUB,IAHUB)=-1
      IF(TCTRL(1:4).EQ.'auto'  )ISPLIMP(LHUB,IAHUB)=0
      IF(TCTRL(1:6).EQ.'always')ISPLIMP(LHUB,IAHUB)=1
      IF(TCTRL(1:6).EQ.'kickup')ISPLIMP(LHUB,IAHUB)=2
      IF(DEF.OR.NTH.OR.ASK     )ISPLIMP(LHUB,IAHUB)=0
	IF(ISPLIMP(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('ISPL not recognized in HUBFILE')
	IF(VAL)CHISP(LHUB,IAHUB)='value read'
C Process BSPLIMP
      DO I=1,10
	CHSPL(LHUB,IAHUB)(I:I)=' '
      ENDDO
	BSPLIMP(LHUB,IAHUB)=+0.0D0
	IF(NRSP.EQ.1.AND.NSPIN.EQ.2.AND.ILSD.EQ.0)
     &BSPLIMP(LHUB,IAHUB)=-0.4D0
	HELP='! initial splitting                    '
	LHLP=40
	CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &  'split',5,IN,1,0,BSPLIMP(LHUB,IAHUB),1,1,CM,1,0,
     &  ' ',1,0,HELP,LHLP,DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	IF(NSPIN.EQ.1)BSPLIMP(LHUB,IAHUB)=+0.0D0
	IF(VAL)CHSPL(LHUB,IAHUB)='value read'
C Process AMAGIMP
	AMAGIMP(1,LHUB,IAHUB)=AXMAG(1,IS(IAHUB))
	AMAGIMP(2,LHUB,IAHUB)=AXMAG(2,IS(IAHUB))
	AMAGIMP(3,LHUB,IAHUB)=AXMAG(3,IS(IAHUB))
      DO I=1,80
      RAXE(I:I)=' '
	CHMAG(LHUB,IAHUB)(I:I)=' '
      ENDDO
	HELP='! local magnetization axis             '
	LHLP=40
	CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &  'axmag',5,IN,1,0,RE,1,0,CM,1,0,RAXE,80,1,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	IF(VAL)THEN
	CHMAG(LHUB,IAHUB)=RAXE
      CALL LIB_CONVLOW(RAXE,80)
      CALL LIB_RMBLANK(RAXE,80)
	DO I=1,80
	STRARR(I)=RAXE(I:I)
	ENDDO
      CALL LIB_COMPVEC(STRARR,AMAGIMP(1,LHUB,IAHUB),3)
	ABA=SQRT(AMAGIMP(1,LHUB,IAHUB)**2+
     &         AMAGIMP(2,LHUB,IAHUB)**2+
     &         AMAGIMP(3,LHUB,IAHUB)**2)
	IF(ABA.LE.1.D-05)CALL FULLSTOP('Wrong AxMag in HUBFILE')
	AMAGIMP(1,LHUB,IAHUB)=AMAGIMP(1,LHUB,IAHUB)/ABA
	AMAGIMP(2,LHUB,IAHUB)=AMAGIMP(2,LHUB,IAHUB)/ABA
	AMAGIMP(3,LHUB,IAHUB)=AMAGIMP(3,LHUB,IAHUB)/ABA
	ENDIF
	IF(NORBS.EQ.1.AND.ABS(AMAGIMP(1,LHUB,IAHUB)).NE.0.D0
     &             .AND.ABS(AMAGIMP(2,LHUB,IAHUB)).NE.0.D0
     &             .AND.ABS(AMAGIMP(3,LHUB,IAHUB)).NE.1.D0)
     &CALL FULLSTOP('AxMag should be along 001 in HUBFILE')
C **************************************
C Process Slater Integrals, F(0,ll),F(2,ll),F(4,ll), etc.
	DO L=0,LHUB
	FHUB(L)=0.D0
      HELP='! Slater integrals                      '
      LHLP=40
	WRITE(CLHUB,FMT='(I1)')2*L
	CTHUB='f'//CLHUB
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &CTHUB,2,IN,1,0,FHUB(L),1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	ENDDO
	ISORT=IS(IAHUB)
	DO L2=0,2*LHUB,2
	IF(IDMF.EQ.0)FHUB(L2/2)=0.D0
	FSLU(L2,LHUB,LHUB,IAHUB)=FHUB(L2/2)
	FSLG(L2,LHUB,LHUB,IAHUB)=FHUB(L2/2)
	ENDDO
C **************************************
C Process Sigma Calculation Scheme
      TCTRL='auto      '
      HELP='! Sigma Calculation Scheme              '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'sigfun',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KSIGFUN(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none'  )KSIGFUN(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'auto'  )KSIGFUN(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'matinv')KSIGFUN(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'crfinv')KSIGFUN(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'eigenv')KSIGFUN(LHUB,IAHUB)=3
	IF(KSIGFUN(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('<HUB_READHUB>::Wrong <<sigfun>> keyword')
C **************************************
C Process Delta Calculation Scheme
      TCTRL='auto      '
      HELP='! Delta Calculation Scheme              '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'delfun',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDELFUN(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none'  )KDELFUN(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'auto'  )KDELFUN(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'matinv')KDELFUN(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'crfinv')KDELFUN(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'eigenv')KDELFUN(LHUB,IAHUB)=3
	IF(KDELFUN(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('<HUB_READHUB>::Wrong <<delfun>> keyword')
      IF((IDMF.EQ.3.OR.IDMF.EQ.5).AND.KDELFUN(LHUB,IAHUB).NE.1)
     &WRITE(IUT,*)
     &'<HUB_READHUB>:: Set DelFun=MatInv for real/imag DMFT'
C **************************************
C Process Sigma Interpolation method
      TCTRL='none      '
      HELP='! Sigma Interpolation Method            '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'sigint',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KSIGINT(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none')KSIGINT(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'mom2')KSIGINT(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'mom4')KSIGINT(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'mom6')KSIGINT(LHUB,IAHUB)=3
	IF(TCTRL(:NB-1).EQ.'momn')KSIGINT(LHUB,IAHUB)=4
	IF(TCTRL(:NB-1).EQ.'grid')KSIGINT(LHUB,IAHUB)=5
	IF(TCTRL(:NB-1).EQ.'conj')KSIGINT(LHUB,IAHUB)=6
	IF(TCTRL(:NB-1).EQ.'pade')KSIGINT(LHUB,IAHUB)=7
	IF(KSIGINT(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('<HUB_READHUB>::Wrong <<delint>> keyword')
	IF(KSIGINT(LHUB,IAHUB).EQ.1.OR.
     &   KSIGINT(LHUB,IAHUB).EQ.2.OR.
     &   KSIGINT(LHUB,IAHUB).EQ.3)WRITE(IUT,*)
     &'<HUB_READHUB>:: Reverse moments for metallic Delta(w) diverge!'
C **************************************
C Process Delta Interpolation method
      TCTRL='none      '
      HELP='! Delta Interpolation Method            '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'delint',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDELINT(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none')KDELINT(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'mom1')KDELINT(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'mom2')KDELINT(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'mom3')KDELINT(LHUB,IAHUB)=3
	IF(TCTRL(:NB-1).EQ.'mom4')KDELINT(LHUB,IAHUB)=4
	IF(TCTRL(:NB-1).EQ.'mom5')KDELINT(LHUB,IAHUB)=5
	IF(TCTRL(:NB-1).EQ.'mom6')KDELINT(LHUB,IAHUB)=6
	IF(TCTRL(:NB-1).EQ.'momn')KDELINT(LHUB,IAHUB)=7
	IF(TCTRL(:NB-1).EQ.'grid')KDELINT(LHUB,IAHUB)=8
	IF(TCTRL(:NB-1).EQ.'conj')KDELINT(LHUB,IAHUB)=9
	IF(TCTRL(:NB-1).EQ.'pade')KDELINT(LHUB,IAHUB)=10
	IF(KDELINT(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('<HUB_READHUB>::Wrong <<delint>> keyword')
	IF(KDELINT(LHUB,IAHUB).EQ.1.OR.
     &   KDELINT(LHUB,IAHUB).EQ.2.OR.
     &   KDELINT(LHUB,IAHUB).EQ.3)WRITE(IUT,*)
     &'<HUB_READHUB>:: Reverse moments for metallic Delta(w) diverge!'
C **************************************
C Process Sigma Representation
      TCTRL='sigmaw    '
      HELP='! Sigma Represenation: SigmaW/RatInt    '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'sigrat',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDELRAT(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'sigmaw')KSIGRAT(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'ratint')KSIGRAT(LHUB,IAHUB)=1
C **************************************
C Process Delta Representation
      TCTRL='deltaw    '
      HELP='! Delta Represenation: DeltaW/RatInt    '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'delrat',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDELRAT(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'deltaw')KDELRAT(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'ratint')KDELRAT(LHUB,IAHUB)=1
C **************************************
C Process impurity solver choice
      TCTRL='none      '
      HELP='! impurity solver choice                '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'solver',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KSOLVER(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none')KSOLVER(LHUB,IAHUB)=0
C Hartree Fock Solvers
	IF(TCTRL(:NB-1).EQ.'harfock')KSOLVER(LHUB,IAHUB)=11
	IF(TCTRL(:NB-1).EQ.'matfock')KSOLVER(LHUB,IAHUB)=11
C Atomic (Hubbard1) Solvers
	IF(TCTRL(:NB-1).EQ.'sunhub1')KSOLVER(LHUB,IAHUB)=21
	IF(TCTRL(:NB-1).EQ.'crfhub1')KSOLVER(LHUB,IAHUB)=22
C Interpolative (SBMF+Hubbard1) Solvers
	IF(TCTRL(:NB-1).EQ.'sunirat')KSOLVER(LHUB,IAHUB)=31
	IF(TCTRL(:NB-1).EQ.'crfirat')KSOLVER(LHUB,IAHUB)=32
C External Self-energy 
	IF(TCTRL(:NB-1).EQ.'readsig')KSOLVER(LHUB,IAHUB)=41
C SBMF solver
	IF(TCTRL(:NB-1).EQ.'sunsbmf')KSOLVER(LHUB,IAHUB)=61
	IF(TCTRL(:NB-1).EQ.'crfsbmf')KSOLVER(LHUB,IAHUB)=62
C RatFile solver
	IF(TCTRL(:NB-1).EQ.'ratfile')KSOLVER(LHUB,IAHUB)=71
C Quantum Monte Carlo Solver
	IF(TCTRL(:NB-1).EQ.'qmcfile')KSOLVER(LHUB,IAHUB)=81
C Flex Solvers
C	IF(TCTRL(:NB-1).EQ.'matflex')KSOLVER(LHUB,IAHUB)=91
C CHECK SOLVERS
	IF(KSOLVER(LHUB,IAHUB).EQ.-1)
     &   CALL FULLSTOP('Impurity solver not recognized')
	IF(IDMF.EQ.1.AND.KSOLVER(LHUB,IAHUB).GE.10)
     &   CALL FULLSTOP('Wrong solver chioce for LDA+C mode')
	IF(IDMF.EQ.2.AND.KSOLVER(LHUB,IAHUB).GE.20)
     &   CALL FULLSTOP('Wrong Solver choice for LDA+U mode')
	IF(IDMF.EQ.3.AND.KSOLVER(LHUB,IAHUB).GE.90)
     &CALL FULLSTOP('Impurity solver not yet implemented')
	IF(IDMF.EQ.4.AND.KSOLVER(LHUB,IAHUB).GE.90)
     &CALL FULLSTOP('Impurity solver not yet implemented')
	IF(IDMF.EQ.5.AND.KSOLVER(LHUB,IAHUB).GE.90)
     &CALL FULLSTOP('Impurity solver not yet implemented')
	IF(IDMF.EQ.6.AND.KSOLVER(LHUB,IAHUB).GE.90)
     &CALL FULLSTOP('Impurity solver not yet implemented')
C **************************************
C Process FixSig keyword for self-energy
      TCTRL='off       '
      HELP='! fix self-energy [on/off]              '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'fixsig',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KFIXSIG(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'off')KFIXSIG(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'on' )KFIXSIG(LHUB,IAHUB)=1
	IF(KFIXSIG(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('FixSig keyword not recognized')
C **************************************
C Process Sigma(oo) choice
      TCTRL='u(ns-n)   '
      HELP='! Sigma(oo) choice                      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'sigmai',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KSIGMOO(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none'   )KSIGMOO(LHUB,IAHUB)=0
C Matrix Self-Energies based on Nscf
	IF(TCTRL(:NB-1).EQ.'matnscf')KSIGMOO(LHUB,IAHUB)=11
	IF(TCTRL(:NB-1).EQ.'matfock')KSIGMOO(LHUB,IAHUB)=11
	IF(TCTRL(:NB-1).EQ.'u(nscf)')KSIGMOO(LHUB,IAHUB)=11
C Matrix Self-Energies based on Nimp
	IF(TCTRL(:NB-1).EQ.'matnimp')KSIGMOO(LHUB,IAHUB)=12
	IF(TCTRL(:NB-1).EQ.'matfocn')KSIGMOO(LHUB,IAHUB)=12
	IF(TCTRL(:NB-1).EQ.'u(nimp)')KSIGMOO(LHUB,IAHUB)=12
C SU(N) Self-Energies based on Nscf
	IF(TCTRL(:NB-1).EQ.'sunnscf')KSIGMOO(LHUB,IAHUB)=21
	IF(TCTRL(:NB-1).EQ.'u(ns-n)')KSIGMOO(LHUB,IAHUB)=21
C SU(N) Self-Energies based on Nimp
	IF(TCTRL(:NB-1).EQ.'sunnimp')KSIGMOO(LHUB,IAHUB)=22
	IF(TCTRL(:NB-1).EQ.'u(is-i)')KSIGMOO(LHUB,IAHUB)=22
C Other choices
	IF(TCTRL(:NB-1).EQ.'sigmaoo')KSIGMOO(LHUB,IAHUB)=31
	IF(TCTRL(:NB-1).EQ.'sigmain')KSIGMOO(LHUB,IAHUB)=41
C **************************************
C Process Impurity Level Choice
      TCTRL='sunlda    '
      HELP='! Impurity Level Choice                 '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'implev',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KIMPLEV(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none'  )KIMPLEV(LHUB,IAHUB)=0
C LDA derived impurity levels Hlda-Vdc-Ef
	IF(TCTRL(:NB-1).EQ.'matlda')KIMPLEV(LHUB,IAHUB)=11
	IF(TCTRL(:NB-1).EQ.'sunlda')KIMPLEV(LHUB,IAHUB)=12
C Double Counting derived impurity levels
	IF(TCTRL(:NB-1).EQ.'matvdc')KIMPLEV(LHUB,IAHUB)=21
	IF(TCTRL(:NB-1).EQ.'sunvdc')KIMPLEV(LHUB,IAHUB)=22
C Fixed Impurity Levels
	IF(TCTRL(:NB-1).EQ.'fixlev')KIMPLEV(LHUB,IAHUB)=30
C **************************************
C Process double counting choice
      TCTRL='none      '
      HELP='! double counting choice                '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'dcount',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDBLCNT(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none'   )KDBLCNT(LHUB,IAHUB)=0
C SU(N) double countings in fully localized limit Nscf
	IF(TCTRL(:NB-1).EQ.'u(ns-a)')KDBLCNT(LHUB,IAHUB)=11
	IF(TCTRL(:NB-1).EQ.'u(na-a)')KDBLCNT(LHUB,IAHUB)=12
	IF(TCTRL(:NB-1).EQ.'sunnscf')KDBLCNT(LHUB,IAHUB)=12
C SU(N) double countings in fully localized limit Nimp
	IF(TCTRL(:NB-1).EQ.'u(is-a)')KDBLCNT(LHUB,IAHUB)=13
	IF(TCTRL(:NB-1).EQ.'u(ia-a)')KDBLCNT(LHUB,IAHUB)=14
	IF(TCTRL(:NB-1).EQ.'sunnimp')KDBLCNT(LHUB,IAHUB)=14
C SU(N) double countings around mean field Nscf and Nimp
	IF(TCTRL(:NB-1).EQ.'u(ns-n)')KDBLCNT(LHUB,IAHUB)=21
	IF(TCTRL(:NB-1).EQ.'u(na-n)')KDBLCNT(LHUB,IAHUB)=22
	IF(TCTRL(:NB-1).EQ.'u(is-i)')KDBLCNT(LHUB,IAHUB)=23
	IF(TCTRL(:NB-1).EQ.'u(ia-i)')KDBLCNT(LHUB,IAHUB)=24
C Externally fixed double counting
	IF(TCTRL(:NB-1).EQ.'fixvdc' )KDBLCNT(LHUB,IAHUB)=30
C Double counting according to impurity levels
	IF(TCTRL(:NB-1).EQ.'fixlev' )KDBLCNT(LHUB,IAHUB)=40
C Other choices
	IF(TCTRL(:NB-1).EQ.'sigma0' )KDBLCNT(LHUB,IAHUB)=50
	IF(TCTRL(:NB-1).EQ.'sigmai' )KDBLCNT(LHUB,IAHUB)=60
	IF(KDBLCNT(LHUB,IAHUB).EQ.-1)
     &   CALL FULLSTOP('Double counting not recognized')
	IF(IDMF.EQ.1.AND.KDBLCNT(LHUB,IAHUB).NE.0)
     &   CALL FULLSTOP('Set double counting to NONE for LDA+C mode')
	IF(IDMF.EQ.2.AND.KDBLCNT(LHUB,IAHUB).GE.20)
     &   CALL FULLSTOP('Wrong double counting for LDA+U mode')
C Process constant alpha for double counting U(n-a)
      CDBLCNT(LHUB,IAHUB)=0.5D0
      HELP='! Constant in double counting U(n-a)    '
      LHLP=40
	CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'aconst',6,IN,1,0,CDBLCNT(LHUB,IAHUB),1,1,CM,1,0,
     &' ',1,0,HELP,LHLP,DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C **************************************
C Process coordinate system for impurity solver
      TCTRL='cubic     '
      HELP='! system: cubic/spherical/spinor        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'crdsys',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KCRDSYS(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'cubic'    )KCRDSYS(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'spherical')KCRDSYS(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'spinor'   )KCRDSYS(LHUB,IAHUB)=3
	IF(TCTRL(:NB-1).EQ.'diagonal' )KCRDSYS(LHUB,IAHUB)=4
	IF(TCTRL(:NB-1).EQ.'custom'   )KCRDSYS(LHUB,IAHUB)=5
	IF(KCRDSYS(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('Coordinate system not recognized')
	IF(KCRDSYS(LHUB,IAHUB).EQ.3.AND.NORBS.EQ.1)
     &CALL FULLSTOP('Spinor representation cannot be set when NORBS=1')
C **************************************
C Process # of runs for impurity solver
	NRUN=1
      HELP='! # of runs for impurity solver         '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'imprun',6,NRUN,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	NRUNIMP(LHUB,IAHUB)=NRUN
C **************************************
C For each run process states which participate
	NDIM=2*(2*LHUB+1)
	DO IRUN=1,NRUNIMP(LHUB,IAHUB)  !!! over runs
	DO IDIM=1,NDIM
	KST(IDIM)=IDIM
	ENDDO
      HELP='! states pointer in each run            '
	CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'kstates(:)',10,KST,NDIM,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	DO IDIM=1,NDIM
	KSTATES(IDIM,IRUN,LHUB,IAHUB)=KST(IDIM)
	ENDDO
C Count degeneracies
	NDEG=0
	DO IDIM=1,NDIM
	IF(KSTATES(IDIM,IRUN,LHUB,IAHUB).NE.0)NDEG=NDEG+1
	ENDDO
	IF(MOD(NDEG,2).NE.0)
     &CALL FULLSTOP('Wrong kstates(:) pointer')
	ENDDO                          !!! over runs
C Process NPOL
      NPOL=-1
      HELP='! # of poles for self-energy            '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'npol',4,NPOL,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	IF(NPOL.EQ.-1.AND.IDMF.EQ.4)
     &CALL FULLSTOP('nPol should be set in HUBFILE')
	IF(NPOL.EQ.-1.AND.IDMF.NE.4)NPOL=0
	NPOLIMP(LHUB,IAHUB)=NPOL
C Process NPLD
      NPLD=0
      IF(KDELINT(LHUB,IAHUB).EQ.1)NPLD=1
      IF(KDELINT(LHUB,IAHUB).EQ.2)NPLD=2
      IF(KDELINT(LHUB,IAHUB).EQ.3)NPLD=3
      IF(KDELINT(LHUB,IAHUB).EQ.4)NPLD=4
      IF(KDELINT(LHUB,IAHUB).EQ.5)NPLD=5
      IF(KDELINT(LHUB,IAHUB).EQ.6)NPLD=6
      HELP='! # of poles for hybridization          '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'npld',4,NPLD,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	NPLDIMP(LHUB,IAHUB)=NPLD
C **************************************
C Process additional information for the solver
      TCTRL='auto      '
      HELP='! additional info=none/auto/save        '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'impinf',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
      CALL LIB_CONVLOW(TCTRL,10)
      CALL LIB_RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	IMPSECT(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none')IMPSECT(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'auto')IMPSECT(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'save')IMPSECT(LHUB,IAHUB)=2
	IF(IMPSECT(LHUB,IAHUB).EQ.-1)
     &CALL FULLSTOP('Impurity information not recognized')
	IMPINFO(LHUB,IAHUB)=0
C ***************************************
C Process additional information for impurity solver
C Locate subsection
	CALL READ_SECT(HUBDATA(ISEC),NSEC,ISEC1,
     &  NSEC1,MSEC1,'<subsection=info',16,'<subsection=',12,'ok',2)
	IF(MSEC1.NE.0.AND.IMPSECT(LHUB,IAHUB).GT.0)THEN
	NINF=MSEC1-1
C Reallocate arrays if needed
	CALL HUB_READHUB_RECOPY(NINF)
C Read additional information into the file
	IMPINFO(LHUB,IAHUB)=NINF
	DO IREC=1,IMPINFO(LHUB,IAHUB)
	IMPDATA(IREC,LHUB,IAHUB)=HUBDATA(ISEC+ISEC1+IREC-1)
	ENDDO
	ENDIF
C **************************************
      CALL READ_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'CORR')
      DEALLOCATE(HUBCOPY)
 1000 CONTINUE                                  !!! over correlated states
C DETERMINE MAX NUMBER OF POLES
	NPOLMEM=1
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	DO L=0,ILSB(ISORT)
	NPOLMEM=MAX(NPOLMEM,NPOLIMP(L,IATOM))
	ENDDO
	ENDDO
	NPLDMEM=1
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	DO L=0,ILSB(ISORT)
	NPLDMEM=MAX(NPLDMEM,NPLDIMP(L,IATOM))
	ENDDO
	ENDDO
	IF(NPLDMEM.LT.0)NPLDMEM=0
C DETERMINE SIZE OF EXTENDED STATES
	NBNDH(1)=0
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
      NBNDH(1)=NBNDH(1)+(2*L+1)*NPOLIMP(L,IATOM)
	ENDDO
	ENDDO
	IF(IDMF.NE.4)NBNDH(1)=0
C Set length of correlated orbitals (panel # 1 only needed)
	NCLM=0
	DO IATOM=1,NATOM            !!! over atoms
	ISORT=IS(IATOM)
	DO IKAP=1,NKAP(1)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)          !!! over l
	IF(.NOT.LTPAN(L,IATOM,IKAP,1))CYCLE
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	DO M=-L,L
	NCLM=NCLM+1
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Set orbital pointers for DMFT
	DO ISPIN=1,NSPIN
	IDIM=(ISPIN-1)*(NBNDT(1)+NBNDH(1))
	DO IKAP=1,NKAP(1)
	DO IATOM=1,NATOM
	DO L=0,ILSB(IS(IATOM))
	IF(.NOT.LTPAN(L,IATOM,IKAP,1))CYCLE
	DO M=-L,L
	LM=L*(L+1)+M+1
	IDIM=IDIM+1
	INDDMF(LM,IATOM,IKAP,ISPIN)=IDIM
	ENDDO 
	ENDDO 
	ENDDO 
	ENDDO 
	ENDDO 
C MEMORY DEALLOCATIONS
	DEALLOCATE(FHUB,KST)

	CONTAINS

	SUBROUTINE HUB_READHUB_RECOPY(NI)
      CHARACTER*150, ALLOCATABLE :: DATA(:,:,:)
C Determine the size of current array
	IMAX=1
	DO KATOM=1,NATOM
	DO LK=0,LMAXBMEM
	IMAX=MAX(IMAX,IMPINFO(LK,KATOM))
	ENDDO
	ENDDO
	IF(IMAX.GE.NI)RETURN ! no reallocation needed
C Copy the data into temporary array
	ALLOCATE(DATA(IMAX,0:LMAXBMEM,NATOMMEM))
	DO KATOM=1,NATOM
	DO LK=0,LMAXBMEM
	DO IREC=1,IMAX
	DATA(IREC,LK,KATOM)=IMPDATA(IREC,LK,KATOM)
	ENDDO
	ENDDO
	ENDDO
	DEALLOCATE(IMPDATA)
	ALLOCATE(IMPDATA(NI,0:LMAXBMEM,NATOMMEM))
	DO KATOM=1,NATOM
	DO LK=0,LMAXBMEM
	DO IREC=1,IMAX
	IMPDATA(IREC,LK,KATOM)=DATA(IREC,LK,KATOM)
	ENDDO
	DO IREC=IMAX+1,NI
	DO I=1,150
	IMPDATA(IREC,LK,KATOM)(I:I)=' '
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	DEALLOCATE(DATA)
	END SUBROUTINE HUB_READHUB_RECOPY

	END SUBROUTINE HUB_READHUB_CORR







	SUBROUTINE HUB_READHUB_DATA(HUBDATA,NLEN,NROR,NRSP,ICUBI)
***********************************************************
*  Reads occupation numbers, other data                   *
***********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE        !!! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS,VDBC             !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
	USE COM_DMFDAT  ,ONLY : DELI,SIGW,DELW,GLOC,OIMP,EIMP,ELOC
     &                       ,SIG0,SIGI,BMAS,RMAS,AINV,ALAM,ELAM
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(VHUB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DHUB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(VCNS(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(SIG0(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(SIGI(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(BMAS(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(RMAS(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(VDBC(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DELI(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(SIGW(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DELW(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(GLOC(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(OIMP(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(EIMP(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(ELOC(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(AINV(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(ALAM(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(ELAM(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	CURMEM=CURMEM+15.D0*DBLE(SIZE(VHUB)*LENCMP)/1024.D0**2
C MEMORY ALLOCATIONS
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
C INITIALIZE OCCUPANCIES "DHUB" AND HUBBARD POTENTIALS "VHUB"
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
C LDA+U ARRAYS
      VHUB(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      VCNS(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DHUB(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
C DMFT ARRAYS
      SIG0(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      SIGI(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      VDBC(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELI(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      BMAS(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      RMAS(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      SIGW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      GLOC(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      EIMP(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ELOC(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      AINV(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ALAM(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(M.NE.K)
     &OIMP(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(M.EQ.K.AND.IORBS.EQ.1)
     &OIMP(IU,IATOM,IUPDN,IORBS)=(1.D0,0.D0)
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C READ PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	DO 1000 IDVR=1,8
	IF(IDVR.EQ.1)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dhub',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.2)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=vhub',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.3)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=vdbc',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.4)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=vcns',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.5)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=oinv',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.6)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=eimp',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.7)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=eloc',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.8)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=bmas',13,'<section=',9,'ok',2)
	IF(MSEC.LE.1)GOTO 1000
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
	IF(IDVR.EQ.1)
     &HELP='! PARTIAL OCCUPANCIES                   '
	IF(IDVR.EQ.2)
     &HELP='! LDA+U POTENTIAL                       '
	IF(IDVR.EQ.3)
     &HELP='! DOUBLE COUNTING POTENTIAL             '
	IF(IDVR.EQ.4)
     &HELP='! CONSTRAINED POTENTIAL                 '
	IF(IDVR.EQ.5)
     &HELP='! INVERSE OVERLAP INVERSE               '
	IF(IDVR.EQ.6)
     &HELP='! MEAN FIELD IMPURITY LEVELS            '
	IF(IDVR.EQ.7)
     &HELP='! BARE IMPURITY LEVELS                  '
	IF(IDVR.EQ.8)
     &HELP='! STATIC SUSCEPTIBILITY                 '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
	IREC=1
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
C READ MATRIX
      NREC=NROR*NRSP*IRE*(1+2*L+1)
	IREC=IREC+1
      CALL HUB_READREP(OCS,LIMB2MEM,NRSP,NROR,L,ICUBI,IRE,
     &     HUBCOPY(IREC),NREC)
      IREC=IREC+NREC-1
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL LIB_GTURNU(OCP,OCS,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &     LIMB2MEM,NRSP,NROR)
      DO IORBS=1,NORBS                       !!! over iorbs
      DO IUPDN=1,NSPIN                       !!! over iupdn
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &DHUB(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.2)
     &VHUB(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.3)
     &VDBC(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.4)
     &VCNS(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.5)
     &OIMP(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.6)
     &EIMP(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.7)
     &ELOC(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.8)
     &BMAS(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO
      ENDDO
	ENDDO                                  !!! over iupdn
	ENDDO                                  !!! over iorbs
	DO IOR=1,NROR                          !!! over ior
	DO ISP=1,NRSP                          !!! over isp
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	VFC=1.D0
	IF(NSPIN.EQ.1.AND.NRSP.EQ.2)
     &VFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &DHUB(IUMK,IATOM,IUPDN,IORBS)=
     &DHUB(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)
	IF(IDVR.EQ.2)
     &VHUB(IUMK,IATOM,IUPDN,IORBS)=
     &VHUB(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
	IF(IDVR.EQ.3)
     &VDBC(IUMK,IATOM,IUPDN,IORBS)=
     &VDBC(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
	IF(IDVR.EQ.4)
     &VCNS(IUMK,IATOM,IUPDN,IORBS)=
     &VCNS(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
	IF(IDVR.EQ.5)
     &OIMP(IUMK,IATOM,IUPDN,IORBS)=
     &OIMP(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
	IF(IDVR.EQ.6)
     &EIMP(IUMK,IATOM,IUPDN,IORBS)=
     &EIMP(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
	IF(IDVR.EQ.7)
     &ELOC(IUMK,IATOM,IUPDN,IORBS)=
     &ELOC(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
	IF(IDVR.EQ.8)
     &BMAS(IUMK,IATOM,IUPDN,IORBS)=
     &BMAS(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
      DEALLOCATE(HUBCOPY)
 1000	CONTINUE
C EXPAND ARRAYS
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IUP=1
	IDN=1
	DFC=1.D0
	VFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
	IF(NSPIN.EQ.1.AND.NRSP.EQ.2)VFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	DHUB(IUMK,IATOM,IUP,1)=DHUB(IUMK,IATOM,IUP,1)*DFC
      DHUB(IUMK,IATOM,IDN,1)=DHUB(IUMK,IATOM,IUP,1)
      VHUB(IUMK,IATOM,IDN,1)=VHUB(IUMK,IATOM,IUP,1)
      VDBC(IUMK,IATOM,IDN,1)=VDBC(IUMK,IATOM,IUP,1)
      VCNS(IUMK,IATOM,IDN,1)=VCNS(IUMK,IATOM,IUP,1)
      OIMP(IUMK,IATOM,IDN,1)=OIMP(IUMK,IATOM,IUP,1)
      EIMP(IUMK,IATOM,IDN,1)=EIMP(IUMK,IATOM,IUP,1)
      ELOC(IUMK,IATOM,IDN,1)=ELOC(IUMK,IATOM,IUP,1)
      BMAS(IUMK,IATOM,IDN,1)=BMAS(IUMK,IATOM,IUP,1)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C MEMORY DEALLOCATIONS
	DEALLOCATE(OCP,OCS,ULR,UJR)
	END








	SUBROUTINE HUB_READHUB_CSYS(HUBDATA,NLEN,NROR,NRSP,ICUBI)
***************************************************************
*  Reads custom transformation matrices                       *
***************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE        !!! from readhub
	USE COM_IMPDAT  ,ONLY : KCRDSYS,DSYS,RSYS
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      COMPLEX*16, ALLOCATABLE :: DINV(:,:)
      COMPLEX*16, ALLOCATABLE :: WORK(:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      COMPLEX*16 TS(7,-3:3,0:3),CVAL
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(DSYS(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(RSYS(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	CURMEM=CURMEM+2.D0*DBLE(SIZE(DSYS)*LENCMP)/1024.D0**2
C MEMORY ALLOCATIONS
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
C INITIALIZE TRANSFORMATION MATRTCES
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      DSYS(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      RSYS(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C Reset these matrices for known transformations
	CALL LIB_TRANSS(TS)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	IF(IORBS.EQ.1)THEN
	ISPIN1=IUPDN
	ISPIN =IUPDN
	ELSEIF(IORBS.EQ.2)THEN
	ISPIN1=IUPDN
	ISPIN =NSPIN+1-IUPDN
	ENDIF
	DO K=-L,L
	IDIM1=K+L+1+(2*L+1)*(ISPIN1-1)
	DO M=-L,L
	IDIM=M+L+1+(2*L+1)*(ISPIN-1)
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
C Cubic Harmonics
	IF(KCRDSYS(L,IATOM).EQ.1)THEN
	IF(ISPIN1.EQ.ISPIN)CVAL=TS(M+L+1,K,L)
	IF(ISPIN1.NE.ISPIN)CVAL=(0.D0,0.D0)
C Spherical Harmonics
	ELSEIF(KCRDSYS(L,IATOM).EQ.2)THEN
	IF(IDIM1.EQ.IDIM)CVAL=(1.D0,0.D0)
	IF(IDIM1.NE.IDIM)CVAL=(0.D0,0.D0)
C At the beginning other representations assume diagonal matrix as well
	ELSE
	IF(IDIM1.EQ.IDIM)CVAL=(1.D0,0.D0)
	IF(IDIM1.NE.IDIM)CVAL=(0.D0,0.D0)
	ENDIF
      DSYS(IU,IATOM,IUPDN,IORBS)=CVAL
      ENDDO                                     !!! over m
      ENDDO                                     !!! over k
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
C READ DIRECT TRANSFORMATION MATRIX
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=csys',13,'<section=',9,'ok',2)
	IF(MSEC.LE.1)GOTO 1000
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! CUSTOM TRANSFORMATION MATRICES        '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
	IREC=1
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
C READ MATRIX
      NREC=NROR*NRSP*IRE*(1+2*L+1)
	IREC=IREC+1
      CALL HUB_READREP(OCS,LIMB2MEM,NRSP,NROR,L,ICUBI,IRE,
     &     HUBCOPY(IREC),NREC)
      IREC=IREC+NREC-1
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL LIB_GTURNU(OCP,OCS,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &     LIMB2MEM,NRSP,NROR)
	IF(KCRDSYS(L,IATOM).EQ.5)THEN
      DO IORBS=1,NORBS                       !!! over iorbs
      DO IUPDN=1,NSPIN                       !!! over iupdn
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      DSYS(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO
      ENDDO
	ENDDO                                  !!! over iupdn
	ENDDO                                  !!! over iorbs
	DO IOR=1,NROR
	DO ISP=1,NRSP
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	VFC=1.D0
	IF(NSPIN.EQ.1.AND.NRSP.EQ.2)
     &VFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      DSYS(IUMK,IATOM,IUPDN,IORBS)=
     &DSYS(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	ENDIF
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
      DEALLOCATE(HUBCOPY)
 1000	CONTINUE
C EXPAND ARRAYS
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      DSYS(IUMK,IATOM,IDN,1)=DSYS(IUMK,IATOM,IUP,1)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C BUILD REVERSE TRANSFORMATION MATRIX
	DO IATOM=1,NATOM                    !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE           !!! at_corr
	DO L=0,ILSB(ISORT)                  !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE         !!! l_corr
	NDIM=2*(2*L+1)
	ALLOCATE(DINV(NDIM,NDIM))
	ALLOCATE(WORK(NDIM))
C Set up matrices
	DO IDIM1=1,NDIM
	DO IDIM =1,NDIM
	DINV(IDIM1,IDIM)=(0.D0,0.D0)
	ENDDO
	ENDDO
	DO IORBS=1,2
	DO IUPDN=1,2
	IF(IORBS.EQ.1)THEN
	ISPIN1=IUPDN
	ISPIN =IUPDN
	ELSEIF(IORBS.EQ.2)THEN
	ISPIN1=IUPDN
	ISPIN =NSPIN+1-IUPDN
	ENDIF
	IF(NORBS.EQ.1.AND.IORBS.EQ.2)CYCLE
	IF(NSPIN.EQ.1)IUPDN0=1
	IF(NSPIN.EQ.2)IUPDN0=IUPDN
	DO K=-L,L
	IDIM1=K+L+1+(2*L+1)*(ISPIN1-1)
	DO M=-L,L
	IDIM=M+L+1+(2*L+1)*(ISPIN-1)
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	DINV(IDIM1,IDIM)=DSYS(IU,IATOM,IUPDN0,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Invert matrix
	CALL LIB_CINV(NDIM,DINV,NDIM,WORK,IFAIL)
	IF(IFAIL.NE.0)
     &CALL FULLSTOP('<HUB_READHUB_CSYS>:: Error in inversion program')
C Reverse transformation matrix
	DO IORBS=1,2
	DO IUPDN=1,2
	IF(IORBS.EQ.1)THEN
	ISPIN1=IUPDN
	ISPIN =IUPDN
	ELSEIF(IORBS.EQ.2)THEN
	ISPIN1=IUPDN
	ISPIN =NSPIN+1-IUPDN
	ENDIF
	IF(NORBS.EQ.1.AND.IORBS.EQ.2)CYCLE
	IF(NSPIN.EQ.1)IUPDN0=1
	IF(NSPIN.EQ.2)IUPDN0=IUPDN
	DO K=-L,L
	IDIM1=K+L+1+(2*L+1)*(ISPIN1-1)
	DO M=-L,L
	IDIM=M+L+1+(2*L+1)*(ISPIN-1)
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	IW=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1 !!! (l,m k)
	RSYS(IU,IATOM,IUPDN0,IORBS)=RSYS(IU,IATOM,IUPDN0,IORBS)+
     &DINV(IDIM1,IDIM)/2.D0*DBLE(NSPIN)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	DEALLOCATE(DINV,WORK)
	ENDDO                               !!! over l
	ENDDO                               !!! over iatom
C MEMORY DEALLOCATIONS
	DEALLOCATE(OCP,OCS,ULR,UJR)
	END



	SUBROUTINE HUB_READHUB_RATP(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*****************************************************************
*  Reads coefficients of rational interpolation for self-energy *
*****************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                   !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                           !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB          !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                                !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                              !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,KHRM,PDMP,ICUBC,IRE       !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG          !!! from readhop
     &                       ,RINPINV,ROUTINV
	USE COM_MSBMESH ,ONLY : NMSC,WMSC                                !!! from readhub
	USE COM_DMFDAT  ,ONLY : SIG0,SIGI,SIGW,DELW,DELI,GLOC            !!! from readhub
	USE COM_IMPDAT  ,ONLY : NPOLIMP,NPLDIMP                          !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT                                     !!! from readhub
	USE COM_FERMI   ,ONLY : EF                                       !!! from readhub
	USE COM_RATDAT  ,ONLY : VRAT,WRAT,PRAT,ORAT,
     &                        VRHB,WRHB,PRHB,ORHB                      !!! from readhub
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL                        !!! from readini
      USE COM_LMTOUT  ,ONLY : SCRFILE                                  !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                                    !!! from channels
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                              !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                              !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      COMPLEX*16, ALLOCATABLE :: SIGL(:,:)
      COMPLEX*16, ALLOCATABLE :: DELL(:,:)
      COMPLEX*16, ALLOCATABLE :: VARR(:,:)
      COMPLEX*16, ALLOCATABLE :: WARR(:,:)
      COMPLEX*16, ALLOCATABLE :: PINV(:,:)
      COMPLEX*16, ALLOCATABLE :: WORK(:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      COMPLEX*16 DIM,ZMSC
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/,DIM/(0.D0,1.D0)/
C MEMORY ALLOCATIONS
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(VRAT(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPOLMEM))
	ALLOCATE(WRAT(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPOLMEM))
	ALLOCATE(PRAT(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPOLMEM))
	ALLOCATE(ORAT(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPOLMEM))
	ALLOCATE(VRHB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPLDMEM))
	ALLOCATE(WRHB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPLDMEM))
	ALLOCATE(PRHB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPLDMEM))
	ALLOCATE(ORHB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM,NPLDMEM))
C INITIALIZE COEFFICIENTS
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      GLOC(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      SIGI(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      SIG0(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELI(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	DO IPOL=1,NPOLMEM                         !!! over poles
C RATIONAL INTERPOLATION ARRAYS
      WRAT(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
      VRAT(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
C Undetermined poles cannot be zero, otherwise the band falls into Ef and
C tetrahedron integration fails
      PRAT(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(M.EQ.K.AND.IORBS.EQ.1)
     &PRAT(IU,IATOM,IUPDN,IORBS,IPOL)=(1.D0,0.D0) 
C Overlap cannot be zero, otherwise inversion will fail
      ORAT(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(M.EQ.K.AND.IORBS.EQ.1)
     &ORAT(IU,IATOM,IUPDN,IORBS,IPOL)=(1.D0,0.D0)
	ENDDO                                     !!! over poles
	DO IPOL=1,NPLDMEM                         !!! over poles
C RATIONAL INTERPOLATION ARRAYS
      WRHB(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
      VRHB(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
C Undetermined poles cannot be zero, otherwise the band falls into Ef and
C tetrahedron integration fails
      PRHB(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(M.EQ.K.AND.IORBS.EQ.1)
     &PRHB(IU,IATOM,IUPDN,IORBS,IPOL)=(1.D0,0.D0) 
C Overlap cannot be zero, otherwise inversion will fail
      ORHB(IU,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(M.EQ.K.AND.IORBS.EQ.1)
     &ORHB(IU,IATOM,IUPDN,IORBS,IPOL)=(1.D0,0.D0)
	ENDDO                                     !!! over poles
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C READ RATIONAL INTERPOLATION COEFFICIENTS
      DO 1000 IDVR=1,2
      IF(IDVR.EQ.1)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=srat',13,'<section=',9,'ok',2)
      IF(IDVR.EQ.2)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=drat',13,'<section=',9,'ok',2)
	IF(MSEC.LE.1)GOTO 1000
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      IF(IDVR.EQ.1)
     &HELP='! RATIONAL INTERPOLATION FOR SIGMA      '
      IF(IDVR.EQ.2)
     &HELP='! RATIONAL INTERPOLATION FOR DELTA      '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C START READING THE ARRAYS
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
	IREC=1
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Process NPOL
      NPOLES=-1
      HELP='! # of poles                            '
      LHLP=40
	IREC=IREC+1
	STRING=HUBCOPY(IREC)
	CALL READ_CHAR(HUBCOPY(IREC),STRING,1,1,
     &  'npoles',6,NPOLES,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
	IF(NPOLES.EQ.-1)
     &CALL FULLSTOP('Cannot read SRAT/DRAT section of HUBFILE')
C SET SELF-ENERGY AT INFINITY
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
C READ MATRIX
      NREC=NROR*NRSP*IRE*(1+2*L+1)
	IREC=IREC+1
      CALL HUB_READREP(OCS,LIMB2MEM,NRSP,NROR,L,ICUBI,IRE,
     &     HUBCOPY(IREC),NREC)
      IREC=IREC+NREC-1
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL LIB_GTURNU(OCP,OCS,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &     LIMB2MEM,NRSP,NROR)
C RESET SIGMA(OO)
      DO IORBS=1,NORBS                       !!! over iorbs
      DO IUPDN=1,NSPIN                       !!! over iupdn
      DO M=-L,L                              !!! over m
      DO K=-L,L                              !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)
     &SIGI(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      IF(IDVR.EQ.2)
     &DELI(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO                                  !!! over k
      ENDDO                                  !!! over m
	ENDDO                                  !!! over iupdn
	ENDDO                                  !!! over iorbs
	DO IOR=1,NROR                          !!! over ior
	DO ISP=1,NRSP                          !!! over isp
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	VFC=1.D0
	IF(NSPIN.EQ.1.AND.NRSP.EQ.2)
     &VFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)
     &SIGI(IUMK,IATOM,IUPDN,IORBS)=
     &SIGI(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
      IF(IDVR.EQ.2)
     &DELI(IUMK,IATOM,IUPDN,IORBS)=
     &DELI(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
C READ RATIONAL ARRAYS
	DO IPOL=1,NPOLES                       !!! over poles
	DO IARR=1,4                            !!! read 4 arrays
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
C Process Polynomial
	HELP='                                       '
	LHLP=40
	IREC=IREC+1
	STRING=HUBCOPY(IREC)
	CALL READ_CHAR(HUBCOPY(IREC),STRING,1,1,
     &  'ipol',4,IP,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
C READ MATRIX
      NREC=NROR*NRSP*IRE*(1+2*L+1)
      IREC=IREC+1
      CALL HUB_READREP(OCS,LIMB2MEM,NRSP,NROR,L,ICUBI,IRE,
     &     HUBCOPY(IREC),NREC)
      IREC=IREC+NREC-1
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL LIB_GTURNU(OCP,OCS,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &     LIMB2MEM,NRSP,NROR)
	IF((IPOL.LE.NPOLIMP(L,IATOM).AND.IDVR.EQ.1).OR.
     &   (IPOL.LE.NPLDIMP(L,IATOM).AND.IDVR.EQ.2))THEN
C RESET ARRAYS
      DO IORBS=1,NORBS                       !!! over iorbs
      DO IUPDN=1,NSPIN                       !!! over iupdn
      DO M=-L,L                              !!! over m
      DO K=-L,L                              !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IARR.EQ.1.AND.IDVR.EQ.1)
     &VRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(IARR.EQ.2.AND.IDVR.EQ.1)
     &WRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(IARR.EQ.3.AND.IDVR.EQ.1)
     &PRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(IARR.EQ.4.AND.IDVR.EQ.1)
     &ORAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(IARR.EQ.1.AND.IDVR.EQ.2)
     &VRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(IARR.EQ.2.AND.IDVR.EQ.2)
     &WRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(IARR.EQ.3.AND.IDVR.EQ.2)
     &PRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
	IF(IARR.EQ.4.AND.IDVR.EQ.2)
     &ORHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=(0.D0,0.D0)
      ENDDO                                  !!! over k
      ENDDO                                  !!! over m
      ENDDO                                  !!! over iupdn
      ENDDO                                  !!! over iorbs
	DO IOR=1,NROR                          !!! over ior
	DO ISP=1,NRSP                          !!! over isp
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	VFC=1.D0
	IF(NSPIN.EQ.1.AND.NRSP.EQ.2)
     &VFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IARR.EQ.1.AND.IDVR.EQ.1)
     &VRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &VRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
	IF(IARR.EQ.2.AND.IDVR.EQ.1)
     &WRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &WRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
	IF(IARR.EQ.3.AND.IDVR.EQ.1)
     &PRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &PRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
	IF(IARR.EQ.4.AND.IDVR.EQ.1)
     &ORAT(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &ORAT(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
	IF(IARR.EQ.1.AND.IDVR.EQ.2)
     &VRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &VRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
	IF(IARR.EQ.2.AND.IDVR.EQ.2)
     &WRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &WRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
	IF(IARR.EQ.3.AND.IDVR.EQ.2)
     &PRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &PRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
	IF(IARR.EQ.4.AND.IDVR.EQ.2)
     &ORHB(IUMK,IATOM,IUPDN,IORBS,IPOL)=
     &ORHB(IUMK,IATOM,IUPDN,IORBS,IPOL)+OCP(IUMK,ISP,IOR)*VFC
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	ENDIF
	ENDDO                                  !!! over arrays
      ENDDO                                  !!! over poles
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
      DEALLOCATE(HUBCOPY)
C EXPAND ARRAYS
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)THEN
      SIGI(IUMK,IATOM,IDN,1)=SIGI(IUMK,IATOM,IUP,1)
	DO IPOL=1,NPOLIMP(L,IATOM)
      VRAT(IUMK,IATOM,IDN,1,IPOL)=VRAT(IUMK,IATOM,IUP,1,IPOL)
      WRAT(IUMK,IATOM,IDN,1,IPOL)=WRAT(IUMK,IATOM,IUP,1,IPOL)
      PRAT(IUMK,IATOM,IDN,1,IPOL)=PRAT(IUMK,IATOM,IUP,1,IPOL)
      ORAT(IUMK,IATOM,IDN,1,IPOL)=ORAT(IUMK,IATOM,IUP,1,IPOL)
	ENDDO
	ELSEIF(IDVR.EQ.2)THEN
      DELI(IUMK,IATOM,IDN,1)=DELI(IUMK,IATOM,IUP,1)
	DO IPOL=1,NPLDIMP(L,IATOM)
      VRHB(IUMK,IATOM,IDN,1,IPOL)=VRHB(IUMK,IATOM,IUP,1,IPOL)
      WRHB(IUMK,IATOM,IDN,1,IPOL)=WRHB(IUMK,IATOM,IUP,1,IPOL)
      PRHB(IUMK,IATOM,IDN,1,IPOL)=PRHB(IUMK,IATOM,IUP,1,IPOL)
      ORHB(IUMK,IATOM,IDN,1,IPOL)=ORHB(IUMK,IATOM,IUP,1,IPOL)
	ENDDO
	ENDIF
	ENDDO
	ENDDO
	ENDDO
	ENDDO
 1000	CONTINUE
C SET FREQUENCY DEPENDENT SELF-ENERGY/HYBRIDIZATION
C Estimate record lenght
	NRECL=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	NRECL=MAX(NRECL,IUMAX*NVAL*4*NSPIN*NORBS)
	ENDDO
C Allocate storage file
	OPEN(UNIT=IUSGM,FILE=SCRFILE(IUSGM),STATUS='NEW',ACCESS='DIRECT',
     &FORM='UNFORMATTED',RECL=NRECL)
	IREC=0
	DO IMSC=0,NMSC                      !!! over Matsubara frequencies
	ZMSC=WMSC(IMSC)
	DO IATOM=1,NATOM                    !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE           !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	DO L=0,ILSB(ISORT)                  !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE         !!! l_corr
C Sigma/Delta(infinity)
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	SIGW(IU,IATOM,IUPDN,IORBS)=SIGI(IU,IATOM,IUPDN,IORBS)
	DELW(IU,IATOM,IUPDN,IORBS)=DELI(IU,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Contributions from the poles
	NDIM=2*(2*L+1)
	EFRM=EF
	ALLOCATE(SIGL(NDIM,NDIM))
	ALLOCATE(DELL(NDIM,NDIM))
	DO IDIM1=1,NDIM
	DO IDIM=1,NDIM
	SIGL(IDIM1,IDIM)=(0.D0,0.D0)
	DELL(IDIM1,IDIM)=(0.D0,0.D0)
	ENDDO
	ENDDO
	ALLOCATE(PINV(NDIM,NDIM))
	ALLOCATE(VARR(NDIM,NDIM))
	ALLOCATE(WARR(NDIM,NDIM))
	ALLOCATE(WORK(NDIM))
C DO OVER SELF-ENERGY POLES
	DO IPOL=1,NPOLIMP(L,IATOM)         !!! over poles
C Set up matrices
	DO IDIM1=1,NDIM
	DO IDIM =1,NDIM
	VARR(IDIM1,IDIM)=(0.D0,0.D0)
	WARR(IDIM1,IDIM)=(0.D0,0.D0)
	PINV(IDIM1,IDIM)=(0.D0,0.D0)
	ENDDO
	ENDDO
	DO IORBS=1,2
	DO IUPDN=1,2
	IF(IORBS.EQ.1)THEN
	ISPIN1=IUPDN
	ISPIN =IUPDN
	ELSEIF(IORBS.EQ.2)THEN
	ISPIN1=IUPDN
	ISPIN =NSPIN+1-IUPDN
	ENDIF
	IF(NORBS.EQ.1.AND.IORBS.EQ.2)CYCLE
	IF(NSPIN.EQ.1)IUPDN0=1
	IF(NSPIN.EQ.2)IUPDN0=IUPDN
	DO K=-L,L
	IDIM1=K+L+1+(2*L+1)*(ISPIN1-1)
	DO M=-L,L
	IDIM=M+L+1+(2*L+1)*(ISPIN-1)
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	PINV(IDIM1,IDIM)=
C    (ZMSC+EFRM)*
     &(ZMSC+DCMPLX(0.D0,0.D0))*
     &     ORAT(IU,IATOM,IUPDN0,IORBS,IPOL)-
     &     PRAT(IU,IATOM,IUPDN0,IORBS,IPOL)
	VARR(IDIM1,IDIM)=VRAT(IU,IATOM,IUPDN0,IORBS,IPOL)
	WARR(IDIM1,IDIM)=WRAT(IU,IATOM,IUPDN0,IORBS,IPOL)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Invert matrix
	CALL LIB_CINV(NDIM,PINV,NDIM,WORK,IFAIL)
	IF(IFAIL.NE.0)THEN
      WRITE(IUN,*)'<HUB_READHUB>:: Error in inversion program'
	DO IDIM1=1,NDIM
	DO IDIM=1,NDIM
	PINV(IDIM1,IDIM)=(0.D0,0.D0)
      ENDDO	
      ENDDO	
	ENDIF
C Construct product
	DO IDIM1=1,NDIM
	DO IDIM=1,NDIM
	DO JDIM1=1,NDIM
	DO JDIM=1,NDIM
	SIGL(IDIM1,IDIM)=SIGL(IDIM1,IDIM)+
     &VARR(IDIM1,JDIM1)*PINV(JDIM1,JDIM)*WARR(JDIM,IDIM) ! swapped indexes in Warr !
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO                                   !!! over poles
C DO OVER HYBRIDIZATION POLES
	DO IPOL=1,NPLDIMP(L,IATOM)              !!! over poles
C Set up matrices
	DO IDIM1=1,NDIM
	DO IDIM =1,NDIM
	PINV(IDIM1,IDIM)=(0.D0,0.D0)
	VARR(IDIM1,IDIM)=(0.D0,0.D0)
	WARR(IDIM1,IDIM)=(0.D0,0.D0)
	ENDDO
	ENDDO
	DO IORBS=1,2
	DO IUPDN=1,2
	IF(IORBS.EQ.1)THEN
	ISPIN1=IUPDN
	ISPIN =IUPDN
	ELSEIF(IORBS.EQ.2)THEN
	ISPIN1=IUPDN
	ISPIN =NSPIN+1-IUPDN
	ENDIF
	IF(NORBS.EQ.1.AND.IORBS.EQ.2)CYCLE
	IF(NSPIN.EQ.1)IUPDN0=1
	IF(NSPIN.EQ.2)IUPDN0=IUPDN
	DO K=-L,L
	IDIM1=K+L+1+(2*L+1)*(ISPIN1-1)
	DO M=-L,L
	IDIM=M+L+1+(2*L+1)*(ISPIN-1)
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	PINV(IDIM1,IDIM)=
C    (ZMSC+EFRM)*
     &(ZMSC+DCMPLX(0.D0,0.D0))*
     &     ORHB(IU,IATOM,IUPDN0,IORBS,IPOL)-
     &     PRHB(IU,IATOM,IUPDN0,IORBS,IPOL)
	VARR(IDIM1,IDIM)=VRHB(IU,IATOM,IUPDN0,IORBS,IPOL)
	WARR(IDIM1,IDIM)=WRHB(IU,IATOM,IUPDN0,IORBS,IPOL)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C Invert matrix
	CALL LIB_CINV(NDIM,PINV,NDIM,WORK,IFAIL)
	IF(IFAIL.NE.0)THEN
      WRITE(IUN,*)'<HUB_READHUB>:: Error in inversion program'
	DO IDIM1=1,NDIM
	DO IDIM=1,NDIM
	PINV(IDIM1,IDIM)=(0.D0,0.D0)
      ENDDO	
      ENDDO	
	ENDIF
C Construct product
	DO IDIM1=1,NDIM
	DO IDIM=1,NDIM
	DO JDIM1=1,NDIM
	DO JDIM=1,NDIM
	DELL(IDIM1,IDIM)=DELL(IDIM1,IDIM)+
     &VARR(IDIM1,JDIM1)*PINV(JDIM1,JDIM)*WARR(JDIM,IDIM) ! swapped indexes in Warr !
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO                                   !!! over poles
C Self-energy/Hybridization
	DO IORBS=1,2
	DO IUPDN=1,2
	IF(IORBS.EQ.1)THEN
	ISPIN1=IUPDN
	ISPIN =IUPDN
	ELSEIF(IORBS.EQ.2)THEN
	ISPIN1=IUPDN
	ISPIN =NSPIN+1-IUPDN
	ENDIF
	IF(NORBS.EQ.1.AND.IORBS.EQ.2)CYCLE
	IF(NSPIN.EQ.1)IUPDN0=1
	IF(NSPIN.EQ.2)IUPDN0=IUPDN
	DO K=-L,L
	IDIM1=K+L+1+(2*L+1)*(ISPIN1-1)
	DO M=-L,L
	IDIM=M+L+1+(2*L+1)*(ISPIN-1)
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	IW=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1 !!! (l,m k)
	SIGW(IU,IATOM,IUPDN0,IORBS)=SIGW(IU,IATOM,IUPDN0,IORBS)+
     &SIGL(IDIM1,IDIM)/2.D0*DBLE(NSPIN)
	DELW(IU,IATOM,IUPDN0,IORBS)=DELW(IU,IATOM,IUPDN0,IORBS)+
     &DELL(IDIM1,IDIM)/2.D0*DBLE(NSPIN)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	DEALLOCATE(SIGL,DELL,PINV,VARR,WARR,WORK)
	ENDDO                               !!! over l_corr
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
C Set limiting behaviour for self-energies
      DO IORBS=1,NORBS
      DO IUPDN=1,NSPIN
      DO L=0,LMAXBMEM 
      DO M=-L,L
      DO K=-L,L
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IW=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1           !  (l,k m)
C Reset Sigma(0)
	IF((IDMF.EQ.3.OR.IDMF.EQ.4.OR.IDMF.EQ.6).AND.IMSC.EQ.0)
     &SIG0(IU,IATOM,IUPDN,IORBS)=SIGW(IU,IATOM,IUPDN,IORBS)
C Do NOT Reset Sigma(infinity)
C	IF((IDMF.EQ.3.OR.IDMF.EQ.4).AND.IMSC.EQ.NMSC)
C    &SIGI(IU,IATOM,IUPDN,IORBS)=SIGW(IU,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO                               !!! over atoms
	ENDDO                               !!! over Matsubara frequencies
C MEMORY DEALLOCATIONS
      DEALLOCATE(OCP,OCS,ULR,UJR)
	END SUBROUTINE HUB_READHUB_RATP





	SUBROUTINE HUB_READHUB_SIGW(HUBDATA,NLEN,NROR,NRSP,ICUBI)
***************************************************************
*  Reads self-energy/hybridization data on imaginary axis     *
***************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                          !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE        !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV                 !!! from readhub
	USE COM_MSBMESH ,ONLY : NMSB,WMSB,KGRD                  !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
	USE COM_DMFDAT  ,ONLY : SIGW,DELW,SIG0,SIGI,DELI,GLOC   !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_LMTOUT  ,ONLY : SCRFILE                         !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from channels
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	REAL*8, ALLOCATABLE :: WMSI(:)
	COMPLEX*16, ALLOCATABLE :: SM(:)
	COMPLEX*16, ALLOCATABLE :: SL(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: SIGL(:,:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: DELL(:,:,:,:,:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
	DATA IMA/0/   !!! use D(-iW)=D(iW)^{*} property or not
	IF(IDMF.NE.3)RETURN
	IF(KGRD.NE.2)CALL FULLSTOP('GRID MUST BE IMAG FOR IMA-DMFT')
C MEMORY ALLOCATIONS
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
C INITIALIZE DMFT ARRAYS
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      GLOC(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C Estimate record lenght
	NRECL=0
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	NRECL=MAX(NRECL,IUMAX*NVAL*4*NSPIN*NORBS)
	ENDDO
	NMSI=NMSB
	ALLOCATE(WMSI(0:NMSI))
	DO IMSB=0,NMSB
	WMSI(IMSB)=WMSB(IMSB)
	ENDDO
C DO OVER TWO SECTIONS
	DO 1000 IDVR=1,2                         !!! over sigw/delw
	IF(IDVR.EQ.1)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sigw',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.2)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=delw',13,'<section=',9,'ok',2)
	IF(MSEC.LE.1)GOTO 1000
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
	IF(IDVR.EQ.1)
     &HELP='! SELF-ENERGY AT iW                     '
	IF(IDVR.EQ.2)
     &HELP='! HYBRIDIZATION FUNCTION AT iW          '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process NMSI
      NMSI=NMSB
      HELP='! # of input Matsubara frequencies      '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmsi',4,NMSI,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Reallocate storage file	
	IF(IDVR.EQ.1)THEN
	DEALLOCATE(WMSI)
	ALLOCATE(WMSI(0:NMSI))
	CLOSE(UNIT=IUSGM,STATUS='DELETE')
	OPEN(UNIT=IUSGM,FILE=SCRFILE(IUSGM),STATUS='NEW',ACCESS='DIRECT',
     &FORM='UNFORMATTED',RECL=NRECL)
	IREC=0
	DO IMSB=0,MAX(NMSB,NMSI)   !!! over Matsubara frequencies
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                      !!! over atoms
	ENDDO                      !!! over Matsubara frequencies
	ENDIF
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
      CALL LIB_TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C Estimate # of data to read
	NDAT=0
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	NDAT=NDAT+(NMSI+1)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C Loop over the file
	IPOS=2
	DO IDAT=1,NDAT                         !!! over data
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),FMT='(A)')TEXT
	STRING=HUBCOPY(IPOS)
C Process State
      SNOT='          '
      HELP='! Correlated state pointer              '
      LHLP=40
      CALL READ_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &'cstate',6,IN,1,0,RE,1,0,CM,1,0,SNOT,20,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL GET_STATE(SNOT,20,IATOM,ISPIN,MQ,L,M,ICUBC)
	IF(L.GT.LMAXBMEM.OR.L.LT.0)
     &CALL FULLSTOP('Wrong l for correlated state') 
	ISORT=IS(IATOM)
C Process frequency grid point
	HELP='                                       '
	LHLP=40
	IMSB=-1
	CALL READ_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &  'imsb',4,IMSB,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
	IF(IMSB.EQ.-1)CALL FULLSTOP('<READHUB>:: ERROR READING IMSB')
C Process frequency
	HELP='                                       '
	WFREQ=0.D0
	LHLP=40
	CALL READ_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &  'wmsb',4,IN,1,0,WFREQ,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
	WMSI(IMSB)=WFREQ
	IF(NMSB.EQ.NMSI)THEN
	IF(ABS(WMSB(IMSB)-WMSI(IMSB)).LE.1.D-05)
     &WMSI(IMSB)=WMSB(IMSB)
	ENDIF
C READ MATRIX
      NREC=NROR*NRSP*IRE*(1+2*L+1)
	IPOS=IPOS+1
      CALL HUB_READREP(OCS,LIMB2MEM,NRSP,NROR,L,ICUBI,IRE,
     &     HUBCOPY(IPOS),NREC)
      IPOS=IPOS+NREC-1
C RECORD LOCATOR
	IREC=0
	DO IMSBR=0,MAX(NMSB,NMSI)
	DO IATOMR=1,NATOM
	ISORT=IS(IATOMR)
	IF(MCRL(IATOMR).EQ.0)CYCLE
	IREC=IREC+1
	IF(IMSBR.EQ.IMSB.AND.IATOMR.EQ.IATOM)EXIT
	ENDDO
	IF(IMSBR.EQ.IMSB.AND.IATOMR.EQ.IATOM)EXIT
	ENDDO
C READ THE DATA
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL LIB_GTURNU(OCP,OCS,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &     LIMB2MEM,NRSP,NROR)
      DO IORBS=1,NORBS                       !!! over iorbs
      DO IUPDN=1,NSPIN                       !!! over iupdn
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &SIGW(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	IF(IDVR.EQ.2)
     &DELW(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO
      ENDDO
	ENDDO                                  !!! over iupdn
	ENDDO                                  !!! over iorbs
	DO IOR=1,NROR                          !!! over ior
	DO ISP=1,NRSP                          !!! over isp
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	VFC=1.D0
	IF(NSPIN.EQ.1.AND.NRSP.EQ.2)
     &VFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &SIGW(IUMK,IATOM,IUPDN,IORBS)=
     &SIGW(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
	IF(IDVR.EQ.2)
     &DELW(IUMK,IATOM,IUPDN,IORBS)=
     &DELW(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                                  !!! over idat
      DEALLOCATE(HUBCOPY)
 1000	CONTINUE                               !!! over sigw/delw
C REINTERPOLATE ONTO NEW MATSUBARA GRID
	ALLOCATE(SM(0:NMSI))
	ALLOCATE(SL(0:NMSB))
C START LOOPING OVER ATOMS
	DO 3000 IATOM=1,NATOM                  !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO 2000 L=0,ILSB(ISORT)                !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE
C ALLOCATE ARRAYS
	NM=MAX(NMSB,NMSI)
	ALLOCATE(SIGL(-L:L,-L:L,0:NM,NSPINMEM,NORBSMEM))
	ALLOCATE(DELL(-L:L,-L:L,0:NM,NSPINMEM,NORBSMEM))
C READ SELF-ENERGY
	IREC=0
	DO IMSB=0,NMSI                         !!! over Matsubara frequencies
	DO JATOM=1,NATOM                       !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                                  !!! over atoms
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	SIGL(K,M,IMSB,IUPDN,IORBS)=SIGW(IU,IATOM,IUPDN,IORBS)
	DELL(K,M,IMSB,IUPDN,IORBS)=DELW(IU,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO                                  !!! over Matsubara frequencies
C START LOOPING:
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
C SELF-ENERGY
	DO IMSI=0,NMSI
	SM(IMSI)=SIGL(K,M,IMSI,IUPDN,IORBS)
	ENDDO
	NSTA=NMSB
	DO IMSB=NMSB,0,-1
	IF(WMSB(IMSB).GE.WMSI(0))NSTA=IMSB
	ENDDO
	IF(NSTA.EQ.NMSB)CALL FULLSTOP('<HUB_READHUB_SIGW> : Wrong grid')
	NEND=0
	DO IMSB=0,NMSB
	IF(WMSB(IMSB).LE.WMSI(NMSI))NEND=IMSB
	ENDDO
	IF(NEND.EQ.0)CALL FULLSTOP('<HUB_READHUB_SIGW> : Wrong grid')
      CALL LIB_CSPLINES(NMSI+1,WMSI,SM,NEND+1-NSTA,WMSB(NSTA),SL(NSTA))
	DO IMSB=NSTA,NEND
	SIGL(K,M,IMSB,IUPDN,IORBS)=SL(IMSB)
	ENDDO
	DO IMSB=0,NSTA-1
	SIGL(K,M,IMSB,IUPDN,IORBS)=SL(NSTA)
	ENDDO
	DO IMSB=NEND+1,NMSB
	SIGL(K,M,IMSB,IUPDN,IORBS)=DCMPLX(DREAL(SL(NEND)),0.D0)
	ENDDO
C HYBRIDIZATION
	DO IMSI=0,NMSI
	SM(IMSI)=DELL(K,M,IMSI,IUPDN,IORBS)
	ENDDO
	NSTA=NMSB
	DO IMSB=NMSB,0,-1
	IF(WMSB(IMSB).GE.WMSI(0))NSTA=IMSB
	ENDDO
	IF(NSTA.EQ.NMSB)CALL FULLSTOP('<HUB_READHUB_SIGW> : Wrong grid')
	NEND=0
	DO IMSB=0,NMSB
	IF(WMSB(IMSB).LE.WMSI(NMSI))NEND=IMSB
	ENDDO
	IF(NEND.EQ.0)CALL FULLSTOP('<HUB_READHUB_SIGW> : Wrong grid')
      CALL LIB_CSPLINES(NMSI+1,WMSI,SM,NEND+1-NSTA,WMSB(NSTA),SL(NSTA))
	DO IMSB=NSTA,NEND
	DELL(K,M,IMSB,IUPDN,IORBS)=SL(IMSB)
	ENDDO
	DO IMSB=0,NSTA-1
	DELL(K,M,IMSB,IUPDN,IORBS)=SL(NSTA)
	ENDDO
	DO IMSB=NEND+1,NMSB
	DELL(K,M,IMSB,IUPDN,IORBS)=DCMPLX(0.D0,0.D0)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C STORE SELF-ENERGY/HYBRIDIZATION
	IREC=0
	DO IMSB=0,NMSB                      !!! over Matsubara frequencies
	DO JATOM=1,NATOM                    !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE           !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	SIGW(IU,IATOM,IUPDN,IORBS)=SIGL(K,M,IMSB,IUPDN,IORBS)
	DELW(IU,IATOM,IUPDN,IORBS)=DELL(K,M,IMSB,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IDN,1)=SIGW(IU,IATOM,IUP,1)
      DELW(IU,IATOM,IDN,1)=DELW(IU,IATOM,IUP,1)
	ENDDO
	ENDDO
C Set limiting values
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	IF(IORBS.EQ.1)THEN
	IDNUP=IUPDN
	ELSEIF(IORBS.EQ.2)THEN
	IDNUP=NSPIN-IUPDN+1
	ENDIF
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1           !  (l,k m)
	IW=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k)
C Sigma(0)
	IF(IMSB.EQ.0)
     &SIG0(IU,IATOM,IUPDN,IORBS)=SIGW(IU,IATOM,IUPDN,IORBS)
C Sigma(infinity) in Hermitian form
	IF(IMSB.EQ.NMSB)
     &SIGI(IU,IATOM,IUPDN,IORBS)=0.5D0*(SIGW(IU,IATOM,IUPDN,IORBS)
     &                         + DCONJG(SIGW(IW,IATOM,IDNUP,IORBS)))
C Delta(infinity)
	IF(IMSB.EQ.NMSB)
     &DELI(IU,IATOM,IUPDN,IORBS)=DELW(IU,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                               !!! over atoms
	ENDDO                               !!! over Matsubara frequencies
	DEALLOCATE(SIGL,DELL)
 2000	CONTINUE                            !!! over l
 3000	CONTINUE                            !!! over atoms
C MEMORY DEALLOCATIONS
	DEALLOCATE(OCP,OCS,ULR,UJR,SM,SL,WMSI)
	END





	SUBROUTINE HUB_READHUB_SIGR(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads Self-energy settings                          *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                          !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,KHRM,ICUBC,IRE   !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV                 !!! from readhub
	USE COM_MSBMESH ,ONLY : NOMG,WOMG,KGRD                  !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
	USE COM_IMPDAT  ,ONLY : NPOLIMP                         !!! from readhub
	USE COM_DMFDAT  ,ONLY : SIG0,SIGI,SIGW,DELW,GLOC        !!! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS,VDBC             !!! from readhub
	USE COM_RATDAT  ,ONLY : VRAT,WRAT,PRAT,ORAT             !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_LMTOUT  ,ONLY : SCRFILE                         !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from channels
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	REAL*8, ALLOCATABLE :: WMSI(:)
	COMPLEX*16, ALLOCATABLE :: SM(:)
	COMPLEX*16, ALLOCATABLE :: SL(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: SIGL(:,:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: DELL(:,:,:,:,:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
	DATA IMA/0/   !!! use D(-iW)=D(iW)^{*} property or not
	IF(IDMF.NE.5)RETURN
	IF(KGRD.NE.1)CALL FULLSTOP('GRID MUST BE REAL FOR REA-DMFT')
C MEMORY ALLOCATIONS
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
C INITIALIZE SELF-ENERGY ARRAY
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      GLOC(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C Estimate record lenght
	NRECL=0
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	NRECL=MAX(NRECL,IUMAX*NVAL*NSPIN*NORBS)
	ENDDO
	NMSI=NOMG
	ALLOCATE(WMSI(0:NMSI))
	DO IOMG=0,NOMG
	WMSI(IOMG)=WOMG(IOMG)
	ENDDO
	DO 1000 IDVR=1,2                         !!! over sigw/delw
	IF(IDVR.EQ.1)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sigr',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.2)
     &CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=delr',13,'<section=',9,'ok',2)
	IF(MSEC.LE.1)GOTO 1000
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
	IF(IDVR.EQ.1)
     &HELP='! DMFT SELF-ENERGY AT REAL AXIS         '
	IF(IDVR.EQ.2)
     &HELP='! HYBRIDIZATION FUNCTION AT REAL AXIS   '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process NOMG
      NMSI=NOMG
      HELP='! # of input frequencies                '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmsi',4,NMSI,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Reallocate storage file	
	IF(IDVR.EQ.1)THEN
	DEALLOCATE(WMSI)
	ALLOCATE(WMSI(0:NMSI))
	CLOSE(UNIT=IUSGM,STATUS='DELETE')
	OPEN(UNIT=IUSGM,FILE=SCRFILE(IUSGM),STATUS='NEW',ACCESS='DIRECT',
     &FORM='UNFORMATTED',RECL=NRECL)
	IREC=0
	DO IOMG=0,NMSI                        !!! over frequencies
	DO IATOM=1,NATOM                      !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                                  !!! over atoms
	ENDDO                                  !!! over frequencies
	ENDIF
C Start reading the data
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C Evaluate # of data to read
	NDAT=0
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	NDAT=NDAT+(NMSI+1)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C Start looping over the data
	IPOS=2
	DO IDAT=1,NDAT
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),FMT='(A)')TEXT
	STRING=HUBCOPY(IPOS)
C Process State
      SNOT='          '
      HELP='! Correlated state pointer              '
      LHLP=40
      CALL READ_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &'cstate',6,IN,1,0,RE,1,0,CM,1,0,SNOT,20,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
	CALL GET_STATE(SNOT,20,IATOM,ISPIN,MQ,L,M,ICUBC)
	IF(L.GT.LMAXBMEM.OR.L.LT.0)
     &CALL FULLSTOP('Wrong l for correlated state') 
	ISORT=IS(IATOM)
C Process frequency grid point
	HELP='                                       '
	LHLP=40
	IOMG=-1
	CALL READ_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &  'iomg',4,IOMG,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
	IF(IOMG.EQ.-1)CALL FULLSTOP('<READHUB>:: ERROR READING IOMG')
C Process Frequency
	HELP='                                       '
	LHLP=40
	CALL READ_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &  'womg',4,IN,1,0,WFREQ,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.,.FALSE.)
	WMSI(IOMG)=WFREQ
	IF(NOMG.EQ.NMSI)THEN
	IF(ABS(WOMG(IOMG)-WMSI(IOMG)).LE.1.D-05)
     &WMSI(IOMG)=WOMG(IOMG)
	ENDIF
C READ MATRIX
      NREC=NROR*NRSP*IRE*(1+2*L+1)
	IPOS=IPOS+1
      CALL HUB_READREP(OCS,LIMB2MEM,NRSP,NROR,L,ICUBI,IRE,
     &     HUBCOPY(IPOS),NREC)
      IPOS=IPOS+NREC-1
C Record locator
	IREC=0
	DO IOMGR=0,NMSI                        !!! over IOMG
	DO IATOMR=1,NATOM                      !!! over atoms
	ISORT=IS(IATOMR)
	IF(MCRL(IATOMR).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IF(IOMGR.EQ.IOMG.AND.IATOMR.EQ.IATOM)EXIT
	ENDDO
	IF(IOMGR.EQ.IOMG.AND.IATOMR.EQ.IATOM)EXIT
	ENDDO
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL LIB_GTURNU(OCP,OCS,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &     LIMB2MEM,NRSP,NROR)
      DO IORBS=1,NORBS                       !!! over iorbs
      DO IUPDN=1,NSPIN                       !!! over iupdn
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)
     &SIGW(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      IF(IDVR.EQ.2)
     &DELW(IUMK,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO
      ENDDO
	ENDDO                                  !!! over iupdn
	ENDDO                                  !!! over iorbs
	DO IOR=1,NROR                          !!! over ior
	DO ISP=1,NRSP                          !!! over isp
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	VFC=1.D0
	IF(NSPIN.EQ.1.AND.NRSP.EQ.2)
     &VFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)
     &SIGW(IUMK,IATOM,IUPDN,IORBS)=
     &SIGW(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
      IF(IDVR.EQ.2)
     &DELW(IUMK,IATOM,IUPDN,IORBS)=
     &DELW(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK,ISP,IOR)*VFC
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                                  !!! over idat
      DEALLOCATE(HUBCOPY)
C STORE SELF-ENERGY
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IREC=0
	DO IOMG=0,NMSI                         !!! over Matsubara frequencies
	DO JATOM=1,NATOM                       !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)
     &SIGW(IU,IATOM,IDN,1)=SIGW(IU,IATOM,IUP,1)
      IF(IDVR.EQ.2)
     &DELW(IU,IATOM,IDN,1)=DELW(IU,IATOM,IUP,1)
	ENDDO
	ENDDO
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                               !!! over atoms
	ENDDO                               !!! over Matsubara frequencies
      ENDDO                               !!! over l
      ENDDO                               !!! over atoms
 1000 CONTINUE
C REINTERPOLATE TO NEW REAL AXIS GRID
	ALLOCATE(SM(0:NMSI))
	ALLOCATE(SL(0:NOMG))
C START LOOPING OVER ATOMS
	DO 3000 IATOM=1,NATOM               !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO 2000 L=0,ILSB(ISORT)             !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE
C ALLOCATE ARRAYS
	NM=MAX(NOMG,NMSI)
	ALLOCATE(SIGL(-L:L,-L:L,0:NM,NSPINMEM,NORBSMEM))
	ALLOCATE(DELL(-L:L,-L:L,0:NM,NSPINMEM,NORBSMEM))
C READ SELF-ENERGY
	IREC=0
	DO IOMG=0,NMSI                         !!! over Matsubara frequencies
	DO JATOM=1,NATOM                       !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                                  !!! over atoms
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	SIGL(K,M,IOMG,IUPDN,IORBS)=SIGW(IU,IATOM,IUPDN,IORBS)
	DELL(K,M,IOMG,IUPDN,IORBS)=DELW(IU,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO                                  !!! over Matsubara frequencies
C START INTERPOLATION
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
C SELF-ENERGY
	DO IMSI=0,NMSI
	SM(IMSI)=SIGL(K,M,IMSI,IUPDN,IORBS)
	ENDDO
	NSTA=NOMG
	DO IOMG=NOMG,0,-1
	IF(WOMG(IOMG).GE.WMSI(0))NSTA=IOMG
	ENDDO
	IF(NSTA.EQ.NOMG)CALL FULLSTOP('<HUB_READHUB_SIGR> : Wrong grid')
	NEND=0
	DO IOMG=0,NOMG
	IF(WOMG(IOMG).LE.WMSI(NMSI))NEND=IOMG
	ENDDO
	IF(NEND.EQ.0)CALL FULLSTOP('<HUB_READHUB_SIGR> : Wrong grid')
      CALL LIB_CSPLINES(NMSI+1,WMSI,SM,NEND+1-NSTA,WOMG(NSTA),SL(NSTA))
	DO IOMG=NSTA,NEND
	SIGL(K,M,IOMG,IUPDN,IORBS)=SL(IOMG)
	ENDDO
	DO IOMG=0,NSTA-1
	SIGL(K,M,IOMG,IUPDN,IORBS)=SL(NSTA)
	ENDDO
	DO IOMG=NEND+1,NOMG
	SIGL(K,M,IOMG,IUPDN,IORBS)=DCMPLX(DREAL(SL(NEND)),0.D0)
	ENDDO
C HYBRIDIZATION
	DO IMSI=0,NMSI
	SM(IMSI)=DELL(K,M,IMSI,IUPDN,IORBS)
	ENDDO
	NSTA=NOMG
	DO IOMG=NOMG,0,-1
	IF(WOMG(IOMG).GE.WMSI(0))NSTA=IOMG
	ENDDO
	IF(NSTA.EQ.NOMG)CALL FULLSTOP('<HUB_READHUB_SIGR> : Wrong grid')
	NEND=0
	DO IOMG=0,NOMG
	IF(WOMG(IOMG).LE.WMSI(NMSI))NEND=IOMG
	ENDDO
	IF(NEND.EQ.0)CALL FULLSTOP('<HUB_READHUB_SIGR> : Wrong grid')
      CALL LIB_CSPLINES(NMSI+1,WMSI,SM,NEND+1-NSTA,WOMG(NSTA),SL(NSTA))
	DO IOMG=NSTA,NEND
	DELL(K,M,IOMG,IUPDN,IORBS)=SL(IOMG)
	ENDDO
	DO IOMG=0,NSTA-1
	DELL(K,M,IOMG,IUPDN,IORBS)=SL(NSTA)
	ENDDO
	DO IOMG=NEND+1,NOMG
	DELL(K,M,IOMG,IUPDN,IORBS)=DCMPLX(DREAL(SL(NEND)),0.D0)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C STORE SELF-ENERGY/HYBRIDIZATION
	IREC=0
	DO IOMG=0,NOMG                      !!! over Matsubara frequencies
	DO JATOM=1,NATOM                    !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE           !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	SIGW(IU,IATOM,IUPDN,IORBS)=SIGL(K,M,IOMG,IUPDN,IORBS)
	DELW(IU,IATOM,IUPDN,IORBS)=DELL(K,M,IOMG,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IDN,1)=SIGW(IU,IATOM,IUP,1)
      DELW(IU,IATOM,IDN,1)=DELW(IU,IATOM,IUP,1)
	ENDDO
	ENDDO
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((GLOC(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                               !!! over atoms
	ENDDO                               !!! over Matsubara frequencies
	DEALLOCATE(SIGL,DELL)
 2000 CONTINUE                            !!! over l
 3000 CONTINUE                            !!! over atoms
C MEMORY DEALLOCATIONS
      DEALLOCATE(OCP,OCS,ULR,UJR,WMSI)
	END SUBROUTINE HUB_READHUB_SIGR



	SUBROUTINE HUB_READHUB_SHOW
*******************************************************
*  Shows header of HUBFILE.                           * 
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                                 !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB                !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                                      !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                                    !!! from readini
	USE COM_MSBMESH ,ONLY : NMSB,NMSL,TEMP,OBND,WMIN,WMAX,NOMG,KGRD        !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT,EPSDMF,MIXBRD,
     &                        ADMIXL,ADMIX1,MIXFUN                           !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE,IWR                   !!! from readhub
     &                       ,KDELFUN,KDELINT,KDELRAT,KSOLVER,KDBLCNT
     &                       ,CDBLCNT,KSIGMOO,KIMPLEV,KFIXSIG
     &                       ,KSIGFUN,KSIGINT,KSIGRAT
     &                       ,KHRM,KSUM,PDMP,GDMP                            !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG                !!! from readhub
     &                       ,RINPINV,ROUTINV
     &                       ,CHSYS,CHAXE,CHANG,CHINV
     &                       ,CHMAG,CHSPL,CHISP
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                                      !!! from readhub
	USE COM_IMPDAT  ,ONLY : KCRDSYS,NRUNIMP,KSTATES,IMPSECT,
     &                        IMPINFO,NPOLIMP,NPLDIMP,AMAGIMP,
     &                        BSPLIMP,ISPLIMP,IMPDATA
      USE COM_ORDDAT  ,ONLY : KORD,NORD,OEPS,OSIG,NSIG                       !!! from readhub
	USE COM_FERMI   ,ONLY : EF,ZBND,KFRM                                   !!! from ttrints
      USE COM_LMTINP  ,ONLY : ICHUB                                          !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL                              !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                                    !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
	IF(ICHUB.LE.0)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
C ARE THERE CORRELATIONS?
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
C START STORAGE
      WRITE(IUN,10000)
10000 FORMAT(/' <<<--- HUBFILE READ --->>>'/)
	WRITE(IUN,10100)
	WRITE(IUN,10110)
10100	FORMAT(1X,'<FILE=HUBFILE,INPUT=MODERN,TRACE=TRUE>')
10110 FORMAT('****************************************************')
C PRINT CONTROL PARAMETERS
	WRITE(IUN,10120)
10120 FORMAT(1X,'<SECTION=CTRL>',14X,'! CONTROL PARAMETERS:')
C Process SCHM
      HELP='! LDA+U/C/IMA/RAT/REA/MAS               '
	IF(IDMF.EQ.0)TCTRL='LDA      '      
	IF(IDMF.EQ.1)TCTRL='LDA+C    '    
	IF(IDMF.EQ.2)TCTRL='LDA+U    ' 
	IF(IDMF.EQ.3)TCTRL='IMA-DMFT '  
	IF(IDMF.EQ.4)TCTRL='RAT-DMFT '
	IF(IDMF.EQ.5)TCTRL='REA-DMFT '
	IF(IDMF.EQ.6)TCTRL='QUP-DMFT '
	WRITE(IUN,10130)TCTRL,HELP
10130 FORMAT(1X,'Scheme=',A10,11X,A40)
C Process CUBC
      HELP='! Cubic/Spherical harmonics (output)    '
	IF(ICUBC.EQ.1)TCTRL='cubic    '
	IF(ICUBC.EQ.2)TCTRL='spherical'
	IF(ICUBC.EQ.3)TCTRL='spinor'
	WRITE(IUN,10150)TCTRL,HELP
10150 FORMAT(1X,'Yharm =',A10,11X,A40)
C Process KOR
      IF(NORBS.EQ.1)TCTRL='One     '
      IF(NORBS.EQ.2)TCTRL='Both    '
      HELP='! One/Both orbits to read               '
	WRITE(IUN,10170)TCTRL,HELP
10170 FORMAT(1X,'Rorbs =',A10,11X,A40)
C Process KSP
      IF(NSPIN.EQ.1)TCTRL='One     '
      IF(NSPIN.EQ.2)TCTRL='Both    '
      HELP='! One/Both spins  to read               '
	WRITE(IUN,10180)TCTRL,HELP
10180 FORMAT(1X,'Rspin =',A10,11X,A40)
C Process REIM
      HELP='! Real/Complex input/output             '
	IF(IRE.EQ.1)TCTRL='real      '
	IF(IRE.EQ.2)TCTRL='complex   '
	WRITE(IUN,10190)TCTRL,HELP
10190 FORMAT(1X,'Format=',A10,11X,A40)
C Process NHUB
      HELP='! # of correlated states                '
	WRITE(IUN,10200)NHUB,HELP
10200 FORMAT(1X,'Ncrl  =',I3,18X,A40)
C PRINT DMFT DATA
	WRITE(IUN,10310)
10310 FORMAT(1X,'<SECTION=DMFT>',14X,'! DMFT SETTINGS:')
C Process NMSB
      HELP='! # of Matsubara frequencies            '
	WRITE(IUN,10330)NMSB,HELP
10330 FORMAT(1X,'Nmsb  =',I10,11X,A40)
C Process OBND
      HELP='! Effective bandwidth for the grid      '
	WRITE(IUN,10331)OBND,HELP
10331 FORMAT(1X,'Obnd  =',G14.7,7X,A40)
C Process NMSL
      HELP='! # of linear Matsubara frequencies     '
	WRITE(IUN,10332)NMSL,HELP
10332 FORMAT(1X,'Nmsl  =',I10,11X,A40)
C Process NOMG
      HELP='! # of frequency points on the real axis'
	WRITE(IUN,10333)NOMG,HELP
10333 FORMAT(1X,'Nomg  =',I10,11X,A40)
C Process WMIN
      HELP='! Leftmost real axis frequency          '
	WRITE(IUN,10334)WMIN,HELP
10334 FORMAT(1X,'Wmin  =',G14.7,7X,A40)
C Process WMAX
      HELP='! Ritemost real axis frequency          '
	WRITE(IUN,10335)WMIN,HELP
10335 FORMAT(1X,'Wmax  =',G14.7,7X,A40)
C Process KGRD
      HELP='! Ritemost real axis frequency          '
	IF(KGRD.EQ.1)TCTRL=' Real     '
	IF(KGRD.EQ.2)TCTRL=' Imag     '
	WRITE(IUN,10336)TCTRL,HELP
10336 FORMAT(1X,'Kgrd  =',A10,11X,A40)
C Process Ef
      HELP='! Fermi energy (Ry)                     '
	WRITE(IUN,10337)EF,HELP
10337 FORMAT(1X,'Chem=',G14.7,7X,A40)
C Process Kfrm
      HELP='! Fermi search options                 '
	IF(KFRM.EQ.0)TCTRL=' Auto     '
	IF(KFRM.EQ.1)TCTRL=' Band     '
	IF(KFRM.EQ.2)TCTRL=' Keep     '
	WRITE(IUN,10338)TCTRL,HELP
10338 FORMAT(1X,'Kfrm=',A10,11X,A40)
C Process Zbnd
      HELP='! # of particles for Ef                 '
	WRITE(IUN,10339)ZBND,HELP
10339 FORMAT(1X,'Fill=',G14.7,7X,A40)
C PRINT DMFT DATA
	WRITE(IUN,10340)
10340 FORMAT(1X,'<SECTION=ITER>',14X,'! DMFT LOOP SETTINGS:')
C Process NDIT
      HELP='! # of DMFT iterations                  '
	WRITE(IUN,10341)NDIT,HELP
10341 FORMAT(1X,'Ndit  =',I10,11X,A40)
C Process Broyden
      HELP='! Use Broyden mixing for DMFT loop      '
	IF(MIXBRD)WRITE(IUN,10342)HELP
	IF(.NOT.MIXBRD)WRITE(IUN,10343)HELP
10342 FORMAT(1X,'Broy  = yes',17X,A40)
10343 FORMAT(1X,'Broy  = no ',17X,A40)
C Process Mixing function
      HELP='! Mix Delta(w)/Sigma(w) in DMFT loop    '
	IF(MIXFUN.EQ.1)WRITE(IUN,10344)HELP
	IF(MIXFUN.EQ.2)WRITE(IUN,10345)HELP
10344 FORMAT(1X,'MixFun= Delta',15X,A40)
10345 FORMAT(1X,'MixFun= Sigma',15X,A40)
C Process AdmixL
      HELP='! Mixing parameter in DMFT loop         '
	WRITE(IUN,10346)ADMIXL,HELP
10346 FORMAT(1X,'Mixing=',G14.7,7X,A40)
C Process Admix1
      HELP='! Mixing parameter to start DMFT        '
	WRITE(IUN,10347)ADMIX1,HELP
10347 FORMAT(1X,'Admix1=',G14.7,7X,A40)
C Process EpsDMF
      HELP='! Accuracy                              '
	WRITE(IUN,10348)EPSDMF,HELP
10348 FORMAT(1X,'EpsDMF=',G14.7,7X,A40)
C PRINT GLOC DATA
	WRITE(IUN,10410)
10410 FORMAT(1X,'<SECTION=GLOC>',14X,'! GREEN FUNCTIONS K SUMS')
C Process Khrm
      HELP='! Use Hermitianness of Hamiltonian      '
	IF(KHRM.EQ.0)TCTRL=' None     '
	IF(KHRM.EQ.1)TCTRL=' True     '
	WRITE(IUN,10412)TCTRL,HELP
10412 FORMAT(1X,'Hermit=',A10,11X,A40)
C Process Ksum
      HELP='! K-sums for Green function ttrs/sums   '
	IF(KSUM.EQ.0)TCTRL=' TTRs     '
	IF(KSUM.EQ.1)TCTRL=' Ksum     '
	WRITE(IUN,10413)TCTRL,HELP
10413 FORMAT(1X,'KeySum=',A10,11X,A40)
C Process Pdmp
      HELP='! Damping in pole expansion for Sigma(w)'
	WRITE(IUN,10414)PDMP,HELP
10414 FORMAT(1X,'DamSig=',G14.7,7X,A40)
C Process Gdmp
      HELP='! Damping in Green function k-sum       '
	WRITE(IUN,10416)GDMP,HELP
10416 FORMAT(1X,'DamGrf=',G14.7,7X,A40)
C Process KORD
      HELP='! Ordering scheme: func/imax            '
      IF(KORD.EQ.1)TCTRL=' Func     '
      IF(KORD.EQ.2)TCTRL=' Imax     '
	WRITE(IUN,10418)TCTRL,HELP
10418 FORMAT(1X,'Korder=',A10,11X,A40)
C Process NORD
      HELP='! # of nearest bands to consider        '
	WRITE(IUN,10420)NORD,HELP
10420 FORMAT(1X,'Norder=',I10,11X,A40)
C Process NSIG
      HELP='! # of correlated bands                 '
	WRITE(IUN,10425)NSIG,HELP
10425 FORMAT(1X,'OrdBnd=',I10,11X,A40)
C Process OEPS
      HELP='! Ordering accuracy                     '
	WRITE(IUN,10430)OEPS,HELP
10430 FORMAT(1X,'OrdEps=',G14.7,7X,A40)
C Process OSIG
      HELP='! Max.imaginary self-energy             '
	WRITE(IUN,10440)OSIG,HELP
10440 FORMAT(1X,'OrdSig=',G14.7,7X,A40)
C PRINT CORRELATED STATES
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	WRITE(IUN,10510)
10510 FORMAT(1X,'<SECTION=CORR>',14X,'! CORRELATED STATES:')
C Process LHUB
      HELP='! Correlated state pointer              '
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(IUN,10540)TCTRL,HELP
10540 FORMAT(1X,'cState  =',A10,9X,A40)
C ************************
C Output coordinate system
      HELP='! global/local coordinate system        '
	CALL FIND_LBLANK(CHSYS(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUN,10550)CHSYS(L,IATOM),HELP
10550 FORMAT(1X,'OutSys  =',A10,9X,A40)
      HELP='! rotational axis                       '
	CALL FIND_LBLANK(CHAXE(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUN,10560)CHAXE(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUN,10565)CHAXE(L,IATOM)
10560 FORMAT(1X,'OutAxis =',A18,1X,A40)
10565 FORMAT(1X,'OutAxis =',A80)
      HELP='! rotational angle                      '
	CALL FIND_LBLANK(CHANG(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUN,10570)CHANG(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUN,10575)CHANG(L,IATOM)
10570 FORMAT(1X,'OutAngle=',A18,1X,A40)
10575 FORMAT(1X,'OutAngle=',A80)
C Apply inversion
      HELP='! apply inversion after rotation        '
	CALL FIND_LBLANK(CHINV(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUN,10580)CHINV(L,IATOM),HELP
10580 FORMAT(1X,'OutInv  =',A10,9X,A40)
C ************************
C Input coordinate system
      HELP='! global/local coordinate system        '
	CALL FIND_LBLANK(CHSYS(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUN,10590)CHSYS(L,IATOM),HELP
10590 FORMAT(1X,'InpSys  =',A10,9X,A40)
      HELP='! rotational axis                       '
	CALL FIND_LBLANK(CHAXE(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUN,10600)CHAXE(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUN,10605)CHAXE(L,IATOM)
10600 FORMAT(1X,'InpAxis =',A18,1X,A40)
10605 FORMAT(1X,'InpAxis =',A80)
      HELP='! rotational angle                      '
	CALL FIND_LBLANK(CHANG(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUN,10610)CHANG(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUN,10615)CHANG(L,IATOM)
10610 FORMAT(1X,'InpAngle=',A18,1X,A40)
10615 FORMAT(1X,'InpAngle=',A80)
C Apply inversion
      HELP='! apply inversion after rotation        '
	CALL FIND_LBLANK(CHINV(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUN,10620)CHINV(L,IATOM),HELP
10620 FORMAT(1X,'InpInv  =',A10,9X,A40)
C **************************************
C Process Slater Integrals, F(0,ll),F(2,ll),F(4,ll), etc.
	DO L2=0,2*L,2
      HELP='! Slater integrals                      '
	IF(L2.EQ.0)
     &WRITE(IUN,10710)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.2)
     &WRITE(IUN,10720)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.4)
     &WRITE(IUN,10730)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.6)
     &WRITE(IUN,10740)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.8)
     &WRITE(IUN,10750)FSLU(L2,L,L,IATOM),HELP
10710 FORMAT(1X,'F0 =',G14.7,10X,A40)
10720 FORMAT(1X,'F2 =',G14.7,10X,A40)
10730 FORMAT(1X,'F4 =',G14.7,10X,A40)
10740 FORMAT(1X,'F6 =',G14.7,10X,A40)
10750 FORMAT(1X,'F8 =',G14.7,10X,A40)
	ENDDO
C **************************************
C Process Sigma Calculation Scheme
      HELP='! Sigma Calculation Scheme              '
	IF(KSIGFUN(L,IATOM).EQ.0)TCTRL='none      '
	IF(KSIGFUN(L,IATOM).EQ.1)TCTRL='MatInv    '  
	IF(KSIGFUN(L,IATOM).EQ.2)TCTRL='CrfInv    '  
	IF(KSIGFUN(L,IATOM).EQ.3)TCTRL='EigenV    '  
	WRITE(IUN,10751)TCTRL,HELP
10751 FORMAT(1X,'SigFun =',A10,10X,A40)
C **************************************
C Process Delta Calculation Scheme
      HELP='! Delta Calculation Scheme              '
	IF(KDELFUN(L,IATOM).EQ.0)TCTRL='none      '
	IF(KDELFUN(L,IATOM).EQ.1)TCTRL='MatInv    '  
	IF(KDELFUN(L,IATOM).EQ.2)TCTRL='CrfInv    '  
	IF(KDELFUN(L,IATOM).EQ.3)TCTRL='EigenV    '  
	WRITE(IUN,10752)TCTRL,HELP
10752 FORMAT(1X,'DelFun =',A10,10X,A40)
C **************************************
C Process Sigma interpolation method
      HELP='! Sigma interpolation method            '
	IF(KSIGINT(L,IATOM).EQ. 0)TCTRL='none      '
	IF(KSIGINT(L,IATOM).EQ. 1)TCTRL='Mom2      '  
	IF(KSIGINT(L,IATOM).EQ. 2)TCTRL='Mom4      '
	IF(KSIGINT(L,IATOM).EQ. 3)TCTRL='Mom6      '
	IF(KSIGINT(L,IATOM).EQ. 4)TCTRL='MomN      '
	IF(KSIGINT(L,IATOM).EQ. 5)TCTRL='Grid      '
	IF(KSIGINT(L,IATOM).EQ. 6)TCTRL='Conj      '
	IF(KSIGINT(L,IATOM).EQ. 7)TCTRL='Pade      '
	WRITE(IUN,10753)TCTRL,HELP
10753 FORMAT(1X,'SigInt =',A10,10X,A40)
C **************************************
C Process Delta interpolation method
      HELP='! Delta interpolation method            '
	IF(KDELINT(L,IATOM).EQ. 0)TCTRL='none      '
	IF(KDELINT(L,IATOM).EQ. 1)TCTRL='Mom1      '  
	IF(KDELINT(L,IATOM).EQ. 2)TCTRL='Mom2      '
	IF(KDELINT(L,IATOM).EQ. 3)TCTRL='Mom3      '
	IF(KDELINT(L,IATOM).EQ. 4)TCTRL='Mom4      '
	IF(KDELINT(L,IATOM).EQ. 5)TCTRL='Mom5      '
	IF(KDELINT(L,IATOM).EQ. 6)TCTRL='Mom6      '
	IF(KDELINT(L,IATOM).EQ. 7)TCTRL='MomN      '
	IF(KDELINT(L,IATOM).EQ. 8)TCTRL='Grid      '
	IF(KDELINT(L,IATOM).EQ. 9)TCTRL='Conj      '
	IF(KDELINT(L,IATOM).EQ.10)TCTRL='Pade      '
	WRITE(IUN,10754)TCTRL,HELP
10754 FORMAT(1X,'DelInt =',A10,10X,A40)
C **************************************
C Force rational Sigma representation
      HELP='! Sigma representation: SigmaW/RatInt   '
	IF(KSIGRAT(L,IATOM).EQ.0)TCTRL='SigmaW    '
	IF(KSIGRAT(L,IATOM).EQ.1)TCTRL='RatInt    '  
	WRITE(IUN,10755)TCTRL,HELP
10755 FORMAT(1X,'SigRat =',A10,10X,A40)
C **************************************
C Force rational Delta representation
      HELP='! Delta representation: DeltaW/RatInt   '
	IF(KDELRAT(L,IATOM).EQ.0)TCTRL='DeltaW    '
	IF(KDELRAT(L,IATOM).EQ.1)TCTRL='RatInt    '  
	WRITE(IUN,10756)TCTRL,HELP
10756 FORMAT(1X,'DelRat =',A10,10X,A40)
C **************************************
C Process Impurity Solver Choice
      HELP='! impurity solver choice                '
	IF(KSOLVER(L,IATOM).EQ.0 )TCTRL='None      '
	IF(KSOLVER(L,IATOM).EQ.11)TCTRL='HarFock   '  
	IF(KSOLVER(L,IATOM).EQ.21)TCTRL='SunHub1   '
	IF(KSOLVER(L,IATOM).EQ.22)TCTRL='CrfHub1   '
	IF(KSOLVER(L,IATOM).EQ.31)TCTRL='SunIrat   '
	IF(KSOLVER(L,IATOM).EQ.32)TCTRL='CrfIrat   '
	IF(KSOLVER(L,IATOM).EQ.41)TCTRL='ReadSig   '
 	IF(KSOLVER(L,IATOM).EQ.71)TCTRL='RatFile   '
 	IF(KSOLVER(L,IATOM).EQ.81)TCTRL='QmcFile   '
	WRITE(IUN,10760)TCTRL,HELP
10760 FORMAT(1X,'Solver =',A10,10X,A40)
C **************************************
C Process Self-Energy Freezer
      HELP='! Fix self-energy [on/off]              '
	IF(KFIXSIG(L,IATOM).EQ.0)TCTRL='off       '
	IF(KFIXSIG(L,IATOM).EQ.1)TCTRL='on        '  
	WRITE(IUN,10761)TCTRL,HELP
10761 FORMAT(1X,'FixSig =',A10,10X,A40)
C **************************************
C Process Sigma(oo)
      HELP='! Sigma(oo) choice                      '
	IF(KSIGMOO(L,IATOM).EQ.0 )TCTRL='None      '
	IF(KSIGMOO(L,IATOM).EQ.11)TCTRL='U(Nscf)   '  
	IF(KSIGMOO(L,IATOM).EQ.12)TCTRL='U(Nimp)   '
	IF(KSIGMOO(L,IATOM).EQ.21)TCTRL='U(Ns-n)   '  
	IF(KSIGMOO(L,IATOM).EQ.22)TCTRL='U(Is-I)   '
	IF(KSIGMOO(L,IATOM).EQ.31)TCTRL='SigmaOO   '  
	IF(KSIGMOO(L,IATOM).EQ.41)TCTRL='SigmaIN   '
	WRITE(IUN,10762)TCTRL,HELP
10762 FORMAT(1X,'SigmaI =',A10,10X,A40)
C **************************************
C Process impurity level choice
      HELP='! impurity level choice                 '
	IF(KIMPLEV(L,IATOM).EQ.0 )TCTRL='None      '
	IF(KIMPLEV(L,IATOM).EQ.11)TCTRL='MatLDA    '  
	IF(KIMPLEV(L,IATOM).EQ.12)TCTRL='SunLDA    ' 
	IF(KIMPLEV(L,IATOM).EQ.21)TCTRL='MatVdc    '  
	IF(KIMPLEV(L,IATOM).EQ.22)TCTRL='SunVdc    '  
	IF(KIMPLEV(L,IATOM).EQ.30)TCTRL='FixLev    '  
	WRITE(IUN,10763)TCTRL,HELP
10763 FORMAT(1X,'ImpLev =',A10,10X,A40)
C **************************************
C Process double counting choice
      HELP='! double counting choice                '
	IF(KDBLCNT(L,IATOM).EQ.0 )TCTRL='None      '
	IF(KDBLCNT(L,IATOM).EQ.11)TCTRL='U(Ns-a)   '  
	IF(KDBLCNT(L,IATOM).EQ.12)TCTRL='SunNscf   '  ! U(Na-a)
	IF(KDBLCNT(L,IATOM).EQ.13)TCTRL='U(Is-a)   '  
	IF(KDBLCNT(L,IATOM).EQ.14)TCTRL='U(ia-a)   '  
	IF(KDBLCNT(L,IATOM).EQ.21)TCTRL='U(Ns-N)   '  
	IF(KDBLCNT(L,IATOM).EQ.22)TCTRL='U(Na-N)   '  
	IF(KDBLCNT(L,IATOM).EQ.23)TCTRL='U(Is-I)   '  
	IF(KDBLCNT(L,IATOM).EQ.24)TCTRL='U(Ia-I)   '  
	IF(KDBLCNT(L,IATOM).EQ.30)TCTRL='FixVdc    '  
	IF(KDBLCNT(L,IATOM).EQ.40)TCTRL='FixLev    '  
	IF(KDBLCNT(L,IATOM).EQ.50)TCTRL='Sigma0    '
	IF(KDBLCNT(L,IATOM).EQ.60)TCTRL='SigmaI    '
	WRITE(IUN,10765)TCTRL,HELP
10765 FORMAT(1X,'Dcount =',A10,10X,A40)
C **************************************
C Process double counting constant
      HELP='! Constant in U(n-a) double counting    '
	WRITE(IUN,10766)CDBLCNT(L,IATOM),HELP
10766 FORMAT(1X,'Aconst =',G12.5,8X,A40)
C **************************************
C Process iSpl
	HELP='! splitting key: auto/always/kickup     '
	CALL FIND_LBLANK(CHISP(L,IATOM),10,NBL)
	IF(ISPLIMP(L,IATOM).EQ.0)TCTRL='auto      '
	IF(ISPLIMP(L,IATOM).EQ.1)TCTRL='always    '
	IF(ISPLIMP(L,IATOM).EQ.2)TCTRL='kickup    '
	IF(NBL.GT.0)
     &WRITE(IUN,10771)TCTRL,HELP
10771 FORMAT(1X,'iSpl =',A10,12X,A40)
C Process Bspl
	HELP='! initial splitting                     '
	CALL FIND_LBLANK(CHSPL(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUN,10772)BSPLIMP(L,IATOM),HELP
10772 FORMAT(1X,'Split =',G14.7,7X,A40)
C Process Magnetization Axis
	HELP='! local magnetization axis              '
	CALL FIND_LBLANK(CHMAG(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUN,10773)CHMAG(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUN,10774)CHMAG(L,IATOM)
10773 FORMAT(1X,'AxMag =',A18,1X,A40)
10774 FORMAT(1X,'AxMag =',A80)
C **************************************
C Process coordinate system for impurity solver
      HELP='! system cubic/spherical/spinor         '
	IF(KCRDSYS(L,IATOM).EQ.1)TCTRL='cubic     '
	IF(KCRDSYS(L,IATOM).EQ.2)TCTRL='spherical '
	IF(KCRDSYS(L,IATOM).EQ.3)TCTRL='spinor    '
	IF(KCRDSYS(L,IATOM).EQ.4)TCTRL='diagonal  '
	IF(KCRDSYS(L,IATOM).EQ.5)TCTRL='custom    '
	WRITE(IUN,10775)TCTRL,HELP
10775 FORMAT(1X,'CrdSys =',A10,10X,A40)
C **************************************
C Process # of runs for impurity solver
      HELP='! # of runs for impurity solver         '
	WRITE(IUN,10780)NRUNIMP(L,IATOM),HELP
10780 FORMAT(1X,'ImpRun =',I10,10X,A40)
C **************************************
C For each run process states which participate
	NDEG=2*(2*L+1)
	DO IRUN=1,NRUNIMP(L,IATOM)  !!! over runs
      HELP='! states pointer in each run            '
	IF(L.EQ.0)
     &WRITE(IUN,10785)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG),HELP
10785 FORMAT(1X,'kStates(:)=',2(I1,1X),13X,A40)
	IF(L.EQ.1)
     &WRITE(IUN,10786)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG),HELP
10786 FORMAT(1X,'kStates(:)=',3(I1,1X),3X,3(I1,1X),1X,A40)
	IF(L.EQ.2)
     &WRITE(IUN,10787)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG)
10787 FORMAT(1X,'kStates(:)=',5(I2,1X),3X,5(I2,1X))
	IF(L.EQ.3)
     &WRITE(IUN,10788)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG)
10788 FORMAT(1X,'kStates(:)=',7(I2,1X),3X,7(I2,1X))
	ENDDO
C Process NPOL
      HELP='! # of poles for self-energy            '
	WRITE(IUN,10794)NPOLIMP(L,IATOM),HELP
10794 FORMAT(1X,'nPol  =',I10,11X,A40)
C Process NPLD
      HELP='! # of poles for hybridization          '
	WRITE(IUN,10795)NPLDIMP(L,IATOM),HELP
10795 FORMAT(1X,'nPld  =',I10,11X,A40)
C Process ImpSect
      HELP='! additional information                '
	IF(IMPSECT(L,IATOM).EQ.0)TCTRL='None      '
	IF(IMPSECT(L,IATOM).EQ.1)TCTRL='Auto      '  
	IF(IMPSECT(L,IATOM).EQ.2)TCTRL='Save      '
	WRITE(IUN,10796)TCTRL,HELP
10796 FORMAT(1X,'ImpInf =',A10,10X,A40)
C **************************************
C PRINT ADDITIONAL INFORMATION DATA
	WRITE(IUN,10810)
10810 FORMAT(1X,'<Subsection=INFO>',11X,'! ADDITION INFORMATION:')
	IF(IMPINFO(L,IATOM).EQ.0)THEN
	WRITE(IUN,10811)
10811	FORMAT('!!! No data available.')
	ELSE
	DO IREC=1,IMPINFO(L,IATOM)
	CALL FIND_LBLANK(IMPDATA(IREC,L,IATOM),150,NBL)
	WRITE(IUN,10813)IMPDATA(IREC,L,IATOM)(1:NBL)
	ENDDO
10813 FORMAT(1X,A)
	ENDIF
	ENDDO                                  !!! over correlated states
	ENDDO                                  !!! over correlated states
C MEMORY DEALLOCATIONS
	DEALLOCATE(FHUB,OCP,OCS)
	END








	SUBROUTINE HUB_WRITEHUB(KW)
*******************************************************
*  Updates HUBFILE.                                   * 
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_LMTOUT  ,ONLY : ICHBR,HBRFILE                   !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC            !!! from readhub
      USE COM_IMPDAT  ,ONLY : IMPSECT,IMPINFO,IMPDATA         !!! from readhub
      USE COM_HUBSYS  ,ONLY : CHSYS,CHAXE,CHANG,CHINV         !!! from readhub
      USE COM_MSBMESH ,ONLY : KGRD                            !!! from readhub
	USE COM_FERMI   ,ONLY : EF                              !!! from ttrints
      USE COM_CHANNELS,ONLY : IUINP                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*150, ALLOCATABLE :: HUBDATA(:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL EXIST,DEF,ASK,VAL,NTH,MDF
      CHARACTER STRING*150,FILE*130,HELP*40,TCTRL*10
      DATA PI/3.141592653589793D0/
C RETURN KEYS
	IF(ICHUB.EQ.0.AND.KW.EQ.1)RETURN
	IF(ICHBR.EQ.0.AND.KW.EQ.2)RETURN
C ********************************
C UNDERSTAND THE LENGHT OF HUBFILE
      CALL FILE_LENGHT(HUBFILE,NLEN,IUINP)
      ALLOCATE(HUBDATA(NLEN))
C ********************************
C READ HUBFILE TO INTERNAL FILE
      OPEN(UNIT=IUINP,FILE=HUBFILE,STATUS='OLD',FORM='FORMATTED')
      DO ILEN=1,NLEN
      READ(IUINP,FMT='(A150)')HUBDATA(ILEN)
      ENDDO
      CLOSE(IUINP)
C OPEN BLANK FILE
	IF(KW.EQ.1)FILE=HUBFILE
	IF(KW.EQ.2)FILE=HBRFILE
      INQUIRE(FILE=FILE,EXIST=EXIST)
      IF(EXIST)THEN
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='OLD',FORM='FORMATTED')
	CLOSE(UNIT=IUINP,STATUS='DELETE')
	ENDIF
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='NEW',FORM='FORMATTED')
C START STORAGE
	WRITE(IUINP,10100)
	WRITE(IUINP,10110)
10100	FORMAT(1X,'<FILE=HUBFILE,INPUT=MODERN,TRACE=TRUE>')
10110 FORMAT('****************************************************')
10120	FORMAT(150(A1))
C PRINT CONTROL PARAMETERS
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=ctrl',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
C Upgrade IHarm
      IF(ICUBC.EQ.1)TCTRL='Cubic     '
      IF(ICUBC.EQ.2)TCTRL='Spherical '
      IF(ICUBC.EQ.3)TCTRL='Spinor    '
      HELP='! Cubic/Spherical harmonics (input)     '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'iharm',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.TRUE.)
C Upgrade NROR
      IF(NORBS.EQ.1)TCTRL='One       '
      IF(NORBS.EQ.2)TCTRL='Both      '
      HELP='! One/Both orbits to read               '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'rorbs',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.TRUE.)
C Upgrade NRSP
      IF(NSPIN.EQ.1)TCTRL='One       '
      IF(NSPIN.EQ.2)TCTRL='Both      '
      HELP='! One/Both spins  to read               '
      LHLP=40
      CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'rspin',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.TRUE.)
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C PRINT DMFT DATA
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dmft',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
C Upgrade Fermi level position
      EFRM=EF
      HELP='! Fermi energy (Ry)                     '
	LHLP=40
	CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &  'chem',4,IN,1,0,EFRM,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.TRUE.,.TRUE.)
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C PRINT ITER DATA
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=iter',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C PRINT GLOC DATA
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=gloc',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C PRINT CORRELATED STATES
	ILOC=1
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
      CALL READ_SECT(HUBDATA(ILOC),NLEN-ILOC+1,KSEC,NSEC,MSEC,
     &'<section=corr',13,'<section=',9,'ok',2)
	ISEC=KSEC+ILOC-1
	ILOC=ISEC+1
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Upgrade input coordinate system to output system
C System
      HELP='! global/local coordinate system        '
      LHLP=40
	CALL FIND_LBLANK(CHSYS(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpsys',6,IN,1,0,RE,1,0,CM,1,0,CHSYS(L,IATOM),10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.TRUE.)
C Rotation axis
      HELP='! rotational axe                        '
      LHLP=40
	CALL FIND_LBLANK(CHAXE(L,IATOM),80,NBL)
	IF(NBL.GT.0)
     &CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpaxis',7,IN,1,0,RE,1,0,CM,1,0,CHAXE(L,IATOM),80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.TRUE.)
C Rotation angle
      HELP='! rotational angle                      '
      LHLP=40
	CALL FIND_LBLANK(CHANG(L,IATOM),80,NBL)
	IF(NBL.GT.0)
     &CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpangle',8,IN,1,0,RE,1,0,CM,1,0,CHANG(L,IATOM),80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.TRUE.)
C Inversion
      HELP='! apply inversion after rotation        '
      LHLP=40
	CALL FIND_LBLANK(CHINV(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &CALL READ_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpinv',6,IN,1,0,RE,1,0,CM,1,0,CHINV(L,IATOM),10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.,.FALSE.)
C Locate subsection INFO
	CALL READ_SECT(HUBDATA(ISEC),NSEC,ISEC1,
     &  NSEC1,MSEC1,'<subsection=info',16,'<subsection=',12,'ok',2)
C Write all information without this subsection
      DO ILIN=1,MSEC-MSEC1
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C If subsection is present, upgrade it as well
	IF(IMPINFO(L,IATOM).GT.0)THEN
	WRITE(IUINP,10200)
10200 FORMAT(1X,'<Subsection=INFO>',14X,'! Additional INFO:')
      DO IREC=1,IMPINFO(L,IATOM)
	STRING=IMPDATA(IREC,L,IATOM)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	ENDIF
	ENDDO                                  !!! over correlated states
	ENDDO                                  !!! over correlated states
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	DO IDVR=1,8
	CALL HUB_WRITEHUB_DATA(IUINP,IDVR)
	ENDDO
C PRINT TRANSFORMATION MATRICES
	CALL HUB_WRITEHUB_CSYS(IUINP)
C ***********************************************
C PRINT RATONAL INTERPOLATION DATA
C ***********************************************
	IF(IDMF.EQ.3.OR.IDMF.EQ.4.OR.IDMF.EQ.5.OR.IDMF.EQ.6)THEN
	DO IDVR=1,2
	CALL HUB_WRITEHUB_RATP(IUINP,IDVR)
	ENDDO
	ENDIF
C ***********************************************
C PRINT SELF-ENERGY AT IMAGINARY AXIS
C ***********************************************
      IF(IDMF.EQ.3.OR.IDMF.EQ.4.OR.IDMF.EQ.5.OR.IDMF.EQ.6)THEN
      IF(KGRD.EQ.1)THEN
C Keep Imaginary Axis Self-energy in the new file
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sigw',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C Keep Imaginary Axis Hybridization in the new file
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=delw',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C Update imaginary axis self-energy/hybridization if available
	ELSEIF(KGRD.EQ.2)THEN
	DO IDVR=1,2
	CALL HUB_WRITEHUB_SIGW(IUINP,IDVR)
	ENDDO
	ENDIF
	ENDIF
C *********************************************	
C PRINT SELF-ENERGY AT REAL AXIS
C *********************************************
      IF(IDMF.EQ.3.OR.IDMF.EQ.4.OR.IDMF.EQ.5.OR.IDMF.EQ.6)THEN
      IF(KGRD.EQ.2)THEN
C Keep real axis Self energy in the new file
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sigr',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C Keep real axis Hybridization in the new file
      CALL READ_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=delr',13,'<section=',9,'ok',2)
	ALLOCATE(HUBCOPY(NSEC))
	DO ILIN=1,MSEC
	HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
	ENDDO
      DO ILIN=1,MSEC
	STRING=HUBDATA(ILIN+ISEC-1)
	CALL FIND_LBLANK(STRING,150,NBL)
      WRITE(IUINP,10120)(STRING(I:I),I=1,NBL)
      ENDDO
	DEALLOCATE(HUBCOPY)
C Update real axis self-energy/hybridization if available
	ELSEIF(KGRD.EQ.1)THEN
	DO IDVR=1,2
	CALL HUB_WRITEHUB_SIGR(IUINP,IDVR)
	ENDDO
	ENDIF
	ENDIF
C Storage is done, close the channel
	CLOSE(IUINP)
C REFORMAT FILE
	DEALLOCATE(HUBDATA)
      CALL FILE_LENGHT(FILE,NLEN,IUINP)
      ALLOCATE(HUBDATA(NLEN))
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='OLD',FORM='FORMATTED')
      DO ILEN=1,NLEN
      READ(IUINP,FMT='(A150)')HUBDATA(ILEN)
      ENDDO
      CLOSE(IUINP)
      CALL READ_OVWR(FILE,HUBDATA,NLEN)
C MEMORY DEALLOCATIONS
	DEALLOCATE(HUBDATA)
	END





	SUBROUTINE HUB_WRITEHUB_DATA(ICHAN,IDVR)
*******************************************************
*  Prints occupancies to the terminal                 *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from msbmesh
	USE COM_DMFDAT  ,ONLY : SIG0,SIGI,DELI,SIGW,BMAS
     &                       ,DELW,GLOC,OIMP,EIMP,ELOC
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE,IWR    !!! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS,VDBC             !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,LIB_TDP*10,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
C RETURN KEYS
      IF(ICHUB.LE.0)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	IF(IDVR.EQ.1)WRITE(ICHAN,10010)
10010 FORMAT(1X,'<SECTION=DHUB>',14X,'! PARTIAL OCCUPANCIES:')
	IF(IDVR.EQ.2)WRITE(ICHAN,10020)
10020 FORMAT(1X,'<SECTION=VHUB>',14X,'! LDA+U POTENTIAL:')
	IF(IDVR.EQ.3)WRITE(ICHAN,10030)
10030 FORMAT(1X,'<SECTION=VDBC>',14X,'! DBL.COUNTING POT:')
	IF(IDVR.EQ.4)WRITE(ICHAN,10040)
10040 FORMAT(1X,'<SECTION=VCNS>',14X,'! CONSTRAINED POTENTIAL:')
	IF(IDVR.EQ.5)WRITE(ICHAN,10050)
10050 FORMAT(1X,'<SECTION=OINV>',14X,'! INVERSE OVERLAP INVERSE:')
	IF(IDVR.EQ.6)WRITE(ICHAN,10060)
10060 FORMAT(1X,'<SECTION=EIMP>',14X,'! MEAN FIELD IMPURITY LEVELS:')
	IF(IDVR.EQ.7)WRITE(ICHAN,10070)
10070 FORMAT(1X,'<SECTION=ELOC>',14X,'! BARE IMPURITY LEVELS:')
	IF(IDVR.EQ.8)WRITE(ICHAN,10080)
10080 FORMAT(1X,'<SECTION=BMAS>',14X,'! STATIC SUSCEPTIBILITY:')
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(ICHAN,10100)TCTRL
C SET PARTIAL OCCUPATION NUMBERS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=DHUB(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=VHUB(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.3)
     &OCP(IUMK,IUPDN,IORBS)=VDBC(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.4)
     &OCP(IUMK,IUPDN,IORBS)=VCNS(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.5)
     &OCP(IUMK,IUPDN,IORBS)=OIMP(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.6)
     &OCP(IUMK,IUPDN,IORBS)=EIMP(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.7)
     &OCP(IUMK,IUPDN,IORBS)=ELOC(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.8)
     &OCP(IUMK,IUPDN,IORBS)=BMAS(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
	CALL LIB_LTURNU(OCS,OCP,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &            LIMB2MEM,NSPIN,NORBS)
C STORE MATRIX TO A GIVEN CHANNEL
      CALL HUB_WRITEREP(OCS,LIMB2MEM,NSPIN,NORBS,NSPIN,NORBS,
     &                  L,ICUBC,IRE,ICHAN)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
C MEMORY DEALLOCATIONS
 1000	CONTINUE
	DEALLOCATE(FHUB,OCP,OCS,ULR,UJR)
C FORMATS
10100 FORMAT(1X,'cState=',A10,11X, '! spin up/dn-up/dn data are :')
10105 FORMAT(1X,'cState=',A10,' ; wMsb = ',G14.7)
	END




	SUBROUTINE HUB_WRITEHUB_CSYS(ICHAN)
*******************************************************
*  Prints transformation matrices                     *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from msbmesh
	USE COM_IMPDAT  ,ONLY : KCRDSYS,DSYS,RSYS               !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE,IWR    !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,LIB_TDP*10,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
C RETURN KEYS
      IF(ICHUB.LE.0)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT TRANSFORMATION MATRICES
	WRITE(ICHAN,10010)
10010 FORMAT(1X,'<SECTION=CSYS>',14X,'! TRANSFORMATION MATRICES:')
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(ICHAN,10100)TCTRL
C SET TRANSFORMATION MATRICES
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      OCP(IUMK,IUPDN,IORBS)=DSYS(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
	CALL LIB_LTURNU(OCS,OCP,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &            LIMB2MEM,NSPIN,NORBS)
C STORE MATRIX TO A GIVEN CHANNEL
      CALL HUB_WRITEREP(OCS,LIMB2MEM,NSPIN,NORBS,NSPIN,NORBS,
     &                  L,ICUBC,IRE,ICHAN)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
C MEMORY DEALLOCATIONS
 1000	CONTINUE
	DEALLOCATE(FHUB,OCP,OCS,ULR,UJR)
C FORMATS
10100 FORMAT(1X,'cState=',A10,11X, '! spin up/dn-up/dn data are :')
10105 FORMAT(1X,'cState=',A10,' ; wMsb = ',G14.7)
	END




	SUBROUTINE HUB_WRITEHUB_RATP(ICHAN,IDVR)
*******************************************************
*  Writes rational interpolation for Sigma/Delta      *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE,IWR    !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
	USE COM_RATDAT  ,ONLY : VRAT,WRAT,PRAT,ORAT             !!! from readhub
     &                       ,VRHB,WRHB,PRHB,ORHB             !!! from readhub
	USE COM_DMFDAT  ,ONLY : SIGI,DELI                       !!! from readhub
	USE COM_IMPDAT  ,ONLY : NPOLIMP,NPLDIMP                 !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,LIB_TDP*10,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
	IF(IDMF.NE.3.AND.IDMF.NE.4.AND.IDMF.NE.5.AND.IDMF.NE.6)RETURN
      IF(ICHUB.LE.0)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
      IF(IDVR.EQ.1)
     &WRITE(ICHAN,10010)
10010 FORMAT(1X,'<SECTION=SRAT>',14X,'! RAT. INTERPOLATION FOR SIGMA:')
      IF(IDVR.EQ.2)
     &WRITE(ICHAN,10020)
10020 FORMAT(1X,'<SECTION=DRAT>',14X,'! RAT. INTERPOLATION FOR DELTA:')
	IREC=0
C Process NPOL
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
      IF(IDVR.EQ.1)NPOLES=NPOLIMP(L,IATOM)
      IF(IDVR.EQ.2)NPOLES=NPLDIMP(L,IATOM)
	WRITE(ICHAN,10100)TCTRL,NPOLES
	IF(IDVR.EQ.1)
     &WRITE(ICHAN,10050)
	IF(IDVR.EQ.2)
     &WRITE(ICHAN,10055)
	WRITE(ICHAN,10101)TCTRL
C STORE ARRAYS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=SIGI(IUMK,IATOM,IUPDN,IORBS)
      IF(IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=DELI(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
	CALL LIB_LTURNU(OCS,OCP,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &            LIMB2MEM,NSPIN,NORBS)
C STORE MATRIX TO A GIVEN CHANNEL
      CALL HUB_WRITEREP(OCS,LIMB2MEM,NSPIN,NORBS,NSPIN,NORBS,
     &                  L,ICUBC,IRE,ICHAN)
C STORE RATIONAL INTERPOLATION COEFFICIENTS
	DO IPOL=1,NPOLES                       !!! over poles
	DO IARR=1,4                            !!! over arrays
	IF(IARR.EQ.1)WRITE(ICHAN,10060)
	IF(IARR.EQ.2)WRITE(ICHAN,10070)
	IF(IARR.EQ.3)WRITE(ICHAN,10080)
	IF(IARR.EQ.4)WRITE(ICHAN,10090)
	WRITE(ICHAN,10105)TCTRL,IPOL
C STORE ARRAYS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IARR.EQ.1.AND.IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=VRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)
      IF(IARR.EQ.2.AND.IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=WRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)
      IF(IARR.EQ.3.AND.IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=PRAT(IUMK,IATOM,IUPDN,IORBS,IPOL)
      IF(IARR.EQ.4.AND.IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=ORAT(IUMK,IATOM,IUPDN,IORBS,IPOL)
      IF(IARR.EQ.1.AND.IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=VRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)
      IF(IARR.EQ.2.AND.IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=WRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)
      IF(IARR.EQ.3.AND.IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=PRHB(IUMK,IATOM,IUPDN,IORBS,IPOL)
      IF(IARR.EQ.4.AND.IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=ORHB(IUMK,IATOM,IUPDN,IORBS,IPOL)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C TRANSFER TO LOCAL COORDINATE SYSTEM 
	CALL LIB_LTURNU(OCS,OCP,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &            LIMB2MEM,NSPIN,NORBS)
C STORE MATRIX TO A GIVEN CHANNEL
      CALL HUB_WRITEREP(OCS,LIMB2MEM,NSPIN,NORBS,NSPIN,NORBS,
     &                  L,ICUBC,IRE,ICHAN)
	ENDDO                                  !!! over arrays
	ENDDO                                  !!! over poles
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
C MEMORY DEALLOCATIONS
 1000 DEALLOCATE(FHUB,OCP,OCS,ULR,UJR)
C FORMATS
10050 FORMAT('!!! Sigma(infinity)')
10055 FORMAT('!!! Delta(infinity)')
10060 FORMAT('!!! V coefficients')
10070 FORMAT('!!! W coefficients')
10080 FORMAT('!!! P poles positions')
10090 FORMAT('!!! O overlap matrix')
10100 FORMAT(1X,'cState=',A10,' ; nPoles = ',I3)
10101 FORMAT(1X,'cState=',A10)
10105 FORMAT(1X,'cState=',A10,' ; iPol = ',I3)
	END




	SUBROUTINE HUB_WRITEHUB_SIGW(ICHAN,IDVR)
*******************************************************
*  Writes self-energy to the channel ICHAN            *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_MSBMESH ,ONLY : NMSB,WMSB                       !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from msbmesh
	USE COM_DMFDAT  ,ONLY : SIGW,DELW,GLOC,OIMP             !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE,IWR    !!! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS                  !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                       !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,LIB_TDP*10,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
	IF(IDMF.NE.3.AND.IDMF.NE.4.AND.IDMF.NE.6)RETURN
      IF(ICHUB.LE.0)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT SELF-ENERGY, HYBRIDIZATION
	IF(IDVR.EQ.1)WRITE(ICHAN,10010)
10010 FORMAT(1X,'<SECTION=SIGW>',14X,'! DMFT SELF-ENERGY:')
	IF(IDVR.EQ.2)WRITE(ICHAN,10020)
10020 FORMAT(1X,'<SECTION=DELW>',14X,'! HYBRIDIZATION FUNCTION:')
	IREC=0
C Process NMSB
      HELP='! # of frequencies                      '
	WRITE(ICHAN,10030)NMSB,HELP
10030 FORMAT(1X,'nMsi  =',I10,11X,A40)
	DO IMSB=0,NMSB                         !!! over Matsubara frequencies
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(ICHAN,10100)TCTRL,IMSB,WMSB(IMSB)
C SET LDA+U OCCUPATION NUMBERS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=SIGW(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=DELW(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
	CALL LIB_LTURNU(OCS,OCP,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &            LIMB2MEM,NSPIN,NORBS)
C STORE MATRIX TO A GIVEN CHANNEL
      CALL HUB_WRITEREP(OCS,LIMB2MEM,NSPIN,NORBS,NSPIN,NORBS,
     &                  L,ICUBC,IRE,ICHAN)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
	ENDDO                                  !!! over Matsubara frequencies
C MEMORY DEALLOCATIONS
 1000	CONTINUE
	DEALLOCATE(FHUB,OCP,OCS,ULR,UJR)
C FORMATS
10100 FORMAT(1X,'cState=',A10,' ; iMsb = ',I7,' ; wMsb = ',G14.7)
	END





	SUBROUTINE HUB_WRITEHUB_SIGR(ICHAN,IDVR)
*******************************************************
*  Writes self-energy to the channel ICHAN            *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_MSBMESH ,ONLY : NOMG,WOMG                       !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from msbmesh
	USE COM_DMFDAT  ,ONLY : SIGW,DELW,GLOC,OIMP             !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ICUBC,IRE,IWR    !!! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS                  !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                       !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:,:,:)
	COMPLEX*16, ALLOCATABLE :: OCS(:,:,:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16, ALLOCATABLE :: UJR(:,:,:,:)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,LIB_TDP*10,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
	IF(IDMF.NE.5)RETURN
      IF(ICHUB.LE.0)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM,2,2))
	ALLOCATE(OCS(LIMB2MEM,2,2))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
      ALLOCATE(UJR(2,2,0:LMAXBMEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL LIB_TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
      CALL LIB_TRANSUJ(ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),UJR(1,1,L,IATOM))
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
      IF(IDVR.EQ.1)WRITE(ICHAN,10010)
      IF(IDVR.EQ.2)WRITE(ICHAN,10020)
10010 FORMAT(1X,'<SECTION=SIGR>',14X,'! SELF-ENERGY AT REAL AXIS:')
10020 FORMAT(1X,'<SECTION=DELR>',14X,'! HYBRIDIZATION AT REAL AXIS:')
	IREC=0
C Process NOMG
      HELP='! # of frequencies                      '
	WRITE(ICHAN,10030)NOMG,HELP
10030 FORMAT(1X,'nMsi  =',I10,11X,A40)
	DO IMSB=0,NOMG                         !!! over Matsubara frequencies
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(ICHAN,10100)TCTRL,IMSB,WOMG(IMSB)
C SET LDA+U OCCUPATION NUMBERS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      IF(IDVR.EQ.1)
     &OCP(IUMK,IUPDN,IORBS)=SIGW(IUMK,IATOM,IUPDN,IORBS)
      IF(IDVR.EQ.2)
     &OCP(IUMK,IUPDN,IORBS)=DELW(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
	CALL LIB_LTURNU(OCS,OCP,L,ULR(1,IATOM),UJR(1,1,L,IATOM),
     &            LIMB2MEM,NSPIN,NORBS)
C STORE MATRIX TO A GIVEN CHANNEL
      CALL HUB_WRITEREP(OCS,LIMB2MEM,NSPIN,NORBS,NSPIN,NORBS,
     &                  L,ICUBC,IRE,ICHAN)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
	ENDDO                                  !!! over Matsubara frequencies
C MEMORY DEALLOCATIONS
 1000 DEALLOCATE(FHUB,OCP,OCS,ULR,UJR)
C FORMATS
10100 FORMAT(1X,'cState=',A10,' ; iOmg = ',I7,' ; wOmg = ',G14.7)
	END
